/***************************************************************
 * Name:      datumdialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include "datumdialog.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "model/dbmodel.h"

IMPLEMENT_DYNAMIC_CLASS(DatumDialog, wxDialog)

BEGIN_EVENT_TABLE(DatumDialog, wxDialog)
    EVT_BUTTON(wxID_OK, DatumDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, DatumDialog::OnCancel)
END_EVENT_TABLE()

DatumDialog::DatumDialog(wxWindow *parent)
    : wxDialog(parent, wxID_ANY, "Datum aplikace")
{
    createControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void DatumDialog::createControls()
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    m_mainSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Aktuální datum aplikace")), 0, wxALL|wxEXPAND, 5);
    m_datum  = new wxDatePickerCtrl(this, wxID_ANY, dxsettings.datum(), wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    m_datum->SetRange(wxDateTime(1,wxDateTime::Jan,idbmodel->currentRok()),wxDateTime(31,wxDateTime::Dec,idbmodel->currentRok()));
    m_mainSizer->Add(m_datum, 0, wxALL|wxEXPAND, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void DatumDialog::OnOk(wxCommandEvent& /*event*/)
{
    dxsettings.setDatum(m_datum->GetValue());
    EndModal(wxID_OK);
}

void DatumDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}
