/***************************************************************
 * Name:      dphdruhydialog.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2015
 * License:   GPL3
 **************************************************************/

#ifndef DPHDRUHYDIALOG_H
#define DPHDRUHYDIALOG_H

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif
#include "gui/widgets/dxlistctrl.h"
#include <wx/imaglist.h>
#include "model/dbmodel.h"
#include "gui/filters/dphdruhfilter.h"
#include "gui/widgets/dxtoolbar.h"

class DphDruhyDialog: public wxDialog
{
    DECLARE_DYNAMIC_CLASS(DphDruhyDialog)
    DECLARE_EVENT_TABLE()
public:
    enum DType
    {
        DD_ALL = 0,
        DD_PRIJEM,
        DD_VYDAJ
    };
    DphDruhyDialog():m_bar(nullptr),m_select(false),m_type(DD_ALL),m_mainSizer(nullptr),m_list(nullptr),m_selectedId(-1),m_sortCol(0),m_sortAsc(true),m_filter(false) {}
    DphDruhyDialog(wxWindow *parent, bool select = false, DType type = DD_ALL, const wxDateTime &datum = wxDateTime::Today());

    long selectedID() const;
private:
    dxToolBar *m_bar;
    bool m_select;
    DType m_type;
    wxBoxSizer *m_mainSizer;
    dxListCtrl *m_list;
    wxScopedPtr<wxImageList> m_imageList;
    long m_selectedId;
    wxDateTime m_datum;
    dxDphDruhArray m_dphdruhy;
    int m_sortCol;
    bool m_sortAsc;
    bool m_filter;
    dxFilterArray m_filters;

    void OnOk(wxCommandEvent& event);
    void OnCancel(wxCommandEvent& event);
    void OnItemRightClick(wxListEvent& event);
    void OnItemActivated(wxListEvent& event);
    void OnItemSelected(wxListEvent& event);
    void OnItemDeselected(wxListEvent& event);
    void setSelectedId(long selectedIndex);
    void OnEnableDphDruh(wxCommandEvent& event);
    void fillList();
    void OnColumnClick(wxListEvent& event);
    void setColumnImage(int column, int image);
    void getData();
    void OnFilter(wxCommandEvent& event);
    void OnFilterClear(wxCommandEvent& event);
};

#endif  /* DPHDRUHYDIALOG_H */

