/***************************************************************
 * Name:      firmadialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include "firmadialog.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"

IMPLEMENT_DYNAMIC_CLASS(FirmaDialog, wxDialog)

BEGIN_EVENT_TABLE(FirmaDialog, wxDialog)
    EVT_BUTTON(wxID_OK, FirmaDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, FirmaDialog::OnCancel)
    EVT_BUTTON(ID_FIRMA_LOGO, FirmaDialog::OnLogo)
    EVT_BUTTON(ID_FIRMA_RAZITKO, FirmaDialog::OnRazitko)
END_EVENT_TABLE()

FirmaDialog::FirmaDialog(wxWindow *parent, bool edit)
    : wxDialog(parent, wxID_ANY, edit ? "Firma" : "Zadejte firmu"), m_edit(edit)
{
    createControls();
    if(m_edit) fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void FirmaDialog::createControls()
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Název")), wxGBPosition(0,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_nazev = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_nazev, wxGBPosition(0,1), wxGBSpan(1,3), wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Ulice"), wxGBPosition(1,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_ulice = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_ulice, wxGBPosition(1,1));
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("čp")), wxGBPosition(1,2), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_cp = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_cp, wxGBPosition(1,3));

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("PSČ")), wxGBPosition(2,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_psc = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_psc, wxGBPosition(2,1));
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Místo")), wxGBPosition(2,2), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_misto = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_misto, wxGBPosition(2,3));

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Země")), wxGBPosition(3,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_zeme = new wxTextCtrl(this, wxID_ANY);
    if(!m_edit) m_zeme->SetValue(wxString::FromUTF8("Česká republika"));
    gridSizer->Add(m_zeme, wxGBPosition(3,1), wxGBSpan(1,3), wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("IČ")), wxGBPosition(4,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_ic = new wxTextCtrl(this, wxID_ANY);
    m_ic->SetValidator(wxTextValidator(wxFILTER_DIGITS));
    gridSizer->Add(m_ic, wxGBPosition(4,1), wxGBSpan(1,3), wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("DIČ")), wxGBPosition(5,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_dic = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_dic, wxGBPosition(5,1), wxGBSpan(1,3), wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Registrace"), wxGBPosition(6,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_registrace = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_registrace, wxGBPosition(6,1), wxGBSpan(1,3), wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Paušální výdaje")), wxGBPosition(7,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_pausal = new wxCheckBox(this, wxID_ANY, "");
    gridSizer->Add(m_pausal, wxGBPosition(7,1));

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Plátce DPH")), wxGBPosition(8,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_platce = new wxCheckBox(this, wxID_ANY, "");
    gridSizer->Add(m_platce, wxGBPosition(8,1));

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Telefon"), wxGBPosition(9,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_telefon = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_telefon, wxGBPosition(9,1), wxGBSpan(1,3), wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "E-mail"), wxGBPosition(10,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_email = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_email, wxGBPosition(10,1), wxGBSpan(1,3), wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "WWW"), wxGBPosition(11,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_www = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_www, wxGBPosition(11,1), wxGBSpan(1,3), wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Logo"), wxGBPosition(12,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_logo = new wxTextCtrl(this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_READONLY);
    m_logo->SetToolTip("");
    gridSizer->Add(m_logo, wxGBPosition(12,1), wxGBSpan(1,2), wxEXPAND);
    m_logoSelect = new wxButton(this, ID_FIRMA_LOGO, "...");
    gridSizer->Add(m_logoSelect, wxGBPosition(12,3), wxDefaultSpan, wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Razítko")), wxGBPosition(13,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_razitko = new wxTextCtrl(this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_READONLY);
    m_razitko->SetToolTip("");
    gridSizer->Add(m_razitko, wxGBPosition(13,1), wxGBSpan(1,2), wxEXPAND);
    m_razitkoSelect = new wxButton(this, ID_FIRMA_RAZITKO, "...");
    gridSizer->Add(m_razitkoSelect, wxGBPosition(13,3), wxDefaultSpan, wxEXPAND);

    m_mainSizer->Add(gridSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void FirmaDialog::fillControls()
{
    Firma firma = idbmodel->firma();
    Adresa adresa = idbmodel->getAdresa(firma.adresaid());
    m_nazev->SetValue(firma.nazev());
    m_ulice->SetValue(adresa.isValid()?adresa.ulice():"");
    m_cp->SetValue(adresa.isValid()?adresa.cp():"");
    m_psc->SetValue(adresa.isValid()?adresa.psc():"");
    m_misto->SetValue(adresa.isValid()?adresa.misto():"");
    m_zeme->SetValue(adresa.isValid()?adresa.zeme():"");
    m_ic->SetValue(firma.ic());
    m_dic->SetValue(firma.dic());
    m_registrace->SetValue(firma.registrace());
    m_pausal->SetValue(firma.pausal());
    m_platce->SetValue(firma.platce());
    m_telefon->SetValue(firma.telefon());
    m_email->SetValue(firma.email());
    m_www->SetValue(firma.www());
    if(wxFileName::FileExists(firma.logo()))
    {
        wxFileName fn(firma.logo());
        m_logo->SetValue(fn.GetFullName());
        m_logo->SetToolTip(fn.GetFullPath());
    }
    if(wxFileName::FileExists(firma.razitko()))
    {
        wxFileName fn(firma.razitko());
        m_razitko->SetValue(fn.GetFullName());
        m_razitko->SetToolTip(fn.GetFullPath());
    }
}

void FirmaDialog::OnOk(wxCommandEvent& /*event*/)
{
    if(m_nazev->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Název nemůže být prázdný"), "Informace", m_edit? wxOK|wxICON_INFORMATION : wxOK|wxICON_WARNING);
        dialog.ShowModal();
        m_nazev->SetFocus();
        return;
    }
    if(m_ic->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("IČ nemůže být prázdné"), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        m_ic->SetFocus();
        return;
    }
    if(!dxutils::checkIC(m_ic->GetValue()))
    {
        wxMessageBox(wxString::FromUTF8("IČ není platné"), "Informace");
    }
    if(!m_edit)
    {
        if(dxsettings.firmaExists(m_nazev->GetValue()))
        {
            wxString msg;
            msg << "Firma " << m_nazev->GetValue() << wxString::FromUTF8(", IČ ") << m_ic->GetValue() << wxString::FromUTF8(" už existuje");
            wxMessageDialog dialog(this, msg, "Informace", wxOK|wxICON_WARNING);
            dialog.ShowModal();
            m_nazev->SetFocus();
            return;
        }
        idbmodel->addAdresa(Adresa(1, m_ulice->GetValue(), m_cp->GetValue(), m_misto->GetValue(), m_psc->GetValue(), m_zeme->GetValue()));
        idbmodel->setFirma(Firma(m_nazev->GetValue(), m_ic->GetValue(), m_dic->GetValue(), m_registrace->GetValue(), m_platce->IsChecked(), m_pausal->IsChecked(),
                             m_telefon->GetValue(), m_email->GetValue(), m_www->GetValue(), m_logo->GetToolTipText(), m_razitko->GetToolTipText(), 1));
        wxString dbname = dxutils::getUniqueDbName();
        dxsettings.setDbname(dbname);
        dxsettings.addDbnamePair(dbname, m_nazev->GetValue()+"_"+m_ic->GetValue());
    }
    else
    {
        if(idbmodel->firma().nazev() != m_nazev->GetValue() && dxsettings.firmaExists(m_nazev->GetValue()))
        {
            wxString msg;
            msg << "Firma " << m_nazev->GetValue() << wxString::FromUTF8(" už existuje");
            wxMessageDialog dialog(this, msg, "Informace", wxOK|wxICON_WARNING);
            dialog.ShowModal();
            m_nazev->SetFocus();
            return;
        }
        idbmodel->editAdresa(Adresa(1, m_ulice->GetValue(), m_cp->GetValue(), m_misto->GetValue(), m_psc->GetValue(), m_zeme->GetValue()));
        idbmodel->setFirma(Firma(m_nazev->GetValue(), m_ic->GetValue(), m_dic->GetValue(), m_registrace->GetValue(), m_platce->IsChecked(), m_pausal->IsChecked(),
                             m_telefon->GetValue(), m_email->GetValue(), m_www->GetValue(), m_logo->GetToolTipText(), m_razitko->GetToolTipText(), 1));
        dxsettings.editDbnamePair(dxsettings.dbname(), m_nazev->GetValue()+"_"+m_ic->GetValue());
    }
    EndModal(wxID_OK);
}

void FirmaDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

void FirmaDialog::OnLogo(wxCommandEvent &/*event*/)
{
    wxFileName fn(m_logo->GetToolTipText());
    wxFileDialog dialog(this, wxString::FromUTF8("Otevřít obrázek"), fn.Exists()?fn.GetPath():wxGetHomeDir(), fn.Exists()?fn.GetFullName():"",
                        wxString::FromUTF8("Soubory obrázků (*.gif;*.png;*.jpg)|*.gif;*.png;*.jpg"),
                        wxFD_OPEN|wxFD_FILE_MUST_EXIST);
    if(dialog.ShowModal() == wxID_OK)
    {
        wxString path = dialog.GetPath();
        if(!path.empty())
        {
            m_logo->SetToolTip(path);
            m_logo->SetValue(dialog.GetFilename());
        }
    }
}

void FirmaDialog::OnRazitko(wxCommandEvent &/*event*/)
{
    wxFileName fn(m_razitko->GetToolTipText());
    wxFileDialog dialog(this, wxString::FromUTF8("Otevřít obrázek"), fn.Exists()?fn.GetPath():wxGetHomeDir(), fn.Exists()?fn.GetFullName():"",
                        wxString::FromUTF8("Soubory obrázků (*.gif;*.png;*.jpg)|*.gif;*.png;*.jpg"),
                        wxFD_OPEN|wxFD_FILE_MUST_EXIST);
    if(dialog.ShowModal() == wxID_OK)
    {
        wxString path = dialog.GetPath();
        if(!path.empty())
        {
            m_razitko->SetToolTip(path);
            m_razitko->SetValue(dialog.GetFilename());
        }
    }
}

