/***************************************************************
 * Name:      inventuradialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include "inventuradialog.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"

IMPLEMENT_DYNAMIC_CLASS(InventuraDialog, wxDialog)

BEGIN_EVENT_TABLE(InventuraDialog, wxDialog)
    EVT_BUTTON(wxID_OK, InventuraDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, InventuraDialog::OnCancel)
    EVT_CHOICE(ID_INVENTURA_SKLAD, InventuraDialog::OnSklad)
    EVT_DATE_CHANGED(ID_INVENTURA_DATUM, InventuraDialog::OnDatum)
    EVT_TIME_CHANGED(ID_INVENTURA_CAS, InventuraDialog::OnCas)
END_EVENT_TABLE()

//Nova inventura
InventuraDialog::InventuraDialog(wxWindow *parent)
    : wxDialog(parent, wxID_ANY, "Inventura skladu"), m_edit(false)
{
    m_id = idbmodel->getLastId("inventura");
    createControls();
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

//Editace inventury
InventuraDialog::InventuraDialog(wxWindow *parent, int id, bool edit)
    : wxDialog(parent, wxID_ANY, "Inventura skladu"), m_id(id), m_edit(edit)
{
    createControls();
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void InventuraDialog::createControls()
{
    int i=0; //row

    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Číslo inventury")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_cislo = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_cislo, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Popis"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_popis = new wxTextCtrl(this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE);
    m_popis->SetMinSize(wxSize(-1,80));
    gridSizer->Add(m_popis, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Inventuru\nprovedl"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_provedl = new wxTextCtrl(this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE);
    m_provedl->SetMinSize(wxSize(-1,80));
    gridSizer->Add(m_provedl, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Datum inventury"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_datum = new wxDatePickerCtrl(this, ID_INVENTURA_DATUM, dxsettings.datum(), wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    gridSizer->Add(m_datum, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Čas inventury")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_time = new wxTimePickerCtrl(this, ID_INVENTURA_CAS);
    gridSizer->Add(m_time, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Sklad"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_sklad = new wxChoice(this, ID_INVENTURA_SKLAD);
    gridSizer->Add(m_sklad, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    m_table = new wxGrid(this, wxID_ANY);
    m_table->CreateGrid(1,4);
    m_table->UseNativeColHeader();
    m_table->HideRowLabels();
    m_table->SetColLabelValue(0, wxString::FromUTF8("Číslo"));
    m_table->SetColLabelValue(1, wxString::FromUTF8("Zásoba"));
    m_table->SetColLabelValue(2, wxString::FromUTF8("Evidovaný stav"));
    m_table->SetColLabelValue(3, wxString::FromUTF8("Zjištěný stav"));
    m_table->SetColSize(0, 100);
    m_table->SetColSize(1, 200);
    m_table->SetColSize(2, 150);
    m_table->SetColSize(3, 150);
    m_table->SetColFormatFloat(2, 10, 3);
    m_table->SetColFormatFloat(3, 10, 3);
    gridSizer->Add(m_table, wxGBPosition(i,0), wxGBSpan(1,2), wxEXPAND);

    m_mainSizer->Add(gridSizer, 1, wxALL|wxEXPAND, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void InventuraDialog::fillControls()
{
    if(m_edit)
    {
        Inventura iv = idbmodel->getInventura(m_id);
        if(iv.isValid())
        {
            m_cislo->SetValue(iv.cislo());
            m_popis->SetValue(iv.popis());
            m_provedl->SetValue(iv.provedl());
            m_datum->SetValue(iv.datum());
            m_datum->Disable();
            m_time->SetValue(iv.datum());
            m_time->Disable();
            m_sklad->Append(idbmodel->getSkladNazevById(iv.skladid()));
            m_sklad->SetSelection(0);
            m_sklad->Disable();
        }
    }
    else
    {
        m_cislo->SetValue(idbmodel->getCisloDokladu("IV"));
        m_time->SetValue(wxDateTime::Now());
        dxSkladArray sklady = idbmodel->sklady();
        for(size_t i=0; i < sklady.GetCount(); i++)
        {
            m_sklad->Append(sklady[i].nazev(), new Sklad(sklady[i].id(), "", "", sklady[i].ocenovani()));
        }
        m_sklad->SetSelection(0);
        if(!m_edit) m_datum->SetRange(wxMax(idbmodel->getInventuraLastDateForSklad(static_cast<Sklad*>(m_sklad->GetClientData(static_cast<unsigned int>(m_sklad->GetSelection())))->id()),
                                            wxDateTime(1,wxDateTime::Jan,idbmodel->currentRok())),wxDateTime(31,wxDateTime::Dec,idbmodel->currentRok()));
    }
    fillTable();
}

void InventuraDialog::fillTable()
{
    if(m_edit)
    {
        dxIvPolozkaArray polozky = idbmodel->ivpolozkyByInventuraId(m_id);
        if(static_cast<size_t>(m_table->GetNumberRows()) < polozky.GetCount())
        {
            m_table->AppendRows(static_cast<int>(polozky.GetCount())-m_table->GetNumberRows(), false);
        }
        else if(static_cast<size_t>(m_table->GetNumberRows()) > polozky.GetCount())
        {
            m_table->DeleteRows(static_cast<int>(polozky.GetCount()), m_table->GetNumberRows()-static_cast<int>(polozky.GetCount()));
        }
        for(size_t i=0; i < polozky.GetCount(); i++)
        {
            m_table->SetCellValue(static_cast<int>(i), 0, idbmodel->hasZasoba(polozky[i].zasobaid()) ? idbmodel->getZasoba(polozky[i].zasobaid()).cislo() : "");
            m_table->SetCellValue(static_cast<int>(i), 1, idbmodel->hasZasoba(polozky[i].zasobaid()) ? idbmodel->getZasoba(polozky[i].zasobaid()).nazev() : "");
            m_table->SetCellValue(static_cast<int>(i), 2, wxString::Format("%g", polozky[i].evidovano()));
            m_table->SetCellValue(static_cast<int>(i), 3, wxString::Format("%g", polozky[i].zjisteno()));
            m_table->SetReadOnly(static_cast<int>(i), 0);
            m_table->SetReadOnly(static_cast<int>(i), 1);
            m_table->SetReadOnly(static_cast<int>(i), 2);
            m_table->SetReadOnly(static_cast<int>(i), 3, false);
        }
    }
    else
    {
        size_t rows = idbmodel->getZasobyCountBySkladId(static_cast<Sklad*>(m_sklad->GetClientData(static_cast<unsigned int>(m_sklad->GetSelection())))->id());
        if(m_table->GetNumberRows() < static_cast<int>(rows))
        {
            m_table->AppendRows(static_cast<int>(rows)-m_table->GetNumberRows(), false);
        }
        else if(m_table->GetNumberRows() > static_cast<int>(rows))
        {
            m_table->DeleteRows(static_cast<int>(rows), m_table->GetNumberRows()-static_cast<int>(rows));
        }
        dxZasobaArray zasoby = idbmodel->zasobyBySkladId(static_cast<Sklad*>(m_sklad->GetClientData(static_cast<unsigned int>(m_sklad->GetSelection())))->id());
        for(size_t i=0; i < rows; i++)
        {
            m_table->SetCellValue(static_cast<int>(i), 0, zasoby[i].cislo());
            m_table->SetCellValue(static_cast<int>(i), 1, zasoby[i].nazev());
            double pocet = idbmodel->stavZasobyToDate(zasoby[i].id(), getDatum());
            m_table->SetCellValue(static_cast<int>(i), 2, wxString::Format("%g", pocet));
            m_table->SetCellValue(static_cast<int>(i), 3, wxString::Format("%g", pocet));
            m_table->SetReadOnly(static_cast<int>(i), 0);
            m_table->SetReadOnly(static_cast<int>(i), 1);
            m_table->SetReadOnly(static_cast<int>(i), 2);
            m_table->SetReadOnly(static_cast<int>(i), 3, false);
        }
    }
}

wxDateTime InventuraDialog::getDatum() const
{
    return wxDateTime(m_datum->GetValue().GetDay(), m_datum->GetValue().GetMonth(), m_datum->GetValue().GetYear(),
                      m_time->GetValue().GetHour(), m_time->GetValue().GetMinute(), m_time->GetValue().GetSecond());
}

void InventuraDialog::OnOk(wxCommandEvent& /*event*/)
{
    if(m_edit)
    {
        Inventura iv = idbmodel->getInventura(m_id);
        if(iv.isValid())
        {
            idbmodel->editInventura(Inventura(m_id, iv.datum(), m_cislo->GetValue(), m_popis->GetValue(), m_provedl->GetValue(), iv.skladid(), iv.obdobirok()));
            dxIvPolozkaArray polozky = idbmodel->ivpolozkyByInventuraId(m_id);
            for(size_t i=0; i < polozky.GetCount(); i++)
            {
                double evidovano;
                m_table->GetCellValue(static_cast<int>(i),2).ToDouble(&evidovano);
                double zjisteno;
                m_table->GetCellValue(static_cast<int>(i),3).ToDouble(&zjisteno);
                double pocet, celkem;
                idbmodel->getZasobaStav(polozky[i].zasobaid(), iv.datum(), &pocet, &celkem);
                bool prijemka = zjisteno>=evidovano;
                double pocetPohyb = prijemka ? zjisteno-evidovano : evidovano-zjisteno;
                double cenamjPohyb = wxIsNullDouble(pocetPohyb) ? 0.0 :(prijemka?(wxIsNullDouble(pocet)?0.0:celkem/pocet):0.0);
                idbmodel->editIvPolozka(IvPolozka(polozky[i].id(), evidovano, zjisteno, polozky[i].zasobaid(), m_id));
                Pohybzasob pz = idbmodel->getPohybzasobByInventura(polozky[i].zasobaid(), polozky[i].inventuraid());
                if(pz.isValid())
                {
                    idbmodel->editPohybzasob(Pohybzasob(pz.id(), iv.datum(), pz.cislo(), pocetPohyb, cenamjPohyb,
                                                        prijemka, false, -1, polozky[i].zasobaid(), idbmodel->currentRok(), true, m_id));
                }
                else
                {
                    idbmodel->addPohybzasob(Pohybzasob(idbmodel->getLastId("pohybzasob"), iv.datum(), wxEmptyString, pocetPohyb, cenamjPohyb,
                                                       prijemka, false, -1, polozky[i].zasobaid(), idbmodel->currentRok(), true, m_id));
                }
            }
        }
    }
    else
    {
        int skladid = static_cast<Sklad*>(m_sklad->GetClientData(static_cast<unsigned int>(m_sklad->GetSelection())))->id();
        wxDateTime datum = getDatum();
        if(datum.IsEarlierThan(idbmodel->getInventuraLastDateForSklad(skladid)))
        {
            datum = idbmodel->getInventuraLastDateForZasoba(skladid)+wxTimeSpan(0,0,1);
        }
        idbmodel->addInventura(Inventura(m_id, datum, m_cislo->GetValue(), m_popis->GetValue(), m_provedl->GetValue(), skladid, idbmodel->currentRok()));
        dxZasobaArray zasoby = idbmodel->zasobyBySkladId(skladid);
        for(size_t i=0; i < zasoby.GetCount(); i++)
        {
            double evidovano;
            m_table->GetCellValue(static_cast<int>(i),2).ToDouble(&evidovano);
            double zjisteno;
            m_table->GetCellValue(static_cast<int>(i),3).ToDouble(&zjisteno);
            double pocet, celkem;
            idbmodel->getZasobaStav(zasoby[i].id(), datum, &pocet, &celkem);
            bool prijemka = zjisteno>=evidovano;
            double pocetPohyb = prijemka ? zjisteno-evidovano : evidovano-zjisteno;
            double cenamjPohyb = wxIsNullDouble(pocetPohyb) ? 0.0 :(prijemka?(wxIsNullDouble(pocet)?0.0:celkem/pocet):0.0);
            idbmodel->addIvPolozka(IvPolozka(idbmodel->getLastId("ivpolozka"), evidovano, zjisteno, zasoby[i].id(), m_id));
            idbmodel->addPohybzasob(Pohybzasob(idbmodel->getLastId("pohybzasob"), datum, wxEmptyString, pocetPohyb, cenamjPohyb,
                                               prijemka, false, -1, zasoby[i].id(), idbmodel->currentRok(), true, m_id));
        }
        idbmodel->increaseRadaCislo("IV");
    }
    EndModal(wxID_OK);
}

void InventuraDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

void InventuraDialog::OnSklad(wxCommandEvent &/*event*/)
{
    if(!m_edit)
    {
        int skladid = static_cast<Sklad*>(m_sklad->GetClientData(m_sklad->GetSelection()))->id();
        if(m_datum->GetValue().IsEarlierThan(idbmodel->getInventuraLastDateForSklad(skladid)))
        {
            m_datum->SetValue(idbmodel->getInventuraLastDateForSklad(skladid));
        }
        m_datum->SetRange(wxMax(idbmodel->getInventuraLastDateForSklad(skladid),
                                        wxDateTime(1,wxDateTime::Jan,idbmodel->currentRok())),wxDateTime(31,wxDateTime::Dec,idbmodel->currentRok()));
    }
    fillTable();
    m_mainSizer->Layout();
    m_mainSizer->Fit(this);
}

void InventuraDialog::OnDatum(wxDateEvent &/*event*/)
{
    if(!m_edit)
    {
        fillTable();
    }
}

void InventuraDialog::OnCas(wxDateEvent &/*event*/)
{
    if(!m_edit)
    {
        fillTable();
    }
}
