/***************************************************************
 * Name:      jizdadialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2018
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include "jizdadialog.h"
#include <wx/valnum.h>
#include "utils/dxutils.h"
#include "utils/dxsettings.h"

IMPLEMENT_DYNAMIC_CLASS(JizdaDialog, wxDialog)

BEGIN_EVENT_TABLE(JizdaDialog, wxDialog)
    EVT_BUTTON(wxID_OK, JizdaDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, JizdaDialog::OnCancel)
    EVT_BUTTON(ID_JIZDA_CESTA, JizdaDialog::OnSelectCesta)
    EVT_BUTTON(ID_JIZDA_UCEL, JizdaDialog::OnSelectUcel)
    EVT_DATE_CHANGED(ID_JIZDA_STARTDATE, JizdaDialog::OnDate)
    EVT_DATE_CHANGED(ID_JIZDA_ENDDATE, JizdaDialog::OnDate)
    EVT_TIME_CHANGED(ID_JIZDA_STARTTIME, JizdaDialog::OnTime)
    EVT_TIME_CHANGED(ID_JIZDA_ENDTIME, JizdaDialog::OnTime)
    EVT_TEXT(ID_JIZDA_STARTKM, JizdaDialog::OnKm)
    EVT_TEXT(ID_JIZDA_ENDKM, JizdaDialog::OnKm)
    EVT_CHOICE(ID_JIZDA_VOZIDLO, JizdaDialog::OnVozidlo)
END_EVENT_TABLE()

//Nova zasoba
JizdaDialog::JizdaDialog(wxWindow *parent)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Jízda")), m_edit(0)
{
    createControls();
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

//Editace zasoby
JizdaDialog::JizdaDialog(wxWindow *parent, int id, int edit)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Jízda")), m_id(id), m_edit(edit)
{
    createControls();
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void JizdaDialog::createControls()
{
    int i=0; //row

    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Vozidlo"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_vozidlo = new wxChoice(this, ID_JIZDA_VOZIDLO);
    dxVozidloArray vozidla = idbmodel->vozidla();
    for(size_t i=0; i < vozidla.GetCount(); i++)
    {
        m_vozidlo->Append(vozidla[i].rz(), new Vozidlo(vozidla[i].id(), wxEmptyString, wxEmptyString, -1, 0.0, false, -1));
    }
    gridSizer->Add(m_vozidlo, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Řidič")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_ridic = new wxComboBox(this, wxID_ANY);
    dxRidicArray ridici = idbmodel->ridicy();
    for(size_t i=0; i< ridici.GetCount(); i++)
    {
        m_ridic->Append(ridici[i].jmeno());
    }
    gridSizer->Add(m_ridic, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Cesta"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_cesta = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_cesta, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_cestaSelect = new wxButton(this, ID_JIZDA_CESTA, "...");
    m_cestaSelect->Enable(idbmodel->cestyCount());
    gridSizer->Add(m_cestaSelect, wxGBPosition(i,2), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Účel cesty")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_ucel = new wxTextCtrl(this, wxID_ANY);
    m_ucel->SetMinSize(wxSize(200,-1));
    gridSizer->Add(m_ucel, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_ucelSelect = new wxButton(this, ID_JIZDA_UCEL, "...");
    m_ucelSelect->Enable(idbmodel->ucelyCount());
    gridSizer->Add(m_ucelSelect, wxGBPosition(i,2), wxDefaultSpan, wxEXPAND);
    i++;

    wxDateTime datum = dxsettings.datum();
    datum.SetHour(wxDateTime::Now().GetHour());
    datum.SetMinute(0);
    datum.SetSecond(0);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Začátek jízdy")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_startDate = new wxDatePickerCtrl(this, ID_JIZDA_STARTDATE, datum-wxTimeSpan(1), wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    m_startDate->SetRange(wxDateTime(1,wxDateTime::Jan,idbmodel->currentRok()-1), datum);
    gridSizer->Add(m_startDate, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);
    i++;

    m_startTime = new wxTimePickerCtrl(this, ID_JIZDA_STARTTIME, datum-wxTimeSpan(1));
    gridSizer->Add(m_startTime, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Konec jízdy")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_endDate = new wxDatePickerCtrl(this, ID_JIZDA_ENDDATE, datum, wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    m_endDate->SetRange(wxDateTime(1,wxDateTime::Jan,idbmodel->currentRok()-1), datum);
    gridSizer->Add(m_endDate, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);
    i++;

    m_endTime = new wxTimePickerCtrl(this, ID_JIZDA_ENDTIME, datum);
    gridSizer->Add(m_endTime, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Počáteční stav tachometru")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_startKm = new wxTextCtrl(this, ID_JIZDA_STARTKM, "", wxDefaultPosition, wxDefaultSize, wxTE_RIGHT, wxMakeIntegerValidator(&m_startKmValue));
    gridSizer->Add(m_startKm, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Konečný stav tachometru")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_endKm = new wxTextCtrl(this, ID_JIZDA_ENDKM, "", wxDefaultPosition, wxDefaultSize, wxTE_RIGHT, wxMakeIntegerValidator(&m_endKmValue));
    gridSizer->Add(m_endKm, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);

    m_mainSizer->Add(gridSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void JizdaDialog::fillControls()
{
    if(m_edit)
    {
        Jizda ja = idbmodel->getJizda(m_id);
        if(ja.isValid())
        {
            m_cesta->SetValue(ja.cesta());
            m_ucel->SetValue(ja.ucel());
            for(unsigned int i=0; i<m_vozidlo->GetCount(); i++)
            {
                if(static_cast<Vozidlo*>(m_vozidlo->GetClientData(i))->id() == ja.vozidloid())
                {
                    m_vozidlo->SetSelection(static_cast<int>(i));
                    break;
                }
            }
            for(unsigned int i=0; i<m_ridic->GetCount(); i++)
            {
                if(m_ridic->GetString(i) == idbmodel->jmenoRidic(ja.ridicid()))
                {
                    m_ridic->SetSelection(static_cast<int>(i));
                    break;
                }
            }
            m_startDate->SetValue(ja.startcas());
            m_startTime->SetValue(ja.startcas());
            m_endDate->SetValue(ja.endcas());
            m_endTime->SetValue(ja.endcas());
            m_startKmValue = ja.startkm();
            m_endKmValue = ja.endkm();
        }
    }
    else
    {
        m_vozidlo->SetSelection(0);
        if(m_ridic->GetCount()) m_ridic->SetSelection(0);
        m_startKmValue = idbmodel->tachometrEnd(static_cast<Vozidlo*>(m_vozidlo->GetClientData(0))->id());
        m_endKmValue = m_startKmValue+1;
    }
}

void JizdaDialog::OnOk(wxCommandEvent& /*event*/)
{
    if(m_ridic->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Jméno řidiče nemůže být prázdné"), "Informace", m_edit? wxOK|wxICON_INFORMATION : wxOK|wxICON_WARNING);
        dialog.ShowModal();
        m_ridic->SetFocus();
        return;
    }
    if(m_cesta->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Cesta nemůže být prázdná"), "Informace", m_edit? wxOK|wxICON_INFORMATION : wxOK|wxICON_WARNING);
        dialog.ShowModal();
        m_cesta->SetFocus();
        return;
    }
    if(m_ucel->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Účel cesty nemůže být prázdný"), "Informace", m_edit? wxOK|wxICON_INFORMATION : wxOK|wxICON_WARNING);
        dialog.ShowModal();
        m_ucel->SetFocus();
        return;
    }
    m_startKm->GetValidator()->TransferFromWindow();
    m_endKm->GetValidator()->TransferFromWindow();
    wxDateTime startcas = createDateTime(true);
    wxDateTime endcas = createDateTime(false);
    wxString ridic = m_ridic->GetValue();
    int ridicId = -1;
    if(!idbmodel->hasRidic(ridic))
    {
        ridicId = idbmodel->getLastId("ridic");
        idbmodel->addRidic(Ridic(ridicId, ridic));
    }
    else
    {
        ridicId = idbmodel->getRidic(ridic).id();
    }
    if(m_edit) idbmodel->editJizda(Jizda(m_id, startcas, endcas, m_startKmValue, m_endKmValue, m_cesta->GetValue(), m_ucel->GetValue(), static_cast<Vozidlo*>(m_vozidlo->GetClientData(m_vozidlo->GetSelection()))->id(), ridicId));
    else idbmodel->addJizda(Jizda(idbmodel->getLastId("jizda"), startcas, endcas, m_startKmValue, m_endKmValue, m_cesta->GetValue(), m_ucel->GetValue(),
                                  static_cast<Vozidlo*>(m_vozidlo->GetClientData(m_vozidlo->GetSelection()))->id(), ridicId));
    EndModal(wxID_OK);
}

void JizdaDialog::OnCancel(wxCommandEvent& /*event*/)
{
    if(m_edit == 0 || m_edit == 2)
    {
        idbmodel->deleteJizda(m_id);
    }
    EndModal(wxID_CANCEL);
}

wxDateTime JizdaDialog::createDateTime(bool start)
{
    wxDateTime date = start?m_startDate->GetValue():m_endDate->GetValue();
    date.SetHour(start?m_startTime->GetValue().GetHour():m_endTime->GetValue().GetHour());
    date.SetMinute(start?m_startTime->GetValue().GetMinute():m_endTime->GetValue().GetMinute());
    date.SetSecond(start?m_startTime->GetValue().GetSecond():m_endTime->GetValue().GetSecond());
    return date;
}

void JizdaDialog::OnSelectCesta(wxCommandEvent &/*event*/)
{
    wxSingleChoiceDialog dialog(this, "Vyberte cestu", wxString::FromUTF8("Výběr"), idbmodel->cesty());
    dialog.SetSelection(0);
    if(dialog.ShowModal() == wxID_OK)
        m_cesta->SetValue(dialog.GetStringSelection());
}

void JizdaDialog::OnSelectUcel(wxCommandEvent &/*event*/)
{
    wxSingleChoiceDialog dialog(this, wxString::FromUTF8("Vyberte účel cesty"), wxString::FromUTF8("Výběr"), idbmodel->ucely());
    dialog.SetSelection(0);
    if(dialog.ShowModal() == wxID_OK)
        m_ucel->SetValue(dialog.GetStringSelection());
}

void JizdaDialog::OnDate(wxDateEvent &event)
{
    if(event.GetId() == ID_JIZDA_STARTDATE)
    {
        if(createDateTime(true).IsLaterThan(createDateTime(false)))
        {
            m_endDate->SetValue(createDateTime(true));
            m_endTime->SetValue(createDateTime(true));
        }
    }
    else
    {
        if(createDateTime(true).IsLaterThan(createDateTime(false)))
        {
            m_startDate->SetValue(createDateTime(false));
            m_startTime->SetValue(createDateTime(false));
        }
    }
}

void JizdaDialog::OnTime(wxDateEvent &event)
{
    if(event.GetId() == ID_JIZDA_STARTTIME)
    {
        if(createDateTime(true).IsLaterThan(createDateTime(false)))
        {
            m_endDate->SetValue(createDateTime(true));
            m_endTime->SetValue(createDateTime(true));
        }
    }
    else
    {
        if(createDateTime(true).IsLaterThan(createDateTime(false)))
        {
            m_startDate->SetValue(createDateTime(false));
            m_startTime->SetValue(createDateTime(false));
        }
    }
}

void JizdaDialog::OnKm(wxCommandEvent &event)
{
    m_startKm->GetValidator()->TransferFromWindow();
    m_endKm->GetValidator()->TransferFromWindow();
    if(event.GetId() == ID_JIZDA_STARTKM)
    {
        if(m_startKmValue > m_endKmValue)
        {
            m_endKmValue = m_startKmValue+1;
            m_endKm->GetValidator()->TransferToWindow();
        }
    }
//    else
//    {
//        if(m_startKmValue > m_endKmValue)
//        {
//            m_startKmValue = m_endKmValue;
//            m_startKm->GetValidator()->TransferToWindow();
//        }
    //    }
}

void JizdaDialog::OnVozidlo(wxCommandEvent &/*event*/)
{
    m_startKmValue = idbmodel->tachometrEnd(static_cast<Vozidlo*>(m_vozidlo->GetClientData(static_cast<unsigned int>(m_vozidlo->GetSelection())))->id());
    m_startKm->GetValidator()->TransferToWindow();
}
