/***************************************************************
 * Name:      kontaktdialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include "kontaktdialog.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"

IMPLEMENT_DYNAMIC_CLASS(KontaktDialog, wxDialog)

BEGIN_EVENT_TABLE(KontaktDialog, wxDialog)
    EVT_BUTTON(wxID_OK, KontaktDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, KontaktDialog::OnCancel)
END_EVENT_TABLE()

KontaktDialog::KontaktDialog(wxWindow *parent, int id, int edit)
    : wxDialog(parent, wxID_ANY, edit ? "Kontakt" : "Zadejte kontakt"), m_edit(edit)
{
    createControls();
    if(m_edit)
    {
        m_id = id;
        m_adresaid = idbmodel->hasKontakt(id) ? idbmodel->getKontakt(id).adresaid(): -1;
        fillControls();
    }
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void KontaktDialog::createControls()
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Název")), wxGBPosition(0,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_nazev = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_nazev, wxGBPosition(0,1), wxGBSpan(1,3), wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Obchodní jméno")), wxGBPosition(1,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_jmeno = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_jmeno, wxGBPosition(1,1), wxGBSpan(1,3), wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Ulice"), wxGBPosition(2,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_ulice = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_ulice, wxGBPosition(2,1));
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("čp")), wxGBPosition(2,2), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_cp = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_cp, wxGBPosition(2,3));

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("PSČ")), wxGBPosition(3,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_psc = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_psc, wxGBPosition(3,1));
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Místo")), wxGBPosition(3,2), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_misto = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_misto, wxGBPosition(3,3));

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Země")), wxGBPosition(4,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_zeme = new wxTextCtrl(this, wxID_ANY);
    if(!m_edit) m_zeme->SetValue(wxString::FromUTF8("Česká republika"));
    gridSizer->Add(m_zeme, wxGBPosition(4,1), wxGBSpan(1,3), wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("IČ")), wxGBPosition(5,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_ic = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_ic, wxGBPosition(5,1), wxGBSpan(1,3), wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("DIČ")), wxGBPosition(6,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_dic = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_dic, wxGBPosition(6,1), wxGBSpan(1,3), wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "E-mail"), wxGBPosition(7,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_email = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_email, wxGBPosition(7,1), wxGBSpan(1,3), wxEXPAND);

    m_mainSizer->Add(gridSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void KontaktDialog::fillControls()
{
    Kontakt kontakt = idbmodel->getKontakt(m_id);
    if(kontakt.isValid())
    {
        Adresa adresa = idbmodel->getAdresa(kontakt.adresaid());
        if(adresa.isValid())
        {
            m_nazev->SetValue(kontakt.nazev());
            m_jmeno->SetValue(kontakt.jmeno());
            m_ulice->SetValue(adresa.ulice());
            m_cp->SetValue(adresa.cp());
            m_psc->SetValue(adresa.psc());
            m_misto->SetValue(adresa.misto());
            m_zeme->SetValue(adresa.zeme());
            m_ic->SetValue(kontakt.ic());
            m_dic->SetValue(kontakt.dic());
            m_email->SetValue(kontakt.email());
        }
    }
}

void KontaktDialog::OnOk(wxCommandEvent& /*event*/)
{
    if(m_nazev->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Název nemůže být prázdný"), "Informace", m_edit? wxOK|wxICON_INFORMATION : wxOK|wxICON_WARNING);
        dialog.ShowModal();
        m_nazev->SetFocus();
        return;
    }
    if(nazevExists())
    {
        wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Název %s už existuje"), m_nazev->GetValue()), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        m_nazev->SetFocus();
        return;
    }
    if(!m_edit)
    {
        m_id = idbmodel->getLastId("kontakt");
        m_adresaid = idbmodel->getLastId("adresa");
        idbmodel->addAdresa(Adresa(m_adresaid, m_ulice->GetValue(), m_cp->GetValue(), m_misto->GetValue(), m_psc->GetValue(), m_zeme->GetValue()));
        idbmodel->addKontakt(Kontakt(m_id, m_nazev->GetValue(), m_jmeno->GetValue().IsEmpty()?m_nazev->GetValue():m_jmeno->GetValue(), m_ic->GetValue(), m_dic->GetValue(), m_email->GetValue(), true, m_adresaid));
    }
    else
    {
        idbmodel->editAdresa(Adresa(m_adresaid, m_ulice->GetValue(), m_cp->GetValue(), m_misto->GetValue(), m_psc->GetValue(), m_zeme->GetValue()));
        idbmodel->editKontakt(Kontakt(m_id, m_nazev->GetValue(), m_jmeno->GetValue().IsEmpty()?m_nazev->GetValue():m_jmeno->GetValue(), m_ic->GetValue(), m_dic->GetValue(), m_email->GetValue(), true, m_adresaid));
    }
    EndModal(wxID_OK);
}

void KontaktDialog::OnCancel(wxCommandEvent& /*event*/)
{
    if(m_edit == 2) idbmodel->deleteKontakt(m_id);
    EndModal(wxID_CANCEL);
}

bool KontaktDialog::nazevExists()
{
    if(m_edit)
    {
        wxString nazev = idbmodel->hasKontakt(m_id) ? idbmodel->getKontakt(m_id).nazev() : "";
        if(nazev == m_nazev->GetValue())
            return false;
        else
            return idbmodel->hasKontakt(m_nazev->GetValue());
    }
    else
    {
        return idbmodel->hasKontakt(m_nazev->GetValue());
    }
}

