/***************************************************************
 * Name:      pohybzasobdialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include "pohybzasobdialog.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"

IMPLEMENT_DYNAMIC_CLASS(PohybzasobDialog, wxDialog)

BEGIN_EVENT_TABLE(PohybzasobDialog, wxDialog)
    EVT_BUTTON(wxID_OK, PohybzasobDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, PohybzasobDialog::OnCancel)
    EVT_DATE_CHANGED(ID_POHYBZASOB_DATUM, PohybzasobDialog::OnDatum)
END_EVENT_TABLE()

//Novy pohybzasob
PohybzasobDialog::PohybzasobDialog(wxWindow *parent, bool prijemka, int zasobaid)
    : wxDialog(parent, wxID_ANY, prijemka ? wxString::FromUTF8("Příjem na sklad") : wxString::FromUTF8("Výdej ze skladu")), m_edit(false), m_prijemka(prijemka), m_zasobaid(zasobaid)
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    createControls();

    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

//Editace pohybuzasob; nelze editovat pohyb z faktury nebo inventury
PohybzasobDialog::PohybzasobDialog(wxWindow *parent, int id)
    : wxDialog(parent, wxID_ANY, "Pohyb"), m_edit(true), m_id(id)
{
    Pohybzasob pz = idbmodel->getPohybzasob(id);
    if(pz.isValid())
    {
        m_prijemka = pz.prijemka();
        m_zasobaid = pz.zasobaid();
        SetTitle(m_prijemka ? wxString::FromUTF8("Příjem na sklad") : wxString::FromUTF8("Výdej ze skladu"));
    }

    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    createControls();

    if(pz.isValid())
    {
        m_datum->SetValue(pz.datum());
        m_cislo->SetValue(pz.cislo());
        m_pocet->SetValue(pz.pocet());
        if(m_prijemka) m_cenamj->SetValue(pz.cenamj());
    }

    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void PohybzasobDialog::createControls()
{
    wxGridBagSizer *gridSizer = new wxGridBagSizer(5,5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Zásoba")), wxGBPosition(0,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, idbmodel->getZasoba(m_zasobaid).nazev()), wxGBPosition(0,1), wxGBSpan(1,2), wxALIGN_CENTER_VERTICAL);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Datum pohybu"), wxGBPosition(1,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_datum = new wxDatePickerCtrl(this, ID_POHYBZASOB_DATUM, wxMax(idbmodel->getInventuraLastDateForZasoba(m_zasobaid), dxsettings.datum()), wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    m_datum->SetRange(wxMax(idbmodel->getInventuraLastDateForZasoba(m_zasobaid), wxDateTime(1,wxDateTime::Jan,idbmodel->currentRok())),wxDateTime(31,wxDateTime::Dec,idbmodel->currentRok()));
    if(m_edit) m_datum->Disable();
    gridSizer->Add(m_datum, wxGBPosition(1,1), wxGBSpan(1,2), wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Číslo")), wxGBPosition(2,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_cislo = new wxTextCtrl(this, wxID_ANY, idbmodel->getCisloDokladu(m_prijemka?"PR":"VY"));
    gridSizer->Add(m_cislo, wxGBPosition(2,1), wxGBSpan(1,2), wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Množství")), wxGBPosition(3,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_pocet = new wxSpinCtrlDouble(this, wxID_ANY);
    m_pocet->SetDigits(3);
    if(m_prijemka) m_pocet->SetRange(0.0, 99999999999999.0);
    else m_pocet->SetRange(0.0, idbmodel->stavZasobyToDate(m_zasobaid, m_edit?(idbmodel->hasPohybzasob(m_id)?idbmodel->getPohybzasob(m_id).datum():wxDateTime::Now()):wxDateTime::Now()));
    gridSizer->Add(m_pocet, wxGBPosition(3,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, idbmodel->getZasoba(m_zasobaid).mj()), wxGBPosition(3,2), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);

    if(m_prijemka)
    {
        gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Pořizovací cena")), wxGBPosition(4,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
        m_cenamj = new wxSpinCtrlDouble(this, wxID_ANY);
        m_cenamj->SetDigits(2);
        m_cenamj->SetRange(0.0, 1000000.0);
        gridSizer->Add(m_cenamj, wxGBPosition(4,1), wxGBSpan(1,2), wxEXPAND);
    }

    m_mainSizer->Add(gridSizer, 0, wxALL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);
}

void PohybzasobDialog::OnOk(wxCommandEvent& /*event*/)
{
    if(wxIsNullDouble(m_pocet->GetValue()))
    {
        wxMessageDialog dialog(this, m_prijemka?wxString::FromUTF8("Nulový příjem na sklad"):wxString::FromUTF8("Nulový výdej ze skladu"), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        m_pocet->SetFocus();
        return;
    }
    if(m_prijemka && wxIsNullDouble(m_cenamj->GetValue()))
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Nulová cena za měrnou jednotku"), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        m_cenamj->SetFocus();
        return;
    }
    wxDateTime datum = wxDateTime::Now();
    if(m_datum->GetValue().IsSameDate(idbmodel->getInventuraLastDateForZasoba(m_zasobaid)))
    {
        datum = idbmodel->getInventuraLastDateForZasoba(m_zasobaid)+wxTimeSpan(0,0,1);
    }
    else
    {
        datum.SetDay(m_datum->GetValue().GetDay());
        datum.SetMonth(m_datum->GetValue().GetMonth());
        datum.SetYear(m_datum->GetValue().GetYear());
    }
    if(!m_edit)
    {
        if(!m_prijemka)
        {
            double stav = idbmodel->stavZasobyToDate(m_zasobaid, datum);
            if(m_pocet->GetValue() > stav)
            {
                wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Stav zásoby %.3f na skladě je nižší než požadovaný výdej %.3f"), stav, m_pocet->GetValue()),
                                       "Informace", wxOK|wxICON_WARNING);
                dialog.ShowModal();
                m_pocet->SetRange(0.0, stav);
                m_pocet->SetFocus();
                return;
            }
        }
        idbmodel->addPohybzasob(Pohybzasob(idbmodel->getLastId("pohybzasob"), datum, m_cislo->GetValue(), m_pocet->GetValue(), m_prijemka?m_cenamj->GetValue():0.0, m_prijemka,
                                               false, -1, m_zasobaid, idbmodel->currentRok(), false, -1));
        if(m_prijemka)
        {
            idbmodel->increaseRadaCislo("PR");
        }
        else
        {
            idbmodel->increaseRadaCislo("VY");
        }
    }
    else
    {
        if(idbmodel->hasPohybzasob(m_id))
        {
            idbmodel->editPohybzasob(Pohybzasob(m_id, idbmodel->getPohybzasob(m_id).datum(), m_cislo->GetValue(), m_pocet->GetValue(), m_prijemka?m_cenamj->GetValue():0.0, m_prijemka,
                                                false, -1, m_zasobaid, idbmodel->currentRok(), false, -1));
        }
    }
    EndModal(wxID_OK);
}

void PohybzasobDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

void PohybzasobDialog::OnDatum(wxDateEvent &/*event*/)
{
    if(!m_prijemka)
    {
        wxDateTime datum = wxDateTime::Now();
        datum.SetDay(m_datum->GetValue().GetDay());
        datum.SetMonth(m_datum->GetValue().GetMonth());
        datum.SetYear(m_datum->GetValue().GetYear());
        m_pocet->SetRange(0.0, idbmodel->stavZasobyToDate(m_zasobaid, datum));
    }
}
