/***************************************************************
 * Name:      pokladnadialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include "pokladnadialog.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"

IMPLEMENT_DYNAMIC_CLASS(PokladnaDialog, wxDialog)

BEGIN_EVENT_TABLE(PokladnaDialog, wxDialog)
    EVT_BUTTON(wxID_OK, PokladnaDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, PokladnaDialog::OnCancel)
END_EVENT_TABLE()

PokladnaDialog::PokladnaDialog(wxWindow *parent, int id, bool edit)
    : wxDialog(parent, wxID_ANY, edit ? "Pokladna" : "Zadejte pokladnu"), m_edit(edit)
{
    createControls();
    if(m_edit)
    {
        m_id = id;
    }
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void PokladnaDialog::createControls()
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Zkratka"), wxGBPosition(0,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_zkratka = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_zkratka, wxGBPosition(0,1), wxDefaultSpan, wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Popis"), wxGBPosition(1,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_popis = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_popis, wxGBPosition(1,1), wxDefaultSpan, wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Měna")), wxGBPosition(2,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_mena = new wxChoice(this, wxID_ANY);
    if(m_edit) m_mena->Disable();
    gridSizer->Add(m_mena, wxGBPosition(2,1), wxDefaultSpan, wxEXPAND);

    m_mainSizer->Add(gridSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void PokladnaDialog::fillControls()
{
    if(m_edit)
    {
        Pokladna pk = idbmodel->getPokladna(m_id);
        if(pk.isValid())
        {
            m_zkratka->SetValue(pk.zkratka());
            m_popis->SetValue(pk.popis());
            dxMenaArray meny = idbmodel->meny();
            for(size_t i=0; i<meny.GetCount(); i++)
            {
                m_mena->Append(idbmodel->nazevMena(meny[i].id()));
                if(pk.menaid() == meny[i].id())
                    m_mena->SetSelection(static_cast<int>(i));
            }
        }
    }
    else
    {
        dxMenaArray meny = idbmodel->meny();
        for(size_t i=0; i<meny.GetCount(); i++)
            m_mena->Append(idbmodel->nazevMena(meny[i].id()));
        m_mena->SetSelection(0);
    }
}

void PokladnaDialog::OnOk(wxCommandEvent& /*event*/)
{
    if(m_zkratka->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Zkratka nemůže být prázdná"), "Informace", m_edit? wxOK|wxICON_INFORMATION : wxOK|wxICON_WARNING);
        dialog.ShowModal();
        return;
    }
    if(zkratkaExists())
    {
        wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Zkratka '%s' už existuje\nZvolte jinou"), m_zkratka->GetValue()), "Informace", wxOK|wxICON_INFORMATION);
        dialog.ShowModal();
        return;
    }
    if(!m_edit)
    {
        idbmodel->addPokladna(Pokladna(idbmodel->getLastId("pokladna"), m_zkratka->GetValue(), m_popis->GetValue(),
                                    idbmodel->itemMena(m_mena->GetSelection()).id()));
    }
    else
    {
        idbmodel->editPokladna(Pokladna(m_id, m_zkratka->GetValue(), m_popis->GetValue(),
                                        idbmodel->itemMena(m_mena->GetSelection()).id()));
    }
    EndModal(wxID_OK);
}

void PokladnaDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

bool PokladnaDialog::zkratkaExists()
{
    if(m_edit)
    {
        if(idbmodel->getPokladnaZkratka(m_id) == m_zkratka->GetValue())
            return false;
        else
            return idbmodel->hasPokladna(m_zkratka->GetValue());
    }
    else
    {
        return idbmodel->hasPokladna(m_zkratka->GetValue());
    }
}

