/***************************************************************
 * Name:      predkontacedialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include "predkontacedialog.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"

IMPLEMENT_DYNAMIC_CLASS(PredkontaceDialog, wxDialog)

BEGIN_EVENT_TABLE(PredkontaceDialog, wxDialog)
    EVT_BUTTON(wxID_OK, PredkontaceDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, PredkontaceDialog::OnCancel)
    EVT_CHECKBOX(ID_PREDKONTACE_CHECK, PredkontaceDialog::OnPrijem)
END_EVENT_TABLE()

PredkontaceDialog::PredkontaceDialog(wxWindow *parent, int id, int edit)
    : wxDialog(parent, wxID_ANY, edit ? wxString::FromUTF8("Předkontace") : wxString::FromUTF8("Zadejte předkontaci")), m_edit(edit)
{
    createControls();
    if(m_edit)
    {
        m_id = id;
    }
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void PredkontaceDialog::createControls()
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Zkratka"), wxGBPosition(0,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_zkratka = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_zkratka, wxGBPosition(0,1), wxDefaultSpan, wxEXPAND);
    m_prijem = new wxCheckBox(this, ID_PREDKONTACE_CHECK, wxString::FromUTF8("Příjem"));
    m_prijem->SetValue(true);
    gridSizer->Add(m_prijem, wxGBPosition(0,2), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Popis"), wxGBPosition(1,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_popis = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_popis, wxGBPosition(1,1), wxGBSpan(1,2), wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Sloupec deníku")), wxGBPosition(2,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_sloupec = new wxChoice(this, wxID_ANY);
    gridSizer->Add(m_sloupec, wxGBPosition(2,1), wxGBSpan(1,2), wxEXPAND);

    m_mainSizer->Add(gridSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void PredkontaceDialog::fillControls()
{
    m_prijmy = idbmodel->deniksloupcePrijem();
    m_vydaje = idbmodel->deniksloupceVydaj();
    if(m_edit)
    {
        Predkontace pk = idbmodel->getPredkontace(m_id);
        if(pk.isValid())
        {
            m_zkratka->SetValue(pk.zkratka());
            m_popis->SetValue(pk.popis());
            m_prijem->SetValue(pk.prijem());
            if(pk.prijem())
            {
                for(size_t i=0; i<m_prijmy.GetCount(); i++)
                {
                    m_sloupec->Append(m_prijmy[i].popis());
                    if(pk.deniksloupecid() == m_prijmy[i].sloupec())
                        m_sloupec->SetSelection((int)i);
                }
            }
            else
            {
                for(size_t i=0; i<m_vydaje.GetCount(); i++)
                {
                    m_sloupec->Append(m_vydaje[i].popis());
                    if(pk.deniksloupecid() == m_vydaje[i].sloupec())
                        m_sloupec->SetSelection((int)i);
                }
            }
        }
    }
    else
    {
        m_prijem->SetValue(true);
        for(size_t i=0; i<m_prijmy.GetCount(); i++)
            m_sloupec->Append(m_prijmy[i].popis());
        m_sloupec->SetSelection(0);
    }
}

void PredkontaceDialog::OnOk(wxCommandEvent& /*event*/)
{
    if(m_zkratka->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Zkratka nemůže být prázdná"), "Informace", m_edit? wxOK|wxICON_INFORMATION : wxOK|wxICON_WARNING);
        dialog.ShowModal();
        m_zkratka->SetFocus();
        return;
    }
    if(zkratkaExists())
    {
        wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Zkratka '%s' už existuje\nZvolte jinou"), m_zkratka->GetValue()), "Informace", wxOK|wxICON_INFORMATION);
        dialog.ShowModal();
        m_zkratka->SetFocus();
        return;
    }
    if(!m_edit)
    {
        idbmodel->addPredkontace(Predkontace(idbmodel->getLastId("predkontace"), m_zkratka->GetValue(), m_popis->GetValue(),
                                             m_prijem->IsChecked(), m_prijem->IsChecked() ?
                                             m_prijmy[m_sloupec->GetSelection()].sloupec() :
                                             m_vydaje[m_sloupec->GetSelection()].sloupec()));
    }
    else
    {
        idbmodel->editPredkontace(Predkontace(m_id, m_zkratka->GetValue(), m_popis->GetValue(),
                                             m_prijem->IsChecked(), m_prijem->IsChecked() ?
                                             m_prijmy[m_sloupec->GetSelection()].sloupec() :
                                             m_vydaje[m_sloupec->GetSelection()].sloupec()));
    }
    EndModal(wxID_OK);
}

void PredkontaceDialog::OnCancel(wxCommandEvent& /*event*/)
{
    if(m_edit == 2) idbmodel->deletePredkontace(m_id);
    EndModal(wxID_CANCEL);
}

void PredkontaceDialog::OnPrijem(wxCommandEvent& /*event*/)
{
    m_sloupec->Clear();
    if(m_prijem->IsChecked())
    {
        for(size_t i=0; i<m_prijmy.GetCount(); i++)
        {
            m_sloupec->Append(m_prijmy[i].popis());
        }
    }
    else
    {
        for(size_t i=0; i<m_vydaje.GetCount(); i++)
        {
            m_sloupec->Append(m_vydaje[i].popis());
        }
    }
    m_sloupec->SetSelection(0);
}

bool PredkontaceDialog::zkratkaExists()
{
    if(m_edit)
    {
        wxString zkratka = idbmodel->hasPredkontace(m_id) ? idbmodel->getPredkontace(m_id).zkratka() : "";
        if(zkratka == m_zkratka->GetValue())
            return false;
        else
            return idbmodel->hasPredkontace(m_zkratka->GetValue());
    }
    else
    {
        return idbmodel->hasPredkontace(m_zkratka->GetValue());
    }
}

