/***************************************************************
 * Name:      predvolbydialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include <wx/filename.h>
#include "predvolbydialog.h"
#include "radadialog.h"
#include "dxdefs.h"
#include "utils/dxsettings.h"
#include "resources/images/rady32.xpm"
#include "resources/images/seznam32.xpm"
#include "resources/images/email32.xpm"

IMPLEMENT_DYNAMIC_CLASS(RadyPanel, wxPanel)

BEGIN_EVENT_TABLE(RadyPanel, wxPanel)
    EVT_LIST_ITEM_RIGHT_CLICK(ID_PREDVOLBY_RADY_LIST, RadyPanel::OnItemRightClick)
    EVT_LIST_ITEM_ACTIVATED(ID_PREDVOLBY_RADY_LIST, RadyPanel::OnItemActivated)
    EVT_LIST_ITEM_SELECTED(ID_PREDVOLBY_RADY_LIST, RadyPanel::OnItemSelected)
    EVT_LIST_ITEM_DESELECTED(ID_PREDVOLBY_RADY_LIST, RadyPanel::OnItemDeselected)
    EVT_MENU(ID_PREDVOLBY_RADY_EDIT, RadyPanel::OnEditRada)
END_EVENT_TABLE()

RadyPanel::RadyPanel(wxWindow *parent)
    : wxPanel(parent), m_selectedId(wxEmptyString)
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    m_list = new wxListCtrl(this, ID_PREDVOLBY_RADY_LIST, wxDefaultPosition, wxDefaultSize,
                            wxLC_REPORT|wxLC_HRULES|wxLC_VRULES|wxLC_SINGLE_SEL);
    wxListItem col0;
    col0.SetId(0);
    col0.SetText("Popis");
    col0.SetWidth(250);
    m_list->InsertColumn(0, col0);
    wxListItem col1;
    col1.SetId(1);
    col1.SetText("Prefix");
    col1.SetWidth(50);
    m_list->InsertColumn(1, col1);
    wxListItem col2;
    col2.SetId(2);
    col2.SetText(wxString::FromUTF8("Znaků"));
    col2.SetWidth(50);
    m_list->InsertColumn(2, col2);
    wxListItem col3;
    col3.SetId(3);
    col3.SetText(wxString::FromUTF8("Aktuální číslo"));
    col3.SetWidth(115);
    m_list->InsertColumn(3, col3);
    fillList();
    m_mainSizer->Add(m_list, 1, wxALL|wxEXPAND, 5);
    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void RadyPanel::OnItemRightClick(wxListEvent &event)
{
    setSelectedId(event.GetIndex());
    if(!m_selectedId.IsEmpty())
    {
        wxMenu popup;
        popup.Append(ID_PREDVOLBY_RADY_EDIT, wxString::FromUTF8("Upravit řadu"));
        PopupMenu(&popup, event.GetPoint());
    }
}

void RadyPanel::OnItemActivated(wxListEvent &event)
{
    setSelectedId(event.GetIndex());
    RadaDialog dialog(this, m_selectedId);
    if(dialog.ShowModal() == wxID_OK)
        fillList();
}

void RadyPanel::OnItemSelected(wxListEvent &event)
{
    setSelectedId(event.GetIndex());
}

void RadyPanel::OnItemDeselected(wxListEvent& /*event*/)
{
    setSelectedId(-1);
}

void RadyPanel::OnEditRada(wxCommandEvent& /*event*/)
{
    if(m_selectedId.IsEmpty()) return;
    RadaDialog dialog(this, m_selectedId);
    if(dialog.ShowModal() == wxID_OK)
        fillList();
}

void RadyPanel::setSelectedId(long selectedIndex)
{
    if(selectedIndex < 0)
        m_selectedId = wxEmptyString;
    else
        m_selectedId = m_rady[selectedIndex].id();
}

void RadyPanel::fillList()
{
    m_list->DeleteAllItems();
    m_list->Hide();
    m_rady = idbmodel->rady();
    for(size_t i=0; i < m_rady.GetCount(); i++)
    {
        wxListItem item;
        item.SetId(static_cast<long>(i));
        item.SetText(m_rady[i].popis());
        m_list->InsertItem(item);
        m_list->SetItem(static_cast<long>(i), 1, m_rady[i].prefix());
        m_list->SetItem(static_cast<long>(i), 2, wxString::Format("%d", m_rady[i].znaku()));
        m_list->SetItem(static_cast<long>(i), 3, wxString::Format("%d", m_rady[i].cislo()));
    }
    m_list->Show();
    setSelectedId(-1);
}

wxString RadyPage::GetName() const
{
    return wxString::FromUTF8("Číselné řady");
}

wxBitmap RadyPage::GetLargeIcon() const
{
    return wxBitmap(rady32_xpm);
}

wxWindow *RadyPage::CreateWindow(wxWindow *parent)
{
    return new RadyPanel(parent);
}

FakturacePanel::FakturacePanel(wxWindow *parent)
    : wxPanel(parent)
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Splatnost"), wxGBPosition(0,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_splatnost = new wxSpinCtrl(this);
    m_splatnost->SetRange(0,1000);
    gridSizer->Add(m_splatnost, wxGBPosition(0,1), wxDefaultSpan, wxEXPAND);
    m_mainSizer->Add(gridSizer, 0, wxALL, 5);
    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);

    if(wxPreferencesEditor::ShouldApplyChangesImmediately())
    {
        m_splatnost->Connect(wxEVT_SPINCTRL, wxSpinEventHandler(FakturacePanel::SaveSplatnost), nullptr, this);
    }
}

bool FakturacePanel::TransferDataToWindow()
{
    m_splatnost->SetValue(dxsettings.splatnost());
    return true;
}

bool FakturacePanel::TransferDataFromWindow()
{
    wxSpinEvent dummy;
    SaveSplatnost(dummy);
    return true;
}

void FakturacePanel::SaveSplatnost(wxSpinEvent &/*event*/)
{
    dxsettings.setSplatnost(m_splatnost->GetValue());
    idbmodel->editSettings(Settings(idbmodel->currentRok(), m_splatnost->GetValue()));
}

wxString FakturacePage::GetName() const
{
    return "Fakturace";
}

wxBitmap FakturacePage::GetLargeIcon() const
{
    return wxBitmap(seznam32_xpm);
}

wxWindow *FakturacePage::CreateWindow(wxWindow *parent)
{
    return new FakturacePanel(parent);
}


wxString EmailPage::GetName() const
{
    return "E-mail";
}

wxBitmap EmailPage::GetLargeIcon() const
{
    return wxBitmap(email32_xpm);
}

wxWindow *EmailPage::CreateWindow(wxWindow *parent)
{
    return new EmailPanel(parent);
}


EmailPanel::EmailPanel(wxWindow *parent)
    : wxPanel(parent), m_useSsl(false)
{
    int i = 0;
#ifdef CURL_VERSION_SSL
    m_useSsl = true;
#endif

    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "SMTP server"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_server = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_server, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Port"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_port = new wxSpinCtrl(this);
    m_port->SetRange(0,65535);
    gridSizer->Add(m_port, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    if(m_useSsl)
    {
        gridSizer->Add(new wxStaticText(this, wxID_ANY, "SSL"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
        m_ssl = new wxChoice(this, wxID_ANY);
        m_ssl->Append(wxString::FromUTF8("Žádné"));
        m_ssl->Append("SSL/TLS");
        gridSizer->Add(m_ssl, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
        i++;
    }

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Použít autentifikaci")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_auth = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_auth, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Uživatelské jméno")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_username = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_username, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Heslo"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_password = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_password, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Odesílatel")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_sender = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_sender, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Předmět")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_subject = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_subject, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Text"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_text = new wxTextCtrl(this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxSize(300,70), wxTE_MULTILINE);
    gridSizer->Add(m_text, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);

    m_mainSizer->Add(gridSizer, 0, wxALL, 5);
    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);

    if(wxPreferencesEditor::ShouldApplyChangesImmediately())
    {
        m_server->Connect(wxEVT_TEXT, wxTextEventHandler(EmailPanel::OnCommandEvent), nullptr, this);
        m_port->Connect(wxEVT_SPINCTRL, wxSpinEventHandler(EmailPanel::OnSpinEvent), nullptr, this);
        if(m_useSsl)
        {
            m_ssl->Connect(wxEVT_CHOICE, wxCommandEventHandler(EmailPanel::OnCommandEvent), nullptr, this);
        }
        m_auth->Connect(wxEVT_CHECKBOX, wxCommandEventHandler(EmailPanel::OnCommandEvent), nullptr, this);
        m_username->Connect(wxEVT_TEXT, wxTextEventHandler(EmailPanel::OnCommandEvent), nullptr, this);
        m_password->Connect(wxEVT_TEXT, wxTextEventHandler(EmailPanel::OnCommandEvent), nullptr, this);
        m_sender->Connect(wxEVT_TEXT, wxTextEventHandler(EmailPanel::OnCommandEvent), nullptr, this);
        m_subject->Connect(wxEVT_TEXT, wxTextEventHandler(EmailPanel::OnCommandEvent), nullptr, this);
        m_text->Connect(wxEVT_TEXT, wxTextEventHandler(EmailPanel::OnCommandEvent), nullptr, this);
    }
}

bool EmailPanel::TransferDataToWindow()
{
    Smtp smtp = idbmodel->smtp();
    m_server->SetValue(smtp.server());
    m_port->SetValue(smtp.port());
    if(m_useSsl)
    {
        m_ssl->Select(smtp.ssl());
    }
    m_auth->SetValue(smtp.autentifikace());
    m_username->SetValue(smtp.username());
    m_password->SetValue(smtp.password());
    m_sender->SetValue(smtp.sender());
    m_subject->SetValue(smtp.subject());
    m_text->SetValue(smtp.text());

    return true;
}

bool EmailPanel::TransferDataFromWindow()
{
    if(m_useSsl)
    {
        idbmodel->setSmtp(Smtp(m_server->GetValue(), m_port->GetValue(), m_ssl->GetSelection(), m_auth->GetValue(), m_username->GetValue(),
                               m_password->GetValue(), m_sender->GetValue(), m_subject->GetValue(), m_text->GetValue()));
    }
    else
    {
        idbmodel->setSmtp(Smtp(m_server->GetValue(), m_port->GetValue(), 0, m_auth->GetValue(), m_username->GetValue(),
                               m_password->GetValue(), m_sender->GetValue(), m_subject->GetValue(), m_text->GetValue()));
    }
    return true;
}

void EmailPanel::OnCommandEvent(wxCommandEvent& /*event*/)
{
    TransferDataFromWindow();
}

void EmailPanel::OnSpinEvent(wxSpinEvent& /*event*/)
{
    TransferDataFromWindow();
}
