/***************************************************************
 * Name:      prijemvydejdialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include "prijemvydejdialog.h"
#include "resources/images/download.xpm"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "bankydialog.h"
#include "pokladnydialog.h"
#include "predkontacesdialog.h"
#include "kontaktydialog.h"
#include <wx/stdpaths.h>
#include <wx/valnum.h>
#include <wx/url.h>
#include <wx/wfstream.h>
#include <wx/tokenzr.h>
#include <wx/numdlg.h>
#include <memory>

IMPLEMENT_DYNAMIC_CLASS(PrijemVydejDialog, wxDialog)

BEGIN_EVENT_TABLE(PrijemVydejDialog, wxDialog)
    EVT_BUTTON(wxID_OK, PrijemVydejDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, PrijemVydejDialog::OnCancel)
    EVT_BUTTON(ID_PRIJEMVYDEJ_BANKAPOKLADNA_SELECT, PrijemVydejDialog::OnSelectBankaPokladna)
    EVT_BUTTON(ID_PRIJEMVYDEJ_PREDKONTACE_SELECT, PrijemVydejDialog::OnSelectPredkontace)
    EVT_BUTTON(ID_PRIJEMVYDEJ_PROTISTRANA_SELECT, PrijemVydejDialog::OnSelectProtistrana)
    EVT_CHOICE(ID_PRIJEMVYDEJ_MENA_CHOICE, PrijemVydejDialog::OnChoiceMena)
    EVT_BUTTON(ID_PRIJEMVYDEJ_KURZ_DOWNLOAD, PrijemVydejDialog::OnKurzDownload)
    EVT_DATE_CHANGED(ID_PRIJEMVYDEJ_DATUMCTRL, PrijemVydejDialog::OnDatumChanged)
    EVT_BUTTON(ID_PRIJEMVYDEJ_PAUSAL_ADD, PrijemVydejDialog::OnPausalAdd)
END_EVENT_TABLE()

PrijemVydejDialog::PrijemVydejDialog(wxWindow *parent, int id, int edit, bool prijem, bool pokladna, bool banka)
    : wxDialog(parent, wxID_ANY, wxEmptyString), m_edit(edit), m_prijem(prijem), m_pokladna(pokladna), m_banka(banka)
{
    if(m_edit)
    {
        m_id = id;
        m_uhrada = idbmodel->isDenikUhrada(id);
    }
    else
    {
        m_uhrada = false;
    }
    createControls();
    fillControls();
    if(prijem)
    {
        if(edit) SetTitle(wxString::FromUTF8("Příjem"));
        else SetTitle(wxString::FromUTF8("Zadejte příjem"));
    }
    else
    {
        if(edit) SetTitle(wxString::FromUTF8("Výdaj"));
        else SetTitle(wxString::FromUTF8("Zadejte výdaj"));
    }
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void PrijemVydejDialog::createControls()
{
    int i=0; //row

    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Datum"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_datum = new wxDatePickerCtrl(this, ID_PRIJEMVYDEJ_DATUMCTRL, dxsettings.datum(), wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    m_datum->SetRange(wxDateTime(1,wxDateTime::Jan,idbmodel->currentRok()),dxsettings.datum());
    gridSizer->Add(m_datum, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);
    i++;

    if(!m_pokladna && !m_banka) //Musi se vybrat kam to zauctovat
    {
        gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Zaúčtovat")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
        m_bankapokladnaChoice = new wxChoice(this, ID_PRIJEMVYDEJ_BANKAPOKLADNA_CHOICE);
        gridSizer->Add(m_bankapokladnaChoice, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);
        i++;
    }

    m_bankapokladnaLabel = new wxStaticText(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_bankapokladnaLabel, wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_bankapokladnaText = new wxTextCtrl(this, wxID_ANY);
    if(m_uhrada) m_bankapokladnaText->Disable();
    gridSizer->Add(m_bankapokladnaText, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_bankapokladnaSelect = new wxButton(this, ID_PRIJEMVYDEJ_BANKAPOKLADNA_SELECT, "...");
    if(m_uhrada) m_bankapokladnaSelect->Disable();
    gridSizer->Add(m_bankapokladnaSelect, wxGBPosition(i,2));
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Předkontace")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_predkontaceText = new wxTextCtrl(this, wxID_ANY);
    if(m_uhrada) m_predkontaceText->Disable();
    gridSizer->Add(m_predkontaceText, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_predkontaceSelect = new wxButton(this, ID_PRIJEMVYDEJ_PREDKONTACE_SELECT, "...");
    if(m_uhrada) m_predkontaceSelect->Disable();
    m_predkontaceSelect->SetToolTip(wxString::FromUTF8("Vyberte předkontaci"));
    gridSizer->Add(m_predkontaceSelect, wxGBPosition(i,2));
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Protistrana"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_protistranaText = new wxTextCtrl(this, wxID_ANY);
    if(m_uhrada) m_protistranaText->Disable();
    gridSizer->Add(m_protistranaText, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_protistranaSelect = new wxButton(this, ID_PRIJEMVYDEJ_PROTISTRANA_SELECT, "...");
    if(m_uhrada) m_protistranaSelect->Disable();
    m_protistranaSelect->SetToolTip("Vyberte protistranu");
    gridSizer->Add(m_protistranaSelect, wxGBPosition(i,2));
    i++;

    if(m_prijem && idbmodel->isFirmaPausal())
    {
        gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Paušální výdaj %")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
        m_pausal = new wxChoice(this, wxID_ANY);
        m_pausal->Append("80");
        m_pausal->Append("60");
        m_pausal->Append("40");
        m_pausal->Append("30");
        m_pausal->Append("0");
        m_pausal->Select(1);
        gridSizer->Add(m_pausal, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
        m_pausalAdd = new wxButton(this, ID_PRIJEMVYDEJ_PAUSAL_ADD, "...");
        m_pausalAdd->SetToolTip(wxString::FromUTF8("Přidat hodnotu paušálních výdajů"));
        gridSizer->Add(m_pausalAdd, wxGBPosition(i,2));
        i++;
    }

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Popis dokladu"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_popis = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_popis, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Číslo dokladu")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_cislodokladu = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_cislodokladu, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);
    i++;

    m_vypisLabel = new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Číslo výpisu"));
    gridSizer->Add(m_vypisLabel, wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_vypis = new wxSpinCtrl(this);
    m_vypis->SetRange(1,65000);
    gridSizer->Add(m_vypis, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);
    i++;
    if(!m_banka)
    {
        m_vypisLabel->Hide();
        m_vypis->Hide();
    }

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Referenční doklad")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_reference = new wxTextCtrl(this, wxID_ANY);
    if(m_uhrada) m_reference->Disable();
    m_reference->SetToolTip(wxString::FromUTF8("Číslo faktury, variabilní symbol atd."));
    gridSizer->Add(m_reference, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Měna")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_mena = new wxChoice(this, ID_PRIJEMVYDEJ_MENA_CHOICE);
    if(m_uhrada) m_mena->Disable();
    gridSizer->Add(m_mena, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);
    i++;

    m_kurzLabel = new wxStaticText(this, wxID_ANY, "Kurz");
    gridSizer->Add(m_kurzLabel, wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_kurz = new ValueCtrl(this, ID_PRIJEMVYDEJ_KURZ_TEXT, "", wxDefaultPosition, wxDefaultSize, wxTE_RIGHT,
                                wxMakeFloatingPointValidator(6, &m_kurzValue, wxNUM_VAL_THOUSANDS_SEPARATOR));

    m_kurzDownload = new wxBitmapButton(this, ID_PRIJEMVYDEJ_KURZ_DOWNLOAD, wxBitmap(download_xpm));
    m_kurzDownload->SetBitmapDisabled(dxutils::GreyScaledIcon(download_xpm));
    m_kurzDownload->SetToolTip(wxString::FromUTF8("Stáhnout kurz z webu ČNB"));
#if wxUSE_URL
    gridSizer->Add(m_kurz, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(m_kurzDownload, wxGBPosition(i,2));
#else
    gridSizer->Add(m_kurz, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);
#endif
    i++;

    m_castkaciziLabel = new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Částka v cizí měně"));
    gridSizer->Add(m_castkaciziLabel, wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_castkacizi = new ValueCtrl(this, ID_PRIJEMVYDEJ_CASTKACIZI_TEXT, "", wxDefaultPosition, wxDefaultSize, wxTE_RIGHT,
                                wxMakeFloatingPointValidator(2, &m_castkaciziValue, wxNUM_VAL_THOUSANDS_SEPARATOR));
    gridSizer->Add(m_castkacizi, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Částka v Kč")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_castka = new ValueCtrl(this, ID_PRIJEMVYDEJ_CASTKA_TEXT, "", wxDefaultPosition, wxDefaultSize, wxTE_RIGHT,
                             wxMakeFloatingPointValidator(2, &m_castkaValue, wxNUM_VAL_THOUSANDS_SEPARATOR));
    gridSizer->Add(m_castka, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);

    gridSizer->SetEmptyCellSize(wxSize(0,0));
    m_mainSizer->Add(gridSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void PrijemVydejDialog::fillControls()
{
    if(m_edit)
    {
        Denik dk = idbmodel->getDenik(m_id);
        if(dk.isValid())
        {
            m_datum->SetValue(dk.datum());
            if(!m_pokladna && !m_banka)
            {
                m_bankapokladnaChoice->Append("Pokladna");
                m_bankapokladnaChoice->Append(wxString::FromUTF8("Bankovní účet"));
                if(idbmodel->hasPokladna(dk.pokladnaid()))
                {
                    m_bankapokladnaChoice->SetSelection(0);
                    m_pokladna = true;
                    m_banka = false;
                }
                else
                {
                    m_bankapokladnaChoice->SetSelection(1);
                    m_pokladna = false;
                    m_banka = true;
                }
            }
            if(m_pokladna)
            {
                int mena = 0;
                m_bankapokladnaLabel->SetLabel("Pokladna");
                m_bankapokladnaText->SetValue(idbmodel->getPokladnaZkratka(dk.pokladnaid()));
                m_bankapokladnaSelect->SetToolTip("Vyberte pokladnu");
                Pokladna pk = idbmodel->getPokladna(dk.pokladnaid());
                if(pk.isValid() && pk.menaid() != 1)
                {
                    Mena mn = idbmodel->getMena(pk.menaid());
                    m_mena->Append(idbmodel->nazevMena(pk.menaid()), new Mena(mn.id(), mn.kod(), mn.nazev()));
                }
                else
                {
                    dxMenaArray meny = idbmodel->meny();
                    for(size_t i=0; i<meny.GetCount(); i++)
                    {
                        if(dk.menaid() == meny[i].id()) mena = static_cast<int>(i);
                        m_mena->Append(idbmodel->nazevMena(meny[i].id()), new Mena(meny[i].id(), meny[i].kod(), meny[i].nazev()));
                    }
                }
                m_mena->SetSelection(mena);
            }
            else
            {
                int mena = 0;
                m_bankapokladnaLabel->SetLabel(wxString::FromUTF8("Bankovní účet"));
                m_bankapokladnaText->SetValue(idbmodel->getBankaZkratka(dk.bankaid()));
                m_bankapokladnaSelect->SetToolTip(wxString::FromUTF8("Vyberte bankovní účet"));
                Banka bk = idbmodel->getBanka(dk.bankaid());
                if(bk.isValid() && bk.menaid() != 1)
                {
                    Mena mn = idbmodel->getMena(bk.menaid());
                    m_mena->Append(idbmodel->nazevMena(bk.menaid()), new Mena(mn.id(), mn.kod(), mn.nazev()));
                }
                else
                {
                    dxMenaArray meny = idbmodel->meny();
                    for(size_t i=0; i<meny.GetCount(); i++)
                    {
                        if(dk.menaid() == meny[i].id()) mena = static_cast<int>(i);
                        m_mena->Append(idbmodel->nazevMena(meny[i].id()), new Mena(meny[i].id(), meny[i].kod(), meny[i].nazev()));
                    }
                }
                m_mena->SetSelection(mena);
            }
            m_predkontaceText->SetValue(idbmodel->hasPredkontace(dk.predkontaceid())?idbmodel->getPredkontace(dk.predkontaceid()).zkratka():"");
            m_protistranaText->SetValue(idbmodel->hasKontakt(dk.protistranaid())?idbmodel->getKontakt(dk.protistranaid()).nazev():"");
            m_popis->SetValue(dk.popis());
            m_cislodokladu->SetValue(dk.cislo());
            m_vypis->SetValue(dk.vypis());
            m_castkaValue = dk.castka();
            m_castkaciziValue = dk.castkacizi();
            m_kurzValue = dk.kurz();
            m_reference->SetValue(dk.reference());
            if(m_prijem && idbmodel->isFirmaPausal())
            {
                wxString value(wxString::Format("%d", dk.pausal()));
                if(m_pausal->FindString(value) == wxNOT_FOUND)
                {
                    wxArrayString values = m_pausal->GetStrings();
                    m_pausal->Clear();
                    values.Add(value);
                    values.Sort(true);
                    m_pausal->Append(values);
                    for(size_t i=0; i<values.GetCount(); i++)
                    {
                        if(values[i] == value)
                        {
                            m_pausal->SetSelection(static_cast<int>(i));
                            break;
                        }
                    }
                }
                else
                {
                    m_pausal->SetSelection(m_pausal->FindString(value));
                }
            }
        }
        if(getMenaId() != 1)
        {
            if(idbmodel->hasKurz(static_cast<Mena*>(m_mena->GetClientData(static_cast<unsigned int>(m_mena->GetSelection())))->kod(), m_datum->GetValue())) m_kurzDownload->Disable();
            else m_kurzDownload->Enable();
        }
        showhideKurz();
    }
    else
    {
        fillMena();
        m_datum->SetValue(dxsettings.datum());
        if(!m_pokladna && !m_banka)
        {
            m_bankapokladnaChoice->Append("Pokladna");
            m_bankapokladnaChoice->Append(wxString::FromUTF8("Bankovní účet"));
            m_bankapokladnaChoice->SetSelection(0);
            m_pokladna = true;
            m_banka = false;
        }
        if(m_pokladna)
        {
            m_bankapokladnaLabel->SetLabel("Pokladna");
            m_bankapokladnaSelect->SetToolTip("Vyberte pokladnu");
            if(m_prijem) m_cislodokladu->SetValue(idbmodel->getCisloDokladu("PP"));
            else m_cislodokladu->SetValue(idbmodel->getCisloDokladu("PV"));
        }
        else
        {
            m_bankapokladnaLabel->SetLabel(wxString::FromUTF8("Bankovní účet"));
            m_bankapokladnaSelect->SetToolTip(wxString::FromUTF8("Vyberte bankovní účet"));
            if(m_prijem) m_cislodokladu->SetValue(idbmodel->getCisloDokladu("BP"));
            else m_cislodokladu->SetValue(idbmodel->getCisloDokladu("BV"));
        }
        m_bankapokladnaText->SetValue(wxEmptyString);
        m_predkontaceText->SetValue(wxEmptyString);
        m_castkaValue = 0.0;
        m_vypis->SetValue(1);
    }
    Connect(ID_PRIJEMVYDEJ_BANKAPOKLADNA_CHOICE, wxEVT_CHOICE, wxCommandEventHandler(PrijemVydejDialog::OnChoiceBankaPokladna), nullptr, this);
}

void PrijemVydejDialog::fillMena()
{
    m_mena->Clear();
    if(m_pokladna)
    {
        Pokladna pk = idbmodel->getPokladna(m_bankapokladnaText->GetValue());
        if(pk.isValid() && pk.menaid() != 1)
        {
            Mena mn = idbmodel->getMena(pk.menaid());
            m_mena->Append(idbmodel->nazevMena(pk.menaid()), new Mena(mn.id(), mn.kod(), mn.nazev()));
        }
        else
        {
            dxMenaArray meny = idbmodel->meny();
            for(size_t i=0; i<meny.GetCount(); i++)
                m_mena->Append(idbmodel->nazevMena(meny[i].id()), new Mena(meny[i].id(), meny[i].kod(), meny[i].nazev()));
        }
    }
    else
    {
        Banka bk = idbmodel->getBanka(m_bankapokladnaText->GetValue());
        if(bk.isValid() && bk.menaid() != 1)
        {
            Mena mn = idbmodel->getMena(bk.menaid());
            m_mena->Append(idbmodel->nazevMena(bk.menaid()), new Mena(mn.id(), mn.kod(), mn.nazev()));
        }
        else
        {
            dxMenaArray meny = idbmodel->meny();
            for(size_t i=0; i<meny.GetCount(); i++)
                m_mena->Append(idbmodel->nazevMena(meny[i].id()), new Mena(meny[i].id(), meny[i].kod(), meny[i].nazev()));
        }
    }
    m_mena->Select(0);
    wxCommandEvent ev;
    ev.SetInt(0);
    OnChoiceMena(ev);
}

int PrijemVydejDialog::getMenaId() const
{
    if(m_mena->HasClientUntypedData())
        return static_cast<Mena*>(m_mena->GetClientData(static_cast<unsigned int>(m_mena->GetSelection())))->id();
    return 1;
}

int PrijemVydejDialog::getMenaId(unsigned int index) const
{
    if(m_mena->HasClientUntypedData())
        return static_cast<Mena*>(m_mena->GetClientData(index))->id();
    return 1;
}

void PrijemVydejDialog::OnOk(wxCommandEvent& event)
{
    if(m_bankapokladnaText->GetValue().IsEmpty())
    {
        wxString msg;
        if(m_pokladna) msg = wxString::FromUTF8("Pokladna nemůže být prázdná");
        else msg = wxString::FromUTF8("Bankovní účet nemůže být prázdný");
        wxMessageDialog dialog(this, msg, "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        OnSelectBankaPokladna(event);
        return;
    }
    if(m_pokladna && !idbmodel->hasPokladna(m_bankapokladnaText->GetValue()))
    {
        wxMessageDialog dialog(this, wxString::Format("Pokladna %s neexistuje", m_bankapokladnaText->GetValue()), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        OnSelectBankaPokladna(event);
        return;
    }
    if(!m_pokladna && !idbmodel->hasBanka(m_bankapokladnaText->GetValue()))
    {
        wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Bankovní účet %s neexistuje"), m_bankapokladnaText->GetValue()), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        OnSelectBankaPokladna(event);
        return;
    }
    if(m_predkontaceText->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Předkontace nemůže být prázdná"), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        OnSelectPredkontace(event);
        return;
    }
    if(!idbmodel->hasPredkontace(m_predkontaceText->GetValue()))
    {
        wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Předkontace %s neexistuje"), m_predkontaceText->GetValue()), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        OnSelectPredkontace(event);
        return;
    }
    int pausal = 0;
    if(m_prijem && idbmodel->isFirmaPausal())
    {
        pausal = wxAtoi(m_pausal->GetStringSelection());
    }
    int protistranaid = 0;
    if(!m_protistranaText->GetValue().IsEmpty())
    {
        if(idbmodel->hasKontakt(m_protistranaText->GetValue()))
        {
            protistranaid = idbmodel->getKontaktIdByNazev(m_protistranaText->GetValue());
        }
        else
        {
            protistranaid = idbmodel->getLastId("kontakt");
            int adresaid = idbmodel->getLastId("adresa");
            idbmodel->addAdresa(Adresa(adresaid, wxEmptyString, wxEmptyString, wxEmptyString, wxEmptyString, wxEmptyString));
            idbmodel->addKontakt(Kontakt(protistranaid, m_protistranaText->GetValue(), m_protistranaText->GetValue(), wxEmptyString, wxEmptyString, wxEmptyString, true, adresaid));
        }
    }
    m_castka->GetValidator()->TransferFromWindow();
    m_castkacizi->GetValidator()->TransferFromWindow();
    m_kurz->GetValidator()->TransferFromWindow();
    if(m_edit)
    {
        idbmodel->editDenik(Denik(m_id, m_datum->GetValue(), m_cislodokladu->GetValue(), m_popis->GetValue(),
                              m_castkaValue, m_castkaciziValue, m_kurzValue, m_reference->GetValue(),
                              getMenaId(), idbmodel->getPredkontaceIdByZkratka(m_predkontaceText->GetValue()),
                              m_pokladna?idbmodel->getPokladnaIdByZkratka(m_bankapokladnaText->GetValue()):-1,
                              m_banka?idbmodel->getBankaIdByZkratka(m_bankapokladnaText->GetValue()):-1,
                              m_banka?m_vypis->GetValue():-1, protistranaid, idbmodel->currentRok(), pausal));
    }
    else
    {
        idbmodel->addDenik(Denik(idbmodel->getLastId("denik"), m_datum->GetValue(), m_cislodokladu->GetValue(), m_popis->GetValue(),
                                 m_castkaValue, m_castkaciziValue, m_kurzValue, m_reference->GetValue(),
                                 getMenaId(), idbmodel->getPredkontaceIdByZkratka(m_predkontaceText->GetValue()),
                                 m_pokladna?idbmodel->getPokladnaIdByZkratka(m_bankapokladnaText->GetValue()):-1,
                                 m_banka?idbmodel->getBankaIdByZkratka(m_bankapokladnaText->GetValue()):-1,
                                 m_banka?m_vypis->GetValue():-1, protistranaid, idbmodel->currentRok(), pausal));
    }
    if(m_edit == 0 || m_edit == 2)
    {
        if(m_prijem)
        {
            if(m_pokladna) idbmodel->increaseRadaCislo("PP");
            else idbmodel->increaseRadaCislo("BP");
        }
        else
        {
            if(m_pokladna) idbmodel->increaseRadaCislo("PV");
            else idbmodel->increaseRadaCislo("BV");
        }
    }
    EndModal(wxID_OK);
}

void PrijemVydejDialog::OnCancel(wxCommandEvent& /*event*/)
{
    if(m_edit == 2) idbmodel->deleteDenik(m_id);
    EndModal(wxID_CANCEL);
}

void PrijemVydejDialog::OnSelectBankaPokladna(wxCommandEvent& /*event*/)
{
    if(m_pokladna)
    {
        PokladnyDialog dialog(this, true);
        if(dialog.ShowModal() == wxID_OK)
        {
            m_bankapokladnaText->SetValue(idbmodel->getPokladnaZkratka(dialog.selectedID()));
        }
    }
    else
    {
        BankyDialog dialog(this, true);
        if(dialog.ShowModal() == wxID_OK)
        {
            m_bankapokladnaText->SetValue(idbmodel->getBankaZkratka(dialog.selectedID()));
            m_vypis->SetValue(idbmodel->getLastVypis(dialog.selectedID()));
        }
    }
    fillMena();
}

void PrijemVydejDialog::OnChoiceBankaPokladna(wxCommandEvent &event)
{
    if(event.GetInt() == 0) //pokladna
    {
        m_pokladna = true;
        m_banka = false;
        m_bankapokladnaLabel->SetLabel("Pokladna");
        m_bankapokladnaText->SetValue(idbmodel->getPokladnaZkratka(0));
        m_bankapokladnaSelect->SetToolTip("Vyberte pokladnu");
        if(m_prijem) m_cislodokladu->SetValue(idbmodel->getCisloDokladu("PP"));
        else m_cislodokladu->SetValue(idbmodel->getCisloDokladu("PV"));
        m_vypisLabel->Hide();
        m_vypis->Hide();
    }
    else
    {
        m_pokladna = false;
        m_banka = true;
        m_bankapokladnaLabel->SetLabel(wxString::FromUTF8("Bankovní účet"));
        m_bankapokladnaText->SetValue(idbmodel->getBankaZkratka(0));
        m_bankapokladnaSelect->SetToolTip(wxString::FromUTF8("Vyberte bankovní účet"));
        if(m_prijem) m_cislodokladu->SetValue(idbmodel->getCisloDokladu("BP"));
        else m_cislodokladu->SetValue(idbmodel->getCisloDokladu("BV"));
        m_vypisLabel->Show();
        m_vypis->Show();
        m_vypis->SetValue(idbmodel->getLastVypis(idbmodel->getBankaIdByZkratka(m_bankapokladnaText->GetValue())));
    }
    fillMena();
    m_mainSizer->Layout();
    m_mainSizer->Fit(this);
}

void PrijemVydejDialog::OnChoiceMena(wxCommandEvent &event)
{
    m_castkacizi->GetValidator()->TransferFromWindow();
    int menaid = getMenaId(event.GetInt());
    if(menaid == 1) //CZK;
    {
        m_castkaciziLabel->SetLabel(wxString::FromUTF8("Částka v cizí měně"));
        m_kurzValue = 1;
        m_castkaciziValue = m_castkaValue;
    }
    else
    {
        if(idbmodel->hasKurz(static_cast<Mena*>(m_mena->GetClientData(event.GetInt()))->kod(), m_datum->GetValue())) m_kurzDownload->Disable();
        else m_kurzDownload->Enable();
        m_castkaciziLabel->SetLabel(wxString::Format(wxString::FromUTF8("Částka v %s"), idbmodel->zkratkaMena(menaid)));
        m_kurzValue = idbmodel->getKurz(static_cast<Mena*>(m_mena->GetClientData(event.GetInt()))->kod(), m_datum->GetValue()).hodnota();
        m_castkaciziValue = m_edit ? (idbmodel->hasDenik(m_id)?idbmodel->getDenik(m_id).castkacizi():0.0) : 0.0;
        m_castkaValue = m_kurzValue*m_castkaciziValue;
    }
    m_castka->GetValidator()->TransferToWindow();
    m_castkacizi->GetValidator()->TransferToWindow();
    m_kurz->GetValidator()->TransferToWindow();
    showhideKurz();
    m_mainSizer->Layout();
    m_mainSizer->Fit(this);
}

void PrijemVydejDialog::OnSelectPredkontace(wxCommandEvent& /*event*/)
{
    PredkontacesDialog dialog(this, true, m_prijem?PredkontacesDialog::PK_PRIJEM:PredkontacesDialog::PK_VYDAJ);
    if(dialog.ShowModal() == wxID_OK)
    {
        Predkontace pk = idbmodel->getPredkontace(dialog.selectedID());
        if(pk.isValid())
        {
            m_predkontaceText->SetValue(pk.zkratka());
            if(m_popis->GetValue().IsEmpty()) m_popis->SetValue(pk.popis());
        }
    }
}

void PrijemVydejDialog::OnSelectProtistrana(wxCommandEvent& /*event*/)
{
    KontaktyDialog dialog(this, true);
    if(dialog.ShowModal() == wxID_OK)
    {
        if(idbmodel->hasKontakt(dialog.selectedID())) m_protistranaText->SetValue(idbmodel->getKontakt(dialog.selectedID()).nazev());
    }
}

void PrijemVydejDialog::OnKurzDownload(wxCommandEvent& /*event*/)
{
#if wxUSE_URL
    if(idbmodel->hasKurz(static_cast<Mena*>(m_mena->GetClientData(m_mena->GetSelection()))->kod(), m_datum->GetValue()))
        return;
    dxutils::kurzyDownload(m_datum->GetValue());
    m_kurzDownload->Disable();
    m_kurzValue = idbmodel->getKurz(static_cast<Mena*>(m_mena->GetClientData(static_cast<unsigned int>(m_mena->GetSelection())))->kod(), m_datum->GetValue()).hodnota();
    m_kurz->GetValidator()->TransferToWindow();
    calculateValues(ID_PRIJEMVYDEJ_KURZ_TEXT);
#endif
}

void PrijemVydejDialog::OnDatumChanged(wxDateEvent& /*event*/)
{
    if(getMenaId() == 1)
    {
        m_kurzDownload->Disable();
        return;
    }
    if(idbmodel->hasKurz(static_cast<Mena*>(m_mena->GetClientData(static_cast<unsigned int>(m_mena->GetSelection())))->kod(), m_datum->GetValue()))
        m_kurzDownload->Disable();
    else
        m_kurzDownload->Enable();
    m_kurzValue = idbmodel->getKurz(static_cast<Mena*>(m_mena->GetClientData(static_cast<unsigned int>(m_mena->GetSelection())))->kod(), m_datum->GetValue()).hodnota();
    m_kurz->GetValidator()->TransferToWindow();
    calculateValues(ID_PRIJEMVYDEJ_KURZ_TEXT);
}

void PrijemVydejDialog::OnPausalAdd(wxCommandEvent &/*event*/)
{
    long pausal = wxGetNumberFromUser(wxString::FromUTF8("Nová hodnota paušálních výdajů"), "%", wxString::FromUTF8("Paušální výdaj"), 1, 1);
    if(pausal == -1)
    {
        return;
    }
    wxString value(wxString::Format("%lu", pausal));
    if(m_pausal->FindString(value) == wxNOT_FOUND)
    {
        wxArrayString values = m_pausal->GetStrings();
        m_pausal->Clear();
        values.Add(value);
        values.Sort(true);
        m_pausal->Append(values);
        for(size_t i=0; i<values.GetCount(); i++)
        {
            if(values[i] == value)
            {
                m_pausal->SetSelection(static_cast<int>(i));
                break;
            }
        }
    }
}

void PrijemVydejDialog::showhideKurz()
{
    if(getMenaId() == 1)
    {
        m_kurzLabel->Hide();
        m_kurz->Hide();
        m_castkaciziLabel->Hide();
        m_castkacizi->Hide();
#if wxUSE_URL
        m_kurzDownload->Hide();
#endif
    }
    else
    {
        m_kurzLabel->Show();
        m_kurz->Show();
        m_castkaciziLabel->Show();
        m_castkacizi->Show();
#if wxUSE_URL
        m_kurzDownload->Show();
#endif
    }
    m_mainSizer->Layout();
    m_mainSizer->Fit(this);
}

void PrijemVydejDialog::calculateValues(int id)
{
    if(id == ID_PRIJEMVYDEJ_KURZ_TEXT)
    {
        m_kurz->GetValidator()->TransferFromWindow();
        m_castkacizi->GetValidator()->TransferFromWindow();
        m_castkaValue = m_kurzValue*m_castkaciziValue;
        m_castka->GetValidator()->TransferToWindow();
    }
    else if(id == ID_PRIJEMVYDEJ_CASTKA_TEXT)
    {
        m_kurz->GetValidator()->TransferFromWindow();
        m_castka->GetValidator()->TransferFromWindow();
        if(m_kurzValue != 0.0) m_castkaciziValue = m_castkaValue/m_kurzValue;
        else m_castkaciziValue = 0;
        m_castkacizi->GetValidator()->TransferToWindow();
    }
    else
    {
        m_kurz->GetValidator()->TransferFromWindow();
        m_castkacizi->GetValidator()->TransferFromWindow();
        m_castkaValue = m_kurzValue*m_castkaciziValue;
        m_castka->GetValidator()->TransferToWindow();
    }
}

BEGIN_EVENT_TABLE(ValueCtrl, wxTextCtrl)
    EVT_KEY_UP(ValueCtrl::OnKeyUp)
END_EVENT_TABLE()

void ValueCtrl::OnKeyUp(wxKeyEvent &event)
{
    m_parent->calculateValues(event.GetId());
    event.Skip();
}
