/***************************************************************
 * Name:      prijemvydejdialog.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#ifndef PRIJEMVYDEJDIALOG_H
#define PRIJEMVYDEJDIALOG_H

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif
#include <wx/spinctrl.h>
#include <wx/datectrl.h>
#include <wx/dateevt.h>
#include "model/dbmodel.h"

class PrijemVydejDialog;

class ValueCtrl : public wxTextCtrl
{
    DECLARE_EVENT_TABLE()
public:
    ValueCtrl(wxWindow *parent, wxWindowID id, const wxString &value, const wxPoint &pos, const wxSize &size,
              int style, const wxValidator& validator = wxDefaultValidator)
        : wxTextCtrl(parent, id, value, pos, size, style, validator), m_parent((PrijemVydejDialog*)parent)
    {}
private:
    PrijemVydejDialog *m_parent;

    void OnKeyUp(wxKeyEvent& event);
};

class PrijemVydejDialog: public wxDialog
{
    DECLARE_DYNAMIC_CLASS(PrijemVydejDialog)
    DECLARE_EVENT_TABLE()
public:
    PrijemVydejDialog() : m_id(-1), m_edit(false), m_prijem(false), m_pokladna(false), m_banka(false),
    m_uhrada(false), m_mainSizer(nullptr), m_datum(nullptr), m_bankapokladnaChoice(nullptr), m_bankapokladnaLabel(nullptr),
    m_bankapokladnaText(nullptr), m_bankapokladnaSelect(nullptr), m_predkontaceText(nullptr), m_predkontaceSelect(nullptr),
    m_protistranaText(nullptr), m_protistranaSelect(nullptr), m_popis(nullptr), m_cislodokladu(nullptr), m_mena(nullptr),
    m_castka(nullptr), m_castkaValue(0.0), m_castkacizi(nullptr), m_castkaciziValue(0.0), m_castkaciziLabel(nullptr),
    m_kurz(nullptr), m_kurzDownload(nullptr), m_kurzValue(0.0), m_kurzLabel(nullptr), m_reference(nullptr),
    m_vypisLabel(nullptr), m_vypis(nullptr), m_pausal(nullptr), m_pausalAdd(nullptr) {}
    PrijemVydejDialog(wxWindow *parent, int id, int edit, bool prijem, bool pokladna, bool banka);

    void calculateValues(int id);
private:
    int m_id;
    /* 0 - novy, 1 - editace, 2 - kopie */
    int m_edit;
    bool m_prijem;
    bool m_pokladna;
    bool m_banka;
    bool m_uhrada;
    wxBoxSizer *m_mainSizer;
    wxDatePickerCtrl *m_datum;
    wxChoice *m_bankapokladnaChoice;
    wxStaticText *m_bankapokladnaLabel;
    wxTextCtrl *m_bankapokladnaText;
    wxButton *m_bankapokladnaSelect;
    wxTextCtrl *m_predkontaceText;
    wxButton *m_predkontaceSelect;
    wxTextCtrl *m_protistranaText;
    wxButton *m_protistranaSelect;
    wxTextCtrl *m_popis;
    wxTextCtrl *m_cislodokladu;
    wxChoice *m_mena;
    ValueCtrl *m_castka;
    double m_castkaValue;
    ValueCtrl *m_castkacizi;
    double m_castkaciziValue;
    wxStaticText *m_castkaciziLabel;
    ValueCtrl *m_kurz;
    wxBitmapButton *m_kurzDownload;
    double m_kurzValue;
    wxStaticText *m_kurzLabel;
    wxTextCtrl *m_reference;
    wxStaticText *m_vypisLabel;
    wxSpinCtrl *m_vypis;
    wxChoice *m_pausal;
    wxButton *m_pausalAdd;

    void createControls();
    void fillControls();
    void fillMena();
    int getMenaId() const;
    int getMenaId(unsigned int index) const;
    void OnOk(wxCommandEvent& event);
    void OnCancel(wxCommandEvent& event);
    void OnSelectBankaPokladna(wxCommandEvent& event);
    void OnChoiceBankaPokladna(wxCommandEvent& event);
    void OnChoiceMena(wxCommandEvent& event);
    void OnSelectPredkontace(wxCommandEvent& event);
    void OnSelectProtistrana(wxCommandEvent& event);
    void OnKurzDownload(wxCommandEvent& event);
    void OnDatumChanged(wxDateEvent& event);
    void OnPausalAdd(wxCommandEvent& event);
    void showhideKurz();
};

#endif  /* PRIJEMVYDEJDIALOG_H */

