/***************************************************************
 * Name:      radadialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include "radadialog.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"

IMPLEMENT_DYNAMIC_CLASS(RadaDialog, wxDialog)

BEGIN_EVENT_TABLE(RadaDialog, wxDialog)
    EVT_BUTTON(wxID_OK, RadaDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, RadaDialog::OnCancel)
    EVT_TEXT(ID_RADA_PREFIX, RadaDialog::OnPrefix)
    EVT_SPINCTRL(ID_RADA_CISLO, RadaDialog::OnCislo)
END_EVENT_TABLE()

RadaDialog::RadaDialog(wxWindow *parent, const wxString &id)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Číselná řada")), m_id(id)
{
    createControls();
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void RadaDialog::createControls()
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Popis"), wxGBPosition(0,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_popis = new wxStaticText(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_popis, wxGBPosition(0,1), wxDefaultSpan, wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Prefix"), wxGBPosition(1,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_prefix = new wxTextCtrl(this, ID_RADA_PREFIX);
    m_prefix->SetMaxLength(6);
    gridSizer->Add(m_prefix, wxGBPosition(1,1), wxDefaultSpan, wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Počet znaků")), wxGBPosition(2,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_znaku = new wxSpinCtrl(this, wxID_ANY);
    gridSizer->Add(m_znaku, wxGBPosition(2,1), wxDefaultSpan, wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Aktuální číslo")), wxGBPosition(3,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_cislo = new wxSpinCtrl(this, ID_RADA_CISLO);
    m_cislo->SetRange(1,9999999);
    gridSizer->Add(m_cislo, wxGBPosition(3,1), wxDefaultSpan, wxEXPAND);

    m_mainSizer->Add(gridSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void RadaDialog::fillControls()
{
    Rada ra = idbmodel->getRada(m_id);
    if(ra.isValid())
    {
        m_popis->SetLabel(ra.popis());
        m_prefix->ChangeValue(ra.prefix());
        m_znaku->SetValue(ra.znaku());
        m_znaku->SetRange(ra.prefix().Len()+wxMax(4,dxutils::pocetCislic(ra.cislo())),100);
        m_cislo->SetValue(ra.cislo());
    }
}

void RadaDialog::OnOk(wxCommandEvent& /*event*/)
{
    idbmodel->editRada(Rada(m_id, m_popis->GetLabel(), m_prefix->GetValue(), m_znaku->GetValue(), m_cislo->GetValue()));
    EndModal(wxID_OK);
}

void RadaDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

void RadaDialog::OnPrefix(wxCommandEvent& /*event*/)
{
    m_znaku->SetRange(m_prefix->GetValue().Len()+wxMax(4,dxutils::pocetCislic(m_cislo->GetValue())),100);
}

void RadaDialog::OnCislo(wxSpinEvent& /*event*/)
{
    m_znaku->SetRange(m_prefix->GetValue().Len()+wxMax(4,dxutils::pocetCislic(m_cislo->GetValue())),100);
}
