/***************************************************************
 * Name:      uhradadialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include "uhradadialog.h"
#include "resources/images/download.xpm"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "bankydialog.h"
#include "pokladnydialog.h"

IMPLEMENT_DYNAMIC_CLASS(UhradaDialog, wxDialog)

BEGIN_EVENT_TABLE(UhradaDialog, wxDialog)
    EVT_BUTTON(wxID_OK, UhradaDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, UhradaDialog::OnCancel)
    EVT_BUTTON(ID_UHRADA_BANKAPOKLADNA_SELECT, UhradaDialog::OnSelectBankaPokladna)
    EVT_BUTTON(ID_UHRADA_KURZ_DOWNLOAD, UhradaDialog::OnKurzDownload)
    EVT_DATE_CHANGED(ID_UHRADA_DATUMCTRL, UhradaDialog::OnDatumChanged)
    EVT_SPINCTRLDOUBLE(ID_UHRADA_CASTKA_SPIN, UhradaDialog::OnCastka)
    EVT_SPINCTRLDOUBLE(ID_UHRADA_CASTKACIZI_SPIN, UhradaDialog::OnCastkacizi)
    EVT_SPINCTRLDOUBLE(ID_UHRADA_KURZ_SPIN, UhradaDialog::OnKurz)
END_EVENT_TABLE()

UhradaDialog::UhradaDialog(wxWindow *parent,  const Faktura &faktura)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Úhrada faktury")), m_faktura(faktura)
{
    m_uhradaFaktury = true;
    m_pokladna = idbmodel->faUhradaPokladnou(faktura.uhradaid());
    m_menaid = faktura.menaid();
    m_prijem = faktura.vydana();
    createControls();
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

UhradaDialog::UhradaDialog(wxWindow *parent, const ZavPoh &zavpoh)
    : wxDialog(parent, wxID_ANY, zavpoh.zavazek()?wxString::FromUTF8("Úhrada závazku"):wxString::FromUTF8("Úhrada pohledávky")), m_zavpoh(zavpoh)
{
    m_uhradaFaktury = false;
    m_pokladna = idbmodel->faUhradaPokladnou(zavpoh.uhradaid());
    m_menaid = zavpoh.menaid();
    m_prijem = !zavpoh.zavazek();
    createControls();
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void UhradaDialog::createControls()
{
    int i=0; //row

    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Datum"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_datum = new wxDatePickerCtrl(this, ID_UHRADA_DATUMCTRL, dxsettings.datum(), wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    m_datum->SetRange(wxDateTime(1,wxDateTime::Jan,idbmodel->currentRok()),dxsettings.datum());
    gridSizer->Add(m_datum, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Zaúčtovat")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_bankapokladnaChoice = new wxChoice(this, ID_UHRADA_BANKAPOKLADNA_CHOICE);
    gridSizer->Add(m_bankapokladnaChoice, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);
    i++;

    m_bankapokladnaLabel = new wxStaticText(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_bankapokladnaLabel, wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_bankapokladnaText = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_bankapokladnaText, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_bankapokladnaSelect = new wxButton(this, ID_UHRADA_BANKAPOKLADNA_SELECT, "...");
    gridSizer->Add(m_bankapokladnaSelect, wxGBPosition(i,2));
    i++;

    m_vypisLabel = new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Číslo výpisu"));
    gridSizer->Add(m_vypisLabel, wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_vypis = new wxSpinCtrl(this);
    m_vypis->SetRange(1,65000);
    gridSizer->Add(m_vypis, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);
    i++;
    if(m_pokladna)
    {
        m_vypisLabel->Hide();
        m_vypis->Hide();
    }

    if(m_prijem && idbmodel->isFirmaPausal())
    {
        gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Paušální výdaj %")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
        m_pausal = new wxSpinCtrl(this);
        m_pausal->SetRange(0,100);
        gridSizer->Add(m_pausal, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);
        i++;
    }

    m_kurzLabel = new wxStaticText(this, wxID_ANY, "Kurz");
    gridSizer->Add(m_kurzLabel, wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_kurz = new wxSpinCtrlDouble(this, ID_UHRADA_KURZ_SPIN);
    m_kurz->SetDigits(6);
    m_kurzDownload = new wxBitmapButton(this, ID_UHRADA_KURZ_DOWNLOAD, wxBitmap(download_xpm));
    m_kurzDownload->SetBitmapDisabled(dxutils::GreyScaledIcon(download_xpm));
    m_kurzDownload->SetToolTip(wxString::FromUTF8("Stáhnout kurz z webu ČNB"));
#if wxUSE_URL
    gridSizer->Add(m_kurz, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(m_kurzDownload, wxGBPosition(i,2));
#else
    gridSizer->Add(m_kurz, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);
#endif
    i++;

    m_castkaciziLabel = new wxStaticText(this, wxID_ANY, wxString::Format(wxString::FromUTF8("Částka v %s"), idbmodel->zkratkaMena(m_menaid)));
    gridSizer->Add(m_castkaciziLabel, wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_castkacizi = new wxSpinCtrlDouble(this, ID_UHRADA_CASTKACIZI_SPIN);
    m_castkacizi->SetDigits(2);
    gridSizer->Add(m_castkacizi, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Částka v Kč")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_castka = new wxSpinCtrlDouble(this, ID_UHRADA_CASTKA_SPIN);
    m_castka->SetDigits(2);
    gridSizer->Add(m_castka, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);

    gridSizer->SetEmptyCellSize(wxSize(0,0));
    m_mainSizer->Add(gridSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void UhradaDialog::fillControls()
{
    m_datum->SetValue(dxsettings.datum());
    m_bankapokladnaChoice->Append("Pokladna");
    m_bankapokladnaChoice->Append(wxString::FromUTF8("Bankovní účet"));
    if(m_pokladna)
    {
        m_bankapokladnaChoice->SetSelection(0);
        m_bankapokladnaLabel->SetLabel("Pokladna");
        m_bankapokladnaSelect->SetToolTip("Vyberte pokladnu");
    }
    else
    {
        m_bankapokladnaChoice->SetSelection(1);
        m_bankapokladnaLabel->SetLabel(wxString::FromUTF8("Bankovní účet"));
        m_bankapokladnaSelect->SetToolTip(wxString::FromUTF8("Vyberte bankovní účet"));
    }
    bankapokladnaTextSetValue();
    if(m_menaid == 1)
    {
        double max = m_uhradaFaktury ? m_faktura.celkem()-idbmodel->uhrazenoByFakturaId(m_faktura.id()) : m_zavpoh.celkem()-idbmodel->uhrazenoByZavpohId(m_zavpoh.id());
        m_castka->SetRange(0.01, max);
        m_castka->SetValue(max);
    }
    else
    {
        double max = m_uhradaFaktury ? m_faktura.celkem()-idbmodel->uhrazenoByFakturaId(m_faktura.id()) : m_zavpoh.celkem()-idbmodel->uhrazenoByZavpohId(m_zavpoh.id());
        m_castkacizi->SetRange(0.01, max);
        m_castkacizi->SetValue(max);
        m_kurz->SetValue(idbmodel->getKurz(idbmodel->kodMena(m_menaid)).hodnota());
        m_castka->SetRange(m_kurz->GetValue()*0.01, m_kurz->GetValue()*max);
        m_castka->SetValue(m_kurz->GetValue()*max);
        if(idbmodel->hasKurz(idbmodel->kodMena(m_menaid), m_datum->GetValue()))
            m_kurzDownload->Disable();
        else
            m_kurzDownload->Enable();
    }
    if(m_prijem && idbmodel->isFirmaPausal())
    {
        m_pausal->SetValue(m_uhradaFaktury ? m_faktura.pausal() : m_zavpoh.pausal());
    }
    showhideKurz();
    Connect(ID_UHRADA_BANKAPOKLADNA_CHOICE, wxEVT_CHOICE, wxCommandEventHandler(UhradaDialog::OnChoiceBankaPokladna), nullptr, this);
}

void UhradaDialog::OnOk(wxCommandEvent& event)
{
    if(m_bankapokladnaText->GetValue().IsEmpty())
    {
        wxString msg;
        if(m_pokladna) msg = wxString::FromUTF8("Pokladna nemůže být prázdná");
        else msg = wxString::FromUTF8("Bankovní účet nemůže být prázdný");
        wxMessageDialog dialog(this, msg, "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        OnSelectBankaPokladna(event);
        return;
    }
    if(m_pokladna && !idbmodel->hasPokladna(m_bankapokladnaText->GetValue()))
    {
        wxMessageDialog dialog(this, wxString::Format("Pokladna %s neexistuje", m_bankapokladnaText->GetValue()), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        OnSelectBankaPokladna(event);
        return;
    }
    if(!m_pokladna && !idbmodel->hasBanka(m_bankapokladnaText->GetValue()))
    {
        wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Bankovní účet %s neexistuje"), m_bankapokladnaText->GetValue()), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        OnSelectBankaPokladna(event);
        return;
    }
    bool platce = m_uhradaFaktury ? m_faktura.platce() : m_zavpoh.platce();
    if(platce)
        writeUhradaPlatce();
    else
        writeUhradaNeplatce();
    EndModal(wxID_OK);
}

void UhradaDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

void UhradaDialog::OnSelectBankaPokladna(wxCommandEvent& /*event*/)
{
    if(m_pokladna)
    {
        PokladnyDialog dialog(this, true);
        if(dialog.ShowModal() == wxID_OK)
        {
            m_bankapokladnaText->SetValue(idbmodel->getPokladnaZkratka(dialog.selectedID()));
        }
    }
    else
    {
        BankyDialog dialog(this, true);
        if(dialog.ShowModal() == wxID_OK)
        {
            m_bankapokladnaText->SetValue(idbmodel->getBankaZkratka(dialog.selectedID()));
        }
    }
}

void UhradaDialog::OnChoiceBankaPokladna(wxCommandEvent &event)
{
    if(event.GetInt() == 0) //pokladna
    {
        m_pokladna = true;
        m_bankapokladnaLabel->SetLabel("Pokladna");
        m_bankapokladnaText->SetValue(idbmodel->getPokladnaZkratka(0));
        m_bankapokladnaSelect->SetToolTip("Vyberte pokladnu");
        m_vypisLabel->Hide();
        m_vypis->Hide();
    }
    else
    {
        m_pokladna = false;
        m_bankapokladnaLabel->SetLabel(wxString::FromUTF8("Bankovní účet"));
        m_bankapokladnaText->SetValue(idbmodel->getBankaZkratka(0));
        m_bankapokladnaSelect->SetToolTip(wxString::FromUTF8("Vyberte bankovní účet"));
        m_vypisLabel->Show();
        m_vypis->Show();
        m_vypis->SetValue(idbmodel->getLastVypis(idbmodel->getBankaIdByZkratka(m_bankapokladnaText->GetValue())));
    }
    m_mainSizer->Layout();
    m_mainSizer->Fit(this);
}

void UhradaDialog::OnKurzDownload(wxCommandEvent& /*event*/)
{
#if wxUSE_URL
    if(idbmodel->hasKurz(idbmodel->kodMena(m_menaid), m_datum->GetValue()))
        return;
    dxutils::kurzyDownload(m_datum->GetValue());
    m_kurzDownload->Disable();
    m_kurz->SetValue(idbmodel->getKurz(idbmodel->kodMena(m_menaid), m_datum->GetValue()).hodnota());
    m_castka->SetRange(0.0, m_kurz->GetValue()*m_castkacizi->GetMax());
    calculateValues(ID_UHRADA_KURZ_SPIN);
#endif
}

void UhradaDialog::OnDatumChanged(wxDateEvent& /*event*/)
{
    if(m_menaid == 1)
    {
        m_kurzDownload->Disable();
        return;
    }
    if(idbmodel->hasKurz(idbmodel->kodMena(m_menaid), m_datum->GetValue()))
        m_kurzDownload->Disable();
    else
        m_kurzDownload->Enable();
    m_kurz->SetValue(idbmodel->getKurz(idbmodel->kodMena(m_menaid), m_datum->GetValue()).hodnota());
}

void UhradaDialog::OnCastka(wxSpinDoubleEvent &/*event*/)
{
    calculateValues(ID_UHRADA_CASTKA_SPIN);
}

void UhradaDialog::OnCastkacizi(wxSpinDoubleEvent &/*event*/)
{
    calculateValues(ID_UHRADA_CASTKACIZI_SPIN);
}

void UhradaDialog::OnKurz(wxSpinDoubleEvent &/*event*/)
{
    m_castka->SetRange(0.0, m_kurz->GetValue()*m_castkacizi->GetMax());
    calculateValues(ID_UHRADA_KURZ_SPIN);
}

void UhradaDialog::showhideKurz()
{
    if(m_menaid == 1)
    {
        m_kurzLabel->Hide();
        m_kurz->Hide();
        m_castkaciziLabel->Hide();
        m_castkacizi->Hide();
#if wxUSE_URL
        m_kurzDownload->Hide();
#endif
    }
    else
    {
        m_kurzLabel->Show();
        m_kurz->Show();
        m_castkaciziLabel->Show();
        m_castkacizi->Show();
#if wxUSE_URL
        m_kurzDownload->Show();
#endif
    }
    m_mainSizer->Layout();
    m_mainSizer->Fit(this);
}

void UhradaDialog::writeUhradaNeplatce()
{
    wxString cislodokladu = "";
    if(m_prijem)
    {
        if(m_pokladna) cislodokladu = idbmodel->getCisloDokladu("PP");
        else cislodokladu = idbmodel->getCisloDokladu("BP");
    }
    else
    {
        if(m_pokladna) cislodokladu = idbmodel->getCisloDokladu("PV");
        else cislodokladu = idbmodel->getCisloDokladu("BV");
    }
    int pausal = 0;
    if(m_prijem && idbmodel->isFirmaPausal())
    {
        pausal = m_pausal->GetValue();
    }
    wxString popis = "";
    int iddenik = idbmodel->getLastId("denik");
    if(m_uhradaFaktury)
    {
        if(m_prijem) popis = wxString::Format(wxString::FromUTF8("Úhrada faktury vydané %s"), m_faktura.cislo());
        else popis = wxString::Format(wxString::FromUTF8("Úhrada faktury přijaté %s"), m_faktura.cislo());
        idbmodel->addDenik(Denik(iddenik, m_datum->GetValue(), cislodokladu, popis,
                                 m_castka->GetValue(), m_menaid==1?m_castka->GetValue():m_castkacizi->GetValue(), m_menaid==1?1.0:m_kurz->GetValue(), m_faktura.cislo(),
                                 m_menaid, m_faktura.predkontaceid(),
                                 m_pokladna?idbmodel->getPokladnaIdByZkratka(m_bankapokladnaText->GetValue()):-1,
                                 !m_pokladna?idbmodel->getBankaIdByZkratka(m_bankapokladnaText->GetValue()):-1,
                                 !m_pokladna?m_vypis->GetValue():-1, m_faktura.protistranaid(),
                                 idbmodel->currentRok(), pausal));
    }
    else
    {
        if(m_prijem) popis = wxString::Format(wxString::FromUTF8("Úhrada pohledávky %s"), m_zavpoh.cislo());
        else popis = wxString::Format(wxString::FromUTF8("Úhrada závazku %s"), m_zavpoh.cislo());
        idbmodel->addDenik(Denik(iddenik, m_datum->GetValue(), cislodokladu, popis,
                                 m_castka->GetValue(), m_menaid==1?m_castka->GetValue():m_castkacizi->GetValue(), m_menaid==1?1.0:m_kurz->GetValue(), m_zavpoh.cislo(),
                                 m_menaid, m_zavpoh.predkontaceid(),
                                 m_pokladna?idbmodel->getPokladnaIdByZkratka(m_bankapokladnaText->GetValue()):-1,
                                 !m_pokladna?idbmodel->getBankaIdByZkratka(m_bankapokladnaText->GetValue()):-1,
                                 !m_pokladna?m_vypis->GetValue():-1, m_zavpoh.protistranaid(),
                                 idbmodel->currentRok(), pausal));
    }
    idbmodel->addUhrada(Uhrada(idbmodel->getLastId("uhrada"), m_menaid==1?m_castka->GetValue():m_castkacizi->GetValue(), iddenik,
                                   m_uhradaFaktury?m_faktura.id():-1,m_uhradaFaktury?-1:m_zavpoh.id(),idbmodel->currentRok()));
    if(m_prijem)
    {
        if(m_pokladna) idbmodel->increaseRadaCislo("PP");
        else idbmodel->increaseRadaCislo("BP");
    }
    else
    {
        if(m_pokladna) idbmodel->increaseRadaCislo("PV");
        else idbmodel->increaseRadaCislo("BV");
    }
}

void UhradaDialog::writeUhradaPlatce()
{
    if(dxutils::isSameDouble(m_uhradaFaktury?idbmodel->dphFaktury(m_faktura.id()):idbmodel->dphZavPoh(m_zavpoh.id()), 0.0))
    {
        writeUhradaNeplatce();
        return;
    }

    double celkem = m_uhradaFaktury?m_faktura.celkem():m_zavpoh.celkem();
    double castka = m_menaid==1?m_castka->GetValue():m_castkacizi->GetValue();
    double dph = m_uhradaFaktury?idbmodel->dphFaktury(m_faktura.id()):idbmodel->dphZavPoh(m_zavpoh.id());
    double pomer = castka/celkem;
    double zaklad = pomer*(celkem-dph);
    dph = pomer*dph;

    //write zaklad - start
    wxString cislodokladu = "";
    if(m_prijem)
    {
        if(m_pokladna) cislodokladu = idbmodel->getCisloDokladu("PP");
        else cislodokladu = idbmodel->getCisloDokladu("BP");
    }
    else
    {
        if(m_pokladna) cislodokladu = idbmodel->getCisloDokladu("PV");
        else cislodokladu = idbmodel->getCisloDokladu("BV");
    }
    int pausal = 0;
    if(m_prijem && idbmodel->isFirmaPausal())
    {
        pausal = m_pausal->GetValue();
    }
    wxString popis = "";
    int iddenik = idbmodel->getLastId("denik");
    if(m_uhradaFaktury)
    {
        if(m_prijem) popis = wxString::Format(wxString::FromUTF8("Úhrada faktury vydané %s"), m_faktura.cislo());
        else popis = wxString::Format(wxString::FromUTF8("Úhrada faktury přijaté %s"), m_faktura.cislo());
        idbmodel->addDenik(Denik(iddenik, m_datum->GetValue(), cislodokladu, popis,
                                     m_menaid==1?zaklad:zaklad*m_kurz->GetValue(), zaklad, m_menaid==1?1.0:m_kurz->GetValue(), m_faktura.cislo(),
                                     m_menaid, m_faktura.predkontaceid(),
                                     m_pokladna?idbmodel->getPokladnaIdByZkratka(m_bankapokladnaText->GetValue()):-1,
                                     !m_pokladna?idbmodel->getBankaIdByZkratka(m_bankapokladnaText->GetValue()):-1,
                                     !m_pokladna?m_vypis->GetValue():-1, m_faktura.protistranaid(),
                                     idbmodel->currentRok(), pausal));
    }
    else
    {
        if(m_prijem) popis = wxString::Format(wxString::FromUTF8("Úhrada pohledávky %s"), m_zavpoh.cislo());
        else popis = wxString::Format(wxString::FromUTF8("Úhrada závazku %s"), m_zavpoh.cislo());
        idbmodel->addDenik(Denik(iddenik, m_datum->GetValue(), cislodokladu, popis,
                                     m_menaid==1?zaklad:zaklad*m_kurz->GetValue(), zaklad, m_menaid==1?1.0:m_kurz->GetValue(), m_zavpoh.cislo(),
                                     m_menaid, m_zavpoh.predkontaceid(),
                                     m_pokladna?idbmodel->getPokladnaIdByZkratka(m_bankapokladnaText->GetValue()):-1,
                                     !m_pokladna?idbmodel->getBankaIdByZkratka(m_bankapokladnaText->GetValue()):-1,
                                     !m_pokladna?m_vypis->GetValue():-1, m_zavpoh.protistranaid(),
                                     idbmodel->currentRok(), pausal));
    }
    idbmodel->addUhrada(Uhrada(idbmodel->getLastId("uhrada"), zaklad, iddenik,
                                   m_uhradaFaktury?m_faktura.id():-1,m_uhradaFaktury?-1:m_zavpoh.id(),idbmodel->currentRok()));
    if(m_prijem)
    {
        if(m_pokladna) idbmodel->increaseRadaCislo("PP");
        else idbmodel->increaseRadaCislo("BP");
    }
    else
    {
        if(m_pokladna) idbmodel->increaseRadaCislo("PV");
        else idbmodel->increaseRadaCislo("BV");
    }
    //write zaklad - end

    //write dph - start
    if(m_prijem)
    {
        if(m_pokladna) cislodokladu = idbmodel->getCisloDokladu("PP");
        else cislodokladu = idbmodel->getCisloDokladu("BP");
        popis = wxString::FromUTF8("DPH příjem");
    }
    else
    {
        if(m_pokladna) cislodokladu = idbmodel->getCisloDokladu("PV");
        else cislodokladu = idbmodel->getCisloDokladu("BV");
        popis = wxString::FromUTF8("DPH výdaj");
    }
    iddenik = idbmodel->getLastId("denik");
    if(m_uhradaFaktury)
    {
        idbmodel->addDenik(Denik(iddenik, m_datum->GetValue(), cislodokladu, popis,
                                 m_menaid==1?dph:dph*m_kurz->GetValue(), dph, m_menaid==1?1.0:m_kurz->GetValue(), m_faktura.cislo(),
                                 m_menaid, m_prijem?idbmodel->getPredkontaceIdByZkratka("P102"):idbmodel->getPredkontaceIdByZkratka("V102"),
                                 m_pokladna?idbmodel->getPokladnaIdByZkratka(m_bankapokladnaText->GetValue()):-1,
                                 !m_pokladna?idbmodel->getBankaIdByZkratka(m_bankapokladnaText->GetValue()):-1,
                                 !m_pokladna?m_vypis->GetValue():-1, m_faktura.protistranaid(),
                                 idbmodel->currentRok()));
    }
    else
    {
        idbmodel->addDenik(Denik(iddenik, m_datum->GetValue(), cislodokladu, popis,
                                 m_menaid==1?dph:dph*m_kurz->GetValue(), dph, m_menaid==1?1.0:m_kurz->GetValue(), m_zavpoh.cislo(),
                                 m_menaid, m_prijem?idbmodel->getPredkontaceIdByZkratka("P102"):idbmodel->getPredkontaceIdByZkratka("V102"),
                                 m_pokladna?idbmodel->getPokladnaIdByZkratka(m_bankapokladnaText->GetValue()):-1,
                                 !m_pokladna?idbmodel->getBankaIdByZkratka(m_bankapokladnaText->GetValue()):-1,
                                 !m_pokladna?m_vypis->GetValue():-1, m_zavpoh.protistranaid(),
                                 idbmodel->currentRok()));
    }
    idbmodel->addUhrada(Uhrada(idbmodel->getLastId("uhrada"), dph, iddenik,
                                   m_uhradaFaktury?m_faktura.id():-1,m_uhradaFaktury?-1:m_zavpoh.id(),idbmodel->currentRok()));
    if(m_prijem)
    {
        if(m_pokladna) idbmodel->increaseRadaCislo("PP");
        else idbmodel->increaseRadaCislo("BP");
    }
    else
    {
        if(m_pokladna) idbmodel->increaseRadaCislo("PV");
        else idbmodel->increaseRadaCislo("BV");
    }
    //write dph - end
}

void UhradaDialog::bankapokladnaTextSetValue()
{
    wxString zkratka = wxEmptyString;
    if(m_pokladna)
    {
        if(m_uhradaFaktury)
        {
            zkratka = idbmodel->getPokladnaZkratka(m_faktura.pokladnaid());
        }
        else
        {
            zkratka = idbmodel->getPokladnaZkratka(m_zavpoh.pokladnaid());
        }
    }
    else
    {
        if(m_uhradaFaktury)
        {
            zkratka = idbmodel->getBankaZkratka(m_faktura.bankaid());
        }
        else
        {
            zkratka = idbmodel->getBankaZkratka(m_zavpoh.bankaid());
        }
    }
    m_bankapokladnaText->SetValue(zkratka);
}

void UhradaDialog::calculateValues(int id)
{
    if(id == ID_UHRADA_KURZ_SPIN)
    {
        m_castka->SetValue(m_kurz->GetValue()*m_castkacizi->GetValue());
    }
    else if(id == ID_UHRADA_CASTKA_SPIN)
    {
        if(m_kurz->GetValue() != 0.0) m_castkacizi->SetValue(m_castka->GetValue()/m_kurz->GetValue());
        else m_castkacizi->SetValue(0);
    }
    else
    {
        m_castka->SetValue(m_kurz->GetValue()*m_castkacizi->GetValue());
    }
}
