/***************************************************************
 * Name:      vozidladialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2018
 * License:   GPL3
 **************************************************************/

#include "vozidladialog.h"
#include "vozidlodialog.h"
#include "dxdefs.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "resources/images/add.xpm"
#include "resources/images/edit.xpm"
#include "resources/images/copy.xpm"
#include "resources/images/delete.xpm"
#include "resources/images/checked.xpm"
#include "resources/images/unchecked.xpm"

IMPLEMENT_DYNAMIC_CLASS(VozidlaDialog, wxDialog)

BEGIN_EVENT_TABLE(VozidlaDialog, wxDialog)
    EVT_BUTTON(wxID_OK, VozidlaDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, VozidlaDialog::OnCancel)
    EVT_LIST_ITEM_RIGHT_CLICK(ID_VOZIDLA_LIST, VozidlaDialog::OnItemRightClick)
    EVT_LIST_ITEM_ACTIVATED(ID_VOZIDLA_LIST, VozidlaDialog::OnItemActivated)
    EVT_LIST_ITEM_SELECTED(ID_VOZIDLA_LIST, VozidlaDialog::OnItemSelected)
    EVT_LIST_ITEM_DESELECTED(ID_VOZIDLA_LIST, VozidlaDialog::OnItemDeselected)
    EVT_MENU(ID_VOZIDLA_ADD, VozidlaDialog::OnAddVozidlo)
    EVT_MENU(ID_VOZIDLA_EDIT, VozidlaDialog::OnEditVozidlo)
    EVT_MENU(ID_VOZIDLA_COPY, VozidlaDialog::OnCopyVozidlo)
    EVT_MENU(ID_VOZIDLA_DELETE, VozidlaDialog::OnDeleteVozidlo)
END_EVENT_TABLE()

VozidlaDialog::VozidlaDialog(wxWindow *parent, bool select)
    : wxDialog(parent, wxID_ANY, "Vozidla"),
      m_select(select), m_selectedId(-1)
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    m_bar = new dxToolBar(this);
    m_bar->AddTool(ID_VOZIDLA_ADD, wxEmptyString, wxBitmap(add_xpm), dxutils::GreyScaledIcon(add_xpm), wxITEM_NORMAL, wxString::FromUTF8("Přidat vozidlo"));
    m_bar->AddTool(ID_VOZIDLA_EDIT, wxEmptyString, wxBitmap(edit_xpm), dxutils::GreyScaledIcon(edit_xpm), wxITEM_NORMAL, "Upravit vozidlo");
    m_bar->AddTool(ID_VOZIDLA_COPY, wxEmptyString, wxBitmap(copy_xpm), dxutils::GreyScaledIcon(copy_xpm), wxITEM_NORMAL, wxString::FromUTF8("Kopírovat vozidlo"));
    m_bar->AddTool(ID_VOZIDLA_DELETE, wxEmptyString, wxBitmap(delete_xpm), dxutils::GreyScaledIcon(delete_xpm), wxITEM_NORMAL, "Smazat vozidlo");
    m_bar->Realize();
    m_mainSizer->Add(m_bar, 0, wxALL|wxEXPAND, 5);
    wxSize imageSize(16, 16);
    m_imageList.reset(new wxImageList(imageSize.GetWidth(), imageSize.GetHeight()));
    m_imageList->Add(wxImage(checked_xpm).Scale(16, 16));
    m_imageList->Add(wxImage(unchecked_xpm).Scale(16, 16));
    m_list = new wxListCtrl(this, ID_VOZIDLA_LIST, wxDefaultPosition, wxDefaultSize,
                            wxLC_REPORT|wxLC_HRULES|wxLC_VRULES|wxLC_SINGLE_SEL);
    m_list->SetImageList(m_imageList.get(), wxIMAGE_LIST_SMALL);
    wxListItem col0;
    col0.SetId(0);
    col0.SetText("RZ");
    col0.SetWidth(80);
    m_list->InsertColumn(0, col0);
    wxListItem col1;
    col1.SetId(1);
    col1.SetText(wxString::FromUTF8("Jméno"));
    col1.SetWidth(250);
    m_list->InsertColumn(1, col1);
    wxListItem col2;
    col2.SetId(2);
    col2.SetText(wxString::FromUTF8("Spotřeba"));
    col2.SetWidth(80);
    col2.SetAlign(wxLIST_FORMAT_RIGHT);
    m_list->InsertColumn(2, col2);
    wxListItem col3;
    col3.SetId(3);
    col3.SetText(wxString::FromUTF8("Soukromé"));
    col3.SetWidth(80);
    m_list->InsertColumn(3, col3);
    fillList();
    m_mainSizer->Add(m_list, 1, wxALL|wxEXPAND, 5);
    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, select? "&Vybrat" : "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);
    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void VozidlaDialog::OnOk(wxCommandEvent& /*event*/)
{
    EndModal(wxID_OK);
}

void VozidlaDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

void VozidlaDialog::OnItemRightClick(wxListEvent &event)
{
    setSelectedId(event.GetIndex());
    wxMenu popup;
    popup.Append(ID_VOZIDLA_ADD, wxString::FromUTF8("Přidat vozidlo"));
    if(m_selectedId != -1)
    {
        popup.AppendSeparator();
        popup.Append(ID_PREDKONTACES_EDIT, "Upravit vozidlo");
        popup.Append(ID_PREDKONTACES_COPY, wxString::FromUTF8("Kopírovat vozidlo"));
        popup.Append(ID_PREDKONTACES_DELETE, "Smazat vozidlo");
    }
    PopupMenu(&popup, event.GetPoint());
}

void VozidlaDialog::OnItemActivated(wxListEvent &event)
{
    setSelectedId(event.GetIndex());
    if(m_select)
    {
        EndModal(wxID_OK);
    }
    else
    {
        VozidloDialog dialog(this, m_selectedId, 1);
        if(dialog.ShowModal() == wxID_OK)
            fillList();
    }
}

void VozidlaDialog::OnItemSelected(wxListEvent &event)
{
    setSelectedId(event.GetIndex());
}

void VozidlaDialog::OnItemDeselected(wxListEvent& /*event*/)
{
    setSelectedId(-1);
}

void VozidlaDialog::OnEditVozidlo(wxCommandEvent& /*event*/)
{
    if(m_selectedId == -1) return;
    VozidloDialog dialog(this, m_selectedId, 1);
    if(dialog.ShowModal() == wxID_OK)
        fillList();
}

void VozidlaDialog::OnCopyVozidlo(wxCommandEvent& /*event*/)
{
    if(m_selectedId == -1) return;
    VozidloDialog dialog(this, idbmodel->copyVozidlo(m_selectedId), 2);
    if(dialog.ShowModal() == wxID_OK)
        fillList();
}

void VozidlaDialog::OnAddVozidlo(wxCommandEvent& /*event*/)
{
    VozidloDialog dialog(this, -1, 0);
    if(dialog.ShowModal() == wxID_OK)
        fillList();
}

void VozidlaDialog::OnDeleteVozidlo(wxCommandEvent& /*event*/)
{
    Vozidlo vo = idbmodel->getVozidlo(m_selectedId);
    if(vo.isValid())
    {
        wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Opravdu smazat vozidlo %s\n%s?"), vo.rz(),
                                                      vo.jmeno()), "Dotaz", wxOK|wxCANCEL|wxICON_QUESTION);
        if(dialog.ShowModal() == wxID_OK)
        {
            idbmodel->deleteVozidlo(m_selectedId);
            fillList();
        }
    }
}

void VozidlaDialog::setSelectedId(long selectedIndex)
{
    if(selectedIndex < 0)
        m_selectedId = -1;
    else
        m_selectedId = idbmodel->getVozidloIdByRz(m_list->GetItemText(selectedIndex,0));
    enableTools();
}

void VozidlaDialog::fillList()
{
    m_list->DeleteAllItems();
    m_list->Hide();
    dxVozidloArray vozidla = idbmodel->vozidla();
    for(size_t i=0; i < vozidla.GetCount(); i++)
    {
        wxListItem item;
        item.SetId(static_cast<long>(i));
        item.SetText(vozidla[i].rz());
        item.SetImage(-1);
        m_list->InsertItem(item);
        m_list->SetItem(static_cast<long>(i), 1, vozidla[i].jmeno());
        m_list->SetItem(static_cast<long>(i), 2, wxString::Format("%.2f", vozidla[i].spotreba()));
        m_list->SetItem(static_cast<long>(i), 3, wxEmptyString, vozidla[i].soukrome()?0:1);
    }
    m_list->Show();
    if(m_select && vozidla.GetCount())
        m_list->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
    else
        setSelectedId(-1);
}

long VozidlaDialog::selectedID() const
{
    return m_selectedId;
}

void VozidlaDialog::enableTools()
{
    if(m_selectedId == -1)
    {
        m_bar->EnableTool(ID_VOZIDLA_EDIT, false);
        m_bar->EnableTool(ID_VOZIDLA_COPY, false);
        m_bar->EnableTool(ID_VOZIDLA_DELETE, false);
    }
    else
    {
        m_bar->EnableTool(ID_VOZIDLA_EDIT, true);
        m_bar->EnableTool(ID_VOZIDLA_COPY, true);
        m_bar->EnableTool(ID_VOZIDLA_DELETE, true);
    }
}
