/***************************************************************
 * Name:      vozidlodialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2018
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include "vozidlodialog.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"

IMPLEMENT_DYNAMIC_CLASS(VozidloDialog, wxDialog)

BEGIN_EVENT_TABLE(VozidloDialog, wxDialog)
    EVT_BUTTON(wxID_OK, VozidloDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, VozidloDialog::OnCancel)
END_EVENT_TABLE()

VozidloDialog::VozidloDialog(wxWindow *parent, int id, int edit)
    : wxDialog(parent, wxID_ANY, edit ? "Vozidlo" : "Zadejte vozidlo"), m_edit(edit)
{
    createControls();
    if(m_edit)
    {
        m_id = id;
        fillControls();
    }
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void VozidloDialog::createControls()
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "RZ"), wxGBPosition(0,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_rz = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_rz, wxGBPosition(0,1), wxDefaultSpan, wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Jméno")), wxGBPosition(1,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_jmeno = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_jmeno, wxGBPosition(1,1), wxDefaultSpan, wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Druh"), wxGBPosition(2,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_druh = new wxChoice(this, wxID_ANY);
    dxDruhVozidlaArray druhy = idbmodel->druhvozidel();
    for(size_t i=0; i<druhy.GetCount(); i++)
    {
        m_druh->Append(druhy[i].jmeno());
    }
    if(!m_edit) m_druh->SetSelection(0);
    gridSizer->Add(m_druh, wxGBPosition(2,1), wxDefaultSpan, wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Palivo"), wxGBPosition(3,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_phm = new wxChoice(this, wxID_ANY);
    dxPhmArray phm = idbmodel->phm();
    for(size_t i=0; i<phm.GetCount(); i++)
    {
        m_phm->Append(phm[i].jmeno());
    }
    if(!m_edit) m_phm->SetSelection(0);
    gridSizer->Add(m_phm, wxGBPosition(3,1), wxDefaultSpan, wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Spotřeba")), wxGBPosition(4,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_spotreba = new wxSpinCtrlDouble(this, wxID_ANY);
    m_spotreba->SetDigits(2);
    m_spotreba->SetRange(0.0, 1000000.0);
    gridSizer->Add(m_spotreba, wxGBPosition(4,1), wxDefaultSpan, wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Soukromé")), wxGBPosition(5,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_soukrome = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_soukrome, wxGBPosition(5,1), wxDefaultSpan, wxEXPAND);

    m_mainSizer->Add(gridSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void VozidloDialog::fillControls()
{
    Vozidlo vozidlo = idbmodel->getVozidlo(m_id);
    if(vozidlo.isValid())
    {
        m_rz->SetValue(vozidlo.rz());
        m_jmeno->SetValue(vozidlo.jmeno());
        m_druh->SetSelection(vozidlo.druhvozidla()-1);
        m_spotreba->SetValue(vozidlo.spotreba());
        m_soukrome->SetValue(vozidlo.soukrome());
        m_phm->SetSelection(vozidlo.phm()-1);
    }
}

void VozidloDialog::OnOk(wxCommandEvent& /*event*/)
{
    if(m_rz->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("RZ nemůže být prázdná"), "Informace", m_edit? wxOK|wxICON_INFORMATION : wxOK|wxICON_WARNING);
        dialog.ShowModal();
        m_rz->SetFocus();
        return;
    }
    if(rzExists())
    {
        wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("RZ %s už existuje"), m_rz->GetValue()), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        m_rz->SetFocus();
        return;
    }
    if(!m_edit)
    {
        idbmodel->addVozidlo(Vozidlo(idbmodel->getLastId("vozidlo"), m_jmeno->GetValue(), m_rz->GetValue(), m_druh->GetSelection()+1, m_spotreba->GetValue(),
                                     m_soukrome->GetValue(), m_phm->GetSelection()+1));
    }
    else
    {
        idbmodel->editVozidlo(Vozidlo(m_id, m_jmeno->GetValue(), m_rz->GetValue(), m_druh->GetSelection()+1, m_spotreba->GetValue(),
                                      m_soukrome->GetValue(), m_phm->GetSelection()+1));
    }
    EndModal(wxID_OK);
}

void VozidloDialog::OnCancel(wxCommandEvent& /*event*/)
{
    if(m_edit == 2) idbmodel->deleteVozidlo(m_id);
    EndModal(wxID_CANCEL);
}

bool VozidloDialog::rzExists()
{
    if(m_edit)
    {
        wxString rz = idbmodel->hasVozidlo(m_id) ? idbmodel->getVozidlo(m_id).rz() : "";
        if(rz == m_rz->GetValue())
            return false;
        else
            return idbmodel->hasVozidlo(m_rz->GetValue());
    }
    else
    {
        return idbmodel->hasVozidlo(m_rz->GetValue());
    }
}

