/***************************************************************
 * Name:      zasobadialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include "zasobadialog.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "resources/images/add.xpm"
#include "resources/images/edit.xpm"
#include "resources/images/delete.xpm"
#include "pohybzasobdialog.h"

IMPLEMENT_DYNAMIC_CLASS(ZasobaDialog, wxDialog)

BEGIN_EVENT_TABLE(ZasobaDialog, wxDialog)
    EVT_BUTTON(wxID_OK, ZasobaDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, ZasobaDialog::OnCancel)
    EVT_CHOICE(ID_ZASOBA_SKLAD, ZasobaDialog::OnSklad)
    EVT_CHOICE(ID_ZASOBA_OCENOVANI, ZasobaDialog::OnOcenovani)
    EVT_MENU(ID_ZASOBA_POHYB_ADD, ZasobaDialog::OnAddPohyb)
    EVT_MENU(ID_ZASOBA_POHYB_EDIT, ZasobaDialog::OnEditPohyb)
    EVT_MENU(ID_ZASOBA_POHYB_DELETE, ZasobaDialog::OnDeletePohyb)
    EVT_LIST_ITEM_RIGHT_CLICK(ID_ZASOBA_POHYBY, ZasobaDialog::OnPohybRightClick)
    EVT_LIST_ITEM_ACTIVATED(ID_ZASOBA_POHYBY, ZasobaDialog::OnPohybActivated)
    EVT_LIST_ITEM_SELECTED(ID_ZASOBA_POHYBY, ZasobaDialog::OnPohybSelected)
    EVT_LIST_ITEM_DESELECTED(ID_ZASOBA_POHYBY, ZasobaDialog::OnPohybDeselected)
    EVT_TEXT(ID_ZASOBA_MJ, ZasobaDialog::OnMj)
END_EVENT_TABLE()

//Nova zasoba
ZasobaDialog::ZasobaDialog(wxWindow *parent)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Karta zásob")), m_edit(0), m_pohybSelected(-1)
{
    m_id = idbmodel->getLastId("zasoba");
    idbmodel->addZasoba(Zasoba(m_id, wxEmptyString, wxEmptyString, wxEmptyString, 0, wxEmptyString, 0.0, 0.0, idbmodel->firstSklad().id(), 0.0, false, 0));
    createControls();
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

//Editace zasoby
ZasobaDialog::ZasobaDialog(wxWindow *parent, int id, int edit)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Karta zásob")), m_id(id), m_edit(edit), m_pohybSelected(-1)
{
    createControls();
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void ZasobaDialog::createControls()
{
    int i=0; //row

    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Číslo karty")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_cislo = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_cislo, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Název")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_nazev = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_nazev, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Popis"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_popis = new wxTextCtrl(this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE);
    m_popis->SetMinSize(wxSize(-1,80));
    gridSizer->Add(m_popis, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Měrná\njednotka")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_mj = new wxTextCtrl(this, ID_ZASOBA_MJ);
    gridSizer->Add(m_mj, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Sklad"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_sklad = new wxChoice(this, ID_ZASOBA_SKLAD);
    dxSkladArray sklady = idbmodel->sklady();
    for(size_t i=0; i < sklady.GetCount(); i++)
    {
        m_sklad->Append(sklady[i].nazev(), new Sklad(sklady[i].id(), "", "", sklady[i].ocenovani()));
    }
    gridSizer->Add(m_sklad, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Oceňování\nzásob")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_ocenovani = new wxChoice(this, ID_ZASOBA_OCENOVANI);
    m_ocenovani->Append("FIFO");
    m_ocenovani->Append(wxString::FromUTF8("Vážený aritmetický průměr"));
    gridSizer->Add(m_ocenovani, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Zásoba:")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_pocet = new wxStaticText(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_pocet, wxGBPosition(i,1), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Hodnota zásoby:")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_celkem = new wxStaticText(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_celkem, wxGBPosition(i,1), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Prodejní cena\nza mj")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_cenamj = new wxSpinCtrlDouble(this);
    m_cenamj->SetDigits(2);
    m_cenamj->SetRange(0.0, 9999999.0);
    gridSizer->Add(m_cenamj, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    if(idbmodel->isFirmaPlatce())
    {
        m_bezdph = new wxCheckBox(this, wxID_ANY, "bez DPH");
        m_bezdph->SetValue(true);
        gridSizer->Add(m_bezdph, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
        i++;

        m_sazba = new wxChoice(this, wxID_ANY);
        dxDphSazbaArray sazby = idbmodel->dphsazby();
        for(size_t j=0; j<sazby.GetCount(); j++)
        {
            m_sazba->Append(wxString::Format("%d%%", sazby[j].sazba()), new DphSazba(sazby[j].id(), sazby[j].sazba(), sazby[j].rok(), sazby[j].zakladni()));
        }
        m_sazba->Select(0);
        gridSizer->Add(new wxStaticText(this, wxID_ANY, "Sazba DPH"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
        gridSizer->Add(m_sazba, wxGBPosition(i,1), wxGBSpan(1,3), wxEXPAND);
        i++;
    }

    wxStaticBoxSizer *box = new wxStaticBoxSizer(wxVERTICAL, this, wxString::FromUTF8("Pohyby zásoby"));
    wxBoxSizer *sizer = new wxBoxSizer(wxVERTICAL);
    m_pohybyBar = new dxToolBar(this);
    m_pohybyBar->AddTool(ID_ZASOBA_POHYB_ADD, wxEmptyString, wxBitmap(add_xpm), dxutils::GreyScaledIcon(add_xpm), wxITEM_NORMAL, wxString::FromUTF8("Přidat pohyb zásoby"));
    m_pohybyBar->AddTool(ID_ZASOBA_POHYB_EDIT, wxEmptyString, wxBitmap(edit_xpm), dxutils::GreyScaledIcon(edit_xpm), wxITEM_NORMAL, wxString::FromUTF8("Upravit pohyb zásoby"));
    m_pohybyBar->AddTool(ID_ZASOBA_POHYB_DELETE, wxEmptyString, wxBitmap(delete_xpm), dxutils::GreyScaledIcon(delete_xpm), wxITEM_NORMAL, wxString::FromUTF8("Smazat pohyb zásoby"));
    m_pohybyBar->Realize();
    sizer->Add(m_pohybyBar, 0, wxALL|wxEXPAND, 5);
    m_pohybyList = new wxListCtrl(this, ID_ZASOBA_POHYBY, wxDefaultPosition, wxDefaultSize, wxLC_REPORT|wxLC_HRULES|wxLC_VRULES|wxLC_SINGLE_SEL);
    m_pohybyList->InsertColumn(0, "Datum", wxLIST_FORMAT_LEFT, 150);
    m_pohybyList->InsertColumn(1, wxString::FromUTF8("Číslo dokladu"), wxLIST_FORMAT_LEFT, 120);
    m_pohybyList->InsertColumn(2, wxString::FromUTF8("Číslo faktury, inventury"), wxLIST_FORMAT_LEFT, 200);
    m_pohybyList->InsertColumn(3, "Popis", wxLIST_FORMAT_LEFT, 350);
    m_pohybyList->InsertColumn(4, "Pohyb (mj)", wxLIST_FORMAT_RIGHT, 100);
    m_pohybyList->InsertColumn(5, "Cena za mj", wxLIST_FORMAT_RIGHT, 100);
    enablePohybyTools();
    sizer->Add(m_pohybyList, 1, wxALL|wxEXPAND, 5);
    box->Add(sizer, 1, wxGROW);
    gridSizer->Add(box, wxGBPosition(i,0), wxGBSpan(1,2), wxEXPAND);

    m_mainSizer->Add(gridSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void ZasobaDialog::fillControls()
{
    if(m_edit)
    {
        Zasoba za = idbmodel->getZasoba(m_id);
        if(za.isValid())
        {
            m_cislo->SetValue(za.cislo());
            m_nazev->SetValue(za.nazev());
            m_popis->SetValue(za.popis());
            m_ocenovani->SetSelection(za.ocenovani());
            m_mj->SetValue(za.mj());
            for(unsigned int i=0; i<m_sklad->GetCount(); i++)
            {
                if(static_cast<Sklad*>(m_sklad->GetClientData(i))->id() == za.skladid())
                {
                    m_sklad->SetSelection(static_cast<int>(i));
                    break;
                }
            }
            m_cenamj->SetValue(za.cenamj());
            if(idbmodel->isFirmaPlatce())
            {
                m_bezdph->SetValue(za.bezdph());
                m_sazba->SetStringSelection(wxString::Format("%d%%", za.dph()));
            }
        }
        fillPohyby();
    }
    else
    {
        m_cislo->SetValue(idbmodel->getCisloDokladu("ZB"));
        m_sklad->SetSelection(0);
        m_ocenovani->SetSelection(idbmodel->firstSklad().ocenovani());
    }
}

void ZasobaDialog::fillPohyby()
{
    m_pohybyList->DeleteAllItems();
    m_pohybyList->Hide();
    m_pohyby = idbmodel->pohybyzasobByZasobaId(m_id);
    for(size_t i=0; i < m_pohyby.GetCount(); i++)
    {
        wxListItem item;
        item.SetId(static_cast<long>(i));
        item.SetText(m_pohyby[i].datum().Format("%d.%m.%Y %H:%M:%S"));
        m_pohybyList->InsertItem(item);
        m_pohybyList->SetItem(static_cast<long>(i), 1, m_pohyby[i].cislo());
        if(m_pohyby[i].inventura()) m_pohybyList->SetItem(static_cast<long>(i), 2, idbmodel->getInventuraCisloById(m_pohyby[i].inventuraid()));
        else m_pohybyList->SetItem(static_cast<long>(i), 2, idbmodel->getFakturaCisloById(m_pohyby[i].fakturaid()));
        wxString popis;
        if(m_pohyby[i].pocatek())
        {
            popis = wxString::FromUTF8("Počáteční stav");
        }
        else
        {
            if(m_pohyby[i].inventura())
            {
                popis = wxString::FromUTF8("Zaevidování inventurního stavu");
            }
            else
            {
                if(m_pohyby[i].prijemka()) popis = wxString::FromUTF8("Příjem na sklad");
                else popis = wxString::FromUTF8("Výdej ze skladu");
            }
        }
        m_pohybyList->SetItem(static_cast<long>(i), 3, popis);
        m_pohybyList->SetItem(static_cast<long>(i), 4, m_pohyby[i].prijemka() ? wxString::Format("+%.3f", m_pohyby[i].pocet()) : wxString::Format("-%.3f", m_pohyby[i].pocet()));
        m_pohybyList->SetItem(static_cast<long>(i), 5, m_pohyby[i].prijemka() ? wxString::Format("%.2f", m_pohyby[i].cenamj()): "");
    }
    m_pohybyList->Show();
    m_pohybSelected = -1;
    enablePohybyTools();
    updateLabels();
}

void ZasobaDialog::enablePohybyTools()
{
    if(isEditDeleteEnabled())
    {
        m_pohybyBar->EnableTool(ID_ZASOBA_POHYB_EDIT, true);
        m_pohybyBar->EnableTool(ID_ZASOBA_POHYB_DELETE, true);
    }
    else
    {
        m_pohybyBar->EnableTool(ID_ZASOBA_POHYB_EDIT, false);
        m_pohybyBar->EnableTool(ID_ZASOBA_POHYB_DELETE, false);
    }
}

void ZasobaDialog::OnOk(wxCommandEvent& /*event*/)
{
    if(m_nazev->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Název nemůže být prázdný"), "Informace", m_edit? wxOK|wxICON_INFORMATION : wxOK|wxICON_WARNING);
        dialog.ShowModal();
        m_nazev->SetFocus();
        return;
    }
    idbmodel->editZasoba(Zasoba(m_id, m_cislo->GetValue(), m_nazev->GetValue(), m_popis->GetValue(), m_ocenovani->GetSelection(), m_mj->GetValue(),
                                0.0, 0.0, static_cast<Sklad*>(m_sklad->GetClientData(static_cast<unsigned int>(m_sklad->GetSelection())))->id(), m_cenamj->GetValue(), idbmodel->isFirmaPlatce()?m_bezdph->GetValue():0,
                                idbmodel->isFirmaPlatce()?static_cast<DphSazba*>(m_sazba->GetClientData(static_cast<unsigned int>(m_sazba->GetSelection())))->sazba():0));
    if(m_edit == 0 || m_edit == 2)
    {
        idbmodel->increaseRadaCislo("ZB");
    }
    EndModal(wxID_OK);
}

void ZasobaDialog::OnCancel(wxCommandEvent& /*event*/)
{
    if(m_edit == 0 || m_edit == 2)
    {
        idbmodel->deleteZasoba(m_id);
    }
    EndModal(wxID_CANCEL);
}

void ZasobaDialog::OnSklad(wxCommandEvent &/*event*/)
{
    m_ocenovani->SetSelection(static_cast<Sklad*>(m_sklad->GetClientData(static_cast<unsigned int>(m_sklad->GetSelection())))->ocenovani());
    idbmodel->changeZasobaOcenovani(m_id, m_ocenovani->GetSelection());
    updateLabels();
}

void ZasobaDialog::OnOcenovani(wxCommandEvent &/*event*/)
{
    idbmodel->changeZasobaOcenovani(m_id, m_ocenovani->GetSelection());
    updateLabels();
}

void ZasobaDialog::OnMj(wxCommandEvent &/*event*/)
{
    updateLabels();
}

void ZasobaDialog::OnAddPohyb(wxCommandEvent &/*event*/)
{
    wxArrayString choices;
    choices.Add(wxString::FromUTF8("Příjem na sklad"));
    choices.Add(wxString::FromUTF8("Výdej ze skladu"));
    wxSingleChoiceDialog vyber(this, "Vyberte typ pohybu", "Typ pohybu", choices);
    vyber.SetSelection(0);
    if(vyber.ShowModal() == wxID_OK)
    {
        PohybzasobDialog dialog(this, !vyber.GetSelection(), m_id);
        if(dialog.ShowModal() == wxID_OK)
        {
            fillPohyby();
        }
    }
}

void ZasobaDialog::OnEditPohyb(wxCommandEvent &/*event*/)
{
    PohybzasobDialog dialog(this, m_pohyby[static_cast<size_t>(m_pohybSelected)].id());
    if(dialog.ShowModal() == wxID_OK)
    {
        fillPohyby();
    }
}

void ZasobaDialog::OnDeletePohyb(wxCommandEvent &/*event*/)
{
    wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Opravdu smazat pohyb:\n%s\n%.3f %s\nz %s?"), m_pohyby[static_cast<size_t>(m_pohybSelected)].prijemka()?wxString::FromUTF8("Příjem na sklad"):wxString::FromUTF8("Výdej ze skladu"),
                                                  m_pohyby[static_cast<size_t>(m_pohybSelected)].pocet(), m_mj->GetValue(), m_pohyby[static_cast<size_t>(m_pohybSelected)].datum().Format("%d.%m.%Y %H:%M:%S")),
                           "Dotaz", wxOK|wxCANCEL|wxICON_QUESTION);
    if(dialog.ShowModal() == wxID_OK)
    {
        idbmodel->deletePohybzasob(m_pohyby[static_cast<size_t>(m_pohybSelected)].id());
        fillPohyby();
    }
}

void ZasobaDialog::OnPohybRightClick(wxListEvent &event)
{
    m_pohybSelected = event.GetIndex();
    enablePohybyTools();
    wxMenu popup;
    popup.Append(ID_ZASOBA_POHYB_ADD, wxString::FromUTF8("Nový pohyb zásoby"));
    if(isEditDeleteEnabled())
    {
        popup.AppendSeparator();
        popup.Append(ID_ZASOBA_POHYB_EDIT, wxString::FromUTF8("Upravit pohyb zásoby"));
        popup.Append(ID_ZASOBA_POHYB_DELETE, wxString::FromUTF8("Smazat pohyb zásoby"));
    }
    PopupMenu(&popup, event.GetPoint()+m_pohybyList->GetPosition());
}

void ZasobaDialog::OnPohybActivated(wxListEvent &event)
{
    m_pohybSelected = event.GetIndex();
    if(isEditDeleteEnabled())
    {
        PohybzasobDialog dialog(this, m_pohyby[static_cast<size_t>(m_pohybSelected)].id());
        if(dialog.ShowModal() == wxID_OK)
        {
            fillPohyby();
        }
    }
}

void ZasobaDialog::OnPohybSelected(wxListEvent &event)
{
    m_pohybSelected = event.GetIndex();
    enablePohybyTools();
}

void ZasobaDialog::OnPohybDeselected(wxListEvent &/*event*/)
{
    m_pohybSelected = -1;
    enablePohybyTools();
}

bool ZasobaDialog::isEditDeleteEnabled()
{
    if(m_pohybSelected == -1)
    {
        return false;
    }
    if(m_pohyby[static_cast<size_t>(m_pohybSelected)].datum().IsEarlierThan(idbmodel->getInventuraLastDateForZasoba(m_id)))
    {
        return false;
    }
    if(m_pohyby[static_cast<size_t>(m_pohybSelected)].inventura())
    {
        return false;
    }
    if(m_pohyby[static_cast<size_t>(m_pohybSelected)].pocatek())
    {
        return false;
    }
    if(idbmodel->hasFaktura(m_pohyby[static_cast<size_t>(m_pohybSelected)].fakturaid()))
    {
        return false;
    }
    return true;
}

void ZasobaDialog::updateLabels()
{
    Zasoba za = idbmodel->getZasoba(m_id);
    if(za.isValid())
    {
        m_pocet->SetLabel(wxString::Format("%.3f %s", za.pocet(), m_mj->GetValue()));
        m_celkem->SetLabel(wxString::Format(wxString::FromUTF8("%.2f Kč"), za.celkem()));
    }
}
