/***************************************************************
 * Name:      zavpohdialog.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#ifndef ZAVPOHDIALOG_H
#define ZAVPOHDIALOG_H

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif
#include <wx/gbsizer.h>
#include <wx/datectrl.h>
#include <wx/dateevt.h>
#include "model/dbmodel.h"
#include "gui/widgets/dxlistctrl.h"
#include "gui/widgets/dxtoolbar.h"

class ZavpohDialog;

class ZpPolozkaCtrl: public dxListCtrl
{
public:
    ZpPolozkaCtrl(ZavpohDialog *dialog, wxWindow *parent, const wxWindowID id)
        : dxListCtrl(parent, id), m_parent(dialog) {}

    enum PColumn
    {
        PCOL_POPIS = 0,
        PCOL_MNOZSTVI,
        PCOL_MJ,
        PCOL_CENAMJ,
        PCOL_CENACELKEM,
        PCOL_MAX, // pocet sloupcu
    };

    void CreateColumns();
private:
    ZavpohDialog *m_parent;

    virtual wxString OnGetItemText(long item, long column) const;
    virtual int OnGetItemColumnImage(long item, long column) const;
    virtual wxListItemAttr *OnGetItemAttr(long item) const;
};

class ZavpohDialog: public wxDialog
{
    DECLARE_DYNAMIC_CLASS(ZavpohDialog)
    DECLARE_EVENT_TABLE()
public:
    ZavpohDialog() : m_id(-1), m_edit(0), m_zavazek(false), m_platce(false), m_mainSizer(nullptr),
      m_popis(nullptr), m_cislo(nullptr), m_vs(nullptr), m_uhrada(nullptr), m_uhradaAdd(nullptr),
      m_bankapokladnaText(nullptr), m_bankapokladnaSelect(nullptr), m_predkontaceText(nullptr),
      m_predkontaceSelect(nullptr), m_vystaveni(nullptr), m_duzp(nullptr), m_dppd(nullptr), m_splatnost(nullptr), m_mena(nullptr),
      m_celkem(nullptr), m_nazev(nullptr), m_jmeno(nullptr), m_ulice(nullptr), m_cp(nullptr), m_misto(nullptr), m_psc(nullptr), m_zeme(nullptr),
      m_ic(nullptr), m_dic(nullptr), m_email(nullptr), m_kontaktSelect(nullptr), m_kontaktClear(nullptr), m_polozkaSelected(-1),
      m_polozkaBar(nullptr), m_polozkyList(nullptr), m_dph(nullptr), m_dphdruhText(nullptr), m_dphdruhSelect(nullptr), m_kurz(nullptr),
      m_kurzDownload(nullptr), m_kurzLabel(nullptr), m_kurzValue(0.0), m_pausal(nullptr), m_pausalAdd(nullptr), m_zaokrouhleni(nullptr) {}
    //novy zavazek, pohledavka
    ZavpohDialog(bool zavazek, wxWindow *parent);
    //editace zavazku, pohledavky
    ZavpohDialog(int id, wxWindow *parent, int edit = 1);

    wxString getPolozkaText(long item, long column) const;
private:
    int m_id;
    /* 0 - novy, 1 - editace, 2 - kopie */
    int m_edit;
    bool m_zavazek;
    bool m_platce;
    /* pro Cancel editovane zavazku,pohledavky */
    dxFaPolozkaArray m_oldPolozky;
    dxFaPolozkaArray m_polozky;
    wxBoxSizer *m_mainSizer;
    wxTextCtrl *m_popis;
    wxTextCtrl *m_cislo;
    wxTextCtrl *m_vs;
    wxChoice *m_uhrada;
    wxButton *m_uhradaAdd;
    wxTextCtrl *m_bankapokladnaText;
    wxButton *m_bankapokladnaSelect;
    wxTextCtrl *m_predkontaceText;
    wxButton *m_predkontaceSelect;
    wxDatePickerCtrl *m_vystaveni;
    wxDatePickerCtrl *m_duzp;
    wxDatePickerCtrl *m_dppd;
    wxDatePickerCtrl *m_splatnost;
    wxChoice *m_mena;
    wxStaticText *m_celkem;
    wxTextCtrl *m_nazev;
    wxTextCtrl *m_jmeno;
    wxTextCtrl *m_ulice;
    wxTextCtrl *m_cp;
    wxTextCtrl *m_misto;
    wxTextCtrl *m_psc;
    wxTextCtrl *m_zeme;
    wxTextCtrl *m_ic;
    wxTextCtrl *m_dic;
    wxTextCtrl *m_email;
    wxButton *m_kontaktSelect;
    wxBitmapButton *m_kontaktClear;
    long m_polozkaSelected;
    dxToolBar *m_polozkaBar;
    ZpPolozkaCtrl *m_polozkyList;
    wxListCtrl *m_dph;
    wxTextCtrl *m_dphdruhText;
    wxButton *m_dphdruhSelect;
    wxTextCtrl *m_kurz;
    wxBitmapButton *m_kurzDownload;
    wxStaticText *m_kurzLabel;
    double m_kurzValue;
    wxChoice *m_pausal;
    wxButton *m_pausalAdd;
    wxButton *m_zaokrouhleni;

    void createControls();
    wxGridBagSizer *createZavpohGrid();
    wxStaticBoxSizer *createKontaktGrid();
    wxStaticBoxSizer *createPolozkyGrid();
    void createDphGrid();
    void fillControls();
    void OnOk(wxCommandEvent& event);
    void OnCancel(wxCommandEvent& event);
    void OnChoiceUhrada(wxCommandEvent& event);
    void OnChoiceMena(wxCommandEvent& event);
    void OnAddUhrada(wxCommandEvent& event);
    void OnSelectBankaPokladna(wxCommandEvent& event);
    void OnSelectPredkontace(wxCommandEvent& event);
    void OnSelectDphDruh(wxCommandEvent& event);
    void OnVystaveniChanged(wxDateEvent& event);
    void OnDuzpChanged(wxDateEvent& event);
    void OnDppdChanged(wxDateEvent& event);
    void OnSelectKontakt(wxCommandEvent& event);
    void OnClearKontakt(wxCommandEvent& event);
    void enableDisableKontakt();
    void OnPolozkaRightClick(wxListEvent& event);
    void OnPolozkaActivated(wxListEvent& event);
    void OnPolozkaSelected(wxListEvent& event);
    void OnPolozkaDeselected(wxListEvent& event);
    void enablePolozkyTools();
    void updateData();
    void updateDph();
    void OnEditPolozka(wxCommandEvent& event);
    void OnAddPolozka(wxCommandEvent& event);
    void OnDeletePolozka(wxCommandEvent& event);
    void fillMena();
    int getMenaId() const;
    int getMenaId(unsigned int index) const;
    void showhideKurz();
    void OnKurzDownload(wxCommandEvent& event);
    void OnKurzChanged(wxCommandEvent& event);
    void OnPausalAdd(wxCommandEvent& event);
    void printZavpoh();
    void writeUhrada();
    bool isEdited();
    void OnZaokrouhlit(wxCommandEvent& event);
};

#endif  /* ZAVPOHDIALOG_H */

