/***************************************************************
 * Name:      pokladnafilter.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2018
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include "pokladnafilter.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "model/dbmodel.h"

IMPLEMENT_DYNAMIC_CLASS(PokladnaFilter, wxDialog)

BEGIN_EVENT_TABLE(PokladnaFilter, wxDialog)
    EVT_BUTTON(wxID_OK, PokladnaFilter::OnOk)
    EVT_BUTTON(wxID_CANCEL, PokladnaFilter::OnCancel)
END_EVENT_TABLE()

PokladnaFilter::PokladnaFilter(wxWindow *parent)
    : wxDialog(parent, wxID_ANY, "Filtr pro pokladnu")
{
    m_filters.Add(Filter(FA_LESS, false, dxsettings.datum()));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_EQUAL, false, 1));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_LESS, false, 0.0));
    createControls();
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

PokladnaFilter::PokladnaFilter(wxWindow *parent, dxFilterArray filters)
    : wxDialog(parent, wxID_ANY, "Filtr pro pokladnu"), m_filters(filters)
{
    createControls();
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

dxFilterArray PokladnaFilter::filters() const
{
    return m_filters;
}

void PokladnaFilter::createControls()
{
    int i = 0; //row

    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Datum"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_datumAction = new wxChoice(this, wxID_ANY);
    m_datumAction->Append(wxString::FromUTF8("Starší než"), new FilterAction(FA_LESS));
    m_datumAction->Append(wxString::FromUTF8("Starší než nebo rovné"), new FilterAction(FA_LESSEQUAL));
    m_datumAction->Append(wxString::FromUTF8("Rovné"), new FilterAction(FA_EQUAL));
    m_datumAction->Append(wxString::FromUTF8("Novější než nebo rovné"), new FilterAction(FA_GREATEREQUAL));
    m_datumAction->Append(wxString::FromUTF8("Novější než"), new FilterAction(FA_GREATER));
    gridSizer->Add(m_datumAction, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_datumValue = new wxDatePickerCtrl(this, wxID_ANY, dxsettings.datum(), wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    m_datumValue->SetRange(wxDateTime(1,wxDateTime::Jan,idbmodel->currentRok()), wxDateTime(31,wxDateTime::Dec,idbmodel->currentRok()));
    gridSizer->Add(m_datumValue, wxGBPosition(i,2), wxDefaultSpan, wxEXPAND);
    m_datumUsage = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_datumUsage, wxGBPosition(i,3), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Doklad"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_dokladAction = new wxChoice(this, wxID_ANY);
    m_dokladAction->Append("Rovno", new FilterAction(FA_EQUAL));
    m_dokladAction->Append("Obsahuje", new FilterAction(FA_CONTAINS));
    gridSizer->Add(m_dokladAction, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_dokladValue = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_dokladValue, wxGBPosition(i,2), wxDefaultSpan, wxEXPAND);
    m_dokladUsage = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_dokladUsage, wxGBPosition(i,3), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Pokladna"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_pokladnaAction = new wxChoice(this, wxID_ANY);
    m_pokladnaAction->Append("Rovno", new FilterAction(FA_EQUAL));
    m_pokladnaAction->Append("Obsahuje", new FilterAction(FA_CONTAINS));
    gridSizer->Add(m_pokladnaAction, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_pokladnaValue = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_pokladnaValue, wxGBPosition(i,2), wxDefaultSpan, wxEXPAND);
    m_pokladnaUsage = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_pokladnaUsage, wxGBPosition(i,3), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Typ"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_typAction = new wxChoice(this, wxID_ANY);
    m_typAction->Append("Rovno", new FilterAction(FA_EQUAL));
    gridSizer->Add(m_typAction, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_typValue = new wxChoice(this, wxID_ANY);
    m_typValue->Append(wxString::FromUTF8("Příjem"), new int(1));
    m_typValue->Append(wxString::FromUTF8("Výdaj"), new int(0));
    gridSizer->Add(m_typValue, wxGBPosition(i,2), wxDefaultSpan, wxEXPAND);
    m_typUsage = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_typUsage, wxGBPosition(i,3), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Popis"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_popisAction = new wxChoice(this, wxID_ANY);
    m_popisAction->Append("Rovno", new FilterAction(FA_EQUAL));
    m_popisAction->Append("Obsahuje", new FilterAction(FA_CONTAINS));
    gridSizer->Add(m_popisAction, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_popisValue = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_popisValue, wxGBPosition(i,2), wxDefaultSpan, wxEXPAND);
    m_popisUsage = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_popisUsage, wxGBPosition(i,3), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Částka")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_castkaAction = new wxChoice(this, wxID_ANY);
    m_castkaAction->Append(wxString::FromUTF8("Menší než"), new FilterAction(FA_LESS));
    m_castkaAction->Append(wxString::FromUTF8("Menší než nebo rovno"), new FilterAction(FA_LESSEQUAL));
    m_castkaAction->Append("Rovno", new FilterAction(FA_EQUAL));
    m_castkaAction->Append(wxString::FromUTF8("Větší než nebo rovno"), new FilterAction(FA_GREATEREQUAL));
    m_castkaAction->Append(wxString::FromUTF8("Větší než"), new FilterAction(FA_GREATER));
    gridSizer->Add(m_castkaAction, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_castkaValue = new wxSpinCtrlDouble(this, wxID_ANY);
    m_castkaValue->SetDigits(2);
    m_castkaValue->SetRange(0.0, 1000000.0);
    gridSizer->Add(m_castkaValue, wxGBPosition(i,2), wxDefaultSpan, wxEXPAND);
    m_castkaUsage = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_castkaUsage, wxGBPosition(i,3), wxDefaultSpan, wxEXPAND);

    m_mainSizer->Add(gridSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void PokladnaFilter::fillControls()
{
    if(m_filters.GetCount() == 6)
    {
        if(m_filters[0].type() == FT_DATUM)
        {
            m_datumAction->SetSelection(m_filters[0].action());
            m_datumValue->SetValue(m_filters[0].datum());
            m_datumUsage->SetValue(m_filters[0].active());
        }
        if(m_filters[1].type() == FT_TEXT)
        {
            m_dokladAction->SetSelection(m_filters[1].action()==FA_EQUAL?0:1);
            m_dokladValue->ChangeValue(m_filters[1].text());
            m_dokladUsage->SetValue(m_filters[1].active());
        }
        if(m_filters[2].type() == FT_TEXT)
        {
            m_pokladnaAction->SetSelection(m_filters[2].action()==FA_EQUAL?0:1);
            m_pokladnaValue->ChangeValue(m_filters[2].text());
            m_pokladnaUsage->SetValue(m_filters[2].active());
        }
        if(m_filters[3].type() == FT_INTEGER)
        {
            m_typAction->SetSelection(0);
            m_typValue->SetSelection(m_filters[3].intvalue()?0:1);
            m_typUsage->SetValue(m_filters[3].active());
        }
        if(m_filters[4].type() == FT_TEXT)
        {
            m_popisAction->SetSelection(m_filters[4].action()==FA_EQUAL?0:1);
            m_popisValue->ChangeValue(m_filters[4].text());
            m_popisUsage->SetValue(m_filters[4].active());
        }
        if(m_filters[5].type() == FT_DOUBLE)
        {
            m_castkaAction->SetSelection(m_filters[5].action());
            m_castkaValue->SetValue(m_filters[5].doublevalue());
            m_castkaUsage->SetValue(m_filters[5].active());
        }
    }
}

void PokladnaFilter::OnOk(wxCommandEvent& /*event*/)
{
    m_filters[0].setAction(*static_cast<FilterAction*>(m_datumAction->GetClientData(static_cast<unsigned int>(m_datumAction->GetSelection()))));
    m_filters[0].setDatum(m_datumValue->GetValue());
    m_filters[0].setActive(m_datumUsage->GetValue());

    m_filters[1].setAction(*static_cast<FilterAction*>(m_dokladAction->GetClientData(static_cast<unsigned int>(m_dokladAction->GetSelection()))));
    m_filters[1].setText(m_dokladValue->GetValue());
    m_filters[1].setActive(m_dokladUsage->GetValue());

    m_filters[2].setAction(*static_cast<FilterAction*>(m_pokladnaAction->GetClientData(static_cast<unsigned int>(m_pokladnaAction->GetSelection()))));
    m_filters[2].setText(m_pokladnaValue->GetValue());
    m_filters[2].setActive(m_pokladnaUsage->GetValue());

    m_filters[3].setAction(FA_EQUAL);
    m_filters[3].setIntvalue(*static_cast<int*>(m_typValue->GetClientData(static_cast<unsigned int>(m_typValue->GetSelection()))));
    m_filters[3].setActive(m_typUsage->GetValue());

    m_filters[4].setAction(*static_cast<FilterAction*>(m_popisAction->GetClientData(static_cast<unsigned int>(m_popisAction->GetSelection()))));
    m_filters[4].setText(m_popisValue->GetValue());
    m_filters[4].setActive(m_popisUsage->GetValue());

    m_filters[5].setAction(*static_cast<FilterAction*>(m_castkaAction->GetClientData(static_cast<unsigned int>(m_castkaAction->GetSelection()))));
    m_filters[5].setDoublevalue(m_castkaValue->GetValue());
    m_filters[5].setActive(m_castkaUsage->GetValue());

    EndModal(wxID_OK);
}

void PokladnaFilter::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

