/***************************************************************
 * Name:      skladfilter.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2018
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include "skladfilter.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "model/dbmodel.h"

IMPLEMENT_DYNAMIC_CLASS(SkladFilter, wxDialog)

BEGIN_EVENT_TABLE(SkladFilter, wxDialog)
    EVT_BUTTON(wxID_OK, SkladFilter::OnOk)
    EVT_BUTTON(wxID_CANCEL, SkladFilter::OnCancel)
END_EVENT_TABLE()

SkladFilter::SkladFilter(wxWindow *parent)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Filtr pro knihu zásob"))
{
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_LESS, false, 0.0));
    m_filters.Add(Filter(FA_LESS, false, 0.0));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    createControls();
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

SkladFilter::SkladFilter(wxWindow *parent, dxFilterArray filters)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Filtr pro knihu zásob")), m_filters(filters)
{
    createControls();
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

dxFilterArray SkladFilter::filters() const
{
    return m_filters;
}

void SkladFilter::createControls()
{
    int i = 0; //row

    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Číslo")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_cisloAction = new wxChoice(this, wxID_ANY);
    m_cisloAction->Append("Rovno", new FilterAction(FA_EQUAL));
    m_cisloAction->Append("Obsahuje", new FilterAction(FA_CONTAINS));
    gridSizer->Add(m_cisloAction, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_cisloValue = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_cisloValue, wxGBPosition(i,2), wxDefaultSpan, wxEXPAND);
    m_cisloUsage = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_cisloUsage, wxGBPosition(i,3), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Název")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_nazevAction = new wxChoice(this, wxID_ANY);
    m_nazevAction->Append("Rovno", new FilterAction(FA_EQUAL));
    m_nazevAction->Append("Obsahuje", new FilterAction(FA_CONTAINS));
    gridSizer->Add(m_nazevAction, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_nazevValue = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_nazevValue, wxGBPosition(i,2), wxDefaultSpan, wxEXPAND);
    m_nazevUsage = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_nazevUsage, wxGBPosition(i,3), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Stav"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_stavAction = new wxChoice(this, wxID_ANY);
    m_stavAction->Append(wxString::FromUTF8("Menší než"), new FilterAction(FA_LESS));
    m_stavAction->Append(wxString::FromUTF8("Menší než nebo rovno"), new FilterAction(FA_LESSEQUAL));
    m_stavAction->Append("Rovno", new FilterAction(FA_EQUAL));
    m_stavAction->Append(wxString::FromUTF8("Větší než nebo rovno"), new FilterAction(FA_GREATEREQUAL));
    m_stavAction->Append(wxString::FromUTF8("Větší než"), new FilterAction(FA_GREATER));
    gridSizer->Add(m_stavAction, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_stavValue = new wxSpinCtrlDouble(this, wxID_ANY);
    m_stavValue->SetDigits(2);
    m_stavValue->SetRange(0.0, 10000000.0);
    gridSizer->Add(m_stavValue, wxGBPosition(i,2), wxDefaultSpan, wxEXPAND);
    m_stavUsage = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_stavUsage, wxGBPosition(i,3), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Hodnota"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_hodnotaAction = new wxChoice(this, wxID_ANY);
    m_hodnotaAction->Append(wxString::FromUTF8("Menší než"), new FilterAction(FA_LESS));
    m_hodnotaAction->Append(wxString::FromUTF8("Menší než nebo rovno"), new FilterAction(FA_LESSEQUAL));
    m_hodnotaAction->Append("Rovno", new FilterAction(FA_EQUAL));
    m_hodnotaAction->Append(wxString::FromUTF8("Větší než nebo rovno"), new FilterAction(FA_GREATEREQUAL));
    m_hodnotaAction->Append(wxString::FromUTF8("Větší než"), new FilterAction(FA_GREATER));
    gridSizer->Add(m_hodnotaAction, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_hodnotaValue = new wxSpinCtrlDouble(this, wxID_ANY);
    m_hodnotaValue->SetDigits(2);
    m_hodnotaValue->SetRange(0.0, 10000000.0);
    gridSizer->Add(m_hodnotaValue, wxGBPosition(i,2), wxDefaultSpan, wxEXPAND);
    m_hodnotaUsage = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_hodnotaUsage, wxGBPosition(i,3), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Sklad"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_skladAction = new wxChoice(this, wxID_ANY);
    m_skladAction->Append("Rovno", new FilterAction(FA_EQUAL));
    m_skladAction->Append("Obsahuje", new FilterAction(FA_CONTAINS));
    gridSizer->Add(m_skladAction, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_skladValue = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_skladValue, wxGBPosition(i,2), wxDefaultSpan, wxEXPAND);
    m_skladUsage = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_skladUsage, wxGBPosition(i,3), wxDefaultSpan, wxEXPAND);

    m_mainSizer->Add(gridSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void SkladFilter::fillControls()
{
    if(m_filters.GetCount() == 5)
    {
        if(m_filters[0].type() == FT_TEXT)
        {
            m_cisloAction->SetSelection(m_filters[0].action()==FA_EQUAL?0:1);
            m_cisloValue->ChangeValue(m_filters[0].text());
            m_cisloUsage->SetValue(m_filters[0].active());
        }
        if(m_filters[1].type() == FT_TEXT)
        {
            m_nazevAction->SetSelection(m_filters[1].action()==FA_EQUAL?0:1);
            m_nazevValue->ChangeValue(m_filters[1].text());
            m_nazevUsage->SetValue(m_filters[1].active());
        }
        if(m_filters[2].type() == FT_DOUBLE)
        {
            m_stavAction->SetSelection(m_filters[2].action());
            m_stavValue->SetValue(m_filters[2].doublevalue());
            m_stavUsage->SetValue(m_filters[2].active());
        }
        if(m_filters[3].type() == FT_DOUBLE)
        {
            m_hodnotaAction->SetSelection(m_filters[3].action());
            m_hodnotaValue->SetValue(m_filters[3].doublevalue());
            m_hodnotaUsage->SetValue(m_filters[3].active());
        }
        if(m_filters[4].type() == FT_TEXT)
        {
            m_skladAction->SetSelection(m_filters[4].action()==FA_EQUAL?0:1);
            m_skladValue->ChangeValue(m_filters[4].text());
            m_skladUsage->SetValue(m_filters[4].active());
        }
    }
}

void SkladFilter::OnOk(wxCommandEvent& /*event*/)
{
    m_filters[0].setAction(*static_cast<FilterAction*>(m_cisloAction->GetClientData(static_cast<unsigned int>(m_cisloAction->GetSelection()))));
    m_filters[0].setText(m_cisloValue->GetValue());
    m_filters[0].setActive(m_cisloUsage->GetValue());

    m_filters[1].setAction(*static_cast<FilterAction*>(m_nazevAction->GetClientData(static_cast<unsigned int>(m_nazevAction->GetSelection()))));
    m_filters[1].setText(m_nazevValue->GetValue());
    m_filters[1].setActive(m_nazevUsage->GetValue());

    m_filters[2].setAction(*static_cast<FilterAction*>(m_stavAction->GetClientData(static_cast<unsigned int>(m_stavAction->GetSelection()))));
    m_filters[2].setDoublevalue(m_stavValue->GetValue());
    m_filters[2].setActive(m_stavUsage->GetValue());

    m_filters[3].setAction(*static_cast<FilterAction*>(m_hodnotaAction->GetClientData(static_cast<unsigned int>(m_hodnotaAction->GetSelection()))));
    m_filters[3].setDoublevalue(m_hodnotaValue->GetValue());
    m_filters[3].setActive(m_hodnotaUsage->GetValue());

    m_filters[4].setAction(*static_cast<FilterAction*>(m_skladAction->GetClientData(static_cast<unsigned int>(m_skladAction->GetSelection()))));
    m_filters[4].setText(m_skladValue->GetValue());
    m_filters[4].setActive(m_skladUsage->GetValue());

    EndModal(wxID_OK);
}

void SkladFilter::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

