/***************************************************************
 * Name:      vydanafilter.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2018
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include "vydanafilter.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "model/dbmodel.h"

IMPLEMENT_DYNAMIC_CLASS(VydanaFilter, wxDialog)

BEGIN_EVENT_TABLE(VydanaFilter, wxDialog)
    EVT_BUTTON(wxID_OK, VydanaFilter::OnOk)
    EVT_BUTTON(wxID_CANCEL, VydanaFilter::OnCancel)
END_EVENT_TABLE()

VydanaFilter::VydanaFilter(wxWindow *parent)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Filtr pro faktury vydané"))
{
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_LESS, false, dxsettings.datum()));
    m_filters.Add(Filter(FA_LESS, false, dxsettings.datum()));
    m_filters.Add(Filter(FA_LESS, false, 0.0));
    m_filters.Add(Filter(FA_LESS, false, 0.0));
    createControls();
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

VydanaFilter::VydanaFilter(wxWindow *parent, dxFilterArray filters)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Filtr pro faktury vydané")), m_filters(filters)
{
    createControls();
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

dxFilterArray VydanaFilter::filters() const
{
    return m_filters;
}

void VydanaFilter::createControls()
{
    int i = 0; //row

    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Doklad"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_dokladAction = new wxChoice(this, wxID_ANY);
    m_dokladAction->Append("Rovno", new FilterAction(FA_EQUAL));
    m_dokladAction->Append("Obsahuje", new FilterAction(FA_CONTAINS));
    gridSizer->Add(m_dokladAction, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_dokladValue = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_dokladValue, wxGBPosition(i,2), wxDefaultSpan, wxEXPAND);
    m_dokladUsage = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_dokladUsage, wxGBPosition(i,3), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Popis"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_popisAction = new wxChoice(this, wxID_ANY);
    m_popisAction->Append("Rovno", new FilterAction(FA_EQUAL));
    m_popisAction->Append("Obsahuje", new FilterAction(FA_CONTAINS));
    gridSizer->Add(m_popisAction, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_popisValue = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_popisValue, wxGBPosition(i,2), wxDefaultSpan, wxEXPAND);
    m_popisUsage = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_popisUsage, wxGBPosition(i,3), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Odběratel")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_protistranaAction = new wxChoice(this, wxID_ANY);
    m_protistranaAction->Append("Rovno", new FilterAction(FA_EQUAL));
    m_protistranaAction->Append("Obsahuje", new FilterAction(FA_CONTAINS));
    gridSizer->Add(m_protistranaAction, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_protistranaValue = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_protistranaValue, wxGBPosition(i,2), wxDefaultSpan, wxEXPAND);
    m_protistranaUsage = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_protistranaUsage, wxGBPosition(i,3), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Vystavena"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_vystavenoAction = new wxChoice(this, wxID_ANY);
    m_vystavenoAction->Append(wxString::FromUTF8("Starší než"), new FilterAction(FA_LESS));
    m_vystavenoAction->Append(wxString::FromUTF8("Starší než nebo rovné"), new FilterAction(FA_LESSEQUAL));
    m_vystavenoAction->Append(wxString::FromUTF8("Rovné"), new FilterAction(FA_EQUAL));
    m_vystavenoAction->Append(wxString::FromUTF8("Novější než nebo rovné"), new FilterAction(FA_GREATEREQUAL));
    m_vystavenoAction->Append(wxString::FromUTF8("Novější než"), new FilterAction(FA_GREATER));
    gridSizer->Add(m_vystavenoAction, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_vystavenoValue = new wxDatePickerCtrl(this, wxID_ANY, dxsettings.datum(), wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    m_vystavenoValue->SetRange(wxDateTime(1,wxDateTime::Jan,idbmodel->currentRok()), wxDateTime(31,wxDateTime::Dec,idbmodel->currentRok()));
    gridSizer->Add(m_vystavenoValue, wxGBPosition(i,2), wxDefaultSpan, wxEXPAND);
    m_vystavenoUsage = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_vystavenoUsage, wxGBPosition(i,3), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Splatná")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_splatnoAction = new wxChoice(this, wxID_ANY);
    m_splatnoAction->Append(wxString::FromUTF8("Starší než"), new FilterAction(FA_LESS));
    m_splatnoAction->Append(wxString::FromUTF8("Starší než nebo rovné"), new FilterAction(FA_LESSEQUAL));
    m_splatnoAction->Append(wxString::FromUTF8("Rovné"), new FilterAction(FA_EQUAL));
    m_splatnoAction->Append(wxString::FromUTF8("Novější než nebo rovné"), new FilterAction(FA_GREATEREQUAL));
    m_splatnoAction->Append(wxString::FromUTF8("Novější než"), new FilterAction(FA_GREATER));
    gridSizer->Add(m_splatnoAction, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_splatnoValue = new wxDatePickerCtrl(this, wxID_ANY, dxsettings.datum(), wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    m_splatnoValue->SetRange(wxDateTime(1,wxDateTime::Jan,idbmodel->currentRok()), wxDateTime(31,wxDateTime::Dec,idbmodel->currentRok()));
    gridSizer->Add(m_splatnoValue, wxGBPosition(i,2), wxDefaultSpan, wxEXPAND);
    m_splatnoUsage = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_splatnoUsage, wxGBPosition(i,3), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Celkem"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_celkemAction = new wxChoice(this, wxID_ANY);
    m_celkemAction->Append(wxString::FromUTF8("Menší než"), new FilterAction(FA_LESS));
    m_celkemAction->Append(wxString::FromUTF8("Menší než nebo rovno"), new FilterAction(FA_LESSEQUAL));
    m_celkemAction->Append("Rovno", new FilterAction(FA_EQUAL));
    m_celkemAction->Append(wxString::FromUTF8("Větší než nebo rovno"), new FilterAction(FA_GREATEREQUAL));
    m_celkemAction->Append(wxString::FromUTF8("Větší než"), new FilterAction(FA_GREATER));
    gridSizer->Add(m_celkemAction, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_celkemValue = new wxSpinCtrlDouble(this, wxID_ANY);
    m_celkemValue->SetDigits(2);
    m_celkemValue->SetRange(0.0, 1000000.0);
    gridSizer->Add(m_celkemValue, wxGBPosition(i,2), wxDefaultSpan, wxEXPAND);
    m_celkemUsage = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_celkemUsage, wxGBPosition(i,3), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Uhradit"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_uhrazenoAction = new wxChoice(this, wxID_ANY);
    m_uhrazenoAction->Append(wxString::FromUTF8("Menší než"), new FilterAction(FA_LESS));
    m_uhrazenoAction->Append(wxString::FromUTF8("Menší než nebo rovno"), new FilterAction(FA_LESSEQUAL));
    m_uhrazenoAction->Append("Rovno", new FilterAction(FA_EQUAL));
    m_uhrazenoAction->Append(wxString::FromUTF8("Větší než nebo rovno"), new FilterAction(FA_GREATEREQUAL));
    m_uhrazenoAction->Append(wxString::FromUTF8("Větší než"), new FilterAction(FA_GREATER));
    gridSizer->Add(m_uhrazenoAction, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_uhrazenoValue = new wxSpinCtrlDouble(this, wxID_ANY);
    m_uhrazenoValue->SetDigits(2);
    m_uhrazenoValue->SetRange(0.0, 1000000.0);
    gridSizer->Add(m_uhrazenoValue, wxGBPosition(i,2), wxDefaultSpan, wxEXPAND);
    m_uhrazenoUsage = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_uhrazenoUsage, wxGBPosition(i,3), wxDefaultSpan, wxEXPAND);

    m_mainSizer->Add(gridSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void VydanaFilter::fillControls()
{
    if(m_filters.GetCount() == 7)
    {
        if(m_filters[0].type() == FT_TEXT)
        {
            m_dokladAction->SetSelection(m_filters[0].action()==FA_EQUAL?0:1);
            m_dokladValue->ChangeValue(m_filters[0].text());
            m_dokladUsage->SetValue(m_filters[0].active());
        }
        if(m_filters[1].type() == FT_TEXT)
        {
            m_popisAction->SetSelection(m_filters[1].action()==FA_EQUAL?0:1);
            m_popisValue->ChangeValue(m_filters[1].text());
            m_popisUsage->SetValue(m_filters[1].active());
        }
        if(m_filters[2].type() == FT_TEXT)
        {
            m_protistranaAction->SetSelection(m_filters[2].action()==FA_EQUAL?0:1);
            m_protistranaValue->ChangeValue(m_filters[2].text());
            m_protistranaUsage->SetValue(m_filters[2].active());
        }
        if(m_filters[3].type() == FT_DATUM)
        {
            m_vystavenoAction->SetSelection(m_filters[3].action());
            m_vystavenoValue->SetValue(m_filters[3].datum());
            m_vystavenoUsage->SetValue(m_filters[3].active());
        }
        if(m_filters[4].type() == FT_DATUM)
        {
            m_splatnoAction->SetSelection(m_filters[4].action());
            m_splatnoValue->SetValue(m_filters[4].datum());
            m_splatnoUsage->SetValue(m_filters[4].active());
        }
        if(m_filters[5].type() == FT_DOUBLE)
        {
            m_celkemAction->SetSelection(m_filters[5].action());
            m_celkemValue->SetValue(m_filters[5].doublevalue());
            m_celkemUsage->SetValue(m_filters[5].active());
        }
        if(m_filters[6].type() == FT_DOUBLE)
        {
            m_uhrazenoAction->SetSelection(m_filters[6].action());
            m_uhrazenoValue->SetValue(m_filters[6].doublevalue());
            m_uhrazenoUsage->SetValue(m_filters[6].active());
        }
    }
}

void VydanaFilter::OnOk(wxCommandEvent& /*event*/)
{
    m_filters[0].setAction(*static_cast<FilterAction*>(m_dokladAction->GetClientData(static_cast<unsigned int>(m_dokladAction->GetSelection()))));
    m_filters[0].setText(m_dokladValue->GetValue());
    m_filters[0].setActive(m_dokladUsage->GetValue());

    m_filters[1].setAction(*static_cast<FilterAction*>(m_popisAction->GetClientData(static_cast<unsigned int>(m_popisAction->GetSelection()))));
    m_filters[1].setText(m_popisValue->GetValue());
    m_filters[1].setActive(m_popisUsage->GetValue());

    m_filters[2].setAction(*static_cast<FilterAction*>(m_protistranaAction->GetClientData(static_cast<unsigned int>(m_protistranaAction->GetSelection()))));
    m_filters[2].setText(m_protistranaValue->GetValue());
    m_filters[2].setActive(m_protistranaUsage->GetValue());

    m_filters[3].setAction(*static_cast<FilterAction*>(m_vystavenoAction->GetClientData(static_cast<unsigned int>(m_vystavenoAction->GetSelection()))));
    m_filters[3].setDatum(m_vystavenoValue->GetValue());
    m_filters[3].setActive(m_vystavenoUsage->GetValue());

    m_filters[4].setAction(*static_cast<FilterAction*>(m_splatnoAction->GetClientData(static_cast<unsigned int>(m_splatnoAction->GetSelection()))));
    m_filters[4].setDatum(m_splatnoValue->GetValue());
    m_filters[4].setActive(m_splatnoUsage->GetValue());

    m_filters[5].setAction(*static_cast<FilterAction*>(m_celkemAction->GetClientData(static_cast<unsigned int>(m_celkemAction->GetSelection()))));
    m_filters[5].setDoublevalue(m_celkemValue->GetValue());
    m_filters[5].setActive(m_celkemUsage->GetValue());

    m_filters[6].setAction(*static_cast<FilterAction*>(m_uhrazenoAction->GetClientData(static_cast<unsigned int>(m_uhrazenoAction->GetSelection()))));
    m_filters[6].setDoublevalue(m_uhrazenoValue->GetValue());
    m_filters[6].setActive(m_uhrazenoUsage->GetValue());

    EndModal(wxID_OK);
}

void VydanaFilter::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

