/***************************************************************
 * Name:      bankapage.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#ifndef BANKAPAGE_H
#define BANKAPAGE_H

#include <wx/imaglist.h>
#include "gui/widgets/dxlistctrl.h"
#include "dxpage.h"
#include "gui/filters/filter.h"
#include "gui/widgets/dxtoolbar.h"

class BankaPage;

class BankaCtrl: public dxListCtrl
{
public:
    BankaCtrl(BankaPage *page, wxWindow *parent, const wxWindowID id)
        : dxListCtrl(parent, id), m_page(page) {}

    enum BColumn
    {
        BCOL_DATUM = 0,
        BCOL_CISLO,
        BCOL_BANKA,
        BCOL_VYPIS,
        BCOL_TYP,
        BCOL_POPIS,
        BCOL_CASTKA,
        BCOL_MAX, // pocet sloupcu
        BCOL_DEF_SORT = BCOL_DATUM
    };

    void CreateColumns();
private:
    BankaPage *m_page;

    virtual wxString OnGetItemText(long item, long column) const;
    virtual int OnGetItemColumnImage(long item, long column) const;
    virtual wxListItemAttr *OnGetItemAttr(long item) const;
};

class BankaPage : public dxPage
{
    DECLARE_DYNAMIC_CLASS(BankaPage)
    DECLARE_EVENT_TABLE()
public:
    BankaPage() : dxPage(nullptr, NAV_BANKA), m_mainSizer(nullptr), m_list(nullptr), m_selected(-1),
    m_bar(nullptr), m_sortCol(BankaCtrl::BCOL_DATUM), m_sortAsc(true), m_filter(false) {}
    explicit BankaPage(wxWindow *parent);

    void updateData(int id) override;
    wxString getItemText(long item, long column) const;
private:
    wxBoxSizer *m_mainSizer;
    BankaCtrl *m_list;
    long m_selected;
    dxToolBar *m_bar;
    dxDenikViewArray m_denik;
    BankaCtrl::BColumn m_sortCol;
    bool m_sortAsc;
    wxScopedPtr<wxImageList> m_imageList;
    bool m_filter;
    dxFilterArray m_filters;

    void OnPrijem(wxCommandEvent& event);
    void OnVydaj(wxCommandEvent& event);
    void OnEditBanka(wxCommandEvent& event);
    void OnCopyBanka(wxCommandEvent& event);
    void OnDeleteBanka(wxCommandEvent& event);
    void OnPrintDoklad(wxCommandEvent& event);
    void OnPrintBanka(wxCommandEvent& event);
    void OnExportBanka(wxCommandEvent& event);
    void OnSeznam(wxCommandEvent& event);
    void OnItemRightClick(wxListEvent& event);
    void OnItemActivated(wxListEvent& event);
    void OnItemSelected(wxListEvent& event);
    void OnItemDeselected(wxListEvent& event);
    void OnColumnClick(wxListEvent& event);
    void enableTools();
    DbModel::DSort columnToDenikSort(BankaCtrl::BColumn column);
    void setColumnImage(BankaCtrl::BColumn column, int image);
    void OnFilter(wxCommandEvent& event);
    void OnFilterClear(wxCommandEvent& event);
    void getData();
    void refreshItems();
};

#endif  /* BANKAPAGE_H */

