/***************************************************************
 * Name:      denikpage.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include <wx/wfstream.h>
#include <wx/txtstrm.h>
#include "denikpage.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "resources/images/prijem.xpm"
#include "resources/images/vydaj.xpm"
#include "resources/images/edit.xpm"
#include "resources/images/delete.xpm"
#include "resources/images/csv.xpm"
#include "resources/images/print.xpm"
#include "resources/images/asc.xpm"
#include "resources/images/desc.xpm"
#include "resources/images/printdoc.xpm"
#include "resources/images/copy.xpm"
#include "resources/images/uhrada.xpm"
#include "resources/images/filter.xpm"
#include "resources/images/filter_clear.xpm"
#include "gui/dialogs/prijemvydejdialog.h"
#include "gui/prints/listdenikprint.h"
#include "gui/prints/simpledenikprint.h"
#include "gui/prints/fulldenikprint.h"
#include "gui/prints/pokladnidokladprint.h"
#include "gui/prints/bankovnidokladprint.h"
#include "gui/dialogs/neuhrazenedialog.h"
#include "gui/filters/denikfilter.h"

void DenikCtrl::CreateColumns()
{
    InsertColumn(DCOL_DATUM, "Datum");
    InsertColumn(DCOL_CISLO, "Doklad");
    InsertColumn(DCOL_TYP, "Typ");
    InsertColumn(DCOL_POPIS, "Popis");
    InsertColumn(DCOL_CASTKA, wxString::FromUTF8("Částka"), wxLIST_FORMAT_RIGHT);
    setResizeColumn(DCOL_POPIS);
}

wxString DenikCtrl::OnGetItemText(long item, long column) const
{
    return m_page->getItemText(item, column);
}

int DenikCtrl::OnGetItemColumnImage(long /*item*/, long /*column*/) const
{
    return -1;
}

wxListItemAttr *DenikCtrl::OnGetItemAttr(long /*item*/) const
{
    return nullptr;
}

IMPLEMENT_DYNAMIC_CLASS(DenikPage, wxPanel)

BEGIN_EVENT_TABLE(DenikPage, wxPanel)
    EVT_MENU(ID_PAGE_DENIK_PRIJEM, DenikPage::OnPrijem)
    EVT_MENU(ID_PAGE_DENIK_VYDAJ, DenikPage::OnVydaj)
    EVT_MENU(ID_PAGE_DENIK_EDIT, DenikPage::OnEditDenik)
    EVT_MENU(ID_PAGE_DENIK_COPY, DenikPage::OnCopyDenik)
    EVT_MENU(ID_PAGE_DENIK_DELETE, DenikPage::OnDeleteDenik)
    EVT_MENU(ID_PAGE_DENIK_PRINTDOKLAD, DenikPage::OnPrintDoklad)
    EVT_MENU(ID_PAGE_DENIK_PRINT, DenikPage::OnPrintDenik)
    EVT_MENU(ID_PAGE_DENIK_EXPORT, DenikPage::OnExportDenik)
    EVT_LIST_ITEM_RIGHT_CLICK(ID_PAGE_DENIK_LIST, DenikPage::OnItemRightClick)
    EVT_LIST_ITEM_ACTIVATED(ID_PAGE_DENIK_LIST, DenikPage::OnItemActivated)
    EVT_LIST_ITEM_SELECTED(ID_PAGE_DENIK_LIST, DenikPage::OnItemSelected)
    EVT_LIST_ITEM_DESELECTED(ID_PAGE_DENIK_LIST, DenikPage::OnItemDeselected)
    EVT_LIST_COL_CLICK(ID_PAGE_DENIK_LIST, DenikPage::OnColumnClick)
    EVT_MENU(ID_PAGE_DENIK_UHRADA, DenikPage::OnUhrada)
    EVT_MENU(ID_PAGE_DENIK_FILTER, DenikPage::OnFilter)
    EVT_MENU(ID_PAGE_DENIK_FILTERCLEAR, DenikPage::OnFilterClear)
END_EVENT_TABLE()

DenikPage::DenikPage(wxWindow *parent)
    : dxPage(parent, NAV_DENIK), m_selected(-1), m_sortCol(DenikCtrl::DCOL_DEF_SORT), m_sortAsc(true), m_filter(false)
{
    m_filters.Add(Filter(FA_LESS, false, dxsettings.datum()));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_EQUAL, false, 1));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_LESS, false, 0.0));
    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    m_bar = new dxToolBar(this);
    m_bar->AddTool(ID_PAGE_DENIK_PRIJEM, wxEmptyString, wxBitmap(prijem_xpm), dxutils::GreyScaledIcon(prijem_xpm), wxITEM_NORMAL, wxString::FromUTF8("Nový příjem"));
    m_bar->AddTool(ID_PAGE_DENIK_VYDAJ, wxEmptyString, wxBitmap(vydaj_xpm), dxutils::GreyScaledIcon(vydaj_xpm), wxITEM_NORMAL, wxString::FromUTF8("Nový výdaj"));
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_DENIK_UHRADA, wxEmptyString, wxBitmap(uhrada_xpm), dxutils::GreyScaledIcon(uhrada_xpm), wxITEM_NORMAL, wxString::FromUTF8("Uhradit fakturu, závazek nebo pohledávku"));
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_DENIK_EDIT, wxEmptyString, wxBitmap(edit_xpm), dxutils::GreyScaledIcon(edit_xpm), wxITEM_NORMAL, wxString::FromUTF8("Upravit záznam"));
    m_bar->AddTool(ID_PAGE_DENIK_COPY, wxEmptyString, wxBitmap(copy_xpm), dxutils::GreyScaledIcon(copy_xpm), wxITEM_NORMAL, wxString::FromUTF8("Kopírovat záznam"));
    m_bar->AddTool(ID_PAGE_DENIK_DELETE, wxEmptyString, wxBitmap(delete_xpm), dxutils::GreyScaledIcon(delete_xpm), wxITEM_NORMAL, wxString::FromUTF8("Smazat záznam"));
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_DENIK_PRINTDOKLAD, wxEmptyString, wxBitmap(printdoc_xpm), dxutils::GreyScaledIcon(printdoc_xpm), wxITEM_NORMAL, "Vytisknout doklad");
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_DENIK_PRINT, wxEmptyString, wxBitmap(print_xpm), dxutils::GreyScaledIcon(print_xpm), wxITEM_NORMAL, wxString::FromUTF8("Vytisknout deník"));
    m_bar->AddTool(ID_PAGE_DENIK_EXPORT, wxEmptyString, wxBitmap(csv_xpm), dxutils::GreyScaledIcon(csv_xpm), wxITEM_NORMAL, wxString::FromUTF8("Exportovat deník"));
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_DENIK_FILTER, wxEmptyString, wxBitmap(filter_xpm), dxutils::GreyScaledIcon(filter_xpm), wxITEM_CHECK, "Filtrovat");
    m_bar->AddTool(ID_PAGE_DENIK_FILTERCLEAR, wxEmptyString, wxBitmap(filter_clear_xpm), dxutils::GreyScaledIcon(filter_clear_xpm), wxITEM_NORMAL, wxString::FromUTF8("Zrušit filtr"));
    m_bar->Realize();
    m_bar->EnableTool(ID_PAGE_DENIK_FILTERCLEAR, false);
    m_mainSizer->Add(m_bar, 0, wxALL|wxEXPAND, 5);
    wxSize imageSize(16, 16);
    m_imageList.reset(new wxImageList(imageSize.GetWidth(), imageSize.GetHeight()));
    m_imageList->Add(wxImage(asc_xpm).Scale(16, 16));
    m_imageList->Add(wxImage(desc_xpm).Scale(16, 16));
    m_list = new DenikCtrl(this, this, ID_PAGE_DENIK_LIST);
    m_list->SetImageList(m_imageList.get(), wxIMAGE_LIST_SMALL);
    m_list->CreateColumns();
    enableTools();
    m_mainSizer->Add(m_list, 1, wxALL|wxEXPAND, 5);
    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void DenikPage::updateData(int id)
{
    if(!(id==ID_MODEL_ALL || id==ID_MODEL_BANKA || id==ID_MODEL_POKLADNA)) return;
    long oldcnt = m_list->GetItemCount();
    setColumnImage(m_sortCol, m_sortAsc?0:1);
    getData();
    idbmodel->sortDenikView(m_denik, columnToDenikSort(m_sortCol), m_sortAsc);
    long cnt = static_cast<long>(m_denik.GetCount());
    m_list->SetItemCount(static_cast<long>(cnt));
    if(cnt)
    {
        if(m_selected >= cnt)
        {
            m_list->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
            m_selected = 0;
        }
        if(oldcnt <= cnt) m_list->RefreshItems(0, --cnt);
        else m_list->Refresh();
        m_bar->EnableTool(ID_PAGE_DENIK_PRINT, true);
        m_bar->EnableTool(ID_PAGE_DENIK_EXPORT, true);
        m_bar->EnableTool(ID_PAGE_DENIK_FILTER, true);
        m_bar->EnableTool(ID_PAGE_DENIK_FILTERCLEAR, m_filter?true:false);
    }
    else
    {
        m_list->Refresh();
        m_bar->EnableTool(ID_PAGE_DENIK_EDIT, false);
        m_bar->EnableTool(ID_PAGE_DENIK_COPY, false);
        m_bar->EnableTool(ID_PAGE_DENIK_DELETE, false);
        m_bar->EnableTool(ID_PAGE_DENIK_PRINTDOKLAD, false);
        m_bar->EnableTool(ID_PAGE_DENIK_PRINT, false);
        m_bar->EnableTool(ID_PAGE_DENIK_EXPORT, false);
        m_bar->EnableTool(ID_PAGE_DENIK_FILTER, m_filter?true:false);
        m_bar->EnableTool(ID_PAGE_DENIK_FILTERCLEAR, m_filter?true:false);
    }
    if(idbmodel->someNeuhrazenaFakturaOrZavpoh())
    {
        m_bar->EnableTool(ID_PAGE_DENIK_UHRADA, true);
    }
    else
    {
        m_bar->EnableTool(ID_PAGE_DENIK_UHRADA, false);
    }
}

wxString DenikPage::getItemText(long item, long column) const
{
    if(item < 0 || item >= static_cast<long>(m_denik.GetCount()))
        return "";
    switch(column)
    {
    case DenikCtrl::DCOL_DATUM:
        return m_denik[static_cast<size_t>(item)].datum().FormatDate();
    case DenikCtrl::DCOL_CISLO:
        return m_denik[static_cast<size_t>(item)].cislo();
    case DenikCtrl::DCOL_TYP:
        return m_denik[static_cast<size_t>(item)].prijem() ? wxString::FromUTF8("Příjem") : wxString::FromUTF8("Výdaj");
    case DenikCtrl::DCOL_POPIS:
        return m_denik[static_cast<size_t>(item)].popis();
    case DenikCtrl::DCOL_CASTKA:
        return wxString::Format("%.2f", m_denik[static_cast<size_t>(item)].castka());
    default:
        return "";
    }
}

void DenikPage::OnPrijem(wxCommandEvent& /*event*/)
{
    PrijemVydejDialog dialog(this, -1, false, true, false, false);
    dialog.ShowModal();
}

void DenikPage::OnVydaj(wxCommandEvent& /*event*/)
{
    PrijemVydejDialog dialog(this, -1, false, false, false, false);
    dialog.ShowModal();
}

void DenikPage::OnEditDenik(wxCommandEvent& /*event*/)
{
    DenikView dk = m_denik[static_cast<size_t>(m_selected)];
    PrijemVydejDialog dialog(this, dk.id(), true, dk.prijem(), idbmodel->hasPokladna(dk.pokladnabanka()), idbmodel->hasBanka(dk.pokladnabanka()));
    dialog.ShowModal();
}

void DenikPage::OnCopyDenik(wxCommandEvent& /*event*/)
{
    Denik dk = idbmodel->getDenik(idbmodel->copyDenik(m_denik[static_cast<size_t>(m_selected)].id()));
    if(dk.isValid())
    {
        PrijemVydejDialog dialog(this, dk.id(), 2, idbmodel->isPredkontacePrijem(dk.predkontaceid()),
                                 idbmodel->hasPokladna(dk.pokladnaid()), idbmodel->hasBanka(dk.bankaid()));
        dialog.ShowModal();
    }
}

void DenikPage::OnDeleteDenik(wxCommandEvent& /*event*/)
{
    DenikView dk = m_denik[static_cast<size_t>(m_selected)];
    wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Opravdu smazat záznam\n%s\n%s?"), dk.popis(), dk.cislo()),
                           "Dotaz", wxOK|wxCANCEL|wxICON_QUESTION);
    if(dialog.ShowModal() == wxID_OK)
    {
        idbmodel->deleteDenik(dk.id());
    }
}

void DenikPage::OnPrintDoklad(wxCommandEvent &/*event*/)
{
    if(idbmodel->hasPokladna(m_denik[static_cast<size_t>(m_selected)].pokladnabanka()))
    {
        PokladniDokladPrint print(this, idbmodel->getDenik(m_denik[static_cast<size_t>(m_selected)].id()));
        //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
        print.Print();
#else
        print.Preview();
#endif
    }
    else
    {
        BankovniDokladPrint print(this, idbmodel->getDenik(m_denik[static_cast<size_t>(m_selected)].id()));
        //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
        print.Print();
#else
        print.Preview();
#endif
    }
}

void DenikPage::OnPrintDenik(wxCommandEvent& /*event*/)
{
    const wxString choices[] = { "Seznam", wxString::FromUTF8("Zjednodušený"), wxString::FromUTF8("Plný") } ;
    wxSingleChoiceDialog choice(this, wxString::FromUTF8("Vytisknout pěněžní deník jako:"), "Vyberte", WXSIZEOF(choices), choices);
    if(choice.ShowModal() == wxID_OK)
    {
        switch(choice.GetSelection())
        {
        case 0:
        {
            ListDenikPrint dialog(this);
            //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
            dialog.Print();
#else
            dialog.Preview();
#endif
            return;
        }
        case 1:
        {
            SimpleDenikPrint dialog(this);
            //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
            dialog.Print();
#else
            dialog.Preview();
#endif
            return;
        }
        case 2:
        {
            FullDenikPrint dialog(this);
            //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
            dialog.Print();
#else
            dialog.Preview();
#endif
            return;
        }
        default: break;
        }
    }
}

void DenikPage::OnExportDenik(wxCommandEvent& /*event*/)
{
    wxFileDialog dlg(this, wxString::FromUTF8("Exportovat deník do"), wxEmptyString, wxEmptyString, "CSV soubory (*.csv)|*.csv",
                     wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
    if(dlg.ShowModal() != wxID_OK) return;
    wxString path = dlg.GetPath();
    dxutils::correctEmptyFileExt("csv", path);
    wxString delim = ",";
    wxFileOutputStream output(path);
    wxTextOutputStream text(output);
    text << "Datum" << delim << "Doklad" << delim << "Typ" << delim << "Popis" << delim << wxString::FromUTF8("Částka") << delim << endl;
    dxDenikViewArray denik = idbmodel->denikview();
    for(size_t i=0; i<denik.GetCount(); i++)
    {
        text << denik[i].datum().FormatDate();
        text << delim;
        text << dxutils::enquotes(denik[i].cislo(), delim);
        text << delim;
        text << (denik[i].prijem() ? wxString::FromUTF8("Příjem") : wxString::FromUTF8("Výdaj"));
        text << delim;
        text << dxutils::enquotes(denik[i].popis(), delim);
        text << delim;
        text << dxutils::enquotes(wxString::Format(wxString::FromUTF8("%.2f Kč"), denik[i].castka()), delim);
        text << delim;
        text << endl;
    }
}

void DenikPage::OnItemRightClick(wxListEvent &event)
{
    m_selected = event.GetIndex();
    enableTools();
    wxMenu popup;
    popup.Append(ID_PAGE_DENIK_PRIJEM, wxString::FromUTF8("Nový příjem"));
    popup.Append(ID_PAGE_DENIK_VYDAJ, wxString::FromUTF8("Nový výdaj"));
    if(m_selected != -1)
    {
        popup.AppendSeparator();
        popup.Append(ID_PAGE_DENIK_EDIT, wxString::FromUTF8("Upravit záznam"));
        popup.Append(ID_PAGE_DENIK_COPY, wxString::FromUTF8("Kopírovat záznam"));
        popup.Append(ID_PAGE_DENIK_DELETE, wxString::FromUTF8("Smazat záznam"));
        popup.AppendSeparator();
        popup.Append(ID_PAGE_DENIK_PRINTDOKLAD, "Vytisknout doklad");
    }
    PopupMenu(&popup, event.GetPoint()+m_list->GetPosition());
}

void DenikPage::OnItemActivated(wxListEvent &event)
{
    m_selected = event.GetIndex();
    enableTools();
    DenikView dk = m_denik[static_cast<size_t>(m_selected)];
    PrijemVydejDialog dialog(this, dk.id(), true, dk.prijem(), idbmodel->hasPokladna(dk.pokladnabanka()), idbmodel->hasBanka(dk.pokladnabanka()));
    dialog.ShowModal();
}

void DenikPage::OnItemSelected(wxListEvent &event)
{
    m_selected = event.GetIndex();
    enableTools();
}

void DenikPage::OnItemDeselected(wxListEvent& /*event*/)
{
    m_selected = -1;
    enableTools();
}

void DenikPage::OnColumnClick(wxListEvent &event)
{
    if(0 > event.GetColumn() || event.GetColumn() >= DenikCtrl::DCOL_MAX) return;
    setColumnImage(m_sortCol, -1);
    m_sortCol = static_cast<DenikCtrl::DColumn>(event.GetColumn());
    m_sortAsc = !m_sortAsc;
    setColumnImage(m_sortCol, m_sortAsc?0:1);
    if(m_denik.GetCount()) idbmodel->sortDenikView(m_denik, columnToDenikSort(m_sortCol), m_sortAsc);
    refreshItems();
}

void DenikPage::enableTools()
{
    if(m_selected == -1)
    {
        m_bar->EnableTool(ID_PAGE_DENIK_EDIT, false);
        m_bar->EnableTool(ID_PAGE_DENIK_COPY, false);
        m_bar->EnableTool(ID_PAGE_DENIK_DELETE, false);
        m_bar->EnableTool(ID_PAGE_DENIK_PRINTDOKLAD, false);
    }
    else
    {
        m_bar->EnableTool(ID_PAGE_DENIK_EDIT, true);
        m_bar->EnableTool(ID_PAGE_DENIK_COPY, true);
        m_bar->EnableTool(ID_PAGE_DENIK_DELETE, true);
        m_bar->EnableTool(ID_PAGE_DENIK_PRINTDOKLAD, true);
    }
}

DbModel::DSort DenikPage::columnToDenikSort(DenikCtrl::DColumn column)
{
    switch(column)
    {
    case DenikCtrl::DCOL_CASTKA: return DbModel::DS_CASTKA;
    case DenikCtrl::DCOL_CISLO: return DbModel::DS_CISLO;
    case DenikCtrl::DCOL_DATUM: return DbModel::DS_DATUM;
    case DenikCtrl::DCOL_POPIS: return DbModel::DS_POPIS;
    case DenikCtrl::DCOL_TYP: return DbModel::DS_PRIJEM;
    default: return DbModel::DS_DATUM;
    }
}

void DenikPage::setColumnImage(DenikCtrl::DColumn column, int image)
{
    wxListItem item;
    item.SetMask(wxLIST_MASK_IMAGE);
    item.SetImage(image);
    m_list->SetColumn(column, item);
}

void DenikPage::OnUhrada(wxCommandEvent &/*event*/)
{
    NeuhrazeneDialog dialog(this);
    dialog.ShowModal();
}

void DenikPage::OnFilter(wxCommandEvent &/*event*/)
{
    DenikFilter filter(this,m_filters);
    if(filter.ShowModal() == wxID_OK)
    {
        m_filters = filter.filters();
        m_filter = false;
        for(size_t i=0; i<m_filters.GetCount(); i++)
        {
            if(m_filters[i].active())
            {
                m_filter = true;
                break;
            }
        }
        m_bar->ToggleTool(ID_PAGE_DENIK_FILTER, m_filter);
        m_bar->EnableTool(ID_PAGE_DENIK_FILTERCLEAR, m_filter);
        if(m_list->GetColumnCount() == static_cast<int>(m_filters.GetCount()))
        {
            for(size_t i=0; i<m_filters.GetCount(); i++)
            {
                m_list->SetColumnHeaderFiltered(static_cast<int>(i), m_filters[i].active());
            }
            m_list->Update();
        }
    }
    else
    {
        m_filter = false;
        m_bar->ToggleTool(ID_PAGE_DENIK_FILTER, false);
        m_bar->EnableTool(ID_PAGE_DENIK_FILTERCLEAR, false);
        for(int i=0; i<m_list->GetColumnCount(); i++)
        {
            m_list->SetColumnHeaderFiltered(i, false);
        }
        m_list->Update();
    }
    updateData(ID_MODEL_ALL);
}

void DenikPage::OnFilterClear(wxCommandEvent &/*event*/)
{
    m_bar->ToggleTool(ID_PAGE_DENIK_FILTER, false);
    m_filter = false;
    m_bar->EnableTool(ID_PAGE_DENIK_FILTERCLEAR, false);
    for(int i=0; i<m_list->GetColumnCount(); i++)
    {
        m_list->SetColumnHeaderFiltered(i, false);
    }
    m_list->Update();
    updateData(ID_MODEL_ALL);
}

void DenikPage::getData()
{
    if(m_filter)
    {
        m_denik.Clear();
        dxDenikViewArray denik = idbmodel->denikview();
        for(size_t i=0; i<denik.GetCount(); i++)
        {
            if(m_filters[0].matchFilter(denik[i].datum()) && m_filters[1].matchFilter(denik[i].cislo()) && m_filters[2].matchFilter(denik[i].prijem())
                    && m_filters[3].matchFilter(denik[i].popis()) && m_filters[4].matchFilter(denik[i].castka()))
            {
                m_denik.Add(denik[i]);
            }
        }
    }
    else
    {
        m_denik = idbmodel->denikview();
    }
}

void DenikPage::refreshItems()
{
    long oldcnt = m_list->GetItemCount();
    long cnt = static_cast<long>(m_denik.GetCount());
    if(cnt)
    {
        if(m_selected >= cnt)
        {
            m_list->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
            m_selected = 0;
        }
        if(oldcnt <= cnt) m_list->RefreshItems(0, --cnt);
        else m_list->Refresh();
    }
    else
    {
        m_list->Refresh();
    }
}
