/***************************************************************
 * Name:      dxbook.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include "dxbook.h"
#include "dxpage.h"

BEGIN_EVENT_TABLE(dxBook, wxBookCtrlBase)
    EVT_SIZE(dxBook::OnSize)
END_EVENT_TABLE()

bool dxBook::Create(wxWindow *parent, wxWindowID id, const wxPoint &pos, const wxSize &size, long style, const wxString &name)
{
    if((style & wxBK_ALIGN_MASK) == wxBK_DEFAULT )
    {
        style |= wxBK_TOP;
    }
    if(!wxControl::Create(parent, id, pos, size, style, wxDefaultValidator, name))
        return false;
    return true;
}

bool dxBook::SetPageText(size_t /*n*/, const wxString& /*strText*/)
{
    return true;
}

wxString dxBook::GetPageText(size_t /*n*/) const
{
    return wxEmptyString;
}

int dxBook::GetPageImage(size_t /*n*/) const
{
    return wxNOT_FOUND;
}

bool dxBook::SetPageImage(size_t /*n*/, int /*imageId*/)
{
    return false;
}

bool dxBook::InsertPage(size_t n, wxWindow *page, const wxString &text, bool bSelect, int imageId)
{
    if(!wxBookCtrlBase::InsertPage(n, page, text, bSelect, imageId))
        return false;
    if(n <= static_cast<size_t>(m_selected))
        m_selected++;
    int selNew = wxNOT_FOUND;
    if(bSelect)
        selNew = static_cast<int>(n);
    else if(m_selected == wxNOT_FOUND)
        selNew = 0;
    if(selNew != m_selected)
        page->Hide();
    if(selNew != wxNOT_FOUND)
        SetSelection(static_cast<size_t>(selNew));
    return true;
}

int dxBook::GetSelection() const
{
    return m_selected;
}

wxSize dxBook::CalcSizeFromPage(const wxSize &sizePage) const
{
    return sizePage;
}

wxBookCtrlEvent *dxBook::CreatePageChangingEvent() const
{
    return new wxBookCtrlEvent(wxEVT_BOOKCTRL_PAGE_CHANGING, GetId());
}

void dxBook::MakeChangedEvent(wxBookCtrlEvent &event)
{
    event.SetEventType(wxEVT_BOOKCTRL_PAGE_CHANGED);
}

void dxBook::UpdateSelectedPage(size_t newsel)
{
    m_selected = static_cast<int>(newsel);
}

void dxBook::OnSize(wxSizeEvent &event)
{
    wxWindow* const page = GetCurrentPage();
    if(page)
        page->SetSize(event.GetSize());
}

bool dxBook::AppendPage(wxWindow *page)
{
    if(!wxBookCtrlBase::AddPage(page, wxEmptyString, true))
        return false;
    return true;
}

bool dxBook::HasPage(NAVIGATION nav) const
{
    const size_t nCount = m_pages.size();
    for(size_t nPage = 0; nPage < nCount; nPage++)
    {
        const dxPage * const pPage = static_cast<dxPage*>(m_pages[nPage]);
        if(pPage)
        {
            if(pPage->navigation() == nav)
                return true;
        }
    }
    return false;
}

void dxBook::SelectPage(NAVIGATION nav)
{
    const size_t nCount = m_pages.size();
    for(size_t nPage = 0; nPage < nCount; nPage++)
    {
        dxPage *pPage = static_cast<dxPage*>(m_pages[nPage]);
        if(pPage)
        {
            if(pPage->navigation() == nav)
            {
                SetSelection(nPage);
                return;
            }
        }
    }
}

void dxBook::UpdatePages(int id)
{
    const size_t nCount = m_pages.size();
    for(size_t nPage = 0; nPage < nCount; nPage++)
    {
        dxPage *pPage = static_cast<dxPage*>(m_pages[nPage]);
        if(pPage)
        {
            pPage->updateData(id);
        }
    }
}

wxWindow *dxBook::DoRemovePage(size_t page)
{
    const size_t page_count = GetPageCount();
    wxWindow *win = wxBookCtrlBase::DoRemovePage(page);
    if(win)
    {
        if(m_selected >= static_cast<int>(page))
        {
            int sel = m_selected - 1;
            if(page_count == 1)
                sel = wxNOT_FOUND;
            else if((page_count == 2) || (sel == -1))
                sel = 0;
            m_selected = (m_selected == static_cast<int>(page)) ? wxNOT_FOUND : m_selected - 1;
            if((sel != wxNOT_FOUND) && (sel != m_selected))
                SetSelection(static_cast<size_t>(sel));
        }
    }
    return win;
}

void dxBook::Init()
{
    m_selected = wxNOT_FOUND;
}


