/***************************************************************
 * Name:      jizdapage.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2018
 * License:   GPL3
 **************************************************************/

#include <wx/wfstream.h>
#include <wx/txtstrm.h>
#include "jizdapage.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "resources/images/add.xpm"
#include "resources/images/edit.xpm"
#include "resources/images/delete.xpm"
#include "resources/images/csv.xpm"
#include "resources/images/print.xpm"
#include "resources/images/asc.xpm"
#include "resources/images/desc.xpm"
#include "resources/images/printdoc.xpm"
#include "resources/images/copy.xpm"
#include "resources/images/seznam.xpm"
#include "resources/images/filter.xpm"
#include "resources/images/filter_clear.xpm"
#include "gui/dialogs/vozidlodialog.h"
#include "gui/dialogs/vozidladialog.h"
#include "gui/dialogs/jizdadialog.h"
#include "gui/dialogs/nahradadialog.h"
#include "gui/prints/knihajizdprint.h"
#include "gui/prints/jizdaprint.h"

void JizdaCtrl::CreateColumns()
{
    InsertColumn(JCOL_DATUM, "Datum");
    InsertColumn(JCOL_CESTA, "Cesta");
    InsertColumn(JCOL_UCEL, wxString::FromUTF8("Účel"));
    InsertColumn(JCOL_KM, "km", wxLIST_FORMAT_RIGHT);
    InsertColumn(JCOL_VOZIDLO, "Vozidlo");
    InsertColumn(JCOL_RIDIC, wxString::FromUTF8("Řidič"));
    setResizeColumn(JCOL_CESTA);
    SetColumnWidth(0, 100);
    SetColumnWidth(2, 150);
    SetColumnWidth(3, 50);
    SetColumnWidth(4, 200);
    SetColumnWidth(5, 120);
}

wxString JizdaCtrl::OnGetItemText(long item, long column) const
{
    return m_page->getItemText(item, column);
}

int JizdaCtrl::OnGetItemColumnImage(long /*item*/, long /*column*/) const
{
    return -1;
}

wxListItemAttr *JizdaCtrl::OnGetItemAttr(long /*item*/) const
{
    return nullptr;
}

IMPLEMENT_DYNAMIC_CLASS(JizdaPage, wxPanel)

BEGIN_EVENT_TABLE(JizdaPage, wxPanel)
    EVT_MENU(ID_PAGE_JIZDA_ADD, JizdaPage::OnAddJizda)
    EVT_MENU(ID_PAGE_JIZDA_EDIT, JizdaPage::OnEditJizda)
    EVT_MENU(ID_PAGE_JIZDA_COPY, JizdaPage::OnCopyJizda)
    EVT_MENU(ID_PAGE_JIZDA_DELETE, JizdaPage::OnDeleteJizda)
    EVT_MENU(ID_PAGE_JIZDA_PRINT, JizdaPage::OnPrintKnihaJizd)
    EVT_MENU(ID_PAGE_JIZDA_EXPORT, JizdaPage::OnExportKnihaJizd)
    EVT_LIST_ITEM_RIGHT_CLICK(ID_PAGE_JIZDA_LIST, JizdaPage::OnItemRightClick)
    EVT_LIST_ITEM_ACTIVATED(ID_PAGE_JIZDA_LIST, JizdaPage::OnItemActivated)
    EVT_LIST_ITEM_SELECTED(ID_PAGE_JIZDA_LIST, JizdaPage::OnItemSelected)
    EVT_LIST_ITEM_DESELECTED(ID_PAGE_JIZDA_LIST, JizdaPage::OnItemDeselected)
    EVT_LIST_COL_CLICK(ID_PAGE_JIZDA_LIST, JizdaPage::OnColumnClick)
    EVT_MENU(ID_PAGE_JIZDA_JAPRINT, JizdaPage::OnPrintJizda)
    EVT_MENU(ID_PAGE_JIZDA_SEZNAM, JizdaPage::OnVozidla)
    EVT_MENU(ID_PAGE_JIZDA_NAHRADA, JizdaPage::OnNahrada)
    EVT_MENU(ID_PAGE_JIZDA_FILTER, JizdaPage::OnFilter)
    EVT_MENU(ID_PAGE_JIZDA_FILTERCLEAR, JizdaPage::OnFilterClear)
END_EVENT_TABLE()

JizdaPage::JizdaPage(wxWindow *parent)
    : dxPage(parent, NAV_JIZDA), m_selected(-1), m_sortCol(JizdaCtrl::JCOL_DEF_SORT), m_sortAsc(true), m_filter(false)
{
    m_filters.Add(Filter(FA_LESS, false, dxsettings.datum()));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_LESS, false, 0));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    m_bar = new dxToolBar(this);
    m_bar->AddTool(ID_PAGE_JIZDA_ADD, wxEmptyString, wxBitmap(add_xpm), dxutils::GreyScaledIcon(add_xpm), wxITEM_NORMAL, wxString::FromUTF8("Nová jízda"));
    m_bar->AddTool(ID_PAGE_JIZDA_EDIT, wxEmptyString, wxBitmap(edit_xpm), dxutils::GreyScaledIcon(edit_xpm), wxITEM_NORMAL, wxString::FromUTF8("Upravit jízdu"));
    m_bar->AddTool(ID_PAGE_JIZDA_COPY, wxEmptyString, wxBitmap(copy_xpm), dxutils::GreyScaledIcon(copy_xpm), wxITEM_NORMAL, wxString::FromUTF8("Kopírovat jízdu"));
    m_bar->AddTool(ID_PAGE_JIZDA_DELETE, wxEmptyString, wxBitmap(delete_xpm), dxutils::GreyScaledIcon(delete_xpm), wxITEM_NORMAL, wxString::FromUTF8("Smazat jízdu"));
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_JIZDA_JAPRINT, wxEmptyString, wxBitmap(printdoc_xpm), dxutils::GreyScaledIcon(printdoc_xpm), wxITEM_NORMAL, wxString::FromUTF8("Vytisknout jízdu"));
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_JIZDA_PRINT, wxEmptyString, wxBitmap(print_xpm), dxutils::GreyScaledIcon(print_xpm), wxITEM_NORMAL, wxString::FromUTF8("Vytisknout knihu jízd"));
    m_bar->AddTool(ID_PAGE_JIZDA_EXPORT, wxEmptyString, wxBitmap(csv_xpm), dxutils::GreyScaledIcon(csv_xpm), wxITEM_NORMAL, wxString::FromUTF8("Exportovat knihu jízd"));
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_JIZDA_SEZNAM, wxEmptyString, wxBitmap(seznam_xpm), dxutils::GreyScaledIcon(seznam_xpm), wxITEM_NORMAL, "Seznam vozidel");
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_JIZDA_FILTER, wxEmptyString, wxBitmap(filter_xpm), dxutils::GreyScaledIcon(filter_xpm), wxITEM_CHECK, "Filtrovat");
    m_bar->AddTool(ID_PAGE_JIZDA_FILTERCLEAR, wxEmptyString, wxBitmap(filter_clear_xpm), dxutils::GreyScaledIcon(filter_clear_xpm), wxITEM_NORMAL, wxString::FromUTF8("Zrušit filtr"));
    m_bar->Realize();
    m_bar->EnableTool(ID_PAGE_JIZDA_FILTERCLEAR, false);
    m_mainSizer->Add(m_bar, 0, wxALL|wxEXPAND, 5);
    wxSize imageSize(16, 16);
    m_imageList.reset(new wxImageList(imageSize.GetWidth(), imageSize.GetHeight()));
    m_imageList->Add(wxImage(asc_xpm).Scale(16, 16));
    m_imageList->Add(wxImage(desc_xpm).Scale(16, 16));
    m_list = new JizdaCtrl(this, this, ID_PAGE_JIZDA_LIST);
    m_list->SetImageList(m_imageList.get(), wxIMAGE_LIST_SMALL);
    m_list->CreateColumns();
    enableTools();
    m_mainSizer->Add(m_list, 1, wxALL|wxEXPAND, 5);
    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void JizdaPage::updateData(int id)
{
    if(!(id==ID_MODEL_ALL || id==ID_MODEL_JIZDA)) return;
    long oldcnt = m_list->GetItemCount();
    setColumnImage(m_sortCol, m_sortAsc?0:1);
    getData();
    idbmodel->sortJizdaView(m_jizdy, columnToJizdaSort(m_sortCol), m_sortAsc);
    long cnt = static_cast<long>(m_jizdy.GetCount());
    m_list->SetItemCount(cnt);
    if(cnt)
    {
        if(m_selected >= cnt)
        {
            m_list->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
            m_selected = 0;
        }
        if(oldcnt <= cnt) m_list->RefreshItems(0, --cnt);
        else m_list->Refresh();
        m_bar->EnableTool(ID_PAGE_JIZDA_PRINT, true);
        m_bar->EnableTool(ID_PAGE_JIZDA_EXPORT, true);
        m_bar->EnableTool(ID_PAGE_JIZDA_FILTER, true);
        m_bar->EnableTool(ID_PAGE_JIZDA_FILTERCLEAR, m_filter?true:false);
    }
    else
    {
        m_list->Refresh();
        m_bar->EnableTool(ID_PAGE_JIZDA_EDIT, false);
        m_bar->EnableTool(ID_PAGE_JIZDA_COPY, false);
        m_bar->EnableTool(ID_PAGE_JIZDA_DELETE, false);
        m_bar->EnableTool(ID_PAGE_JIZDA_JAPRINT, false);
        m_bar->EnableTool(ID_PAGE_JIZDA_PRINT, false);
        m_bar->EnableTool(ID_PAGE_JIZDA_EXPORT, false);
        m_bar->EnableTool(ID_PAGE_JIZDA_FILTER, m_filter?true:false);
        m_bar->EnableTool(ID_PAGE_JIZDA_FILTERCLEAR, m_filter?true:false);
    }
}

wxString JizdaPage::getItemText(long item, long column) const
{
    if(item < 0 || item >= static_cast<long>(m_jizdy.GetCount()))
        return "";
    switch(column)
    {
    case JizdaCtrl::JCOL_DATUM:
    {
        return m_jizdy[static_cast<size_t>(item)].datum();
    }
    case JizdaCtrl::JCOL_CESTA:
    {
        return m_jizdy[static_cast<size_t>(item)].cesta();
    }
    case JizdaCtrl::JCOL_KM:
    {
        return wxString::Format("%d", m_jizdy[static_cast<size_t>(item)].km());
    }
    case JizdaCtrl::JCOL_UCEL:
    {
        return m_jizdy[static_cast<size_t>(item)].ucel();
    }
    case JizdaCtrl::JCOL_VOZIDLO:
    {
        return m_jizdy[static_cast<size_t>(item)].vozidlo();
    }
    case JizdaCtrl::JCOL_RIDIC:
    {
        return m_jizdy[static_cast<size_t>(item)].ridic();
    }
    default:
    {
        return "";
    }
    }
}

void JizdaPage::OnAddJizda(wxCommandEvent &/*event*/)
{
    if(!idbmodel->getVozidlaCount())
    {
        VozidloDialog dialog(this, -1, false);
        if(dialog.ShowModal() != wxID_OK)
        {
            return;
        }
    }
    JizdaDialog dialog(this);
    dialog.ShowModal();
}

void JizdaPage::OnEditJizda(wxCommandEvent &/*event*/)
{
    JizdaDialog dialog(this, m_jizdy[static_cast<size_t>(m_selected)].id());
    dialog.ShowModal();
}

void JizdaPage::OnCopyJizda(wxCommandEvent &/*event*/)
{
    JizdaDialog dialog(this, idbmodel->copyJizda(m_jizdy[static_cast<size_t>(m_selected)].id()), 2);
    dialog.ShowModal();
}

void JizdaPage::OnDeleteJizda(wxCommandEvent &/*event*/)
{
    wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Opravdu smazat jízdu\n%s\n%s?"), m_jizdy[static_cast<size_t>(m_selected)].datum(), m_jizdy[static_cast<size_t>(m_selected)].cesta()),
                           "Dotaz", wxOK|wxCANCEL|wxICON_QUESTION);
    if(dialog.ShowModal() == wxID_OK)
    {
        idbmodel->deleteJizda(m_jizdy[static_cast<size_t>(m_selected)].id());
    }
}

void JizdaPage::OnPrintKnihaJizd(wxCommandEvent &/*event*/)
{
    KnihaJizdPrint print(this);
    //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
    print.Print();
#else
    print.Preview();
#endif
}

void JizdaPage::OnExportKnihaJizd(wxCommandEvent &/*event*/)
{
    wxFileDialog dlg(this, wxString::FromUTF8("Exportovat knihu jízd do"), wxEmptyString, wxEmptyString, "CSV soubory (*.csv)|*.csv",
                     wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
    if(dlg.ShowModal() != wxID_OK) return;
    wxString path = dlg.GetPath();
    dxutils::correctEmptyFileExt("csv", path);
    wxString delim = ",";
    wxFileOutputStream output(path);
    wxTextOutputStream text(output);
    text << "Datum" << delim << "Cesta" << delim << wxString::FromUTF8("Účel") << delim << "km" << delim << "Vozidlo" << delim << wxString::FromUTF8("Řidič") << endl;
    dxJizdaViewArray jizdy = idbmodel->jizdyview();
    for(size_t i=0; i<jizdy.GetCount(); ++i)
    {
        text << dxutils::enquotes(jizdy[i].datum(), delim);
        text << delim;
        text << dxutils::enquotes(jizdy[i].cesta(), delim);
        text << delim;
        text << dxutils::enquotes(jizdy[i].ucel(), delim);
        text << delim;
        text << dxutils::enquotes(wxString::Format("%d", jizdy[i].km()), delim);
        text << delim;
        text << dxutils::enquotes(jizdy[i].vozidlo(), delim);
        text << delim;
        text << dxutils::enquotes(jizdy[i].ridic(), delim);
        text << delim;
        text << endl;
    }
}

void JizdaPage::OnItemRightClick(wxListEvent &event)
{
    m_selected = event.GetIndex();
    enableTools();
    wxMenu popup;
    popup.Append(ID_PAGE_JIZDA_ADD, wxString::FromUTF8("Nová jízda"));
    if(m_selected != -1)
    {
        popup.AppendSeparator();
        popup.Append(ID_PAGE_JIZDA_EDIT, wxString::FromUTF8("Upravit jízdu"));
        popup.Append(ID_PAGE_JIZDA_COPY, wxString::FromUTF8("Kopírovat jízdu"));
        popup.Append(ID_PAGE_JIZDA_DELETE, wxString::FromUTF8("Smazat jízdu"));
        popup.AppendSeparator();
        popup.Append(ID_PAGE_JIZDA_JAPRINT, wxString::FromUTF8("Vytisknout jízdu"));
        if(m_jizdy[static_cast<size_t>(m_selected)].nahrada())
        {
            popup.AppendSeparator();
            popup.Append(ID_PAGE_JIZDA_NAHRADA, wxString::FromUTF8("Spočítat náhradu za použití vozidla"));
        }
    }
    PopupMenu(&popup, event.GetPoint()+m_list->GetPosition());
}

void JizdaPage::OnItemActivated(wxListEvent &event)
{
    m_selected = event.GetIndex();
    enableTools();
    JizdaDialog dialog(this, m_jizdy[static_cast<size_t>(m_selected)].id());
    dialog.ShowModal();
}

void JizdaPage::OnItemSelected(wxListEvent &event)
{
    m_selected = event.GetIndex();
    enableTools();
}

void JizdaPage::OnItemDeselected(wxListEvent &/*event*/)
{
    m_selected = -1;
    enableTools();
}

void JizdaPage::OnColumnClick(wxListEvent &event)
{
    if(0 > event.GetColumn() || event.GetColumn() >= JizdaCtrl::JCOL_MAX) return;
    setColumnImage(m_sortCol, -1);
    m_sortCol = static_cast<JizdaCtrl::JColumn>(event.GetColumn());
    m_sortAsc = !m_sortAsc;
    setColumnImage(m_sortCol, m_sortAsc?0:1);
    if(m_jizdy.GetCount()) idbmodel->sortJizdaView(m_jizdy, columnToJizdaSort(m_sortCol), m_sortAsc);
    refreshItems();
}

void JizdaPage::enableTools()
{
    if(m_selected == -1)
    {
        m_bar->EnableTool(ID_PAGE_JIZDA_EDIT, false);
        m_bar->EnableTool(ID_PAGE_JIZDA_COPY, false);
        m_bar->EnableTool(ID_PAGE_JIZDA_DELETE, false);
        m_bar->EnableTool(ID_PAGE_JIZDA_JAPRINT, false);
    }
    else
    {
        m_bar->EnableTool(ID_PAGE_JIZDA_EDIT, true);
        m_bar->EnableTool(ID_PAGE_JIZDA_COPY, true);
        m_bar->EnableTool(ID_PAGE_JIZDA_DELETE, true);
        m_bar->EnableTool(ID_PAGE_JIZDA_JAPRINT, true);
    }
}

DbModel::JSort JizdaPage::columnToJizdaSort(JizdaCtrl::JColumn column)
{
    switch(column)
    {
    case JizdaCtrl::JCOL_CESTA: return DbModel::JA_CESTA;
    case JizdaCtrl::JCOL_DATUM: return DbModel::JA_DATUM;
    case JizdaCtrl::JCOL_KM: return DbModel::JA_KM;
    case JizdaCtrl::JCOL_UCEL: return DbModel::JA_UCEL;
    case JizdaCtrl::JCOL_VOZIDLO: return DbModel::JA_VOZIDLO;
    case JizdaCtrl::JCOL_RIDIC: return DbModel::JA_RIDIC;
    default: return DbModel::JA_DATUM;
    }
}

void JizdaPage::setColumnImage(JizdaCtrl::JColumn column, int image)
{
    wxListItem item;
    item.SetMask(wxLIST_MASK_IMAGE);
    item.SetImage(image);
    m_list->SetColumn(column, item);
}

void JizdaPage::OnPrintJizda(wxCommandEvent &/*event*/)
{
    JizdaPrint print(this, idbmodel->getJizda(m_jizdy[static_cast<size_t>(m_selected)].id()));
    //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
    print.Print();
#else
    print.Preview();
#endif
}

void JizdaPage::OnVozidla(wxCommandEvent &/*event*/)
{
    VozidlaDialog dialog(this);
    dialog.ShowModal();
}

void JizdaPage::OnNahrada(wxCommandEvent &/*event*/)
{
    if(m_selected != -1 && m_jizdy[static_cast<size_t>(m_selected)].isValid())
    {
        NahradaDialog dialog(this, idbmodel->getJizda(m_jizdy[static_cast<size_t>(m_selected)].id()));
        dialog.ShowModal();
    }
}

void JizdaPage::OnFilter(wxCommandEvent &/*event*/)
{
    JizdaFilter filter(this,m_filters);
    if(filter.ShowModal() == wxID_OK)
    {
        m_filters = filter.filters();
        m_filter = false;
        for(size_t i=0; i<m_filters.GetCount(); i++)
        {
            if(m_filters[i].active())
            {
                m_filter = true;
                break;
            }
        }
        m_bar->ToggleTool(ID_PAGE_JIZDA_FILTER, m_filter);
        m_bar->EnableTool(ID_PAGE_JIZDA_FILTERCLEAR, m_filter);
        if(m_list->GetColumnCount() == static_cast<int>(m_filters.GetCount()))
        {
            for(size_t i=0; i<m_filters.GetCount(); i++)
            {
                m_list->SetColumnHeaderFiltered(static_cast<int>(i), m_filters[i].active());
            }
            m_list->Update();
        }
    }
    else
    {
        m_filter = false;
        m_bar->ToggleTool(ID_PAGE_JIZDA_FILTER, false);
        m_bar->EnableTool(ID_PAGE_JIZDA_FILTERCLEAR, false);
        for(int i=0; i<m_list->GetColumnCount(); i++)
        {
            m_list->SetColumnHeaderFiltered(i, false);
        }
        m_list->Update();
    }
    updateData(ID_MODEL_ALL);
}

void JizdaPage::OnFilterClear(wxCommandEvent &/*event*/)
{
    m_bar->ToggleTool(ID_PAGE_JIZDA_FILTER, false);
    m_filter = false;
    m_bar->EnableTool(ID_PAGE_JIZDA_FILTERCLEAR, false);
    for(int i=0; i<m_list->GetColumnCount(); i++)
    {
        m_list->SetColumnHeaderFiltered(i, false);
    }
    m_list->Update();
    updateData(ID_MODEL_ALL);
}

void JizdaPage::getData()
{
    if(m_filter)
    {
        m_jizdy.Clear();
        dxJizdaViewArray jizdy = idbmodel->jizdyview();
        for(size_t i=0; i<jizdy.GetCount(); i++)
        {
            if(m_filters[0].matchFilter(jizdy[i].endcas()) && m_filters[1].matchFilter(jizdy[i].cesta()) && m_filters[2].matchFilter(jizdy[i].ucel())
                    && m_filters[3].matchFilter(jizdy[i].km()) && m_filters[4].matchFilter(jizdy[i].vozidlo()) && m_filters[5].matchFilter(jizdy[i].ridic()))
            {
                m_jizdy.Add(jizdy[i]);
            }
        }
    }
    else
    {
        m_jizdy = idbmodel->jizdyview();
    }
}

void JizdaPage::refreshItems()
{
    long oldcnt = m_list->GetItemCount();
    long cnt = static_cast<long>(m_jizdy.GetCount());
    if(cnt)
    {
        if(m_selected >= cnt)
        {
            m_list->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
            m_selected = 0;
        }
        if(oldcnt <= cnt) m_list->RefreshItems(0, --cnt);
        else m_list->Refresh();
    }
    else
    {
        m_list->Refresh();
    }
}


