/***************************************************************
 * Name:      jizdapage.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2018
 * License:   GPL3
 **************************************************************/

#ifndef JIZDAPAGE_H
#define JIZDAPAGE_H

#include <wx/imaglist.h>
#include "gui/widgets/dxlistctrl.h"
#include "dxpage.h"
#include "gui/filters/jizdafilter.h"
#include "gui/widgets/dxtoolbar.h"

class JizdaPage;

class JizdaCtrl: public dxListCtrl
{
public:
    JizdaCtrl(JizdaPage *page, wxWindow *parent, const wxWindowID id)
        : dxListCtrl(parent, id), m_page(page) {}

    enum JColumn
    {
        JCOL_DATUM = 0,
        JCOL_CESTA,
        JCOL_UCEL,
        JCOL_KM,
        JCOL_VOZIDLO,
        JCOL_RIDIC,
        JCOL_MAX, // pocet sloupcu
        JCOL_DEF_SORT = JCOL_DATUM
    };

    void CreateColumns();
private:
    JizdaPage *m_page;

    virtual wxString OnGetItemText(long item, long column) const;
    virtual int OnGetItemColumnImage(long item, long column) const;
    virtual wxListItemAttr *OnGetItemAttr(long item) const;
};

class JizdaPage : public dxPage
{
    DECLARE_DYNAMIC_CLASS(SkladPage)
    DECLARE_EVENT_TABLE()
public:
    JizdaPage() : dxPage(nullptr, NAV_DENIK), m_mainSizer(nullptr), m_list(nullptr), m_selected(-1),
    m_bar(nullptr), m_sortCol(JizdaCtrl::JCOL_DATUM), m_sortAsc(true), m_filter(false) {}
    explicit JizdaPage(wxWindow *parent);

    void updateData(int id) override;
    wxString getItemText(long item, long column) const;
private:
    wxBoxSizer *m_mainSizer;
    JizdaCtrl *m_list;
    long m_selected;
    dxToolBar *m_bar;
    JizdaCtrl::JColumn m_sortCol;
    bool m_sortAsc;
    bool m_filter;
    wxScopedPtr<wxImageList> m_imageList;
    dxJizdaViewArray m_jizdy;
    dxFilterArray m_filters;

    void OnAddJizda(wxCommandEvent& event);
    void OnEditJizda(wxCommandEvent& event);
    void OnCopyJizda(wxCommandEvent& event);
    void OnDeleteJizda(wxCommandEvent& event);
    void OnPrintKnihaJizd(wxCommandEvent& event);
    void OnExportKnihaJizd(wxCommandEvent& event);
    void OnItemRightClick(wxListEvent& event);
    void OnItemActivated(wxListEvent& event);
    void OnItemSelected(wxListEvent& event);
    void OnItemDeselected(wxListEvent& event);
    void OnColumnClick(wxListEvent& event);
    void enableTools();
    DbModel::JSort columnToJizdaSort(JizdaCtrl::JColumn column);
    void setColumnImage(JizdaCtrl::JColumn column, int image);
    void OnPrintJizda(wxCommandEvent& event);
    void OnVozidla(wxCommandEvent& event);
    void OnNahrada(wxCommandEvent& event);
    void OnFilter(wxCommandEvent& event);
    void OnFilterClear(wxCommandEvent& event);
    void getData();
    void refreshItems();
};

#endif  /* JIZDAPAGE_H */

