/***************************************************************
 * Name:      majetekpage.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#ifndef MAJETEKPAGE_H
#define MAJETEKPAGE_H

#include <wx/imaglist.h>
#include "gui/widgets/dxlistctrl.h"
#include "dxpage.h"
#include "gui/filters/filter.h"
#include "gui/widgets/dxtoolbar.h"

class MajetekPage;

class MajetekCtrl: public dxListCtrl
{
public:
    MajetekCtrl(MajetekPage *page, wxWindow *parent, const wxWindowID id)
        : dxListCtrl(parent, id), m_page(page) {}

    enum MColumn
    {
        MCOL_CISLO = 0,
        MCOL_NAZEV,
        MCOL_TYP,
        MCOL_ODPIS,
        MCOL_PORIZEN,
        MCOL_PORCENA,
        MCOL_ZUSCENA,
        MCOL_MAX, // pocet sloupcu
        MCOL_DEF_SORT = MCOL_CISLO
    };

    void CreateColumns();
private:
    MajetekPage *m_page;

    virtual wxString OnGetItemText(long item, long column) const;
    virtual int OnGetItemColumnImage(long item, long column) const;
    virtual wxListItemAttr *OnGetItemAttr(long item) const;
};

class MajetekPage : public dxPage
{
    DECLARE_DYNAMIC_CLASS(MajetekPage)
    DECLARE_EVENT_TABLE()
public:
    MajetekPage() : dxPage(nullptr, NAV_DENIK), m_mainSizer(nullptr), m_list(nullptr), m_selected(-1),
    m_bar(nullptr), m_sortCol(MajetekCtrl::MCOL_CISLO), m_sortAsc(true), m_filter(false) {}
    explicit MajetekPage(wxWindow *parent);

    void updateData(int id) override;
    wxString getItemText(long item, long column) const;
    wxListItemAttr *getItemAttr(long item) const;
private:
    wxBoxSizer *m_mainSizer;
    MajetekCtrl *m_list;
    long m_selected;
    dxToolBar *m_bar;
    MajetekCtrl::MColumn m_sortCol;
    bool m_sortAsc;
    wxScopedPtr<wxImageList> m_imageList;
    dxMajetekViewArray m_majetek;
    bool m_filter;
    dxFilterArray m_filters;

    void OnAddMajetek(wxCommandEvent& event);
    void OnEditMajetek(wxCommandEvent& event);
    void OnCopyMajetek(wxCommandEvent& event);
    void OnDeleteMajetek(wxCommandEvent& event);
    void OnPrintKnihaMajetku(wxCommandEvent& event);
    void OnExportKnihaMajetku(wxCommandEvent& event);
    void OnItemRightClick(wxListEvent& event);
    void OnItemActivated(wxListEvent& event);
    void OnItemSelected(wxListEvent& event);
    void OnItemDeselected(wxListEvent& event);
    void OnColumnClick(wxListEvent& event);
    void enableTools();
    DbModel::MSort columnToMajetekSort(MajetekCtrl::MColumn column);
    void setColumnImage(MajetekCtrl::MColumn column, int image);
    void OnPrintMajetek(wxCommandEvent& event);
    void OnCalcOdpisy(wxCommandEvent& event);
    void OnFilter(wxCommandEvent& event);
    void OnFilterClear(wxCommandEvent& event);
    void getData();
    void refreshItems();
};

#endif  /* MAJETEKPAGE_H */

