/***************************************************************
 * Name:      pohledavkapage.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include <wx/wfstream.h>
#include <wx/txtstrm.h>
#include "pohledavkapage.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "resources/images/add.xpm"
#include "resources/images/edit.xpm"
#include "resources/images/delete.xpm"
#include "resources/images/csv.xpm"
#include "resources/images/print.xpm"
#include "resources/images/asc.xpm"
#include "resources/images/desc.xpm"
#include "resources/images/uhrada.xpm"
#include "resources/images/printdoc.xpm"
#include "resources/images/copy.xpm"
#include "resources/images/seznam.xpm"
#include "resources/images/filter.xpm"
#include "resources/images/filter_clear.xpm"
#include "gui/dialogs/zavpohdialog.h"
#include "gui/dialogs/uhradadialog.h"
#include "gui/prints/knihazavpohprint.h"
#include "gui/prints/zavpohprint.h"
#include "gui/dialogs/uhradydialog.h"
#include "gui/filters/pohledavkafilter.h"

void PohledavkaCtrl::CreateColumns()
{
    InsertColumn(PCOL_DOKLAD, "Doklad");
    InsertColumn(PCOL_POPIS, "Popis");
    InsertColumn(PCOL_PROTISTRANA, "Protistrana");
    InsertColumn(PCOL_VYSTAVENA, "Vystavena");
    InsertColumn(PCOL_SPLATNA, wxString::FromUTF8("Splatná"));
    InsertColumn(PCOL_CELKEM, "Celkem", wxLIST_FORMAT_RIGHT);
    InsertColumn(PCOL_UHRADIT, "Uhradit", wxLIST_FORMAT_RIGHT);
    setResizeColumn(PCOL_POPIS);
}

wxString PohledavkaCtrl::OnGetItemText(long item, long column) const
{
    return m_page->getItemText(item, column);
}

int PohledavkaCtrl::OnGetItemColumnImage(long /*item*/, long /*column*/) const
{
    return -1;
}

wxListItemAttr *PohledavkaCtrl::OnGetItemAttr(long item) const
{
    return m_page->getItemAttr(item);
}

IMPLEMENT_DYNAMIC_CLASS(PohledavkaPage, wxPanel)

BEGIN_EVENT_TABLE(PohledavkaPage, wxPanel)
    EVT_MENU(ID_PAGE_POHLEDAVKA_ADD, PohledavkaPage::OnAddPohledavka)
    EVT_MENU(ID_PAGE_POHLEDAVKA_EDIT, PohledavkaPage::OnEditPohledavka)
    EVT_MENU(ID_PAGE_POHLEDAVKA_COPY, PohledavkaPage::OnCopyPohledavka)
    EVT_MENU(ID_PAGE_POHLEDAVKA_DELETE, PohledavkaPage::OnDeletePohledavka)
    EVT_MENU(ID_PAGE_POHLEDAVKA_PRINT, PohledavkaPage::OnPrintKnihaPohledavek)
    EVT_MENU(ID_PAGE_POHLEDAVKA_EXPORT, PohledavkaPage::OnExportKnihaPohledavek)
    EVT_LIST_ITEM_RIGHT_CLICK(ID_PAGE_POHLEDAVKA_LIST, PohledavkaPage::OnItemRightClick)
    EVT_LIST_ITEM_ACTIVATED(ID_PAGE_POHLEDAVKA_LIST, PohledavkaPage::OnItemActivated)
    EVT_LIST_ITEM_SELECTED(ID_PAGE_POHLEDAVKA_LIST, PohledavkaPage::OnItemSelected)
    EVT_LIST_ITEM_DESELECTED(ID_PAGE_POHLEDAVKA_LIST, PohledavkaPage::OnItemDeselected)
    EVT_LIST_COL_CLICK(ID_PAGE_POHLEDAVKA_LIST, PohledavkaPage::OnColumnClick)
    EVT_MENU(ID_PAGE_POHLEDAVKA_UHRADA, PohledavkaPage::OnUhraditPohledavka)
    EVT_MENU(ID_PAGE_POHLEDAVKA_POPRINT, PohledavkaPage::OnPrintPohledavka)
    EVT_MENU(ID_PAGE_POHLEDAVKA_UHRADY, PohledavkaPage::OnUhrady)
    EVT_MENU(ID_PAGE_POHLEDAVKA_FILTER, PohledavkaPage::OnFilter)
    EVT_MENU(ID_PAGE_POHLEDAVKA_FILTERCLEAR, PohledavkaPage::OnFilterClear)
END_EVENT_TABLE()

PohledavkaPage::PohledavkaPage(wxWindow *parent)
    : dxPage(parent, NAV_POHLEDAVKA), m_selected(-1), m_sortCol(PohledavkaCtrl::PCOL_DEF_SORT), m_sortAsc(true), m_filter(false)
{
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_LESS, false, dxsettings.datum()));
    m_filters.Add(Filter(FA_LESS, false, dxsettings.datum()));
    m_filters.Add(Filter(FA_LESS, false, 0.0));
    m_filters.Add(Filter(FA_LESS, false, 0.0));
    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    m_bar = new dxToolBar(this);
    m_bar->AddTool(ID_PAGE_POHLEDAVKA_ADD, wxEmptyString, wxBitmap(add_xpm), dxutils::GreyScaledIcon(add_xpm), wxITEM_NORMAL, wxString::FromUTF8("Nová pohledávka"));
    m_bar->AddTool(ID_PAGE_POHLEDAVKA_EDIT, wxEmptyString, wxBitmap(edit_xpm), dxutils::GreyScaledIcon(edit_xpm), wxITEM_NORMAL, wxString::FromUTF8("Upravit pohledávku"));
    m_bar->AddTool(ID_PAGE_POHLEDAVKA_COPY, wxEmptyString, wxBitmap(copy_xpm), dxutils::GreyScaledIcon(copy_xpm), wxITEM_NORMAL, wxString::FromUTF8("Kopírovat pohledávku"));
    m_bar->AddTool(ID_PAGE_POHLEDAVKA_DELETE, wxEmptyString, wxBitmap(delete_xpm), dxutils::GreyScaledIcon(delete_xpm), wxITEM_NORMAL, wxString::FromUTF8("Smazat pohledávku"));
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_POHLEDAVKA_UHRADA, wxEmptyString, wxBitmap(uhrada_xpm), dxutils::GreyScaledIcon(uhrada_xpm), wxITEM_NORMAL, wxString::FromUTF8("Uhradit pohledávku"));
    m_bar->AddTool(ID_PAGE_POHLEDAVKA_UHRADY, wxEmptyString, wxBitmap(seznam_xpm), dxutils::GreyScaledIcon(seznam_xpm), wxITEM_NORMAL, wxString::FromUTF8("Zobrazit úhrady"));
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_POHLEDAVKA_POPRINT, wxEmptyString, wxBitmap(printdoc_xpm), dxutils::GreyScaledIcon(printdoc_xpm), wxITEM_NORMAL, wxString::FromUTF8("Vytisknout pohledávku"));
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_POHLEDAVKA_PRINT, wxEmptyString, wxBitmap(print_xpm), dxutils::GreyScaledIcon(print_xpm), wxITEM_NORMAL, wxString::FromUTF8("Vytisknout knihu pohledávek"));
    m_bar->AddTool(ID_PAGE_POHLEDAVKA_EXPORT, wxEmptyString, wxBitmap(csv_xpm), dxutils::GreyScaledIcon(csv_xpm), wxITEM_NORMAL, wxString::FromUTF8("Exportovat knihu pohledávek"));
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_POHLEDAVKA_FILTER, wxEmptyString, wxBitmap(filter_xpm), dxutils::GreyScaledIcon(filter_xpm), wxITEM_CHECK, "Filtrovat");
    m_bar->AddTool(ID_PAGE_POHLEDAVKA_FILTERCLEAR, wxEmptyString, wxBitmap(filter_clear_xpm), dxutils::GreyScaledIcon(filter_clear_xpm), wxITEM_NORMAL, wxString::FromUTF8("Zrušit filtr"));
    m_bar->Realize();
    m_mainSizer->Add(m_bar, 0, wxALL|wxEXPAND, 5);
    wxSize imageSize(16, 16);
    m_imageList.reset(new wxImageList(imageSize.GetWidth(), imageSize.GetHeight()));
    m_imageList->Add(wxImage(asc_xpm).Scale(16, 16));
    m_imageList->Add(wxImage(desc_xpm).Scale(16, 16));
    m_list = new PohledavkaCtrl(this, this, ID_PAGE_POHLEDAVKA_LIST);
    m_list->SetImageList(m_imageList.get(), wxIMAGE_LIST_SMALL);
    m_list->CreateColumns();
    enableTools();
    m_mainSizer->Add(m_list, 1, wxALL|wxEXPAND, 5);
    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void PohledavkaPage::updateData(int id)
{
    if(!(id==ID_MODEL_ALL || id==ID_MODEL_POHLEDAVKA)) return;
    long oldcnt = m_list->GetItemCount();
    setColumnImage(m_sortCol, m_sortAsc?0:1);
    getData();
    idbmodel->sortZavPohView(m_pohledavky, columnToZavPohSort(m_sortCol), m_sortAsc);
    long cnt = static_cast<long>(m_pohledavky.GetCount());
    m_list->SetItemCount(cnt);
    if(cnt)
    {
        if(m_selected >= cnt || m_selected < 0)
        {
            m_list->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
            m_selected = 0;
        }
        if(oldcnt <= cnt) m_list->RefreshItems(0, --cnt);
        else m_list->Refresh();
        m_bar->EnableTool(ID_PAGE_POHLEDAVKA_PRINT, true);
        m_bar->EnableTool(ID_PAGE_POHLEDAVKA_EXPORT, true);
        m_bar->EnableTool(ID_PAGE_POHLEDAVKA_UHRADA, idbmodel->isZavpohUhrazena(m_pohledavky[static_cast<size_t>(m_selected)].id())?false:true);
        m_bar->EnableTool(ID_PAGE_POHLEDAVKA_FILTER, true);
        m_bar->EnableTool(ID_PAGE_POHLEDAVKA_FILTERCLEAR, m_filter?true:false);
    }
    else
    {
        m_list->Refresh();
        m_bar->EnableTool(ID_PAGE_POHLEDAVKA_EDIT, false);
        m_bar->EnableTool(ID_PAGE_POHLEDAVKA_COPY, false);
        m_bar->EnableTool(ID_PAGE_POHLEDAVKA_DELETE, false);
        m_bar->EnableTool(ID_PAGE_POHLEDAVKA_POPRINT, false);
        m_bar->EnableTool(ID_PAGE_POHLEDAVKA_UHRADA, false);
        m_bar->EnableTool(ID_PAGE_POHLEDAVKA_PRINT, false);
        m_bar->EnableTool(ID_PAGE_POHLEDAVKA_EXPORT, false);
        m_bar->EnableTool(ID_PAGE_POHLEDAVKA_FILTER, m_filter?true:false);
        m_bar->EnableTool(ID_PAGE_POHLEDAVKA_FILTERCLEAR, m_filter?true:false);
    }
}

wxString PohledavkaPage::getItemText(long item, long column) const
{
    if(item < 0 || item >= static_cast<long>(m_pohledavky.GetCount()))
        return "";
    switch(column)
    {
    case PohledavkaCtrl::PCOL_DOKLAD:
        return m_pohledavky[static_cast<size_t>(item)].cislo();
    case PohledavkaCtrl::PCOL_POPIS:
        return m_pohledavky[static_cast<size_t>(item)].popis();
    case PohledavkaCtrl::PCOL_PROTISTRANA:
        return m_pohledavky[static_cast<size_t>(item)].protistrana();
    case PohledavkaCtrl::PCOL_VYSTAVENA:
        return m_pohledavky[static_cast<size_t>(item)].vystaveni().FormatDate();
    case PohledavkaCtrl::PCOL_SPLATNA:
        return m_pohledavky[static_cast<size_t>(item)].splatnost().FormatDate();
    case PohledavkaCtrl::PCOL_CELKEM:
        return wxString::Format("%.2f", m_pohledavky[static_cast<size_t>(item)].celkem());
    case PohledavkaCtrl::PCOL_UHRADIT:
        return wxString::Format("%.2f", dxutils::isSameDouble(m_pohledavky[static_cast<size_t>(item)].celkem(), m_pohledavky[static_cast<size_t>(item)].uhrazeno())?
                                    0.0: m_pohledavky[static_cast<size_t>(item)].celkem()-m_pohledavky[static_cast<size_t>(item)].uhrazeno());
    default:
        return "";
    }
}

wxListItemAttr *PohledavkaPage::getItemAttr(long item) const
{
    if(item < 0 || item >= static_cast<long>(m_pohledavky.GetCount()))
        return nullptr;
    if(dxutils::isSameDouble(m_pohledavky[static_cast<size_t>(item)].celkem(), m_pohledavky[static_cast<size_t>(item)].uhrazeno()))
        return nullptr;
    else
    {
        if(m_pohledavky[static_cast<size_t>(item)].splatnost().IsLaterThan(dxsettings.datum()))
        {
            static wxListItemAttr green(wxColour(64,157,64), wxNullColour, wxNullFont);
            return &green;
        }
        else
        {
            static wxListItemAttr red(*wxRED, wxNullColour, wxNullFont);
            return &red;
        }
    }
}

void PohledavkaPage::OnAddPohledavka(wxCommandEvent& /*event*/)
{
    ZavpohDialog dialog(false, this);
    dialog.ShowModal();
}

void PohledavkaPage::OnEditPohledavka(wxCommandEvent& /*event*/)
{
    ZavpohDialog dialog(m_pohledavky[static_cast<size_t>(m_selected)].id(), this);
    dialog.ShowModal();
}

void PohledavkaPage::OnCopyPohledavka(wxCommandEvent& /*event*/)
{
    ZavpohDialog dialog(idbmodel->copyZavPoh(m_pohledavky[static_cast<size_t>(m_selected)].id()), this, 2);
    dialog.ShowModal();
}

void PohledavkaPage::OnDeletePohledavka(wxCommandEvent& /*event*/)
{
    ZavPohView zp = m_pohledavky[static_cast<size_t>(m_selected)];
    wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Opravdu smazat pohledávku\n%s\n%s?"), zp.popis(), zp.cislo()),
                           "Dotaz", wxOK|wxCANCEL|wxICON_QUESTION);
    if(dialog.ShowModal() == wxID_OK)
    {
        idbmodel->deleteZavPoh(zp.id());
    }
}

void PohledavkaPage::OnUhraditPohledavka(wxCommandEvent &/*event*/)
{
    UhradaDialog dialog(this, idbmodel->getZavPoh(m_pohledavky[static_cast<size_t>(m_selected)].id()));
    dialog.ShowModal();
}

void PohledavkaPage::OnPrintKnihaPohledavek(wxCommandEvent& /*event*/)
{
    KnihaZavpohPrint print(this, false);
    //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
    print.Print();
#else
    print.Preview();
#endif
}

void PohledavkaPage::OnExportKnihaPohledavek(wxCommandEvent& /*event*/)
{
    wxFileDialog dlg(this, wxString::FromUTF8("Exportovat knihu pohledávek do"), wxEmptyString, wxEmptyString, "CSV soubory (*.csv)|*.csv",
                     wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
    if(dlg.ShowModal() != wxID_OK) return;
    wxString path = dlg.GetPath();
    dxutils::correctEmptyFileExt("csv", path);
    wxString delim = ",";
    wxFileOutputStream output(path);
    wxTextOutputStream text(output);
    text << "Doklad" << delim << "Popis" << delim << "Protistrana" << delim << "Vystavena" << delim
         << wxString::FromUTF8("Splatná") << delim << "Celkem" << delim << "Uhradit" << delim << wxString::FromUTF8("Měna") << endl;
    dxZavPohArray pohledavky = idbmodel->pohledavky();
    for(size_t i=0; i<pohledavky.GetCount(); i++)
    {
        text << dxutils::enquotes(pohledavky[i].cislo(), delim);
        text << delim;
        text << dxutils::enquotes(pohledavky[i].popis(), delim);
        text << delim;
        text << dxutils::enquotes(idbmodel->getKontakt(pohledavky[i].protistranaid()).jmeno(), delim);
        text << delim;
        text << pohledavky[i].vystaveni().FormatDate();
        text << delim;
        text << pohledavky[i].splatnost().FormatDate();
        text << delim;
        text << dxutils::enquotes(wxString::Format("%.2f", pohledavky[i].celkem()), delim);
        text << delim;
        text << dxutils::enquotes(wxString::Format("%.2f", pohledavky[i].celkem()-idbmodel->uhrazenoByFakturaId(m_pohledavky[i].id())), delim);
        text << delim;
        text << idbmodel->zkratkaMena(pohledavky[i].menaid());
        text << endl;
    }
}

void PohledavkaPage::OnItemRightClick(wxListEvent &event)
{
    m_selected = event.GetIndex();
    enableTools();
    wxMenu popup;
    popup.Append(ID_PAGE_POHLEDAVKA_ADD, wxString::FromUTF8("Nová pohledávka"));
    if(m_selected != -1)
    {
        popup.AppendSeparator();
        popup.Append(ID_PAGE_POHLEDAVKA_EDIT, wxString::FromUTF8("Upravit pohledávku"));
        popup.Append(ID_PAGE_POHLEDAVKA_COPY, wxString::FromUTF8("Kopírovat pohledávku"));
        popup.Append(ID_PAGE_POHLEDAVKA_DELETE, wxString::FromUTF8("Smazat pohledávku"));
        if(!idbmodel->isZavpohUhrazena(m_pohledavky[static_cast<size_t>(m_selected)].id()))
        {
            popup.AppendSeparator();
            popup.Append(ID_PAGE_POHLEDAVKA_UHRADA, wxString::FromUTF8("Uhradit pohledávku"));
        }
        if(idbmodel->someUhradaForZavpoh(m_pohledavky[static_cast<size_t>(m_selected)].id()))
        {
            popup.AppendSeparator();
            popup.Append(ID_PAGE_POHLEDAVKA_UHRADY, wxString::FromUTF8("Zobrazit úhrady"));
        }
        popup.AppendSeparator();
        popup.Append(ID_PAGE_POHLEDAVKA_POPRINT, wxString::FromUTF8("Vytisknout pohledávku"));
    }
    PopupMenu(&popup, event.GetPoint()+m_list->GetPosition());
}

void PohledavkaPage::OnItemActivated(wxListEvent &event)
{
    m_selected = event.GetIndex();
    enableTools();
    ZavpohDialog dialog(m_pohledavky[static_cast<size_t>(m_selected)].id(), this);
    dialog.ShowModal();
}

void PohledavkaPage::OnItemSelected(wxListEvent &event)
{
    m_selected = event.GetIndex();
    enableTools();
}

void PohledavkaPage::OnItemDeselected(wxListEvent& /*event*/)
{
    m_selected = -1;
    enableTools();
}

void PohledavkaPage::OnColumnClick(wxListEvent &event)
{
    if(0 > event.GetColumn() || event.GetColumn() >= PohledavkaCtrl::PCOL_MAX) return;
    setColumnImage(m_sortCol, -1);
    m_sortCol = static_cast<PohledavkaCtrl::PColumn>(event.GetColumn());
    m_sortAsc = !m_sortAsc;
    setColumnImage(m_sortCol, m_sortAsc?0:1);
    if(m_pohledavky.GetCount()) idbmodel->sortZavPohView(m_pohledavky, columnToZavPohSort(m_sortCol), m_sortAsc);
    refreshItems();
}

void PohledavkaPage::enableTools()
{
    if(m_selected == -1)
    {
        m_bar->EnableTool(ID_PAGE_POHLEDAVKA_EDIT, false);
        m_bar->EnableTool(ID_PAGE_POHLEDAVKA_COPY, false);
        m_bar->EnableTool(ID_PAGE_POHLEDAVKA_DELETE, false);
        m_bar->EnableTool(ID_PAGE_POHLEDAVKA_POPRINT, false);
        m_bar->EnableTool(ID_PAGE_POHLEDAVKA_UHRADA, false);
        m_bar->EnableTool(ID_PAGE_POHLEDAVKA_UHRADY, false);
    }
    else
    {
        m_bar->EnableTool(ID_PAGE_POHLEDAVKA_EDIT, true);
        m_bar->EnableTool(ID_PAGE_POHLEDAVKA_COPY, true);
        m_bar->EnableTool(ID_PAGE_POHLEDAVKA_DELETE, true);
        m_bar->EnableTool(ID_PAGE_POHLEDAVKA_POPRINT, true);
        m_bar->EnableTool(ID_PAGE_POHLEDAVKA_UHRADA, idbmodel->isZavpohUhrazena(m_pohledavky[static_cast<size_t>(m_selected)].id())?false:true);
        m_bar->EnableTool(ID_PAGE_POHLEDAVKA_UHRADY, idbmodel->someUhradaForZavpoh(m_pohledavky[static_cast<size_t>(m_selected)].id()));
    }
}

DbModel::FZPSort PohledavkaPage::columnToZavPohSort(PohledavkaCtrl::PColumn column)
{
    switch(column)
    {
    case PohledavkaCtrl::PCOL_DOKLAD: return DbModel::FZP_DOKLAD;
    case PohledavkaCtrl::PCOL_POPIS: return DbModel::FZP_POPIS;
    case PohledavkaCtrl::PCOL_PROTISTRANA: return DbModel::FZP_PROTISTRANA;
    case PohledavkaCtrl::PCOL_VYSTAVENA: return DbModel::FZP_VYSTAVENA;
    case PohledavkaCtrl::PCOL_SPLATNA: return DbModel::FZP_SPLATNA;
    case PohledavkaCtrl::PCOL_CELKEM: return DbModel::FZP_CELKEM;
    case PohledavkaCtrl::PCOL_UHRADIT: return DbModel::FZP_UHRADIT;
    default: return DbModel::FZP_DOKLAD;
    }
}

void PohledavkaPage::setColumnImage(PohledavkaCtrl::PColumn column, int image)
{
    wxListItem item;
    item.SetMask(wxLIST_MASK_IMAGE);
    item.SetImage(image);
    m_list->SetColumn(column, item);
}

void PohledavkaPage::OnPrintPohledavka(wxCommandEvent &/*event*/)
{
    ZavpohPrint print(this, idbmodel->getZavPoh(m_pohledavky[static_cast<size_t>(m_selected)].id()));
    //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
    print.Print();
#else
    print.Preview();
#endif
}

void PohledavkaPage::OnUhrady(wxCommandEvent &/*event*/)
{
    UhradyDialog dialog(this, -1, m_pohledavky[static_cast<size_t>(m_selected)].id());
    dialog.ShowModal();
}

void PohledavkaPage::OnFilter(wxCommandEvent &/*event*/)
{
    PohledavkaFilter filter(this,m_filters);
    if(filter.ShowModal() == wxID_OK)
    {
        m_filters = filter.filters();
        m_filter = false;
        for(size_t i=0; i<m_filters.GetCount(); i++)
        {
            if(m_filters[i].active())
            {
                m_filter = true;
                break;
            }
        }
        m_bar->ToggleTool(ID_PAGE_POHLEDAVKA_FILTER, m_filter);
        m_bar->EnableTool(ID_PAGE_POHLEDAVKA_FILTERCLEAR, m_filter);
        if(m_list->GetColumnCount() == static_cast<int>(m_filters.GetCount()))
        {
            for(size_t i=0; i<m_filters.GetCount(); i++)
            {
                m_list->SetColumnHeaderFiltered(static_cast<int>(i), m_filters[i].active());
            }
            m_list->Update();
        }
    }
    else
    {
        m_filter = false;
        m_bar->ToggleTool(ID_PAGE_POHLEDAVKA_FILTER, false);
        m_bar->EnableTool(ID_PAGE_POHLEDAVKA_FILTERCLEAR, false);
        for(int i=0; i<m_list->GetColumnCount(); i++)
        {
            m_list->SetColumnHeaderFiltered(i, false);
        }
        m_list->Update();
    }
    updateData(ID_MODEL_ALL);
}

void PohledavkaPage::OnFilterClear(wxCommandEvent &/*event*/)
{
    m_bar->ToggleTool(ID_PAGE_POHLEDAVKA_FILTER, false);
    m_filter = false;
    m_bar->EnableTool(ID_PAGE_POHLEDAVKA_FILTERCLEAR, false);
    for(int i=0; i<m_list->GetColumnCount(); i++)
    {
        m_list->SetColumnHeaderFiltered(i, false);
    }
    m_list->Update();
    updateData(ID_MODEL_ALL);
}

void PohledavkaPage::getData()
{
    if(m_filter)
    {
        m_pohledavky.Clear();
        dxZavPohViewArray pohledavky = idbmodel->pohledavkyview();
        for(size_t i=0; i<pohledavky.GetCount(); i++)
        {
            if(m_filters[0].matchFilter(pohledavky[i].cislo()) && m_filters[1].matchFilter(pohledavky[i].popis()) && m_filters[2].matchFilter(pohledavky[i].protistrana()) && m_filters[3].matchFilter(pohledavky[i].vystaveni())
                    && m_filters[4].matchFilter(pohledavky[i].splatnost()) && m_filters[5].matchFilter(pohledavky[i].celkem()) && m_filters[6].matchFilter(pohledavky[i].celkem()-pohledavky[i].uhrazeno()))
            {
                m_pohledavky.Add(pohledavky[i]);
            }
        }
    }
    else
    {
        m_pohledavky = idbmodel->pohledavkyview();
    }
}

void PohledavkaPage::refreshItems()
{
    long oldcnt = m_list->GetItemCount();
    long cnt = static_cast<long>(m_pohledavky.GetCount());
    if(cnt)
    {
        if(m_selected >= cnt)
        {
            m_list->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
            m_selected = 0;
        }
        if(oldcnt <= cnt) m_list->RefreshItems(0, --cnt);
        else m_list->Refresh();
    }
    else
    {
        m_list->Refresh();
    }
}
