/***************************************************************
 * Name:      pohledavkapage.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#ifndef POHLEDAVKAPAGE_H
#define POHLEDAVKAPAGE_H

#include <wx/imaglist.h>
#include "gui/widgets/dxlistctrl.h"
#include "dxpage.h"
#include "gui/filters/filter.h"
#include "gui/widgets/dxtoolbar.h"

class PohledavkaPage;

class PohledavkaCtrl: public dxListCtrl
{
public:
    PohledavkaCtrl(PohledavkaPage *page, wxWindow *parent, const wxWindowID id)
        : dxListCtrl(parent, id), m_page(page) {}

    enum PColumn
    {
        PCOL_DOKLAD = 0,
        PCOL_POPIS,
        PCOL_PROTISTRANA,
        PCOL_VYSTAVENA,
        PCOL_SPLATNA,
        PCOL_CELKEM,
        PCOL_UHRADIT,
        PCOL_MAX, // pocet sloupcu
        PCOL_DEF_SORT = PCOL_DOKLAD
    };

    void CreateColumns();
private:
    PohledavkaPage *m_page;

    virtual wxString OnGetItemText(long item, long column) const;
    virtual int OnGetItemColumnImage(long item, long column) const;
    virtual wxListItemAttr *OnGetItemAttr(long item) const;
};

class PohledavkaPage : public dxPage
{
    DECLARE_DYNAMIC_CLASS(PohledavkaPage)
    DECLARE_EVENT_TABLE()
public:
    PohledavkaPage() : dxPage(nullptr, NAV_POHLEDAVKA), m_mainSizer(nullptr), m_list(nullptr), m_selected(-1),
    m_bar(nullptr), m_sortCol(PohledavkaCtrl::PCOL_DOKLAD), m_sortAsc(true), m_filter(false) {}
    explicit PohledavkaPage(wxWindow *parent);

    void updateData(int id) override;
    wxString getItemText(long item, long column) const;
    wxListItemAttr *getItemAttr(long item) const;
private:
    wxBoxSizer *m_mainSizer;
    PohledavkaCtrl *m_list;
    long m_selected;
    dxToolBar *m_bar;
    PohledavkaCtrl::PColumn m_sortCol;
    bool m_sortAsc;
    wxScopedPtr<wxImageList> m_imageList;
    dxZavPohViewArray m_pohledavky;
    bool m_filter;
    dxFilterArray m_filters;

    void OnAddPohledavka(wxCommandEvent& event);
    void OnEditPohledavka(wxCommandEvent& event);
    void OnCopyPohledavka(wxCommandEvent& event);
    void OnDeletePohledavka(wxCommandEvent& event);
    void OnUhraditPohledavka(wxCommandEvent& event);
    void OnPrintKnihaPohledavek(wxCommandEvent& event);
    void OnExportKnihaPohledavek(wxCommandEvent& event);
    void OnItemRightClick(wxListEvent& event);
    void OnItemActivated(wxListEvent& event);
    void OnItemSelected(wxListEvent& event);
    void OnItemDeselected(wxListEvent& event);
    void OnColumnClick(wxListEvent& event);
    void enableTools();
    DbModel::FZPSort columnToZavPohSort(PohledavkaCtrl::PColumn column);
    void setColumnImage(PohledavkaCtrl::PColumn column, int image);
    void OnPrintPohledavka(wxCommandEvent& event);
    void OnUhrady(wxCommandEvent& event);
    void OnFilter(wxCommandEvent& event);
    void OnFilterClear(wxCommandEvent& event);
    void getData();
    void refreshItems();
};

#endif  /* POHLEDAVKAPAGE_H */

