/***************************************************************
 * Name:      prijatapage.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include <wx/wfstream.h>
#include <wx/txtstrm.h>
#include "prijatapage.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "resources/images/add.xpm"
#include "resources/images/edit.xpm"
#include "resources/images/delete.xpm"
#include "resources/images/csv.xpm"
#include "resources/images/print.xpm"
#include "resources/images/asc.xpm"
#include "resources/images/desc.xpm"
#include "resources/images/uhrada.xpm"
#include "resources/images/printdoc.xpm"
#include "resources/images/copy.xpm"
#include "resources/images/seznam.xpm"
#include "resources/images/filter.xpm"
#include "resources/images/filter_clear.xpm"
#include "gui/dialogs/fakturadialog.h"
#include "gui/dialogs/uhradadialog.h"
#include "gui/prints/knihafakturprint.h"
#include "gui/prints/fakturaneplatceprint.h"
#include "gui/prints/fakturaplatceprint.h"
#include "gui/dialogs/uhradydialog.h"
#include "gui/filters/prijatafilter.h"

void PrijataCtrl::CreateColumns()
{
    InsertColumn(PCOL_DOKLAD, "Doklad");
    InsertColumn(PCOL_POPIS, "Popis");
    InsertColumn(PCOL_ODBERRATEL, "Dodavatel");
    InsertColumn(PCOL_VYSTAVENA, "Vystavena");
    InsertColumn(PCOL_SPLATNA, wxString::FromUTF8("Splatná"));
    InsertColumn(PCOL_CELKEM, "Celkem", wxLIST_FORMAT_RIGHT);
    InsertColumn(PCOL_UHRADIT, "Uhradit", wxLIST_FORMAT_RIGHT);
    setResizeColumn(PCOL_POPIS);
}

wxString PrijataCtrl::OnGetItemText(long item, long column) const
{
    return m_page->getItemText(item, column);
}

int PrijataCtrl::OnGetItemColumnImage(long /*item*/, long /*column*/) const
{
    return -1;
}

wxListItemAttr *PrijataCtrl::OnGetItemAttr(long item) const
{
    return m_page->getItemAttr(item);
}

IMPLEMENT_DYNAMIC_CLASS(PrijataPage, wxPanel)

BEGIN_EVENT_TABLE(PrijataPage, wxPanel)
    EVT_MENU(ID_PAGE_PRIJATA_ADD, PrijataPage::OnAddFaktura)
    EVT_MENU(ID_PAGE_PRIJATA_EDIT, PrijataPage::OnEditFaktura)
    EVT_MENU(ID_PAGE_PRIJATA_COPY, PrijataPage::OnCopyFaktura)
    EVT_MENU(ID_PAGE_PRIJATA_DELETE, PrijataPage::OnDeleteFaktura)
    EVT_MENU(ID_PAGE_PRIJATA_PRINT, PrijataPage::OnPrintKnihaFaktur)
    EVT_MENU(ID_PAGE_PRIJATA_EXPORT, PrijataPage::OnExportKnihaFaktur)
    EVT_LIST_ITEM_RIGHT_CLICK(ID_PAGE_PRIJATA_LIST, PrijataPage::OnItemRightClick)
    EVT_LIST_ITEM_ACTIVATED(ID_PAGE_PRIJATA_LIST, PrijataPage::OnItemActivated)
    EVT_LIST_ITEM_SELECTED(ID_PAGE_PRIJATA_LIST, PrijataPage::OnItemSelected)
    EVT_LIST_ITEM_DESELECTED(ID_PAGE_PRIJATA_LIST, PrijataPage::OnItemDeselected)
    EVT_LIST_COL_CLICK(ID_PAGE_PRIJATA_LIST, PrijataPage::OnColumnClick)
    EVT_MENU(ID_PAGE_PRIJATA_UHRADA, PrijataPage::OnUhraditFaktura)
    EVT_MENU(ID_PAGE_PRIJATA_FAPRINT, PrijataPage::OnPrintFaktura)
    EVT_MENU(ID_PAGE_PRIJATA_UHRADY, PrijataPage::OnUhrady)
    EVT_MENU(ID_PAGE_PRIJATA_FILTER, PrijataPage::OnFilter)
    EVT_MENU(ID_PAGE_PRIJATA_FILTERCLEAR, PrijataPage::OnFilterClear)
END_EVENT_TABLE()

PrijataPage::PrijataPage(wxWindow *parent)
    : dxPage(parent, NAV_PRIJATA), m_selected(-1), m_sortCol(PrijataCtrl::PCOL_DEF_SORT), m_sortAsc(true), m_filter(false)
{
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_LESS, false, dxsettings.datum()));
    m_filters.Add(Filter(FA_LESS, false, dxsettings.datum()));
    m_filters.Add(Filter(FA_LESS, false, 0.0));
    m_filters.Add(Filter(FA_LESS, false, 0.0));
    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    m_bar = new dxToolBar(this);
    m_bar->AddTool(ID_PAGE_PRIJATA_ADD, wxEmptyString, wxBitmap(add_xpm), dxutils::GreyScaledIcon(add_xpm), wxITEM_NORMAL, wxString::FromUTF8("Nová faktura"));
    m_bar->AddTool(ID_PAGE_PRIJATA_EDIT, wxEmptyString, wxBitmap(edit_xpm), dxutils::GreyScaledIcon(edit_xpm), wxITEM_NORMAL, "Upravit fakturu");
    m_bar->AddTool(ID_PAGE_PRIJATA_COPY, wxEmptyString, wxBitmap(copy_xpm), dxutils::GreyScaledIcon(copy_xpm), wxITEM_NORMAL, wxString::FromUTF8("Kopírovat fakturu"));
    m_bar->AddTool(ID_PAGE_PRIJATA_DELETE, wxEmptyString, wxBitmap(delete_xpm), dxutils::GreyScaledIcon(delete_xpm), wxITEM_NORMAL, "Smazat fakturu");
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_PRIJATA_UHRADA, wxEmptyString, wxBitmap(uhrada_xpm), dxutils::GreyScaledIcon(uhrada_xpm), wxITEM_NORMAL, "Uhradit fakturu");
    m_bar->AddTool(ID_PAGE_PRIJATA_UHRADY, wxEmptyString, wxBitmap(seznam_xpm), dxutils::GreyScaledIcon(seznam_xpm), wxITEM_NORMAL, wxString::FromUTF8("Zobrazit úhrady"));
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_PRIJATA_FAPRINT, wxEmptyString, wxBitmap(printdoc_xpm), dxutils::GreyScaledIcon(printdoc_xpm), wxITEM_NORMAL, "Vytisknout fakturu");
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_PRIJATA_PRINT, wxEmptyString, wxBitmap(print_xpm), dxutils::GreyScaledIcon(print_xpm), wxITEM_NORMAL, "Vytisknout knihu faktur");
    m_bar->AddTool(ID_PAGE_PRIJATA_EXPORT, wxEmptyString, wxBitmap(csv_xpm), dxutils::GreyScaledIcon(csv_xpm), wxITEM_NORMAL, "Exportovat knihu faktur");
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_PRIJATA_FILTER, wxEmptyString, wxBitmap(filter_xpm), dxutils::GreyScaledIcon(filter_xpm), wxITEM_CHECK, "Filtrovat");
    m_bar->AddTool(ID_PAGE_PRIJATA_FILTERCLEAR, wxEmptyString, wxBitmap(filter_clear_xpm), dxutils::GreyScaledIcon(filter_clear_xpm), wxITEM_NORMAL, wxString::FromUTF8("Zrušit filtr"));
    m_bar->Realize();
    m_mainSizer->Add(m_bar, 0, wxALL|wxEXPAND, 5);
    wxSize imageSize(16, 16);
    m_imageList.reset(new wxImageList(imageSize.GetWidth(), imageSize.GetHeight()));
    m_imageList->Add(wxImage(asc_xpm).Scale(16, 16));
    m_imageList->Add(wxImage(desc_xpm).Scale(16, 16));
    m_list = new PrijataCtrl(this, this, ID_PAGE_PRIJATA_LIST);
    m_list->SetImageList(m_imageList.get(), wxIMAGE_LIST_SMALL);
    m_list->CreateColumns();
    enableTools();
    m_mainSizer->Add(m_list, 1, wxALL|wxEXPAND, 5);
    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void PrijataPage::updateData(int id)
{
    if(!(id==ID_MODEL_ALL || id==ID_MODEL_PRIJATA)) return;
    long oldcnt = m_list->GetItemCount();
    setColumnImage(m_sortCol, m_sortAsc?0:1);
    getData();
    idbmodel->sortFakturyView(m_faktury, columnToFakturaSort(m_sortCol), m_sortAsc);
    long cnt = static_cast<long>(m_faktury.GetCount());
    m_list->SetItemCount(static_cast<long>(cnt));
    if(cnt)
    {
        if(m_selected >= cnt || m_selected < 0)
        {
            m_list->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
            m_selected = 0;
        }
        if(oldcnt <= cnt) m_list->RefreshItems(0, --cnt);
        else m_list->Refresh();
        m_bar->EnableTool(ID_PAGE_PRIJATA_PRINT, true);
        m_bar->EnableTool(ID_PAGE_PRIJATA_EXPORT, true);
        m_bar->EnableTool(ID_PAGE_PRIJATA_UHRADA, idbmodel->isFakturaUhrazena(m_faktury[static_cast<size_t>(m_selected)].id())?false:true);
        m_bar->EnableTool(ID_PAGE_PRIJATA_FILTER, true);
        m_bar->EnableTool(ID_PAGE_PRIJATA_FILTERCLEAR, m_filter?true:false);
    }
    else
    {
        m_list->Refresh();
        m_bar->EnableTool(ID_PAGE_PRIJATA_EDIT, false);
        m_bar->EnableTool(ID_PAGE_PRIJATA_COPY, false);
        m_bar->EnableTool(ID_PAGE_PRIJATA_DELETE, false);
        m_bar->EnableTool(ID_PAGE_PRIJATA_FAPRINT, false);
        m_bar->EnableTool(ID_PAGE_PRIJATA_UHRADA, false);
        m_bar->EnableTool(ID_PAGE_PRIJATA_PRINT, false);
        m_bar->EnableTool(ID_PAGE_PRIJATA_EXPORT, false);
        m_bar->EnableTool(ID_PAGE_PRIJATA_FILTER, m_filter?true:false);
        m_bar->EnableTool(ID_PAGE_PRIJATA_FILTERCLEAR, m_filter?true:false);
    }
}

wxString PrijataPage::getItemText(long item, long column) const
{
    if(item < 0 || item >= static_cast<long>(m_faktury.GetCount()))
        return "";
    switch(column)
    {
    case PrijataCtrl::PCOL_DOKLAD:
        return m_faktury[static_cast<size_t>(item)].cislo();
    case PrijataCtrl::PCOL_POPIS:
        return m_faktury[static_cast<size_t>(item)].popis();
    case PrijataCtrl::PCOL_ODBERRATEL:
        return m_faktury[static_cast<size_t>(item)].protistrana();
    case PrijataCtrl::PCOL_VYSTAVENA:
        return m_faktury[static_cast<size_t>(item)].vystaveni().FormatDate();
    case PrijataCtrl::PCOL_SPLATNA:
        return m_faktury[static_cast<size_t>(item)].splatnost().FormatDate();
    case PrijataCtrl::PCOL_CELKEM:
        return wxString::Format("%.2f", m_faktury[static_cast<size_t>(item)].celkem());
    case PrijataCtrl::PCOL_UHRADIT:
        return wxString::Format("%.2f", dxutils::isSameDouble(m_faktury[static_cast<size_t>(item)].celkem(), m_faktury[static_cast<size_t>(item)].uhrazeno()) ?
                                    0.0 : m_faktury[static_cast<size_t>(item)].celkem()-m_faktury[static_cast<size_t>(item)].uhrazeno());
    default:
        return "";
    }
}

wxListItemAttr *PrijataPage::getItemAttr(long item) const
{
    if(item < 0 || item >= static_cast<long>(m_faktury.GetCount()))
        return nullptr;
    if(idbmodel->isFakturaUhrazena(m_faktury[static_cast<size_t>(item)].id()))
        return nullptr;
    else
    {
        if(m_faktury[static_cast<size_t>(item)].splatnost().IsLaterThan(dxsettings.datum()))
        {
            static wxListItemAttr green(wxColour(64,157,64), wxNullColour, wxNullFont);
            return &green;
        }
        else
        {
            static wxListItemAttr red(*wxRED, wxNullColour, wxNullFont);
            return &red;
        }
    }
}

void PrijataPage::OnAddFaktura(wxCommandEvent& /*event*/)
{
    wxInt64 typid = 0;
    wxSingleChoiceDialog vyber(this, "Vyberte typ faktury", "Typ faktury", idbmodel->getFaTypyAsStrings());
    if(vyber.ShowModal() == wxID_OK)
    {
        typid = idbmodel->itemFaTyp(static_cast<size_t>(vyber.GetSelection())).id();
    }
    else
    {
        return;
    }
    FakturaDialog dialog(typid, false, this);
    dialog.ShowModal();
}

void PrijataPage::OnEditFaktura(wxCommandEvent& /*event*/)
{
    FakturaDialog dialog(m_faktury[static_cast<size_t>(m_selected)].id(), this);
    dialog.ShowModal();
}

void PrijataPage::OnCopyFaktura(wxCommandEvent& /*event*/)
{
    FakturaDialog dialog(idbmodel->copyFaktura(m_faktury[static_cast<size_t>(m_selected)].id()), this, 2);
    dialog.ShowModal();
}

void PrijataPage::OnDeleteFaktura(wxCommandEvent& /*event*/)
{
    FakturaView fa = m_faktury[static_cast<size_t>(m_selected)];
    wxMessageDialog dialog(this, wxString::Format("Opravdu smazat fakturu\n%s\n%s?", fa.popis(), fa.cislo()),
                           "Dotaz", wxOK|wxCANCEL|wxICON_QUESTION);
    if(dialog.ShowModal() == wxID_OK)
    {
        idbmodel->deleteFaktura(fa.id());
    }
}

void PrijataPage::OnUhraditFaktura(wxCommandEvent &/*event*/)
{
    UhradaDialog dialog(this, idbmodel->getFaktura(m_faktury[static_cast<size_t>(m_selected)].id()));
    dialog.ShowModal();
}

void PrijataPage::OnPrintKnihaFaktur(wxCommandEvent& /*event*/)
{
    KnihaFakturPrint print(this, false);
    //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
    print.Print();
#else
    print.Preview();
#endif
}

void PrijataPage::OnExportKnihaFaktur(wxCommandEvent& /*event*/)
{
    wxFileDialog dlg(this, "Exportovat knihu faktur do", wxEmptyString, wxEmptyString, "CSV soubory (*.csv)|*.csv",
                     wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
    if(dlg.ShowModal() != wxID_OK) return;
    wxString path = dlg.GetPath();
    dxutils::correctEmptyFileExt("csv", path);
    wxString delim = ",";
    wxFileOutputStream output(path);
    wxTextOutputStream text(output);
    text << "Doklad" << delim << "Popis" << delim << wxString::FromUTF8("Odběratel") << delim << "Vystavena" << delim
         << wxString::FromUTF8("Splatná") << delim << "Celkem" << delim << "Uhradit" << delim << wxString::FromUTF8("Měna") << endl;
    dxFakturaArray faktury = idbmodel->prijate();
    for(size_t i=0; i<faktury.GetCount(); i++)
    {
        text << dxutils::enquotes(faktury[i].cislo(), delim);
        text << delim;
        text << dxutils::enquotes(faktury[i].popis(), delim);
        text << delim;
        text << dxutils::enquotes(idbmodel->getKontakt(faktury[i].protistranaid()).jmeno(), delim);
        text << delim;
        text << faktury[i].vystaveni().FormatDate();
        text << delim;
        text << faktury[i].splatnost().FormatDate();
        text << delim;
        text << dxutils::enquotes(wxString::Format("%.2f", faktury[i].celkem()), delim);
        text << delim;
        text << dxutils::enquotes(wxString::Format("%.2f", faktury[i].celkem()-idbmodel->uhrazenoByFakturaId(faktury[i].id())), delim);
        text << delim;
        text << idbmodel->zkratkaMena(faktury[i].menaid());
        text << endl;
    }
}

void PrijataPage::OnItemRightClick(wxListEvent &event)
{
    m_selected = event.GetIndex();
    enableTools();
    wxMenu popup;
    popup.Append(ID_PAGE_PRIJATA_ADD, wxString::FromUTF8("Nová faktura"));
    if(m_selected != -1)
    {
        popup.AppendSeparator();
        popup.Append(ID_PAGE_PRIJATA_EDIT, "Upravit fakturu");
        popup.Append(ID_PAGE_PRIJATA_COPY, wxString::FromUTF8("Kopírovat fakturu"));
        popup.Append(ID_PAGE_PRIJATA_DELETE, "Smazat fakturu");
        if(!idbmodel->isFakturaUhrazena(m_faktury[static_cast<size_t>(m_selected)].id()))
        {
            popup.AppendSeparator();
            popup.Append(ID_PAGE_PRIJATA_UHRADA, "Uhradit fakturu");
        }
        if(idbmodel->someUhradaForFaktura(m_faktury[static_cast<size_t>(m_selected)].id()))
        {
            popup.AppendSeparator();
            popup.Append(ID_PAGE_PRIJATA_UHRADY, wxString::FromUTF8("Zobrazit úhrady"));
        }
        popup.AppendSeparator();
        popup.Append(ID_PAGE_PRIJATA_FAPRINT, "Vytisknout fakturu");
    }
    PopupMenu(&popup, event.GetPoint()+m_list->GetPosition());
}

void PrijataPage::OnItemActivated(wxListEvent &event)
{
    m_selected = event.GetIndex();
    enableTools();
    FakturaDialog dialog(m_faktury[static_cast<size_t>(m_selected)].id(), this);
    dialog.ShowModal();
}

void PrijataPage::OnItemSelected(wxListEvent &event)
{
    m_selected = event.GetIndex();
    enableTools();
}

void PrijataPage::OnItemDeselected(wxListEvent& /*event*/)
{
    m_selected = -1;
    enableTools();
}

void PrijataPage::OnColumnClick(wxListEvent &event)
{
    if(0 > event.GetColumn() || event.GetColumn() >= PrijataCtrl::PCOL_MAX) return;
    setColumnImage(m_sortCol, -1);
    m_sortCol = static_cast<PrijataCtrl::PColumn>(event.GetColumn());
    m_sortAsc = !m_sortAsc;
    setColumnImage(m_sortCol, m_sortAsc?0:1);
    if(m_faktury.GetCount()) idbmodel->sortFakturyView(m_faktury, columnToFakturaSort(m_sortCol), m_sortAsc);
    refreshItems();
}

void PrijataPage::enableTools()
{
    if(m_selected == -1)
    {
        m_bar->EnableTool(ID_PAGE_PRIJATA_EDIT, false);
        m_bar->EnableTool(ID_PAGE_PRIJATA_COPY, false);
        m_bar->EnableTool(ID_PAGE_PRIJATA_DELETE, false);
        m_bar->EnableTool(ID_PAGE_PRIJATA_FAPRINT, false);
        m_bar->EnableTool(ID_PAGE_PRIJATA_UHRADA, false);
        m_bar->EnableTool(ID_PAGE_PRIJATA_UHRADY, false);
    }
    else
    {
        m_bar->EnableTool(ID_PAGE_PRIJATA_EDIT, true);
        m_bar->EnableTool(ID_PAGE_PRIJATA_COPY, true);
        m_bar->EnableTool(ID_PAGE_PRIJATA_DELETE, true);
        m_bar->EnableTool(ID_PAGE_PRIJATA_FAPRINT, true);
        m_bar->EnableTool(ID_PAGE_PRIJATA_UHRADA, idbmodel->isFakturaUhrazena(m_faktury[static_cast<size_t>(m_selected)].id())?false:true);
        m_bar->EnableTool(ID_PAGE_PRIJATA_UHRADY, idbmodel->someUhradaForFaktura(m_faktury[static_cast<size_t>(m_selected)].id()));
    }
}

DbModel::FZPSort PrijataPage::columnToFakturaSort(PrijataCtrl::PColumn column)
{
    switch(column)
    {
    case PrijataCtrl::PCOL_DOKLAD: return DbModel::FZP_DOKLAD;
    case PrijataCtrl::PCOL_POPIS: return DbModel::FZP_POPIS;
    case PrijataCtrl::PCOL_ODBERRATEL: return DbModel::FZP_PROTISTRANA;
    case PrijataCtrl::PCOL_VYSTAVENA: return DbModel::FZP_VYSTAVENA;
    case PrijataCtrl::PCOL_SPLATNA: return DbModel::FZP_SPLATNA;
    case PrijataCtrl::PCOL_CELKEM: return DbModel::FZP_CELKEM;
    case PrijataCtrl::PCOL_UHRADIT: return DbModel::FZP_UHRADIT;
    default: return DbModel::FZP_DOKLAD;
    }
}

void PrijataPage::setColumnImage(PrijataCtrl::PColumn column, int image)
{
    wxListItem item;
    item.SetMask(wxLIST_MASK_IMAGE);
    item.SetImage(image);
    m_list->SetColumn(column, item);
}

void PrijataPage::OnPrintFaktura(wxCommandEvent &/*event*/)
{
    if(m_faktury[static_cast<size_t>(m_selected)].platce())
    {
        FakturaPlatcePrint print(this, idbmodel->getFaktura(m_faktury[static_cast<size_t>(m_selected)].id()));
        //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
        print.Print();
#else
        print.Preview();
#endif
    }
    else
    {
        FakturaNeplatcePrint print(this, idbmodel->getFaktura(m_faktury[static_cast<size_t>(m_selected)].id()));
        //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
        print.Print();
#else
        print.Preview();
#endif
    }
}

void PrijataPage::OnUhrady(wxCommandEvent &/*event*/)
{
    UhradyDialog dialog(this, m_faktury[static_cast<size_t>(m_selected)].id(), -1);
    dialog.ShowModal();
}

void PrijataPage::OnFilter(wxCommandEvent &/*event*/)
{
    PrijataFilter filter(this,m_filters);
    if(filter.ShowModal() == wxID_OK)
    {
        m_filters = filter.filters();
        m_filter = false;
        for(size_t i=0; i<m_filters.GetCount(); i++)
        {
            if(m_filters[i].active())
            {
                m_filter = true;
                break;
            }
        }
        m_bar->ToggleTool(ID_PAGE_PRIJATA_FILTER, m_filter);
        m_bar->EnableTool(ID_PAGE_PRIJATA_FILTERCLEAR, m_filter);
        if(m_list->GetColumnCount() == static_cast<int>(m_filters.GetCount()))
        {
            for(size_t i=0; i<m_filters.GetCount(); i++)
            {
                m_list->SetColumnHeaderFiltered(static_cast<int>(i), m_filters[i].active());
            }
            m_list->Update();
        }
    }
    else
    {
        m_filter = false;
        m_bar->ToggleTool(ID_PAGE_PRIJATA_FILTER, false);
        m_bar->EnableTool(ID_PAGE_PRIJATA_FILTERCLEAR, false);
        for(int i=0; i<m_list->GetColumnCount(); i++)
        {
            m_list->SetColumnHeaderFiltered(i, false);
        }
        m_list->Update();
    }
    updateData(ID_MODEL_ALL);
}

void PrijataPage::OnFilterClear(wxCommandEvent &/*event*/)
{
    m_bar->ToggleTool(ID_PAGE_PRIJATA_FILTER, false);
    m_filter = false;
    m_bar->EnableTool(ID_PAGE_PRIJATA_FILTERCLEAR, false);
    for(int i=0; i<m_list->GetColumnCount(); i++)
    {
        m_list->SetColumnHeaderFiltered(i, false);
    }
    m_list->Update();
    updateData(ID_MODEL_ALL);
}

void PrijataPage::getData()
{
    if(m_filter)
    {
        m_faktury.Clear();
        dxFakturaViewArray faktury = idbmodel->prijateview();
        for(size_t i=0; i<faktury.GetCount(); i++)
        {
            if(m_filters[0].matchFilter(faktury[i].cislo()) && m_filters[1].matchFilter(faktury[i].popis()) && m_filters[2].matchFilter(faktury[i].protistrana()) && m_filters[3].matchFilter(faktury[i].vystaveni())
                    && m_filters[4].matchFilter(faktury[i].splatnost()) && m_filters[5].matchFilter(faktury[i].celkem()) && m_filters[6].matchFilter(faktury[i].celkem()-faktury[i].uhrazeno()))
            {
                m_faktury.Add(faktury[i]);
            }
        }
    }
    else
    {
        m_faktury = idbmodel->prijateview();
    }
}

void PrijataPage::refreshItems()
{
    long oldcnt = m_list->GetItemCount();
    long cnt = static_cast<long>(m_faktury.GetCount());
    if(cnt)
    {
        if(m_selected >= cnt)
        {
            m_list->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
            m_selected = 0;
        }
        if(oldcnt <= cnt) m_list->RefreshItems(0, --cnt);
        else m_list->Refresh();
    }
    else
    {
        m_list->Refresh();
    }
}


