/***************************************************************
 * Name:      skladpage.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#include <wx/wfstream.h>
#include <wx/txtstrm.h>
#include "skladpage.h"
#include "utils/dxutils.h"
#include "resources/images/add.xpm"
#include "resources/images/edit.xpm"
#include "resources/images/delete.xpm"
#include "resources/images/csv.xpm"
#include "resources/images/print.xpm"
#include "resources/images/asc.xpm"
#include "resources/images/desc.xpm"
#include "resources/images/inventura.xpm"
#include "resources/images/printdoc.xpm"
#include "resources/images/copy.xpm"
#include "resources/images/prijemka.xpm"
#include "resources/images/vydejka.xpm"
#include "resources/images/seznam.xpm"
#include "resources/images/filter.xpm"
#include "resources/images/filter_clear.xpm"
#include "gui/dialogs/zasobadialog.h"
#include "gui/dialogs/skladdialog.h"
#include "gui/dialogs/skladydialog.h"
#include "gui/dialogs/pohybzasobdialog.h"
#include "gui/dialogs/inventuradialog.h"
#include "gui/prints/knihazasobprint.h"
#include "gui/prints/kartazasobyprint.h"
#include "gui/filters/skladfilter.h"

void SkladCtrl::CreateColumns()
{
    InsertColumn(SCOL_CISLO, wxString::FromUTF8("Číslo karty"));
    InsertColumn(SCOL_NAZEV, wxString::FromUTF8("Název"));
    InsertColumn(SCOL_STAV, "Stav", wxLIST_FORMAT_RIGHT);
    InsertColumn(SCOL_HODNOTA, "Hodnota", wxLIST_FORMAT_RIGHT);
    InsertColumn(SCOL_SKLAD, "Sklad");
    setResizeColumn(SCOL_NAZEV);
    SetColumnWidth(0, 100);
    SetColumnWidth(2, 100);
    SetColumnWidth(3, 100);
    SetColumnWidth(4, 130);
}

wxString SkladCtrl::OnGetItemText(long item, long column) const
{
    return m_page->getItemText(item, column);
}

int SkladCtrl::OnGetItemColumnImage(long /*item*/, long /*column*/) const
{
    return -1;
}

wxListItemAttr *SkladCtrl::OnGetItemAttr(long /*item*/) const
{
    return nullptr;
}

IMPLEMENT_DYNAMIC_CLASS(SkladPage, wxPanel)

BEGIN_EVENT_TABLE(SkladPage, wxPanel)
    EVT_MENU(ID_PAGE_SKLAD_ADD, SkladPage::OnAddZasoba)
    EVT_MENU(ID_PAGE_SKLAD_EDIT, SkladPage::OnEditZasoba)
    EVT_MENU(ID_PAGE_SKLAD_COPY, SkladPage::OnCopyZasoba)
    EVT_MENU(ID_PAGE_SKLAD_DELETE, SkladPage::OnDeleteZasoba)
    EVT_MENU(ID_PAGE_SKLAD_PRINT, SkladPage::OnPrintKnihaZasob)
    EVT_MENU(ID_PAGE_SKLAD_EXPORT, SkladPage::OnExportKnihaZasob)
    EVT_LIST_ITEM_RIGHT_CLICK(ID_PAGE_SKLAD_LIST, SkladPage::OnItemRightClick)
    EVT_LIST_ITEM_ACTIVATED(ID_PAGE_SKLAD_LIST, SkladPage::OnItemActivated)
    EVT_LIST_ITEM_SELECTED(ID_PAGE_SKLAD_LIST, SkladPage::OnItemSelected)
    EVT_LIST_ITEM_DESELECTED(ID_PAGE_SKLAD_LIST, SkladPage::OnItemDeselected)
    EVT_LIST_COL_CLICK(ID_PAGE_SKLAD_LIST, SkladPage::OnColumnClick)
    EVT_MENU(ID_PAGE_SKLAD_ZAPRINT, SkladPage::OnPrintZasoba)
    EVT_MENU(ID_PAGE_SKLAD_INVENTURA, SkladPage::OnInventura)
    EVT_MENU(ID_PAGE_SKLAD_PRIJEM, SkladPage::OnPrijemka)
    EVT_MENU(ID_PAGE_SKLAD_VYDAJ, SkladPage::OnVydejka)
    EVT_MENU(ID_PAGE_SKLAD_SEZNAM, SkladPage::OnSklady)
    EVT_MENU(ID_PAGE_SKLAD_FILTER, SkladPage::OnFilter)
    EVT_MENU(ID_PAGE_SKLAD_FILTERCLEAR, SkladPage::OnFilterClear)
END_EVENT_TABLE()

SkladPage::SkladPage(wxWindow *parent)
    : dxPage(parent, NAV_SKLAD), m_selected(-1), m_sortCol(SkladCtrl::SCOL_DEF_SORT), m_sortAsc(true), m_filter(false)
{
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_LESS, false, 0.0));
    m_filters.Add(Filter(FA_LESS, false, 0.0));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    m_bar = new dxToolBar(this);
    m_bar->AddTool(ID_PAGE_SKLAD_PRIJEM, wxEmptyString, wxBitmap(prijemka_xpm), dxutils::GreyScaledIcon(prijemka_xpm), wxITEM_NORMAL, wxString::FromUTF8("Nová příjemka"));
    m_bar->AddTool(ID_PAGE_SKLAD_VYDAJ, wxEmptyString, wxBitmap(vydejka_xpm), dxutils::GreyScaledIcon(vydejka_xpm), wxITEM_NORMAL, wxString::FromUTF8("Nová výdejka"));
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_SKLAD_ADD, wxEmptyString, wxBitmap(add_xpm), dxutils::GreyScaledIcon(add_xpm), wxITEM_NORMAL, wxString::FromUTF8("Nová zásoba"));
    m_bar->AddTool(ID_PAGE_SKLAD_EDIT, wxEmptyString, wxBitmap(edit_xpm), dxutils::GreyScaledIcon(edit_xpm), wxITEM_NORMAL, wxString::FromUTF8("Upravit zásobu"));
    m_bar->AddTool(ID_PAGE_SKLAD_COPY, wxEmptyString, wxBitmap(copy_xpm), dxutils::GreyScaledIcon(copy_xpm), wxITEM_NORMAL, wxString::FromUTF8("Kopírovat zásobu"));
    m_bar->AddTool(ID_PAGE_SKLAD_DELETE, wxEmptyString, wxBitmap(delete_xpm), dxutils::GreyScaledIcon(delete_xpm), wxITEM_NORMAL, wxString::FromUTF8("Smazat zásobu"));
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_SKLAD_ZAPRINT, wxEmptyString, wxBitmap(printdoc_xpm), dxutils::GreyScaledIcon(printdoc_xpm), wxITEM_NORMAL, wxString::FromUTF8("Vytisknout kartu zásoby"));
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_SKLAD_INVENTURA, wxEmptyString, wxBitmap(inventura_xpm), dxutils::GreyScaledIcon(inventura_xpm), wxITEM_NORMAL, "Inventura");
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_SKLAD_PRINT, wxEmptyString, wxBitmap(print_xpm), dxutils::GreyScaledIcon(print_xpm), wxITEM_NORMAL, wxString::FromUTF8("Vytisknout knihu zásob"));
    m_bar->AddTool(ID_PAGE_SKLAD_EXPORT, wxEmptyString, wxBitmap(csv_xpm), dxutils::GreyScaledIcon(csv_xpm), wxITEM_NORMAL, wxString::FromUTF8("Exportovat knihu zásob"));
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_SKLAD_SEZNAM, wxEmptyString, wxBitmap(seznam_xpm), dxutils::GreyScaledIcon(seznam_xpm), wxITEM_NORMAL, wxString::FromUTF8("Seznam skladů"));
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_SKLAD_FILTER, wxEmptyString, wxBitmap(filter_xpm), dxutils::GreyScaledIcon(filter_xpm), wxITEM_CHECK, "Filtrovat");
    m_bar->AddTool(ID_PAGE_SKLAD_FILTERCLEAR, wxEmptyString, wxBitmap(filter_clear_xpm), dxutils::GreyScaledIcon(filter_clear_xpm), wxITEM_NORMAL, wxString::FromUTF8("Zrušit filtr"));
    m_bar->Realize();
    m_mainSizer->Add(m_bar, 0, wxALL|wxEXPAND, 5);
    wxSize imageSize(16, 16);
    m_imageList.reset(new wxImageList(imageSize.GetWidth(), imageSize.GetHeight()));
    m_imageList->Add(wxImage(asc_xpm).Scale(16, 16));
    m_imageList->Add(wxImage(desc_xpm).Scale(16, 16));
    m_list = new SkladCtrl(this, this, ID_PAGE_SKLAD_LIST);
    m_list->SetImageList(m_imageList.get(), wxIMAGE_LIST_SMALL);
    m_list->CreateColumns();
    enableTools();
    m_mainSizer->Add(m_list, 1, wxALL|wxEXPAND, 5);
    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void SkladPage::updateData(int id)
{
    if(!(id==ID_MODEL_ALL || id==ID_MODEL_ZASOBA)) return;
    long oldcnt = m_list->GetItemCount();
    setColumnImage(m_sortCol, m_sortAsc?0:1);
    getData();
    idbmodel->sortZasobyView(m_zasoby, columnToZasobaSort(m_sortCol), m_sortAsc);
    long cnt = static_cast<long>(m_zasoby.GetCount());
    m_list->SetItemCount(cnt);
    if(cnt)
    {
        if(m_selected >= cnt)
        {
            m_list->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
            m_selected = 0;
        }
        if(oldcnt <= cnt) m_list->RefreshItems(0, --cnt);
        else m_list->Refresh();
        m_bar->EnableTool(ID_PAGE_SKLAD_INVENTURA, true);
        m_bar->EnableTool(ID_PAGE_SKLAD_PRINT, true);
        m_bar->EnableTool(ID_PAGE_SKLAD_EXPORT, true);
        m_bar->EnableTool(ID_PAGE_SKLAD_FILTER, true);
        m_bar->EnableTool(ID_PAGE_SKLAD_FILTERCLEAR, m_filter?true:false);
    }
    else
    {
        m_list->Refresh();
        m_bar->EnableTool(ID_PAGE_SKLAD_PRIJEM, false);
        m_bar->EnableTool(ID_PAGE_SKLAD_VYDAJ, false);
        m_bar->EnableTool(ID_PAGE_SKLAD_EDIT, false);
        m_bar->EnableTool(ID_PAGE_SKLAD_COPY, false);
        m_bar->EnableTool(ID_PAGE_SKLAD_DELETE, false);
        m_bar->EnableTool(ID_PAGE_SKLAD_ZAPRINT, false);
        m_bar->EnableTool(ID_PAGE_SKLAD_INVENTURA, false);
        m_bar->EnableTool(ID_PAGE_SKLAD_PRINT, false);
        m_bar->EnableTool(ID_PAGE_SKLAD_EXPORT, false);
        m_bar->EnableTool(ID_PAGE_SKLAD_FILTER, m_filter?true:false);
        m_bar->EnableTool(ID_PAGE_SKLAD_FILTERCLEAR, m_filter?true:false);
    }
}

wxString SkladPage::getItemText(long item, long column) const
{
    if(item < 0 || item >= static_cast<long>(m_zasoby.GetCount()))
        return "";
    switch(column)
    {
    case SkladCtrl::SCOL_CISLO:
    {
        return m_zasoby[static_cast<size_t>(item)].cislo();
    }
    case SkladCtrl::SCOL_NAZEV:
    {
        return m_zasoby[static_cast<size_t>(item)].nazev();
    }
    case SkladCtrl::SCOL_STAV:
    {
        return wxString::Format("%.3f", m_zasoby[static_cast<size_t>(item)].pocet());
    }
    case SkladCtrl::SCOL_HODNOTA:
    {
        return wxString::Format("%.2f", m_zasoby[static_cast<size_t>(item)].celkem());
    }
    case SkladCtrl::SCOL_SKLAD:
    {
        return m_zasoby[static_cast<size_t>(item)].sklad();
    }
    default:
    {
        return "";
    }
    }
}

void SkladPage::OnAddZasoba(wxCommandEvent &/*event*/)
{
    if(!idbmodel->getSkladyCount())
    {
        SkladDialog dialog(this, -1, false);
        if(dialog.ShowModal() != wxID_OK)
        {
            return;
        }
    }
    ZasobaDialog dialog(this);
    dialog.ShowModal();
}

void SkladPage::OnEditZasoba(wxCommandEvent &/*event*/)
{
    ZasobaDialog dialog(this, m_zasoby[static_cast<size_t>(m_selected)].id());
    dialog.ShowModal();
}

void SkladPage::OnCopyZasoba(wxCommandEvent &/*event*/)
{
    ZasobaDialog dialog(this, idbmodel->copyZasoba(m_zasoby[static_cast<size_t>(m_selected)].id()), 2);
    dialog.ShowModal();
}

void SkladPage::OnDeleteZasoba(wxCommandEvent &/*event*/)
{
    wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Opravdu smazat zásobu\n%s\n%s?"), m_zasoby[static_cast<size_t>(m_selected)].nazev(), m_zasoby[static_cast<size_t>(m_selected)].cislo()),
                           "Dotaz", wxOK|wxCANCEL|wxICON_QUESTION);
    if(dialog.ShowModal() == wxID_OK)
    {
        idbmodel->deleteZasoba(m_zasoby[static_cast<size_t>(m_selected)].id());
    }
}

void SkladPage::OnPrintKnihaZasob(wxCommandEvent &/*event*/)
{
    KnihaZasobPrint print(this);
    //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
    print.Print();
#else
    print.Preview();
#endif
}

void SkladPage::OnExportKnihaZasob(wxCommandEvent &/*event*/)
{
    wxFileDialog dlg(this, wxString::FromUTF8("Exportovat knihu zásob do"), wxEmptyString, wxEmptyString, "CSV soubory (*.csv)|*.csv",
                     wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
    if(dlg.ShowModal() != wxID_OK) return;
    wxString path = dlg.GetPath();
    dxutils::correctEmptyFileExt("csv", path);
    wxString delim = ",";
    wxFileOutputStream output(path);
    wxTextOutputStream text(output);
    text << wxString::FromUTF8("Číslo") << delim << wxString::FromUTF8("Název") << delim << "Popis" << delim
         << "Stav" << delim << "Hodnota" << delim << "Sklad" << endl;
    dxZasobaArray zasoby = idbmodel->zasoby();
    for(size_t i=0; i<zasoby.GetCount(); ++i)
    {
        text << dxutils::enquotes(zasoby[i].cislo(), delim);
        text << delim;
        text << dxutils::enquotes(zasoby[i].nazev(), delim);
        text << delim;
        text << dxutils::enquotes(zasoby[i].popis(), delim);
        text << delim;
        text << dxutils::enquotes(wxString::Format("%.3f %s", zasoby[i].pocet(), zasoby[i].mj()), delim);
        text << delim;
        text << dxutils::enquotes(wxString::Format(wxString::FromUTF8("%.2f Kč"), zasoby[i].celkem()), delim);
        text << delim;
        text << dxutils::enquotes(idbmodel->getSkladNazevById(zasoby[i].skladid()), delim);
        text << endl;
    }
}

void SkladPage::OnItemRightClick(wxListEvent &event)
{
    m_selected = event.GetIndex();
    enableTools();
    wxMenu popup;
    popup.Append(ID_PAGE_SKLAD_ADD, wxString::FromUTF8("Nová zásoba"));
    if(m_selected != -1)
    {
        popup.AppendSeparator();
        popup.Append(ID_PAGE_SKLAD_EDIT, wxString::FromUTF8("Upravit zásobu"));
        popup.Append(ID_PAGE_SKLAD_COPY, wxString::FromUTF8("Kopírovat zásobu"));
        popup.Append(ID_PAGE_SKLAD_DELETE, wxString::FromUTF8("Smazat zásobu"));
        popup.AppendSeparator();
        popup.Append(ID_PAGE_SKLAD_ZAPRINT, wxString::FromUTF8("Vytisknout kartu zásoby"));
    }
    PopupMenu(&popup, event.GetPoint()+m_list->GetPosition());
}

void SkladPage::OnItemActivated(wxListEvent &event)
{
    m_selected = event.GetIndex();
    enableTools();
    ZasobaDialog dialog(this, m_zasoby[static_cast<size_t>(m_selected)].id());
    dialog.ShowModal();
}

void SkladPage::OnItemSelected(wxListEvent &event)
{
    m_selected = event.GetIndex();
    enableTools();
}

void SkladPage::OnItemDeselected(wxListEvent &/*event*/)
{
    m_selected = -1;
    enableTools();
}

void SkladPage::OnColumnClick(wxListEvent &event)
{
    if(0 > event.GetColumn() || event.GetColumn() >= SkladCtrl::SCOL_MAX) return;
    setColumnImage(m_sortCol, -1);
    m_sortCol = static_cast<SkladCtrl::SColumn>(event.GetColumn());
    m_sortAsc = !m_sortAsc;
    setColumnImage(m_sortCol, m_sortAsc?0:1);
    if(m_zasoby.GetCount()) idbmodel->sortZasobyView(m_zasoby, columnToZasobaSort(m_sortCol), m_sortAsc);
    refreshItems();
}

void SkladPage::enableTools()
{
    if(m_selected == -1)
    {
        m_bar->EnableTool(ID_PAGE_SKLAD_PRIJEM, false);
        m_bar->EnableTool(ID_PAGE_SKLAD_VYDAJ, false);
        m_bar->EnableTool(ID_PAGE_SKLAD_EDIT, false);
        m_bar->EnableTool(ID_PAGE_SKLAD_COPY, false);
        m_bar->EnableTool(ID_PAGE_SKLAD_DELETE, false);
        m_bar->EnableTool(ID_PAGE_SKLAD_ZAPRINT, false);
    }
    else
    {
        m_bar->EnableTool(ID_PAGE_SKLAD_PRIJEM, true);
        m_bar->EnableTool(ID_PAGE_SKLAD_VYDAJ, true);
        m_bar->EnableTool(ID_PAGE_SKLAD_EDIT, true);
        m_bar->EnableTool(ID_PAGE_SKLAD_COPY, true);
        m_bar->EnableTool(ID_PAGE_SKLAD_DELETE, true);
        m_bar->EnableTool(ID_PAGE_SKLAD_ZAPRINT, true);
    }
}

DbModel::ZSort SkladPage::columnToZasobaSort(SkladCtrl::SColumn column)
{
    switch(column)
    {
    case SkladCtrl::SCOL_CISLO: return DbModel::ZB_CISLO;
    case SkladCtrl::SCOL_NAZEV: return DbModel::ZB_NAZEV;
    case SkladCtrl::SCOL_STAV: return DbModel::ZB_STAV;
    case SkladCtrl::SCOL_HODNOTA: return DbModel::ZB_HODNOTA;
    case SkladCtrl::SCOL_SKLAD: return DbModel::ZB_SKLAD;
    default: return DbModel::ZB_CISLO;
    }
}

void SkladPage::setColumnImage(SkladCtrl::SColumn column, int image)
{
    wxListItem item;
    item.SetMask(wxLIST_MASK_IMAGE);
    item.SetImage(image);
    m_list->SetColumn(column, item);
}

void SkladPage::OnPrintZasoba(wxCommandEvent &/*event*/)
{
    KartaZasobyPrint print(this, idbmodel->getZasoba(m_zasoby[static_cast<size_t>(m_selected)].id()));
    //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
    print.Print();
#else
    print.Preview();
#endif
}

void SkladPage::OnInventura(wxCommandEvent &/*event*/)
{
    InventuraDialog dialog(this);
    dialog.ShowModal();
}

void SkladPage::OnPrijemka(wxCommandEvent &/*event*/)
{
    PohybzasobDialog dialog(this, true, m_zasoby[static_cast<size_t>(m_selected)].id());
    dialog.ShowModal();
}

void SkladPage::OnVydejka(wxCommandEvent &/*event*/)
{
    if(wxIsNullDouble(m_zasoby[static_cast<size_t>(m_selected)].pocet()))
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Nulová hodnota zásoby"), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        return;
    }
    PohybzasobDialog dialog(this, false, m_zasoby[static_cast<size_t>(m_selected)].id());
    dialog.ShowModal();
}

void SkladPage::OnSklady(wxCommandEvent &/*event*/)
{
    SkladyDialog dialog(this);
    dialog.ShowModal();
}

void SkladPage::OnFilter(wxCommandEvent &/*event*/)
{
    SkladFilter filter(this,m_filters);
    if(filter.ShowModal() == wxID_OK)
    {
        m_filters = filter.filters();
        m_filter = false;
        for(size_t i=0; i<m_filters.GetCount(); i++)
        {
            if(m_filters[i].active())
            {
                m_filter = true;
                break;
            }
        }
        m_bar->ToggleTool(ID_PAGE_SKLAD_FILTER, m_filter);
        m_bar->EnableTool(ID_PAGE_SKLAD_FILTERCLEAR, m_filter);
        if(m_list->GetColumnCount() == static_cast<int>(m_filters.GetCount()))
        {
            for(size_t i=0; i<m_filters.GetCount(); i++)
            {
                m_list->SetColumnHeaderFiltered(static_cast<int>(i), m_filters[i].active());
            }
            m_list->Update();
        }
    }
    else
    {
        m_filter = false;
        m_bar->ToggleTool(ID_PAGE_SKLAD_FILTER, false);
        m_bar->EnableTool(ID_PAGE_SKLAD_FILTERCLEAR, false);
        for(int i=0; i<m_list->GetColumnCount(); i++)
        {
            m_list->SetColumnHeaderFiltered(i, false);
        }
        m_list->Update();
    }
    updateData(ID_MODEL_ALL);
}

void SkladPage::OnFilterClear(wxCommandEvent &/*event*/)
{
    m_bar->ToggleTool(ID_PAGE_SKLAD_FILTER, false);
    m_filter = false;
    m_bar->EnableTool(ID_PAGE_SKLAD_FILTERCLEAR, false);
    for(int i=0; i<m_list->GetColumnCount(); i++)
    {
        m_list->SetColumnHeaderFiltered(i, false);
    }
    m_list->Update();
    updateData(ID_MODEL_ALL);
}

void SkladPage::getData()
{
    if(m_filter)
    {
        m_zasoby.Clear();
        dxZasobaViewArray zasoby = idbmodel->zasobyview();
        for(size_t i=0; i<zasoby.GetCount(); i++)
        {
            if(m_filters[0].matchFilter(zasoby[i].cislo()) && m_filters[1].matchFilter(zasoby[i].nazev()) && m_filters[2].matchFilter(zasoby[i].pocet()) && m_filters[3].matchFilter(zasoby[i].celkem())
                    && m_filters[4].matchFilter(zasoby[i].sklad()))
            {
                m_zasoby.Add(zasoby[i]);
            }
        }
    }
    else
    {
        m_zasoby = idbmodel->zasobyview();
    }
}

void SkladPage::refreshItems()
{
    long oldcnt = m_list->GetItemCount();
    long cnt = static_cast<long>(m_zasoby.GetCount());
    if(cnt)
    {
        if(m_selected >= cnt)
        {
            m_list->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
            m_selected = 0;
        }
        if(oldcnt <= cnt) m_list->RefreshItems(0, --cnt);
        else m_list->Refresh();
    }
    else
    {
        m_list->Refresh();
    }
}
