/***************************************************************
 * Name:      vydanapage.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#ifndef VYDANAPAGE_H
#define VYDANAPAGE_H

#include <wx/imaglist.h>
#include "gui/widgets/dxlistctrl.h"
#include "dxpage.h"
#include "gui/filters/filter.h"
#include "gui/widgets/dxtoolbar.h"

class VydanaPage;

class VydanaCtrl: public dxListCtrl
{
public:
    VydanaCtrl(VydanaPage *page, wxWindow *parent, const wxWindowID id)
        : dxListCtrl(parent, id), m_page(page) {}

    enum VColumn
    {
        VCOL_DOKLAD = 0,
        VCOL_POPIS,
        VCOL_ODBERRATEL,
        VCOL_VYSTAVENA,
        VCOL_SPLATNA,
        VCOL_CELKEM,
        VCOL_UHRADIT,
        VCOL_MAX, // pocet sloupcu
        VCOL_DEF_SORT = VCOL_DOKLAD
    };

    void CreateColumns();
private:
    VydanaPage *m_page;

    virtual wxString OnGetItemText(long item, long column) const;
    virtual int OnGetItemColumnImage(long item, long column) const;
    virtual wxListItemAttr *OnGetItemAttr(long item) const;
};

class VydanaPage : public dxPage
{
    DECLARE_DYNAMIC_CLASS(VydanaPage)
    DECLARE_EVENT_TABLE()
public:
    VydanaPage() : dxPage(nullptr, NAV_VYDANA), m_mainSizer(nullptr), m_list(nullptr), m_selected(-1),
    m_bar(nullptr), m_sortCol(VydanaCtrl::VCOL_DOKLAD), m_sortAsc(true), m_filter(false) {}
    explicit VydanaPage(wxWindow *parent);

    void updateData(int id) override;
    wxString getItemText(long item, long column) const;
    wxListItemAttr *getItemAttr(long item) const;
private:
    wxBoxSizer *m_mainSizer;
    VydanaCtrl *m_list;
    long m_selected;
    dxToolBar *m_bar;
    VydanaCtrl::VColumn m_sortCol;
    bool m_sortAsc;
    wxScopedPtr<wxImageList> m_imageList;
    dxFakturaViewArray m_faktury;
    bool m_filter;
    dxFilterArray m_filters;

    void OnAddFaktura(wxCommandEvent& event);
    void OnEditFaktura(wxCommandEvent& event);
    void OnCopyFaktura(wxCommandEvent& event);
    void OnDeleteFaktura(wxCommandEvent& event);
    void OnUhraditFaktura(wxCommandEvent& event);
    void OnPrintKnihaFaktur(wxCommandEvent& event);
    void OnExportKnihaFaktur(wxCommandEvent& event);
    void OnItemRightClick(wxListEvent& event);
    void OnItemActivated(wxListEvent& event);
    void OnItemSelected(wxListEvent& event);
    void OnItemDeselected(wxListEvent& event);
    void OnColumnClick(wxListEvent& event);
    void enableTools();
    DbModel::FZPSort columnToFakturaSort(VydanaCtrl::VColumn column);
    void setColumnImage(VydanaCtrl::VColumn column, int image);
    void OnPrintFaktura(wxCommandEvent& event);
    void OnEmailFaktura(wxCommandEvent& event);
    void OnSaveFaktura(wxCommandEvent& event);
    void OnUhrady(wxCommandEvent& event);
    void previewInternal();
    void saveInternal();
    void OnFilter(wxCommandEvent& event);
    void OnFilterClear(wxCommandEvent& event);
    void getData();
    void refreshItems();
    void OnIsdoc(wxCommandEvent& event);
};

#endif  /* VYDANAPAGE_H */

