/***************************************************************
 * Name:      zavazekpage.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include <wx/wfstream.h>
#include <wx/txtstrm.h>
#include "zavazekpage.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "resources/images/add.xpm"
#include "resources/images/edit.xpm"
#include "resources/images/delete.xpm"
#include "resources/images/csv.xpm"
#include "resources/images/print.xpm"
#include "resources/images/asc.xpm"
#include "resources/images/desc.xpm"
#include "resources/images/uhrada.xpm"
#include "resources/images/printdoc.xpm"
#include "resources/images/copy.xpm"
#include "resources/images/seznam.xpm"
#include "resources/images/filter.xpm"
#include "resources/images/filter_clear.xpm"
#include "gui/dialogs/zavpohdialog.h"
#include "gui/dialogs/uhradadialog.h"
#include "gui/prints/knihazavpohprint.h"
#include "gui/prints/zavpohprint.h"
#include "gui/dialogs/uhradydialog.h"
#include "gui/filters/zavazekfilter.h"

void ZavazekCtrl::CreateColumns()
{
    InsertColumn(PCOL_DOKLAD, "Doklad");
    InsertColumn(PCOL_POPIS, "Popis");
    InsertColumn(PCOL_PROTISTRANA, "Protistrana");
    InsertColumn(PCOL_VYSTAVENA, "Vystaven");
    InsertColumn(PCOL_SPLATNA, wxString::FromUTF8("Splatný"));
    InsertColumn(PCOL_CELKEM, "Celkem", wxLIST_FORMAT_RIGHT);
    InsertColumn(PCOL_UHRADIT, "Uhradit", wxLIST_FORMAT_RIGHT);
    setResizeColumn(PCOL_POPIS);
}

wxString ZavazekCtrl::OnGetItemText(long item, long column) const
{
    return m_page->getItemText(item, column);
}

int ZavazekCtrl::OnGetItemColumnImage(long /*item*/, long /*column*/) const
{
    return -1;
}

wxListItemAttr *ZavazekCtrl::OnGetItemAttr(long item) const
{
    return m_page->getItemAttr(item);
}

IMPLEMENT_DYNAMIC_CLASS(ZavazekPage, wxPanel)

BEGIN_EVENT_TABLE(ZavazekPage, wxPanel)
    EVT_MENU(ID_PAGE_ZAVAZEK_ADD, ZavazekPage::OnAddZavazek)
    EVT_MENU(ID_PAGE_ZAVAZEK_EDIT, ZavazekPage::OnEditZavazek)
    EVT_MENU(ID_PAGE_ZAVAZEK_COPY, ZavazekPage::OnCopyZavazek)
    EVT_MENU(ID_PAGE_ZAVAZEK_DELETE, ZavazekPage::OnDeleteZavazek)
    EVT_MENU(ID_PAGE_ZAVAZEK_PRINT, ZavazekPage::OnPrintKnihaZavazku)
    EVT_MENU(ID_PAGE_ZAVAZEK_EXPORT, ZavazekPage::OnExportKnihaZavazku)
    EVT_LIST_ITEM_RIGHT_CLICK(ID_PAGE_ZAVAZEK_LIST, ZavazekPage::OnItemRightClick)
    EVT_LIST_ITEM_ACTIVATED(ID_PAGE_ZAVAZEK_LIST, ZavazekPage::OnItemActivated)
    EVT_LIST_ITEM_SELECTED(ID_PAGE_ZAVAZEK_LIST, ZavazekPage::OnItemSelected)
    EVT_LIST_ITEM_DESELECTED(ID_PAGE_ZAVAZEK_LIST, ZavazekPage::OnItemDeselected)
    EVT_LIST_COL_CLICK(ID_PAGE_ZAVAZEK_LIST, ZavazekPage::OnColumnClick)
    EVT_MENU(ID_PAGE_ZAVAZEK_UHRADA, ZavazekPage::OnUhraditZavazek)
    EVT_MENU(ID_PAGE_ZAVAZEK_ZAPRINT, ZavazekPage::OnPrintZavazek)
    EVT_MENU(ID_PAGE_ZAVAZEK_UHRADY, ZavazekPage::OnUhrady)
    EVT_MENU(ID_PAGE_ZAVAZEK_FILTER, ZavazekPage::OnFilter)
    EVT_MENU(ID_PAGE_ZAVAZEK_FILTERCLEAR, ZavazekPage::OnFilterClear)
END_EVENT_TABLE()

ZavazekPage::ZavazekPage(wxWindow *parent)
    : dxPage(parent, NAV_ZAVAZEK), m_selected(-1), m_sortCol(ZavazekCtrl::PCOL_DEF_SORT), m_sortAsc(true), m_filter(false)
{
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_LESS, false, dxsettings.datum()));
    m_filters.Add(Filter(FA_LESS, false, dxsettings.datum()));
    m_filters.Add(Filter(FA_LESS, false, 0.0));
    m_filters.Add(Filter(FA_LESS, false, 0.0));
    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    m_bar = new dxToolBar(this);
    m_bar->AddTool(ID_PAGE_ZAVAZEK_ADD, wxEmptyString, wxBitmap(add_xpm), dxutils::GreyScaledIcon(add_xpm), wxITEM_NORMAL, wxString::FromUTF8("Nový závazek"));
    m_bar->AddTool(ID_PAGE_ZAVAZEK_EDIT, wxEmptyString, wxBitmap(edit_xpm), dxutils::GreyScaledIcon(edit_xpm), wxITEM_NORMAL, wxString::FromUTF8("Upravit závazek"));
    m_bar->AddTool(ID_PAGE_ZAVAZEK_COPY, wxEmptyString, wxBitmap(copy_xpm), dxutils::GreyScaledIcon(copy_xpm), wxITEM_NORMAL, wxString::FromUTF8("Kopírovat závazek"));
    m_bar->AddTool(ID_PAGE_ZAVAZEK_DELETE, wxEmptyString, wxBitmap(delete_xpm), dxutils::GreyScaledIcon(delete_xpm), wxITEM_NORMAL, wxString::FromUTF8("Smazat závazek"));
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_ZAVAZEK_UHRADA, wxEmptyString, wxBitmap(uhrada_xpm), dxutils::GreyScaledIcon(uhrada_xpm), wxITEM_NORMAL, wxString::FromUTF8("Uhradit závazek"));
    m_bar->AddTool(ID_PAGE_ZAVAZEK_UHRADY, wxEmptyString, wxBitmap(seznam_xpm), dxutils::GreyScaledIcon(seznam_xpm), wxITEM_NORMAL, wxString::FromUTF8("Zobrazit úhrady"));
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_ZAVAZEK_ZAPRINT, wxEmptyString, wxBitmap(printdoc_xpm), dxutils::GreyScaledIcon(printdoc_xpm), wxITEM_NORMAL, wxString::FromUTF8("Vytisknout závazek"));
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_ZAVAZEK_PRINT, wxEmptyString, wxBitmap(print_xpm), dxutils::GreyScaledIcon(print_xpm), wxITEM_NORMAL, wxString::FromUTF8("Vytisknout knihu závazků"));
    m_bar->AddTool(ID_PAGE_ZAVAZEK_EXPORT, wxEmptyString, wxBitmap(csv_xpm), dxutils::GreyScaledIcon(csv_xpm), wxITEM_NORMAL, wxString::FromUTF8("Exportovat knihu závazků"));
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_ZAVAZEK_FILTER, wxEmptyString, wxBitmap(filter_xpm), dxutils::GreyScaledIcon(filter_xpm), wxITEM_CHECK, "Filtrovat");
    m_bar->AddTool(ID_PAGE_ZAVAZEK_FILTERCLEAR, wxEmptyString, wxBitmap(filter_clear_xpm), dxutils::GreyScaledIcon(filter_clear_xpm), wxITEM_NORMAL, wxString::FromUTF8("Zrušit filtr"));
    m_bar->Realize();
    m_mainSizer->Add(m_bar, 0, wxALL|wxEXPAND, 5);
    wxSize imageSize(16, 16);
    m_imageList.reset(new wxImageList(imageSize.GetWidth(), imageSize.GetHeight()));
    m_imageList->Add(wxImage(asc_xpm).Scale(16, 16));
    m_imageList->Add(wxImage(desc_xpm).Scale(16, 16));
    m_list = new ZavazekCtrl(this, this, ID_PAGE_ZAVAZEK_LIST);
    m_list->SetImageList(m_imageList.get(), wxIMAGE_LIST_SMALL);
    m_list->CreateColumns();
    enableTools();
    m_mainSizer->Add(m_list, 1, wxALL|wxEXPAND, 5);
    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void ZavazekPage::updateData(int id)
{
    if(!(id==ID_MODEL_ALL || id==ID_MODEL_ZAVAZEK)) return;
    long oldcnt = m_list->GetItemCount();
    setColumnImage(m_sortCol, m_sortAsc?0:1);
    getData();
    idbmodel->sortZavPohView(m_zavazky, columnToZavPohSort(m_sortCol), m_sortAsc);
    size_t cnt = m_zavazky.GetCount();
    m_list->SetItemCount(cnt);
    if(cnt)
    {
        if((size_t)m_selected >= cnt)
        {
            m_list->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
            m_selected = 0;
        }
        m_list->RefreshItems(0, --cnt);
        m_bar->EnableTool(ID_PAGE_ZAVAZEK_PRINT, true);
        m_bar->EnableTool(ID_PAGE_ZAVAZEK_EXPORT, true);
        m_bar->EnableTool(ID_PAGE_ZAVAZEK_UHRADA, idbmodel->isZavpohUhrazena(m_zavazky[m_selected].id())?false:true);
        m_bar->EnableTool(ID_PAGE_ZAVAZEK_FILTER, true);
        m_bar->EnableTool(ID_PAGE_ZAVAZEK_FILTERCLEAR, m_filter?true:false);
    }
    else
    {
        m_list->Refresh();
        m_bar->EnableTool(ID_PAGE_ZAVAZEK_EDIT, false);
        m_bar->EnableTool(ID_PAGE_ZAVAZEK_COPY, false);
        m_bar->EnableTool(ID_PAGE_ZAVAZEK_DELETE, false);
        m_bar->EnableTool(ID_PAGE_ZAVAZEK_ZAPRINT, false);
        m_bar->EnableTool(ID_PAGE_ZAVAZEK_UHRADA, false);
        m_bar->EnableTool(ID_PAGE_ZAVAZEK_PRINT, false);
        m_bar->EnableTool(ID_PAGE_ZAVAZEK_EXPORT, false);
        m_bar->EnableTool(ID_PAGE_ZAVAZEK_FILTER, m_filter?true:false);
        m_bar->EnableTool(ID_PAGE_ZAVAZEK_FILTERCLEAR, m_filter?true:false);
    }
}

wxString ZavazekPage::getItemText(long item, long column) const
{
    if(item < 0 || item >= (long)m_zavazky.GetCount())
        return "";
    switch(column)
    {
    case ZavazekCtrl::PCOL_DOKLAD:
        return m_zavazky[item].cislo();
    case ZavazekCtrl::PCOL_POPIS:
        return m_zavazky[item].popis();
    case ZavazekCtrl::PCOL_PROTISTRANA:
        return m_zavazky[item].protistrana();
    case ZavazekCtrl::PCOL_VYSTAVENA:
        return m_zavazky[item].vystaveni().Format("%x");
    case ZavazekCtrl::PCOL_SPLATNA:
        return m_zavazky[item].splatnost().Format("%x");
    case ZavazekCtrl::PCOL_CELKEM:
        return wxString::Format("%.2f", m_zavazky[item].celkem());
    case ZavazekCtrl::PCOL_UHRADIT:
        return wxString::Format("%.2f", dxutils::isSameDouble(m_zavazky[item].celkem(), m_zavazky[item].uhrazeno())?
                                    0.0: m_zavazky[item].celkem()-m_zavazky[item].uhrazeno());
    default:
        return "";
    }
}

wxListItemAttr *ZavazekPage::getItemAttr(long item) const
{
    if(item < 0 || item >= (long)m_zavazky.GetCount())
        return NULL;
    if(dxutils::isSameDouble(m_zavazky[item].celkem(), m_zavazky[item].uhrazeno()))
        return NULL;
    else
    {
        if(m_zavazky[item].splatnost().IsLaterThan(dxsettings.datum()))
        {
            static wxListItemAttr green(wxColour(64,157,64), wxNullColour, wxNullFont);
            return &green;
        }
        else
        {
            static wxListItemAttr red(*wxRED, wxNullColour, wxNullFont);
            return &red;
        }
    }
}

void ZavazekPage::OnAddZavazek(wxCommandEvent& /*event*/)
{
    ZavpohDialog dialog(true, this);
    dialog.ShowModal();
}

void ZavazekPage::OnEditZavazek(wxCommandEvent& /*event*/)
{
    ZavpohDialog dialog(m_zavazky[m_selected].id(), this);
    dialog.ShowModal();
}

void ZavazekPage::OnCopyZavazek(wxCommandEvent& /*event*/)
{
    ZavpohDialog dialog(idbmodel->copyZavPoh(m_zavazky[m_selected].id()), this, 2);
    dialog.ShowModal();
}

void ZavazekPage::OnDeleteZavazek(wxCommandEvent& /*event*/)
{
    ZavPohView zp = m_zavazky[m_selected];
    wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Opravdu smazat závazek\n%s\n%s?"), zp.popis(), zp.cislo()),
                           "Dotaz", wxOK|wxCANCEL|wxICON_QUESTION);
    if(dialog.ShowModal() == wxID_OK)
    {
        idbmodel->deleteZavPoh(zp.id());
    }
}

void ZavazekPage::OnUhraditZavazek(wxCommandEvent &/*event*/)
{
    UhradaDialog dialog(this, idbmodel->getZavPoh(m_zavazky[m_selected].id()));
    dialog.ShowModal();
}

void ZavazekPage::OnPrintKnihaZavazku(wxCommandEvent& /*event*/)
{
    KnihaZavpohPrint print(this, true);
    //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
    print.Print();
#else
    print.Preview();
#endif
}

void ZavazekPage::OnExportKnihaZavazku(wxCommandEvent& /*event*/)
{
    wxFileDialog dlg(this, wxString::FromUTF8("Exportovat knihu závazků do"), wxEmptyString, wxEmptyString, "CSV soubory (*.csv)|*.csv",
                     wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
    if(dlg.ShowModal() != wxID_OK) return;
    wxString path = dlg.GetPath();
    dxutils::correctEmptyFileExt("csv", path);
    wxString delim = ",";
    wxFileOutputStream output(path);
    wxTextOutputStream text(output);
    text << "Doklad" << delim << "Popis" << delim << "Protistrana" << delim << "Vystaven" << delim
         << wxString::FromUTF8("Splatný") << delim << "Celkem" << delim << "Uhradit" << delim << wxString::FromUTF8("Měna") << endl;
    dxZavPohArray zavazky = idbmodel->zavazky();
    for(size_t i=0; i<zavazky.GetCount(); i++)
    {
        text << dxutils::enquotes(zavazky[i].cislo(), delim);
        text << delim;
        text << dxutils::enquotes(zavazky[i].popis(), delim);
        text << delim;
        text << dxutils::enquotes(idbmodel->getKontakt(zavazky[i].protistranaid()).jmeno(), delim);
        text << delim;
        text << zavazky[i].vystaveni().Format("%x");
        text << delim;
        text << zavazky[i].splatnost().Format("%x");
        text << delim;
        text << dxutils::enquotes(wxString::Format("%.2f", m_zavazky[i].celkem()), delim);
        text << delim;
        text << dxutils::enquotes(wxString::Format("%.2f", zavazky[i].celkem()-idbmodel->uhrazenoByFakturaId(m_zavazky[i].id())), delim);
        text << delim;
        text << idbmodel->zkratkaMena(zavazky[i].menaid());
        text << endl;
    }
}

void ZavazekPage::OnItemRightClick(wxListEvent &event)
{
    m_selected = event.GetIndex();
    enableTools();
    wxMenu popup;
    popup.Append(ID_PAGE_ZAVAZEK_ADD, wxString::FromUTF8("Nový závazek"));
    if(m_selected != -1)
    {
        popup.AppendSeparator();
        popup.Append(ID_PAGE_ZAVAZEK_EDIT, wxString::FromUTF8("Upravit závazek"));
        popup.Append(ID_PAGE_ZAVAZEK_COPY, wxString::FromUTF8("Kopírovat závazek"));
        popup.Append(ID_PAGE_ZAVAZEK_DELETE, wxString::FromUTF8("Smazat závazek"));
        if(!idbmodel->isZavpohUhrazena(m_zavazky[m_selected].id()))
        {
            popup.AppendSeparator();
            popup.Append(ID_PAGE_ZAVAZEK_UHRADA, wxString::FromUTF8("Uhradit závazek"));
        }
        if(idbmodel->someUhradaForZavpoh(m_zavazky[static_cast<size_t>(m_selected)].id()))
        {
            popup.AppendSeparator();
            popup.Append(ID_PAGE_ZAVAZEK_UHRADY, wxString::FromUTF8("Zobrazit úhrady"));
        }
        popup.AppendSeparator();
        popup.Append(ID_PAGE_ZAVAZEK_ZAPRINT, wxString::FromUTF8("Vytisknout závazek"));
    }
    PopupMenu(&popup, event.GetPoint()+m_list->GetPosition());
}

void ZavazekPage::OnItemActivated(wxListEvent &event)
{
    m_selected = event.GetIndex();
    enableTools();
    ZavpohDialog dialog(m_zavazky[static_cast<size_t>(m_selected)].id(), this);
    dialog.ShowModal();
}

void ZavazekPage::OnItemSelected(wxListEvent &event)
{
    m_selected = event.GetIndex();
    enableTools();
}

void ZavazekPage::OnItemDeselected(wxListEvent& /*event*/)
{
    m_selected = -1;
    enableTools();
}

void ZavazekPage::OnColumnClick(wxListEvent &event)
{
    if(0 > event.GetColumn() || event.GetColumn() >= ZavazekCtrl::PCOL_MAX) return;
    setColumnImage(m_sortCol, -1);
    m_sortCol = static_cast<ZavazekCtrl::PColumn>(event.GetColumn());
    m_sortAsc = !m_sortAsc;
    setColumnImage(m_sortCol, m_sortAsc?0:1);
    if(m_zavazky.GetCount()) idbmodel->sortZavPohView(m_zavazky, columnToZavPohSort(m_sortCol), m_sortAsc);
    refreshItems();
}

void ZavazekPage::enableTools()
{
    if(m_selected == -1)
    {
        m_bar->EnableTool(ID_PAGE_ZAVAZEK_EDIT, false);
        m_bar->EnableTool(ID_PAGE_ZAVAZEK_COPY, false);
        m_bar->EnableTool(ID_PAGE_ZAVAZEK_DELETE, false);
        m_bar->EnableTool(ID_PAGE_ZAVAZEK_ZAPRINT, false);
        m_bar->EnableTool(ID_PAGE_ZAVAZEK_UHRADA, false);
        m_bar->EnableTool(ID_PAGE_ZAVAZEK_UHRADY, false);
    }
    else
    {
        m_bar->EnableTool(ID_PAGE_ZAVAZEK_EDIT, true);
        m_bar->EnableTool(ID_PAGE_ZAVAZEK_COPY, true);
        m_bar->EnableTool(ID_PAGE_ZAVAZEK_DELETE, true);
        m_bar->EnableTool(ID_PAGE_ZAVAZEK_ZAPRINT, true);
        m_bar->EnableTool(ID_PAGE_ZAVAZEK_UHRADA, idbmodel->isZavpohUhrazena(m_zavazky[static_cast<size_t>(m_selected)].id())?false:true);
        m_bar->EnableTool(ID_PAGE_ZAVAZEK_UHRADY, idbmodel->someUhradaForZavpoh(m_zavazky[static_cast<size_t>(m_selected)].id()));
    }
}

DbModel::FZPSort ZavazekPage::columnToZavPohSort(ZavazekCtrl::PColumn column)
{
    switch(column)
    {
    case ZavazekCtrl::PCOL_DOKLAD: return DbModel::FZP_DOKLAD;
    case ZavazekCtrl::PCOL_POPIS: return DbModel::FZP_POPIS;
    case ZavazekCtrl::PCOL_PROTISTRANA: return DbModel::FZP_PROTISTRANA;
    case ZavazekCtrl::PCOL_VYSTAVENA: return DbModel::FZP_VYSTAVENA;
    case ZavazekCtrl::PCOL_SPLATNA: return DbModel::FZP_SPLATNA;
    case ZavazekCtrl::PCOL_CELKEM: return DbModel::FZP_CELKEM;
    case ZavazekCtrl::PCOL_UHRADIT: return DbModel::FZP_UHRADIT;
    default: return DbModel::FZP_DOKLAD;
    }
}

void ZavazekPage::setColumnImage(ZavazekCtrl::PColumn column, int image)
{
    wxListItem item;
    item.SetMask(wxLIST_MASK_IMAGE);
    item.SetImage(image);
    m_list->SetColumn(column, item);
}

void ZavazekPage::OnPrintZavazek(wxCommandEvent &/*event*/)
{
    ZavpohPrint print(this, idbmodel->getZavPoh(m_zavazky[static_cast<size_t>(m_selected)].id()));
    //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
    print.Print();
#else
    print.Preview();
#endif
}

void ZavazekPage::OnUhrady(wxCommandEvent &/*event*/)
{
    UhradyDialog dialog(this, -1, m_zavazky[static_cast<size_t>(m_selected)].id());
    dialog.ShowModal();
}

void ZavazekPage::OnFilter(wxCommandEvent &/*event*/)
{
    ZavazekFilter filter(this,m_filters);
    if(filter.ShowModal() == wxID_OK)
    {
        m_filters = filter.filters();
        m_filter = false;
        for(size_t i=0; i<m_filters.GetCount(); i++)
        {
            if(m_filters[i].active())
            {
                m_filter = true;
                break;
            }
        }
        m_bar->ToggleTool(ID_PAGE_ZAVAZEK_FILTER, m_filter);
        m_bar->EnableTool(ID_PAGE_ZAVAZEK_FILTERCLEAR, m_filter);
        if(m_list->GetColumnCount() == static_cast<int>(m_filters.GetCount()))
        {
            for(size_t i=0; i<m_filters.GetCount(); i++)
            {
                m_list->SetColumnHeaderFiltered(static_cast<int>(i), m_filters[i].active());
            }
            m_list->Update();
        }
    }
    else
    {
        m_filter = false;
        m_bar->ToggleTool(ID_PAGE_ZAVAZEK_FILTER, false);
        m_bar->EnableTool(ID_PAGE_ZAVAZEK_FILTERCLEAR, false);
        for(int i=0; i<m_list->GetColumnCount(); i++)
        {
            m_list->SetColumnHeaderFiltered(i, false);
        }
        m_list->Update();
    }
    updateData(ID_MODEL_ALL);
}

void ZavazekPage::OnFilterClear(wxCommandEvent &/*event*/)
{
    m_bar->ToggleTool(ID_PAGE_ZAVAZEK_FILTER, false);
    m_filter = false;
    m_bar->EnableTool(ID_PAGE_ZAVAZEK_FILTERCLEAR, false);
    for(int i=0; i<m_list->GetColumnCount(); i++)
    {
        m_list->SetColumnHeaderFiltered(i, false);
    }
    m_list->Update();
    updateData(ID_MODEL_ALL);
}

void ZavazekPage::getData()
{
    if(m_filter)
    {
        m_zavazky.Clear();
        dxZavPohViewArray zavazky = idbmodel->zavazkyview();
        for(size_t i=0; i<zavazky.GetCount(); i++)
        {
            if(m_filters[0].matchFilter(zavazky[i].cislo()) && m_filters[1].matchFilter(zavazky[i].popis()) && m_filters[2].matchFilter(zavazky[i].protistrana()) && m_filters[3].matchFilter(zavazky[i].vystaveni())
                    && m_filters[4].matchFilter(zavazky[i].splatnost()) && m_filters[5].matchFilter(zavazky[i].celkem()) && m_filters[6].matchFilter(zavazky[i].celkem()-zavazky[i].uhrazeno()))
            {
                m_zavazky.Add(zavazky[i]);
            }
        }
    }
    else
    {
        m_zavazky = idbmodel->zavazkyview();
    }
}

void ZavazekPage::refreshItems()
{
    long oldcnt = m_list->GetItemCount();
    long cnt = static_cast<long>(m_zavazky.GetCount());
    if(cnt)
    {
        if(m_selected >= cnt)
        {
            m_list->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
            m_selected = 0;
        }
        if(oldcnt <= cnt) m_list->RefreshItems(0, --cnt);
        else m_list->Refresh();
    }
    else
    {
        m_list->Refresh();
    }
}
