/***************************************************************
 * Name:      zavazekpage.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#ifndef ZAVAZEKPAGE_H
#define ZAVAZEKPAGE_H

#include <wx/imaglist.h>
#include "gui/widgets/dxlistctrl.h"
#include "dxpage.h"
#include "gui/filters/filter.h"
#include "gui/widgets/dxtoolbar.h"

class ZavazekPage;

class ZavazekCtrl: public dxListCtrl
{
public:
    ZavazekCtrl(ZavazekPage *page, wxWindow *parent, const wxWindowID id)
        : dxListCtrl(parent, id), m_page(page) {}

    enum PColumn
    {
        PCOL_DOKLAD = 0,
        PCOL_POPIS,
        PCOL_PROTISTRANA,
        PCOL_VYSTAVENA,
        PCOL_SPLATNA,
        PCOL_CELKEM,
        PCOL_UHRADIT,
        PCOL_MAX, // pocet sloupcu
        PCOL_DEF_SORT = PCOL_DOKLAD
    };

    void CreateColumns();
private:
    ZavazekPage *m_page;

    virtual wxString OnGetItemText(long item, long column) const;
    virtual int OnGetItemColumnImage(long item, long column) const;
    virtual wxListItemAttr *OnGetItemAttr(long item) const;
};

class ZavazekPage : public dxPage
{
    DECLARE_DYNAMIC_CLASS(ZavazekPage)
    DECLARE_EVENT_TABLE()
public:
    ZavazekPage() : dxPage(NULL, NAV_ZAVAZEK), m_mainSizer(NULL), m_list(NULL), m_selected(-1),
      m_bar(NULL), m_sortCol(ZavazekCtrl::PCOL_DOKLAD), m_sortAsc(true), m_filter(false) {}
    explicit ZavazekPage(wxWindow *parent);

    void updateData(int id);
    wxString getItemText(long item, long column) const;
    wxListItemAttr *getItemAttr(long item) const;
private:
    wxBoxSizer *m_mainSizer;
    ZavazekCtrl *m_list;
    long m_selected;
    dxToolBar *m_bar;
    ZavazekCtrl::PColumn m_sortCol;
    bool m_sortAsc;
    wxScopedPtr<wxImageList> m_imageList;
    dxZavPohViewArray m_zavazky;
    bool m_filter;
    dxFilterArray m_filters;

    void OnAddZavazek(wxCommandEvent& event);
    void OnEditZavazek(wxCommandEvent& event);
    void OnCopyZavazek(wxCommandEvent& event);
    void OnDeleteZavazek(wxCommandEvent& event);
    void OnUhraditZavazek(wxCommandEvent& event);
    void OnPrintKnihaZavazku(wxCommandEvent& event);
    void OnExportKnihaZavazku(wxCommandEvent& event);
    void OnItemRightClick(wxListEvent& event);
    void OnItemActivated(wxListEvent& event);
    void OnItemSelected(wxListEvent& event);
    void OnItemDeselected(wxListEvent& event);
    void OnColumnClick(wxListEvent& event);
    void enableTools();
    DbModel::FZPSort columnToZavPohSort(ZavazekCtrl::PColumn column);
    void setColumnImage(ZavazekCtrl::PColumn column, int image);
    void OnPrintZavazek(wxCommandEvent& event);
    void OnUhrady(wxCommandEvent& event);
    void OnFilter(wxCommandEvent& event);
    void OnFilterClear(wxCommandEvent& event);
    void getData();
    void refreshItems();
    void OnDeleteUhrady(wxCommandEvent& event);
};

#endif  /* ZAVAZEKPAGE_H */

