/***************************************************************
 * Name:      bankovnidokladprint.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include "bankovnidokladprint.h"
#include "utils/dxutils.h"

BankovniDokladPrint::BankovniDokladPrint(wxWindow *parent, const Denik &denik)
    : DocumentPrint(parent, wxString::FromUTF8("Bankovní doklad")), m_denik(denik)
{
    createDocument();
}

void BankovniDokladPrint::createDocument()
{
    int halfPage = wxRound(((double)m_pageWidth)/2.0);
    int height = 0;
    bool prijem = idbmodel->isDenikPrijem(m_denik.id());
    m_document = new Document;
    m_document->addElement(LineElement(1, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop), LineStyle(*wxBLACK, BOLD_WIDTH)));
    TextElement e = TextElement(1, wxString::Format((prijem?wxString::FromUTF8("Příjmový bankovní doklad č.%s"):
                                                            wxString::FromUTF8("Výdajový bankovní doklad č.%s")), m_denik.cislo()),
                                wxPoint(m_marginLeft,m_marginTop), m_pageWidth-m_marginLeft-m_marginRight, wxTEXT_ALIGNMENT_RIGHT,
                                TextStyle(wxFont(wxFontInfo(16).FaceName(FACENAME).Bold())));
    m_document->addElement(e);
    height += e.height();
    m_document->addElement(LineElement(1, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height)));
    wxArrayString kontakt;
    Kontakt ko = idbmodel->getKontakt(m_denik.protistranaid());
    Adresa ad = idbmodel->getAdresa(ko.isValid() ? ko.adresaid() : -1);
    kontakt.Add(prijem?wxString::FromUTF8("Přijato od"):wxString::FromUTF8("Vydáno"));
    kontakt.Add(" ");
    kontakt.Add(ko.isValid() ? ko.jmeno() : " ");
    kontakt.Add(ad.isValid() ? wxString::Format("%s %s", ad.ulice(), ad.cp()) : " ");
    kontakt.Add(ad.isValid() ? wxString::Format("%s %s", ad.psc(), ad.misto()) : " ");
    kontakt.Add(ad.isValid() ? ad.zeme() : " ");
    kontakt.Add(" ");
    kontakt.Add(ko.isValid() ? wxString::Format(wxString::FromUTF8("IČ: %s"), ko.ic()) : " ");
    kontakt.Add(ko.isValid() ? wxString::Format(wxString::FromUTF8("DIČ: %s"), ko.dic()) : " ");
    TextParagraphElement p = TextParagraphElement(1, kontakt, wxPoint(halfPage,m_marginTop+height), halfPage-m_marginRight);
    p.setStyle(m_document->style());
    m_document->addElement(p);
    int kontaktHeight = p.height();
    Firma fi = idbmodel->firma();
    ad = idbmodel->getAdresa(fi.adresaid());
    wxArrayString firma;
    firma.Add(fi.nazev());
    firma.Add(ad.isValid() ? wxString::Format("%s %s", ad.ulice(), ad.cp()) : " ");
    firma.Add(ad.isValid() ? wxString::Format("%s %s", ad.psc(), ad.misto()) : " ");
    firma.Add(ad.isValid() ? ad.zeme() : " ");
    firma.Add(" ");
    firma.Add(wxString::Format(wxString::FromUTF8("IČ: %s"), fi.ic()));
    firma.Add(wxString::Format(wxString::FromUTF8("DIČ: %s"), fi.dic()));
    p = TextParagraphElement(1, firma, wxPoint(m_marginLeft,m_marginTop+height), halfPage-m_marginLeft);
    p.setStyle(m_document->style());
    m_document->addElement(p);
    int firmaHeight = p.height();
    m_document->addElement(LineElement(1, wxPoint(halfPage,m_marginTop+height), wxPoint(halfPage,m_marginTop+height+wxMax(firmaHeight, kontaktHeight))));
    height += wxMax(firmaHeight, kontaktHeight);
    m_document->addElement(LineElement(1, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height)));
    wxArrayString text;
    Banka ba = idbmodel->getBanka(m_denik.bankaid());
    text.Add(ba.isValid()?wxString::Format(wxString::FromUTF8("Účet: %s,%s/%s"), ba.popis(), ba.cislouctu(), ba.kodbanky()):" ");
    text.Add(wxString::Format("Datum: %s", m_denik.datum().FormatDate()));
    text.Add("");
    text.Add(wxString::Format(wxString::FromUTF8("Částka: %.2f%s"), m_denik.menaid()==1?m_denik.castka():m_denik.castkacizi(),
                              idbmodel->zkratkaMena(m_denik.menaid())));
    text.Add(wxString::Format(wxString::FromUTF8("Účel platby: %s"), m_denik.popis()));
    p = TextParagraphElement(1, text, wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight);
    p.setStyle(m_document->style());
    m_document->addElement(p);
    height += p.height();
    m_document->addElement(LineElement(1, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height)));
    wxArrayString bak;
    bak.Add(" ");
    bak.Add(" ");
    bak.Add(" ");
    bak.Add(" ");
    bak.Add(" ");
    bak.Add(wxString::FromUTF8("Vystavil"));
    p = TextParagraphElement(1, bak, wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight, wxTEXT_ALIGNMENT_CENTER);
    p.setStyle(m_document->style());
    m_document->addElement(p);
    height += p.height();
    m_document->addElement(LineElement(1, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
    m_document->addElement(LineElement(1, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
    m_document->addElement(LineElement(1, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
}
