/***************************************************************
 * Name:      inventuraprint.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#include "inventuraprint.h"
#include "utils/dxutils.h"

InventuraPrint::InventuraPrint(wxWindow *parent, const Inventura &inventura)
    : DocumentPrint(parent, "Inventura"), m_inventura(inventura)
{
    m_marginLeft = 10-PAGE_OFFSET;
    m_marginRight = 10+PAGE_OFFSET;
    m_marginTop = 10-PAGE_OFFSET;
    m_marginBottom = 10;
    createDocument();
}

void InventuraPrint::createDocument()
{
    int pageHeight = m_pageHeight-m_marginTop-m_marginBottom-10;
    int height = 0;
    int page = 1;
    m_document = new Document;
    m_document->addElement(LineElement(1, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop), LineStyle(*wxBLACK, BOLD_WIDTH)));
    TextElement text = TextElement(1, wxString::Format(wxString::FromUTF8("Inventura č.%s"), m_inventura.cislo()),
                                wxPoint(m_marginLeft,m_marginTop), m_pageWidth-m_marginLeft-m_marginRight, wxTEXT_ALIGNMENT_RIGHT,
                                TextStyle(wxFont(wxFontInfo(16).FaceName(FACENAME).Bold())));
    m_document->addElement(text);
    height += text.height();
    m_document->addElement(LineElement(1, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height)));
    Firma fi = idbmodel->firma();
    Adresa ad = idbmodel->getAdresa(fi.adresaid());
    wxArrayString firma;
    firma.Add(fi.nazev());
    firma.Add(ad.isValid() ? wxString::Format("%s %s", ad.ulice(), ad.cp()) : " ");
    firma.Add(ad.isValid() ? wxString::Format("%s %s", ad.psc(), ad.misto()) : " ");
    firma.Add(ad.isValid() ? ad.zeme() : " ");
    firma.Add(" ");
    firma.Add(wxString::Format(wxString::FromUTF8("IČ: %s"), fi.ic()));
    firma.Add(wxString::Format(wxString::FromUTF8("DIČ: %s"), fi.dic()));
    TextParagraphElement paragraph = TextParagraphElement(1, firma, wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight);
    paragraph.setStyle(m_document->style());
    m_document->addElement(paragraph);
    height += paragraph.height();
    m_document->addElement(LineElement(1, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height)));
    wxArrayString texty;
    texty.Add("Popis:");
    texty.Add(m_inventura.popis());
    wxArrayInt width;
    width.Add(15);
    width.Add(m_pageWidth-m_marginLeft-m_marginRight-15);
    TableRowElement row = TableRowElement(1, texty, wxPoint(m_marginLeft,m_marginTop+height), width);
    row.setStyle(m_document->style());
    height += row.height();
    if(height > pageHeight)
    {
        height -= row.height();
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        page++;
        row.setPage(page);
        row.setPos(wxPoint(m_marginLeft,m_marginTop));
        m_document->addElement(row);
        height = row.height();
    }
    else
    {
        m_document->addElement(row);
    }
    texty.Clear();
    texty.Add("Provedl:");
    texty.Add(m_inventura.provedl());
    width.Clear();
    width.Add(20);
    width.Add(m_pageWidth-m_marginLeft-m_marginRight-20);
    row = TableRowElement(page, texty, wxPoint(m_marginLeft,m_marginTop+height), width);
    row.setStyle(m_document->style());
    height += row.height();
    if(height > pageHeight)
    {
        height -= row.height();
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        page++;
        row.setPage(page);
        row.setPos(wxPoint(m_marginLeft,m_marginTop));
        m_document->addElement(row);
        height = row.height();
    }
    else
    {
        m_document->addElement(row);
    }
    text = TextElement(page, wxString::Format("Datum: %s", m_inventura.datum().Format("%d.%m.%Y %H:%M:%S")), wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight);
    text.setStyle(m_document->style());
    height += text.height();
    if(height > pageHeight)
    {
        height -= text.height();
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        page++;
        text.setPage(page);
        text.setPos(wxPoint(m_marginLeft,m_marginTop));
        m_document->addElement(text);
        height = text.height();
    }
    else
    {
        m_document->addElement(text);
    }
    m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height)));
    texty.Clear();
    texty.Add(wxString::FromUTF8("Číslo"));
    texty.Add(wxString::FromUTF8("Zásoba"));
    texty.Add(wxString::FromUTF8("Evidovaný stav"));
    texty.Add(wxString::FromUTF8("Zjištěný stav"));
    width.Clear();
    width.Add(30);
    width.Add(m_pageWidth-m_marginLeft-m_marginRight-150);
    width.Add(60);
    width.Add(60);
    wxArrayInt alig;
    alig.Add(wxTEXT_ALIGNMENT_LEFT);
    alig.Add(wxTEXT_ALIGNMENT_LEFT);
    alig.Add(wxTEXT_ALIGNMENT_RIGHT);
    alig.Add(wxTEXT_ALIGNMENT_RIGHT);
    row = TableRowElement(page, texty, wxPoint(m_marginLeft,m_marginTop+height), width, alig);
    row.setStyle(TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Italic())));
    row.setBorder(true);
    height += row.height();
    if(height > pageHeight)
    {
        height -= row.height();
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        page++;
        row.setPage(page);
        row.setPos(wxPoint(m_marginLeft,m_marginTop));
        m_document->addElement(row);
        height = row.height();
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop), LineStyle(*wxBLACK, BOLD_WIDTH)));
    }
    else
    {
        m_document->addElement(row);
    }
    dxIvPolozkaArray polozky = idbmodel->ivpolozkyByInventuraId(m_inventura.id());
    for(size_t i=0; i < polozky.GetCount(); i++)
    {
        texty.Clear();
        Zasoba zb = idbmodel->getZasoba(polozky[i].zasobaid());
        texty.Add(zb.isValid() ? zb.cislo() : "");
        texty.Add(zb.isValid() ? zb.nazev() : "");
        texty.Add(wxString::Format("%.3f", polozky[i].evidovano()));
        texty.Add(wxString::Format("%.3f", polozky[i].zjisteno()));
        row = TableRowElement(page, texty, wxPoint(m_marginLeft,m_marginTop+height), width, alig);
        row.setStyle(m_document->style());
        row.setBorder(true);
        height += row.height();
        if(height > pageHeight)
        {
            height -= row.height();
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            page++;
            row.setPage(page);
            row.setPos(wxPoint(m_marginLeft,m_marginTop));
            m_document->addElement(row);
            height = row.height();
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop), LineStyle(*wxBLACK, BOLD_WIDTH)));
        }
        else
        {
            m_document->addElement(row);
        }
    }
    m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
    m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
    m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
    m_document->setPages(page);
}
