/***************************************************************
 * Name:      jizdaprint.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2018
 * License:   GPL3
 **************************************************************/

#include "jizdaprint.h"
#include "utils/dxutils.h"

JizdaPrint::JizdaPrint(wxWindow *parent, const Jizda &jizda)
    : DocumentPrint(parent, wxString::FromUTF8("Jízda")), m_jizda(jizda)
{
    m_marginLeft = 10-PAGE_OFFSET;
    m_marginRight = 10+PAGE_OFFSET;
    m_marginTop = 10-PAGE_OFFSET;
    m_marginBottom = 10;
    createDocument();
}

void JizdaPrint::createDocument()
{
    int height = 0;
    m_document = new Document;
    m_document->addElement(LineElement(1, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop), LineStyle(*wxBLACK, BOLD_WIDTH)));
    TextElement text = TextElement(1, wxString::Format(wxString::FromUTF8("Jízda %s"), m_jizda.datum()),
                                wxPoint(m_marginLeft,m_marginTop), m_pageWidth-m_marginLeft-m_marginRight, wxTEXT_ALIGNMENT_RIGHT,
                                TextStyle(wxFont(wxFontInfo(16).FaceName(FACENAME).Bold())));
    m_document->addElement(text);
    height += text.height();
    m_document->addElement(LineElement(1, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height)));
    Firma fi = idbmodel->firma();
    Adresa ad = idbmodel->getAdresa(fi.adresaid());
    wxArrayString firma;
    firma.Add(fi.nazev());
    firma.Add(ad.isValid() ? wxString::Format("%s %s", ad.ulice(), ad.cp()) : " ");
    firma.Add(ad.isValid() ? wxString::Format("%s %s", ad.psc(), ad.misto()) : " ");
    firma.Add(ad.isValid() ? ad.zeme() : " ");
    firma.Add(" ");
    firma.Add(wxString::Format(wxString::FromUTF8("IČ: %s"), fi.ic()));
    firma.Add(wxString::Format(wxString::FromUTF8("DIČ: %s"), fi.dic()));
    TextParagraphElement paragraph = TextParagraphElement(1, firma, wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight);
    paragraph.setStyle(m_document->style());
    m_document->addElement(paragraph);
    height += paragraph.height();
    m_document->addElement(LineElement(1, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height)));
    text = TextElement(1, wxString::Format(wxString::FromUTF8("Začátek cesty: %s"), m_jizda.startcas().Format("%x %X")), wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight);
    text.setStyle(m_document->style());
    m_document->addElement(text);
    height += text.height();
    text = TextElement(1, wxString::Format("Konec cesty: %s", m_jizda.endcas().Format("%x %X")), wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight);
    text.setStyle(m_document->style());
    m_document->addElement(text);
    height += text.height();
    text = TextElement(1, wxString::Format(wxString::FromUTF8("Počáteční stav tachometru: %d"), m_jizda.startkm()), wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight);
    text.setStyle(m_document->style());
    m_document->addElement(text);
    height += text.height();
    text = TextElement(1, wxString::Format(wxString::FromUTF8("Konečný stav tachometru: %d"), m_jizda.endkm()), wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight);
    text.setStyle(m_document->style());
    m_document->addElement(text);
    height += text.height();
    text = TextElement(1, wxString::Format("Cesta: %s", m_jizda.cesta()), wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight);
    text.setStyle(m_document->style());
    m_document->addElement(text);
    height += text.height();
    text = TextElement(1, wxString::Format(wxString::FromUTF8("Účel cesty: %s"), m_jizda.ucel()), wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight);
    text.setStyle(m_document->style());
    m_document->addElement(text);
    height += text.height();
    Vozidlo vozidlo = idbmodel->getVozidlo(m_jizda.vozidloid());
    if(vozidlo.isValid())
    {
        text = TextElement(1, wxString::Format("Vozidlo: %s %s", vozidlo.jmeno(), vozidlo.rz()), wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight);
        text.setStyle(m_document->style());
        m_document->addElement(text);
        height += text.height();
    }
    Ridic ridic = idbmodel->getRidic(m_jizda.ridicid());
    if(ridic.isValid())
    {
        text = TextElement(1, wxString::Format(wxString::FromUTF8("Řidič: %s"), ridic.jmeno()), wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight);
        text.setStyle(m_document->style());
        m_document->addElement(text);
        height += text.height();
    }
    m_document->addElement(LineElement(1, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
    m_document->addElement(LineElement(1, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
    m_document->addElement(LineElement(1, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
}
