/***************************************************************
 * Name:      kartamajetkuprint.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include "kartamajetkuprint.h"
#include "utils/dxutils.h"

KartaMajetkuPrint::KartaMajetkuPrint(wxWindow *parent, const Majetek &majetek)
    : DocumentPrint(parent, "Karta majetku"), m_majetek(majetek)
{
    createDocument();
}

void KartaMajetkuPrint::createDocument()
{
    int halfPage = wxRound(((double)m_pageWidth)/2.0);
    int pageHeight = m_pageHeight-m_marginTop-m_marginBottom-10;
    int height = 0;
    int page = 1;
    m_document = new Document;
    m_document->addElement(LineElement(1, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop), LineStyle(*wxBLACK, BOLD_WIDTH)));
    TextElement text = TextElement(1, wxString::Format(wxString::FromUTF8("Karta majetku č.%s"), m_majetek.cislo()),
                                wxPoint(m_marginLeft,m_marginTop), m_pageWidth-m_marginLeft-m_marginRight, wxTEXT_ALIGNMENT_RIGHT,
                                TextStyle(wxFont(wxFontInfo(16).FaceName(FACENAME).Bold())));
    m_document->addElement(text);
    height += text.height();
    m_document->addElement(LineElement(1, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height)));
    Firma fi = idbmodel->firma();
    Adresa ad = idbmodel->getAdresa(fi.adresaid());
    wxArrayString firma;
    firma.Add(fi.nazev());
    firma.Add(ad.isValid() ? wxString::Format("%s %s", ad.ulice(), ad.cp()) : " ");
    firma.Add(ad.isValid() ? wxString::Format("%s %s", ad.psc(), ad.misto()) : " ");
    firma.Add(ad.isValid() ? ad.zeme() : " ");
    firma.Add(" ");
    firma.Add(wxString::Format(wxString::FromUTF8("IČ: %s"), fi.ic()));
    firma.Add(wxString::Format(wxString::FromUTF8("DIČ: %s"), fi.dic()));
    TextParagraphElement paragraph = TextParagraphElement(1, firma, wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight);
    paragraph.setStyle(m_document->style());
    m_document->addElement(paragraph);
    height += paragraph.height();
    m_document->addElement(LineElement(1, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height)));
    text = TextElement(1, wxString::Format(wxString::FromUTF8("Název: %s"), m_majetek.nazev()), wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight);
    text.setStyle(m_document->style());
    m_document->addElement(text);
    height += text.height();
    wxArrayString texty;
    texty.Add("Popis:");
    texty.Add(m_majetek.popis());
    wxArrayInt width;
    width.Add(15);
    width.Add(m_pageWidth-m_marginLeft-m_marginRight-15);
    TableRowElement row = TableRowElement(1, texty, wxPoint(m_marginLeft,m_marginTop+height), width);
    row.setStyle(m_document->style());
    height += row.height();
    if(height > pageHeight)
    {
        height -= row.height();
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        page++;
        row.setPage(page);
        row.setPos(wxPoint(m_marginLeft,m_marginTop));
        m_document->addElement(row);
        height = row.height();
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop), LineStyle(*wxBLACK, BOLD_WIDTH)));
    }
    else
    {
        m_document->addElement(row);
    }
    text = TextElement(page, wxString::Format("Typ: %s", m_majetek.typAsString()), wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight);
    text.setStyle(m_document->style());
    height += text.height();
    if(height > pageHeight)
    {
        height -= text.height();
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        page++;
        text.setPage(page);
        text.setPos(wxPoint(m_marginLeft,m_marginTop));
        m_document->addElement(text);
        height = text.height();
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop), LineStyle(*wxBLACK, BOLD_WIDTH)));
    }
    else
    {
        m_document->addElement(text);
    }
    text = TextElement(page, wxString::Format("Odpis: %s", m_majetek.odpisAsString()), wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight);
    text.setStyle(m_document->style());
    height += text.height();
    if(height > pageHeight)
    {
        height -= text.height();
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        page++;
        text.setPage(page);
        text.setPos(wxPoint(m_marginLeft,m_marginTop));
        m_document->addElement(text);
        height = text.height();
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop), LineStyle(*wxBLACK, BOLD_WIDTH)));
    }
    else
    {
        m_document->addElement(text);
    }
    if(m_majetek.odpis()==1 || m_majetek.odpis()==2)
    {
        text = TextElement(page, idbmodel->hasOdpiskupina(m_majetek.skupinaid())?
                               wxString::Format(wxString::FromUTF8("Odpisová skupina: %s"), idbmodel->getOdpiskupina(m_majetek.skupinaid()).popis()):" ",
                           wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight);
        text.setStyle(m_document->style());
        height += text.height();
        if(height > pageHeight)
        {
            height -= text.height();
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            page++;
            text.setPage(page);
            text.setPos(wxPoint(m_marginLeft,m_marginTop));
            m_document->addElement(text);
            height = text.height();
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop), LineStyle(*wxBLACK, BOLD_WIDTH)));
        }
        else
        {
            m_document->addElement(text);
        }
    }
    text = TextElement(page, wxString::Format(wxString::FromUTF8("Pořízen: %s"), m_majetek.porizen().FormatDate()), wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight);
    text.setStyle(m_document->style());
    height += text.height();
    if(height > pageHeight)
    {
        height -= text.height();
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        page++;
        text.setPage(page);
        text.setPos(wxPoint(m_marginLeft,m_marginTop));
        m_document->addElement(text);
        height = text.height();
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop), LineStyle(*wxBLACK, BOLD_WIDTH)));
    }
    else
    {
        m_document->addElement(text);
    }
    text = TextElement(page, wxString::Format(wxString::FromUTF8("Pořízovací cena: %.2f"), m_majetek.cena()), wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight);
    text.setStyle(m_document->style());
    height += text.height();
    if(height > pageHeight)
    {
        height -= text.height();
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        page++;
        text.setPage(page);
        text.setPos(wxPoint(m_marginLeft,m_marginTop));
        m_document->addElement(text);
        height = text.height();
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop), LineStyle(*wxBLACK, BOLD_WIDTH)));
    }
    else
    {
        m_document->addElement(text);
    }
    if(m_majetek.vyrazen())
    {
        text = TextElement(page, wxString::Format(wxString::FromUTF8("Vyřazen: %s"), m_majetek.porizen().FormatDate()), wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight);
        text.setStyle(m_document->style());
        height += text.height();
        if(height > pageHeight)
        {
            height -= text.height();
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            page++;
            text.setPage(page);
            text.setPos(wxPoint(m_marginLeft,m_marginTop));
            m_document->addElement(text);
            height = text.height();
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop), LineStyle(*wxBLACK, BOLD_WIDTH)));
        }
        else
        {
            m_document->addElement(text);
        }
    }
    else
    {
        text = TextElement(page, wxString::Format(wxString::FromUTF8("Zůstatková cena: %.2f"), idbmodel->getZustatkovaCenaMajetku(m_majetek.id())), wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight);
        text.setStyle(m_document->style());
        height += text.height();
        if(height > pageHeight)
        {
            height -= text.height();
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            page++;
            text.setPage(page);
            text.setPos(wxPoint(m_marginLeft,m_marginTop));
            m_document->addElement(text);
            height = text.height();
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop), LineStyle(*wxBLACK, BOLD_WIDTH)));
        }
        else
        {
            m_document->addElement(text);
        }
        if(m_majetek.odpis()<1 || m_majetek.odpis()>2)
        {
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            return;
        }
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height)));
        text = TextElement(page, "Odpisy:", wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight);
        text.setStyle(m_document->style());
        height += text.height();
        if(height > pageHeight)
        {
            height -= text.height();
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            page++;
            text.setPage(page);
            text.setPos(wxPoint(m_marginLeft,m_marginTop));
            m_document->addElement(text);
            height = text.height();
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop), LineStyle(*wxBLACK, BOLD_WIDTH)));
        }
        else
        {
            m_document->addElement(text);
        }
        dxOdpisArray odpisy = idbmodel->odpisyMajetku(m_majetek.id());
        for(size_t i=0; i < odpisy.GetCount(); i++)
        {
            wxArrayString otexty;
            otexty.Add(wxString::Format("%d", odpisy[i].rok()));
            otexty.Add(wxString::Format("%.0f", odpisy[i].odpis()));
            wxArrayInt owidth;
            owidth.Add(halfPage-m_marginLeft);
            owidth.Add(halfPage-m_marginRight);
            wxArrayInt oalig;
            oalig.Add(wxTEXT_ALIGNMENT_LEFT);
            oalig.Add(wxTEXT_ALIGNMENT_RIGHT);
            row = TableRowElement(page, otexty, wxPoint(m_marginLeft,m_marginTop+height), owidth, oalig);
            row.setStyle(m_document->style());
            row.setBorder(true);
            height += row.height();
            if(height > pageHeight)
            {
                height -= row.height();
                m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
                m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
                m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
                page++;
                row.setPage(page);
                row.setPos(wxPoint(m_marginLeft,m_marginTop));
                m_document->addElement(row);
                height = row.height();
                m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop), LineStyle(*wxBLACK, BOLD_WIDTH)));
            }
            else
            {
                m_document->addElement(row);
            }
        }
    }
    m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
    m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
    m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
    m_document->setPages(page);
}
