/***************************************************************
 * Name:      simpledenikprint.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include "simpledenikprint.h"

SimpleDenikPrint::SimpleDenikPrint(wxWindow *parent)
    : TablePrint(parent, wxString::FromUTF8("Tisk - Peněžní deník"))
{
    createTable();
}

void SimpleDenikPrint::createTable()
{
    m_headerLeftText = idbmodel->firma().nazev();
    m_headerCenterText = wxString::FromUTF8("Peněžní deník");
    m_headerRightText = wxString::Format(wxString::FromUTF8("Účetní rok %d"), idbmodel->currentRok());
    m_headerLine = true;
    m_table = new Table(11);
    m_table->addCellToHeader("Datum");
    m_table->addCellToHeader("Doklad");
    m_table->addCellToHeader("Popis");
    m_table->addCellToHeader(wxString::FromUTF8("Pokladna\nPříjem"));
    m_table->addCellToHeader(wxString::FromUTF8("Pokladna\nVýdaj"));
    m_table->addCellToHeader(wxString::FromUTF8("Banka\nPříjem"));
    m_table->addCellToHeader(wxString::FromUTF8("Banka\nVýdaj"));
    m_table->addCellToHeader(wxString::FromUTF8("Příjem\nDaňový"));
    m_table->addCellToHeader(wxString::FromUTF8("Příjem\nNedaňový"));
    m_table->addCellToHeader(wxString::FromUTF8("Výdaj\nDaňový"));
    m_table->addCellToHeader(wxString::FromUTF8("Výdaj\nNedaňový"));
    double pokladnaprijem = 0.0;
    double pokladnavydaj = 0.0;
    double bankaprijem = 0.0;
    double bankavydaj = 0.0;
    double prijemdanovy = 0.0;
    double prijemnedanovy = 0.0;
    double vydajdanovy = 0.0;
    double vydajnedanovy = 0.0;
    dxDenikArray denik = idbmodel->denik();
    for(size_t i=0; i<denik.GetCount(); i++)
    {
        m_table->addRow();
        Denik de = denik[i];
        m_table->addCell(de.datum().FormatDate());
        m_table->addCell(de.cislo());
        m_table->addCell(de.popis());
        if(idbmodel->hasPokladna(de.pokladnaid()))
        {
            if(idbmodel->isPredkontacePrijem(de.predkontaceid()))
            {
                pokladnaprijem += de.castka();
                m_table->addCell(wxString::Format("%.2f", de.castka()), wxTEXT_ALIGNMENT_RIGHT);
                m_table->addCell("");
            }
            else
            {
                pokladnavydaj += de.castka();
                m_table->addCell("");
                m_table->addCell(wxString::Format("%.2f", de.castka()), wxTEXT_ALIGNMENT_RIGHT);
            }
            m_table->addCell("");
            m_table->addCell("");
        }
        else
        {
            m_table->addCell("");
            m_table->addCell("");
            if(idbmodel->isPredkontacePrijem(de.predkontaceid()))
            {
                bankaprijem += de.castka();
                m_table->addCell(wxString::Format("%.2f", de.castka()), wxTEXT_ALIGNMENT_RIGHT);
                m_table->addCell("");
            }
            else
            {
                bankavydaj += de.castka();
                m_table->addCell("");
                m_table->addCell(wxString::Format("%.2f", de.castka()), wxTEXT_ALIGNMENT_RIGHT);
            }
        }
        if(idbmodel->isPredkontacePrijem(de.predkontaceid()))
        {
            if(idbmodel->isPredkontaceDanova(de.predkontaceid()))
            {
                prijemdanovy += de.castka();
                m_table->addCell(wxString::Format("%.2f", de.castka()), wxTEXT_ALIGNMENT_RIGHT);
                m_table->addCell("");
            }
            else
            {
                prijemnedanovy += de.castka();
                m_table->addCell("");
                m_table->addCell(wxString::Format("%.2f", de.castka()), wxTEXT_ALIGNMENT_RIGHT);
            }
            m_table->addCell("");
            m_table->addCell("");
        }
        else
        {
            m_table->addCell("");
            m_table->addCell("");
            if(idbmodel->isPredkontaceDanova(de.predkontaceid()))
            {
                vydajdanovy += de.castka();
                m_table->addCell(wxString::Format("%.2f", de.castka()), wxTEXT_ALIGNMENT_RIGHT);
                m_table->addCell("");
            }
            else
            {
                vydajnedanovy += de.castka();
                m_table->addCell("");
                m_table->addCell(wxString::Format("%.2f", de.castka()), wxTEXT_ALIGNMENT_RIGHT);
            }
        }
    }
    TextStyle style(wxFont(wxFontInfo(8).FaceName(FACENAME).Bold()));
    m_table->addRow();
    m_table->addCell("");
    m_table->addCell("");
    m_table->addCell("Celkem", wxTEXT_ALIGNMENT_LEFT, style);
    m_table->addCell((pokladnaprijem ? wxString::Format("%.2f", pokladnaprijem) : ""), wxTEXT_ALIGNMENT_RIGHT, style);
    m_table->addCell((pokladnavydaj ? wxString::Format("%.2f", pokladnavydaj) : ""), wxTEXT_ALIGNMENT_RIGHT, style);
    m_table->addCell((bankaprijem ? wxString::Format("%.2f", bankaprijem) : ""), wxTEXT_ALIGNMENT_RIGHT, style);
    m_table->addCell((bankavydaj ? wxString::Format("%.2f", bankavydaj) : ""), wxTEXT_ALIGNMENT_RIGHT, style);
    m_table->addCell((prijemdanovy ? wxString::Format("%.2f", prijemdanovy) : ""), wxTEXT_ALIGNMENT_RIGHT, style);
    m_table->addCell((prijemnedanovy ? wxString::Format("%.2f", prijemnedanovy) : ""), wxTEXT_ALIGNMENT_RIGHT, style);
    m_table->addCell((vydajdanovy ? wxString::Format("%.2f", vydajdanovy) : ""), wxTEXT_ALIGNMENT_RIGHT, style);
    m_table->addCell((vydajnedanovy ? wxString::Format("%.2f", vydajnedanovy) : ""), wxTEXT_ALIGNMENT_RIGHT, style);
    m_table->setColumnWidth(2,37);
    m_table->refreshCellsSize();
}
