/***************************************************************
 * Name:      dphdp3.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#include <wx/wfstream.h>
#include <wx/sstream.h>
#include <wx/tokenzr.h>
#include "dphdp3.h"
#include "utils/dxutils.h"
#include "config.h"

Dphdp3::Dphdp3()
{
    Veta vetad(0, "VetaD", 1, 1);
    dxAtributArray avetad;
    Atribut c_okec(A_INT, "c_okec", true, 6);
    avetad.Add(c_okec);
    Atribut ctvrt(A_INT, "ctvrt", false, 1);
    avetad.Add(ctvrt);
    Atribut d_poddp(A_DATUM, "d_poddp", true, 10, 0, "%d.%m.%Y");
    avetad.Add(d_poddp);
    Atribut d_zjist(A_DATUM, "d_zjist", false, 10, 0, "%d.%m.%Y");
    avetad.Add(d_zjist);
    Atribut dapdph_forma(A_STRING, "dapdph_forma", true, 1);
    avetad.Add(dapdph_forma);
    Atribut dokument(A_STRING, "dokument", true, 3);
    avetad.Add(dokument);
    Atribut k_uladis(A_STRING, "k_uladis", true, 3);
    avetad.Add(k_uladis);
    Atribut kod_zo(A_STRING, "kod_zo", false, 3);
    avetad.Add(kod_zo);
    Atribut mesic(A_INT, "mesic", false, 2);
    avetad.Add(mesic);
    Atribut rok(A_INT, "rok", true, 4);
    avetad.Add(rok);
    Atribut trans(A_STRING, "trans", false, 1);
    avetad.Add(trans);
    Atribut typ_platce(A_STRING, "typ_platce", true, 1);
    avetad.Add(typ_platce);
    Atribut zdobd_do(A_DATUM, "zdobd_do", false, 10, 0, "%d.%m.%Y");
    avetad.Add(zdobd_do);
    Atribut zdobd_od(A_DATUM, "zdobd_od", false, 10, 0, "%d.%m.%Y");
    avetad.Add(zdobd_od);
    vetad.setAtributy(avetad);
    m_vety.Add(vetad);

    Veta vetap(1, "VetaP", 1, 1);
    dxAtributArray avetap;
    Atribut c_orient(A_STRING, "c_orient", false, 4);
    avetap.Add(c_orient);
    Atribut c_pop(A_INT, "c_pop", false, 6);
    avetap.Add(c_pop);
    Atribut c_pracufo(A_INT, "c_pracufo", false, 4);
    avetap.Add(c_pracufo);
    Atribut c_telef(A_STRING, "c_telef", false, 14);
    avetap.Add(c_telef);
    Atribut c_ufo(A_INT, "c_ufo", true, 3);
    avetap.Add(c_ufo);
    Atribut dic(A_STRING, "dic", true, 10);
    avetap.Add(dic);
    Atribut email(A_STRING, "email", false, 255);
    avetap.Add(email);
    Atribut jmeno(A_STRING, "jmeno", false, 20);
    avetap.Add(jmeno);
    Atribut naz_obce(A_STRING, "naz_obce", false, 48);
    avetap.Add(naz_obce);
    Atribut opr_jmeno(A_STRING, "opr_jmeno", false, 20);
    avetap.Add(opr_jmeno);
    Atribut opr_postaveni(A_STRING, "opr_postaveni", false, 40);
    avetap.Add(opr_postaveni);
    Atribut opr_prijmeni(A_STRING, "opr_prijmeni", false, 36);
    avetap.Add(opr_prijmeni);
    Atribut prijmeni(A_STRING, "prijmeni", false, 36);
    avetap.Add(prijmeni);
    Atribut psc(A_STRING, "psc", false, 5);
    avetap.Add(psc);
    Atribut sest_jmeno(A_STRING, "sest_jmeno", false, 20);
    avetap.Add(sest_jmeno);
    Atribut sest_prijmeni(A_STRING, "sest_prijmeni", false, 36);
    avetap.Add(sest_prijmeni);
    Atribut sest_telef(A_STRING, "sest_telef", false, 14);
    avetap.Add(sest_telef);
    Atribut stat(A_STRING, "stat", false, 25);
    avetap.Add(stat);
    Atribut titul(A_STRING, "titul", false, 10);
    avetap.Add(titul);
    Atribut typ_ds(A_STRING, "typ_ds", true, 1);
    avetap.Add(typ_ds);
    Atribut ulice(A_STRING, "ulice", false, 38);
    avetap.Add(ulice);
    Atribut zast_dat_nar(A_DATUM, "zast_dat_nar", false, 10, 0, "%d.%m.%Y");
    avetap.Add(zast_dat_nar);
    Atribut zast_ev_cislo(A_STRING, "zast_ev_cislo", false, 36);
    avetap.Add(zast_ev_cislo);
    Atribut zast_ic(A_STRING, "zast_ic", false, 10);
    avetap.Add(zast_ic);
    Atribut zast_jmeno(A_STRING, "zast_jmeno", false, 20);
    avetap.Add(zast_jmeno);
    Atribut zast_kod(A_STRING, "zast_kod", false, 2);
    avetap.Add(zast_kod);
    Atribut zast_nazev(A_STRING, "zast_nazev", false, 255);
    avetap.Add(zast_nazev);
    Atribut zast_prijmeni(A_STRING, "zast_prijmeni", false, 36);
    avetap.Add(zast_prijmeni);
    Atribut zast_typ(A_STRING, "zast_typ", false, 1);
    avetap.Add(zast_typ);
    Atribut zkrobchjm(A_STRING, "zkrobchjm", false, 255);
    avetap.Add(zkrobchjm);
    vetap.setAtributy(avetap);
    m_vety.Add(vetap);

    Veta veta1(2, "Veta1", 0, 1);
    dxAtributArray aveta1;
    Atribut dan23(A_DOUBLE, "dan23", false, 10, 2);
    aveta1.Add(dan23);
    Atribut dan5(A_DOUBLE, "dan5", false, 10, 2);
    aveta1.Add(dan5);
    Atribut dan_dzb23(A_DOUBLE, "dan_dzb23", false, 10, 2);
    aveta1.Add(dan_dzb23);
    Atribut dan_dzb5(A_DOUBLE, "dan_dzb5", false, 10, 2);
    aveta1.Add(dan_dzb5);
    Atribut dan_pdop_nrg(A_DOUBLE, "dan_pdop_nrg", false, 10, 2);
    aveta1.Add(dan_pdop_nrg);
    Atribut dan_psl23_e(A_DOUBLE, "dan_psl23_e", false, 10, 2);
    aveta1.Add(dan_psl23_e);
    Atribut dan_psl23_z(A_DOUBLE, "dan_psl23_z", false, 10, 2);
    aveta1.Add(dan_psl23_z);
    Atribut dan_psl5_e(A_DOUBLE, "dan_psl5_e", false, 10, 2);
    aveta1.Add(dan_psl5_e);
    Atribut dan_psl5_z(A_DOUBLE, "dan_psl5_z", false, 10, 2);
    aveta1.Add(dan_psl5_z);
    Atribut dan_pzb23(A_DOUBLE, "dan_pzb23", false, 10, 2);
    aveta1.Add(dan_pzb23);
    Atribut dan_pzb5(A_DOUBLE, "dan_pzb5", false, 10, 2);
    aveta1.Add(dan_pzb5);
    Atribut dan_rpren23(A_DOUBLE, "dan_rpren23", false, 10, 2);
    aveta1.Add(dan_rpren23);
    Atribut dan_rpren5(A_DOUBLE, "dan_rpren5", false, 10, 2);
    aveta1.Add(dan_rpren5);
    Atribut dov_zb23(A_DOUBLE, "dov_zb23", false, 10, 2);
    aveta1.Add(dov_zb23);
    Atribut dov_zb5(A_DOUBLE, "dov_zb5", false, 10, 2);
    aveta1.Add(dov_zb5);
    Atribut obrat23(A_DOUBLE, "obrat23", false, 10, 2);
    aveta1.Add(obrat23);
    Atribut obrat5(A_DOUBLE, "obrat5", false, 10, 2);
    aveta1.Add(obrat5);
    Atribut p_dop_nrg(A_DOUBLE, "p_dop_nrg", false, 10, 2);
    aveta1.Add(p_dop_nrg);
    Atribut p_sl23_e(A_DOUBLE, "p_sl23_e", false, 10, 2);
    aveta1.Add(p_sl23_e);
    Atribut p_sl23_z(A_DOUBLE, "p_sl23_z", false, 10, 2);
    aveta1.Add(p_sl23_z);
    Atribut p_sl5_e(A_DOUBLE, "p_sl5_e", false, 10, 2);
    aveta1.Add(p_sl5_e);
    Atribut p_sl5_z(A_DOUBLE, "p_sl5_z", false, 10, 2);
    aveta1.Add(p_sl5_z);
    Atribut p_zb23(A_DOUBLE, "p_zb23", false, 10, 2);
    aveta1.Add(p_zb23);
    Atribut p_zb5(A_DOUBLE, "p_zb5", false, 10, 2);
    aveta1.Add(p_zb5);
    Atribut rez_pren23(A_DOUBLE, "rez_pren23", false, 10, 2);
    aveta1.Add(rez_pren23);
    Atribut rez_pren5(A_DOUBLE, "rez_pren5", false, 10, 2);
    aveta1.Add(rez_pren5);
    veta1.setAtributy(aveta1);
    m_vety.Add(veta1);

    Veta veta2(3, "Veta2", 0, 1);
    dxAtributArray aveta2;
    Atribut dod_dop_nrg(A_DOUBLE, "dod_dop_nrg", false, 10, 2);
    aveta2.Add(dod_dop_nrg);
    Atribut dod_zb(A_DOUBLE, "dod_zb", false, 10, 2);
    aveta2.Add(dod_zb);
    Atribut pln_ost(A_DOUBLE, "pln_ost", false, 10, 2);
    aveta2.Add(pln_ost);
    Atribut pln_rez_pren(A_DOUBLE, "pln_rez_pren", false, 10, 2);
    aveta2.Add(pln_rez_pren);
    Atribut pln_sluzby(A_DOUBLE, "pln_sluzby", false, 10, 2);
    aveta2.Add(pln_sluzby);
    Atribut pln_vyvoz(A_DOUBLE, "pln_vyvoz", false, 10, 2);
    aveta2.Add(pln_vyvoz);
    Atribut pln_zaslani(A_DOUBLE, "pln_zaslani", false, 10, 2);
    aveta2.Add(pln_zaslani);
    veta2.setAtributy(aveta2);
    m_vety.Add(veta2);

    Veta veta3(4, "Veta3", 0, 1);
    dxAtributArray aveta3;
    Atribut dov_osv(A_DOUBLE, "dov_osv", false, 10, 2);
    aveta3.Add(dov_osv);
    Atribut opr_dluz(A_DOUBLE, "opr_dluz", false, 10, 2);
    aveta3.Add(opr_dluz);
    Atribut opr_verit(A_DOUBLE, "opr_verit", false, 10, 2);
    aveta3.Add(opr_verit);
    Atribut tri_dozb(A_DOUBLE, "tri_dozb", false, 10, 2);
    aveta3.Add(tri_dozb);
    Atribut tri_pozb(A_DOUBLE, "tri_pozb", false, 10, 2);
    aveta3.Add(tri_pozb);
    veta3.setAtributy(aveta3);
    m_vety.Add(veta3);

    Veta veta4(5, "Veta4", 0, 1);
    dxAtributArray aveta4;
    Atribut dov_cu(A_DOUBLE, "dov_cu", false, 10, 2);
    aveta4.Add(dov_cu);
    Atribut nar_maj(A_DOUBLE, "nar_maj", false, 10, 2);
    aveta4.Add(nar_maj);
    Atribut nar_zdp23(A_DOUBLE, "nar_zdp23", false, 10, 2);
    aveta4.Add(nar_zdp23);
    Atribut nar_zdp5(A_DOUBLE, "nar_zdp5", false, 10, 2);
    aveta4.Add(nar_zdp5);
    Atribut od_maj(A_DOUBLE, "od_maj", false, 10, 2);
    aveta4.Add(od_maj);
    Atribut od_zdp23(A_DOUBLE, "od_zdp23", false, 10, 2);
    aveta4.Add(od_zdp23);
    Atribut od_zdp5(A_DOUBLE, "od_zdp5", false, 10, 2);
    aveta4.Add(od_zdp5);
    Atribut odkr_maj(A_DOUBLE, "odkr_maj", false, 10, 2);
    aveta4.Add(odkr_maj);
    Atribut odkr_zdp23(A_DOUBLE, "odkr_zdp23", false, 10, 2);
    aveta4.Add(odkr_zdp23);
    Atribut odkr_zdp5(A_DOUBLE, "odkr_zdp5", false, 10, 2);
    aveta4.Add(odkr_zdp5);
    Atribut odp_cu(A_DOUBLE, "odp_cu", false, 10, 2);
    aveta4.Add(odp_cu);
    Atribut odp_cu_nar(A_DOUBLE, "odp_cu_nar", false, 10, 2);
    aveta4.Add(odp_cu_nar);
    Atribut odp_rez_nar(A_DOUBLE, "odp_rez_nar", false, 10, 2);
    aveta4.Add(odp_rez_nar);
    Atribut odp_rezim(A_DOUBLE, "odp_rezim", false, 10, 2);
    aveta4.Add(odp_rezim);
    Atribut odp_sum_kr(A_DOUBLE, "odp_sum_kr", false, 10, 2);
    aveta4.Add(odp_sum_kr);
    Atribut odp_sum_nar(A_DOUBLE, "odp_sum_nar", false, 10, 2);
    aveta4.Add(odp_sum_nar);
    Atribut odp_tuz23(A_DOUBLE, "odp_tuz23", false, 10, 2);
    aveta4.Add(odp_tuz23);
    Atribut odp_tuz23_nar(A_DOUBLE, "odp_tuz23_nar", false, 10, 2);
    aveta4.Add(odp_tuz23_nar);
    Atribut odp_tuz5(A_DOUBLE, "odp_tuz5", false, 10, 2);
    aveta4.Add(odp_tuz5);
    Atribut odp_tuz5_nar(A_DOUBLE, "odp_tuz5_nar", false, 10, 2);
    aveta4.Add(odp_tuz5_nar);
    Atribut pln23(A_DOUBLE, "pln23", false, 10, 2);
    aveta4.Add(pln23);
    Atribut pln5(A_DOUBLE, "pln5", false, 10, 2);
    aveta4.Add(pln5);
    veta4.setAtributy(aveta4);
    m_vety.Add(veta4);

    Veta veta5(6, "Veta5", 0, 1);
    dxAtributArray aveta5;
    Atribut koef_p20_nov(A_DOUBLE, "koef_p20_nov", false, 6, 2);
    aveta5.Add(koef_p20_nov);
    Atribut koef_p20_vypor(A_DOUBLE, "koef_p20_vypor", false, 6, 2);
    aveta5.Add(koef_p20_vypor);
    Atribut odp_uprav_kf(A_DOUBLE, "odp_uprav_kf", false, 10, 2);
    aveta5.Add(odp_uprav_kf);
    Atribut pln_nkf(A_DOUBLE, "pln_nkf", false, 10, 2);
    aveta5.Add(pln_nkf);
    Atribut plnosv_kf(A_DOUBLE, "plnosv_kf", false, 10, 2);
    aveta5.Add(plnosv_kf);
    Atribut plnosv_nkf(A_DOUBLE, "plnosv_nkf", false, 10, 2);
    aveta5.Add(plnosv_nkf);
    Atribut vypor_odp(A_DOUBLE, "vypor_odp", false, 10, 2);
    aveta5.Add(vypor_odp);
    veta5.setAtributy(aveta5);
    m_vety.Add(veta5);

    Veta veta6(7, "Veta6", 0, 1);
    dxAtributArray aveta6;
    Atribut dan_vrac(A_DOUBLE, "dan_vrac", false, 10, 2);
    aveta6.Add(dan_vrac);
    Atribut dan_zocelk(A_DOUBLE, "dan_zocelk", false, 10, 2);
    aveta6.Add(dan_zocelk);
    Atribut dano(A_DOUBLE, "dano", false, 10, 2);
    aveta6.Add(dano);
    Atribut dano_da(A_DOUBLE, "dano_da", false, 10, 2);
    aveta6.Add(dano_da);
    Atribut dano_no(A_DOUBLE, "dano_no", false, 10, 2);
    aveta6.Add(dano_no);
    Atribut odp_zocelk(A_DOUBLE, "odp_zocelk", false, 10, 2);
    aveta6.Add(odp_zocelk);
    Atribut uprav_odp(A_DOUBLE, "uprav_odp", false, 10, 2);
    aveta6.Add(uprav_odp);
    veta6.setAtributy(aveta6);
    m_vety.Add(veta6);
}

void Dphdp3::setAtributValue(int vetaId, const wxString &name, const wxString &value)
{
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        if(vetaId == m_vety[i].id())
        {
            m_vety[i].setAtributValue(name, value);
        }
    }
}

void Dphdp3::setAtributValue(int vetaId, const wxString &name, int value)
{
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        if(vetaId == m_vety[i].id())
        {
            m_vety[i].setAtributValue(name, value);
        }
    }
}

void Dphdp3::setAtributValue(int vetaId, const wxString &name, double value)
{
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        if(vetaId == m_vety[i].id())
        {
            m_vety[i].setAtributValue(name, value);
        }
    }
}

void Dphdp3::setAtributValue(int vetaId, const wxString &name, const wxDateTime &value)
{
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        if(vetaId == m_vety[i].id())
        {
            m_vety[i].setAtributValue(name, value);
        }
    }
}

void Dphdp3::setAtributValue(const wxString &name, const wxString &value)
{
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        m_vety[i].setAtributValue(name, value);
    }
}

void Dphdp3::setAtributValue(const wxString &name, int value)
{
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        m_vety[i].setAtributValue(name, value);
    }
}

void Dphdp3::setAtributValue(const wxString &name, double value)
{
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        m_vety[i].setAtributValue(name, value);
    }
}

void Dphdp3::setAtributValue(const wxString &name, const wxDateTime &value)
{
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        m_vety[i].setAtributValue(name, value);
    }
}

wxString Dphdp3::atributValue(const wxString &name) const
{
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        dxAtributArray atr = m_vety[i].atributy();
        for(size_t j=0; j<atr.GetCount(); j++)
        {
            if(atr[j].name() == name)
            {
                return atr[j].value();
            }
        }
    }
    return wxEmptyString;
}

int Dphdp3::atributValueInt(const wxString &name) const
{
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        dxAtributArray atr = m_vety[i].atributy();
        for(size_t j=0; j<atr.GetCount(); j++)
        {
            if(atr[j].name() == name)
            {
                return atr[j].valueInt();
            }
        }
    }
    return 0;
}

double Dphdp3::atributValueDouble(const wxString &name) const
{
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        dxAtributArray atr = m_vety[i].atributy();
        for(size_t j=0; j<atr.GetCount(); j++)
        {
            if(atr[j].name() == name)
            {
                return atr[j].valueDouble();
            }
        }
    }
    return 0.0;
}

wxDateTime Dphdp3::atributValueDatum(const wxString &name) const
{
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        dxAtributArray atr = m_vety[i].atributy();
        for(size_t j=0; j<atr.GetCount(); j++)
        {
            if(atr[j].name() == name)
            {
                return atr[j].valueDatum();
            }
        }
    }
    return wxDateTime();
}

bool Dphdp3::saveToXml(const wxString &path)
{
    wxFileOutputStream fout(path);
    if(fout.IsOk())
    {
        return createXml().Save(fout);
    }
    else
    {
        return false;
    }
}

wxString Dphdp3::saveToString()
{
    wxStringOutputStream fout;
    if(fout.IsOk())
    {
        createXml().Save(fout);
        return fout.GetString();
    }
    else
    {
        return wxEmptyString;
    }
}

bool Dphdp3::readFromXml(const wxString &path)
{
    clearData();
    if(!wxFile::Exists(path)) return false;
    wxFileInputStream fin(path);
    if(fin.IsOk())
    {
        wxXmlDocument xml;
        if(!xml.Load(fin)) return false;
        if(xml.GetRoot()->GetName() != "Pisemnost") return false;
        if(!xml.GetRoot()->GetChildren()) return false;
        if(xml.GetRoot()->GetChildren()->GetName() != "DPHDP3") return false;
        wxXmlNode *dph = xml.GetRoot()->GetChildren();
        wxXmlNode *veta = dph->GetChildren();
        while(veta)
        {
            wxXmlAttribute *atribut = veta->GetAttributes();
            while(atribut)
            {
                setAtributValue(atribut->GetName(), atribut->GetValue());
                atribut = atribut->GetNext();
            }
            veta = veta->GetNext();
        }
        return true;
    }
    else
    {
        return false;
    }
}

bool Dphdp3::readFromString(const wxString &data)
{
    clearData();
    if(data.IsEmpty()) return false;
    wxStringInputStream fin(data);
    if(fin.IsOk())
    {
        wxXmlDocument xml;
        if(!xml.Load(fin)) return false;
        if(xml.GetRoot()->GetName() != "Pisemnost") return false;
        if(!xml.GetRoot()->GetChildren()) return false;
        if(xml.GetRoot()->GetChildren()->GetName() != "DPHDP3") return false;
        wxXmlNode *dph = xml.GetRoot()->GetChildren();
        wxXmlNode *veta = dph->GetChildren();
        while(veta)
        {
            wxXmlAttribute *atribut = veta->GetAttributes();
            while(atribut)
            {
                setAtributValue(atribut->GetName(), atribut->GetValue());
                atribut = atribut->GetNext();
            }
            veta = veta->GetNext();
        }
        return true;
    }
    else
    {
        return false;
    }
}

void Dphdp3::clearData()
{
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        m_vety[i].clearData();
    }
}

dxAtributArray Dphdp3::atributy() const
{
    dxAtributArray atr;
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        dxAtributArray a = m_vety[i].atributy();
        for(size_t j=0; j<a.GetCount(); j++)
        {
            atr.Add(a[j]);
        }
    }
    return atr;
}

bool Dphdp3::isEmpty()
{
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        dxAtributArray a = m_vety[i].atributy();
        for(size_t j=0; j<a.GetCount(); j++)
        {
            if(a[j].isForXml()) return false;
        }
    }
    return true;
}

void Dphdp3::fillRadek46()
{
    double valueNar = 0.0, valueKr = 0.0;
    valueNar += atributValueDouble("odp_tuz23_nar");
    valueNar += atributValueDouble("odp_tuz5_nar");
    valueNar += atributValueDouble("odp_cu_nar");
    valueNar += atributValueDouble("od_zdp23");
    valueNar += atributValueDouble("od_zdp5");
    valueNar += atributValueDouble("odp_rez_nar");
    valueKr += atributValueDouble("odp_tuz23");
    valueKr += atributValueDouble("odp_tuz5");
    valueKr += atributValueDouble("odp_cu");
    valueKr += atributValueDouble("odkr_zdp23");
    valueKr += atributValueDouble("odkr_zdp5");
    valueKr += atributValueDouble("odp_rezim");
    if(valueNar != 0.0) setAtributValue("odp_sum_nar", valueNar);
    if(valueKr != 0.0) setAtributValue("odp_sum_kr", valueKr);
}

void Dphdp3::fillRadek52()
{
    if(atributValueDouble("odp_sum_kr") != 0.0)
    {
        setAtributValue("odp_uprav_kf", round(atributValueDouble("odp_sum_kr")*atributValueDouble("koef_p20_nov")/100.0));
    }
}

void Dphdp3::fillVeta6()
{
    double value62 = 0.0;
    value62 += atributValueDouble("dan23");
    value62 += atributValueDouble("dan5");
    value62 += atributValueDouble("dan_pzb23");
    value62 += atributValueDouble("dan_pzb5");
    value62 += atributValueDouble("dan_psl23_e");
    value62 += atributValueDouble("dan_psl5_e");
    value62 += atributValueDouble("dan_dzb23");
    value62 += atributValueDouble("dan_dzb5");
    value62 += atributValueDouble("dan_pdop_nrg");
    value62 += atributValueDouble("dan_rpren23");
    value62 += atributValueDouble("dan_rpren5");
    value62 += atributValueDouble("dan_psl23_z");
    value62 += atributValueDouble("dan_psl5_z");
    value62 -= atributValueDouble("dan_vrac");
    if(value62 != 0.0) setAtributValue("dan_zocelk", value62);
    double value63 = 0.0;
    value63 += atributValueDouble("odp_sum_nar");
    value63 += atributValueDouble("odp_uprav_kf");
    value63 += atributValueDouble("vypor_odp");
    value63 += atributValueDouble("uprav_odp");
    if(value63 != 0.0) setAtributValue("odp_zocelk", value63);
    if(value62>value63) setAtributValue("dano_da", value62-value63);
    else setAtributValue("dano_no", value63-value62);
}

ATRIBUTTYPE Dphdp3::atributType(const wxString &name) const
{
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        dxAtributArray atr = m_vety[i].atributy();
        for(size_t j=0; j<atr.GetCount(); j++)
        {
            if(atr[j].name() == name)
            {
                return atr[j].type();
            }
        }
    }
    return A_NONE;
}

void Dphdp3::fillDodatecne(const Dphdp3 &radne)
{
    fillRadek46();
    fillRadek52();
    fillVeta6();
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        if(m_vety[i].name() != "VetaD" && m_vety[i].name() != "VetaP")
        {
            dxAtributArray atr = m_vety[i].atributy();
            for(size_t j=0; j<atr.GetCount(); j++)
            {
                if(atr[j].name() == "koef_p20_nov") continue;
                if(radne.atributValueDouble(atr[j].name()) != 0.0)
                {
                    if((atr[j].valueDouble()-radne.atributValueDouble(atr[j].name())) != 0.0) m_vety[i].setAtributValue(atr[j].name(), atr[j].valueDouble()-radne.atributValueDouble(atr[j].name()));
                    else m_vety[i].clearAtributValue(atr[j].name());
                }
            }
        }
    }
    setAtributValue("dano_da", wxEmptyString);
    setAtributValue("dano_no", wxEmptyString);
    setAtributValue("dano", atributValueDouble("dan_zocelk")-atributValueDouble("odp_zocelk"));
}

void Dphdp3::addVetaR(const wxString &duvod)
{
    wxStringTokenizer tok(duvod, "\r\n");
    int poradi = 1;
    while(tok.HasMoreTokens())
    {
        wxString line = tok.NextToken();
        if(line.Length() < 73)
        {
            Veta vetaR(static_cast<int>(m_vety.GetCount()), "VetaR", 0, 999);
            dxAtributArray arr;
            Atribut kod_sekce(A_STRING, "kod_sekce", true, 1);
            kod_sekce.setValue("D");
            arr.Add(kod_sekce);;
            Atribut aporadi(A_INT, "poradi", true , 3);
            aporadi.setValue(poradi);
            arr.Add(aporadi);
            Atribut t_prilohy(A_STRING, "t_prilohy", false, 72);
            t_prilohy.setValue(line);
            arr.Add(t_prilohy);
            vetaR.setAtributy(arr);
            m_vety.Add(vetaR);
            poradi++;
        }
        else
        {
            size_t len = 0;
            for(size_t i=0; i<line.Length(); i++)
            {
                if(len == 72)
                {
                    Veta vetaR(static_cast<int>(m_vety.GetCount()), "VetaR", 0, 999);
                    dxAtributArray arr;
                    Atribut kod_sekce(A_STRING, "kod_sekce", true, 1);
                    kod_sekce.setValue("D");
                    arr.Add(kod_sekce);;
                    Atribut aporadi(A_INT, "poradi", true , 3);
                    aporadi.setValue(poradi);
                    arr.Add(aporadi);
                    Atribut t_prilohy(A_STRING, "t_prilohy", false, 72);
                    t_prilohy.setValue(line.Left(len));
                    line = line.Mid(len);
                    arr.Add(t_prilohy);
                    vetaR.setAtributy(arr);
                    m_vety.Add(vetaR);
                    poradi++;
                    len = 0;
                    i = 0;
                }
                len++;
            }
            if(line.Length())
            {
                Veta vetaR(static_cast<int>(m_vety.GetCount()), "VetaR", 0, 999);
                dxAtributArray arr;
                Atribut kod_sekce(A_STRING, "kod_sekce", true, 1);
                kod_sekce.setValue("D");
                arr.Add(kod_sekce);;
                Atribut aporadi(A_INT, "poradi", true , 3);
                aporadi.setValue(poradi);
                arr.Add(aporadi);
                Atribut t_prilohy(A_STRING, "t_prilohy", false, 72);
                t_prilohy.setValue(line);
                arr.Add(t_prilohy);
                vetaR.setAtributy(arr);
                m_vety.Add(vetaR);
                poradi++;
            }
        }
    }
}

wxXmlDocument Dphdp3::createXml()
{
    wxXmlDocument xml;
    wxXmlNode *rootNode = new wxXmlNode(wxXML_ELEMENT_NODE, "Pisemnost");
    rootNode->AddAttribute("nazevSW", PACKAGE);
    rootNode->AddAttribute("verzeSW", VERSION);
    wxXmlNode *dphNode = new wxXmlNode(wxXML_ELEMENT_NODE, "DPHDP3");
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        if(m_vety[i].isForXml())
        {
            dphNode->AddChild(m_vety[i].createXmlNode());
        }
    }
    rootNode->AddChild(dphNode);
    xml.SetRoot(rootNode);
    return xml;
}
