/***************************************************************
 * Name:      dphdp3wizard.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#ifndef DPHDP3WIZARD_H
#define DPHDP3WIZARD_H

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif
#include <wx/wizard.h>
#include <wx/datectrl.h>
#include <wx/dateevt.h>
#include <wx/spinctrl.h>
#include "dphdp3.h"
#include "model/dbmodel.h"

class Dphdp3Wizard;

class Dp3Page1 : public wxWizardPageSimple
{
    DECLARE_EVENT_TABLE()
public:
    explicit Dp3Page1(wxWizard *parent);

private:
    wxBoxSizer *m_mainSizer;
    wxButton *m_priznani;

    void OnPriznani(wxCommandEvent &event);
};

class Dp3Page2 : public wxWizardPageSimple
{
    DECLARE_EVENT_TABLE()
public:
    explicit Dp3Page2(wxWizard *parent);

    wxDateTime vyhotoveni() const;
    int obdobi() const; //0 - mesic, 1 - ctvrtleti, 2 - oddo
    int mesic() const;
    int ctvrtleti() const;
    int rok() const;
    wxDateTime start() const;
    wxDateTime end() const;
    wxString kodZo() const;
    double koefP20Nov() const;
    double koefP20Vypor() const;
    double vyporOdp() const;
private:
    Dphdp3Wizard *m_wizard;
    wxBoxSizer *m_mainSizer;
    wxRadioButton *m_mesicBtn;
    wxChoice *m_mesicSelect;
    wxChoice *m_mesicRok;
    wxRadioButton *m_ctvrtletiBtn;
    wxChoice *m_ctvrtletiSelect;
    wxChoice *m_ctvrtletiRok;
    wxRadioButton *m_oddoBtn;
    wxDatePickerCtrl *m_od;
    wxDatePickerCtrl *m_do;
    wxDatePickerCtrl *m_vyhotoveni;
    wxSpinCtrlDouble *m_koefP20Nov;
    wxChoice *m_kodZo;
    wxButton *m_koefP20Vypor;
    double m_koefP20VyporValue;
    double m_vyporOdp;

    void OnMesic(wxCommandEvent& event);
    void OnCtvrtleti(wxCommandEvent& event);
    void OnOddo(wxCommandEvent& event);
    void OnOd(wxDateEvent& event);
    void OnDo(wxDateEvent& event);
    void OnMesicChoice(wxCommandEvent& event);
    void OnCtvrtletiChoice(wxCommandEvent& event);
    void OnKoefP20Vypor(wxCommandEvent& event);
};

class Dp3Page3 : public wxWizardPageSimple
{
    DECLARE_EVENT_TABLE()
public:
    explicit Dp3Page3(wxWizard *parent);

    wxString forma() const;
    wxDateTime start() const;
    wxDateTime end() const;
    int ulozeneID();
    wxDateTime dzjist() const;
    wxString duvod() const;
private:
    Dphdp3Wizard *m_wizard;
    wxBoxSizer *m_mainSizer;
    wxRadioButton *m_radne;
    wxRadioButton *m_radneOpravne;
    wxRadioButton *m_dodatecne;
    wxRadioButton *m_dodatecneOpravne;
    wxChoice *m_ulozene;
    wxDatePickerCtrl *m_dzjist;
    wxTextCtrl *m_duvod;

    void OnRadne(wxCommandEvent& event);
    void OnDodatecne(wxCommandEvent& event);
};

class Dp3Page4 : public wxWizardPageSimple
{
    DECLARE_EVENT_TABLE()
public:
    explicit Dp3Page4(wxWizard *parent);

private:
    Dphdp3Wizard *m_wizard;
    wxBoxSizer *m_mainSizer;
    wxButton *m_print;
    wxButton *m_export;

    void OnPrint(wxCommandEvent& event);
    void OnExport(wxCommandEvent& event);
};

class Dphdp3Wizard : public wxWizard
{
    DECLARE_DYNAMIC_CLASS(Dphdp3Wizard)
    DECLARE_EVENT_TABLE()
public:
    Dphdp3Wizard() : m_page1(nullptr), m_page2(nullptr), m_page3(nullptr), m_page4(nullptr) {}
    explicit Dphdp3Wizard(wxWindow *parent);

    wxWizardPage *GetFirstPage() const { return m_page1; }
    void fillData();
    dxAtributArray atributy() const;
    void saveXml(const wxString &path);
    int rok() const;
    void fillDataForKoefP20Vypor();
    Dphdp3 dphdp3() const;
    void setDphdp3(const Dphdp3 &dphdp3);
private:
    Dp3Page1 *m_page1;
    Dp3Page2 *m_page2;
    Dp3Page3 *m_page3;
    Dp3Page4 *m_page4;
    Dphdp3 m_dphdp3;

    void OnPageChanged(wxWizardEvent& event);
    void OnFinished(wxWizardEvent& event);
    void fillVeta1to6(const wxDateTime &start, const wxDateTime &end);
    void fillVeta1to6Summary(const wxDateTime &start, const wxDateTime &end);
    void fillVeta1(const wxArrayString &radky, const wxString &obrat23Name, const wxString &dan23Name, const wxString &obrat5Name, const wxString &dan5Name, const wxDateTime &start, const wxDateTime &end);
    void fillVeta2(const wxString &radky, const wxString &obratName, const wxDateTime &start, const wxDateTime &end);
    void fillVeta3(const wxString &radky, const wxString &obratName, const wxDateTime &start, const wxDateTime &end, bool obrat);
    void fillVeta4(const wxArrayString &radky, const wxString &obrat23Name, const wxString &dan23Name, const wxString &krac23Name, const wxString &obrat5Name, const wxString &dan5Name, const wxString &krac5Name,
                   const wxDateTime &start, const wxDateTime &end);
    void fillVeta4Radek42(const wxArrayString &radky, const wxString &obrat23Name, const wxString &dan23Name, const wxString &krac23Name, const wxDateTime &start, const wxDateTime &end);
    void fillRadek50(const wxDateTime &start, const wxDateTime &end);
};

#endif  /* DPHDP3WIZARD_H */

