/***************************************************************
 * Name:      dphkh1print.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#include <wx/stdpaths.h>
#include <wx/filename.h>
#include "dphkh1print.h"
#include "utils/dxutils.h"

Dphkh1Print::Dphkh1Print(wxWindow *parent, dxVetaArray vety)
    : DocumentPrint(parent, wxString::FromUTF8("Kontrolní hlášení DPH")), m_vety(vety)
{
    fillNewXMLValue();
    m_marginLeft = 10-PAGE_OFFSET;
    m_marginRight = 10+PAGE_OFFSET;
    m_marginTop = 10-PAGE_OFFSET;
    m_marginBottom = 10;
    m_pageHeight = 210;
    m_pageWidth = 297;
    setHeaderCenterText(wxString::FromUTF8("Kontrolní tisk, neslouží k podání"));
    m_paperOrientation = wxLANDSCAPE;
    createDocument();
}

void Dphkh1Print::Preview(bool print)
{
    DocumentPrintout *print1 = new DocumentPrintout(printTitle());
    print1->setDocument(m_document);
    print1->setPaperOrientation(m_paperOrientation);
    print1->setHeaderLeftText(m_headerLeftText);
    print1->setHeaderCenterText(m_headerCenterText);
    print1->setHeaderRightText(m_headerRightText);
    print1->setHeaderLine(m_headerLine);
    print1->setMarginLeft(m_marginLeft);
    print1->setMarginRight(m_marginRight);
    print1->setMarginTop(m_marginTop);
    print1->setMarginBottom(m_marginBottom);
    print1->setPages(m_document->pages());
    print1->setFooter(dxPrint::F_NUMBERING);
    print1->performPageSetup();
    wxPrintDialogData data(print1->printData());
    data.SetMinPage(1);
    data.SetMaxPage(m_document->pages());
    data.SetFromPage(1);
    data.SetToPage(m_document->pages());
    wxPrintPreview *preview = nullptr;
    if(print)
    {
        DocumentPrintout *print2 = new DocumentPrintout(printTitle());
        print2->setDocument(m_document);
        print2->setPaperOrientation(m_paperOrientation);
        print2->setHeaderLeftText(m_headerLeftText);
        print2->setHeaderCenterText(m_headerCenterText);
        print2->setHeaderRightText(m_headerRightText);
        print2->setHeaderLine(m_headerLine);
        print2->setMarginLeft(m_marginLeft);
        print2->setMarginRight(m_marginRight);
        print2->setMarginTop(m_marginTop);
        print2->setMarginBottom(m_marginBottom);
        print2->setPages(m_document->pages());
        print2->setFooter(dxPrint::F_NUMBERING);
        print2->performPageSetup();
        preview = new wxPrintPreview(print1, print2, &data);
    }
    else
    {
        preview = new wxPrintPreview(print1, nullptr, &data);
    }
    if (!preview->IsOk())
    {
        delete preview;
        wxMessageBox("There was a problem with previewing.\nPerhaps your current printer is not set correctly?", "Previewing", wxOK);
        return;
    }
    print1->GetPageSizeMM(&m_pageWidth, &m_pageHeight);
    preview->SetZoom(PREVIEW_ZOOM);
    wxPreviewFrame *frame = new wxPreviewFrame(preview, m_parent, m_title, wxDefaultPosition, wxSize(PREVIEW_WIDTHL,PREVIEW_HEIGHT));
    frame->Centre(wxBOTH);
    frame->Initialize();
    frame->Show();
}

void Dphkh1Print::createDocument()
{
    int ml = m_marginLeft, mt = m_marginTop, mr = m_marginRight;
    int pageHeight = m_pageHeight - m_marginTop - m_marginBottom - 5;
    int page = 1;
    m_document = new Document;
    m_document->setStyle(TextStyle(wxFont(wxFontInfo(8).FaceName(FACENAME))));
    TextElement text = TextElement(1, wxString::FromUTF8("Finančnímu úřadu pro / Specializovanému finančnímu úřadu"), wxPoint(ml,mt+5), 150);
    m_document->addElement(text);
    RectangleElement rect(1, wxPoint(ml+1,mt+9), wxSize(80,6), LineStyle(), 1);
    m_document->addElement(rect);
    wxString c_ufo = atributValue("c_ufo");
    wxString nazu_ufo = "";
    wxXmlDocument ufo;
    ufo.Load(wxStandardPaths::Get().GetUserDataDir()+"/ufo.xml");
    if(ufo.IsOk() && ufo.GetRoot()->GetName() == "Ciselnik")
    {
        wxXmlNode *child = ufo.GetRoot()->GetChildren();
        while(child)
        {
            if(c_ufo == child->GetAttribute("c_ufo"))
            {
                nazu_ufo = child->GetAttribute("nazu_ufo");
                break;
            }
            child = child->GetNext();
        }
    }
    text = TextElement(1, nazu_ufo, wxPoint(ml,mt+10), 80, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("Územní pracoviště v, ve, pro"), wxPoint(ml,mt+15), 150);
    m_document->addElement(text);
    rect = RectangleElement(1, wxPoint(ml+1,mt+19), wxSize(80,6), LineStyle(), 1);
    m_document->addElement(rect);
    wxString c_pracufo = atributValue("c_pracufo");
    wxString nazu_pracufo = "";
    wxXmlDocument pracufo;
    pracufo.Load(wxStandardPaths::Get().GetUserDataDir()+"/pracufo.xml");
    if(pracufo.IsOk() && pracufo.GetRoot()->GetName() == "Ciselnik")
    {
        wxXmlNode *child = pracufo.GetRoot()->GetChildren();
        while(child)
        {
            if(c_pracufo == child->GetAttribute("c_pracufo"))
            {
                nazu_pracufo = child->GetAttribute("nazu_pracufo");
                break;
            }
            child = child->GetNext();
        }
    }
    text = TextElement(1, nazu_pracufo, wxPoint(ml,mt+20), 80, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("Daňové identifikační číslo"), wxPoint(ml,mt+25), 150);
    m_document->addElement(text);
    rect = RectangleElement(1, wxPoint(ml+1,mt+29), wxSize(60,6), LineStyle(), 1);
    m_document->addElement(rect);
    text = TextElement(1, "CZ"+atributValue("dic"), wxPoint(ml,mt+30), 60, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("Druh kontrolního hlášení"), wxPoint(ml,mt+35), 150);
    m_document->addElement(text);
    rect = RectangleElement(1, wxPoint(ml+1,mt+39), wxSize(80,6), LineStyle(), 1);
    m_document->addElement(rect);
    wxString forma = "";
    if(atributValue("khdph_forma") == "B") forma = wxString::FromUTF8("řádné");
    if(atributValue("khdph_forma") == "O") forma = wxString::FromUTF8("řádné/opravné");
    if(atributValue("khdph_forma") == "N") forma = wxString::FromUTF8("následné");
    if(atributValue("khdph_forma") == "E") forma = wxString::FromUTF8("následné/opravné");
    text = TextElement(1, forma, wxPoint(ml,mt+40), 80, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("Důvody pro podání následného kontrolního hlášení zjištěny dne"), wxPoint(ml,mt+45), 150);
    m_document->addElement(text);
    rect = RectangleElement(1, wxPoint(ml+1,mt+49), wxSize(50,6), LineStyle(), 1);
    m_document->addElement(rect);
    text = TextElement(1, atributValue("d_zjist"), wxPoint(ml,mt+50), 50, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("Číslo jednací výzvy"), wxPoint(ml,mt+57), 80);
    m_document->addElement(text);
    rect = RectangleElement(1, wxPoint(ml+32,mt+56), wxSize(70,6), LineStyle(), 1);
    m_document->addElement(rect);
    text = TextElement(1, atributValue("c_jed_vyzvy"), wxPoint(ml+31,mt+57), 50, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("Rychlá odpověď na výzvu"), wxPoint(ml+105,mt+57), 80);
    m_document->addElement(text);
    rect = RectangleElement(1, wxPoint(ml+145,mt+56), wxSize(100,6), LineStyle(), 1);
    m_document->addElement(rect);
    wxString vyzva = "";
    if(atributValue("vyzva_odp") == "B") vyzva = wxString::FromUTF8("Nemám povinnost podat KH (nulové KH)");
    if(atributValue("vyzva_odp") == "P") vyzva = wxString::FromUTF8("Potvrzuji správnost naposledy podaného KH");
    text = TextElement(1, vyzva, wxPoint(ml+144,mt+57), 100, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("KONTROLNÍ HLÁŠENÍ"), wxPoint(ml,mt+63), m_pageWidth-ml-mr, wxTEXT_ALIGNMENT_CENTER, TextStyle(wxFont(wxFontInfo(22).FaceName(FACENAME).Bold())));
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("podle § 101c a násl. zákona č. 235/2004 Sb., o dani z přidané hodnoty,"), wxPoint(ml,mt+72), m_pageWidth-ml-mr, wxTEXT_ALIGNMENT_CENTER, TextStyle(wxFont(wxFontInfo(14).FaceName(FACENAME).Bold())));
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("ve znění pozdějších předpisů (ZDPH)"), wxPoint(ml,mt+78), m_pageWidth-ml-mr, wxTEXT_ALIGNMENT_CENTER, TextStyle(wxFont(wxFontInfo(14).FaceName(FACENAME).Bold())));
    m_document->addElement(text);
    rect = RectangleElement(1, wxPoint(ml+50,mt+85), wxSize(23,6), LineStyle(), 1);
    m_document->addElement(rect);
    text = TextElement(1, atributValue("mesic"), wxPoint(ml+50,mt+86), 23, wxTEXT_ALIGNMENT_CENTER, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    rect = RectangleElement(1, wxPoint(ml+115,mt+85), wxSize(23,6), LineStyle(), 1);
    m_document->addElement(rect);
    text = TextElement(1, atributValue("ctvrt"), wxPoint(ml+115,mt+86), 23, wxTEXT_ALIGNMENT_CENTER, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    rect = RectangleElement(1, wxPoint(ml+147,mt+85), wxSize(20,6), LineStyle(), 1);
    m_document->addElement(rect);
    text = TextElement(1, atributValue("rok"), wxPoint(ml+147,mt+86), 20, wxTEXT_ALIGNMENT_CENTER, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    text = TextElement(1, "rok", wxPoint(ml+132,mt+86), 14, wxTEXT_ALIGNMENT_RIGHT, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("za kalendářní čtvrtletí"), wxPoint(ml+70,mt+86), 44, wxTEXT_ALIGNMENT_RIGHT, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("za kalendářní měsíc"), wxPoint(ml+4,mt+86), 44, wxTEXT_ALIGNMENT_RIGHT, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    rect = RectangleElement(1, wxPoint(ml+50,mt+92), wxSize(23,6), LineStyle(), 1);
    m_document->addElement(rect);
    text = TextElement(1, atributValue("zdobd_od"), wxPoint(ml+50,mt+93), 23, wxTEXT_ALIGNMENT_CENTER, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    rect = RectangleElement(1, wxPoint(ml+115,mt+92), wxSize(23,6), LineStyle(), 1);
    m_document->addElement(rect);
    text = TextElement(1, atributValue("zdobd_do"), wxPoint(ml+115,mt+93), 23, wxTEXT_ALIGNMENT_CENTER, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    text = TextElement(1, "do", wxPoint(ml+70,mt+93), 44, wxTEXT_ALIGNMENT_RIGHT, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    text = TextElement(1, "od", wxPoint(ml+4,mt+93), 44, wxTEXT_ALIGNMENT_RIGHT, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("Právnická osoba:"), wxPoint(ml,mt+100), 80, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(8).FaceName(FACENAME).Bold())));
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("Název právnické osoby"), wxPoint(ml,mt+104), 80);
    m_document->addElement(text);
    rect = RectangleElement(1, wxPoint(ml+1,mt+108), wxSize(m_pageWidth-ml-1-mr,6), LineStyle(), 1);
    m_document->addElement(rect);
    rect = RectangleElement(1, wxPoint(ml+1,mt+115), wxSize(m_pageWidth-ml-1-mr,6), LineStyle(), 1);
    m_document->addElement(rect);
    text = TextElement(1, wxString::FromUTF8("Fyzická osoba:"), wxPoint(ml,mt+122), 45, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(8).FaceName(FACENAME).Bold())));
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("Příjmení"), wxPoint(ml,mt+126), 45);
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("Jméno"), wxPoint(ml+150,mt+126), 45);
    m_document->addElement(text);
    text = TextElement(1, "Titul", wxPoint(ml+235,mt+126), 45);
    m_document->addElement(text);
    rect = RectangleElement(1, wxPoint(ml+1,mt+130), wxSize(145,6), LineStyle(), 1);
    m_document->addElement(rect);
    rect = RectangleElement(1, wxPoint(ml+151,mt+130), wxSize(80,6), LineStyle(), 1);
    m_document->addElement(rect);
    rect = RectangleElement(1, wxPoint(ml+236,mt+130), wxSize(m_pageWidth-ml-236-mr,6), LineStyle(), 1);
    m_document->addElement(rect);
    text = TextElement(1, atributValue("prijmeni"), wxPoint(ml,mt+131), 145, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    text = TextElement(1, atributValue("jmeno"), wxPoint(ml+150,mt+131), 80, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    text = TextElement(1, atributValue("titul"), wxPoint(ml+235,mt+131), 20, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("Sídlo právnické osoby / adresa místa pobytu fyzické osoby podle § 13 odst. 1 daňového řádu:"), wxPoint(ml,mt+136), 200,
                       wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(8).FaceName(FACENAME).Bold())));
    m_document->addElement(text);
    text = TextElement(1, "Obec", wxPoint(ml,mt+140), 45);
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("PSČ"), wxPoint(ml+220,mt+140), 45);
    m_document->addElement(text);
    rect = RectangleElement(1, wxPoint(ml+1,mt+144), wxSize(215,6), LineStyle(), 1);
    m_document->addElement(rect);
    rect = RectangleElement(1, wxPoint(ml+221,mt+144), wxSize(m_pageWidth-ml-221-mr,6), LineStyle(), 1);
    m_document->addElement(rect);
    text = TextElement(1, atributValue("naz_obce"), wxPoint(ml,mt+145), 215, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    text = TextElement(1, atributValue("psc"), wxPoint(ml+220,mt+145), 35, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("Ulice, část obce (číslo popisné / číslo orientační)"), wxPoint(ml,mt+150), 145);
    m_document->addElement(text);
    rect = RectangleElement(1, wxPoint(ml+1,mt+154), wxSize(m_pageWidth-ml-1-mr,6), LineStyle(), 1);
    m_document->addElement(rect);
    text = TextElement(1, atributValue("ulice")+" "+atributValue("c_pop")+(atributValue("c_orient").IsEmpty()?"":"/"+atributValue("c_orient")), wxPoint(ml,mt+155),
                       m_pageWidth-ml-1-mr, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("Stát"), wxPoint(ml,mt+160), 145);
    m_document->addElement(text);
    text = TextElement(1, "Telefon", wxPoint(ml+155,mt+160), 145);
    m_document->addElement(text);
    rect = RectangleElement(1, wxPoint(ml+1,mt+164), wxSize(150,6), LineStyle(), 1);
    m_document->addElement(rect);
    rect = RectangleElement(1, wxPoint(ml+156,mt+164), wxSize(m_pageWidth-ml-156-mr,6), LineStyle(), 1);
    m_document->addElement(rect);
    text = TextElement(1, atributValue("stat"), wxPoint(ml,mt+165), 150, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    text = TextElement(1, atributValue("c_telef"), wxPoint(ml+155,mt+165), 80, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("ID datové schránky"), wxPoint(ml,mt+170), 145);
    m_document->addElement(text);
    text = TextElement(1, "E-mail", wxPoint(ml+100,mt+170), 145);
    m_document->addElement(text);
    rect = RectangleElement(1, wxPoint(ml+1,mt+174), wxSize(95,6), LineStyle(), 1);
    m_document->addElement(rect);
    rect = RectangleElement(1, wxPoint(ml+101,mt+174), wxSize(m_pageWidth-ml-101-mr,6), LineStyle(), 1);
    m_document->addElement(rect);
    text = TextElement(1, atributValue("id_dats"), wxPoint(ml,mt+175), 95, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    text = TextElement(1, atributValue("email"), wxPoint(ml+100,mt+175), m_pageWidth-ml-105-mr, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    text = TextElement(1, "Datum", wxPoint(ml,mt+182), 50);
    m_document->addElement(text);
    rect = RectangleElement(1, wxPoint(ml+13,mt+181), wxSize(40,6), LineStyle(), 1);
    m_document->addElement(rect);
    text = TextElement(1, atributValue("d_poddp"), wxPoint(ml+12,mt+182), 40, wxTEXT_ALIGNMENT_CENTER, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);

    if(hasVeta("VetaA1"))
    {
        int height = 0;
        page++;
        text = TextElement(page, wxString::FromUTF8("A.1."), wxPoint(ml-1,mt), m_pageWidth-ml-mr, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Bold())));
        m_document->addElement(text);
        text = TextElement(page, wxString::FromUTF8("Uskutečněná zdanitelná plnění v režimu přenesení daňové povinnosti, u kterých je povinen přiznat daň příjemce plnění podle § 92a"),
                           wxPoint(ml+11,mt), m_pageWidth-ml-11-mr, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME))));
        m_document->addElement(text);
        height += text.height();
        wxArrayString texty;
        texty.Add(wxString::FromUTF8("Číslo\nřádku"));
        texty.Add(wxString::FromUTF8("DIČ\nodběratele"));
        texty.Add(wxString::FromUTF8("Ev. číslo daňového dokladu"));
        texty.Add("DUZP");
        texty.Add(wxString::FromUTF8("Základ daně"));
        texty.Add(wxString::FromUTF8("Kód\npředmětu plnění"));
        wxArrayInt width;
        width.Add(20);
        width.Add(40);
        width.Add(m_pageWidth-ml-mr-185);
        width.Add(30);
        width.Add(55);
        width.Add(40);
        wxArrayInt alig;
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        TableRowElement row = TableRowElement(page, texty, wxPoint(ml,mt+height), width, alig);
        row.setBorder(true);
        m_document->addElement(row);
        height += row.height();
        alig.Clear();
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_RIGHT);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        for(size_t i=0; i<m_vety.GetCount(); i++)
        {
            if(m_vety[i].name() == "VetaA1")
            {
                texty.Clear();
                texty.Add(m_vety[i].atributValue("c_radku"));
                texty.Add(m_vety[i].atributValue("dic_odb"));
                texty.Add(m_vety[i].atributValue("c_evid_dd"));
                texty.Add(m_vety[i].atributValue("duzp"));
                texty.Add(m_vety[i].atributValue("zakl_dane1"));
                texty.Add(m_vety[i].atributValue("kod_pred_pl"));
                row = TableRowElement(page, texty, wxPoint(ml,mt+height), width, alig);
                row.setBorder(true);
                height += row.height();
                if(height > pageHeight)
                {
                    page++;
                    row.setPage(page);
                    row.setPos(wxPoint(ml,mt));
                    height += row.height();
                    m_document->addElement(row);
                }
                else
                {
                    m_document->addElement(row);
                }
            }
        }
    }

    if(hasVeta("VetaA2"))
    {
        ml = 5;
        mr = 5;
        int height = 0;
        page++;
        text = TextElement(page, wxString::FromUTF8("A.2."), wxPoint(ml-1,mt), m_pageWidth-ml-mr, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Bold())));
        m_document->addElement(text);
        if(m_newXML) text = TextElement(page, wxString::FromUTF8("Přijatá zdanitelná plnění, u kterých je povinen přiznat daň příjemce dle § 108 odst. 2 a 3 (§ 24, § 25)"),
                                        wxPoint(ml+11,mt), m_pageWidth-ml-11-mr, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME))));
        else text = TextElement(page, wxString::FromUTF8("Přijatá zdanitelná plnění, u kterých je povinen přiznat daň příjemce dle § 108 odst. 1 písm. b) a c) (§ 24, § 25)"),
                                wxPoint(ml+11,mt), m_pageWidth-ml-11-mr, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME))));
        m_document->addElement(text);
        height += text.height();
        wxArrayString texty;
        texty.Add(wxString::FromUTF8("Číslo\nřádku"));
        texty.Add(wxString::FromUTF8("Identifikace\ndodavatele\n(VAT ID)"));
        texty.Add(wxString::FromUTF8("Ev. číslo daňového dokladu"));
        texty.Add("DPPD");
        texty.Add(wxString::FromUTF8("Základ daně 1"));
        texty.Add(wxString::FromUTF8("Daň 1"));
        texty.Add(wxString::FromUTF8("Základ daně 2"));
        texty.Add(wxString::FromUTF8("Daň 2"));
        texty.Add(wxString::FromUTF8("Základ daně 3"));
        texty.Add(wxString::FromUTF8("Daň 3"));
        wxArrayInt width;
        width.Add(11);
        width.Add(27);
        width.Add(m_pageWidth-ml-mr-222);
        width.Add(22);
        width.Add(27);
        width.Add(27);
        width.Add(27);
        width.Add(27);
        width.Add(27);
        width.Add(27);
        wxArrayInt alig;
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        TableRowElement row = TableRowElement(page, texty, wxPoint(ml,mt+height), width, alig);
        row.setBorder(true);
        m_document->addElement(row);
        height += row.height();
        alig.Clear();
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_RIGHT);
        alig.Add(wxTEXT_ALIGNMENT_RIGHT);
        alig.Add(wxTEXT_ALIGNMENT_RIGHT);
        alig.Add(wxTEXT_ALIGNMENT_RIGHT);
        alig.Add(wxTEXT_ALIGNMENT_RIGHT);
        alig.Add(wxTEXT_ALIGNMENT_RIGHT);
        for(size_t i=0; i<m_vety.GetCount(); i++)
        {
            if(m_vety[i].name() == "VetaA2")
            {
                texty.Clear();
                texty.Add(m_vety[i].atributValue("c_radku"));
                texty.Add(m_vety[i].atributValue("k_stat")+m_vety[i].atributValue("vatid_dod"));
                texty.Add(m_vety[i].atributValue("c_evid_dd"));
                texty.Add(m_vety[i].atributValue("dppd"));
                texty.Add(m_vety[i].atributValue("zakl_dane1"));
                texty.Add(m_vety[i].atributValue("dan1"));
                texty.Add(m_vety[i].atributValue("zakl_dane2"));
                texty.Add(m_vety[i].atributValue("dan2"));
                texty.Add(m_vety[i].atributValue("zakl_dane3"));
                texty.Add(m_vety[i].atributValue("dan3"));
                row = TableRowElement(page, texty, wxPoint(ml,mt+height), width, alig);
                row.setBorder(true);
                height += row.height();
                if(height > pageHeight)
                {
                    page++;
                    row.setPage(page);
                    row.setPos(wxPoint(ml,mt));
                    height += row.height();
                    m_document->addElement(row);
                }
                else
                {
                    m_document->addElement(row);
                }
            }
        }
    }

    if(hasVeta("VetaA3"))
    {
        ml = 5;
        mr = 5;
        int height = 0;
        page++;
        text = TextElement(page, wxString::FromUTF8("A.3."), wxPoint(ml-1,mt), m_pageWidth-ml-mr, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Bold())));
        m_document->addElement(text);
        text = TextElement(page, wxString::FromUTF8("Uskutečněná plnění ve zvláštním režimu pro investiční zlato podle § 101c odst. 1 písm. c) bod 2"),
                           wxPoint(ml+11,mt), m_pageWidth-ml-11-mr, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME))));
        m_document->addElement(text);
        height += text.height();
        wxArrayString texty;
        texty.Add(wxString::FromUTF8("Číslo\nřádku"));
        texty.Add(wxString::FromUTF8("Identifikace\nodběratele\n(DIČ/VAT ID)"));
        texty.Add(wxString::FromUTF8("Jméno, příjmení / obchodní jméno"));
        texty.Add(wxString::FromUTF8("Datum\nnarození"));
        texty.Add(wxString::FromUTF8("Místo pobytu / Sídlo"));
        texty.Add(wxString::FromUTF8("Ev. číslo daňového dokladu"));
        texty.Add("DUP");
        texty.Add(wxString::FromUTF8("Hodnota osvoboz.\nplnění"));
        wxArrayInt width;
        width.Add(12);
        width.Add(24);
        width.Add(m_pageWidth-ml-mr-226);
        width.Add(20);
        width.Add(40);
        width.Add(82);
        width.Add(20);
        width.Add(28);
        wxArrayInt alig;
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        TableRowElement row = TableRowElement(page, texty, wxPoint(ml,mt+height), width, alig);
        row.setBorder(true);
        m_document->addElement(row);
        height += row.height();
        alig.Clear();
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_LEFT);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_RIGHT);
        for(size_t i=0; i<m_vety.GetCount(); i++)
        {
            if(m_vety[i].name() == "VetaA3")
            {
                texty.Clear();
                texty.Add(m_vety[i].atributValue("c_radku"));
                texty.Add(m_vety[i].atributValue("k_stat")+m_vety[i].atributValue("vatid_odb"));
                texty.Add(m_vety[i].atributValue("jm_prijm_obch"));
                texty.Add(m_vety[i].atributValue("d_narozeni"));
                texty.Add(m_vety[i].atributValue("m_pobytu_sidlo"));
                texty.Add(m_vety[i].atributValue("c_evid_dd"));
                texty.Add(m_vety[i].atributValue("dup"));
                texty.Add(m_vety[i].atributValue("osv_plneni"));
                row = TableRowElement(page, texty, wxPoint(ml,mt+height), width, alig);
                row.setBorder(true);
                height += row.height();
                if(height > pageHeight)
                {
                    page++;
                    row.setPage(page);
                    row.setPos(wxPoint(ml,mt));
                    height += row.height();
                    m_document->addElement(row);
                }
                else
                {
                    m_document->addElement(row);
                }
            }
        }
    }

    if(hasVeta("VetaA4"))
    {
        ml = 5;
        mr = 5;
        int height = 0;
        page++;
        text = TextElement(page, wxString::FromUTF8("A.4."), wxPoint(ml-1,mt), m_pageWidth-ml-mr, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Bold())));
        m_document->addElement(text);
        if(m_newXML) text = TextElement(page, wxString::FromUTF8("Uskutečněná zdanitelná plnění a přijaté úplaty s povinností přiznat daň dle § 108 odst. 1 s hodnotou nad 10.000,- Kč včetně daně a všechny provedené opravy v souvislosti s nedobytnými pohledávkami bez ohledu na limit"),
                                        wxPoint(ml+11,mt), m_pageWidth-ml-11-mr, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME))));
        else text = TextElement(page, wxString::FromUTF8("Uskutečněná zdanitelná plnění a přijaté úplaty s povinností přiznat daň dle § 108 odst. 1 písm. a) s hodnotou nad 10.000,- Kč včetně daně a všechny provedené opravy podle § 44 bez ohledu na limit"),
                                wxPoint(ml+11,mt), m_pageWidth-ml-11-mr, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME))));
        m_document->addElement(text);
        height += text.height();
        wxArrayString texty;
        texty.Add(wxString::FromUTF8("Číslo\nřádku"));
        texty.Add(wxString::FromUTF8("DIČ\nodběratele"));
        texty.Add(wxString::FromUTF8("Ev. číslo daňového dokladu"));
        texty.Add("DPPD");
        texty.Add(wxString::FromUTF8("Základ daně 1"));
        texty.Add(wxString::FromUTF8("Daň 1"));
        texty.Add(wxString::FromUTF8("Základ daně 2"));
        texty.Add(wxString::FromUTF8("Daň 2"));
        texty.Add(wxString::FromUTF8("Základ daně 3"));
        texty.Add(wxString::FromUTF8("Daň 3"));
        texty.Add(wxString::FromUTF8("Kód\nrežimu\nplnění"));
        texty.Add(m_newXML?wxString::FromUTF8("Opravy\nu nedobytné\npohledávky"):wxString::FromUTF8("§44\nZDPH"));
        wxArrayInt width;
        width.Add(11);
        width.Add(27);
        width.Add(m_pageWidth-ml-mr-(m_newXML?240:230));
        width.Add(22);
        width.Add(25);
        width.Add(25);
        width.Add(25);
        width.Add(25);
        width.Add(25);
        width.Add(25);
        width.Add(10);
        width.Add(m_newXML?20:10);
        wxArrayInt alig;
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        TableRowElement row = TableRowElement(page, texty, wxPoint(ml,mt+height), width, alig);
        row.setBorder(true);
        m_document->addElement(row);
        height += row.height();
        alig.Clear();
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_RIGHT);
        alig.Add(wxTEXT_ALIGNMENT_RIGHT);
        alig.Add(wxTEXT_ALIGNMENT_RIGHT);
        alig.Add(wxTEXT_ALIGNMENT_RIGHT);
        alig.Add(wxTEXT_ALIGNMENT_RIGHT);
        alig.Add(wxTEXT_ALIGNMENT_RIGHT);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        for(size_t i=0; i<m_vety.GetCount(); i++)
        {
            if(m_vety[i].name() == "VetaA4")
            {
                texty.Clear();
                texty.Add(m_vety[i].atributValue("c_radku"));
                texty.Add(wxString("CZ")+m_vety[i].atributValue("dic_odb"));
                texty.Add(m_vety[i].atributValue("c_evid_dd"));
                texty.Add(m_vety[i].atributValue("dppd"));
                texty.Add(m_vety[i].atributValue("zakl_dane1"));
                texty.Add(m_vety[i].atributValue("dan1"));
                texty.Add(m_vety[i].atributValue("zakl_dane2"));
                texty.Add(m_vety[i].atributValue("dan2"));
                texty.Add(m_vety[i].atributValue("zakl_dane3"));
                texty.Add(m_vety[i].atributValue("dan3"));
                texty.Add(m_vety[i].atributValue("kod_rezim_pl"));
                texty.Add(m_vety[i].atributValue("zdph_44"));
                row = TableRowElement(page, texty, wxPoint(ml,mt+height), width, alig);
                row.setBorder(true);
                height += row.height();
                if(height > pageHeight)
                {
                    page++;
                    row.setPage(page);
                    row.setPos(wxPoint(ml,mt));
                    height += row.height();
                    m_document->addElement(row);
                }
                else
                {
                    m_document->addElement(row);
                }
            }
        }
    }

    if(hasVeta("VetaB1"))
    {
        ml = 5;
        mr = 5;
        int height = 0;
        page++;
        text = TextElement(page, wxString::FromUTF8("B.1."), wxPoint(ml-1,mt), m_pageWidth-ml-mr, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Bold())));
        m_document->addElement(text);
        text = TextElement(page, wxString::FromUTF8("Přijatá zdanitelná plnění v režimu přenesení daňové povinnosti, u kterých je povinen přiznat daň příjemce podle § 92a"),
                           wxPoint(ml+11,mt), m_pageWidth-ml-11-mr, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME))));
        m_document->addElement(text);
        height += text.height();
        wxArrayString texty;
        texty.Add(wxString::FromUTF8("Číslo\nřádku"));
        texty.Add(wxString::FromUTF8("DIČ\ndodavatele"));
        texty.Add(wxString::FromUTF8("Ev. číslo daňového dokladu"));
        texty.Add("DUZP");
        texty.Add(wxString::FromUTF8("Základ daně 1"));
        texty.Add(wxString::FromUTF8("Daň 1"));
        texty.Add(wxString::FromUTF8("Základ daně 2"));
        texty.Add(wxString::FromUTF8("Daň 2"));
        texty.Add(wxString::FromUTF8("Základ daně 3"));
        texty.Add(wxString::FromUTF8("Daň 3"));
        texty.Add(wxString::FromUTF8("Kód\npředmětu\nplnění"));
        wxArrayInt width;
        width.Add(11);
        width.Add(27);
        width.Add(m_pageWidth-ml-mr-224);
        width.Add(22);
        width.Add(25);
        width.Add(25);
        width.Add(25);
        width.Add(25);
        width.Add(25);
        width.Add(25);
        width.Add(14);
        wxArrayInt alig;
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        TableRowElement row = TableRowElement(page, texty, wxPoint(ml,mt+height), width, alig);
        row.setBorder(true);
        m_document->addElement(row);
        height += row.height();
        alig.Clear();
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_RIGHT);
        alig.Add(wxTEXT_ALIGNMENT_RIGHT);
        alig.Add(wxTEXT_ALIGNMENT_RIGHT);
        alig.Add(wxTEXT_ALIGNMENT_RIGHT);
        alig.Add(wxTEXT_ALIGNMENT_RIGHT);
        alig.Add(wxTEXT_ALIGNMENT_RIGHT);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        for(size_t i=0; i<m_vety.GetCount(); i++)
        {
            if(m_vety[i].name() == "VetaB1")
            {
                texty.Clear();
                texty.Add(m_vety[i].atributValue("c_radku"));
                texty.Add(wxString("CZ")+m_vety[i].atributValue("dic_dod"));
                texty.Add(m_vety[i].atributValue("c_evid_dd"));
                texty.Add(m_vety[i].atributValue("duzp"));
                texty.Add(m_vety[i].atributValue("zakl_dane1"));
                texty.Add(m_vety[i].atributValue("dan1"));
                texty.Add(m_vety[i].atributValue("zakl_dane2"));
                texty.Add(m_vety[i].atributValue("dan2"));
                texty.Add(m_vety[i].atributValue("zakl_dane3"));
                texty.Add(m_vety[i].atributValue("dan3"));
                texty.Add(m_vety[i].atributValue("kod_pred_pl"));
                row = TableRowElement(page, texty, wxPoint(ml,mt+height), width, alig);
                row.setBorder(true);
                height += row.height();
                if(height > pageHeight)
                {
                    page++;
                    row.setPage(page);
                    row.setPos(wxPoint(ml,mt));
                    height += row.height();
                    m_document->addElement(row);
                }
                else
                {
                    m_document->addElement(row);
                }
            }
        }
    }

    if(hasVeta("VetaB2"))
    {
        ml = 5;
        mr = 5;
        int height = 0;
        page++;
        text = TextElement(page, wxString::FromUTF8("B.2."), wxPoint(ml-1,mt), m_pageWidth-ml-mr, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Bold())));
        m_document->addElement(text);
        if(m_newXML)
        {
            text = TextElement(page, wxString::FromUTF8("Přijatá zdanitelná plnění a poskytnuté úplaty, u kterých příjemce uplatňuje nárok na odpočet daně dle § 73 odst. 1 písm. a) s hodnotou nad 10.000,- Kč včetně daně a všechny opravy odpočtu v souvislosti s nedobytnými pohledávkami bez ohledu na limit"),
                               wxPoint(ml+11,mt), m_pageWidth-ml-11-mr, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME))));
        }
        else
        {
            text = TextElement(page, wxString::FromUTF8("Přijatá zdanitelná plnění a poskytnuté úplaty, u kterých příjemce uplatňuje nárok na odpočet daně dle § 73 odst. 1 písm. a) s hodnotou nad 10.000,- Kč včetně daně a všechny přijaté opravy podle § 44 bez ohledu na limit"),
                               wxPoint(ml+11,mt), m_pageWidth-ml-11-mr, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME))));
        }
        m_document->addElement(text);
        height += text.height();
        wxArrayString texty;
        texty.Add(wxString::FromUTF8("Číslo\nřádku"));
        texty.Add(wxString::FromUTF8("DIČ\ndodavatele"));
        texty.Add(wxString::FromUTF8("Ev. číslo daňového dokladu"));
        texty.Add("DPPD");
        texty.Add(wxString::FromUTF8("Základ daně 1"));
        texty.Add(wxString::FromUTF8("Daň 1"));
        texty.Add(wxString::FromUTF8("Základ daně 2"));
        texty.Add(wxString::FromUTF8("Daň 2"));
        texty.Add(wxString::FromUTF8("Základ daně 3"));
        texty.Add(wxString::FromUTF8("Daň 3"));
        texty.Add(wxString::FromUTF8("Použit\npoměr"));
        texty.Add(m_newXML?wxString::FromUTF8("Opravy\nu nedobytné\npohledávky"):wxString::FromUTF8("§44\nZDPH"));
        wxArrayInt width;
        width.Add(11);
        width.Add(27);
        width.Add(m_pageWidth-ml-mr-(m_newXML?240:230));
        width.Add(22);
        width.Add(25);
        width.Add(25);
        width.Add(25);
        width.Add(25);
        width.Add(25);
        width.Add(25);
        width.Add(10);
        width.Add(m_newXML?20:10);
        wxArrayInt alig;
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        TableRowElement row = TableRowElement(page, texty, wxPoint(ml,mt+height), width, alig);
        row.setBorder(true);
        m_document->addElement(row);
        height += row.height();
        alig.Clear();
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_RIGHT);
        alig.Add(wxTEXT_ALIGNMENT_RIGHT);
        alig.Add(wxTEXT_ALIGNMENT_RIGHT);
        alig.Add(wxTEXT_ALIGNMENT_RIGHT);
        alig.Add(wxTEXT_ALIGNMENT_RIGHT);
        alig.Add(wxTEXT_ALIGNMENT_RIGHT);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        for(size_t i=0; i<m_vety.GetCount(); i++)
        {
            if(m_vety[i].name() == "VetaB2")
            {
                texty.Clear();
                texty.Add(m_vety[i].atributValue("c_radku"));
                texty.Add(wxString("CZ")+m_vety[i].atributValue("dic_dod"));
                texty.Add(m_vety[i].atributValue("c_evid_dd"));
                texty.Add(m_vety[i].atributValue("dppd"));
                texty.Add(m_vety[i].atributValue("zakl_dane1"));
                texty.Add(m_vety[i].atributValue("dan1"));
                texty.Add(m_vety[i].atributValue("zakl_dane2"));
                texty.Add(m_vety[i].atributValue("dan2"));
                texty.Add(m_vety[i].atributValue("zakl_dane3"));
                texty.Add(m_vety[i].atributValue("dan3"));
                texty.Add(m_vety[i].atributValue("pomer"));
                texty.Add(m_vety[i].atributValue("zdph_44"));
                row = TableRowElement(page, texty, wxPoint(ml,mt+height), width, alig);
                row.setBorder(true);
                height += row.height();
                if(height > pageHeight)
                {
                    page++;
                    row.setPage(page);
                    row.setPos(wxPoint(ml,mt));
                    height += row.height();
                    m_document->addElement(row);
                }
                else
                {
                    m_document->addElement(row);
                }
            }
        }
    }

    if(hasVeta("VetaA5") || hasVeta("VetaB3") || hasVeta("VetaC"))
    {
        ml = 5;
        mr = 5;
        int height = 0;
        page++;
        if(hasVeta("VetaA5"))
        {
            text = TextElement(page, wxString::FromUTF8("A.5."), wxPoint(ml-1,mt), m_pageWidth-ml-mr, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Bold())));
            m_document->addElement(text);
            text = TextElement(page,
                               m_newXML?wxString::FromUTF8("Ostatní uskutečněná zdanitelná plnění a přijaté úplaty s povinností přiznat daň dle § 108 odst. 1 s hodnotou do 10.000,- Kč včetně daně, nebo plnění, u nichž nevznikla povinnost vystavit daňový doklad"):
                                        wxString::FromUTF8("Ostatní uskutečněná zdanitelná plnění a přijaté úplaty s povinností přiznat daň dle § 108 odst. 1 písm. a) s hodnotou do 10.000,- Kč včetně daně, nebo plnění, u nichž nevznikla povinnost vystavit daňový doklad"),
                               wxPoint(ml+11,mt), m_pageWidth-ml-11-mr, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME))));
            m_document->addElement(text);
            height += text.height();
            wxArrayString texty;
            texty.Add(wxString::FromUTF8("Základ daně 1"));
            texty.Add(wxString::FromUTF8("Daň 1"));
            texty.Add(wxString::FromUTF8("Základ daně 2"));
            texty.Add(wxString::FromUTF8("Daň 2"));
            texty.Add(wxString::FromUTF8("Základ daně 3"));
            texty.Add(wxString::FromUTF8("Daň 3"));
            wxArrayInt width;
            width.Add(44);
            width.Add(44);
            width.Add(44);
            width.Add(44);
            width.Add(44);
            width.Add(44);
            wxArrayInt alig;
            alig.Add(wxTEXT_ALIGNMENT_CENTER);
            alig.Add(wxTEXT_ALIGNMENT_CENTER);
            alig.Add(wxTEXT_ALIGNMENT_CENTER);
            alig.Add(wxTEXT_ALIGNMENT_CENTER);
            alig.Add(wxTEXT_ALIGNMENT_CENTER);
            alig.Add(wxTEXT_ALIGNMENT_CENTER);
            TableRowElement row = TableRowElement(page, texty, wxPoint((m_pageWidth-264)/2,mt+height), width, alig);
            row.setBorder(true);
            m_document->addElement(row);
            height += row.height();
            alig.Clear();
            alig.Add(wxTEXT_ALIGNMENT_RIGHT);
            alig.Add(wxTEXT_ALIGNMENT_RIGHT);
            alig.Add(wxTEXT_ALIGNMENT_RIGHT);
            alig.Add(wxTEXT_ALIGNMENT_RIGHT);
            alig.Add(wxTEXT_ALIGNMENT_RIGHT);
            alig.Add(wxTEXT_ALIGNMENT_RIGHT);
            for(size_t i=0; i<m_vety.GetCount(); i++)
            {
                if(m_vety[i].name() == "VetaA5")
                {
                    texty.Clear();
                    texty.Add(m_vety[i].atributValue("zakl_dane1"));
                    texty.Add(m_vety[i].atributValue("dan1"));
                    texty.Add(m_vety[i].atributValue("zakl_dane2"));
                    texty.Add(m_vety[i].atributValue("dan2"));
                    texty.Add(m_vety[i].atributValue("zakl_dane3"));
                    texty.Add(m_vety[i].atributValue("dan3"));
                    row = TableRowElement(page, texty, wxPoint((m_pageWidth-264)/2,mt+height), width, alig);
                    row.setBorder(true);
                    height += row.height();
                    if(height > pageHeight)
                    {
                        page++;
                        row.setPage(page);
                        row.setPos(wxPoint((m_pageWidth-264)/2,mt));
                        height += row.height();
                        m_document->addElement(row);
                    }
                    else
                    {
                        m_document->addElement(row);
                    }
                }
            }
        }
        if(hasVeta("VetaB3"))
        {
            if(hasVeta("VetaA5")) height += 5;
            text = TextElement(page, wxString::FromUTF8("B.3."), wxPoint(ml-1,mt+height), m_pageWidth-ml-mr, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Bold())));
            m_document->addElement(text);
            text = TextElement(page, wxString::FromUTF8("Přijatá zdanitelná plnění a poskytnuté úplaty, u kterých příjemce uplatňuje nárok na odpočet daně dle § 73 odst. 1 písm.a) s hodnotou do 10.000,- Kč včetně daně"),
                               wxPoint(ml+11,mt+height), m_pageWidth-ml-11-mr, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME))));
            m_document->addElement(text);
            height += text.height();
            wxArrayString texty;
            texty.Add(wxString::FromUTF8("Základ daně 1"));
            texty.Add(wxString::FromUTF8("Daň 1"));
            texty.Add(wxString::FromUTF8("Základ daně 2"));
            texty.Add(wxString::FromUTF8("Daň 2"));
            texty.Add(wxString::FromUTF8("Základ daně 3"));
            texty.Add(wxString::FromUTF8("Daň 3"));
            wxArrayInt width;
            width.Add(44);
            width.Add(44);
            width.Add(44);
            width.Add(44);
            width.Add(44);
            width.Add(44);
            wxArrayInt alig;
            alig.Add(wxTEXT_ALIGNMENT_CENTER);
            alig.Add(wxTEXT_ALIGNMENT_CENTER);
            alig.Add(wxTEXT_ALIGNMENT_CENTER);
            alig.Add(wxTEXT_ALIGNMENT_CENTER);
            alig.Add(wxTEXT_ALIGNMENT_CENTER);
            alig.Add(wxTEXT_ALIGNMENT_CENTER);
            TableRowElement row = TableRowElement(page, texty, wxPoint((m_pageWidth-264)/2,mt+height), width, alig);
            row.setBorder(true);
            m_document->addElement(row);
            height += row.height();
            alig.Clear();
            alig.Add(wxTEXT_ALIGNMENT_RIGHT);
            alig.Add(wxTEXT_ALIGNMENT_RIGHT);
            alig.Add(wxTEXT_ALIGNMENT_RIGHT);
            alig.Add(wxTEXT_ALIGNMENT_RIGHT);
            alig.Add(wxTEXT_ALIGNMENT_RIGHT);
            alig.Add(wxTEXT_ALIGNMENT_RIGHT);
            for(size_t i=0; i<m_vety.GetCount(); i++)
            {
                if(m_vety[i].name() == "VetaB3")
                {
                    texty.Clear();
                    texty.Add(m_vety[i].atributValue("zakl_dane1"));
                    texty.Add(m_vety[i].atributValue("dan1"));
                    texty.Add(m_vety[i].atributValue("zakl_dane2"));
                    texty.Add(m_vety[i].atributValue("dan2"));
                    texty.Add(m_vety[i].atributValue("zakl_dane3"));
                    texty.Add(m_vety[i].atributValue("dan3"));
                    row = TableRowElement(page, texty, wxPoint((m_pageWidth-264)/2,mt+height), width, alig);
                    row.setBorder(true);
                    height += row.height();
                    if(height > pageHeight)
                    {
                        page++;
                        row.setPage(page);
                        row.setPos(wxPoint((m_pageWidth-264)/2,mt));
                        height += row.height();
                        m_document->addElement(row);
                    }
                    else
                    {
                        m_document->addElement(row);
                    }
                }
            }
        }
        if(hasVeta("VetaC"))
        {
            if(hasVeta("VetaA5") || hasVeta("VetaB3")) height += 5;
            text = TextElement(page, wxString::FromUTF8("C. ODDÍL"), wxPoint(ml-1,mt+height), m_pageWidth-ml-mr, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Bold())));
            m_document->addElement(text);
            text = TextElement(page, wxString::FromUTF8("Kontrolní řádky na Daňové přiznání k DPH (DaP)"),
                               wxPoint(ml+21,mt+height), m_pageWidth-ml-21-mr, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME))));
            m_document->addElement(text);
            height += text.height();
            wxArrayString texty;
            texty.Add(wxString::FromUTF8("řádek DaP"));
            texty.Add(" ");
            texty.Add(wxString::FromUTF8("Základ daně"));
            wxArrayInt width;
            width.Add(44);
            width.Add(176);
            width.Add(44);
            wxArrayInt alig;
            alig.Add(wxTEXT_ALIGNMENT_CENTER);
            alig.Add(wxTEXT_ALIGNMENT_CENTER);
            alig.Add(wxTEXT_ALIGNMENT_CENTER);
            TableRowElement row = TableRowElement(page, texty, wxPoint((m_pageWidth-264)/2,mt+height), width, alig);
            row.setBorder(true);
            m_document->addElement(row);
            height += row.height();
            for(size_t i=0; i<m_vety.GetCount(); i++)
            {
                if(m_vety[i].name() == "VetaC")
                {
                    texty.Clear();
                    texty.Add("1");
                    texty.Add(wxString::FromUTF8("A.4. + A.5. celkem základy daně u základní sazby DPH"));
                    texty.Add(m_vety[i].atributValue("obrat23"));
                    alig.Clear();
                    alig.Add(wxTEXT_ALIGNMENT_RIGHT);
                    alig.Add(wxTEXT_ALIGNMENT_LEFT);
                    alig.Add(wxTEXT_ALIGNMENT_RIGHT);
                    row = TableRowElement(page, texty, wxPoint((m_pageWidth-264)/2,mt+height), width, alig);
                    row.setBorder(true);
                    m_document->addElement(row);
                    height += row.height();
                    texty.Clear();
                    texty.Add("2");
                    texty.Add(wxString::FromUTF8("A.4. + A.5. celkem základy daně u první snížené a druhé snížené sazby DPH"));
                    texty.Add(m_vety[i].atributValue("obrat5"));
                    row = TableRowElement(page, texty, wxPoint((m_pageWidth-264)/2,mt+height), width, alig);
                    row.setBorder(true);
                    m_document->addElement(row);
                    height += row.height();
                    texty.Clear();
                    texty.Add("40");
                    texty.Add(wxString::FromUTF8("B.2. + B.3. celkem základy daně u základní sazby DPH"));
                    texty.Add(m_vety[i].atributValue("pln23"));
                    row = TableRowElement(page, texty, wxPoint((m_pageWidth-264)/2,mt+height), width, alig);
                    row.setBorder(true);
                    m_document->addElement(row);
                    height += row.height();
                    texty.Clear();
                    texty.Add("41");
                    texty.Add(wxString::FromUTF8("B.2. + B.3. celkem základy daně u první snížené a druhé snížené sazby DPH"));
                    texty.Add(m_vety[i].atributValue("pln5"));
                    row = TableRowElement(page, texty, wxPoint((m_pageWidth-264)/2,mt+height), width, alig);
                    row.setBorder(true);
                    m_document->addElement(row);
                    height += row.height();
                    texty.Clear();
                    texty.Add("25");
                    texty.Add(wxString::FromUTF8("A.1 celkem základy daně"));
                    texty.Add(m_vety[i].atributValue("pln_rez_pren"));
                    row = TableRowElement(page, texty, wxPoint((m_pageWidth-264)/2,mt+height), width, alig);
                    row.setBorder(true);
                    m_document->addElement(row);
                    height += row.height();
                    texty.Clear();
                    texty.Add("10");
                    texty.Add(wxString::FromUTF8("B.1 celkem základy daně u základní sazby DPH"));
                    texty.Add(m_vety[i].atributValue("rez_pren23"));
                    row = TableRowElement(page, texty, wxPoint((m_pageWidth-264)/2,mt+height), width, alig);
                    row.setBorder(true);
                    m_document->addElement(row);
                    height += row.height();
                    texty.Clear();
                    texty.Add("11");
                    texty.Add(wxString::FromUTF8("B.1 celkem základy daně u první snížené a druhé snížené sazby DPH"));
                    texty.Add(m_vety[i].atributValue("rez_pren5"));
                    row = TableRowElement(page, texty, wxPoint((m_pageWidth-264)/2,mt+height), width, alig);
                    row.setBorder(true);
                    m_document->addElement(row);
                    height += row.height();
                    texty.Clear();
                    texty.Add("3+4+5+6+9+12+13");
                    texty.Add(wxString::FromUTF8("A.2 celkem základy daně"));
                    texty.Add(m_vety[i].atributValue("celk_zd_a2"));
                    row = TableRowElement(page, texty, wxPoint((m_pageWidth-264)/2,mt+height), width, alig);
                    row.setBorder(true);
                    m_document->addElement(row);
                    height += row.height();
                }
            }
        }
    }

    m_document->setPages(page);
}

wxString Dphkh1Print::atributValue(const wxString &name) const
{
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        dxAtributArray atr = m_vety[i].atributy();
        for(size_t j=0; j<atr.GetCount(); j++)
        {
            if(atr[j].name() == name)
            {
                return atr[j].value();
            }
        }
    }
    return wxEmptyString;
}

bool Dphkh1Print::hasVeta(const wxString &name) const
{
    if(!atributValue("vyzva_odp").IsEmpty()) return false;
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        if(m_vety[i].name() == name)
        {
            return true;
        }
    }
    return false;
}

void Dphkh1Print::fillNewXMLValue()
{
    Veta vetaD;
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        if(m_vety[i].name() == "VetaD")
        {
            vetaD = m_vety[i];
            break;
        }
    }
    if(vetaD.atributValue("zdobd_do").IsEmpty())
    {
        int rok = vetaD.atributValueInt("rok");
        wxDateTime::Month mesic = wxDateTime::Jan;
        if(!vetaD.atributValue("mesic").IsEmpty()) mesic = static_cast<wxDateTime::Month>(vetaD.atributValueInt("mesic")-1);
        if(!vetaD.atributValue("ctvrt").IsEmpty()) mesic = dxutils::ctvrtletiMesicEnd(vetaD.atributValueInt("ctvrt"));
        m_newXML = wxDateTime(1, mesic, rok).GetLastMonthDay(mesic, rok).IsLaterThan(wxDateTime(29, wxDateTime::Sep, 2019));
    }
    else
    {
        m_newXML = vetaD.atributValueDatum("zdobd_do").IsLaterThan(wxDateTime(29, wxDateTime::Sep, 2019));
    }
}
