/***************************************************************
 * Name:      dphkh1vetaa2dialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include "dphkh1vetaa2dialog.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "model/dbmodel.h"

IMPLEMENT_DYNAMIC_CLASS(Dphkh1Vetaa2Dialog, wxDialog)

BEGIN_EVENT_TABLE(Dphkh1Vetaa2Dialog, wxDialog)
    EVT_BUTTON(wxID_OK, Dphkh1Vetaa2Dialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, Dphkh1Vetaa2Dialog::OnCancel)
    EVT_SPINCTRLDOUBLE(ID_DPHKH1VETAA2_ZAKLAD1, Dphkh1Vetaa2Dialog::OnZaklad1)
    EVT_SPINCTRLDOUBLE(ID_DPHKH1VETAA2_ZAKLAD2, Dphkh1Vetaa2Dialog::OnZaklad2)
    EVT_SPINCTRLDOUBLE(ID_DPHKH1VETAA2_ZAKLAD3, Dphkh1Vetaa2Dialog::OnZaklad3)
END_EVENT_TABLE()

Dphkh1Vetaa2Dialog::Dphkh1Vetaa2Dialog(wxWindow *parent, Veta veta, int rok)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Upravit řádek A2")), m_veta(veta), m_rok(rok)
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Kód státu")), wxGBPosition(0,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_kStat = new wxTextCtrl(this, wxID_ANY);
    m_kStat->SetMaxLength(2);
    m_kStat->SetValue(m_veta.atributValue("k_stat"));
    gridSizer->Add(m_kStat, wxGBPosition(0,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, "VAT ID"), wxGBPosition(1,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_vatidDod = new wxTextCtrl(this, wxID_ANY);
    m_vatidDod->SetMaxLength(12);
    m_vatidDod->SetValue(m_veta.atributValue("vatid_dod"));
    gridSizer->Add(m_vatidDod, wxGBPosition(1,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Ev. číslo daňového dokladu")), wxGBPosition(2,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_cEvidDd = new wxTextCtrl(this, wxID_ANY);
    m_cEvidDd->SetMaxLength(60);
    m_cEvidDd->SetValue(m_veta.atributValue("c_evid_dd"));
    gridSizer->Add(m_cEvidDd, wxGBPosition(2,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, "DPPD"), wxGBPosition(3,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_dppd = new wxDatePickerCtrl(this, wxID_ANY, wxDateTime::Today(), wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    if(!m_veta.atributValue("dppd").IsEmpty()) m_dppd->SetValue(m_veta.atributValueDatum("dppd"));
    gridSizer->Add(m_dppd, wxGBPosition(3,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Základ daně\npro základní sazbu")), wxGBPosition(4,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_zaklDane1 = new wxSpinCtrlDouble(this, ID_DPHKH1VETAA2_ZAKLAD1);
    m_zaklDane1->SetDigits(2);
    m_zaklDane1->SetRange(0.0, 99999999999999.0);
    if(!m_veta.atributValue("zakl_dane1").IsEmpty()) m_zaklDane1->SetValue(m_veta.atributValueDouble("zakl_dane1"));
    gridSizer->Add(m_zaklDane1, wxGBPosition(4,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Daň\npro základní sazbu")), wxGBPosition(5,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_dan1 = new wxStaticText(this, wxID_ANY, "", wxDefaultPosition, wxDefaultSize, wxALIGN_RIGHT);
    m_dan1->SetLabel(m_veta.atributValue("dan1"));
    gridSizer->Add(m_dan1, wxGBPosition(5,1), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Základ daně\npro 1. sníženou sazbu")), wxGBPosition(6,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_zaklDane2 = new wxSpinCtrlDouble(this, ID_DPHKH1VETAA2_ZAKLAD2);
    m_zaklDane2->SetDigits(2);
    m_zaklDane2->SetRange(0.0, 99999999999999.0);
    if(!m_veta.atributValue("zakl_dane2").IsEmpty()) m_zaklDane2->SetValue(m_veta.atributValueDouble("zakl_dane2"));
    gridSizer->Add(m_zaklDane2, wxGBPosition(6,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Daň\npro 1. sníženou sazbu")), wxGBPosition(7,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_dan2 = new wxStaticText(this, wxID_ANY, "", wxDefaultPosition, wxDefaultSize, wxALIGN_RIGHT);
    m_dan2->SetLabel(m_veta.atributValue("dan2"));
    gridSizer->Add(m_dan2, wxGBPosition(7,1), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Základ daně\npro 2. sníženou sazbu")), wxGBPosition(8,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_zaklDane3 = new wxSpinCtrlDouble(this, ID_DPHKH1VETAA2_ZAKLAD3);
    m_zaklDane3->SetDigits(2);
    m_zaklDane3->SetRange(0.0, 99999999999999.0);
    if(!m_veta.atributValue("zakl_dane3").IsEmpty()) m_zaklDane3->SetValue(m_veta.atributValueDouble("zakl_dane3"));
    gridSizer->Add(m_zaklDane3, wxGBPosition(8,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Daň\npro 2. sníženou sazbu")), wxGBPosition(9,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_dan3 = new wxStaticText(this, wxID_ANY, "", wxDefaultPosition, wxDefaultSize, wxALIGN_RIGHT);
    m_dan3->SetLabel(m_veta.atributValue("dan3"));
    gridSizer->Add(m_dan3, wxGBPosition(9,1), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_mainSizer->Add(gridSizer, 0, wxALL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

Veta Dphkh1Vetaa2Dialog::veta() const
{
    return m_veta;
}

void Dphkh1Vetaa2Dialog::OnOk(wxCommandEvent& /*event*/)
{
    if(m_kStat->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Kód státu nemůže být prázdný"), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        m_kStat->SetFocus();
        return;
    }
    if(m_vatidDod->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Identifikace dodavatele (VAT ID) nemůže být prázdná"), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        m_vatidDod->SetFocus();
        return;
    }
    if(m_cEvidDd->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Ev. číslo daňového dokladu nemůže být prázdné"), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        m_cEvidDd->SetFocus();
        return;
    }
    m_veta.setAtributValue("k_stat", m_kStat->GetValue());
    m_veta.setAtributValue("vatid_dod", m_vatidDod->GetValue());
    m_veta.setAtributValue("c_evid_dd", m_cEvidDd->GetValue());
    m_veta.setAtributValue("dppd", m_dppd->GetValue());
    m_veta.setAtributValue("zakl_dane1", m_zaklDane1->GetValue());
    m_veta.setAtributValue("dan1", m_dan1->GetLabel());
    m_veta.setAtributValue("zakl_dane2", m_zaklDane2->GetValue());
    m_veta.setAtributValue("dan2", m_dan2->GetLabel());
    m_veta.setAtributValue("zakl_dane3", m_zaklDane3->GetValue());
    m_veta.setAtributValue("dan3", m_dan3->GetLabel());
    EndModal(wxID_OK);
}

void Dphkh1Vetaa2Dialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

void Dphkh1Vetaa2Dialog::OnZaklad1(wxSpinDoubleEvent &/*event*/)
{
    m_dan1->SetLabel(wxString::FromCDouble(m_zaklDane1->GetValue()*(idbmodel->zakladniSazba(m_rok)/100.0),2));
}

void Dphkh1Vetaa2Dialog::OnZaklad2(wxSpinDoubleEvent &/*event*/)
{
    m_dan2->SetLabel(wxString::FromCDouble(m_zaklDane2->GetValue()*(idbmodel->prvniSnizenaSazba(m_rok)/100.0),2));
}

void Dphkh1Vetaa2Dialog::OnZaklad3(wxSpinDoubleEvent &/*event*/)
{
    m_dan3->SetLabel(wxString::FromCDouble(m_zaklDane3->GetValue()*(idbmodel->druhaSnizenaSazba(m_rok)/100.0),2));
}
