/***************************************************************
 * Name:      dphkh1vetab1dialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include "dphkh1vetab1dialog.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "model/dbmodel.h"

IMPLEMENT_DYNAMIC_CLASS(Dphkh1Vetab1Dialog, wxDialog)

BEGIN_EVENT_TABLE(Dphkh1Vetab1Dialog, wxDialog)
    EVT_BUTTON(wxID_OK, Dphkh1Vetab1Dialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, Dphkh1Vetab1Dialog::OnCancel)
    EVT_SPINCTRLDOUBLE(ID_DPHKH1VETAB1_ZAKLAD1, Dphkh1Vetab1Dialog::OnZaklad1)
    EVT_SPINCTRLDOUBLE(ID_DPHKH1VETAB1_ZAKLAD2, Dphkh1Vetab1Dialog::OnZaklad2)
    EVT_SPINCTRLDOUBLE(ID_DPHKH1VETAB1_ZAKLAD3, Dphkh1Vetab1Dialog::OnZaklad3)
END_EVENT_TABLE()

Dphkh1Vetab1Dialog::Dphkh1Vetab1Dialog(wxWindow *parent, Veta veta, int rok)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Upravit řádek B1")), m_veta(veta), m_rok(rok)
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("DIČ dodavatele")), wxGBPosition(0,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
     wxTextValidator dicdodval(wxFILTER_NUMERIC);
    m_dicDod = new wxTextCtrl(this, wxID_ANY, "", wxDefaultPosition, wxDefaultSize, 0, dicdodval);
    m_dicDod->SetMaxLength(10);
    m_dicDod->SetValue(m_veta.atributValue("dic_dod"));
    gridSizer->Add(m_dicDod, wxGBPosition(0,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Ev. číslo daňového dokladu")), wxGBPosition(1,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_cEvidDd = new wxTextCtrl(this, wxID_ANY);
    m_cEvidDd->SetMaxLength(60);
    m_cEvidDd->SetValue(m_veta.atributValue("c_evid_dd"));
    gridSizer->Add(m_cEvidDd, wxGBPosition(1,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, "DUZP"), wxGBPosition(2,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_duzp = new wxDatePickerCtrl(this, wxID_ANY, wxDateTime::Today(), wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    if(!m_veta.atributValue("duzp").IsEmpty()) m_duzp->SetValue(m_veta.atributValueDatum("duzp"));
    gridSizer->Add(m_duzp, wxGBPosition(2,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Základ daně\npro základní sazbu")), wxGBPosition(3,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_zaklDane1 = new wxSpinCtrlDouble(this, ID_DPHKH1VETAB1_ZAKLAD1);
    m_zaklDane1->SetDigits(2);
    m_zaklDane1->SetRange(0.0, 99999999999999.0);
    if(!m_veta.atributValue("zakl_dane1").IsEmpty()) m_zaklDane1->SetValue(m_veta.atributValueDouble("zakl_dane1"));
    gridSizer->Add(m_zaklDane1, wxGBPosition(3,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Daň\npro základní sazbu")), wxGBPosition(4,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_dan1 = new wxStaticText(this, wxID_ANY, "", wxDefaultPosition, wxDefaultSize, wxALIGN_RIGHT);
    m_dan1->SetLabel(m_veta.atributValue("dan1"));
    gridSizer->Add(m_dan1, wxGBPosition(4,1), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Základ daně\npro 1. sníženou sazbu")), wxGBPosition(5,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_zaklDane2 = new wxSpinCtrlDouble(this, ID_DPHKH1VETAB1_ZAKLAD2);
    m_zaklDane2->SetDigits(2);
    m_zaklDane2->SetRange(0.0, 99999999999999.0);
    if(!m_veta.atributValue("zakl_dane2").IsEmpty()) m_zaklDane2->SetValue(m_veta.atributValueDouble("zakl_dane2"));
    gridSizer->Add(m_zaklDane2, wxGBPosition(5,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Daň\npro 1. sníženou sazbu")), wxGBPosition(6,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_dan2 = new wxStaticText(this, wxID_ANY, "", wxDefaultPosition, wxDefaultSize, wxALIGN_RIGHT);
    m_dan2->SetLabel(m_veta.atributValue("dan2"));
    gridSizer->Add(m_dan2, wxGBPosition(6,1), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Základ daně\npro 2. sníženou sazbu")), wxGBPosition(7,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_zaklDane3 = new wxSpinCtrlDouble(this, ID_DPHKH1VETAB1_ZAKLAD3);
    m_zaklDane3->SetDigits(2);
    m_zaklDane3->SetRange(0.0, 99999999999999.0);
    if(!m_veta.atributValue("zakl_dane3").IsEmpty()) m_zaklDane3->SetValue(m_veta.atributValueDouble("zakl_dane3"));
    gridSizer->Add(m_zaklDane3, wxGBPosition(7,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Daň\npro 2. sníženou sazbu")), wxGBPosition(8,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_dan3 = new wxStaticText(this, wxID_ANY, "", wxDefaultPosition, wxDefaultSize, wxALIGN_RIGHT);
    m_dan3->SetLabel(m_veta.atributValue("dan3"));
    gridSizer->Add(m_dan3, wxGBPosition(8,1), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Kód předmětu plnění")), wxGBPosition(9,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_kodPredPl = new wxChoice(this, wxID_ANY);
    m_kodPredPl->Append(idbmodel->getDphDruhKpp());
    if(!m_veta.atributValue("kod_pred_pl").IsEmpty()) m_kodPredPl->SetStringSelection(m_veta.atributValue("kod_pred_pl"));
    else m_kodPredPl->SetSelection(0);
    gridSizer->Add(m_kodPredPl, wxGBPosition(9,1), wxDefaultSpan, wxEXPAND);
    m_mainSizer->Add(gridSizer, 0, wxALL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

Veta Dphkh1Vetab1Dialog::veta() const
{
    return m_veta;
}

void Dphkh1Vetab1Dialog::OnOk(wxCommandEvent& /*event*/)
{
    if(m_dicDod->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("DIČ dodavatele nemůže být prázdné"), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        m_dicDod->SetFocus();
        return;
    }
    if(m_cEvidDd->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Ev. číslo daňového dokladu nemůže být prázdné"), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        m_cEvidDd->SetFocus();
        return;
    }
    m_veta.setAtributValue("dic_dod", m_dicDod->GetValue());
    m_veta.setAtributValue("c_evid_dd", m_cEvidDd->GetValue());
    m_veta.setAtributValue("duzp", m_duzp->GetValue());
    m_veta.setAtributValue("zakl_dane1", m_zaklDane1->GetValue());
    m_veta.setAtributValue("dan1", m_dan1->GetLabel());
    m_veta.setAtributValue("zakl_dane2", m_zaklDane2->GetValue());
    m_veta.setAtributValue("dan2", m_dan2->GetLabel());
    m_veta.setAtributValue("zakl_dane3", m_zaklDane3->GetValue());
    m_veta.setAtributValue("dan3", m_dan3->GetLabel());
    m_veta.setAtributValue("kod_pred_pl", m_kodPredPl->GetStringSelection());
    EndModal(wxID_OK);
}

void Dphkh1Vetab1Dialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

void Dphkh1Vetab1Dialog::OnZaklad1(wxSpinDoubleEvent &/*event*/)
{
    m_dan1->SetLabel(wxString::FromCDouble(m_zaklDane1->GetValue()*(idbmodel->zakladniSazba(m_rok)/100.0),2));
}

void Dphkh1Vetab1Dialog::OnZaklad2(wxSpinDoubleEvent &/*event*/)
{
    m_dan2->SetLabel(wxString::FromCDouble(m_zaklDane2->GetValue()*(idbmodel->prvniSnizenaSazba(m_rok)/100.0),2));
}

void Dphkh1Vetab1Dialog::OnZaklad3(wxSpinDoubleEvent &/*event*/)
{
    m_dan3->SetLabel(wxString::FromCDouble(m_zaklDane3->GetValue()*(idbmodel->druhaSnizenaSazba(m_rok)/100.0),2));
}
