/***************************************************************
 * Name:      dphshvetardialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include "dphshvetardialog.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"

IMPLEMENT_DYNAMIC_CLASS(DphshVetarDialog, wxDialog)

BEGIN_EVENT_TABLE(DphshVetarDialog, wxDialog)
    EVT_BUTTON(wxID_OK, DphshVetarDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, DphshVetarDialog::OnCancel)
END_EVENT_TABLE()

DphshVetarDialog::DphshVetarDialog(wxWindow *parent, Veta veta)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Upravit řádek")), m_veta(veta)
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);
    m_kStorno = new wxCheckBox(this, wxID_ANY, wxString::FromUTF8("Storno řádek"));
    m_kStorno->SetValue(!m_veta.atributValue("k_storno").IsEmpty());
    gridSizer->Add(m_kStorno, wxGBPosition(0,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Kód země")), wxGBPosition(0,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    wxTextValidator kstatval(wxFILTER_ALPHA);
    m_kStat = new wxTextCtrl(this, wxID_ANY, "", wxDefaultPosition, wxDefaultSize, 0, kstatval);
    m_kStat->SetMaxLength(2);
    m_kStat->SetValue(m_veta.atributValue("k_stat"));
    gridSizer->Add(m_kStat, wxGBPosition(0,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("DIČ pořizovatele zboží")), wxGBPosition(1,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    wxTextValidator cvatval(wxFILTER_ALPHANUMERIC);
    m_cVat = new wxTextCtrl(this, wxID_ANY, "", wxDefaultPosition, wxDefaultSize, 0, cvatval);
    m_cVat->SetMaxLength(12);
    m_cVat->SetValue(m_veta.atributValue("c_vat"));
    gridSizer->Add(m_cVat, wxGBPosition(1,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Kód plnění")), wxGBPosition(2,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_kPlnEu = new wxChoice(this, wxID_ANY);
    m_kPlnEu->Append("0");
    m_kPlnEu->Append("1");
    m_kPlnEu->Append("2");
    m_kPlnEu->Append("3");
    m_kPlnEu->SetStringSelection(veta.atributValue("k_pln_eu"));
    gridSizer->Add(m_kPlnEu, wxGBPosition(2,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Počet plnění")), wxGBPosition(3,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    wxTextValidator plnpocetval(wxFILTER_DIGITS);
    m_plnPocet = new wxTextCtrl(this, wxID_ANY, "", wxDefaultPosition, wxDefaultSize, 0, plnpocetval);
    m_plnPocet->SetMaxLength(6);
    m_plnPocet->SetValue(veta.atributValue("pln_pocet"));
    gridSizer->Add(m_plnPocet, wxGBPosition(3,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Celková hodnota\nplnění v Kč")), wxGBPosition(4,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    wxTextValidator plnhodnotaval(wxFILTER_DIGITS);
    m_plnHodnota = new wxTextCtrl(this, wxID_ANY, "", wxDefaultPosition, wxDefaultSize, 0, plnhodnotaval);
    m_plnHodnota->SetMaxLength(14);
    m_plnHodnota->SetValue(veta.atributValue("pln_hodnota"));
    gridSizer->Add(m_plnHodnota, wxGBPosition(4,1), wxDefaultSpan, wxEXPAND);
    m_mainSizer->Add(gridSizer, 0, wxALL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

Veta DphshVetarDialog::veta() const
{
    return m_veta;
}

void DphshVetarDialog::OnOk(wxCommandEvent& /*event*/)
{
    if(m_kStat->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Kód země nemůže být prázdný"), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        m_kStat->SetFocus();
        return;
    }
    if(m_cVat->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("DIČ pořizovatele zboží nemůže být prázdné"), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        m_cVat->SetFocus();
        return;
    }
    if(m_plnPocet->GetValue() == "0" || m_plnHodnota->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Počet plnění nemůže být nulový"), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        m_plnPocet->SetFocus();
        return;
    }
    if(m_plnHodnota->GetValue() == "0" || m_plnHodnota->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Celková hodnota plnění nemůže být nulová"), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        m_plnHodnota->SetFocus();
        return;
    }
    m_veta.setAtributValue("k_storno", m_kStorno->GetValue()?"A":"");
    m_veta.setAtributValue("k_stat", m_kStat->GetValue());
    m_veta.setAtributValue("c_vat", m_cVat->GetValue());
    m_veta.setAtributValue("k_pln_eu", m_kPlnEu->GetStringSelection());
    m_veta.setAtributValue("pln_pocet", m_plnPocet->GetValue());
    m_veta.setAtributValue("pln_hodnota", m_plnHodnota->GetValue());
    EndModal(wxID_OK);
}

void DphshVetarDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}
