/***************************************************************
 * Name:      dphdp3wizard.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#include <wx/wfstream.h>
#include <wx/hyperlink.h>
#include "dphshwizard.h"
#include "priznanidialog.h"
#include "dphshvetardialog.h"
#include "dphshprint.h"
#include "dxdefs.h"
#include "utils/dxutils.h"
#include "config.h"
#include "resources/images/add.xpm"
#include "resources/images/edit.xpm"
#include "resources/images/delete.xpm"

BEGIN_EVENT_TABLE(ShPage1, wxWizardPageSimple)
    EVT_RADIOBUTTON(ID_DPHSH_PAGE1_MESIC, ShPage1::OnMesic)
    EVT_RADIOBUTTON(ID_DPHSH_PAGE1_CTVRTLETI, ShPage1::OnCtvrtleti)
    EVT_BUTTON(ID_DPHSH_PAGE1_PRIZNANI, ShPage1::OnPriznani)
END_EVENT_TABLE()

ShPage1::ShPage1(wxWizard *parent)
    : wxWizardPageSimple(parent)
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    m_mainSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Průvodce Vás provede vytvořením\nSouhrnného hlášení VIES")));

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);
    m_mesicBtn = new wxRadioButton(this, ID_DPHSH_PAGE1_MESIC, wxString::FromUTF8("Měsíc"));
    m_mesicBtn->SetValue(false);
    gridSizer->Add(m_mesicBtn, wxGBPosition(0,0), wxDefaultSpan, wxEXPAND);
    m_mesicSelect = new wxChoice(this, wxID_ANY);
    m_mesicSelect->Append(dxutils::mesicString(1));
    m_mesicSelect->Append(dxutils::mesicString(2));
    m_mesicSelect->Append(dxutils::mesicString(3));
    m_mesicSelect->Append(dxutils::mesicString(4));
    m_mesicSelect->Append(dxutils::mesicString(5));
    m_mesicSelect->Append(dxutils::mesicString(6));
    m_mesicSelect->Append(dxutils::mesicString(7));
    m_mesicSelect->Append(dxutils::mesicString(8));
    m_mesicSelect->Append(dxutils::mesicString(9));
    m_mesicSelect->Append(dxutils::mesicString(10));
    m_mesicSelect->Append(dxutils::mesicString(11));
    m_mesicSelect->Append(dxutils::mesicString(12));
    m_mesicSelect->SetSelection(wxMax(wxDateTime::Today().GetMonth()-1,0));
    m_mesicSelect->Enable(false);
    gridSizer->Add(m_mesicSelect, wxGBPosition(0,1), wxDefaultSpan, wxEXPAND);
    m_mesicRok = new wxChoice(this, wxID_ANY);
    m_mesicRok->Append(idbmodel->getObdobiAsStrings());
    m_mesicRok->SetStringSelection(wxString::Format("%d", idbmodel->currentRok()));
    m_mesicRok->Enable(false);
    gridSizer->Add(m_mesicRok, wxGBPosition(0,2), wxDefaultSpan, wxEXPAND);
    m_ctvrtletiBtn = new wxRadioButton(this, ID_DPHSH_PAGE1_CTVRTLETI, wxString::FromUTF8("Čtvrtletí"));
    m_ctvrtletiBtn->SetValue(true);
    gridSizer->Add(m_ctvrtletiBtn, wxGBPosition(1,0), wxDefaultSpan, wxEXPAND);
    m_ctvrtletiSelect = new wxChoice(this, wxID_ANY);
    m_ctvrtletiSelect->Append(dxutils::ctvrtletiString(1));
    m_ctvrtletiSelect->Append(dxutils::ctvrtletiString(2));
    m_ctvrtletiSelect->Append(dxutils::ctvrtletiString(3));
    m_ctvrtletiSelect->Append(dxutils::ctvrtletiString(4));
    m_ctvrtletiSelect->SetSelection(wxMax(wxDateTime::Today().GetMonth()/3-1,0));
    gridSizer->Add(m_ctvrtletiSelect, wxGBPosition(1,1), wxDefaultSpan, wxEXPAND);
    m_ctvrtletiRok = new wxChoice(this, wxID_ANY);
    m_ctvrtletiRok->Append(idbmodel->getObdobiAsStrings());
    m_ctvrtletiRok->SetStringSelection(wxString::Format("%d", idbmodel->currentRok()));
    gridSizer->Add(m_ctvrtletiRok, wxGBPosition(1,2), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Datum vyplnění")), wxGBPosition(2,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_vyhotoveni = new wxDatePickerCtrl(this, wxID_ANY, wxDateTime::Today(), wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    gridSizer->Add(m_vyhotoveni, wxGBPosition(2,1), wxGBSpan(1,2), wxEXPAND);
    m_mainSizer->Add(gridSizer, 0, wxALL, 5);

    m_mainSizer->AddStretchSpacer();
    m_priznani = new wxButton(this, ID_DPHSH_PAGE1_PRIZNANI, wxString::FromUTF8("Nastavení plátce"));
    m_mainSizer->Add(m_priznani, 0, wxALL, 5);
    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void ShPage1::OnPriznani(wxCommandEvent &/*event*/)
{
    PriznaniDialog dialog(this, true);
    dialog.ShowModal();
}

void ShPage1::OnMesic(wxCommandEvent &/*event*/)
{
    m_ctvrtletiSelect->Enable(false);
    m_ctvrtletiRok->Enable(false);
    m_mesicSelect->Enable(true);
    m_mesicRok->Enable(true);
    m_mesicSelect->SetSelection(0);
}

void ShPage1::OnCtvrtleti(wxCommandEvent &/*event*/)
{
    m_mesicSelect->Enable(false);
    m_mesicRok->Enable(false);
    m_ctvrtletiSelect->Enable(true);
    m_ctvrtletiRok->Enable(true);
    m_ctvrtletiSelect->SetSelection(0);
}

wxDateTime ShPage1::vyhotoveni() const
{
    return m_vyhotoveni->GetValue();
}

//0 - mesic, 1 - ctvrtleti
int ShPage1::obdobi() const
{
    if(m_mesicBtn->GetValue()) return 0;
    return 1;
}

int ShPage1::mesic() const
{
    return m_mesicBtn->GetValue()?m_mesicSelect->GetSelection()+1:0;
}

int ShPage1::ctvrtleti() const
{
    return m_ctvrtletiBtn->GetValue()?m_ctvrtletiSelect->GetSelection()+1:0;
}

int ShPage1::rok() const
{
    if(obdobi() == 0) return wxAtoi(m_mesicRok->GetStringSelection());
    return wxAtoi(m_ctvrtletiRok->GetStringSelection());
}

wxDateTime ShPage1::start() const
{
    if(obdobi() == 0) return wxDateTime(1, static_cast<wxDateTime::Month>(mesic()-1), rok());
    return wxDateTime(1, dxutils::ctvrtletiMesicStart(ctvrtleti()), rok());
}

wxDateTime ShPage1::end() const
{
    if(obdobi() == 0) return wxDateTime(start().GetLastMonthDay(static_cast<wxDateTime::Month>(mesic()-1), rok()));
    return wxDateTime(start().GetLastMonthDay(dxutils::ctvrtletiMesicEnd(ctvrtleti()), rok()));
}

BEGIN_EVENT_TABLE(ShPage2, wxWizardPageSimple)
    EVT_WIZARD_PAGE_CHANGING(wxID_ANY, ShPage2::OnPageChanging)
    EVT_RADIOBUTTON(ID_DPHSH_PAGE2_RADNE, ShPage2::OnRadne)
    EVT_RADIOBUTTON(ID_DPHSH_PAGE2_NASLEDNE, ShPage2::OnNasledne)
END_EVENT_TABLE()

ShPage2::ShPage2(wxWizard *parent)
    : wxWizardPageSimple(parent), m_wizard(static_cast<DphshWizard*>(parent))
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    m_radne = new wxRadioButton(this, ID_DPHSH_PAGE2_RADNE, wxString::FromUTF8("Souhrnné hlášení"));
    m_radne->SetValue(true);
    m_mainSizer->Add(m_radne);

    m_nasledne = new wxRadioButton(this, ID_DPHSH_PAGE2_NASLEDNE, wxString::FromUTF8("Následné souhrnné hlášení"));
    if(!idbmodel->someUlozeneDphShv()) m_nasledne->Enable(false);
    m_mainSizer->Add(m_nasledne);

    m_mainSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Následné k:")));

    m_ulozene = new wxChoice(this, wxID_ANY);
    if(idbmodel->someUlozeneDphShv())
    {
        dxUlozeneArray shv = idbmodel->ulozeneDphShv();
        idbmodel->sortUlozene(shv);
        for(size_t i=0; i < shv.GetCount(); i++)
        {
            m_ulozene->Append(shv[i].popis(), new Ulozene(shv[i].id(), shv[i].typ(), shv[i].popis(), shv[i].forma(), shv[i].obdobi(), shv[i].mesic(), shv[i].ctvrtleti(), shv[i].rok(),
                                                         shv[i].from(), shv[i].to(), shv[i].xml()));
        }
        m_ulozene->SetSelection(0);
    }
    m_ulozene->Enable(false);
    m_mainSizer->Add(m_ulozene);

    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

wxString ShPage2::forma() const
{
    if(m_radne->GetValue()) return "R";
    else return "N";
}

wxDateTime ShPage2::start() const
{
    if(m_radne->GetValue()) return wxDateTime::Today();
    else return static_cast<Ulozene*>(m_ulozene->GetClientData(static_cast<unsigned int>(m_ulozene->GetSelection())))->from();
}

wxDateTime ShPage2::end() const
{
    if(m_radne->GetValue()) return wxDateTime::Today();
    else return static_cast<Ulozene*>(m_ulozene->GetClientData(static_cast<unsigned int>(m_ulozene->GetSelection())))->to();
}

int ShPage2::ulozeneID()
{
    if(m_radne->GetValue()) return -1;
    else return static_cast<Ulozene*>(m_ulozene->GetClientData(static_cast<unsigned int>(m_ulozene->GetSelection())))->id();
}

void ShPage2::OnPageChanging(wxWizardEvent &event)
{
    if(event.GetDirection()) m_wizard->fillPage3List();
}

void ShPage2::OnRadne(wxCommandEvent &/*event*/)
{
    m_nasledne->Enable(idbmodel->someUlozeneDphShv()?true:false);
    m_ulozene->Enable(false);
}

void ShPage2::OnNasledne(wxCommandEvent &/*event*/)
{
    m_ulozene->Enable(true);
}

BEGIN_EVENT_TABLE(ShPage3, wxWizardPageSimple)
    EVT_MENU(ID_DPHSH_PAGE3_ADD, ShPage3::OnAdd)
    EVT_MENU(ID_DPHSH_PAGE3_EDIT, ShPage3::OnEdit)
    EVT_MENU(ID_DPHSH_PAGE3_DELETE, ShPage3::OnDelete)
    EVT_LIST_ITEM_RIGHT_CLICK(ID_DPHSH_PAGE3_LIST, ShPage3::OnItemRightClick)
    EVT_LIST_ITEM_ACTIVATED(ID_DPHSH_PAGE3_LIST, ShPage3::OnItemActivated)
    EVT_BUTTON(ID_DPHSH_PAGE3_RELOAD, ShPage3::OnReload)
END_EVENT_TABLE()

ShPage3::ShPage3(wxWizard *parent)
    : wxWizardPageSimple(parent), m_wizard((DphshWizard*)parent)
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    m_bar = new dxToolBar(this);
    m_bar->AddTool(ID_DPHSH_PAGE3_ADD, wxEmptyString, wxBitmap(add_xpm), dxutils::GreyScaledIcon(add_xpm), wxITEM_NORMAL, wxString::FromUTF8("Přidat řádek"));
    m_bar->AddTool(ID_DPHSH_PAGE3_EDIT, wxEmptyString, wxBitmap(edit_xpm), dxutils::GreyScaledIcon(edit_xpm), wxITEM_NORMAL, wxString::FromUTF8("Upravit řádek"));
    m_bar->AddTool(ID_DPHSH_PAGE3_DELETE, wxEmptyString, wxBitmap(delete_xpm), dxutils::GreyScaledIcon(delete_xpm), wxITEM_NORMAL, wxString::FromUTF8("Smazat řádek"));
    m_bar->Realize();
    m_mainSizer->Add(m_bar, 0, wxALL|wxEXPAND, 5);

    m_list = new wxListCtrl(this, ID_DPHSH_PAGE3_LIST, wxDefaultPosition, wxDefaultSize,
                            wxLC_REPORT|wxLC_HRULES|wxLC_VRULES|wxLC_SINGLE_SEL);
    wxListItem col0;
    col0.SetId(0);
    col0.SetText(wxString::FromUTF8("Storno řádek"));
    col0.SetWidth(90);
    m_list->InsertColumn(0, col0);
    wxListItem col1;
    col1.SetId(1);
    col1.SetText(wxString::FromUTF8("Kód země"));
    col1.SetWidth(80);
    m_list->InsertColumn(1, col1);
    wxListItem col2;
    col2.SetId(2);
    col2.SetText(wxString::FromUTF8("DIČ pořizovatele zboží"));
    col2.SetWidth(150);
    m_list->InsertColumn(2, col2);
    wxListItem col3;
    col3.SetId(3);
    col3.SetText(wxString::FromUTF8("Kód plnění"));
    col3.SetWidth(85);
    m_list->InsertColumn(3, col3);
    wxListItem col4;
    col4.SetId(4);
    col4.SetText(wxString::FromUTF8("Počet plnění"));
    col4.SetWidth(90);
    m_list->InsertColumn(4, col4);
    wxListItem col5;
    col5.SetId(5);
    col5.SetText(wxString::FromUTF8("Celková hodnota plnění v Kč"));
    col5.SetWidth(200);
    col5.SetAlign(wxLIST_FORMAT_RIGHT);
    m_list->InsertColumn(5, col5);
    m_mainSizer->Add(m_list, 1, wxALL|wxEXPAND, 5);

    m_mainSizer->Add(new wxButton(this, ID_DPHSH_PAGE3_RELOAD, wxString::FromUTF8("Znovu načíst data z evidence")));

    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void ShPage3::fillList(bool vety)
{
    m_list->DeleteAllItems();
    m_list->Hide();
    if(vety) fillVety();
    for(size_t i=0; i<m_vetyR.GetCount(); i++)
    {
        wxListItem item;
        item.SetId(static_cast<long>(i));
        item.SetText(m_vetyR[i].atributValue("k_storno"));
        m_list->InsertItem(item);
        m_list->SetItem(static_cast<long>(i), 1, m_vetyR[i].atributValue("k_stat"));
        m_list->SetItem(static_cast<long>(i), 2, m_vetyR[i].atributValue("c_vat"));
        m_list->SetItem(static_cast<long>(i), 3, m_vetyR[i].atributValue("k_pln_eu"));
        m_list->SetItem(static_cast<long>(i), 4, m_vetyR[i].atributValue("pln_pocet"));
        m_list->SetItem(static_cast<long>(i), 5, m_vetyR[i].atributValue("pln_hodnota"));
    }
    m_list->Show();
    if(m_vetyR.GetCount())
    {
        m_list->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
        m_bar->EnableTool(ID_DPHSH_PAGE3_EDIT, true);
        m_bar->EnableTool(ID_DPHSH_PAGE3_DELETE, true);
    }
    else
    {
        m_bar->EnableTool(ID_DPHSH_PAGE3_EDIT, false);
        m_bar->EnableTool(ID_DPHSH_PAGE3_DELETE, false);
    }
}

dxVetaArray ShPage3::vetyR() const
{
    return m_vetyR;
}

int ShPage3::obdobi() const
{
    return m_obdobi;
}

int ShPage3::mesic() const
{
    return m_mesic;
}

int ShPage3::ctvrtleti() const
{
    return m_ctvrtleti;
}

int ShPage3::rok() const
{
    return m_rok;
}

bool ShPage3::someVetyR() const
{
    return m_vetyR.GetCount();
}

void ShPage3::fillVety()
{
    m_vetyR.Clear();
    if(m_wizard->forma() == "R") //souhrnné hlášení
    {
        addVetyForDoklady();
    }
    else //následné souhrnné hlášení
    {
        Dphshv shv;
        shv.readFromXml(idbmodel->ulozeneXml(m_wizard->ulozeneID()));
        dxVetaArray vety = shv.vety();
        for(size_t i=0; i<vety.GetCount(); i++)
        {
            if(vety[i].name() == "VetaR")
            {
                vety[i].setAtributValue("k_storno", "A");
                m_vetyR.Add(vety[i]);
            }
            if(vety[i].name() == "VetaD")
            {
                m_mesic = vety[i].atributValueInt("mesic");
                m_ctvrtleti = vety[i].atributValueInt("ctvrt");
                m_rok = vety[i].atributValueInt("rok");
                if(m_mesic) m_obdobi = 0;
                else m_obdobi = 1;
            }
        }
        addVetyForDoklady();
    }
    for(size_t i=0; i<m_vetyR.GetCount(); i++)
    {
        //zaokrouhlit na cele koruny nahoru, tzn 2.3 na 3, 2.7 na 3 atd.
        if(m_vetyR[i].atributValueDouble("pln_hodnotad") != 0.0) m_vetyR[i].setAtributValue("pln_hodnota", int(ceil(m_vetyR[i].atributValueDouble("pln_hodnotad"))));
        m_vetyR[i].clearAtributValue("pln_hodnotad");
    }
}

void ShPage3::addVetyForDoklady()
{
    dxDphDokladArray doklady = idbmodel->dokladyForDph(idbmodel->getDphDruhIdByRadek("20"), m_wizard->start(), m_wizard->end());
    for(size_t i=0; i<doklady.GetCount(); i++)
    {
        double hodnota = 0.0;
        dxDphForDokladArray dph = doklady[i].dph();
        for(size_t j=0; j<dph.GetCount(); j++)
        {
            hodnota += dph[j].zaklad();
        }
        if(hasVetaR(doklady[i].dic(), 0)) editVetaR(doklady[i].dic(), 0, hodnota);
        else addVetaR(doklady[i].dic(), 0, hodnota);
    }
    doklady = idbmodel->dokladyForDph(idbmodel->getDphDruhIdByRadek("21"), m_wizard->start(), m_wizard->end());
    for(size_t i=0; i<doklady.GetCount(); i++)
    {
        double hodnota = 0.0;
        dxDphForDokladArray dph = doklady[i].dph();
        for(size_t j=0; j<dph.GetCount(); j++)
        {
            hodnota += dph[j].zaklad();
        }
        if(hasVetaR(doklady[i].dic(), 3)) editVetaR(doklady[i].dic(), 3, hodnota);
        else addVetaR(doklady[i].dic(), 3, hodnota);
    }
    doklady = idbmodel->dokladyForDph(idbmodel->getDphDruhIdByRadek("31"), m_wizard->start(), m_wizard->end());
    for(size_t i=0; i<doklady.GetCount(); i++)
    {
        double hodnota = 0.0;
        dxDphForDokladArray dph = doklady[i].dph();
        for(size_t j=0; j<dph.GetCount(); j++)
        {
            hodnota += dph[j].zaklad();
        }
        if(hasVetaR(doklady[i].dic(), 2)) editVetaR(doklady[i].dic(), 2, hodnota);
        else addVetaR(doklady[i].dic(), 2, hodnota);
    }
}

bool ShPage3::hasVetaR(const wxString &dic, int kodPlneni)
{
    wxString k_stat = dic.Left(2);
    wxString c_vat = dic.Mid(2);
    for(size_t i=0; i<m_vetyR.GetCount(); i++)
    {
        if(k_stat == m_vetyR[i].atributValue("k_stat") && c_vat == m_vetyR[i].atributValue("c_vat")
                && kodPlneni == m_vetyR[i].atributValueInt("k_pln_eu")
                && m_vetyR[i].atributValue("k_storno").IsEmpty()) return true;
    }
    return false;
}

size_t ShPage3::vetaRIndex(const wxString &dic, int kodPlneni)
{
    wxString k_stat = dic.Left(2);
    wxString c_vat = dic.Mid(2);
    for(size_t i=0; i<m_vetyR.GetCount(); i++)
    {
        if(k_stat == m_vetyR[i].atributValue("k_stat") && c_vat == m_vetyR[i].atributValue("c_vat")
                && kodPlneni == m_vetyR[i].atributValueInt("k_pln_eu")
                && m_vetyR[i].atributValue("k_storno").IsEmpty()) return i;
    }
    return 0;
}

void ShPage3::addVetaR(const wxString &dic, int kodPlneni, double hodnota)
{
    Veta vetar(static_cast<int>(m_vetyR.GetCount()), "VetaR", 1, 1);
    dxAtributArray avetar;
    Atribut c_rad(A_INT, "c_rad", false, 2);
    c_rad.setValue(static_cast<int>(m_vetyR.GetCount())+1);
    avetar.Add(c_rad);
    Atribut c_vat(A_STRING, "c_vat", false, 12);
    c_vat.setValue(dic.Mid(2));
    avetar.Add(c_vat);
    Atribut k_pln_eu(A_INT, "k_pln_eu", false, 1);
    k_pln_eu.setValue(kodPlneni);
    avetar.Add(k_pln_eu);
    Atribut k_stat(A_STRING, "k_stat", false, 2);
    k_stat.setValue(dic.Left(2));
    avetar.Add(k_stat);
    Atribut k_storno(A_STRING, "k_storno", false, 1);
    avetar.Add(k_storno);
    Atribut pln_hodnotad(A_DOUBLE, "pln_hodnotad", false, 16, 2);
    pln_hodnotad.setValue(hodnota);
    avetar.Add(pln_hodnotad);
    Atribut pln_hodnota(A_INT, "pln_hodnota", false, 14);
    avetar.Add(pln_hodnota);
    Atribut pln_pocet(A_INT, "pln_pocet", false, 6);
    pln_pocet.setValue(1);
    avetar.Add(pln_pocet);
    vetar.setAtributy(avetar);
    m_vetyR.Add(vetar);
}

void ShPage3::editVetaR(const wxString &dic, int kodPlneni, double hodnota)
{
    if(hasVetaR(dic, kodPlneni))
    {
        size_t i = vetaRIndex(dic, kodPlneni);
        m_vetyR[i].setAtributValue("pln_hodnotad", m_vetyR[i].atributValueDouble("pln_hodnotad")+hodnota);
        m_vetyR[i].setAtributValue("pln_pocet", m_vetyR[i].atributValueInt("pln_pocet")+1);
    }
}

void ShPage3::OnAdd(wxCommandEvent &/*event*/)
{
    Veta vetar(static_cast<int>(m_vetyR.GetCount()), "VetaR", 1, 1);
    dxAtributArray avetar;
    Atribut c_rad(A_INT, "c_rad", false, 2);
    c_rad.setValue(static_cast<int>(m_vetyR.GetCount())+1);
    avetar.Add(c_rad);
    Atribut c_vat(A_STRING, "c_vat", false, 12);
    avetar.Add(c_vat);
    Atribut k_pln_eu(A_INT, "k_pln_eu", false, 1);
    k_pln_eu.setValue(0);
    avetar.Add(k_pln_eu);
    Atribut k_stat(A_STRING, "k_stat", false, 2);
    avetar.Add(k_stat);
    Atribut k_storno(A_STRING, "k_storno", false, 1);
    avetar.Add(k_storno);
    Atribut pln_hodnota(A_INT, "pln_hodnota", false, 14);
    pln_hodnota.setValue(0);
    avetar.Add(pln_hodnota);
    Atribut pln_pocet(A_INT, "pln_pocet", false, 6);
    pln_pocet.setValue(0);
    avetar.Add(pln_pocet);
    vetar.setAtributy(avetar);
    DphshVetarDialog dialog(this, vetar);
    if(dialog.ShowModal() == wxID_OK)
    {
        m_vetyR.Add(dialog.veta());
        fillList(false);
    }
}

void ShPage3::OnEdit(wxCommandEvent &/*event*/)
{
    long index = selectedIndex();
    if(index == -1) return;
    DphshVetarDialog dialog(this, m_vetyR[static_cast<size_t>(index)]);
    if(dialog.ShowModal() == wxID_OK)
    {
        m_vetyR[static_cast<size_t>(index)] = dialog.veta();
        fillList(false);
    }
}

void ShPage3::OnDelete(wxCommandEvent &/*event*/)
{
    long index = selectedIndex();
    if(index == -1) return;
    wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Smazat řádek pro %s%s?"), m_vetyR[static_cast<size_t>(index)].atributValue("k_stat"),
                           m_vetyR[static_cast<size_t>(index)].atributValue("c_vat")), "Dotaz", wxOK|wxCANCEL|wxICON_QUESTION);
    if(dialog.ShowModal() == wxID_OK)
    {
        m_vetyR.RemoveAt(static_cast<size_t>(index));
        fillList(false);
    }
}

void ShPage3::OnItemRightClick(wxListEvent &event)
{
    long index = selectedIndex();
    if(index == -1) return;
    wxMenu popup;
    popup.Append(ID_DPHSH_PAGE3_ADD, wxString::FromUTF8("Přidat řádek"));
    popup.Append(ID_DPHSH_PAGE3_EDIT, wxString::FromUTF8("Upravit řádek"));
    popup.Append(ID_DPHSH_PAGE3_DELETE, wxString::FromUTF8("Smazat řádek"));
    PopupMenu(&popup, event.GetPoint()+m_list->GetPosition());
}

void ShPage3::OnItemActivated(wxListEvent &event)
{
    DphshVetarDialog dialog(this, m_vetyR[static_cast<size_t>(event.GetIndex())]);
    if(dialog.ShowModal() == wxID_OK)
    {
        m_vetyR[static_cast<size_t>(event.GetIndex())] = dialog.veta();
        fillList(false);
    }
}

void ShPage3::OnReload(wxCommandEvent &/*event*/)
{
    fillList();
}

long ShPage3::selectedIndex()
{
    long index = -1;
    while((index = m_list->GetNextItem(index, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != wxNOT_FOUND)
    {
        return index;
    }
    return index;
}

BEGIN_EVENT_TABLE(ShPage4, wxWizardPageSimple)
    EVT_BUTTON(ID_DPHSH_PRINT, ShPage4::OnPrint)
    EVT_BUTTON(ID_DPHSH_EXPORT, ShPage4::OnExport)
END_EVENT_TABLE()

ShPage4::ShPage4(wxWizard *parent)
    : wxWizardPageSimple(parent), m_wizard(static_cast<DphshWizard*>(parent))
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    m_mainSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Souhrnné hlášení VIES můžete vytisknout nebo\nvyexportovat jako xml soubor.\n\nXml soubor můžete odeslat pomocí datové schránky"
                                                                         "\nnebo podat pomocí")));
    m_mainSizer->Add(new wxHyperlinkCtrl(this, wxID_ANY, "www.daneelektronicky.cz", "http://www.daneelektronicky.cz/"));

    m_mainSizer->AddStretchSpacer();

    wxBoxSizer *buttons = new wxBoxSizer(wxHORIZONTAL);
    m_print = new wxButton(this, ID_DPHSH_PRINT, wxString::FromUTF8("Tisk hlášení"));
    buttons->Add(m_print);
    buttons->AddSpacer(5);
    m_export = new wxButton(this, ID_DPHSH_EXPORT, wxString::FromUTF8("Export hlášení"));
    buttons->Add(m_export);
    m_mainSizer->Add(buttons, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void ShPage4::OnPrint(wxCommandEvent &/*event*/)
{
    if(!m_wizard->someVetyR())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Nejsou žádné doklady pro vybrané období"), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        return;
    }
    m_wizard->fillData();
    DphshPrint print(this, m_wizard->vety());
    //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
    print.Print();
#else
    print.Preview();
#endif
}

void ShPage4::OnExport(wxCommandEvent &/*event*/)
{
    if(!m_wizard->someVetyR())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Nejsou žádné doklady pro vybrané období"), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        return;
    }
    wxFileDialog dlg(this, wxString::FromUTF8("Exportovat hlášení do"), wxEmptyString, wxEmptyString, "XML soubory (*.xml)|*.xml",
                     wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
    if(dlg.ShowModal() != wxID_OK) return;
    wxString path = dlg.GetPath();
    dxutils::correctEmptyFileExt("xml", path);
    m_wizard->fillData();
    m_wizard->saveXml(path);
}

IMPLEMENT_DYNAMIC_CLASS(DphshWizard, wxWizard)

BEGIN_EVENT_TABLE(DphshWizard, wxWizard)
    EVT_WIZARD_PAGE_CHANGED(wxID_ANY, DphshWizard::OnPageChanged)
    EVT_WIZARD_FINISHED(wxID_ANY, DphshWizard::OnFinished)
END_EVENT_TABLE()

DphshWizard::DphshWizard(wxWindow *parent)
    : wxWizard(parent, wxID_ANY, wxString::FromUTF8("Průvodce souhrnným hlášením VIES"))
{
    m_page1 = new ShPage1(this);

    m_page2 = new ShPage2(this);
    m_page1->Chain(m_page2);

    m_page3 = new ShPage3(this);
    m_page2->Chain(m_page3);

    m_page4 = new ShPage4(this);
    m_page3->Chain(m_page4);

    GetPageAreaSizer()->Add(m_page1);
    GetPageAreaSizer()->Add(m_page2);
    GetPageAreaSizer()->Add(m_page3);
    GetPageAreaSizer()->Add(m_page4);
}

void DphshWizard::fillData()
{
    if(!m_dphshv.isEmpty()) return;

    Priznani pr = idbmodel->priznani();

    //VetaD
    m_dphshv.setAtributValue("d_poddp", m_page1->vyhotoveni());
    m_dphshv.setAtributValue("dokument", "SHV");
    m_dphshv.setAtributValue("k_uladis", "DPH");
    if(m_page2->forma() == "R")
    {
        if(m_page1->obdobi() == 1) m_dphshv.setAtributValue("ctvrt", m_page1->ctvrtleti());
        if(m_page1->obdobi() == 0) m_dphshv.setAtributValue("mesic", m_page1->mesic());
        m_dphshv.setAtributValue("rok", m_page1->rok());
    }
    else
    {
        if(m_page3->obdobi() == 1) m_dphshv.setAtributValue("ctvrt", m_page3->ctvrtleti());
        if(m_page3->obdobi() == 0) m_dphshv.setAtributValue("mesic", m_page3->mesic());
        m_dphshv.setAtributValue("rok", m_page3->rok());
    }
    m_dphshv.setAtributValue("shvies_forma", m_page2->forma());

    //VetaP
    m_dphshv.setAtributValue("c_orient", pr.c_orient());
    m_dphshv.setAtributValue("c_pop", pr.c_pop());
    m_dphshv.setAtributValue("c_pracufo", pr.c_pracufo());
    m_dphshv.setAtributValue("c_ufo", pr.c_ufo());
    m_dphshv.setAtributValue("dic", pr.dic());
    m_dphshv.setAtributValue("jmeno", pr.jmeno());
    m_dphshv.setAtributValue("naz_obce", pr.naz_obce());
    m_dphshv.setAtributValue("opr_jmeno", pr.opr_jmeno());
    m_dphshv.setAtributValue("opr_postaveni", pr.opr_postaveni());
    m_dphshv.setAtributValue("opr_prijmeni", pr.opr_prijmeni());
    m_dphshv.setAtributValue("prijmeni", pr.prijmeni());
    m_dphshv.setAtributValue("psc", pr.psc());
    m_dphshv.setAtributValue("stat", pr.stat());
    m_dphshv.setAtributValue("titul", pr.titul());
    m_dphshv.setAtributValue("typ_ds", "F");
    m_dphshv.setAtributValue("ulice", pr.ulice());
    if(pr.zast_typ() == "F") m_dphshv.setAtributValue("zast_dat_nar", pr.zast_dat_nar());
    m_dphshv.setAtributValue("zast_ev_cislo", pr.zast_ev_cislo());
    if(pr.zast_typ() == "P") m_dphshv.setAtributValue("zast_ic", pr.zast_ic());
    m_dphshv.setAtributValue("zast_jmeno", pr.zast_jmeno());
    m_dphshv.setAtributValue("zast_kod", pr.zast_kod());
    m_dphshv.setAtributValue("zast_nazev", pr.zast_nazev());
    m_dphshv.setAtributValue("zast_prijmeni", pr.zast_prijmeni());
    m_dphshv.setAtributValue("zast_typ", pr.zast_typ());

    if(someVetyR())
    {
        dxVetaArray vetyR = m_page3->vetyR();
        for(size_t i=0; i<vetyR.GetCount(); i++)
        {
            m_dphshv.addVetaR(vetyR[i]);
        }
    }
}

dxVetaArray DphshWizard::vety() const
{
    return m_dphshv.vety();
}

void DphshWizard::saveXml(const wxString &path)
{
    m_dphshv.saveToXml(path);
}

wxDateTime DphshWizard::start() const
{
    if(m_page2->forma() == "N") return m_page2->start();
    else return m_page1->start();
}

wxDateTime DphshWizard::end() const
{
    if(m_page2->forma() == "N") return m_page2->end();
    else return m_page1->end();
}

wxString DphshWizard::forma() const
{
    return m_page2->forma();
}

int DphshWizard::ulozeneID()
{
    return m_page2->ulozeneID();
}

void DphshWizard::fillPage3List()
{
    m_page3->fillList();
}

bool DphshWizard::someVetyR() const
{
    return m_page3->someVetyR();
}

void DphshWizard::OnPageChanged(wxWizardEvent &event)
{
    if(!event.GetDirection()) m_dphshv.clearData();
}

void DphshWizard::OnFinished(wxWizardEvent &/*event*/)
{
    if(idbmodel->someDokladForDphSh(m_page1->start(), m_page1->end()) && m_page2->forma() == "R")
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Uložit souhrnné hlášení?"), "Dotaz", wxOK|wxCANCEL|wxICON_QUESTION);
        if(dialog.ShowModal() == wxID_OK)
        {
            m_dphshv.clearData();
            fillData();
            wxString popis = wxString::FromUTF8("Souhrnné hlášení ");
            if(m_page1->obdobi()==0) popis.Append(dxutils::mesicString(m_page1->mesic()));
            else popis.Append(dxutils::ctvrtletiString(m_page1->ctvrtleti()));
            popis.Append(wxString::Format("/%d", m_page1->rok()));
            if(idbmodel->hasUlozene(1, "R", m_page1->obdobi(), m_page1->start(), m_page1->end()))
            {
                int id = idbmodel->getUlozeneId(1, "R", m_page1->obdobi(), m_page1->start(), m_page1->end());
                idbmodel->editUlozene(Ulozene(id, 1, popis,
                                              "R", m_page1->obdobi(), m_page1->mesic(), m_page1->ctvrtleti(), m_page1->rok(),
                                              m_page1->start(), m_page1->end(), idbmodel->ulozeneXml(id)));
                m_dphshv.saveToXml(idbmodel->ulozeneXml(id));
            }
            else
            {
                wxString xmlpath = dxutils::getUniqueXmlPath("DPHSHV");
                m_dphshv.saveToXml(xmlpath);
                idbmodel->addUlozene(Ulozene(idbmodel->getLastId("ulozene"), 1, popis,
                                                 "R", m_page1->obdobi(), m_page1->mesic(), m_page1->ctvrtleti(), m_page1->rok(),
                                                 m_page1->start(), m_page1->end(), xmlpath));
            }
        }
    }
}
