/***************************************************************
 * Name:      dphshwizard.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#ifndef DPHSHWIZARD_H
#define DPHSHWIZARD_H

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif
#include <wx/wizard.h>
#include <wx/datectrl.h>
#include <wx/dateevt.h>
#include <wx/spinctrl.h>
#include <wx/listctrl.h>
#include "dphshv.h"
#include "model/dbmodel.h"
#include "gui/widgets/dxtoolbar.h"

class DphshWizard;

class ShPage1 : public wxWizardPageSimple
{
    DECLARE_EVENT_TABLE()
public:
    explicit ShPage1(wxWizard *parent);

    wxDateTime vyhotoveni() const;
    int obdobi() const; //0 - mesic, 1 - ctvrtleti
    int mesic() const;
    int ctvrtleti() const;
    int rok() const;
    wxDateTime start() const;
    wxDateTime end() const;
private:
    wxBoxSizer *m_mainSizer;
    wxButton *m_priznani;
    wxRadioButton *m_mesicBtn;
    wxChoice *m_mesicSelect;
    wxChoice *m_mesicRok;
    wxRadioButton *m_ctvrtletiBtn;
    wxChoice *m_ctvrtletiSelect;
    wxChoice *m_ctvrtletiRok;
    wxDatePickerCtrl *m_vyhotoveni;

    void OnPriznani(wxCommandEvent &event);
    void OnMesic(wxCommandEvent& event);
    void OnCtvrtleti(wxCommandEvent& event);
};

class ShPage2 : public wxWizardPageSimple
{
    DECLARE_EVENT_TABLE()
public:
    explicit ShPage2(wxWizard *parent);

    wxString forma() const;
    wxDateTime start() const;
    wxDateTime end() const;
    int ulozeneID();
private:
    DphshWizard *m_wizard;
    wxBoxSizer *m_mainSizer;
    wxRadioButton *m_radne;
    wxRadioButton *m_nasledne;
    wxChoice *m_ulozene;

    void OnPageChanging(wxWizardEvent& event);
    void OnRadne(wxCommandEvent& event);
    void OnNasledne(wxCommandEvent& event);
};

class ShPage3 : public wxWizardPageSimple
{
    DECLARE_EVENT_TABLE()
public:
    explicit ShPage3(wxWizard *parent);

    // vety = true, naplni se m_vetyR
    void fillList(bool vety = true);
    dxVetaArray vetyR() const;
    int obdobi() const; //0 - mesic, 1 - ctvrtleti
    int mesic() const;
    int ctvrtleti() const;
    int rok() const;
    bool someVetyR() const;
private:
    DphshWizard *m_wizard;
    wxBoxSizer *m_mainSizer;
    wxListCtrl *m_list;
    dxToolBar *m_bar;
    dxVetaArray m_vetyR;
    int m_obdobi;
    int m_mesic;
    int m_ctvrtleti;
    int m_rok;

    void fillVety();
    void addVetyForDoklady();
    bool hasVetaR(const wxString &dic, int kodPlneni);
    size_t vetaRIndex(const wxString &dic, int kodPlneni);
    void addVetaR(const wxString &dic, int kodPlneni, double hodnota);
    void editVetaR(const wxString &dic, int kodPlneni, double hodnota);
    void OnAdd(wxCommandEvent& event);
    void OnEdit(wxCommandEvent& event);
    void OnDelete(wxCommandEvent& event);
    void OnItemRightClick(wxListEvent& event);
    void OnItemActivated(wxListEvent& event);
    void OnReload(wxCommandEvent& event);
    long selectedIndex();
};

class ShPage4 : public wxWizardPageSimple
{
    DECLARE_EVENT_TABLE()
public:
    explicit ShPage4(wxWizard *parent);

private:
    DphshWizard *m_wizard;
    wxBoxSizer *m_mainSizer;
    wxButton *m_print;
    wxButton *m_export;

    void OnPrint(wxCommandEvent& event);
    void OnExport(wxCommandEvent& event);
};

class DphshWizard : public wxWizard
{
    DECLARE_DYNAMIC_CLASS(Dphdp3Wizard)
    DECLARE_EVENT_TABLE()
public:
    DphshWizard() : m_page1(nullptr), m_page2(nullptr), m_page3(nullptr), m_page4(nullptr) {}
    explicit DphshWizard(wxWindow *parent);

    wxWizardPage *GetFirstPage() const { return m_page1; }
    void fillData();
    dxVetaArray vety() const;
    void saveXml(const wxString &path);
    wxDateTime start() const;
    wxDateTime end() const;
    wxString forma() const;
    int ulozeneID();
    void fillPage3List();
    bool someVetyR() const;
private:
    ShPage1 *m_page1;
    ShPage2 *m_page2;
    ShPage3 *m_page3;
    ShPage4 *m_page4;
    Dphshv m_dphshv;

    void OnPageChanged(wxWizardEvent& event);
    void OnFinished(wxWizardEvent& event);
};

#endif  /* DPHSHWIZARD_H */

