/***************************************************************
 * Name:      prijmyvydajedialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include "prijmyvydajedialog.h"
#include "prijmyvydajeprint.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"

IMPLEMENT_DYNAMIC_CLASS(PrijmyVydajeDialog, wxDialog)

BEGIN_EVENT_TABLE(PrijmyVydajeDialog, wxDialog)
    EVT_BUTTON(ID_PRIJMYVYDAJE_PRINT, PrijmyVydajeDialog::OnPrint)
    EVT_BUTTON(wxID_CLOSE, PrijmyVydajeDialog::OnClose)
END_EVENT_TABLE()

PrijmyVydajeDialog::PrijmyVydajeDialog(wxWindow *parent)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Přehled příjmů a výdajů"))
{
    createControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void PrijmyVydajeDialog::createControls()
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Rok"), wxGBPosition(0,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_rok = new wxChoice(this, wxID_ANY);
    m_rok->Append(idbmodel->getObdobiAsStrings());
    m_rok->SetStringSelection(wxString::Format("%d", idbmodel->currentRok()));
    gridSizer->Add(m_rok, wxGBPosition(0,1), wxDefaultSpan, wxEXPAND);
    if(idbmodel->isFirmaPausal())
    {
        m_pausal = new wxCheckBox(this, wxID_ANY, wxString::FromUTF8("Použít paušální výdaje"));
        m_pausal->SetValue(true);
        gridSizer->Add(m_pausal, wxGBPosition(1,0), wxGBSpan(1,2), wxEXPAND);
    }

    m_mainSizer->Add(gridSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    m_mainSizer->Add(new wxButton(this, ID_PRIJMYVYDAJE_PRINT, wxString::FromUTF8("Vytisknout přehled")), 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_CLOSE, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void PrijmyVydajeDialog::OnPrint(wxCommandEvent& /*event*/)
{
    PrijmyVydajePrint print(this, wxAtoi(m_rok->GetStringSelection()), idbmodel->isFirmaPausal()?m_pausal->IsChecked():false);
    //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
    print.Print();
#else
    print.Preview();
#endif
}

void PrijmyVydajeDialog::OnClose(wxCommandEvent &/*event*/)
{
    EndModal(wxID_OK);
}
