/***************************************************************
 * Name:      prijmyvydajeprint.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#include "prijmyvydajeprint.h"
#include "utils/dxutils.h"

PrijmyVydajePrint::PrijmyVydajePrint(wxWindow *parent, int rok, bool pausal)
    : DocumentPrint(parent, wxString::FromUTF8("Přehled příjmů a výdajů")), m_rok(rok), m_pausal(pausal)
{
    m_marginLeft = 10;
    m_marginRight = 10;
    m_marginTop = 10;
    m_marginBottom = 10;
    createDocument();
}

void PrijmyVydajePrint::createDocument()
{
    int height = 0;
    int page = 1;
    int pageHeight = m_pageHeight-m_marginTop-m_marginBottom-10;
    m_document = new Document;
    TextElement text = TextElement(page, wxString::Format(wxString::FromUTF8("Přehled daňových příjmů a výdajů za rok %d"), m_rok), wxPoint(m_marginLeft,m_marginTop), m_pageWidth-m_marginLeft-m_marginRight,
                                   wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(14).FaceName(FACENAME).Bold())));
    m_document->addElement(text);
    height += text.height();
    Firma fi = idbmodel->firma();
    Adresa ad = idbmodel->getAdresa(fi.adresaid());
    wxArrayString firma;
    firma.Add(fi.nazev());
    firma.Add(ad.isValid() ? wxString::Format("%s %s", ad.ulice(), ad.cp()) : " ");
    firma.Add(ad.isValid() ? wxString::Format("%s %s", ad.psc(), ad.misto()) : " ");
    firma.Add(ad.isValid() ? ad.zeme() : " ");
    firma.Add(wxString::Format(wxString::FromUTF8("IČ: %s"), fi.ic()));
    TextParagraphElement paragraph = TextParagraphElement(page, firma, wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight);
    paragraph.setStyle(m_document->style());
    m_document->addElement(paragraph);
    height += paragraph.height();
    dxDenikArray denik = idbmodel->danovePrijmyVydajeForRok(m_rok);
    if(denik.GetCount())
    {
        height += 5;
        idbmodel->sortDenik(denik, DbModel::DS_SLOUPEC, true);
        text = TextElement(page, wxString::FromUTF8("Daňové příjmy"), wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight, wxTEXT_ALIGNMENT_LEFT,
                           TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Bold())));
        m_document->addElement(text);
        height += text.height();
        wxArrayString texty;
        wxArrayInt width;
        width.Add(m_pageWidth-m_marginLeft-m_marginRight-50);
        width.Add(50);
        wxArrayInt alig;
        alig.Add(wxTEXT_ALIGNMENT_LEFT);
        alig.Add(wxTEXT_ALIGNMENT_RIGHT);
        int sloupec = idbmodel->denikSloupecByDenikId(denik[0].id());
        double prijem = 0.0, prijmy = 0.0;
        double vydaj = 0.0, vydaje = 0.0;
        for(size_t i =0; i < denik.GetCount(); i++)
        {
            if(idbmodel->isDenikPrijem(denik[i].id()))
            {
                if(sloupec != idbmodel->denikSloupecByDenikId(denik[i].id()))
                {
                    texty.Add(idbmodel->getDenikSloupec(sloupec).popis().Before('('));
                    texty.Add(wxString::Format("%.2f", prijem));
                    TableRowElement row = TableRowElement(page, texty, wxPoint(m_marginLeft,m_marginTop+height), width, alig);
                    row.setBorder(true);
                    row.setStyle(m_document->style());
                    height += row.height();
                    if(height > pageHeight)
                    {
                        height -= row.height();
                        page++;
                        row.setPage(page);
                        row.setPos(wxPoint(m_marginLeft,m_marginTop));
                        m_document->addElement(row);
                        height = row.height();
                    }
                    else
                    {
                        m_document->addElement(row);
                    }
                    prijem = denik[i].castka();
                    prijmy += denik[i].castka();
                    sloupec = idbmodel->denikSloupecByDenikId(denik[i].id());
                }
                else
                {
                    prijem += denik[i].castka();
                    prijmy += denik[i].castka();
                }
            }
        }
        if(prijem != 0.0)
        {
            texty.Clear();
            texty.Add(idbmodel->getDenikSloupec(sloupec).popis().Before('('));
            texty.Add(wxString::Format("%.2f", prijem));
            TableRowElement row = TableRowElement(page, texty, wxPoint(m_marginLeft,m_marginTop+height), width, alig);
            row.setBorder(true);
            row.setStyle(m_document->style());
            height += row.height();
            if(height > pageHeight)
            {
                height -= row.height();
                page++;
                row.setPage(page);
                row.setPos(wxPoint(m_marginLeft,m_marginTop));
                m_document->addElement(row);
                height = row.height();
            }
            else
            {
                m_document->addElement(row);
            }
        }
        texty.Clear();
        texty.Add("Celkem");
        texty.Add(wxString::Format("%.2f", prijmy));
        TableRowElement row = TableRowElement(page, texty, wxPoint(m_marginLeft,m_marginTop+height), width, alig);
        row.setBorder(true);
        row.setStyle(TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Bold())));
        height += row.height();
        if(height > pageHeight)
        {
            height -= row.height();
            page++;
            row.setPage(page);
            row.setPos(wxPoint(m_marginLeft,m_marginTop));
            m_document->addElement(row);
            height = row.height();
        }
        else
        {
            m_document->addElement(row);
        }
        if(!m_pausal)
        {
            height += 5;
            idbmodel->sortDenik(denik, DbModel::DS_SLOUPEC, true);
            text = TextElement(page, wxString::FromUTF8("Daňové výdaje"), wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight, wxTEXT_ALIGNMENT_LEFT,
                               TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Bold())));
            m_document->addElement(text);
            height += text.height();
            for(size_t i =0; i < denik.GetCount(); i++)
            {
                if(!idbmodel->isDenikPrijem(denik[i].id()))
                {
                    sloupec = idbmodel->denikSloupecByDenikId(denik[i].id());
                    break;
                }
            }
            for(size_t i =0; i < denik.GetCount(); i++)
            {
                if(!idbmodel->isDenikPrijem(denik[i].id()))
                {
                    if(sloupec != idbmodel->denikSloupecByDenikId(denik[i].id()))
                    {
                        texty.Clear();
                        texty.Add(idbmodel->getDenikSloupec(sloupec).popis().Before('('));
                        texty.Add(wxString::Format("%.2f", vydaj));
                        TableRowElement row = TableRowElement(page, texty, wxPoint(m_marginLeft,m_marginTop+height), width, alig);
                        row.setBorder(true);
                        row.setStyle(m_document->style());
                        height += row.height();
                        if(height > pageHeight)
                        {
                            height -= row.height();
                            page++;
                            row.setPage(page);
                            row.setPos(wxPoint(m_marginLeft,m_marginTop));
                            m_document->addElement(row);
                            height = row.height();
                        }
                        else
                        {
                            m_document->addElement(row);
                        }
                        vydaj = denik[i].castka();
                        vydaje += denik[i].castka();
                        sloupec = idbmodel->denikSloupecByDenikId(denik[i].id());
                    }
                    else
                    {
                        vydaj += denik[i].castka();
                        vydaje += denik[i].castka();
                    }
                }
            }
            if(vydaj != 0.0)
            {
                texty.Clear();
                texty.Add(idbmodel->getDenikSloupec(sloupec).popis().Before('('));
                texty.Add(wxString::Format("%.2f", vydaj));
                TableRowElement row = TableRowElement(page, texty, wxPoint(m_marginLeft,m_marginTop+height), width, alig);
                row.setBorder(true);
                row.setStyle(m_document->style());
                height += row.height();
                if(height > pageHeight)
                {
                    height -= row.height();
                    page++;
                    row.setPage(page);
                    row.setPos(wxPoint(m_marginLeft,m_marginTop));
                    m_document->addElement(row);
                    height = row.height();
                }
                else
                {
                    m_document->addElement(row);
                }
            }
            texty.Clear();
            texty.Add("Celkem");
            texty.Add(wxString::Format("%.2f", vydaje));
            TableRowElement row = TableRowElement(page, texty, wxPoint(m_marginLeft,m_marginTop+height), width, alig);
            row.setBorder(true);
            row.setStyle(TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Bold())));
            height += row.height();
            if(height > pageHeight)
            {
                height -= row.height();
                page++;
                row.setPage(page);
                row.setPos(wxPoint(m_marginLeft,m_marginTop));
                m_document->addElement(row);
                height = row.height();
            }
            else
            {
                m_document->addElement(row);
            }
        }
        else
        {
            height += 5;
            idbmodel->sortDenik(denik, DbModel::DS_PAUSAL, false);
            text = TextElement(page, wxString::FromUTF8("Paušální výdaje"), wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight, wxTEXT_ALIGNMENT_LEFT,
                               TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Bold())));
            m_document->addElement(text);
            height += text.height();
            vydaj = 0.0;
            vydaje = 0.0;
            int pausal = denik[0].pausal();
            for(size_t i =0; i < denik.GetCount(); i++)
            {
                if(idbmodel->isDenikPrijem(denik[i].id()))
                {
                    if(pausal != denik[i].pausal())
                    {
                        texty.Clear();
                        texty.Add(wxString::Format(wxString::FromUTF8("Paušální výdaje %d%%"), pausal));
                        texty.Add(wxString::Format("%.2f", vydaj));
                        TableRowElement row = TableRowElement(page, texty, wxPoint(m_marginLeft,m_marginTop+height), width, alig);
                        row.setBorder(true);
                        row.setStyle(m_document->style());
                        height += row.height();
                        if(height > pageHeight)
                        {
                            height -= row.height();
                            page++;
                            row.setPage(page);
                            row.setPos(wxPoint(m_marginLeft,m_marginTop));
                            m_document->addElement(row);
                            height = row.height();
                        }
                        else
                        {
                            m_document->addElement(row);
                        }
                        vydaj = denik[i].castka()*denik[i].pausal()/100.0;
                        vydaje += denik[i].castka()*denik[i].pausal()/100.0;
                        pausal = denik[i].pausal();
                    }
                    else
                    {
                        vydaj += denik[i].castka()*denik[i].pausal()/100.0;
                        vydaje += denik[i].castka()*denik[i].pausal()/100.0;
                    }
                }
            }
            if(vydaj != 0.0)
            {
                texty.Clear();
                texty.Add(wxString::Format(wxString::FromUTF8("Paušální výdaje %d%%"), pausal));
                texty.Add(wxString::Format("%.2f", vydaj));
                TableRowElement row = TableRowElement(page, texty, wxPoint(m_marginLeft,m_marginTop+height), width, alig);
                row.setBorder(true);
                row.setStyle(m_document->style());
                height += row.height();
                if(height > pageHeight)
                {
                    height -= row.height();
                    page++;
                    row.setPage(page);
                    row.setPos(wxPoint(m_marginLeft,m_marginTop));
                    m_document->addElement(row);
                    height = row.height();
                }
                else
                {
                    m_document->addElement(row);
                }
            }
            texty.Clear();
            texty.Add("Celkem");
            texty.Add(wxString::Format("%.2f", vydaje));
            TableRowElement row = TableRowElement(page, texty, wxPoint(m_marginLeft,m_marginTop+height), width, alig);
            row.setBorder(true);
            row.setStyle(TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Bold())));
            height += row.height();
            if(height > pageHeight)
            {
                height -= row.height();
                page++;
                row.setPage(page);
                row.setPos(wxPoint(m_marginLeft,m_marginTop));
                m_document->addElement(row);
                height = row.height();
            }
            else
            {
                m_document->addElement(row);
            }
        }
        height += 5;
        text = TextElement(page, prijmy>vydaje?wxString::Format("Zisk: %.2f", prijmy-vydaje):wxString::Format(wxString::FromUTF8("Ztráta: %.2f"), prijmy-vydaje), wxPoint(m_marginLeft,m_marginTop+height),
                           m_pageWidth-m_marginLeft-m_marginRight, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Bold())));
        m_document->addElement(text);
        height += text.height();
        if(height > pageHeight)
        {
            height -= text.height();
            page++;
            text.setPage(page);
            text.setPos(wxPoint(m_marginLeft,m_marginTop));
            m_document->addElement(text);
        }
        else
        {
            m_document->addElement(text);
        }
    }
    else
    {
        height += 5;
        text = TextElement(page, wxString::FromUTF8("Žádné daňové příjmy a výdaje"), wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight, wxTEXT_ALIGNMENT_LEFT,
                           TextStyle(wxFont(wxFontInfo(14).FaceName(FACENAME).Bold())));
        m_document->addElement(text);
    }
}
