/***************************************************************
 * Name:      priznanidialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include <wx/arrimpl.cpp>
#include "priznanidialog.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include <wx/valnum.h>
#include <wx/file.h>
#include <wx/stdpaths.h>
#include "okecdialog.h"

WX_DEFINE_OBJARRAY(dxObecArray)
WX_DEFINE_OBJARRAY(dxUfoArray)
WX_DEFINE_OBJARRAY(dxPracufoArray)

IMPLEMENT_DYNAMIC_CLASS(PriznaniDialog, wxDialog)

BEGIN_EVENT_TABLE(PriznaniDialog, wxDialog)
    EVT_BUTTON(wxID_OK, PriznaniDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, PriznaniDialog::OnCancel)
    EVT_CHOICE(ID_PRIZNANI_CUFO, PriznaniDialog::OnCufo)
    EVT_BUTTON(ID_PRIZNANI_COKEC, PriznaniDialog::OnCokecSelect)
    EVT_CHOICE(ID_PRIZNANI_ZAST_TYP, PriznaniDialog::OnZastTyp)
    EVT_TEXT(ID_PRIZNANI_PSC, PriznaniDialog::OnPsc)
END_EVENT_TABLE()

PriznaniDialog::PriznaniDialog(wxWindow *parent, bool edit)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Nastavení přiznání")), m_edit(edit)
{
    fillArrays();
    createControls();
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void PriznaniDialog::fillArrays()
{
    wxXmlDocument obce;
    obce.Load(wxStandardPaths::Get().GetUserDataDir()+"/obce.xml");
    if(obce.IsOk() && obce.GetRoot()->GetName() == "Ciselnik")
    {
        wxXmlNode *child = obce.GetRoot()->GetChildren();
        while(child)
        {
            m_obceArray.Add(new Obec(child->GetAttribute("naz_obce"), wxAtoi(child->GetAttribute("psc")), wxAtoi(child->GetAttribute("c_fu"))));
            child = child->GetNext();
        }
    }
    wxXmlDocument ufo;
    ufo.Load(wxStandardPaths::Get().GetUserDataDir()+"/ufo.xml");
    if(ufo.IsOk() && ufo.GetRoot()->GetName() == "Ciselnik")
    {
        wxXmlNode *child = ufo.GetRoot()->GetChildren();
        while(child)
        {
            m_ufoArray.Add(new Ufo(child->GetAttribute("nazu_ufo"), wxAtoi(child->GetAttribute("c_ufo"))));
            child = child->GetNext();
        }
    }
    wxXmlDocument pracufo;
    pracufo.Load(wxStandardPaths::Get().GetUserDataDir()+"/pracufo.xml");
    if(pracufo.IsOk() && pracufo.GetRoot()->GetName() == "Ciselnik")
    {
        wxXmlNode *child = pracufo.GetRoot()->GetChildren();
        while(child)
        {
            m_pracufoArray.Add(new Pracufo(child->GetAttribute("nazu_pracufo"), wxAtoi(child->GetAttribute("c_pracufo")), wxAtoi(child->GetAttribute("c_pufo")),
                                           wxAtoi(child->GetAttribute("c_ufo"))));
            child = child->GetNext();
        }
    }
}

void PriznaniDialog::createControls()
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxBoxSizer *sizer = new wxBoxSizer(wxHORIZONTAL);
    sizer->Add(createFirmaGrid(), 0, wxALL, 5);
    sizer->Add(createZastupceGrid(), 0, wxALL, 5);
    m_mainSizer->Add(sizer, 0, wxALL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

wxGridBagSizer *PriznaniDialog::createFirmaGrid()
{
    int i=0; //row
    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Jméno")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_jmeno = new wxTextCtrl(this, wxID_ANY);
    m_jmeno->SetMaxLength(20);
    gridSizer->Add(m_jmeno, wxGBPosition(i,1), wxGBSpan(1,5), wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Přijmení")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_prijmeni = new wxTextCtrl(this, wxID_ANY);
    m_prijmeni->SetMaxLength(36);
    gridSizer->Add(m_prijmeni, wxGBPosition(i,1), wxGBSpan(1,5), wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Titul"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_titul = new wxTextCtrl(this, wxID_ANY, "", wxDefaultPosition, wxSize(80,-1));
    m_titul->SetMaxLength(10);
    gridSizer->Add(m_titul, wxGBPosition(i,1), wxDefaultSpan);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Ulice"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_ulice = new wxTextCtrl(this, wxID_ANY, "", wxDefaultPosition, wxSize(145,-1));
    m_ulice->SetMaxLength(38);
    gridSizer->Add(m_ulice, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("čp")), wxGBPosition(i,2), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_cpopText = new wxTextCtrl(this, wxID_ANY, "", wxDefaultPosition, wxSize(45,-1), 0, wxMakeIntegerValidator(&m_cpop));
    m_cpopText->SetMaxLength(6);
    m_cpopText->SetToolTip(wxString::FromUTF8("Číslo popisné"));
    gridSizer->Add(m_cpopText, wxGBPosition(i,3), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("čo")), wxGBPosition(i,4), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_corient = new wxTextCtrl(this, wxID_ANY, "", wxDefaultPosition, wxSize(45,-1));
    m_corient->SetMaxLength(4);
    m_corient->SetToolTip(wxString::FromUTF8("Číslo orientační"));
    gridSizer->Add(m_corient, wxGBPosition(i,5), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("PSČ")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_pscText = new wxTextCtrl(this, ID_PRIZNANI_PSC, "", wxDefaultPosition, wxDefaultSize, 0, wxMakeIntegerValidator(&m_psc));
    m_pscText->SetMaxLength(5);
    gridSizer->Add(m_pscText, wxGBPosition(i,1), wxDefaultSpan);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Obec"), wxGBPosition(i,2), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_nazobce = new wxComboBox(this, wxID_ANY);
    m_nazobce->SetMaxLength(48);
    gridSizer->Add(m_nazobce, wxGBPosition(i,3), wxGBSpan(1,3), wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("DIČ")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    wxTextValidator txtval(wxFILTER_DIGITS, &m_dic);
    m_dicText = new wxTextCtrl(this, wxID_ANY, "", wxDefaultPosition, wxDefaultSize, 0, txtval);
    m_dicText->SetMaxLength(10);
    m_dicText->SetToolTip(wxString::FromUTF8("Kmenová část daňového identifikačního čísla"));
    gridSizer->Add(m_dicText, wxGBPosition(i,1), wxGBSpan(1,5), wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "E-mail"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_email = new wxTextCtrl(this, wxID_ANY);
    m_email->SetMaxLength(255);
    gridSizer->Add(m_email, wxGBPosition(i,1), wxGBSpan(1,5), wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Identifikátor datové\nschránky")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_iddats = new wxTextCtrl(this, wxID_ANY);
    m_iddats->SetMaxLength(7);
    gridSizer->Add(m_iddats, wxGBPosition(i,1), wxGBSpan(1,5), wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Telefon"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_ctelef = new wxTextCtrl(this, wxID_ANY);
    m_ctelef->SetMaxLength(14);
    gridSizer->Add(m_ctelef, wxGBPosition(i,1), wxGBSpan(1,5), wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Finančním úřad")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_cufo = new wxChoice(this, ID_PRIZNANI_CUFO);
    for(size_t i=0; i<m_ufoArray.GetCount(); i++)
    {
        m_cufo->Append(m_ufoArray[i]->nazev(), m_ufoArray[i]);
    }
    gridSizer->Add(m_cufo, wxGBPosition(i,1), wxGBSpan(1,5), wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Územní pracoviště")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_cpracufo = new wxChoice(this, wxID_ANY);
    gridSizer->Add(m_cpracufo, wxGBPosition(i,1), wxGBSpan(1,5), wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Hlavní ekonomická\nčinnost")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_cokecText = new wxTextCtrl(this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_READONLY);
    gridSizer->Add(m_cokecText, wxGBPosition(i,1), wxGBSpan(1,4), wxEXPAND);
    m_cokecSelect = new wxButton(this, ID_PRIZNANI_COKEC, "...", wxDefaultPosition, wxSize(30,-1));
    gridSizer->Add(m_cokecSelect, wxGBPosition(i,5), wxDefaultSpan, wxEXPAND);

    return gridSizer;
}

wxGridBagSizer *PriznaniDialog::createZastupceGrid()
{
    int i=0; //row
    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Typ podepisující osoby")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_zast_typ = new wxChoice(this, ID_PRIZNANI_ZAST_TYP);
    m_zast_typ->Append(wxString::FromUTF8("Žádná"));
    m_zast_typ->Append(wxString::FromUTF8("Fyzická osoba"));
    m_zast_typ->Append(wxString::FromUTF8("Právnická osoba"));
    m_zast_typ->SetSelection(0);
    gridSizer->Add(m_zast_typ, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Kód podepisující osoby")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_zast_kod = new wxChoice(this, wxID_ANY);
    m_zast_kod->Enable(false);
    gridSizer->Add(m_zast_kod, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Jméno")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_zast_jmeno = new wxTextCtrl(this, wxID_ANY);
    m_zast_jmeno->SetMaxLength(20);
    m_zast_jmeno->Enable(false);
    gridSizer->Add(m_zast_jmeno, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Přijmení")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_zast_prijmeni = new wxTextCtrl(this, wxID_ANY);
    m_zast_prijmeni->SetMaxLength(36);
    m_zast_prijmeni->Enable(false);
    gridSizer->Add(m_zast_prijmeni, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Datum narození")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_zast_dat_nar = new wxDatePickerCtrl(this, wxID_ANY, wxDateTime(1, wxDateTime::Jan, 1968), wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    m_zast_dat_nar->Enable(false);
    gridSizer->Add(m_zast_dat_nar, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Evidenční číslo osvědčení\ndaňového poradce")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_zast_ev_cislo = new wxTextCtrl(this, wxID_ANY);
    m_zast_ev_cislo->SetMaxLength(36);
    m_zast_ev_cislo->Enable(false);
    gridSizer->Add(m_zast_ev_cislo, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Název právnické osoby")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_zast_nazev = new wxTextCtrl(this, wxID_ANY);
    m_zast_nazev->SetMaxLength(255);
    m_zast_nazev->Enable(false);
    gridSizer->Add(m_zast_nazev, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("IČ")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_zast_icText = new wxTextCtrl(this, wxID_ANY, "", wxDefaultPosition, wxDefaultSize, 0, wxMakeIntegerValidator(&m_zast_ic));
    m_zast_icText->SetMaxLength(10);
    m_zast_icText->Enable(false);
    gridSizer->Add(m_zast_icText, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Oprávněná osoba\nJméno")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_opr_jmeno = new wxTextCtrl(this, wxID_ANY);
    m_opr_jmeno->SetMaxLength(20);
    m_opr_jmeno->Enable(false);
    gridSizer->Add(m_opr_jmeno, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Oprávněná osoba\nPříjmení")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_opr_prijmeni = new wxTextCtrl(this, wxID_ANY);
    m_opr_prijmeni->SetMaxLength(36);
    m_opr_prijmeni->Enable(false);
    gridSizer->Add(m_opr_prijmeni, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Oprávněná osoba\nVztah k právnické osobě")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_opr_postaveni = new wxTextCtrl(this, wxID_ANY);
    m_opr_postaveni->SetMaxLength(40);
    m_opr_postaveni->Enable(false);
    gridSizer->Add(m_opr_postaveni, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);

    return gridSizer;
}

void PriznaniDialog::fillControls()
{
    Priznani pr;
    if(m_edit)
    {
        pr = idbmodel->priznani();
    }
    else
    {
        Firma firma = idbmodel->firma();
        wxString nazev = firma.nazev();
        switch(dxutils::pocetZnaku(nazev, ' ')) {
        case 0: pr.setPrijmeni(nazev); break;
        case 1: pr.setJmeno(nazev.BeforeFirst(' ')); pr.setPrijmeni(nazev.AfterFirst(' ')); break;
        case 2: pr.setTitul(nazev.BeforeFirst(' ')); pr.setJmeno(nazev.AfterFirst(' ').BeforeLast(' ')); pr.setPrijmeni(nazev.AfterLast(' ')); break;
        }
        Adresa adresa = idbmodel->getAdresa(firma.adresaid());
        pr.setUlice(adresa.isValid()?adresa.ulice():"");
        pr.setC_pop(adresa.isValid()?wxAtoi(adresa.cp()):0);
        int psc = (adresa.isValid()?wxAtoi(adresa.psc()):18000);
        if(dxutils::pocetCislic(psc) != 5) psc = 18000;
        pr.setPsc(wxString::Format("%d", psc));
        int pufo = 0, pracufo = 0, ufo = 0;
        for(size_t i=0; i<m_obceArray.GetCount(); i++)
        {
            if(psc == m_obceArray[i]->psc())
            {
                pufo = m_obceArray[i]->fu();
                break;
            }
        }
        if(!pufo)
        {
            psc = 18000;
            pr.setPsc(wxString::Format("%d", psc));
            for(size_t i=0; i<m_obceArray.GetCount(); i++)
            {
                if(psc == m_obceArray[i]->psc())
                {
                    pufo = m_obceArray[i]->fu();
                    break;
                }
            }
        }
        for(size_t i=0; i<m_pracufoArray.GetCount(); i++)
        {
            if(pufo == m_pracufoArray[i]->pufo())
            {
                ufo = m_pracufoArray[i]->ufo();
                pracufo = m_pracufoArray[i]->pracufo();
                break;
            }
        }
        pr.setC_pracufo(pracufo);
        pr.setC_ufo(ufo);
        if(adresa.isValid() && !adresa.misto().IsEmpty()) pr.setNaz_obce(adresa.misto());
        else pr.setNaz_obce("PRAHA 8");
        if(firma.dic().IsEmpty()) pr.setDic("0");
        else
        {
            if(firma.dic().StartsWith("CZ")) pr.setDic(firma.dic().After('Z'));
            else pr.setDic("0");
        }
        pr.setEmail(firma.email());
        pr.setC_telef(firma.telefon());
        pr.setC_okec(0);
        pr.setZast_ic("0");
    }
    m_jmeno->SetValue(pr.jmeno());
    m_prijmeni->SetValue(pr.prijmeni());
    m_titul->SetValue(pr.titul());
    m_ulice->SetValue(pr.ulice());
    m_cpop = pr.c_pop();
    m_corient->SetValue(pr.c_orient());
    m_psc = wxAtoi(pr.psc());
    m_nazobce->Clear();
    for(size_t i=0; i<m_obceArray.GetCount(); i++)
    {
        if(m_psc == m_obceArray[i]->psc())
        {
            m_nazobce->Append(m_obceArray[i]->nazev(), m_obceArray[i]);
        }
    }
    m_nazobce->SetValue(pr.naz_obce());
    for(size_t i=0; i<m_ufoArray.GetCount(); i++)
    {
        if(pr.c_ufo() == m_ufoArray[i]->ufo())
        {
            m_cufo->SetSelection(m_cufo->FindString(m_ufoArray[i]->nazev()));
            break;
        }
    }
    int cpracufoSel = 0;
    m_cpracufo->Clear();
    for(size_t i=0, j=0; i<m_pracufoArray.GetCount(); i++)
    {
        if(pr.c_ufo() == m_pracufoArray[i]->ufo())
        {
            m_cpracufo->Append(m_pracufoArray[i]->nazev(), m_pracufoArray[i]);
            if(pr.c_pracufo() == m_pracufoArray[i]->pracufo()) cpracufoSel = static_cast<int>(j);
            j++;
        }
    }
    m_cpracufo->SetSelection(cpracufoSel);
    m_dic = pr.dic();
    m_email->SetValue(pr.email());
    m_iddats->SetValue(pr.id_dats());
    m_ctelef->SetValue(pr.c_telef());
    m_cokec = pr.c_okec();
    if(m_cokec)
    {
        wxString cokec = wxString::Format("%d", m_cokec);
        wxXmlDocument okec;
        okec.Load(wxStandardPaths::Get().GetUserDataDir()+"/okec.xml");
        if(okec.IsOk() && okec.GetRoot()->GetName() == "Ciselnik")
        {
            wxXmlNode *child = okec.GetRoot()->GetChildren();
            while(child)
            {
                if(cokec == child->GetAttribute("c_nace"))
                {
                    m_cokecText->SetValue(child->GetAttribute("naz_nace"));
                    m_cokecText->SetToolTip(child->GetAttribute("naz_nace"));
                    break;
                }
                child = child->GetNext();
            }
        }
    }
    wxString zast_typ = pr.zast_typ();
    if(zast_typ.IsEmpty())
    {
        m_zast_ic = 0;
        m_zast_typ->SetSelection(0);
    }
    if(zast_typ == "F")
    {
        m_zast_typ->SetSelection(1);
        m_zast_kod->Enable(true);
        m_zast_kod->Clear();
        m_zast_kod->Append(wxString::FromUTF8("1 - zákonný zástupce nebo opatrovník"));
        m_zast_kod->Append(wxString::FromUTF8("2 - ustanovený zástupce"));
        m_zast_kod->Append(wxString::FromUTF8("3 - společný zástupce, společný zmocněnec"));
        m_zast_kod->Append(wxString::FromUTF8("4a - obecný zmocněnec"));
        m_zast_kod->Append(wxString::FromUTF8("4b - fyzická osoba daňový poradce nebo advokát"));
        m_zast_kod->Append(wxString::FromUTF8("5a - osoba spravující pozůstalost"));
        m_zast_kod->Append(wxString::FromUTF8("5b - zástupce osoby spravující pozůstalost"));
        m_zast_kod->Append(wxString::FromUTF8("6a - dědic po skončení řízení o pozůstalosti"));
        m_zast_kod->Append(wxString::FromUTF8("6b - zástupce dědice po skončení řízení o pozůstalosti"));
        m_zast_kod->Append(wxString::FromUTF8("7a - právní nástupce právnické osoby"));
        m_zast_kod->Append(wxString::FromUTF8("7b - zástupce právního nástupce právnické osoby"));
        if(pr.zast_kod() == "1") m_zast_kod->SetSelection(0);
        if(pr.zast_kod() == "2") m_zast_kod->SetSelection(1);
        if(pr.zast_kod() == "3") m_zast_kod->SetSelection(2);
        if(pr.zast_kod() == "4a") m_zast_kod->SetSelection(3);
        if(pr.zast_kod() == "4b") m_zast_kod->SetSelection(4);
        if(pr.zast_kod() == "5a") m_zast_kod->SetSelection(5);
        if(pr.zast_kod() == "5b") m_zast_kod->SetSelection(6);
        if(pr.zast_kod() == "6a") m_zast_kod->SetSelection(7);
        if(pr.zast_kod() == "6b") m_zast_kod->SetSelection(8);
        if(pr.zast_kod() == "7a") m_zast_kod->SetSelection(9);
        if(pr.zast_kod() == "7b") m_zast_kod->SetSelection(10);
        m_zast_jmeno->Enable(true);
        m_zast_jmeno->SetValue(pr.zast_jmeno());
        m_zast_prijmeni->Enable(true);
        m_zast_prijmeni->SetValue(pr.zast_prijmeni());
        m_zast_dat_nar->Enable(true);
        m_zast_dat_nar->SetValue(pr.zast_dat_nar());
        m_zast_ev_cislo->Enable(true);
        m_zast_ev_cislo->SetValue(pr.zast_ev_cislo());
        m_zast_ic = 0;
    }
    if(zast_typ == "P")
    {
        m_zast_typ->SetSelection(2);
        m_zast_kod->Enable(true);
        m_zast_kod->Clear();
        m_zast_kod->Append(wxString::FromUTF8("2 - ustanovený zástupce"));
        m_zast_kod->Append(wxString::FromUTF8("3 - společný zástupce, společný zmocněnec"));
        m_zast_kod->Append(wxString::FromUTF8("4a - obecný zmocněnec"));
        m_zast_kod->Append(wxString::FromUTF8("4c -právnická osoba vykonávající daňové poradenství"));
        m_zast_kod->Append(wxString::FromUTF8("5a - osoba spravující pozůstalost"));
        m_zast_kod->Append(wxString::FromUTF8("5b - zástupce osoby spravující pozůstalost"));
        m_zast_kod->Append(wxString::FromUTF8("6a - dědic po skončení řízení o pozůstalosti"));
        m_zast_kod->Append(wxString::FromUTF8("6b - zástupce dědice po skončení řízení o pozůstalosti"));
        m_zast_kod->Append(wxString::FromUTF8("7a - právní nástupce právnické osoby"));
        m_zast_kod->Append(wxString::FromUTF8("7b - zástupce právního nástupce právnické osoby"));
        if(pr.zast_kod() == "2") m_zast_kod->SetSelection(0);
        if(pr.zast_kod() == "3") m_zast_kod->SetSelection(1);
        if(pr.zast_kod() == "4a") m_zast_kod->SetSelection(2);
        if(pr.zast_kod() == "4c") m_zast_kod->SetSelection(3);
        if(pr.zast_kod() == "5a") m_zast_kod->SetSelection(4);
        if(pr.zast_kod() == "5b") m_zast_kod->SetSelection(5);
        if(pr.zast_kod() == "6a") m_zast_kod->SetSelection(6);
        if(pr.zast_kod() == "6b") m_zast_kod->SetSelection(7);
        if(pr.zast_kod() == "7a") m_zast_kod->SetSelection(8);
        if(pr.zast_kod() == "7b") m_zast_kod->SetSelection(9);
        m_zast_nazev->Enable(true);
        m_zast_nazev->SetValue(pr.zast_nazev());
        m_zast_ic = wxAtoi(pr.zast_ic());
        m_zast_icText->Enable(true);
        m_opr_jmeno->Enable(true);
        m_opr_jmeno->SetValue(pr.opr_jmeno());
        m_opr_prijmeni->Enable(true);
        m_opr_prijmeni->SetValue(pr.opr_prijmeni());
        m_opr_postaveni->Enable(true);
        m_opr_postaveni->SetValue(pr.opr_postaveni());
    }
}

void PriznaniDialog::OnOk(wxCommandEvent& /*event*/)
{
    m_cpopText->GetValidator()->TransferFromWindow();
    m_pscText->GetValidator()->TransferFromWindow();
    m_dicText->GetValidator()->TransferFromWindow();
    m_zast_icText->GetValidator()->TransferFromWindow();
    if(m_jmeno->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Jméno nemůže být prázdné"), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        m_jmeno->SetFocus();
        return;
    }
    if(m_prijmeni->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Příjmení nemůže být prázdné"), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        m_prijmeni->SetFocus();
        return;
    }
    if(m_ulice->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Ulice nemůže být prázdná"), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        m_ulice->SetFocus();
        return;
    }
    if(m_nazobce->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Obec nemůže být prázdná"), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        m_nazobce->SetFocus();
        return;
    }
    if(m_email->GetValue().IsEmpty() && m_iddats->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Vyplňte buď email\nnebo identifikátor datové schránky"), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        m_email->SetFocus();
        return;
    }
    if(m_cokecText->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Hlavní ekonomická činnost nemůže být prázdná"), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        wxCommandEvent dummy;
        OnCokecSelect(dummy);
        return;
    }
    wxString zast_typ = "";
    wxString zast_kod = "";
    if(m_zast_typ->GetSelection() == 1) //Fyzicka
    {
        if(m_zast_jmeno->GetValue().IsEmpty())
        {
            wxMessageDialog dialog(this, wxString::FromUTF8("Jméno podepisující osoby nemůže být prázdné"), "Informace", wxOK|wxICON_WARNING);
            dialog.ShowModal();
            m_zast_jmeno->SetFocus();
            return;
        }
        if(m_zast_prijmeni->GetValue().IsEmpty())
        {
            wxMessageDialog dialog(this, wxString::FromUTF8("Příjmení podepisující osoby nemůže být prázdné"), "Informace", wxOK|wxICON_WARNING);
            dialog.ShowModal();
            m_zast_prijmeni->SetFocus();
            return;
        }
        zast_typ = "F";
        zast_kod = m_zast_kod->GetStringSelection().BeforeFirst(' ');
    }
    if(m_zast_typ->GetSelection() == 2) //Pravnicka
    {
        if(m_zast_nazev->GetValue().IsEmpty())
        {
            wxMessageDialog dialog(this, wxString::FromUTF8("Název právnické osoby nemůže být prázdný"), "Informace", wxOK|wxICON_WARNING);
            dialog.ShowModal();
            m_zast_nazev->SetFocus();
            return;
        }
        if(m_opr_jmeno->GetValue().IsEmpty())
        {
            wxMessageDialog dialog(this, wxString::FromUTF8("Jméno oprávněné osoby nemůže být prázdné"), "Informace", wxOK|wxICON_WARNING);
            dialog.ShowModal();
            m_opr_jmeno->SetFocus();
            return;
        }
        if(m_opr_prijmeni->GetValue().IsEmpty())
        {
            wxMessageDialog dialog(this, wxString::FromUTF8("Příjmení oprávněné osoby nemůže být prázdné"), "Informace", wxOK|wxICON_WARNING);
            dialog.ShowModal();
            m_opr_prijmeni->SetFocus();
            return;
        }
        if(m_opr_postaveni->GetValue().IsEmpty())
        {
            wxMessageDialog dialog(this, wxString::FromUTF8("Vztah oprávněné osoby k právnické osobě nemůže být prázdný"), "Informace", wxOK|wxICON_WARNING);
            dialog.ShowModal();
            m_opr_postaveni->SetFocus();
            return;
        }
        zast_typ = "P";
        zast_kod = m_zast_kod->GetStringSelection().BeforeFirst(' ');
    }
    idbmodel->setPriznani(Priznani(m_jmeno->GetValue(), m_prijmeni->GetValue(), m_titul->GetValue(), m_ulice->GetValue(), m_cpop, m_corient->GetValue(),
                                   m_nazobce->GetValue(), m_pscText->GetValue(), wxString::FromUTF8("ČESKÁ REPUBLIKA" ), m_dicText->GetValue(), m_email->GetValue(), m_iddats->GetValue(),
                                   m_ctelef->GetValue(), static_cast<Ufo*>(m_cufo->GetClientData(static_cast<unsigned int>(m_cufo->GetSelection())))->ufo(),
                                   m_cpracufo->GetCount()?static_cast<Pracufo*>(m_cpracufo->GetClientData(static_cast<unsigned int>(m_cpracufo->GetSelection())))->pracufo():0,
                                   m_cokec, m_opr_jmeno->GetValue(), m_opr_prijmeni->GetValue(),
                                   m_opr_postaveni->GetValue(), m_zast_dat_nar->GetValue(), m_zast_ev_cislo->GetValue(), m_zast_icText->GetValue(), m_zast_jmeno->GetValue(),
                                   zast_kod, m_zast_nazev->GetValue(), m_zast_prijmeni->GetValue(), zast_typ));
    EndModal(wxID_OK);
}

void PriznaniDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

void PriznaniDialog::OnPsc(wxCommandEvent &/*event*/)
{
    if(m_pscText->GetLineLength(0) < 5) return;
    int psc = m_psc;
    m_pscText->GetValidator()->TransferFromWindow();
    if(psc == m_psc) return;
    pscChanged();
}

void PriznaniDialog::OnCufo(wxCommandEvent &/*event*/)
{
    int ufo = static_cast<Ufo*>(m_cufo->GetClientData(static_cast<unsigned int>(m_cufo->GetSelection())))->ufo();
    m_cpracufo->Clear();
    for(size_t i=0; i<m_pracufoArray.GetCount(); i++)
    {
        if(ufo == m_pracufoArray[i]->ufo())
        {
            m_cpracufo->Append(m_pracufoArray[i]->nazev(), m_pracufoArray[i]);
        }
    }
    m_cpracufo->SetSelection(0);
}

void PriznaniDialog::pscChanged()
{
    m_nazobce->Clear();
    int pufo = 0;
    for(size_t i=0; i<m_obceArray.GetCount(); i++)
    {
        if(m_psc == m_obceArray[i]->psc())
        {
            pufo = m_obceArray[i]->fu();
            m_nazobce->Append(m_obceArray[i]->nazev(), m_obceArray[i]);
        }
    }
    if(m_nazobce->GetCount()) m_nazobce->SetSelection(0);
    int ufo = 0;
    wxString pracufo = wxEmptyString;
    for(size_t i=0; i<m_pracufoArray.GetCount(); i++)
    {
        if(pufo == m_pracufoArray[i]->pufo())
        {
            ufo = m_pracufoArray[i]->ufo();
            pracufo = m_pracufoArray[i]->nazev();
            break;
        }
    }
    for(size_t i=0; i<m_ufoArray.GetCount(); i++)
    {
        if(ufo == m_ufoArray[i]->ufo())
        {
            m_cufo->SetSelection(m_cufo->FindString(m_ufoArray[i]->nazev()));
            break;
        }
    }
    m_cpracufo->Clear();
    for(size_t i=0; i<m_pracufoArray.GetCount(); i++)
    {
        if(ufo == m_pracufoArray[i]->ufo())
        {
            m_cpracufo->Append(m_pracufoArray[i]->nazev(), m_pracufoArray[i]);
        }
    }
    m_cpracufo->SetSelection(m_cpracufo->FindString(pracufo));
}

void PriznaniDialog::OnCokecSelect(wxCommandEvent &/*event*/)
{
    OkecDialog dialog(this, m_cokec);
    if(dialog.ShowModal() == wxID_OK)
    {
        Okec ok = dialog.selectedOkec();
        m_cokec = ok.nace();
        m_cokecText->SetValue(ok.nazev());
        m_cokecText->SetToolTip(ok.nazev());
    }
}

void PriznaniDialog::OnZastTyp(wxCommandEvent &/*event*/)
{
    if(m_zast_typ->GetSelection() == 0) //Zadna
    {
        m_zast_kod->Enable(false);
        m_zast_kod->Clear();
        m_zast_jmeno->Enable(false);
        m_zast_prijmeni->Enable(false);
        m_zast_dat_nar->Enable(false);
        m_zast_ev_cislo->Enable(false);
        m_zast_nazev->Enable(false);
        m_zast_icText->Enable(false);
        m_opr_jmeno->Enable(false);
        m_opr_prijmeni->Enable(false);
        m_opr_postaveni->Enable(false);
    }
    if(m_zast_typ->GetSelection() == 1) //Fyzicka
    {
        m_zast_kod->Enable(true);
        m_zast_kod->Clear();
        m_zast_kod->Append(wxString::FromUTF8("1 - zákonný zástupce nebo opatrovník"));
        m_zast_kod->Append(wxString::FromUTF8("2 - ustanovený zástupce"));
        m_zast_kod->Append(wxString::FromUTF8("3 - společný zástupce, společný zmocněnec"));
        m_zast_kod->Append(wxString::FromUTF8("4a - obecný zmocněnec"));
        m_zast_kod->Append(wxString::FromUTF8("4b - fyzická osoba daňový poradce nebo advokát"));
        m_zast_kod->Append(wxString::FromUTF8("5a - osoba spravující pozůstalost"));
        m_zast_kod->Append(wxString::FromUTF8("5b - zástupce osoby spravující pozůstalost"));
        m_zast_kod->Append(wxString::FromUTF8("6a - dědic po skončení řízení o pozůstalosti"));
        m_zast_kod->Append(wxString::FromUTF8("6b - zástupce dědice po skončení řízení o pozůstalosti"));
        m_zast_kod->Append(wxString::FromUTF8("7a - právní nástupce právnické osoby"));
        m_zast_kod->Append(wxString::FromUTF8("7b - zástupce právního nástupce právnické osoby"));
        m_zast_kod->SetSelection(0);
        m_zast_jmeno->Enable(true);
        m_zast_prijmeni->Enable(true);
        m_zast_dat_nar->Enable(true);
        m_zast_ev_cislo->Enable(true);
        m_zast_nazev->Enable(false);
        m_zast_icText->Enable(false);
        m_opr_jmeno->Enable(false);
        m_opr_prijmeni->Enable(false);
        m_opr_postaveni->Enable(false);
    }
    if(m_zast_typ->GetSelection() == 2) //Pravnicka
    {
        m_zast_kod->Enable(true);
        m_zast_kod->Clear();
        m_zast_kod->Append(wxString::FromUTF8("2 - ustanovený zástupce"));
        m_zast_kod->Append(wxString::FromUTF8("3 - společný zástupce, společný zmocněnec"));
        m_zast_kod->Append(wxString::FromUTF8("4a - obecný zmocněnec"));
        m_zast_kod->Append(wxString::FromUTF8("4c -právnická osoba vykonávající daňové poradenství"));
        m_zast_kod->Append(wxString::FromUTF8("5a - osoba spravující pozůstalost"));
        m_zast_kod->Append(wxString::FromUTF8("5b - zástupce osoby spravující pozůstalost"));
        m_zast_kod->Append(wxString::FromUTF8("6a - dědic po skončení řízení o pozůstalosti"));
        m_zast_kod->Append(wxString::FromUTF8("6b - zástupce dědice po skončení řízení o pozůstalosti"));
        m_zast_kod->Append(wxString::FromUTF8("7a - právní nástupce právnické osoby"));
        m_zast_kod->Append(wxString::FromUTF8("7b - zástupce právního nástupce právnické osoby"));
        m_zast_kod->SetSelection(0);
        m_zast_jmeno->Enable(false);
        m_zast_prijmeni->Enable(false);
        m_zast_dat_nar->Enable(false);
        m_zast_ev_cislo->Enable(false);
        m_zast_nazev->Enable(true);
        m_zast_icText->Enable(true);
        m_opr_jmeno->Enable(true);
        m_opr_prijmeni->Enable(true);
        m_opr_postaveni->Enable(true);
    }
}

wxString Obec::nazev() const
{
    return m_nazev;
}

void Obec::setNazev(const wxString &nazev)
{
    m_nazev = nazev;
}

int Obec::psc() const
{
    return m_psc;
}

void Obec::setPsc(int psc)
{
    m_psc = psc;
}

int Obec::fu() const
{
    return m_fu;
}

void Obec::setFu(int fu)
{
    m_fu = fu;
}

wxString Ufo::nazev() const
{
    return m_nazev;
}

void Ufo::setNazev(const wxString &nazev)
{
    m_nazev = nazev;
}

int Ufo::ufo() const
{
    return m_ufo;
}

void Ufo::setUfo(int ufo)
{
    m_ufo = ufo;
}

wxString Pracufo::nazev() const
{
    return m_nazev;
}

void Pracufo::setNazev(const wxString &nazev)
{
    m_nazev = nazev;
}

int Pracufo::pracufo() const
{
    return m_pracufo;
}

void Pracufo::setPracufo(int pracufo)
{
    m_pracufo = pracufo;
}

int Pracufo::pufo() const
{
    return m_pufo;
}

void Pracufo::setPufo(int pufo)
{
    m_pufo = pufo;
}

int Pracufo::ufo() const
{
    return m_ufo;
}

void Pracufo::setUfo(int ufo)
{
    m_ufo = ufo;
}
