/***************************************************************
 * Name:      ulozenedialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#include "ulozenedialog.h"
#include "dxdefs.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "resources/images/saveas.xpm"
#include "resources/images/preview.xpm"
#include "resources/images/delete.xpm"
#include "dphdp3print.h"
#include "dphkh1print.h"
#include "dphshprint.h"
#include "dphdp3.h"
#include "dphkh1.h"
#include "dphshv.h"

IMPLEMENT_DYNAMIC_CLASS(UlozeneDialog, wxDialog)

BEGIN_EVENT_TABLE(UlozeneDialog, wxDialog)
    EVT_BUTTON(wxID_OK, UlozeneDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, UlozeneDialog::OnCancel)
    EVT_LIST_ITEM_RIGHT_CLICK(ID_ULOZENE_LIST, UlozeneDialog::OnItemRightClick)
    EVT_LIST_ITEM_ACTIVATED(ID_ULOZENE_LIST, UlozeneDialog::OnItemActivated)
    EVT_LIST_ITEM_SELECTED(ID_ULOZENE_LIST, UlozeneDialog::OnItemSelected)
    EVT_LIST_ITEM_DESELECTED(ID_ULOZENE_LIST, UlozeneDialog::OnItemDeselected)
    EVT_MENU(ID_ULOZENE_PREVIEW, UlozeneDialog::OnPreviewUlozene)
    EVT_MENU(ID_ULOZENE_SAVEAS, UlozeneDialog::OnSaveasUlozene)
    EVT_MENU(ID_ULOZENE_DELETE, UlozeneDialog::OnDeleteUlozene)
END_EVENT_TABLE()

UlozeneDialog::UlozeneDialog(wxWindow *parent)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Uložená přiznání")),
      m_selectedIndex(-1)
{
    m_ulozene = idbmodel->ulozene();
    idbmodel->sortUlozene(m_ulozene);

    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    m_bar = new dxToolBar(this);
    m_bar->AddTool(ID_ULOZENE_PREVIEW, wxEmptyString, wxBitmap(preview_xpm), dxutils::GreyScaledIcon(preview_xpm), wxITEM_NORMAL, wxString::FromUTF8("Náhled přiznání"));
    m_bar->AddTool(ID_ULOZENE_SAVEAS, wxEmptyString, wxBitmap(saveas_xpm), dxutils::GreyScaledIcon(saveas_xpm), wxITEM_NORMAL, wxString::FromUTF8("Uložit přiznání jako"));
    m_bar->AddTool(ID_ULOZENE_DELETE, wxEmptyString, wxBitmap(delete_xpm), dxutils::GreyScaledIcon(delete_xpm), wxITEM_NORMAL, wxString::FromUTF8("Smazat přiznání"));
    m_bar->Realize();
    m_mainSizer->Add(m_bar, 0, wxALL|wxEXPAND, 5);
    m_list = new wxListCtrl(this, ID_ULOZENE_LIST, wxDefaultPosition, wxDefaultSize,
                            wxLC_REPORT|wxLC_HRULES|wxLC_VRULES|wxLC_SINGLE_SEL);
    wxListItem col0;
    col0.SetId(0);
    col0.SetText(wxString::FromUTF8("Přiznání"));
    col0.SetWidth(150);
    m_list->InsertColumn(0, col0);
    wxListItem col1;
    col1.SetId(1);
    col1.SetText("Popis");
    col1.SetWidth(300);
    m_list->InsertColumn(1, col1);
    fillList();
    m_mainSizer->Add(m_list, 1, wxALL|wxEXPAND, 5);
    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);
    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void UlozeneDialog::OnOk(wxCommandEvent& /*event*/)
{
    EndModal(wxID_OK);
}

void UlozeneDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

void UlozeneDialog::OnItemRightClick(wxListEvent &event)
{
    setSelectedIndex(event.GetIndex());
    if(m_selectedIndex != -1)
    {
        wxMenu popup;
        popup.Append(ID_ULOZENE_PREVIEW, wxString::FromUTF8("Náhled přiznání"));
        popup.Append(ID_ULOZENE_SAVEAS, wxString::FromUTF8("Uložit přiznání jako"));
        popup.Append(ID_ULOZENE_DELETE, wxString::FromUTF8("Smazat přiznání"));
        PopupMenu(&popup, event.GetPoint());
    }
}

void UlozeneDialog::OnItemActivated(wxListEvent &event)
{
    setSelectedIndex(event.GetIndex());
    switch (m_ulozene[static_cast<size_t>(m_selectedIndex)].typ())
    {
        case 0: {
            Dphdp3 dp; dp.readFromXml(m_ulozene[static_cast<size_t>(m_selectedIndex)].xml());
            Dphdp3Print dialog(this, dp.atributy());
            //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
            dialog.Print();
#else
            dialog.Preview();
#endif
            break;
        }
        case 1: {
            Dphshv dp; dp.readFromXml(m_ulozene[static_cast<size_t>(m_selectedIndex)].xml());
            DphshPrint dialog(this, dp.vety());
            //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
            dialog.Print();
#else
            dialog.Preview();
#endif
            break;
        }
        case 2: {
            Dphkh1 dp; dp.readFromXml(m_ulozene[static_cast<size_t>(m_selectedIndex)].xml());
            Dphkh1Print dialog(this, dp.vety());
            //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
            dialog.Print();
#else
            dialog.Preview();
#endif
            break;
        }
        default: ;
    }
}

void UlozeneDialog::OnItemSelected(wxListEvent &event)
{
    setSelectedIndex(event.GetIndex());
}

void UlozeneDialog::OnItemDeselected(wxListEvent& /*event*/)
{
    setSelectedIndex(-1);
}

void UlozeneDialog::OnSaveasUlozene(wxCommandEvent& /*event*/)
{
    if(m_selectedIndex == -1) return;
    wxFileDialog dlg(this, wxString::FromUTF8("Uložit přiznání/hlášení jako"), wxEmptyString, dxutils::normalizeFilename(m_ulozene[static_cast<size_t>(m_selectedIndex)].popis()), "XML soubory (*.xml)|*.xml",
                     wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
    if(dlg.ShowModal() != wxID_OK) return;
    wxString path = dlg.GetPath();
    dxutils::correctEmptyFileExt("xml", path);
    wxCopyFile(m_ulozene[static_cast<size_t>(m_selectedIndex)].xml(), path);
}

void UlozeneDialog::OnPreviewUlozene(wxCommandEvent& /*event*/)
{
    if(m_selectedIndex == -1) return;
    switch (m_ulozene[static_cast<size_t>(m_selectedIndex)].typ())
    {
        case 0: {
            Dphdp3 dp; dp.readFromXml(m_ulozene[static_cast<size_t>(m_selectedIndex)].xml());
            Dphdp3Print dialog(this, dp.atributy());
            //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
            dialog.Print();
#else
            dialog.Preview();
#endif
            break;
        }
        case 1: {
            Dphshv dp; dp.readFromXml(m_ulozene[static_cast<size_t>(m_selectedIndex)].xml());
            DphshPrint dialog(this, dp.vety());
            //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
            dialog.Print();
#else
            dialog.Preview();
#endif
            break;
        }
        case 2: {
            Dphkh1 dp; dp.readFromXml(m_ulozene[static_cast<size_t>(m_selectedIndex)].xml());
            Dphkh1Print dialog(this, dp.vety());
            //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
            dialog.Print();
#else
            dialog.Preview();
#endif
            break;
        }
        default: ;
    }
}

void UlozeneDialog::OnDeleteUlozene(wxCommandEvent& /*event*/)
{
    if(m_selectedIndex == -1) return;
    wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Opravdu smazat přiznání:\n%s\n%s?"), m_list->GetItemText(m_selectedIndex),
                                                  m_ulozene[static_cast<size_t>(m_selectedIndex)].popis()), "Dotaz", wxOK|wxCANCEL|wxICON_QUESTION);
    if(dialog.ShowModal() == wxID_OK)
    {
        idbmodel->deleteUlozene(m_ulozene[static_cast<size_t>(m_selectedIndex)].id());
        m_ulozene.RemoveAt(static_cast<size_t>(m_selectedIndex));
        fillList();
    }
}

void UlozeneDialog::setSelectedIndex(long selectedIndex)
{
    if(selectedIndex < 0)
        m_selectedIndex = -1;
    else
        m_selectedIndex = selectedIndex;
    enableTools();
}

void UlozeneDialog::fillList()
{
    m_list->DeleteAllItems();
    m_list->Hide();
    for(size_t i=0; i < m_ulozene.GetCount(); i++)
    {
        wxListItem item;
        item.SetId(static_cast<long>(i));
        switch (m_ulozene[i].typ())
        {
            case 0: item.SetText(wxString::FromUTF8("Přiznání k DPH")); break;
            case 1: item.SetText(wxString::FromUTF8("Souhrnné hlášení DPH")); break;
            case 2: item.SetText(wxString::FromUTF8("Kontrolní hlášení DPH")); break;
            default: item.SetText("");
        }
        m_list->InsertItem(item);
        m_list->SetItem(static_cast<long>(i), 1, m_ulozene[i].popis());
    }
    m_list->Show();
    if(m_ulozene.GetCount())
        m_list->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
    else
        setSelectedIndex(-1);
}

void UlozeneDialog::enableTools()
{
    if(m_selectedIndex == -1)
    {
        m_bar->EnableTool(ID_ULOZENE_DELETE, false);
        m_bar->EnableTool(ID_ULOZENE_PREVIEW, false);
        m_bar->EnableTool(ID_ULOZENE_SAVEAS, false);
    }
    else
    {
        m_bar->EnableTool(ID_ULOZENE_DELETE, true);
        m_bar->EnableTool(ID_ULOZENE_PREVIEW, true);
        m_bar->EnableTool(ID_ULOZENE_SAVEAS, true);
    }
}
