/***************************************************************
 * Name:      dokumentdialog.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2015
 * License:   GPL3
 **************************************************************/

#ifndef DOKUMENTDIALOG_H
#define DOKUMENTDIALOG_H

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif
#include <wx/spinctrl.h>
#include "canvases.h"
#include "print/document.h"

class DokumentDialog: public wxDialog
{
    DECLARE_DYNAMIC_CLASS(DokumentDialog)
    DECLARE_EVENT_TABLE()
public:
    DokumentDialog() : m_mainSizer(nullptr), m_marginLeft(nullptr), m_marginTop(nullptr), m_marginRight(nullptr), m_marginBottom(nullptr),
    m_headerLeft(nullptr), m_headerCenter(nullptr), m_headerRight(nullptr), m_headerLine(nullptr), m_footer(nullptr), m_footerText(nullptr),
    m_preview(nullptr), m_fontSelect(nullptr), m_fontColorSelect(nullptr), m_backColorSelect(nullptr) {}
    DokumentDialog(wxWindow *parent, const Document &doc);

    Document document() const;
    void setDocument(const Document &document);
private:
    Document m_document;
    TextElement m_element;
    wxBoxSizer *m_mainSizer;
    wxSpinCtrl *m_marginLeft;
    wxSpinCtrl *m_marginTop;
    wxSpinCtrl *m_marginRight;
    wxSpinCtrl *m_marginBottom;
    wxTextCtrl *m_headerLeft;
    wxTextCtrl *m_headerCenter;
    wxTextCtrl *m_headerRight;
    wxCheckBox *m_headerLine;
    wxChoice *m_footer;
    wxTextCtrl *m_footerText;
    TextCanvas *m_preview;
    wxButton *m_fontSelect;
    wxButton *m_fontColorSelect;
    wxButton *m_backColorSelect;

    void createControls();
    void fillControls();
    void setFontText(const wxFont& font);
    void OnOk(wxCommandEvent& event);
    void OnCancel(wxCommandEvent& event);
    void OnFooter(wxCommandEvent& event);
    void OnFont(wxCommandEvent& event);
    void OnFontColor(wxCommandEvent& event);
    void OnBackColor(wxCommandEvent& event);
};

#endif  /* DOKUMNETDIALOG_H */

