/***************************************************************
 * Name:      imagedialog.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2015
 * License:   GPL3
 **************************************************************/

#ifndef IMAGEDIALOG_H
#define IMAGEDIALOG_H

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif
#include <wx/spinctrl.h>
#include "canvases.h"
#include "print/document.h"

class ImageDialog: public wxDialog
{
    DECLARE_DYNAMIC_CLASS(ImageDialog)
    DECLARE_EVENT_TABLE()
public:
    ImageDialog() : m_mainSizer(nullptr), m_posX(nullptr), m_posY(nullptr), m_imageWidth(nullptr), m_path(nullptr), m_alignment(nullptr),
    m_preview(nullptr), m_pathSelect(nullptr), m_section(nullptr) {}
    explicit ImageDialog(wxWindow *parent);
    ImageDialog(wxWindow *parent, const ImageElement &element);

    ImageElement element() const;
    void setElement(const ImageElement &element);
private:
    ImageElement m_element;
    wxBoxSizer *m_mainSizer;
    wxSpinCtrl *m_posX;
    wxSpinCtrl *m_posY;
    wxStaticText *m_imageWidthTxt;
    wxSpinCtrl *m_imageWidth;
    wxTextCtrl *m_path;
    wxStaticText *m_alignmentTxt;
    wxChoice *m_alignment;
    ImageCanvas *m_preview;
    wxButton *m_pathSelect;
    wxChoice *m_section;

    void createControls();
    void fillControls();
    void OnOk(wxCommandEvent& event);
    void OnCancel(wxCommandEvent& event);
    void OnAlignment(wxCommandEvent& event);
    void OnPath(wxCommandEvent& event);
    void setPath(const wxString &path);
};

#endif  /* IMAGEDIALOG_H */

