/***************************************************************
 * Name:      linedialog.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2015
 * License:   GPL3
 **************************************************************/

#ifndef LINEDIALOG_H
#define LINEDIALOG_H

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif
#include <wx/spinctrl.h>
#include "canvases.h"
#include "print/document.h"

class LineDialog: public wxDialog
{
    DECLARE_DYNAMIC_CLASS(LineDialog)
    DECLARE_EVENT_TABLE()
public:
    LineDialog() : m_mainSizer(nullptr), m_startX(nullptr), m_startY(nullptr), m_endX(nullptr), m_endY(nullptr),
    m_style(nullptr), m_preview(nullptr), m_colorSelect(nullptr), m_lineWidth(nullptr), m_section(nullptr),
    m_anchor(nullptr), m_borderTop(nullptr), m_borderBottom(nullptr) {}
    explicit LineDialog(wxWindow *parent);
    LineDialog(wxWindow *parent, const LineElement &element);

    LineElement element() const;
    void setElement(const LineElement &element);
private:
    LineElement m_element;
    wxBoxSizer *m_mainSizer;
    wxSpinCtrl *m_startX;
    wxSpinCtrl *m_startY;
    wxSpinCtrl *m_endX;
    wxSpinCtrl *m_endY;
    wxChoice *m_style;
    LineCanvas *m_preview;
    wxButton *m_colorSelect;
    wxSpinCtrl *m_lineWidth;
    wxChoice *m_section;
    wxCheckBox *m_anchor;
    wxCheckBox *m_borderTop;
    wxCheckBox *m_borderBottom;

    void createControls();
    void fillControls();
    void OnOk(wxCommandEvent& event);
    void OnCancel(wxCommandEvent& event);
    void OnStyle(wxCommandEvent& event);
    void OnColor(wxCommandEvent& event);
    void OnWidth(wxSpinEvent& event);
    void OnSection(wxCommandEvent& event);
    void OnAnchor(wxCommandEvent& event);
    void OnBorderBottom(wxCommandEvent& event);
    void OnBorderTop(wxCommandEvent& event);
};

#endif  /* LINEDIALOG_H */

