/***************************************************************
 * Name:      paragraphdialog.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2015
 * License:   GPL3
 **************************************************************/

#ifndef PARAGRAPHDIALOG_H
#define PARAGRAPHDIALOG_H

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif
#include <wx/spinctrl.h>
#include "canvases.h"
#include "print/document.h"

class ParagraphDialog: public wxDialog
{
    DECLARE_DYNAMIC_CLASS(ParagraphDialog)
    DECLARE_EVENT_TABLE()
public:
    ParagraphDialog() : m_mainSizer(nullptr), m_posX(nullptr), m_posY(nullptr), m_paragraphWidth(nullptr), m_text(nullptr), m_alignment(nullptr),
    m_preview(nullptr), m_fontSelect(nullptr), m_fontColorSelect(nullptr), m_backColorSelect(nullptr), m_border(nullptr), m_borderStyle(nullptr),
    m_borderWidth(nullptr), m_borderColor(nullptr), m_section(nullptr), m_keyword(nullptr) {}
    ParagraphDialog(wxWindow *parent, const TextStyle &style);
    ParagraphDialog(wxWindow *parent, const TextParagraphElement &element);

    TextParagraphElement element() const;
    void setElement(const TextParagraphElement &element);
private:
    TextParagraphElement m_element;
    wxBoxSizer *m_mainSizer;
    wxSpinCtrl *m_posX;
    wxSpinCtrl *m_posY;
    wxSpinCtrl *m_paragraphWidth;
    wxTextCtrl *m_text;
    wxChoice *m_alignment;
    ParagraphCanvas *m_preview;
    wxButton *m_fontSelect;
    wxButton *m_fontColorSelect;
    wxButton *m_backColorSelect;
    wxCheckBox *m_border;
    wxChoice *m_borderStyle;
    wxSpinCtrl *m_borderWidth;
    wxButton *m_borderColor;
    wxChoice *m_section;
    wxButton *m_keyword;

    void createControls();
    void fillControls();
    void OnOk(wxCommandEvent& event);
    void OnCancel(wxCommandEvent& event);
    void OnAlignment(wxCommandEvent& event);
    void OnFont(wxCommandEvent& event);
    void OnFontColor(wxCommandEvent& event);
    void OnBackColor(wxCommandEvent& event);
    void OnText(wxCommandEvent& event);
    void OnWidth(wxSpinEvent& event);
    void OnBorder(wxCommandEvent& event);
    void OnBorderStyle(wxCommandEvent& event);
    void OnBorderWidth(wxSpinEvent& event);
    void OnBorderColor(wxCommandEvent& event);
    void setPreviewSize();
    void OnKeyword(wxCommandEvent& event);
};

#endif  /* PARAGRAPHDIALOG_H */

