/***************************************************************
 * Name:      rectdialog.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2015
 * License:   GPL3
 **************************************************************/

#ifndef RECTDIALOG_H
#define RECTDIALOG_H

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif
#include <wx/spinctrl.h>
#include "canvases.h"
#include "print/document.h"

class RectDialog: public wxDialog
{
    DECLARE_DYNAMIC_CLASS(RectDialog)
    DECLARE_EVENT_TABLE()
public:
    RectDialog() : m_mainSizer(nullptr), m_startX(nullptr), m_startY(nullptr), m_endX(nullptr), m_endY(nullptr),
    m_style(nullptr), m_preview(nullptr), m_colorSelect(nullptr), m_rectWidth(nullptr), m_section(nullptr),
    m_anchor(nullptr), m_fillet(nullptr) {}
    explicit RectDialog(wxWindow *parent);
    RectDialog(wxWindow *parent, const RectangleElement &element);

    RectangleElement element() const;
    void setElement(const RectangleElement &element);
private:
    RectangleElement m_element;
    wxBoxSizer *m_mainSizer;
    wxSpinCtrl *m_startX;
    wxSpinCtrl *m_startY;
    wxSpinCtrl *m_endX;
    wxSpinCtrl *m_endY;
    wxChoice *m_style;
    RectangleCanvas *m_preview;
    wxButton *m_colorSelect;
    wxSpinCtrl *m_rectWidth;
    wxChoice *m_section;
    wxCheckBox *m_anchor;
    wxSpinCtrl *m_fillet;

    void createControls();
    void fillControls();
    void OnOk(wxCommandEvent& event);
    void OnCancel(wxCommandEvent& event);
    void OnStyle(wxCommandEvent& event);
    void OnColor(wxCommandEvent& event);
    void OnWidth(wxSpinEvent& event);
    void OnSection(wxCommandEvent& event);
    void OnFillet(wxSpinEvent& event);
};

#endif  /* RECTDIALOG_H */

