/***************************************************************
 * Name:      rowdialog.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2015
 * License:   GPL3
 **************************************************************/

#ifndef ROWDIALOG_H
#define ROWDIALOG_H

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif
#include <wx/spinctrl.h>
#include <wx/listctrl.h>
#include "canvases.h"
#include "print/document.h"
#include "gui/widgets/dxtoolbar.h"

class RowDialog: public wxDialog
{
    DECLARE_DYNAMIC_CLASS(RowDialog)
    DECLARE_EVENT_TABLE()
public:
    RowDialog() : m_mainSizer(nullptr), m_posX(nullptr), m_posY(nullptr), m_cells(nullptr), m_bar(nullptr),
    m_preview(nullptr), m_fontSelect(nullptr), m_fontColorSelect(nullptr), m_backColorSelect(nullptr), m_border(nullptr), m_borderStyle(nullptr),
    m_borderWidth(nullptr), m_borderColor(nullptr), m_selectedId(-1), m_section(nullptr) {}
    RowDialog(wxWindow *parent, const TextStyle &style);
    RowDialog(wxWindow *parent, const TableRowElement &element);

    TableRowElement element() const;
    void setElement(const TableRowElement &element);
private:
    TableRowElement m_element;
    wxBoxSizer *m_mainSizer;
    wxSpinCtrl *m_posX;
    wxSpinCtrl *m_posY;
    wxListCtrl *m_cells;
    dxToolBar *m_bar;
    RowCanvas *m_preview;
    wxButton *m_fontSelect;
    wxButton *m_fontColorSelect;
    wxButton *m_backColorSelect;
    wxCheckBox *m_border;
    wxChoice *m_borderStyle;
    wxSpinCtrl *m_borderWidth;
    wxButton *m_borderColor;
    long m_selectedId;
    wxChoice *m_section;

    void createControls();
    void fillControls();
    void OnOk(wxCommandEvent& event);
    void OnCancel(wxCommandEvent& event);
    void OnFont(wxCommandEvent& event);
    void OnFontColor(wxCommandEvent& event);
    void OnBackColor(wxCommandEvent& event);
    void OnBorder(wxCommandEvent& event);
    void OnBorderStyle(wxCommandEvent& event);
    void OnBorderWidth(wxSpinEvent& event);
    void OnBorderColor(wxCommandEvent& event);
    void setPreviewSize();
    void setSelectedId(long selectedIndex);
    void OnItemRightClick(wxListEvent& event);
    void OnItemActivated(wxListEvent& event);
    void OnItemSelected(wxListEvent& event);
    void OnItemDeselected(wxListEvent& event);
    void OnEditCell(wxCommandEvent& event);
    void OnCopyCell(wxCommandEvent& event);
    void OnAddCell(wxCommandEvent& event);
    void OnDeleteCell(wxCommandEvent& event);
    void enableTools();
    void fillList();
};

#endif  /* ROWDIALOG_H */

