/***************************************************************
 * Name:      templateeditor.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2015
 * License:   GPL3
 **************************************************************/

#ifndef TEMPLATEEDITOR_H
#define TEMPLATEEDITOR_H

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif
#include "print/document.h"
#include "gui/widgets/dxtoolbar.h"

class TemplateEditor: public wxDialog
{
    DECLARE_DYNAMIC_CLASS(TemplateEditor)
    DECLARE_EVENT_TABLE()
public:
    TemplateEditor() : m_mainSizer(nullptr), m_bar(nullptr), m_canvas(nullptr), m_list(nullptr), m_selectedId(-1), m_edited(false) {}
    explicit TemplateEditor(wxWindow *parent);
private:
    wxBoxSizer *m_mainSizer;
    dxToolBar *m_bar;
    wxPanel *m_canvas;
    wxListBox *m_list;
    long m_selectedId;
    Document m_document;
    bool m_edited;
    wxString m_filename;

    void OnOpen(wxCommandEvent &event);
    void OnSave(wxCommandEvent &event);
    void OnSaveAs(wxCommandEvent &event);
    void OnClear(wxCommandEvent &event);
    void OnPreview(wxCommandEvent &event);
    void OnClose(wxCloseEvent &event);
    void OnItemRightClick(wxMouseEvent &event);
    void OnItemActivated(wxCommandEvent &event);
    void OnItemSelected(wxCommandEvent &event);
    void OnEditElement(wxCommandEvent& event);
    void OnAddElement(wxCommandEvent& event);
    void OnCopyElement(wxCommandEvent& event);
    void OnDeleteElement(wxCommandEvent& event);
    void OnSize(wxSizeEvent& event);
    void OnPaint(wxPaintEvent& event);
    void OnDocumentProperties(wxCommandEvent& event);
    void setSelectedId(long selectedIndex);
    void fillList();
    void drawMargins(wxDC *dc, double mmToLogical);
    void drawHeader(wxDC *dc, double mmToLogical);
    void drawFooter(wxDC *dc, double mmToLogical);
    void enableTools();
    void addLine();
    void addRectangle();
    void addText();
    void addParagraph();
    void addTableRow();
    void addImage();
    void createNeplataceTemplate();
    void createPlatceTemplate();
    void drawElements(wxDC *dc, double mmToLogical, dxElementsArray elements, int posY, int sectionHeight);
};

#endif  /* TEMPLATEEDITOR_H */

