/***************************************************************
 * Name:      templateprint.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2015
 * License:   GPL3
 **************************************************************/

#include <wx/log.h>
#include "templateprint.h"
#include "model/dbmodel.h"
#include "utils/dxsettings.h"
#include "utils/dxutils.h"
#include "tags.h"

wxString parseFirmaLine(wxString line)
{
    Firma fi = idbmodel->firma();
    Adresa ad = idbmodel->getAdresa(fi.adresaid());
    if(line.Contains("{firma-nazev}"))
    {
        line.Replace("{firma-nazev}", fi.nazev());
    }
    if(line.Contains("{firma-ulice}"))
    {
        line.Replace("{firma-ulice}",ad.isValid() ? wxString::Format("%s %s", ad.ulice(), ad.cp()) : " ");
    }
    if(line.Contains("{firma-misto}"))
    {
        line.Replace("{firma-misto}",ad.isValid() ? ad.misto() : " ");
    }
    if(line.Contains("{firma-psc}"))
    {
        line.Replace("{firma-psc}",ad.isValid() ? ad.psc() : " ");
    }
    if(line.Contains("{firma-zeme}"))
    {
        line.Replace("{firma-zeme}",ad.isValid() ? ad.zeme() : " ");
    }
    if(line.Contains("{firma-registrace}"))
    {
        line.Replace("{firma-registrace}",fi.registrace());
    }
    if(line.Contains("{firma-ic}"))
    {
        line.Replace("{firma-ic}",fi.ic());
    }
    if(line.Contains("{firma-dic}"))
    {
        line.Replace("{firma-dic}",fi.dic());
    }
    if(line.Contains("{firma-telefon}"))
    {
        line.Replace("{firma-telefon}",fi.telefon());
    }
    if(line.Contains("{firma-email}"))
    {
        line.Replace("{firma-email}",fi.email());
    }
    if(line.Contains("{firma-www}"))
    {
        line.Replace("{firma-www}",fi.www());
    }
    return line;
}

wxString parseFakturaLine(wxString line)
{
    if(line.Contains("{faktura-cislo}"))
    {
        line.Replace("{faktura-cislo}","FV0001");
    }
    if(line.Contains("{faktura-celkem}"))
    {
        line.Replace("{faktura-celkem}","336.00");
    }
    if(line.Contains("{faktura-vystaveni}"))
    {
        line.Replace("{faktura-vystaveni}",wxDateTime::Today().FormatDate());
    }
    if(line.Contains("{faktura-splatnost}"))
    {
        line.Replace("{faktura-splatnost}",wxDateTime(wxDateTime::Today()+wxDateSpan::Days(idbmodel->getSplatnost(idbmodel->currentRok()))).FormatDate());
    }
    if(line.Contains("{faktura-duzp}"))
    {
        line.Replace("{faktura-duzp}",wxDateTime::Today().FormatDate());
    }
    if(line.Contains("{faktura-uhrada}"))
    {
        line.Replace("{faktura-uhrada}","Bankou");
    }
    if(line.Contains("{faktura-mena}"))
    {
        line.Replace("{faktura-mena}",wxString::FromUTF8("Kč"));
    }
    if(line.Contains("{faktura-banka}"))
    {
        line.Replace("{faktura-banka}","110233500/1313");
    }
    if(line.Contains("{faktura-vs}"))
    {
        line.Replace("{faktura-vs}","16589248");
    }
    if(line.Contains("{faktura-ss}"))
    {
        line.Replace("{faktura-ss}","56665");
    }
    if(line.Contains("{faktura-ks}"))
    {
        line.Replace("{faktura-ks}","0308");
    }
    return line;
}

wxString parseKontaktLine(wxString line)
{
    if(line.Contains("{kontakt-nazev}"))
    {
        line.Replace("{kontakt-nazev}","GoGo s.r.o");
    }
    if(line.Contains("{kontakt-ulice}"))
    {
        line.Replace("{kontakt-ulice}","Gogova 13");
    }
    if(line.Contains("{kontakt-misto}"))
    {
        line.Replace("{kontakt-misto}","Gogov");
    }
    if(line.Contains("{kontakt-psc}"))
    {
        line.Replace("{kontakt-psc}","123 13");
    }
    if(line.Contains("{kontakt-zeme}"))
    {
        line.Replace("{kontakt-zeme}","Planeta Mars");
    }
    if(line.Contains("{kontakt-ic}"))
    {
        line.Replace("{kontakt-ic}","1231345");
    }
    if(line.Contains("{kontakt-dic}"))
    {
        line.Replace("{kontakt-dic}","CZ1231345");
    }
    return line;
}

wxString parsePolozkaLine(wxString line, FaPolozka polozka, int sazba)
{
    if(line.Contains("{polozka-popis}"))
    {
        line.Replace("{polozka-popis}",polozka.popis());
    }
    if(line.Contains("{polozka-pocet}"))
    {
        line.Replace("{polozka-pocet}",wxString::Format("%.3f", polozka.pocet()));
    }
    if(line.Contains("{polozka-mj}"))
    {
        line.Replace("{polozka-mj}", polozka.mj());
    }
    if(line.Contains("{polozka-cenamj}"))
    {
        line.Replace("{polozka-cenamj}",wxString::Format("%.2f", dxutils::cenamj(polozka.cenamj(), polozka.bezdph(), polozka.sazbaid())));
    }
    if(line.Contains("{polozka-sazba}"))
    {
        line.Replace("{polozka-sazba}",wxString::Format("%d", sazba));
    }
    if(line.Contains("{polozka-dph}"))
    {
        line.Replace("{polozka-dph}",wxString::Format("%.2f", polozka.dph()));
    }
    if(line.Contains("{polozka-celkem}"))
    {
        line.Replace("{polozka-celkem}",wxString::Format("%.2f", polozka.celkem()));
    }
    return line;
}

wxString parseSazbaLine(wxString line, int sazba, double zaklad, double dph)
{
    line.Replace("{dph-zaklad}", wxString::Format("%.2f", zaklad));
    line.Replace("{dph-dph}", wxString::Format("%.2f", dph));
    line.Replace("{dph-sazba}", wxString::Format("%d%%", sazba));
    return line;
}

wxString parseLine(wxString line)
{
    if(isRightFakturaTag(line))
    {
        line = parseFakturaLine(line);
    }
    if(isRightFirmaTag(line))
    {
        line = parseFirmaLine(line);
    }
    if(isRightKontaktTag(line))
    {
        line = parseKontaktLine(line);
    }
    return line;
}

TemplatePrint::TemplatePrint(wxWindow *parent, const wxString &title, const Document &doc)
    : m_pageWidth(210), m_pageHeight(297), m_parent(parent), m_title(title)
{
    m_document = new Document();
    m_document->setStyle(doc.style());
    m_document->setPages(doc.pages());
    m_document->setMarginLeft(doc.marginLeft());
    m_document->setMarginRight(doc.marginRight());
    m_document->setMarginTop(doc.marginTop());
    m_document->setMarginBottom(doc.marginBottom());
    m_document->setFooter(doc.footer());
    m_document->setFooterText(doc.footerText());
    m_document->setHeaderLeftText(doc.headerLeftText());
    m_document->setHeaderCenterText(doc.headerCenterText());
    m_document->setHeaderRightText(doc.headerRightText());
    m_document->setHeaderLine(doc.headerLine());
    parseElements(doc);
    int posY = m_document->marginTop();
    int sheight = m_document->sectionHeight(HEADER1);
    moveElements(m_document->elementsBySection(HEADER1), posY, sheight);
    posY += sheight;
    sheight = m_document->sectionHeight(HEADER2);
    moveElements(m_document->elementsBySection(HEADER2), posY, sheight);
    posY += sheight;
    sheight = m_document->sectionHeight(HEADER3);
    moveElements(m_document->elementsBySection(HEADER3), posY, sheight);
    posY += sheight;
    sheight = m_document->sectionHeight(ITEMS);
    moveElements(m_document->elementsBySection(ITEMS), posY, sheight);
    posY += sheight;
    sheight = m_document->sectionHeight(SUM);
    moveElements(m_document->elementsBySection(SUM), posY, sheight);
    posY += sheight;
    sheight = m_document->sectionHeight(DPH);
    moveElements(m_document->elementsBySection(DPH), posY, sheight);
    sheight = m_document->sectionHeight(FOOTER1);
    posY = PAGE_HEIGHT-m_document->marginBottom()-sheight;
    moveElements(m_document->elementsBySection(FOOTER1), posY, sheight);
    sheight = m_document->sectionHeight(FOOTER2);
    posY -= sheight;
    moveElements(m_document->elementsBySection(FOOTER2), posY, sheight);
    sheight = m_document->sectionHeight(FOOTER3);
    posY -= sheight;
    moveElements(m_document->elementsBySection(FOOTER3), posY, sheight);
    moveElements(m_document->elementsBySection(ANYSECTION), 0, 0);
    m_document->clearElementSelection();
}

void TemplatePrint::Preview(bool print)
{
    TemplatePrintout *print1 = new TemplatePrintout(printTitle());
    print1->setDocument(m_document);
    print1->setHeaderLeftText(m_document->headerLeftText());
    print1->setHeaderCenterText(m_document->headerCenterText());
    print1->setHeaderRightText(m_document->headerRightText());
    print1->setHeaderLine(m_document->headerLine());
    print1->setMarginLeft(m_document->marginLeft());
    print1->setMarginRight(m_document->marginRight());
    print1->setMarginTop(m_document->marginTop());
    print1->setMarginBottom(m_document->marginBottom());
    print1->setPages(m_document->pages());
    print1->performPageSetup();
    wxPrintDialogData data(print1->printData());
    data.SetMinPage(1);
    data.SetMaxPage(m_document->pages());
    data.SetFromPage(1);
    data.SetToPage(m_document->pages());
    wxPrintPreview *preview = nullptr;
    if(print)
    {
        TemplatePrintout *print2 = new TemplatePrintout(printTitle());
        print2->setDocument(m_document);
        print2->setHeaderLeftText(m_document->headerLeftText());
        print2->setHeaderCenterText(m_document->headerCenterText());
        print2->setHeaderRightText(m_document->headerRightText());
        print2->setHeaderLine(m_document->headerLine());
        print2->setMarginLeft(m_document->marginLeft());
        print2->setMarginRight(m_document->marginRight());
        print2->setMarginTop(m_document->marginTop());
        print2->setMarginBottom(m_document->marginBottom());
        print2->setPages(m_document->pages());
        print2->performPageSetup();
        preview = new wxPrintPreview(print1, print2, &data);
    }
    else
    {
        preview = new wxPrintPreview(print1, nullptr, &data);
    }
    if (!preview->IsOk())
    {
        delete preview;
        wxMessageBox("There was a problem with previewing.\nPerhaps your current printer is not set correctly?", "Previewing", wxOK);
        return;
    }
    print1->GetPageSizeMM(&m_pageWidth, &m_pageHeight);
    preview->SetZoom(PREVIEW_ZOOM);
    wxPreviewFrame *frame = new wxPreviewFrame(preview, m_parent, m_title, wxDefaultPosition, wxSize(PREVIEW_WIDTHL,PREVIEW_HEIGHT));
    frame->Centre(wxBOTH);
    frame->Initialize();
    frame->Show();
}

void TemplatePrint::Print()
{
    TemplatePrintout *print = new TemplatePrintout(printTitle());
    print->setDocument(m_document);
    print->setHeaderLeftText(m_document->headerLeftText());
    print->setHeaderCenterText(m_document->headerCenterText());
    print->setHeaderRightText(m_document->headerRightText());
    print->setHeaderLine(m_document->headerLine());
    print->setMarginLeft(m_document->marginLeft());
    print->setMarginRight(m_document->marginRight());
    print->setMarginTop(m_document->marginTop());
    print->setMarginBottom(m_document->marginBottom());
    print->setPages(m_document->pages());
    print->performPageSetup();
    wxPrintDialogData data(print->printData());
    data.SetMinPage(1);
    data.SetMaxPage(m_document->pages());
    print->GetPageSizeMM(&m_pageWidth, &m_pageHeight);
    wxPrinter printer(&data);
    printer.Print(m_parent, print, true);
}

Document *TemplatePrint::document() const
{
    return m_document;
}

void TemplatePrint::setDocument(Document *document)
{
    m_document = document;
}

wxString TemplatePrint::printTitle() const
{
    if(m_title.StartsWith(" "))
        return m_title;
    return wxString(" ")+m_title;
}

void TemplatePrint::parseElements(const Document &doc)
{
    dxElementsArray elements = doc.elements();
    for(size_t i=0; i<elements.GetCount(); i++)
    {
        switch(elements[i]->elementType()) {
        case LINE:
        {
            m_document->addElement(*static_cast<LineElement*>(elements[i]));
            break;
        }
        case RECTANGLE:
        {
            m_document->addElement(*static_cast<RectangleElement*>(elements[i]));
            break;
        }
        case IMAGE:
        {
            m_document->addElement(*static_cast<ImageElement*>(elements[i]));
            break;
        }
        case TEXT:
        {
            TextElement el = *static_cast<TextElement*>(elements[i]);
            el.setValue(parseLine(el.value()));
            m_document->addElement(el);
            break;
        }
        case TEXTPARAGRAPH:
        {
            TextParagraphElement el = *static_cast<TextParagraphElement*>(elements[i]);
            wxArrayString valueo = el.value();
            wxArrayString value;
            for(size_t j=0; j<valueo.GetCount(); j++)
            {
                value.Add(parseLine(valueo[j]));
            }
            el.setValue(value);
            m_document->addElement(el);
            break;
        }
        case TABLEROW:
        {
            TableRowElement el = *static_cast<TableRowElement*>(elements[i]);
            wxString line = wxJoin(el.value(), ';');
            if(isRightPolozkaTag(line))
            {
                bool platce = idbmodel->isFirmaPlatce();
                wxArrayString valueo = el.value();
                wxArrayString value;
                for(size_t j=0; j<valueo.GetCount(); j++)
                {
                    value.Add(parsePolozkaLine(valueo[j], FaPolozka(1, wxString::FromUTF8("Položka č.1"), "ks", 1.0, platce?100.0:121.0, 1, 21.0, 121.0, true, 1, 0, 0, false), 21));
                    value[j] = parseLine(value[j]);
                }
                el.setValue(value);
                m_document->addElement(el);
                TableRowElement el1(el);
                value.Clear();
                for(size_t j=0; j<valueo.GetCount(); j++)
                {
                    value.Add(parsePolozkaLine(valueo[j], FaPolozka(1, wxString::FromUTF8("Položka č.2"), "ks", 1.0, platce?100.0:115.0, 1, 15.0, 115.0, true, 1, 0, 0, false), 15));
                    value[j] = parseLine(value[j]);
                }
                el1.setValue(value);
                el1.setPos(wxPoint(el1.pos().x,el1.pos().y+el.height()));
                m_document->addElement(el1);
                TableRowElement el2(el1);
                value.Clear();
                for(size_t j=0; j<valueo.GetCount(); j++)
                {
                    value.Add(parsePolozkaLine(valueo[j], FaPolozka(1, wxString::FromUTF8("Položka č.3"), "ks", 1.0, 100.0, 1, 0.0, 100.0, true, 1, 0, 0, false), 0));
                    value[j] = parseLine(value[j]);
                }
                el2.setValue(value);
                el2.setPos(wxPoint(el2.pos().x,el2.pos().y+el1.height()));
                m_document->addElement(el2);
            }
            else if(idbmodel->isFirmaPlatce() && isRightSazbaTag(line))
            {
                wxArrayString valueo = el.value();
                wxArrayString value;
                for(size_t j=0; j<valueo.GetCount(); j++)
                {
                    value.Add(parseSazbaLine(valueo[j], 21, 100.0, 21.0));
                    value[j] = parseLine(value[j]);
                }
                el.setValue(value);
                m_document->addElement(el);
                TableRowElement el1(el);
                value.Clear();
                for(size_t j=0; j<valueo.GetCount(); j++)
                {
                    value.Add(parseSazbaLine(valueo[j], 15, 100.0, 15.0));
                    value[j] = parseLine(value[j]);
                }
                el1.setValue(value);
                el1.setPos(wxPoint(el1.pos().x,el1.pos().y+el.height()));
                m_document->addElement(el1);
                TableRowElement el2(el1);
                value.Clear();
                for(size_t j=0; j<valueo.GetCount(); j++)
                {
                    value.Add(parseSazbaLine(valueo[j], 0, 100.0, 0.0));
                    value[j] = parseLine(value[j]);
                }
                el2.setValue(value);
                el2.setPos(wxPoint(el2.pos().x,el2.pos().y+el1.height()));
                m_document->addElement(el2);
            }
            else
            {
                wxArrayString valueo = el.value();
                wxArrayString value;
                for(size_t j=0; j<valueo.GetCount(); j++)
                {
                    value.Add(parseLine(valueo[j]));
                }
                el.setValue(value);
                m_document->addElement(el);
            }
            break;
        }
        default: ;
        }
    }
}

void TemplatePrint::moveElements(dxElementsArray elements, int posY, int sectionHeight)
{
    for(size_t i=0; i<elements.GetCount(); i++)
    {
        switch(elements[i]->elementType()) {
        case LINE:
        {
            LineElement *el = static_cast<LineElement*>(elements[i]);
            if(el->anchor())
            {
                el->setStart(wxPoint(el->start().x, posY));
                el->setEnd(wxPoint(el->end().x, posY+sectionHeight));
                break;
            }
            if(el->borderTop())
            {
                el->setStart(wxPoint(el->start().x, posY));
                el->setEnd(wxPoint(el->end().x, posY));
                break;
            }
            if(el->borderBottom())
            {
                el->setStart(wxPoint(el->start().x, posY+sectionHeight));
                el->setEnd(wxPoint(el->end().x, posY+sectionHeight));
                break;
            }
            el->setStart(wxPoint(el->start().x, el->start().y+posY));
            el->setEnd(wxPoint(el->end().x, el->end().y+posY));
            break;
        }
        case RECTANGLE:
        {
            RectangleElement *el = static_cast<RectangleElement*>(elements[i]);
            if(el->anchor())
            {
                el->setStart(wxPoint(el->start().x, posY));
                el->setSize(wxSize(el->size().x, sectionHeight));
                break;
            }
            el->setStart(wxPoint(el->start().x, el->start().y+posY));
            break;
        }
        case IMAGE:
        {
            ImageElement *el = static_cast<ImageElement*>(elements[i]);
            el->setPos(wxPoint(el->pos().x, el->pos().y+posY));
            break;
        }
        case TEXT:
        {
            TextElement *el = static_cast<TextElement*>(elements[i]);
            el->setPos(wxPoint(el->pos().x, el->pos().y+posY));
            break;
        }
        case TEXTPARAGRAPH:
        {
            TextParagraphElement *el = static_cast<TextParagraphElement*>(elements[i]);
            el->setPos(wxPoint(el->pos().x, el->pos().y+posY));
            break;
        }
        case TABLEROW:
        {
            TableRowElement *el = static_cast<TableRowElement*>(elements[i]);
            el->setPos(wxPoint(el->pos().x, el->pos().y+posY));
            break;
        }
        default: ;
        }
    }
}

TemplatePrintout::TemplatePrintout(const wxString &title)
    : dxPrint(title), m_document(nullptr)
{
}

bool TemplatePrintout::OnPrintPage(int page)
{
    wxDC* dc = GetDC();
    if (dc==nullptr || !dc->IsOk())
    {
        wxLogError(wxString::FromUTF8("Chyba při přípravě tisku, ukončuji tisk"));
        return false;
    }
    drawHeader(dc, m_mmToLogical);
    if(m_document)
    {
        dxElementsArray elements = m_document->elementsByPage(page);
        for(size_t i=0; i<elements.GetCount(); i++)
        {
            elements[i]->DrawToDC(dc, m_mmToLogical, -PAGE_OFFSET, -PAGE_OFFSET);
        }
    }
    drawFooter(page, dc, m_mmToLogical);
    return true;
}

void TemplatePrintout::setDocument(Document *document)
{
    if(document)
    {
        m_document = document;
        m_footer = m_document->footer();
        m_footerText = m_document->footerText();
    }
}
