/***************************************************************
 * Name:      textdialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2015
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include <wx/fontdata.h>
#include <wx/fontdlg.h>
#include <wx/colordlg.h>
#include "textdialog.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "tags.h"

IMPLEMENT_DYNAMIC_CLASS(TextDialog, wxDialog)

BEGIN_EVENT_TABLE(TextDialog, wxDialog)
    EVT_BUTTON(wxID_OK, TextDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, TextDialog::OnCancel)
    EVT_CHOICE(ID_TEXT_ALIGNMENT, TextDialog::OnAlignment)
    EVT_BUTTON(ID_TEXT_FONT, TextDialog::OnFont)
    EVT_BUTTON(ID_TEXT_FONTCOLOR, TextDialog::OnFontColor)
    EVT_BUTTON(ID_TEXT_BACKCOLOR, TextDialog::OnBackColor)
    EVT_TEXT(ID_TEXT_TEXT, TextDialog::OnText)
    EVT_SPINCTRL(ID_TEXT_WIDTH, TextDialog::OnWidth)
    EVT_BUTTON(ID_TEXT_KEYWORD, TextDialog::OnKeyword)
    EVT_CHECKBOX(ID_TEXT_FORCESPACE, TextDialog::OnForceSpace)
END_EVENT_TABLE()

//Novy text
TextDialog::TextDialog(wxWindow *parent, const TextStyle &style)
    : wxDialog(parent, wxID_ANY, "Vlastnosti textu"), m_element(TextElement(1, "Text", wxPoint(0,0), 100, wxTEXT_ALIGNMENT_LEFT, style))
{
    createControls();
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

//Uprava textu
TextDialog::TextDialog(wxWindow *parent, const TextElement &element)
    : wxDialog(parent, wxID_ANY, "Vlastnosti textu"), m_element(element)
{
    createControls();
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

TextElement TextDialog::element() const
{
    return m_element;
}

void TextDialog::setElement(const TextElement &element)
{
    m_element = element;
}

void TextDialog::createControls()
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Sekce"), wxGBPosition(0,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_section = new wxChoice(this, ID_LINE_SECTION);
    m_section->Append(wxString::FromUTF8("Hlavička 1"));
    m_section->Append(wxString::FromUTF8("Hlavička 2"));
    m_section->Append(wxString::FromUTF8("Hlavička 3"));
    m_section->Append(wxString::FromUTF8("Položky"));
    m_section->Append("Celkem");
    m_section->Append("DPH");
    m_section->Append(wxString::FromUTF8("Patička 3"));
    m_section->Append(wxString::FromUTF8("Patička 2"));
    m_section->Append(wxString::FromUTF8("Patička 1"));
    m_section->Append(wxString::FromUTF8("Žádná"));
    m_section->SetSelection(10);
    gridSizer->Add(m_section, wxGBPosition(0,1), wxGBSpan(1,3), wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Pozice textu - Xová souřadnice")), wxGBPosition(1,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_posX = new wxSpinCtrl(this);
    m_posX->SetRange(0,210);
    gridSizer->Add(m_posX, wxGBPosition(1,1), wxGBSpan(1,3), wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Pozice textu - Yová souřadnice")), wxGBPosition(2,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_posY = new wxSpinCtrl(this);
    m_posY->SetRange(0,297);
    gridSizer->Add(m_posY, wxGBPosition(2,1), wxGBSpan(1,3), wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Šířka textu")), wxGBPosition(3,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_textWidth = new wxSpinCtrl(this, ID_TEXT_WIDTH);
    m_textWidth->SetRange(0,210);
    gridSizer->Add(m_textWidth, wxGBPosition(3,1), wxGBSpan(1,3), wxEXPAND);
    m_text = new wxTextCtrl(this, ID_TEXT_TEXT);
    gridSizer->Add(m_text, wxGBPosition(4,0), wxGBSpan(1,3), wxEXPAND);
    m_keyword = new wxButton(this, ID_TEXT_KEYWORD, "...");
    m_keyword->SetToolTip(wxString::FromUTF8("Vložit klíčové slovo"));
    gridSizer->Add(m_keyword, wxGBPosition(4,3), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Zarovnání")), wxGBPosition(5,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_alignment = new wxChoice(this, ID_TEXT_ALIGNMENT);
    m_alignment->Append("Doleva");
    m_alignment->Append(wxString::FromUTF8("Na střed"));
    m_alignment->Append("Doprava");
    gridSizer->Add(m_alignment, wxGBPosition(5,1), wxGBSpan(1,3), wxEXPAND);
    m_fontSelect = new wxButton(this, ID_TEXT_FONT, "Font");
    gridSizer->Add(m_fontSelect, wxGBPosition(6,1), wxDefaultSpan, wxEXPAND);
    m_fontColorSelect = new wxButton(this, ID_TEXT_FONTCOLOR, "Barva textu");
    gridSizer->Add(m_fontColorSelect, wxGBPosition(6,2), wxDefaultSpan, wxEXPAND);
    m_backColorSelect = new wxButton(this, ID_TEXT_BACKCOLOR, wxString::FromUTF8("Barva pozadí"));
    gridSizer->Add(m_backColorSelect, wxGBPosition(6,3), wxDefaultSpan, wxEXPAND);
    m_forceSpace = new wxCheckBox(this, ID_TEXT_FORCESPACE, "Zalomit natvrdo");
    m_forceSpace->SetValue(false);
    gridSizer->Add(m_forceSpace, wxGBPosition(7,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Náhled")), wxGBPosition(8,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_mainSizer->Add(gridSizer, 1, wxLEFT|wxTOP|wxRIGHT, 5);
    m_preview = new TextCanvas(this);
    m_mainSizer->Add(m_preview, 0, wxALL, 2);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void TextDialog::fillControls()
{
    m_posX->SetValue(m_element.pos().x);
    m_posY->SetValue(m_element.pos().y);
    m_textWidth->SetValue(m_element.width());
    m_text->SetValue(m_element.value());
    m_alignment->SetSelection(m_element.alignment()-1);
    m_section->SetSelection(m_element.section());
    m_forceSpace->SetValue(m_element.forceSpace());
    setPreviewSize();
    m_preview->setElement(m_element);
}

void TextDialog::OnOk(wxCommandEvent& /*event*/)
{
    m_element.setPage(1);
    m_element.setPos(wxPoint(m_posX->GetValue(), m_posY->GetValue()));
    m_element.setWidth(m_textWidth->GetValue());
    m_element.setAlignment(static_cast<wxTextAttrAlignment>(m_alignment->GetSelection()+1));
    m_element.setValue(m_text->GetValue());
    m_element.setSection(static_cast<DOCSECTION>(m_section->GetSelection()));
    m_element.setForceSpace(m_forceSpace->GetValue());
    EndModal(wxID_OK);
}

void TextDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

void TextDialog::OnAlignment(wxCommandEvent& /*event*/)
{
    m_element.setAlignment(static_cast<wxTextAttrAlignment>(m_alignment->GetSelection()+1));
    m_preview->SetAlignment(static_cast<wxTextAttrAlignment>(m_alignment->GetSelection()+1));
    m_preview->Refresh();
}

void TextDialog::OnFont(wxCommandEvent& /*event*/)
{
    wxFontData data;
    data.SetInitialFont(m_element.style().font());
    data.SetColour(m_element.style().fontColour());
    wxFontDialog dialog(this, data);
    if(dialog.ShowModal() == wxID_OK)
    {
        wxFontData retData = dialog.GetFontData();
        TextStyle style = m_element.style();
        style.setFont(retData.GetChosenFont());
        style.setFontColour(retData.GetColour());
        m_element.setStyle(style);
        setPreviewSize();
        m_preview->setElement(m_element);
        m_mainSizer->Layout();
        m_mainSizer->Fit(this);
        m_mainSizer->SetSizeHints(this);
        m_preview->Refresh();
    }
}

void TextDialog::OnFontColor(wxCommandEvent& /*event*/)
{
    wxColourData data;
    data.SetColour(m_element.style().fontColour());
    wxColourDialog dialog(this, &data);
    dialog.SetTitle("Vyberte barvu fontu");
    if(dialog.ShowModal() == wxID_OK)
    {
        data = dialog.GetColourData();
        TextStyle style = m_element.style();
        style.setFontColour(data.GetColour());
        m_element.setStyle(style);
        m_preview->setElement(m_element);
        m_preview->Refresh();
    }
}

void TextDialog::OnBackColor(wxCommandEvent& /*event*/)
{
    wxColourData data;
    data.SetColour(m_element.style().backColour());
    wxColourDialog dialog(this, &data);
    dialog.SetTitle(wxString::FromUTF8("Vyberte barvu pozadí"));
    if(dialog.ShowModal() == wxID_OK)
    {
        data = dialog.GetColourData();
        TextStyle style = m_element.style();
        style.setBackColour(data.GetColour());
        m_element.setStyle(style);
        m_preview->setElement(m_element);
        m_preview->Refresh();
    }
}

void TextDialog::OnText(wxCommandEvent& /*event*/)
{
    m_element.setValue(m_text->GetValue());
    setPreviewSize();
    m_preview->setElement(m_element);
    m_mainSizer->Layout();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
    m_preview->Refresh();
}

void TextDialog::OnWidth(wxSpinEvent& /*event*/)
{
    m_element.setWidth(m_textWidth->GetValue());
    setPreviewSize();
    m_preview->setElement(m_element);
    m_mainSizer->Layout();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
    m_preview->Refresh();
}

void TextDialog::setPreviewSize()
{
    wxScreenDC dc;
    int width = dxutils::mmToPx(m_textWidth->GetValue(), &dc, true);
    m_preview->SetMinSize(wxSize(width,m_element.height(true))+wxSize(2*IMAGE_Y_OFFSET,2*IMAGE_Y_OFFSET));
    m_preview->SetSize(wxSize(width,m_element.height(true))+wxSize(2*IMAGE_Y_OFFSET,2*IMAGE_Y_OFFSET));
}

void TextDialog::OnKeyword(wxCommandEvent& /*event*/)
{
    wxArrayString choices;
    for(size_t i=0; i<numFakturaTags; i++)
    {
        choices.Add(fakturaTags[i]);
    }
    for(size_t i=0; i<numFirmaTags; i++)
    {
        choices.Add(firmaTags[i]);
    }
    for(size_t i=0; i<numKontaktTags; i++)
    {
        choices.Add(kontaktTags[i]);
    }
    wxSingleChoiceDialog vyber(this, wxString::FromUTF8("Vyberte klíčové slovo"), wxString::FromUTF8("Klíčové slovo"), choices);
    if(vyber.ShowModal() == wxID_OK)
    {
        m_text->AppendText(vyber.GetStringSelection());
    }
}

void TextDialog::OnForceSpace(wxCommandEvent &/*event*/)
{
    m_element.setForceSpace(m_forceSpace->GetValue());
    setPreviewSize();
    m_preview->setElement(m_element);
    m_mainSizer->Layout();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
    m_preview->Refresh();
}
