/***************************************************************
 * Name:      textdialog.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2015
 * License:   GPL3
 **************************************************************/

#ifndef TEXTDIALOG_H
#define TEXTDIALOG_H

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif
#include <wx/spinctrl.h>
#include "canvases.h"
#include "print/document.h"

class TextDialog: public wxDialog
{
    DECLARE_DYNAMIC_CLASS(TextDialog)
    DECLARE_EVENT_TABLE()
public:
    TextDialog() : m_mainSizer(nullptr), m_posX(nullptr), m_posY(nullptr), m_textWidth(nullptr), m_text(nullptr), m_alignment(nullptr),
        m_preview(nullptr), m_fontSelect(nullptr), m_fontColorSelect(nullptr), m_backColorSelect(nullptr), m_section(nullptr),
        m_keyword(nullptr), m_forceSpace(nullptr) {}
    TextDialog(wxWindow *parent, const TextStyle &style);
    TextDialog(wxWindow *parent, const TextElement &element);

    TextElement element() const;
    void setElement(const TextElement &element);
private:
    TextElement m_element;
    wxBoxSizer *m_mainSizer;
    wxSpinCtrl *m_posX;
    wxSpinCtrl *m_posY;
    wxSpinCtrl *m_textWidth;
    wxTextCtrl *m_text;
    wxChoice *m_alignment;
    TextCanvas *m_preview;
    wxButton *m_fontSelect;
    wxButton *m_fontColorSelect;
    wxButton *m_backColorSelect;
    wxChoice *m_section;
    wxButton *m_keyword;
    wxCheckBox *m_forceSpace;

    void createControls();
    void fillControls();
    void OnOk(wxCommandEvent& event);
    void OnCancel(wxCommandEvent& event);
    void OnAlignment(wxCommandEvent& event);
    void OnFont(wxCommandEvent& event);
    void OnFontColor(wxCommandEvent& event);
    void OnBackColor(wxCommandEvent& event);
    void OnText(wxCommandEvent& event);
    void OnWidth(wxSpinEvent& event);
    void setPreviewSize();
    void OnKeyword(wxCommandEvent& event);
    void OnForceSpace(wxCommandEvent& event);
};

#endif  /* TEXTDIALOG_H */

