/***************************************************************
 * Name:      faktura.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#ifndef FAKTURA_H
#define FAKTURA_H

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif

class Faktura {
public:
    Faktura() : m_id(-1), m_protistranaid(-1), m_celkem(0.0), m_typid(-1), m_uhradaid(-1),
    m_pokladnaid(-1), m_bankaid(-1), m_menaid(-1), m_predkontaceid(-1), m_obdobirok(0), m_vydana(false),
    m_platce(false), m_dphdruhid(-1), m_kurz(1.0), m_pausal(0) {}
    Faktura(int id, int protistranaid, const wxString &popis, double celkem, const wxString &cislo, const wxString &vs,
            const wxString &ss, const wxString &ks, int typid, int uhradaid, int pokladnaid, int bankaid,
            int menaid, int predkontaceid, int obdobirok, bool vydana, bool platce, const wxDateTime &vystaveni,
            const wxDateTime &duzp, const wxDateTime &dppd, const wxDateTime &splatnost, int dphdruhid, double kurz=1.0, int pausal=0);

    bool isValid() const;
    int id() const;
    void setId(int id);
    int protistranaid() const;
    void setProtistranaid(int protistranaid);
    wxString popis() const;
    void setPopis(const wxString &popis);
    double celkem() const;
    void setCelkem(double celkem);
    wxString cislo() const;
    void setCislo(const wxString &cislo);
    wxString vs() const;
    void setVs(const wxString &vs);
    wxString ss() const;
    void setSs(const wxString &ss);
    wxString ks() const;
    void setKs(const wxString &ks);
    int typid() const;
    void setTypid(int typid);
    int uhradaid() const;
    void setUhradaid(int uhradaid);
    int pokladnaid() const;
    void setPokladnaid(int pokladnaid);
    int bankaid() const;
    void setBankaid(int bankaid);
    int menaid() const;
    void setMenaid(int menaid);
    int predkontaceid() const;
    void setPredkontaceid(int predkontaceid);
    int obdobirok() const;
    void setObdobirok(int obdobirok);
    bool vydana() const;
    void setVydana(bool vydana);
    bool platce() const;
    void setPlatce(bool platce);
    wxDateTime vystaveni() const;
    void setVystaveni(const wxDateTime &vystaveni);
    wxDateTime duzp() const;
    void setDuzp(const wxDateTime &duzp);
    wxDateTime splatnost() const;
    void setSplatnost(const wxDateTime &splatnost);
    wxDateTime dppd() const;
    void setDppd(const wxDateTime &dppd);
    int dphdruhid() const;
    void setDphdruhid(int dphdruhid);
    double kurz() const;
    void setKurz(double kurz);
    int pausal() const;
    void setPausal(int pausal);
private:
    int m_id;
    int m_protistranaid;
    wxString m_popis;
    double m_celkem;
    wxString m_cislo;
    wxString m_vs;
    wxString m_ss;
    wxString m_ks;
    int m_typid;
    int m_uhradaid;
    int m_pokladnaid;
    int m_bankaid;
    int m_menaid;
    int m_predkontaceid;
    int m_obdobirok;
    bool m_vydana;
    bool m_platce;
    wxDateTime m_vystaveni;
    wxDateTime m_duzp;
    wxDateTime m_dppd;
    wxDateTime m_splatnost;
    int m_dphdruhid;
    double m_kurz;
    int m_pausal;
};

class FakturaView {
public:
    FakturaView() : m_id(-1), m_celkem(0.0), m_uhrazeno(0.0) {}
    FakturaView(int id, const wxString &cislo, const wxString &popis, const wxString &protistrana, double celkem, const wxDateTime &vystaveni, const wxDateTime &splatnost,
           double uhrazeno, bool platce, int typid);

    int id() const;
    void setId(int id);
    wxString cislo() const;
    void setCislo(const wxString &cislo);
    wxString popis() const;
    void setPopis(const wxString &popis);
    wxString protistrana() const;
    void setProtistrana(const wxString &protistrana);
    double celkem() const;
    void setCelkem(double celkem);
    wxDateTime vystaveni() const;
    void setVystaveni(const wxDateTime &vystaveni);
    wxDateTime splatnost() const;
    void setSplatnost(const wxDateTime &splatnost);
    double uhrazeno() const;
    void setUhrazeno(double uhrazeno);
    bool platce() const;
    void setPlatce(bool platce);
    int typid() const;
    void setTypid(int typid);
private:
    int m_id;
    wxString m_cislo;
    wxString m_popis;
    wxString m_protistrana;
    double m_celkem;
    wxDateTime m_vystaveni;
    wxDateTime m_splatnost;
    double m_uhrazeno;
    bool m_platce;
    int m_typid;
};

#endif  /* FAKTURA_H */

