/***************************************************************
 * Name:      dbsort.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#ifndef DBSORT_H
#define DBSORT_H

#include "dbmodel.h"

static int dxSortDphSazby(DphSazba ***s1, DphSazba ***s2)
{
    if((**s1)->sazba() == (**s2)->sazba())
        return 0;
    if((**s1)->sazba() < (**s2)->sazba())
        return 1;
    return -1;
}

static int dxSortFaPolozky(FaPolozka **fp1, FaPolozka **fp2)
{
    if((*fp1)->id() != (*fp2)->id())
    {
        if((*fp1)->id() < (*fp2)->id())
            return 1;
        return -1;
    }
    return 0;
}

static int dxSortFaPolozkaByDphSazba(FaPolozka **fp1, FaPolozka **fp2)
{
    if(idbmodel->getDphSazba((*fp1)->sazbaid()).sazba() != idbmodel->getDphSazba((*fp2)->sazbaid()).sazba())
    {
        if(idbmodel->getDphSazba((*fp1)->sazbaid()).sazba() < idbmodel->getDphSazba((*fp2)->sazbaid()).sazba())
            return 1;
        return -1;
    }
    return 0;
}

static int dxSortObdobi(Obdobi ***o1, Obdobi ***o2)
{
    if((**o1)->rok() == (**o2)->rok())
        return 0;
    if((**o1)->rok() < (**o2)->rok())
        return -1;
    return 1;
}

static int dxSortDenikByIdAsc(Denik **d1, Denik **d2)
{
    if((*d1)->id() == (*d2)->id())
        return 0;
    if((*d1)->id() < (*d2)->id())
        return -1;
    return 1;
}

static int dxSortDenikByIdDsc(Denik **d1, Denik **d2)
{
    if((*d1)->id() == (*d2)->id())
        return 0;
    if((*d1)->id() < (*d2)->id())
        return 1;
    return -1;
}

static int dxSortDenikByDatumAsc(Denik **d1, Denik **d2)
{
    if((*d1)->datum().IsSameDate((*d2)->datum()))
        return dxSortDenikByIdAsc(d1,d2);
    if((*d1)->datum().IsEarlierThan((*d2)->datum()))
        return -1;
    return 1;
}

static int dxSortDenikByDatumDsc(Denik **d1, Denik **d2)
{
    if((*d1)->datum().IsSameDate((*d2)->datum()))
        return dxSortDenikByIdDsc(d1,d2);
    if((*d1)->datum().IsEarlierThan((*d2)->datum()))
        return 1;
    return -1;
}

static int dxSortDenikByCastkaAsc(Denik **d1, Denik **d2)
{
    if((*d1)->castka() == (*d2)->castka())
        return dxSortDenikByIdAsc(d1,d2);
    if((*d1)->castka() < (*d2)->castka())
        return -1;
    return 1;
}

static int dxSortDenikByCastkaDsc(Denik **d1, Denik **d2)
{
    if((*d1)->castka() == (*d2)->castka())
        return dxSortDenikByIdDsc(d1,d2);
    if((*d1)->castka() < (*d2)->castka())
        return 1;
    return -1;
}

static int dxSortDenikByCisloAsc(Denik **d1, Denik **d2)
{
    if((*d1)->cislo() == (*d2)->cislo())
        return dxSortDenikByIdAsc(d1,d2);
    return (*d1)->cislo().CmpNoCase((*d2)->cislo());
}

static int dxSortDenikByCisloDsc(Denik **d1, Denik **d2)
{
    if((*d2)->cislo() == (*d1)->cislo())
        return dxSortDenikByIdDsc(d1,d2);
    return (*d2)->cislo().CmpNoCase((*d1)->cislo());
}

static int dxSortDenikByPopisAsc(Denik **d1, Denik **d2)
{
    if((*d1)->popis() == (*d2)->popis())
        return dxSortDenikByIdAsc(d1,d2);
    return (*d1)->popis().CmpNoCase((*d2)->popis());
}

static int dxSortDenikByPopisDsc(Denik **d1, Denik **d2)
{
    if((*d1)->popis() == (*d2)->popis())
        return dxSortDenikByIdDsc(d1,d2);
    return (*d2)->popis().CmpNoCase((*d1)->popis());
}

static int dxSortDenikByPrijemAsc(Denik **d1, Denik **d2)
{
    int s1 = idbmodel->isPredkontacePrijem((*d1)->predkontaceid())?0:1;
    int s2 = idbmodel->isPredkontacePrijem((*d2)->predkontaceid())?0:1;
    if(s1 == s2)
        return dxSortDenikByIdAsc(d1,d2);
    if(s1 < s2)
        return -1;
    return 1;
}

static int dxSortDenikByPrijemDsc(Denik **d1, Denik **d2)
{
    int s1 = idbmodel->isPredkontacePrijem((*d1)->predkontaceid())?0:1;
    int s2 = idbmodel->isPredkontacePrijem((*d2)->predkontaceid())?0:1;
    if(s1 == s2)
        return dxSortDenikByIdDsc(d1,d2);
    if(s1 < s2)
        return 1;
    return -1;
}

static int dxSortDenikByPokladnaAsc(Denik **d1, Denik **d2)
{
    if(idbmodel->getPokladnaZkratka((*d1)->pokladnaid()) == idbmodel->getPokladnaZkratka((*d2)->pokladnaid()))
        return dxSortDenikByIdAsc(d1,d2);
    return idbmodel->getPokladnaZkratka((*d1)->pokladnaid()).CmpNoCase(idbmodel->getPokladnaZkratka((*d2)->pokladnaid()));
    return 0;
}

static int dxSortDenikByPokladnaDsc(Denik **d1, Denik **d2)
{
    if(idbmodel->getPokladnaZkratka((*d1)->pokladnaid()) == idbmodel->getPokladnaZkratka((*d2)->pokladnaid()))
        return dxSortDenikByIdDsc(d1,d2);
    return idbmodel->getPokladnaZkratka((*d2)->pokladnaid()).CmpNoCase(idbmodel->getPokladnaZkratka((*d1)->pokladnaid()));
    return 0;
}

static int dxSortDenikByBankaAsc(Denik **d1, Denik **d2)
{
    if(idbmodel->getBankaZkratka((*d1)->bankaid()) == idbmodel->getBankaZkratka((*d2)->bankaid()))
        return dxSortDenikByIdAsc(d1,d2);
    return idbmodel->getBankaZkratka((*d1)->bankaid()).CmpNoCase(idbmodel->getBankaZkratka((*d2)->bankaid()));
    return 0;
}

static int dxSortDenikByBankaDsc(Denik **d1, Denik **d2)
{
    if(idbmodel->getBankaZkratka((*d1)->bankaid()) == idbmodel->getBankaZkratka((*d2)->bankaid()))
            return dxSortDenikByIdDsc(d1,d2);
    return idbmodel->getBankaZkratka((*d2)->bankaid()).CmpNoCase(idbmodel->getBankaZkratka((*d1)->bankaid()));
    return 0;
}

static int dxSortDenikByVypisAsc(Denik **d1, Denik **d2)
{
    if((*d1)->vypis() == (*d2)->vypis())
        return dxSortDenikByIdAsc(d1,d2);
    if((*d1)->vypis() < (*d2)->vypis())
        return -1;
    return 1;
}

static int dxSortDenikByVypisDsc(Denik **d1, Denik **d2)
{
    if((*d1)->vypis() == (*d2)->vypis())
        return dxSortDenikByIdDsc(d1,d2);
    if((*d1)->vypis() < (*d2)->vypis())
        return 1;
    return -1;
}

static int dxSortDenikBySloupecAsc(Denik **d1, Denik **d2)
{
    int id1 = idbmodel->denikSloupecByDenikId((*d1)->id());
    int id2 = idbmodel->denikSloupecByDenikId((*d2)->id());
    if(id1 == id2)
        return dxSortDenikByIdAsc(d1,d2);
    if(id1 < id2)
        return -1;
    return 1;
}

static int dxSortDenikBySloupecDsc(Denik **d1, Denik **d2)
{
    int id1 = idbmodel->denikSloupecByDenikId((*d1)->id());
    int id2 = idbmodel->denikSloupecByDenikId((*d2)->id());
    if(id1 == id2)
        return dxSortDenikByIdDsc(d1,d2);
    if(id1 < id2)
        return 1;
    return -1;
}

static int dxSortDenikByPausalAsc(Denik **d1, Denik **d2)
{
    if((*d1)->pausal() == (*d2)->pausal())
        return dxSortDenikByIdAsc(d1,d2);
    if((*d1)->pausal() < (*d2)->pausal())
        return -1;
    return 1;
}

static int dxSortDenikByPausalDsc(Denik **d1, Denik **d2)
{
    if((*d1)->pausal() == (*d2)->pausal())
        return dxSortDenikByIdDsc(d1,d2);
    if((*d1)->pausal() < (*d2)->pausal())
        return 1;
    return -1;
}

static int dxSortFakturyViewByIdAsc(FakturaView **f1, FakturaView **f2)
{
    if((*f1)->id() == (*f2)->id())
        return 0;
    if((*f1)->id() < (*f2)->id())
        return -1;
    return 1;
}

static int dxSortFakturyViewByIdDsc(FakturaView **f1, FakturaView **f2)
{
    if((*f1)->id() == (*f2)->id())
        return 0;
    if((*f1)->id() < (*f2)->id())
        return 1;
    return -1;
}

static int dxSortFakturyViewByDokladAsc(FakturaView **f1, FakturaView **f2)
{
    if((*f1)->cislo() == (*f2)->cislo())
        return dxSortFakturyViewByIdAsc(f1,f2);
    return (*f1)->cislo().CmpNoCase((*f2)->cislo());
}

static int dxSortFakturyViewByDokladDsc(FakturaView **f1, FakturaView **f2)
{
    if((*f1)->cislo() == (*f2)->cislo())
        return dxSortFakturyViewByIdDsc(f1,f2);
    return (*f2)->cislo().CmpNoCase((*f1)->cislo());
}

static int dxSortFakturyViewByPopisAsc(FakturaView **f1, FakturaView **f2)
{
    if((*f1)->popis() == (*f2)->popis())
        return dxSortFakturyViewByIdAsc(f1,f2);
    return (*f1)->popis().CmpNoCase((*f2)->popis());
}

static int dxSortFakturyViewByPopisDsc(FakturaView **f1, FakturaView **f2)
{
    if((*f1)->popis() == (*f2)->popis())
        return dxSortFakturyViewByIdDsc(f1,f2);
    return (*f2)->popis().CmpNoCase((*f1)->popis());
}

static int dxSortFakturyViewByProtistanaAsc(FakturaView **f1, FakturaView **f2)
{
    if((*f1)->protistrana() == (*f2)->protistrana())
        return dxSortFakturyViewByIdAsc(f1,f2);
    return (*f1)->protistrana().CmpNoCase((*f2)->protistrana());
}

static int dxSortFakturyViewByProtistranaDsc(FakturaView **f1, FakturaView **f2)
{
    if((*f1)->protistrana() == (*f2)->protistrana())
        return dxSortFakturyViewByIdDsc(f1,f2);
    return (*f2)->protistrana().CmpNoCase((*f1)->protistrana());
}

static int dxSortFakturyViewByVystavenaAsc(FakturaView **f1, FakturaView **f2)
{
    if((*f1)->vystaveni().IsSameDate((*f2)->vystaveni()))
        return dxSortFakturyViewByIdAsc(f1,f2);
    if((*f1)->vystaveni().IsEarlierThan((*f2)->vystaveni()))
        return -1;
    return 1;
}

static int dxSortFakturyViewByVystavenaDsc(FakturaView **f1, FakturaView **f2)
{
    if((*f1)->vystaveni().IsSameDate((*f2)->vystaveni()))
        return dxSortFakturyViewByIdDsc(f1,f2);
    if((*f1)->vystaveni().IsEarlierThan((*f2)->vystaveni()))
        return 1;
    return -1;
}

static int dxSortFakturyViewBySplatnaAsc(FakturaView **f1, FakturaView **f2)
{
    if((*f1)->splatnost().IsSameDate((*f2)->splatnost()))
        return dxSortFakturyViewByIdAsc(f1,f2);
    if((*f1)->splatnost().IsEarlierThan((*f2)->splatnost()))
        return -1;
    return 1;
}

static int dxSortFakturyViewBySplatnaDsc(FakturaView **f1, FakturaView **f2)
{
    if((*f1)->splatnost().IsSameDate((*f2)->splatnost()))
        return dxSortFakturyViewByIdDsc(f1,f2);
    if((*f1)->splatnost().IsEarlierThan((*f2)->splatnost()))
        return 1;
    return -1;
}

static int dxSortFakturyViewByCelkemAsc(FakturaView **f1, FakturaView **f2)
{
    if((*f1)->celkem() == (*f2)->celkem())
        return dxSortFakturyViewByIdAsc(f1,f2);
    if((*f1)->celkem() < (*f2)->celkem())
        return -1;
    return 1;
}

static int dxSortFakturyViewByCelkemDsc(FakturaView **f1, FakturaView **f2)
{
    if((*f1)->celkem() == (*f2)->celkem())
        return dxSortFakturyViewByIdDsc(f1,f2);
    if((*f1)->celkem() < (*f2)->celkem())
        return 1;
    return -1;
}

static int dxSortFakturyViewByUhraditAsc(FakturaView **f1, FakturaView **f2)
{
    if((*f1)->celkem()-(*f1)->uhrazeno() == (*f2)->celkem()-(*f2)->uhrazeno())
        return dxSortFakturyViewByIdAsc(f1,f2);
    if((*f1)->celkem()-(*f1)->uhrazeno() < (*f2)->celkem()-(*f2)->uhrazeno())
        return -1;
    return 1;
}

static int dxSortFakturyViewByUhraditDsc(FakturaView **f1, FakturaView **f2)
{
    if((*f1)->celkem()-(*f1)->uhrazeno() == (*f2)->celkem()-(*f2)->uhrazeno())
        return dxSortFakturyViewByIdDsc(f1,f2);
    if((*f1)->celkem()-(*f1)->uhrazeno() < (*f2)->celkem()-(*f2)->uhrazeno())
        return 1;
    return -1;
}

static int dxSortZavPohViewByIdAsc(ZavPohView **f1, ZavPohView **f2)
{
    if((*f1)->id() == (*f2)->id())
        return 0;
    if((*f1)->id() < (*f2)->id())
        return -1;
    return 1;
}

static int dxSortZavPohViewByIdDsc(ZavPohView **f1, ZavPohView **f2)
{
    if((*f1)->id() == (*f2)->id())
        return 0;
    if((*f1)->id() < (*f2)->id())
        return 1;
    return -1;
}

static int dxSortZavPohViewByDokladAsc(ZavPohView **f1, ZavPohView **f2)
{
    if((*f1)->cislo() == (*f2)->cislo())
        return dxSortZavPohViewByIdAsc(f1,f2);
    return (*f1)->cislo().CmpNoCase((*f2)->cislo());
}

static int dxSortZavPohViewByDokladDsc(ZavPohView **f1, ZavPohView **f2)
{
    if((*f1)->cislo() == (*f2)->cislo())
        return dxSortZavPohViewByIdDsc(f1,f2);
    return (*f2)->cislo().CmpNoCase((*f1)->cislo());
}

static int dxSortZavPohViewByPopisAsc(ZavPohView **f1, ZavPohView **f2)
{
    if((*f1)->popis() == (*f2)->popis())
        return dxSortZavPohViewByIdAsc(f1,f2);
    return (*f1)->popis().CmpNoCase((*f2)->popis());
}

static int dxSortZavPohViewByPopisDsc(ZavPohView **f1, ZavPohView **f2)
{
    if((*f1)->popis() == (*f2)->popis())
        return dxSortZavPohViewByIdDsc(f1,f2);
    return (*f2)->popis().CmpNoCase((*f1)->popis());
}

static int dxSortZavPohViewByProtistanaAsc(ZavPohView **f1, ZavPohView **f2)
{
    if((*f1)->protistrana() == (*f2)->protistrana())
        return dxSortZavPohViewByIdAsc(f1,f2);
    return (*f1)->protistrana().CmpNoCase((*f2)->protistrana());
}

static int dxSortZavPohViewByProtistranaDsc(ZavPohView **f1, ZavPohView **f2)
{
    if((*f1)->protistrana() == (*f2)->protistrana())
        return dxSortZavPohViewByIdDsc(f1,f2);
    return (*f2)->protistrana().CmpNoCase((*f1)->protistrana());
}

static int dxSortZavPohViewByVystavenaAsc(ZavPohView **f1, ZavPohView **f2)
{
    if((*f1)->vystaveni().IsSameDate((*f2)->vystaveni()))
        return dxSortZavPohViewByIdAsc(f1,f2);
    if((*f1)->vystaveni().IsEarlierThan((*f2)->vystaveni()))
        return -1;
    return 1;
}

static int dxSortZavPohViewByVystavenaDsc(ZavPohView **f1, ZavPohView **f2)
{
    if((*f1)->vystaveni().IsSameDate((*f2)->vystaveni()))
        return dxSortZavPohViewByIdDsc(f1,f2);
    if((*f1)->vystaveni().IsEarlierThan((*f2)->vystaveni()))
        return 1;
    return -1;
}

static int dxSortZavPohViewBySplatnaAsc(ZavPohView **f1, ZavPohView **f2)
{
    if((*f1)->splatnost().IsSameDate((*f2)->splatnost()))
        return dxSortZavPohViewByIdAsc(f1,f2);
    if((*f1)->splatnost().IsEarlierThan((*f2)->splatnost()))
        return -1;
    return 1;
}

static int dxSortZavPohViewBySplatnaDsc(ZavPohView **f1, ZavPohView **f2)
{
    if((*f1)->splatnost().IsSameDate((*f2)->splatnost()))
        return dxSortZavPohViewByIdDsc(f1,f2);
    if((*f1)->splatnost().IsEarlierThan((*f2)->splatnost()))
        return 1;
    return -1;
}

static int dxSortZavPohViewByCelkemAsc(ZavPohView **f1, ZavPohView **f2)
{
    if((*f1)->celkem() == (*f2)->celkem())
        return dxSortZavPohViewByIdAsc(f1,f2);
    if((*f1)->celkem() < (*f2)->celkem())
        return -1;
    return 1;
}

static int dxSortZavPohViewByCelkemDsc(ZavPohView **f1, ZavPohView **f2)
{
    if((*f1)->celkem() == (*f2)->celkem())
        return dxSortZavPohViewByIdDsc(f1,f2);
    if((*f1)->celkem() < (*f2)->celkem())
        return 1;
    return -1;
}

static int dxSortZavPohViewByUhraditAsc(ZavPohView **f1, ZavPohView **f2)
{
    if((*f1)->celkem()-(*f1)->uhrazeno() == (*f2)->celkem()-(*f2)->uhrazeno())
        return dxSortZavPohViewByIdAsc(f1,f2);
    if((*f1)->celkem()-(*f1)->uhrazeno() < (*f2)->celkem()-(*f2)->uhrazeno())
        return -1;
    return 1;
}

static int dxSortZavPohViewByUhraditDsc(ZavPohView **f1, ZavPohView **f2)
{
    if((*f1)->celkem()-(*f1)->uhrazeno() == (*f2)->celkem()-(*f2)->uhrazeno())
        return dxSortZavPohViewByIdDsc(f1,f2);
    if((*f1)->celkem()-(*f1)->uhrazeno() < (*f2)->celkem()-(*f2)->uhrazeno())
        return 1;
    return -1;
}

static int dxSortMajetekViewByIdAsc(MajetekView **m1, MajetekView **m2)
{
    if((*m1)->id() == (*m2)->id())
        return 0;
    if((*m1)->id() < (*m2)->id())
        return -1;
    return 1;
}

static int dxSortMajetekViewByIdDsc(MajetekView **m1, MajetekView **m2)
{
    if((*m1)->id() == (*m2)->id())
        return 0;
    if((*m1)->id() < (*m2)->id())
        return 1;
    return -1;
}

static int dxSortMajetekViewByCisloAsc(MajetekView **m1, MajetekView **m2)
{
    if((*m1)->cislo() == (*m2)->cislo())
        return dxSortMajetekViewByIdAsc(m1,m2);
    return (*m1)->cislo().CmpNoCase((*m2)->cislo());
}

static int dxSortMajetekViewByCisloDsc(MajetekView **m1, MajetekView **m2)
{
    if((*m1)->cislo() == (*m2)->cislo())
        return dxSortMajetekViewByIdDsc(m1,m2);
    return (*m2)->cislo().CmpNoCase((*m1)->cislo());
}

static int dxSortMajetekViewByNazevAsc(MajetekView **m1, MajetekView **m2)
{
    if((*m1)->nazev() == (*m2)->nazev())
        return dxSortMajetekViewByIdAsc(m1,m2);
    return (*m1)->nazev().CmpNoCase((*m2)->nazev());
}

static int dxSortMajetekViewByNazevDsc(MajetekView **m1, MajetekView **m2)
{
    if((*m1)->nazev() == (*m2)->nazev())
        return dxSortMajetekViewByIdDsc(m1,m2);
    return (*m2)->nazev().CmpNoCase((*m1)->nazev());
}

static int dxSortMajetekViewByTypAsc(MajetekView **m1, MajetekView **m2)
{
    if((*m1)->typ() == (*m2)->typ())
        return dxSortMajetekViewByIdAsc(m1,m2);
    return (*m1)->typ().CmpNoCase((*m2)->typ());
}

static int dxSortMajetekViewByTypDsc(MajetekView **m1, MajetekView **m2)
{
    if((*m1)->typ() == (*m2)->typ())
        return dxSortMajetekViewByIdDsc(m1,m2);
    return (*m2)->typ().CmpNoCase((*m1)->typ());
}

static int dxSortMajetekViewByOdpisAsc(MajetekView **m1, MajetekView **m2)
{
    if((*m1)->odpis() == (*m2)->odpis())
        return dxSortMajetekViewByIdAsc(m1,m2);
    return (*m1)->odpis().CmpNoCase((*m2)->odpis());
}

static int dxSortMajetekViewByOdpisDsc(MajetekView **m1, MajetekView **m2)
{
    if((*m1)->odpis() == (*m2)->odpis())
        return dxSortMajetekViewByIdDsc(m1,m2);
    return (*m2)->odpis().CmpNoCase((*m1)->odpis());
}

static int dxSortMajetekViewByPorizenAsc(MajetekView **m1, MajetekView **m2)
{
    if((*m1)->porizen().IsSameDate((*m2)->porizen()))
        return dxSortMajetekViewByIdAsc(m1,m2);
    if((*m1)->porizen().IsEarlierThan((*m2)->porizen()))
        return -1;
    return 1;
}

static int dxSortMajetekViewByPorizenDsc(MajetekView **m1, MajetekView **m2)
{
    if((*m1)->porizen().IsSameDate((*m2)->porizen()))
        return dxSortMajetekViewByIdDsc(m1,m2);
    if((*m1)->porizen().IsEarlierThan((*m2)->porizen()))
        return 1;
    return -1;
}

static int dxSortMajetekViewByPorcenaAsc(MajetekView **m1, MajetekView **m2)
{
    if((*m1)->cena() == (*m2)->cena())
        return dxSortMajetekViewByIdAsc(m1,m2);
    if((*m1)->cena() < (*m2)->cena())
        return -1;
    return 1;
}

static int dxSortMajetekViewByPorcenaDsc(MajetekView **m1, MajetekView **m2)
{
    if((*m1)->cena() == (*m2)->cena())
        return dxSortMajetekViewByIdDsc(m1,m2);
    if((*m1)->cena() < (*m2)->cena())
        return 1;
    return -1;
}

static int dxSortMajetekViewByZuscenaAsc(MajetekView **m1, MajetekView **m2)
{
    if((*m1)->zustatkovaCena() == (*m2)->zustatkovaCena())
        return dxSortMajetekViewByIdAsc(m1,m2);
    if((*m1)->zustatkovaCena() < (*m2)->zustatkovaCena())
        return -1;
    return 1;
}

static int dxSortMajetekViewByZuscenaDsc(MajetekView **m1, MajetekView **m2)
{
    if((*m1)->zustatkovaCena() == (*m2)->zustatkovaCena())
        return dxSortMajetekViewByIdDsc(m1,m2);
    if((*m1)->zustatkovaCena() < (*m2)->zustatkovaCena())
        return 1;
    return -1;
}

static int dxSortOdpisByRokAsc(Odpis ***o1, Odpis ***o2)
{
    if((**o1)->rok() == (**o2)->rok())
        return 0;
    if((**o1)->rok() < (**o2)->rok())
        return -1;
    return 1;
}

//static int dxSortOdpisByRokDsc(Odpis ***o1, Odpis ***o2)
//{
//    if((**o1)->rok() == (**o2)->rok())
//        return 0;
//    if((**o1)->rok() < (**o2)->rok())
//        return 1;
//    return -1;
//}

static int dxSortOdeslanyByIdAsc(Odeslany **od1, Odeslany **od2)
{
    if((*od1)->id() == (*od2)->id())
        return 0;
    if((*od1)->id() < (*od2)->id())
        return -1;
    return 1;
}

static int dxSortOdeslanyByIdDsc(Odeslany **od1, Odeslany **od2)
{
    if((*od1)->id() == (*od2)->id())
        return 0;
    if((*od1)->id() < (*od2)->id())
        return 1;
    return -1;
}

static int dxSortOdeslanyByCasAsc(Odeslany **od1, Odeslany **od2)
{
    if((*od1)->cas().IsSameTime((*od2)->cas()))
        return dxSortOdeslanyByIdAsc(od1,od2);
    if((*od1)->cas().IsEarlierThan((*od2)->cas()))
        return -1;
    return 1;
}

static int dxSortOdeslanyByCasDsc(Odeslany **od1, Odeslany **od2)
{
    if((*od1)->cas().IsSameTime((*od2)->cas()))
        return dxSortOdeslanyByIdDsc(od1,od2);
    if((*od1)->cas().IsEarlierThan((*od2)->cas()))
        return 1;
    return -1;
}

static int dxSortOdeslanyByFakturaAsc(Odeslany **od1, Odeslany **od2)
{
    if((*od1)->faktura() == (*od2)->faktura())
        return dxSortOdeslanyByIdAsc(od1,od2);
    return (*od1)->faktura().CmpNoCase((*od2)->faktura());
}

static int dxSortOdeslanyByFakturaDsc(Odeslany **od1, Odeslany **od2)
{
    if((*od2)->faktura() == (*od1)->faktura())
        return dxSortOdeslanyByIdDsc(od1,od2);
    return (*od2)->faktura().CmpNoCase((*od1)->faktura());
}

static int dxSortOdeslanyByPrijemceAsc(Odeslany **od1, Odeslany **od2)
{
    if((*od1)->prijemce() == (*od2)->prijemce())
        return dxSortOdeslanyByIdAsc(od1,od2);
    return (*od1)->prijemce().CmpNoCase((*od2)->prijemce());
}

static int dxSortOdeslanyByPrijemceDsc(Odeslany **od1, Odeslany **od2)
{
    if((*od2)->prijemce() == (*od1)->prijemce())
        return dxSortOdeslanyByIdDsc(od1,od2);
    return (*od2)->prijemce().CmpNoCase((*od1)->prijemce());
}

static int dxSortUlozeneDph(Ulozene **u1, Ulozene **u2)
{
    if((*u1)->from().IsSameTime((*u2)->from()))
    {
        if((*u1)->id() == (*u2)->id())
            return 0;
        if((*u1)->id() < (*u2)->id())
            return -1;
        return 1;
    }
    if((*u1)->from().IsEarlierThan((*u2)->from()))
        return 1;
    return -1;
}

static int dxSortDphDoklad(DphDoklad **f1, DphDoklad **f2)
{
    wxDateTime d1 = (**f1).duzp() == (**f1).dppd() ? (**f1).duzp() : (**f1).dppd();
    wxDateTime d2 = (**f2).duzp() == (**f2).dppd() ? (**f2).duzp() : (**f2).dppd();
    if(d1.IsSameDate(d2))
        return 0;
    if(d1.IsEarlierThan(d2))
        return -1;
    return 1;
}

static int dxSortZasobyViewByIdAsc(ZasobaView **z1, ZasobaView **z2)
{
    if((*z1)->id() == (*z2)->id())
        return 0;
    if((*z1)->id() < (*z2)->id())
        return -1;
    return 1;
}

static int dxSortZasobyViewByIdDsc(ZasobaView **z1, ZasobaView **z2)
{
    if((*z1)->id() == (*z2)->id())
        return 0;
    if((*z1)->id() < (*z2)->id())
        return 1;
    return -1;
}

static int dxSortZasobyViewByCisloAsc(ZasobaView **z1, ZasobaView **z2)
{
    if((*z1)->cislo() == (*z2)->cislo())
        return dxSortZasobyViewByIdAsc(z1,z2);
    return (*z1)->cislo().CmpNoCase((*z2)->cislo());
}

static int dxSortZasobyViewByCisloDsc(ZasobaView **z1, ZasobaView **z2)
{
    if((*z2)->cislo() == (*z1)->cislo())
        return dxSortZasobyViewByIdDsc(z1,z2);
    return (*z2)->cislo().CmpNoCase((*z1)->cislo());
}

static int dxSortZasobyViewByNazevAsc(ZasobaView **z1, ZasobaView **z2)
{
    if((*z1)->nazev() == (*z2)->nazev())
        return dxSortZasobyViewByIdAsc(z1,z2);
    return (*z1)->nazev().CmpNoCase((*z2)->nazev());
}

static int dxSortZasobyViewByNazevDsc(ZasobaView **z1, ZasobaView **z2)
{
    if((*z2)->nazev() == (*z1)->nazev())
        return dxSortZasobyViewByIdDsc(z1,z2);
    return (*z2)->nazev().CmpNoCase((*z1)->nazev());
}

static int dxSortZasobyViewByPocetAsc(ZasobaView **z1, ZasobaView **z2)
{
    if((*z1)->pocet() == (*z2)->pocet())
        return dxSortZasobyViewByIdAsc(z1,z2);
    if((*z1)->pocet() < (*z2)->pocet())
        return -1;
    return 1;
}

static int dxSortZasobyViewByPocetDsc(ZasobaView **z1, ZasobaView **z2)
{
    if((*z1)->pocet() == (*z2)->pocet())
        return dxSortZasobyViewByIdDsc(z1,z2);
    if((*z1)->pocet() < (*z2)->pocet())
        return 1;
    return -1;
}

static int dxSortZasobyViewByCelkemAsc(ZasobaView **z1, ZasobaView **z2)
{
    if((*z1)->celkem() == (*z2)->celkem())
        return dxSortZasobyViewByIdAsc(z1,z2);
    if((*z1)->celkem() < (*z2)->celkem())
        return -1;
    return 1;
}

static int dxSortZasobyViewByCelkemDsc(ZasobaView **z1, ZasobaView **z2)
{
    if((*z1)->celkem() == (*z2)->celkem())
        return dxSortZasobyViewByIdDsc(z1,z2);
    if((*z1)->celkem() < (*z2)->celkem())
        return 1;
    return -1;
}

static int dxSortZasobyViewBySkladAsc(ZasobaView **z1, ZasobaView **z2)
{
    if((*z1)->sklad() == (*z2)->sklad())
        return dxSortZasobyViewByIdAsc(z1,z2);
    return (*z1)->sklad().CmpNoCase((*z2)->sklad());
}

static int dxSortZasobyViewBySkladDsc(ZasobaView **z1, ZasobaView **z2)
{
    if((*z1)->sklad() == (*z2)->sklad())
        return dxSortZasobyViewByIdDsc(z1,z2);
    return (*z2)->sklad().CmpNoCase((*z1)->sklad());
}

static int dxSortPohybzasobByIdAsc(Pohybzasob **p1, Pohybzasob **p2)
{
    if((*p1)->id() == (*p2)->id())
        return 0;
    if((*p1)->id() < (*p2)->id())
        return -1;
    return 1;
}

static int dxSortPohybzasobByDatumAsc(Pohybzasob **p1, Pohybzasob **p2)
{
    if((*p1)->datum().IsSameTime((*p2)->datum()))
    {
        return dxSortPohybzasobByIdAsc(p1, p2);
    }
    if((*p1)->datum().IsEarlierThan((*p2)->datum()))
        return -1;
    return 1;
}

static int dxSortDenikViewByIdAsc(DenikView **d1, DenikView **d2)
{
    if((*d1)->id() == (*d2)->id())
        return 0;
    if((*d1)->id() < (*d2)->id())
        return -1;
    return 1;
}

static int dxSortDenikViewByIdDsc(DenikView **d1, DenikView **d2)
{
    if((*d1)->id() == (*d2)->id())
        return 0;
    if((*d1)->id() < (*d2)->id())
        return 1;
    return -1;
}

static int dxSortDenikViewByDatumAsc(DenikView **d1, DenikView **d2)
{
    if((*d1)->datum().IsSameDate((*d2)->datum()))
        return dxSortDenikViewByIdAsc(d1,d2);
    if((*d1)->datum().IsEarlierThan((*d2)->datum()))
        return -1;
    return 1;
}

static int dxSortDenikViewByDatumDsc(DenikView **d1, DenikView **d2)
{
    if((*d1)->datum().IsSameDate((*d2)->datum()))
        return dxSortDenikViewByIdDsc(d1,d2);
    if((*d1)->datum().IsEarlierThan((*d2)->datum()))
        return 1;
    return -1;
}

static int dxSortDenikViewByCastkaAsc(DenikView **d1, DenikView **d2)
{
    if((*d1)->castka() == (*d2)->castka())
        return dxSortDenikViewByIdAsc(d1,d2);
    if((*d1)->castka() < (*d2)->castka())
        return -1;
    return 1;
}

static int dxSortDenikViewByCastkaDsc(DenikView **d1, DenikView **d2)
{
    if((*d1)->castka() == (*d2)->castka())
        return dxSortDenikViewByIdDsc(d1,d2);
    if((*d1)->castka() < (*d2)->castka())
        return 1;
    return -1;
}

static int dxSortDenikViewByCisloAsc(DenikView **d1, DenikView **d2)
{
    if((*d1)->cislo() == (*d2)->cislo())
        return dxSortDenikViewByIdAsc(d1,d2);
    return (*d1)->cislo().CmpNoCase((*d2)->cislo());
}

static int dxSortDenikViewByCisloDsc(DenikView **d1, DenikView **d2)
{
    if((*d2)->cislo() == (*d1)->cislo())
        return dxSortDenikViewByIdDsc(d1,d2);
    return (*d2)->cislo().CmpNoCase((*d1)->cislo());
}

static int dxSortDenikViewByPopisAsc(DenikView **d1, DenikView **d2)
{
    if((*d1)->popis() == (*d2)->popis())
        return dxSortDenikViewByIdAsc(d1,d2);
    return (*d1)->popis().CmpNoCase((*d2)->popis());
}

static int dxSortDenikViewByPopisDsc(DenikView **d1, DenikView **d2)
{
    if((*d1)->popis() == (*d2)->popis())
        return dxSortDenikViewByIdDsc(d1,d2);
    return (*d2)->popis().CmpNoCase((*d1)->popis());
}

static int dxSortDenikViewByPrijemAsc(DenikView **d1, DenikView **d2)
{
    if((*d1)->prijem() == (*d2)->prijem())
        return dxSortDenikViewByIdAsc(d1,d2);
    if((*d1)->prijem() < (*d2)->prijem())
        return -1;
    return 1;
}

static int dxSortDenikViewByPrijemDsc(DenikView **d1, DenikView **d2)
{
    if((*d1)->prijem() == (*d2)->prijem())
        return dxSortDenikViewByIdDsc(d1,d2);
    if((*d1)->prijem() < (*d2)->prijem())
        return 1;
    return -1;
}

static int dxSortDenikViewByPokladnaBankaAsc(DenikView **d1, DenikView **d2)
{
    if((*d1)->pokladnabanka() == (*d2)->pokladnabanka())
        return dxSortDenikViewByIdAsc(d1,d2);
    return (*d1)->pokladnabanka().CmpNoCase((*d2)->pokladnabanka());
    return 0;
}

static int dxSortDenikViewByPokladnaBankaDsc(DenikView **d1, DenikView **d2)
{
    if((*d1)->pokladnabanka() == (*d2)->pokladnabanka())
        return dxSortDenikViewByIdDsc(d1,d2);
    return (*d2)->pokladnabanka().CmpNoCase((*d1)->pokladnabanka());
    return 0;
}

static int dxSortDenikViewByVypisAsc(DenikView **d1, DenikView **d2)
{
    if((*d1)->vypis() == (*d2)->vypis())
        return dxSortDenikViewByIdAsc(d1,d2);
    if((*d1)->vypis() < (*d2)->vypis())
        return -1;
    return 1;
}

static int dxSortDenikViewByVypisDsc(DenikView **d1, DenikView **d2)
{
    if((*d1)->vypis() == (*d2)->vypis())
        return dxSortDenikViewByIdDsc(d1,d2);
    if((*d1)->vypis() < (*d2)->vypis())
        return 1;
    return -1;
}

static int dxSortJizdaViewByIdAsc(JizdaView **j1, JizdaView **j2)
{
    if((*j1)->id() == (*j2)->id())
        return 0;
    if((*j1)->id() < (*j2)->id())
        return -1;
    return 1;
}

static int dxSortJizdaViewByIdDsc(JizdaView **j1, JizdaView **j2)
{
    if((*j1)->id() == (*j2)->id())
        return 0;
    if((*j1)->id() < (*j2)->id())
        return 1;
    return -1;
}

static int dxSortJizdaViewByDatumAsc(JizdaView **j1, JizdaView **j2)
{
    if((*j1)->endcas().IsSameDate((*j2)->endcas()))
        return dxSortJizdaViewByIdAsc(j1,j2);
    if((*j1)->endcas().IsEarlierThan((*j2)->endcas()))
        return -1;
    return 1;
}

static int dxSortJizdaViewByDatumDsc(JizdaView **j1, JizdaView **j2)
{
    if((*j1)->endcas().IsSameDate((*j2)->endcas()))
        return dxSortJizdaViewByIdDsc(j1,j2);
    if((*j1)->endcas().IsEarlierThan((*j2)->endcas()))
        return 1;
    return -1;
}

static int dxSortJizdaViewByCestaAsc(JizdaView **j1, JizdaView **j2)
{
    if((*j1)->cesta() == (*j2)->cesta())
        return dxSortJizdaViewByIdAsc(j1,j2);
    return (*j1)->cesta().CmpNoCase((*j2)->cesta());
    return 0;
}

static int dxSortJizdaViewByCestaDsc(JizdaView **j1, JizdaView **j2)
{
    if((*j1)->cesta() == (*j2)->cesta())
        return dxSortJizdaViewByIdDsc(j1,j2);
    return (*j2)->cesta().CmpNoCase((*j1)->cesta());
    return 0;
}

static int dxSortJizdaViewByUcelAsc(JizdaView **j1, JizdaView **j2)
{
    if((*j1)->ucel() == (*j2)->ucel())
        return dxSortJizdaViewByIdAsc(j1,j2);
    return (*j1)->ucel().CmpNoCase((*j2)->ucel());
    return 0;
}

static int dxSortJizdaViewByUcelDsc(JizdaView **j1, JizdaView **j2)
{
    if((*j1)->ucel() == (*j2)->ucel())
        return dxSortJizdaViewByIdDsc(j1,j2);
    return (*j2)->ucel().CmpNoCase((*j1)->ucel());
    return 0;
}

static int dxSortJizdaViewByVozidloAsc(JizdaView **j1, JizdaView **j2)
{
    if((*j1)->vozidlo() == (*j2)->vozidlo())
        return dxSortJizdaViewByIdAsc(j1,j2);
    return (*j1)->vozidlo().CmpNoCase((*j2)->vozidlo());
    return 0;
}

static int dxSortJizdaViewByVozidloDsc(JizdaView **j1, JizdaView **j2)
{
    if((*j1)->vozidlo() == (*j2)->vozidlo())
        return dxSortJizdaViewByIdDsc(j1,j2);
    return (*j2)->vozidlo().CmpNoCase((*j1)->vozidlo());
    return 0;
}

static int dxSortJizdaViewByRidicAsc(JizdaView **j1, JizdaView **j2)
{
    if((*j1)->ridic() == (*j2)->ridic())
        return dxSortJizdaViewByIdAsc(j1,j2);
    return (*j1)->ridic().CmpNoCase((*j2)->ridic());
    return 0;
}

static int dxSortJizdaViewByRidicDsc(JizdaView **j1, JizdaView **j2)
{
    if((*j1)->ridic() == (*j2)->ridic())
        return dxSortJizdaViewByIdDsc(j1,j2);
    return (*j2)->ridic().CmpNoCase((*j1)->ridic());
    return 0;
}

static int dxSortJizdaViewByKmAsc(JizdaView **j1, JizdaView **j2)
{
    if((*j1)->km() == (*j2)->km())
        return dxSortJizdaViewByIdAsc(j1,j2);
    if((*j1)->km() < (*j2)->km())
        return -1;
    return 1;
}

static int dxSortJizdaViewByKmDsc(JizdaView **j1, JizdaView **j2)
{
    if((*j1)->km() == (*j2)->km())
        return dxSortJizdaViewByIdDsc(j1,j2);
    if((*j1)->km() < (*j2)->km())
        return 1;
    return -1;
}

static int dxSortDphDruhViewByIdAsc(DphDruh **d1, DphDruh **d2)
{
    if((*d1)->id() == (*d2)->id())
        return 0;
    if((*d1)->id() < (*d2)->id())
        return -1;
    return 1;
}

static int dxSortDphDruhViewByIdDsc(DphDruh **d1, DphDruh **d2)
{
    if((*d1)->id() == (*d2)->id())
        return 0;
    if((*d1)->id() < (*d2)->id())
        return 1;
    return -1;
}

static int dxSortDphDruhViewByZkratkaAsc(DphDruh **d1, DphDruh **d2)
{
    if((*d1)->zkratka() == (*d2)->zkratka())
        return dxSortDphDruhViewByIdAsc(d1,d2);
    return ((*d1)->zkratka().CmpNoCase((*d2)->zkratka()));
    return 0;
}

static int dxSortDphDruhViewByZkratkaDsc(DphDruh **d1, DphDruh **d2)
{
    if((*d1)->zkratka() == (*d2)->zkratka())
        return dxSortDphDruhViewByIdDsc(d1,d2);
    return ((*d2)->zkratka().CmpNoCase((*d1)->zkratka()));
    return 0;
}

static int dxSortDphDruhViewByPopisAsc(DphDruh **d1, DphDruh **d2)
{
    if((*d1)->popis() == (*d2)->popis())
        return dxSortDphDruhViewByIdAsc(d1,d2);
    return ((*d1)->popis().CmpNoCase((*d2)->popis()));
    return 0;
}

static int dxSortDphDruhViewByPopisDsc(DphDruh **d1, DphDruh **d2)
{
    if((*d1)->popis() == (*d2)->popis())
        return dxSortDphDruhViewByIdDsc(d1,d2);
    return ((*d2)->popis().CmpNoCase((*d1)->popis()));
    return 0;
}

static int dxSortDphDruhViewByPlneniAsc(DphDruh **d1, DphDruh **d2)
{
    if((*d1)->plneni() == (*d2)->plneni())
        return dxSortDphDruhViewByIdAsc(d1,d2);
    if((*d1)->plneni() < (*d2)->plneni())
        return -1;
    return 1;
}

static int dxSortDphDruhViewByPlneniDsc(DphDruh **d1, DphDruh **d2)
{
    if((*d1)->plneni() == (*d2)->plneni())
        return dxSortDphDruhViewByIdDsc(d1,d2);
    if((*d1)->plneni() < (*d2)->plneni())
        return 1;
    return -1;
}

static int dxSortDphDruhViewByRadkyAsc(DphDruh **d1, DphDruh **d2)
{
    if((*d1)->radky() == (*d2)->radky())
        return dxSortDphDruhViewByIdAsc(d1,d2);
    return ((*d1)->radky().CmpNoCase((*d2)->radky()));
    return 0;
}

static int dxSortDphDruhViewByRadkyDsc(DphDruh **d1, DphDruh **d2)
{
    if((*d1)->radky() == (*d2)->radky())
        return dxSortDphDruhViewByIdDsc(d1,d2);
    return ((*d2)->radky().CmpNoCase((*d1)->radky()));
    return 0;
}

static int dxSortDphDruhViewByKppAsc(DphDruh **d1, DphDruh **d2)
{
    if((*d1)->kpp() == (*d2)->kpp())
        return dxSortDphDruhViewByIdAsc(d1,d2);
    return ((*d1)->kpp().CmpNoCase((*d2)->kpp()));
    return 0;
}

static int dxSortDphDruhViewByKppDsc(DphDruh **d1, DphDruh **d2)
{
    if((*d1)->kpp() == (*d2)->kpp())
        return dxSortDphDruhViewByIdDsc(d1,d2);
    return ((*d2)->kpp().CmpNoCase((*d1)->kpp()));
    return 0;
}

static int dxSortDphDruhViewByKhAsc(DphDruh **d1, DphDruh **d2)
{
    if((*d1)->kh() == (*d2)->kh())
        return dxSortDphDruhViewByIdAsc(d1,d2);
    return ((*d1)->kh().CmpNoCase((*d2)->kh()));
    return 0;
}

static int dxSortDphDruhViewByKhDsc(DphDruh **d1, DphDruh **d2)
{
    if((*d1)->kh() == (*d2)->kh())
        return dxSortDphDruhViewByIdDsc(d1,d2);
    return ((*d2)->kh().CmpNoCase((*d1)->kh()));
    return 0;
}

static int dxSortDphDruhViewByNabizetAsc(DphDruh **d1, DphDruh **d2)
{
    if((*d1)->nabizet() == (*d2)->nabizet())
        return dxSortDphDruhViewByIdAsc(d1,d2);
    if((*d1)->nabizet() < (*d2)->nabizet())
        return -1;
    return 1;
}

static int dxSortDphDruhViewByNabizetDsc(DphDruh **d1, DphDruh **d2)
{
    if((*d1)->nabizet() == (*d2)->nabizet())
        return dxSortDphDruhViewByIdDsc(d1,d2);
    if((*d1)->nabizet() < (*d2)->nabizet())
        return 1;
    return -1;
}

#endif  /* DBSORT_H */
