/***************************************************************
 * Name:      document.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#ifndef DXDOCUMENT_H
#define DXDOCUMENT_H

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif
#include "elements.h"
#include "dxprint.h"

class Document
{
public:
    Document();

    dxElementsArray elements() const;
    void setElements(const dxElementsArray &elements);
    void clearElements();
    void sortElements();
    dxElementsArray elementsByPage(int page) const;
    dxElementsArray elementsBySection(DOCSECTION section) const;
    size_t getElementsCount() const;
    void addElement(const LineElement &element);
    void addElement(const TextElement &element);
    void addElement(const TextParagraphElement &element);
    void addElement(const TableRowElement &element);
    void addElement(const ImageElement &element);
    void addElement(const RectangleElement &element);
    void deleteElement(size_t index);
    void setFromDoc(const Document &doc);
    int sectionHeight(DOCSECTION section) const;
    TextStyle style() const;
    void setStyle(const TextStyle &style);
    void setFont(const wxFont &font);
    void setFontColour(const wxColour &fontColour);
    void setBackColour(const wxColour &backColour);
    int pages() const;
    void setPages(int pages);
    void setElementSelection(size_t index, bool select);
    void clearElementSelection();
    int marginLeft() const;
    void setMarginLeft(int marginLeft);
    int marginRight() const;
    void setMarginRight(int marginRight);
    int marginTop() const;
    void setMarginTop(int marginTop);
    int marginBottom() const;
    void setMarginBottom(int marginBottom);
    dxPrint::Footer footer() const;
    void setFooter(const dxPrint::Footer &footer);
    wxString footerText() const;
    void setFooterText(const wxString &footerText);
    wxString headerLeftText() const;
    void setHeaderLeftText(const wxString &headerLeftText);
    wxString headerCenterText() const;
    void setHeaderCenterText(const wxString &headerCenterText);
    wxString headerRightText() const;
    void setHeaderRightText(const wxString &headerRightText);
    bool headerLine() const;
    void setHeaderLine(bool headerLine);
    bool saveToXml(const wxString &path);
    bool readFromXml(const wxString &path);
    Element *element(size_t index);
private:
    dxElementsArray m_elements;
    TextStyle m_style;
    int m_pages;
    int m_marginLeft;
    int m_marginRight;
    int m_marginTop;
    int m_marginBottom;
    dxPrint::Footer m_footer;
    wxString m_footerText;
    wxString m_headerLeftText;
    wxString m_headerCenterText;
    wxString m_headerRightText;
    bool m_headerLine;
};

#endif  /* DXDOCUMENT_H */

