/***************************************************************
 * Name:      dxsettings.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include <wx/stdpaths.h>
#include <wx/fileconf.h>
#include "dxsettings.h"
#include "dxutils.h"

dxSettings::dxSettings()
{
    wxString iniPath = wxStandardPaths::Get().GetUserDataDir();
    if(!wxDirExists(iniPath))
    {
        wxMkdir(iniPath);
    }
    iniPath << wxFILE_SEP_PATH << "dxde.ini";
    wxFileConfig *conf = new wxFileConfig("APPNAME", "COMPANYNAME", iniPath);
    wxConfigBase::Set(conf);

    m_x = 50;
    m_y = 50;
    m_w = 888;
    m_h = 666;
    m_maximized = false;
    m_auiperspective = "layout2|name=Navigation;caption=Navigation;state=17404;dir=4;layer=0;row=0;pos=0;prop=100000;bestw=200;besth=100;minw=100;minh=100;maxw=-1;maxh=-1;floatx=-1;floaty=-1;floatw=-1;floath=-1|name=Book;caption=Book;state=256;dir=5;layer=0;row=0;pos=0;prop=100000;bestw=20;besth=20;minw=-1;minh=-1;maxw=-1;maxh=-1;floatx=-1;floaty=-1;floatw=-1;floath=-1|dock_size(4,0,0)=202|dock_size(5,0,0)=20|";

    m_dbcount = 0;
    m_dbmodel = DB_SQLITE;
    m_dbname = wxEmptyString;

    m_splatnost = 14;

    m_datum = wxDateTime::Today();

    m_fakturaSablona = "iternal";
    m_zalohaSablona = "internal";

    m_lastBackup = wxDateTime::Today();
    m_backupPath = wxGetHomeDir();
}

dxSettings::~dxSettings()
{}

dxSettings &dxSettings::instance()
{
    static dxSettings s;
    return s;
}

void dxSettings::load()
{
    wxConfigBase::Get()->Read("GUI/x", &m_x, m_x);
    wxConfigBase::Get()->Read("GUI/y", &m_y, m_y);
    wxConfigBase::Get()->Read("GUI/w", &m_w, m_w);
    wxConfigBase::Get()->Read("GUI/h", &m_h, m_h);
    wxConfigBase::Get()->Read("GUI/maximized", &m_maximized, m_maximized);
    wxConfigBase::Get()->Read("GUI/auiperspective", &m_auiperspective, m_auiperspective);
    wxConfigBase::Get()->Read("DB/dbcount", &m_dbcount, m_dbcount);
    wxConfigBase::Get()->Read("DB/dbmodel", (int*)&m_dbmodel, m_dbmodel);
    if(m_dbmodel < DB_SQLITE || m_dbmodel >= DB_MAX)
        m_dbmodel = DB_SQLITE;
    wxConfigBase::Get()->Read("DB/dbname", &m_dbname, m_dbname);
    if(!m_dbcount)
    {
        m_dbname = "";
    }
    m_dbmap.clear();
    if(m_dbcount)
    {
        for(int i=0; i<m_dbcount; i++)
        {
            wxString db, firma;
            wxConfigBase::Get()->Read(wxString::Format("DB/dbname%d",i), &db, wxEmptyString);
            wxConfigBase::Get()->Read(wxString::Format("DB/dbfirma%d",i), &firma, wxEmptyString);
            if(!db.IsEmpty() && !firmaExists(firma))
                m_dbmap.insert(dxStringPair(db,firma));
        }
        if(m_dbname.IsEmpty() && m_dbmap.size())
        {
            m_dbname = (*m_dbmap.begin()).first;
        }
    }
    wxConfigBase::Get()->Read("Template/faktura", &m_fakturaSablona, m_fakturaSablona);
    if(m_fakturaSablona != "internal" && !wxFileExists(m_fakturaSablona)) m_fakturaSablona = "internal";
    wxConfigBase::Get()->Read("Template/zaloha", &m_zalohaSablona, m_zalohaSablona);
    if(m_zalohaSablona != "internal" && !wxFileExists(m_zalohaSablona)) m_zalohaSablona = "internal";
    wxString last = wxEmptyString;
    wxConfigBase::Get()->Read("Backup/lastBackup", &last, m_lastBackup.FormatISODate());
    m_lastBackup.ParseISODate(last);
    wxConfigBase::Get()->Read("Backup/backupPath", &m_backupPath, m_backupPath);
}

void dxSettings::save()
{
    wxConfigBase::Get()->Write("GUI/x", m_x);
    wxConfigBase::Get()->Write("GUI/y", m_y);
    wxConfigBase::Get()->Write("GUI/w", m_w);
    wxConfigBase::Get()->Write("GUI/h", m_h);
    wxConfigBase::Get()->Write("GUI/maximized", m_maximized);
    wxConfigBase::Get()->Write("GUI/auiperspective", m_auiperspective);
    wxConfigBase::Get()->Write("DB/dbcount", (int)m_dbmap.size());
    wxConfigBase::Get()->Write("DB/dbmodel", (int)m_dbmodel);
    wxConfigBase::Get()->Write("DB/dbname", m_dbname);
    if(m_dbmap.size())
    {
        dxStringIt it;
        int i;
        for(i=0, it=m_dbmap.begin(); it!=m_dbmap.end(); it++,i++)
        {
            wxConfigBase::Get()->Write(wxString::Format("DB/dbname%d",i), (*it).first);
            wxConfigBase::Get()->Write(wxString::Format("DB/dbfirma%d",i), (*it).second);
        }
    }
    wxConfigBase::Get()->Write("Template/faktura", m_fakturaSablona);
    wxConfigBase::Get()->Write("Template/zaloha", m_zalohaSablona);
    wxConfigBase::Get()->Write("Backup/lastBackup", m_lastBackup.FormatISODate());
    wxConfigBase::Get()->Write("Backup/backupPath", m_backupPath);
    wxConfigBase::Get()->Flush();
}

int dxSettings::x() const
{
    return m_x;
}

void dxSettings::setX(int x)
{
    m_x = x;
}

int dxSettings::y() const
{
    return m_y;
}

void dxSettings::setY(int y)
{
    m_y = y;
}

int dxSettings::w() const
{
    return m_w;
}

void dxSettings::setW(int w)
{
    m_w = w;
}

int dxSettings::h() const
{
    return m_h;
}

void dxSettings::setH(int h)
{
    m_h = h;
}

bool dxSettings::maximized() const
{
    return m_maximized;
}

void dxSettings::setMaximized(bool maximized)
{
    m_maximized = maximized;
}

DBMODEL dxSettings::dbmodel() const
{
    return m_dbmodel;
}

void dxSettings::setDbmodel(DBMODEL dbmodel)
{
    m_dbmodel = dbmodel;
}

wxString dxSettings::dbname() const
{
    return m_dbname;
}

void dxSettings::setDbname(const wxString &dbname)
{
    m_dbname = dbname;
}

void dxSettings::setDbname()
{
    if(m_dbmap.size())
    {
        m_dbname = m_dbmap.begin()->first;
    }
}

dxStringMap dxSettings::dbmap() const
{
    return m_dbmap;
}

void dxSettings::setDbmap(const dxStringMap &dbmap)
{
    m_dbmap = dbmap;
}

wxArrayString dxSettings::firmy()
{
    wxArrayString firmy;
    dxStringIt it = m_dbmap.begin();
    for(; it != m_dbmap.end(); ++it)
    {
        firmy.Add(wxString::Format(wxString::FromUTF8("%s, IČ: %s"), (*it).second.BeforeLast('_'), (*it).second.AfterLast('_')));
    }
    return firmy;
}

int dxSettings::dbcount() const
{
    return m_dbcount;
}

void dxSettings::setDbcount(int dbcount)
{
    m_dbcount = dbcount;
}

void dxSettings::addDbnamePair(const wxString &dbname, const wxString &firma)
{
    dxStringIt it = m_dbmap.find(dbname);
    if(it == m_dbmap.end())
    {
        m_dbmap.insert(dxStringPair(dbname,firma));
        m_dbcount++;
    }
}

void dxSettings::editDbnamePair(const wxString &dbname, const wxString &firma)
{
    dxStringIt it = m_dbmap.find(dbname);
    if(it != m_dbmap.end())
    {
        (*it).second = firma;
    }
}

bool dxSettings::firmaExists(const wxString &firma)
{
    dxStringIt it = m_dbmap.begin();
    for(; it != m_dbmap.end(); ++it)
    {
        if((*it).second == firma)
            return true;
    }
    return false;
}

wxString dxSettings::dbnameByFirma(const wxString &firma)
{
    dxStringIt it = m_dbmap.begin();
    for(; it != m_dbmap.end(); ++it)
    {
        if((*it).second == firma)
            return (*it).first;
    }
    return "";
}

bool dxSettings::deleteFirma(const wxString &firma)
{
    dxStringIt it = m_dbmap.begin();
    for(; it != m_dbmap.end(); ++it)
    {
        if((*it).second == firma)
        {
            m_dbmap.erase(it);
            m_dbcount--;
            if(!m_dbcount)
            {
                m_dbname = "";
            }
            return true;
        }
    }
    return false;
}

wxString dxSettings::auiperspective() const
{
    return m_auiperspective;
}

void dxSettings::setAuiperspective(const wxString &auiperspective)
{
    m_auiperspective = auiperspective;
}

int dxSettings::splatnost() const
{
    return m_splatnost;
}

void dxSettings::setSplatnost(int splatnost)
{
    m_splatnost = splatnost;
}

wxDateTime dxSettings::datum() const
{
    return m_datum;
}

void dxSettings::setDatum(const wxDateTime &datum)
{
    m_datum = datum;
}

wxString dxSettings::fakturaSablona() const
{
    return m_fakturaSablona;
}

void dxSettings::setFakturaSablona(const wxString &fakturaSablona)
{
    m_fakturaSablona = fakturaSablona;
}

wxString dxSettings::zalohaSablona() const
{
    return m_zalohaSablona;
}

void dxSettings::setZalohaSablona(const wxString &zalohaSablona)
{
    m_zalohaSablona = zalohaSablona;
}

wxDateTime dxSettings::lastBackup() const
{
    return m_lastBackup;
}

void dxSettings::setLastBackup(const wxDateTime &lastBackup)
{
    m_lastBackup = lastBackup;
}

wxString dxSettings::backupPath() const
{
    return m_backupPath;
}

void dxSettings::setBackupPath(const wxString &backupPath)
{
    m_backupPath = backupPath;
}
