/***************************************************************
 * Name:      dxutils.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include <wx/file.h>
#include <wx/stdpaths.h>
#include <wx/filename.h>
#include <wx/numformatter.h>
#include <wx/url.h>
#include <wx/wfstream.h>
#include <wx/tokenzr.h>
#include <memory>
#include <cmath>
#include <fstream>
#include "dxutils.h"
#include "dxsettings.h"
#include "model/dbmodel.h"
#include <curl/curl.h>

static size_t curlCallback(void *contents, size_t size, size_t nmemb, void *userp)
{
    (static_cast<std::string*>(userp))->append(static_cast<char*>(contents), size * nmemb);
    return size * nmemb;
}

static size_t curlCallbackFile(void *contents, size_t size, size_t nmemb, void *userp)
{
    size_t realsize = size * nmemb;
    std::ofstream* file = reinterpret_cast<std::ofstream*>(userp);
    file->write(reinterpret_cast<const char*>(contents), realsize);
    return realsize;
}

dxutils::dxutils()
{
}

dxutils &dxutils::instance()
{
    static dxutils idxutils;
    return idxutils;
}

bool dxutils::checkIC(const wxString &ic)
{
    if(ic.length() != 8) return false;
    int soucet = 0;
    for(int i=0; i<7; i++)
    {
        soucet += (static_cast<int>(ic[i].GetValue())-48)*(8-i);
    }
    soucet = soucet % 11;
    wxUniChar::value_type c;
    if(soucet == 0) c = 1;
    else if(soucet == 10) c = 1;
    else if(soucet == 1) c = 0;
    else c = 11 - soucet;
    return ic[7].GetValue()-48 == c;
}

wxString dxutils::getUniqueDbName()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxString path = wxStandardPaths::Get().GetUserDataDir();
        if(!wxDirExists(path))
        {
            wxMkdir(path);
        }
        wxString dbname = "dxde1.db";
        int i = 2;
        while(wxFile::Exists(path+wxFILE_SEP_PATH+dbname))
        {
            dbname = wxString::Format("dxde%d.db", i);
            i++;
        }
        return dbname;
    }
    return wxEmptyString;
}

wxString dxutils::getUniqueXmlPath(const wxString &base)
{
    wxString path = wxStandardPaths::Get().GetUserDataDir()+wxFILE_SEP_PATH+"xml";
    if(!wxDirExists(path))
    {
        wxMkdir(path);
    }
    wxString xmlname = base+"-1.xml";
    int i = 2;
    while(wxFile::Exists(path+wxFILE_SEP_PATH+xmlname))
    {
        xmlname = base+wxString::Format("-%d.xml", i);
        i++;
    }
    return path+wxFILE_SEP_PATH+xmlname;
}

wxString dxutils::enquotes(const wxString &text, const wxString &delim)
{
    wxString result = text;
    if(result.Contains(delim) || result.Contains("\""))
    {
        result.Replace("\"", "\"\"", true);
        result = wxString() << "\"" << result << "\"";
    }
    result.Replace("\t","    ", true);
    result.Replace("\n"," ", true);
    return result;
}

void dxutils::correctEmptyFileExt(const wxString &ext, wxString &fileName)
{
    wxFileName tempFileName(fileName);
    if(tempFileName.GetExt().IsEmpty())
        fileName << "." << ext;
}

wxString dxutils::slovy(wxString cislo)
{
    if(cislo.IsEmpty())
        return cislo;
    wxString desetinne = cislo.AfterLast(wxNumberFormatter::GetDecimalSeparator());
    cislo = cislo.BeforeLast(wxNumberFormatter::GetDecimalSeparator());
    if(cislo.Len()>6) //vic jak statisice se platit hotove nesmi
        return cislo;
    if(cislo.Len()==1 && cislo[0].GetValue()=='0')
        return "nula";
    size_t len = cislo.Len();
    wxString jednotky = "";
    wxString desitky = "";
    wxString stovky = "";
    wxString tisice = "";
    wxString desetitisice = "";
    wxString statisice = "";
    switch(cislo[cislo.Len()-1].GetValue()) {
    case '1': jednotky = "jedna"; break;
    case '2': jednotky = wxString::FromUTF8("dvě"); break;
    case '3': jednotky = wxString::FromUTF8("tři"); break;
    case '4': jednotky = wxString::FromUTF8("čtyři"); break;
    case '5': jednotky = wxString::FromUTF8("pět"); break;
    case '6': jednotky = wxString::FromUTF8("šest"); break;
    case '7': jednotky = wxString::FromUTF8("sedm"); break;
    case '8': jednotky = wxString::FromUTF8("osm"); break;
    case '9': jednotky = wxString::FromUTF8("devět"); break;
    default: break;
    }
    if(len)
    {
        len--;
        switch(cislo[cislo.Len()-2].GetValue()) {
        case '1':
        {
            switch(cislo[cislo.Len()-1].GetValue()) {
            case '0': desitky = "deset"; jednotky = ""; break;
            case '1': desitky = wxString::FromUTF8("jedenáct"); jednotky = ""; break;
            case '2': desitky = wxString::FromUTF8("dvanáct"); jednotky = ""; break;
            case '3': desitky = wxString::FromUTF8("třináct"); jednotky = ""; break;
            case '4': desitky = wxString::FromUTF8("čtrnáct"); jednotky = ""; break;
            case '5': desitky = wxString::FromUTF8("patnáct"); jednotky = ""; break;
            case '6': desitky = wxString::FromUTF8("šestnáct"); jednotky = ""; break;
            case '7': desitky = wxString::FromUTF8("sedmnáct"); jednotky = ""; break;
            case '8': desitky = wxString::FromUTF8("osmnáct"); jednotky = ""; break;
            case '9': desitky = wxString::FromUTF8("devatenáct"); jednotky = ""; break;
            default: break;
            }
            break;
        }
        case '2': desitky = "dvacet"; break;
        case '3': desitky = wxString::FromUTF8("třicet"); break;
        case '4': desitky = wxString::FromUTF8("čtyřicet"); break;
        case '5': desitky = wxString::FromUTF8("padesát"); break;
        case '6': desitky = wxString::FromUTF8("šedesát"); break;
        case '7': desitky = wxString::FromUTF8("sedmdesát"); break;
        case '8': desitky = wxString::FromUTF8("osmdesát"); break;
        case '9': desitky = wxString::FromUTF8("devadesát"); break;
        default: break;
        }
    }
    if(len)
    {
        len--;
        switch(cislo[cislo.Len()-3].GetValue()) {
        case '1': stovky = "jednosto"; break;
        case '2': stovky = wxString::FromUTF8("dvěstě"); break;
        case '3': stovky = wxString::FromUTF8("třista"); break;
        case '4': stovky = wxString::FromUTF8("čtyřista"); break;
        case '5': stovky = wxString::FromUTF8("pětset"); break;
        case '6': stovky = wxString::FromUTF8("šestset"); break;
        case '7': stovky = "sedmset"; break;
        case '8': stovky = "osmset"; break;
        case '9': stovky = wxString::FromUTF8("devětset"); break;
        default: break;
        }
    }
    if(len)
    {
        len--;
        switch(cislo[cislo.Len()-4].GetValue()) {
        case '0': tisice = wxString::FromUTF8("tisíc"); break;
        case '1': tisice = wxString::FromUTF8("jedentisíc"); break;
        case '2': tisice = wxString::FromUTF8("dvatisíce"); break;
        case '3': tisice = wxString::FromUTF8("třitisíce"); break;
        case '4': tisice = wxString::FromUTF8("čtyřitisíce"); break;
        case '5': tisice = wxString::FromUTF8("pěttisíc"); break;
        case '6': tisice = wxString::FromUTF8("šesttisíc"); break;
        case '7': tisice = wxString::FromUTF8("sedmtisíc"); break;
        case '8': tisice = wxString::FromUTF8("osmtisíc"); break;
        case '9': tisice = wxString::FromUTF8("devěttisíc"); break;
        default: break;
        }
    }
    if(len)
    {
        len--;
        switch(cislo[cislo.Len()-5].GetValue()) {
        case '1':
        {
            switch(cislo[cislo.Len()-4].GetValue()) {
            case '0': desetitisice = "desettisíc"; tisice = ""; break;
            case '1': desetitisice = wxString::FromUTF8("jedenácttisíc"); tisice = ""; break;
            case '2': desetitisice = wxString::FromUTF8("dvanácttisíc"); tisice = ""; break;
            case '3': desetitisice = wxString::FromUTF8("třinácttisíc"); tisice = ""; break;
            case '4': desetitisice = wxString::FromUTF8("čtrnácttisíc"); tisice = ""; break;
            case '5': desetitisice = wxString::FromUTF8("patnácttisíc"); tisice = ""; break;
            case '6': desetitisice = wxString::FromUTF8("šestnácttisíc"); tisice = ""; break;
            case '7': desetitisice = wxString::FromUTF8("sedmnácttisíc"); tisice = ""; break;
            case '8': desetitisice = wxString::FromUTF8("osmnácttisíc"); tisice = ""; break;
            case '9': desetitisice = wxString::FromUTF8("devatenácttisíc"); tisice = ""; break;
            default: break;
            }
            break;
        }
        case '2': desetitisice = "dvacet"; break;
        case '3': desetitisice = wxString::FromUTF8("třicet"); break;
        case '4': desetitisice = wxString::FromUTF8("čtyřicet"); break;
        case '5': desetitisice = wxString::FromUTF8("padesát"); break;
        case '6': desetitisice = wxString::FromUTF8("šedesát"); break;
        case '7': desetitisice = wxString::FromUTF8("sedmdesát"); break;
        case '8': desetitisice = wxString::FromUTF8("osmdesát"); break;
        case '9': desetitisice = wxString::FromUTF8("devadesát"); break;
        default: break;
        }
    }
    if(len)
    {
        len--;
        switch(cislo[cislo.Len()-6].GetValue()) {
        case '1': statisice = "jednosto"; break;
        case '2': statisice = wxString::FromUTF8("dvěstě"); break;
        case '3': statisice = wxString::FromUTF8("třista"); break;
        case '4': statisice = wxString::FromUTF8("čtyřista"); break;
        case '5': statisice = wxString::FromUTF8("pětset"); break;
        case '6': statisice = wxString::FromUTF8("šestset"); break;
        case '7': statisice = "sedmset"; break;
        case '8': statisice = "osmset"; break;
        case '9': statisice = wxString::FromUTF8("devětset"); break;
        default: break;
        }
    }
    if(desetinne == "00")
    {
        return statisice+desetitisice+tisice+stovky+desitky+jednotky;
    }
    else
    {
        return statisice+desetitisice+tisice+stovky+desitky+jednotky+wxNumberFormatter::GetDecimalSeparator()+desetinne;
    }
}

int dxutils::pocetCislic(int c)
{
    int cislice = 0;
    while (c > 0)
    {
        c /= 10;
        cislice++;
    }
    return cislice;
}

int dxutils::mmToPx(double mm, wxDC *dc, bool toScreen)
{
    double sx = 1.0, sy = 1.0;
    if(!toScreen) dc->GetUserScale(&sx, &sy);
    return wxRound((((double)(dc->GetPPI().x) / 25.4) * mm) / sx);
}

wxDateTime dxutils::todayForObdobi()
{
    wxDateTime result = wxDateTime::Today();
    if(result.GetYear() == idbmodel->currentRok()) return result;
    return wxDateTime(31, wxDateTime::Dec, idbmodel->currentRok());
}

wxString dxutils::fontToString(const wxFont &font)
{
    wxString underlined;
    if(font.GetUnderlined())
        underlined = "1";
    else
        underlined = "0";
    return wxString::Format("%d,%d,%d,%d," + underlined + "," + font.GetFaceName(), font.GetPointSize(), font.GetFamily(), font.GetStyle(), font.GetWeight());
}

wxFont dxutils::fontFromString(const wxString &fontstr)
{
    wxArrayString strings(wxSplit(fontstr, ','));
    if(strings.GetCount() == 6)
    {
        int fontPointSize = wxAtoi(strings[0]);
        int fontFamily = wxAtoi(strings[1]);
        int fontStyle = wxAtoi(strings[2]);
        int fontWeight = wxAtoi(strings[3]);
        int fontUnderlined = wxAtoi(strings[4]);
        wxString fontFaceName = strings[5];
        fontFaceName.Replace(" ", "");
        if(fontPointSize > 0)
        {
            return wxFont(fontPointSize, fontFamily, fontStyle, fontWeight, fontUnderlined, fontFaceName);
        }
    }
    return wxFont(wxFontInfo(12).FaceName(FACENAME));
}

double dxutils::cenamj(double cena, bool bezdph, int sazbaid)
{
    if(bezdph) return cena;
    double sazba = 1.0*idbmodel->getDphSazbaValue(sazbaid);
    return cena*(1-round4(sazba/(100.0+sazba)));
}

wxString dxutils::ctvrtletiString(int ctvrtleti)
{
    switch(ctvrtleti) {
    case 1:
        return wxString::FromUTF8("1. čtvrtletí");
    case 2:
        return wxString::FromUTF8("2. čtvrtletí");
    case 3:
        return wxString::FromUTF8("3. čtvrtletí");
    case 4:
        return wxString::FromUTF8("4. čtvrtletí");
    default:
        return "blbost";
    }
}

wxString dxutils::mesicString(int mesic)
{
    switch(mesic) {
    case 1:
        return "Leden";
    case 2:
        return wxString::FromUTF8("Únor");
    case 3:
        return wxString::FromUTF8("Březen");
    case 4:
        return "Duben";
    case 5:
        return wxString::FromUTF8("Květen");
    case 6:
        return wxString::FromUTF8("Červen");
    case 7:
        return wxString::FromUTF8("Červenec");
    case 8:
        return "Srpen";
    case 9:
        return wxString::FromUTF8("Září");
    case 10:
        return wxString::FromUTF8("Říjen");
    case 11:
        return "Listopad";
    case 12:
        return "Prosinec";
    default:
        return "blbost";
    }
}

wxDateTime::Month dxutils::ctvrtletiMesicStart(int ctvrtleti)
{
    switch(ctvrtleti) {
    case 1:
        return wxDateTime::Jan;
    case 2:
        return wxDateTime::Apr;
    case 3:
        return wxDateTime::Jul;
    case 4:
        return wxDateTime::Oct;
    default:
        return wxDateTime::Jan;
    }
}

wxDateTime::Month dxutils::ctvrtletiMesicEnd(int ctvrtleti)
{
    switch(ctvrtleti) {
    case 1:
        return wxDateTime::Mar;
    case 2:
        return wxDateTime::Jun;
    case 3:
        return wxDateTime::Sep;
    case 4:
        return wxDateTime::Dec;
    default:
        return wxDateTime::Mar;
    }
}

void dxutils::kurzyDownload(const wxDateTime &datum)
{
    wxBeginBusyCursor();
#if __WXGTK__
    wxYield();
#endif
    if(downloadFileFromNet(wxString::Format("http://www.cnb.cz/cs/financni_trhy/devizovy_trh/kurzy_devizoveho_trhu/denni_kurz.txt?date=%s", datum.Format("%d.%m.%Y")),
                           wxStandardPaths::Get().GetUserDataDir(), datum.Format("%d.%m.%Y")+".txt"))
    {
        wxFile kurzy(wxStandardPaths::Get().GetUserDataDir()+wxFILE_SEP_PATH+datum.Format("%d.%m.%Y")+".txt", wxFile::read);
        wxString text;
        kurzy.Open(wxStandardPaths::Get().GetUserDataDir()+wxFILE_SEP_PATH+datum.Format("%d.%m.%Y")+".txt");
        kurzy.ReadAll(&text);
        while(text.Find('\n') != wxNOT_FOUND)
        {
            wxString line = text.BeforeFirst('\n');
            if(!line.Contains("kurz") && !line.Contains("#"))
            {
                wxStringTokenizer tok(line, "|");
                if(tok.CountTokens() == 5)
                {
                    double pocet = 1.0, kurz = 1.0;
                    wxString kod;
                    int i=0;
                    while(tok.HasMoreTokens())
                    {
                        wxString token = tok.GetNextToken();
                        switch(i) {
                        case 2: token.ToDouble(&pocet); break;
                        case 3: kod = token; break;
                        case 4: token.Replace(",","."); token.ToCDouble(&kurz); break;
                        default: break;
                        }
                        i++;
                    }
                    idbmodel->addKurz(Kurz(idbmodel->getLastId("kurz"), datum, kurz/pocet, kod, false));
                }
            }
            text = text.AfterFirst('\n');
        }
        kurzy.Close();
        wxRemoveFile(wxStandardPaths::Get().GetUserDataDir()+wxFILE_SEP_PATH+datum.Format("%d.%m.%Y")+".txt");
        wxEndBusyCursor();
        return;
    }
    wxEndBusyCursor();
}

bool dxutils::jednotnekurzyDownload(int rok)
{
    wxBeginBusyCursor();
#if __WXGTK__
    wxYield();
#endif
    if(downloadFileFromNet(wxString::Format("http://dxde.dxsolutions.org/kurzy/%d.txt", rok),
                           wxStandardPaths::Get().GetUserDataDir(), wxString::Format("%d.txt", rok)))
    {
        wxFile kurzy(wxStandardPaths::Get().GetUserDataDir()+wxFILE_SEP_PATH+wxString::Format("%d.txt", rok), wxFile::read);
        wxString text;
        kurzy.Open(wxStandardPaths::Get().GetUserDataDir()+wxFILE_SEP_PATH+wxString::Format("%d.txt", rok));
        kurzy.ReadAll(&text);
        while(text.Find('\n') != wxNOT_FOUND)
        {
            wxString line = text.BeforeFirst('\n');
            if(!line.Contains("kurz") && !line.Contains("#"))
            {
                wxStringTokenizer tok(line, "|");
                if(tok.CountTokens() == 4)
                {
                    double pocet = 1.0, kurz = 1.0;
                    wxString kod;
                    int i=0;
                    while(tok.HasMoreTokens())
                    {
                        wxString token = tok.GetNextToken();
                        switch(i) {
                        case 1: token.ToDouble(&pocet); break;
                        case 2: kod = token; break;
                        case 3: token.Replace(",","."); token.ToCDouble(&kurz); break;
                        default: break;
                        }
                        i++;
                    }
                    idbmodel->addKurz(Kurz(idbmodel->getLastId("kurz"), wxDateTime(31, wxDateTime::Dec, rok), kurz/pocet, kod, true));
                }
            }
            text = text.AfterFirst('\n');
        }
        kurzy.Close();
        wxRemoveFile(wxStandardPaths::Get().GetUserDataDir()+wxFILE_SEP_PATH+wxString::Format("%d.txt", rok));
        wxEndBusyCursor();
        return true;
    }
    wxEndBusyCursor();
    return false;
}

bool dxutils::haveCiselniky()
{
    if(!wxFile::Exists(wxStandardPaths::Get().GetUserDataDir()+"/okec.xml"))
    {
        return false;
    }
    else
    {
        if(wxDateTime::Today().DiffAsDateSpan(wxFileName(wxStandardPaths::Get().GetUserDataDir()+"/okec.xml").GetModificationTime()).GetTotalMonths()>2)
            return false;
    }
    if(!wxFile::Exists(wxStandardPaths::Get().GetUserDataDir()+"/ufo.xml"))
    {
        return false;
    }
    else
    {
        if(wxDateTime::Today().DiffAsDateSpan(wxFileName(wxStandardPaths::Get().GetUserDataDir()+"/ufo.xml").GetModificationTime()).GetTotalMonths()>2)
            return false;
    }
    if(!wxFile::Exists(wxStandardPaths::Get().GetUserDataDir()+"/pracufo.xml"))
    {
        return false;
    }
    else
    {
        if(wxDateTime::Today().DiffAsDateSpan(wxFileName(wxStandardPaths::Get().GetUserDataDir()+"/pracufo.xml").GetModificationTime()).GetTotalMonths()>2)
            return false;
    }
    if(!wxFile::Exists(wxStandardPaths::Get().GetUserDataDir()+"/zeme.xml"))
    {
        return false;
    }
    else
    {
        if(wxDateTime::Today().DiffAsDateSpan(wxFileName(wxStandardPaths::Get().GetUserDataDir()+"/zeme.xml").GetModificationTime()).GetTotalMonths()>2)
            return false;
    }
    if(!wxFile::Exists(wxStandardPaths::Get().GetUserDataDir()+"/obce.xml"))
    {
        return false;
    }
    else
    {
        if(wxDateTime::Today().DiffAsDateSpan(wxFileName(wxStandardPaths::Get().GetUserDataDir()+"/obce.xml").GetModificationTime()).GetTotalMonths()>2)
            return false;
    }
    return true;
}

bool dxutils::downloadCiselniky()
{
    if(!downloadFileFromNet("https://adisepo.mfcr.cz/adistc/epo_ciselnik?C=okec", wxStandardPaths::Get().GetUserDataDir(), "okec.xml")) return false;
    if(!downloadFileFromNet(wxString::Format("https://adisepo.mfcr.cz/adistc/epo_ciselnik?C=ufo&PL=%s", wxDateTime::Today().Format("%Y-%m-%d")),
                            wxStandardPaths::Get().GetUserDataDir(), "ufo.xml")) return false;
    if(!downloadFileFromNet(wxString::Format("https://adisepo.mfcr.cz/adistc/epo_ciselnik?C=pracufo&PL=%s", wxDateTime::Today().Format("%Y-%m-%d")),
                            wxStandardPaths::Get().GetUserDataDir(), "pracufo.xml")) return false;
    if(!downloadFileFromNet(wxString::Format("https://adisepo.mfcr.cz/adistc/epo_ciselnik?C=zeme&PL=%s", wxDateTime::Today().Format("%Y-%m-%d")),
                            wxStandardPaths::Get().GetUserDataDir(), "zeme.xml")) return false;
    if(!downloadFileFromNet(wxString::Format("https://adisepo.mfcr.cz/adistc/epo_ciselnik?C=obce&PL=%s", wxDateTime::Today().Format("%Y-%m-%d")),
                            wxStandardPaths::Get().GetUserDataDir(), "obce.xml")) return false;
    if(!downloadFileFromNet("https://adisepo.mfcr.cz/adistc/epo_ciselnik?C=dp_fu", wxStandardPaths::Get().GetUserDataDir(), "dp_fu.xml")) return false;
    if(!downloadFileFromNet("https://adisepo.mfcr.cz/adistc/epo_ciselnik?C=pbucelur", wxStandardPaths::Get().GetUserDataDir(), "pbucelur.xml")) return false;
    if(!downloadFileFromNet("https://adisepo.mfcr.cz/adistc/epo_ciselnik?C=ucelur", wxStandardPaths::Get().GetUserDataDir(), "ucelur.xml")) return false;
    if(!downloadFileFromNet(wxString::Format("https://adisepo.mfcr.cz/adistc/epo_ciselnik?C=obcezujzko&PL=%s", wxDateTime::Today().Format("%Y-%m-%d")),
                            wxStandardPaths::Get().GetUserDataDir(), "obcezujzko.xml")) return false;
    return true;
}

bool dxutils::downloadFileFromNet(const wxString &path, const wxString &folder, const wxString &filename)
{
    wxString fullpath = folder+wxFILE_SEP_PATH+filename;
    CURL *curl;
    curl = curl_easy_init();
    if(curl)
    {
        std::ofstream file(fullpath.ToStdString(), std::ios::binary);
        curl_easy_setopt(curl, CURLOPT_URL, path.ToStdString().c_str());
        curl_easy_setopt(curl, CURLOPT_SSL_VERIFYHOST, 0);
        curl_easy_setopt(curl, CURLOPT_SSL_VERIFYPEER, 0);
        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, curlCallbackFile);
        curl_easy_setopt(curl, CURLOPT_WRITEDATA, reinterpret_cast<void*>(&file));
        CURLcode res = curl_easy_perform(curl);
        curl_easy_cleanup(curl);
        if(res == CURLE_OK)
        {
            file.close();
            return true;
        }
        file.close();
    }
    return false;
}

size_t dxutils::pocetZnaku(const wxString &text, char znak)
{
    size_t result = 0;
    for(size_t i=0; i<text.Len(); ++i)
    {
        if(text[i] == znak) result++;
    }
    return result;
}

bool dxutils::zalohaDat(wxString path)
{
    bool result = false;
    path.Append(wxFILE_SEP_PATH);
    path.Append(wxDateTime::Today().Format("dxde-%Y-%m-%d"));
    if(!wxDirExists(path))
    {
        wxMkdir(path);
    }
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        dxStringMap dbmap = dxsettings.dbmap();
        dxStringIt it = dbmap.begin();
        for(; it != dbmap.end(); ++it)
        {
            result = wxCopyFile(wxStandardPaths::Get().GetUserDataDir()+wxFILE_SEP_PATH+(*it).first,
                                path+wxFILE_SEP_PATH+(*it).second+".db");
        }
    }
    return result;
}

wxString dxutils::fillByZeroes(const wxString &text, size_t len)
{
    if(len < text.Length()) return text;
    wxString result = "";
    for(size_t i=0; i < len-text.Length(); i++)
    {
        result.Append("0");
    }
    result.Append(text);
    return result;
}

wxString dxutils::normalizeFilename(wxString filename)
{
    filename.Replace(" ","_");
    filename.Replace("/","_");
    filename.Replace("\\","_");
    filename.Replace(".","");
    return filename;
}

wxString dxutils::toHtmlBr(wxString text)
{
    text.Replace("\n","<br/>");
    return text;
}

bool dxutils::isSameDouble(double a, double b, double absEpsilon, double relEpsilon)
{
    // Check if the numbers are really close -- needed when comparing numbers near zero.
    double diff = std::abs(a - b);
    if(diff <= absEpsilon)
        return true;

    // Otherwise fall back to Knuth's algorithm
    return diff <= ( (std::abs(a) < std::abs(b) ? std::abs(b) : std::abs(a)) * relEpsilon);
}

int dxutils::versionToInt(const wxString &version)
{
    wxStringTokenizer tok(version, ".");
    if(tok.CountTokens() == 3)
    {
        int result = 0;
        int i=0;
        long a = 0;
        while(tok.HasMoreTokens())
        {
            wxString token = tok.GetNextToken();
            switch(i) {
            case 0: token.ToLong(&a); result += a*1000; break;
            case 1: token.ToLong(&a); result += a*10; break;
            case 2: token.ToLong(&a); result += a; break;
            default: break;
            }
            i++;
        }
        return result;
    }
    return 0;
}

wxString dxutils::fromStdString(const std::string &str)
{
    if(str.empty()) return "";
    return wxString::FromUTF8(str.c_str());
}

std::string dxutils::toStdString(const wxString &str)
{
    if(str.empty()) return "";
    return str.ToStdString();
}

double dxutils::toCDouble(const std::string &str)
{
    if(str.empty()) return 0.0;
    double v;
    if(fromStdString(str).ToCDouble(&v)) return v;
    else return 0.0;
}

wxString dxutils::availableVersion()
{
    CURL *curl;
    curl = curl_easy_init();
    if(curl)
    {
        std::string response;
        curl_easy_setopt(curl, CURLOPT_URL, "https://dxsolutions.org/dxde-version");
        curl_easy_setopt(curl, CURLOPT_SSL_VERIFYHOST, 0);
        curl_easy_setopt(curl, CURLOPT_SSL_VERIFYPEER, 0);
        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, curlCallback);
        curl_easy_setopt(curl, CURLOPT_WRITEDATA, &response);
        CURLcode res = curl_easy_perform(curl);
        curl_easy_cleanup(curl);
        if(res == CURLE_OK)
        {
            return fromStdString(response).BeforeFirst('\n');
        }
    }
    return "0.00.0";
}

wxIcon dxutils::GreyScaledIcon(const char * const *xpm)
{
    wxImage img(xpm);
    wxIcon icon;
    icon.CopyFromBitmap(wxBitmap(img.ConvertToGreyscale()));
    return icon;
}

int dxutils::modulo97(const wxString &kod, const wxString &cislouctu)
{
    wxString predcisli = cislouctu.BeforeLast('-');
    wxString cislo = cislouctu.AfterLast('-');
    wxString input = wxString(4-kod.Length(),'0')+kod+wxString(6-predcisli.Length(),'0')+predcisli+wxString(10-cislo.Length(),'0')+cislo+"123500";
    wxString output = wxEmptyString;
    for(size_t i = 0; i < input.Length(); i++)
    {
        if(output.Length() < 9)
            output += input[i];
        else
        {
            output = wxString::Format("%d", wxAtoi(output) % 97) + input[i];
        }
    }
    return wxAtoi(output) % 97;
}

wxString dxutils::iban(const wxString &kod, const wxString &cislouctu)
{
    wxString predcisli = cislouctu.BeforeLast('-');
    wxString cislo = cislouctu.AfterLast('-');
    int x = 98-modulo97(kod,cislouctu);
    wxString xx = wxString::Format("%d",x);
    return wxString("CZ")+wxString(2-xx.Length(),'0')+xx+wxString(4-kod.Length(),'0')+kod+wxString(6-predcisli.Length(),'0')+predcisli+wxString(10-cislo.Length(),'0')+cislo;
}
