/***************************************************************
 * Name:      dbmodel.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include <wx/stdpaths.h>
#include <wx/log.h>
#include <wx/arrimpl.cpp>
#include "dbsort.h"
#include "dbmodel.h"
#include "sqlitesql.h"
#include "utils/dxsettings.h"
#include "utils/dxutils.h"
#include "dxdefs.h"

WX_DEFINE_OBJARRAY(dxAdresaArray)
WX_DEFINE_OBJARRAY(dxKontaktArray)
WX_DEFINE_OBJARRAY(dxKontaktViewArray)
WX_DEFINE_OBJARRAY(dxObdobiArray)
WX_DEFINE_OBJARRAY(dxDphSazbaArray)
WX_DEFINE_OBJARRAY(dxMenaArray)
WX_DEFINE_OBJARRAY(dxDenikSloupecArray)
WX_DEFINE_OBJARRAY(dxPredkontaceArray)
WX_DEFINE_OBJARRAY(dxDenikArray)
WX_DEFINE_OBJARRAY(dxDenikViewArray)
WX_DEFINE_OBJARRAY(dxPokladnaArray)
WX_DEFINE_OBJARRAY(dxBankaArray)
WX_DEFINE_OBJARRAY(dxRadaArray)
WX_DEFINE_OBJARRAY(dxKurzArray)
WX_DEFINE_OBJARRAY(dxFaTypArray)
WX_DEFINE_OBJARRAY(dxFaUhradaArray)
WX_DEFINE_OBJARRAY(dxFaPolozkaArray)
WX_DEFINE_OBJARRAY(dxFakturaArray)
WX_DEFINE_OBJARRAY(dxFakturaViewArray)
WX_DEFINE_OBJARRAY(dxZavPohArray)
WX_DEFINE_OBJARRAY(dxZavPohViewArray)
WX_DEFINE_OBJARRAY(dxUhradaArray)
WX_DEFINE_OBJARRAY(dxOdpiskupinaArray)
WX_DEFINE_OBJARRAY(dxMajetekArray)
WX_DEFINE_OBJARRAY(dxMajetekViewArray)
WX_DEFINE_OBJARRAY(dxOdpisArray)
WX_DEFINE_OBJARRAY(dxSettingsArray)
WX_DEFINE_OBJARRAY(dxOdeslanyArray)
WX_DEFINE_OBJARRAY(dxFaDphSazbaArray)
WX_DEFINE_OBJARRAY(dxDphDruhArray)
WX_DEFINE_OBJARRAY(dxUlozeneArray)
WX_DEFINE_OBJARRAY(dxDphDokladArray)
WX_DEFINE_OBJARRAY(dxSkladArray)
WX_DEFINE_OBJARRAY(dxZasobaArray)
WX_DEFINE_OBJARRAY(dxZasobaViewArray)
WX_DEFINE_OBJARRAY(dxPohybzasobArray)
WX_DEFINE_OBJARRAY(dxInventuraArray)
WX_DEFINE_OBJARRAY(dxIvPolozkaArray)
WX_DEFINE_OBJARRAY(dxRidicArray)
WX_DEFINE_OBJARRAY(dxPhmArray)
WX_DEFINE_OBJARRAY(dxDruhVozidlaArray)
WX_DEFINE_OBJARRAY(dxVozidloArray)
WX_DEFINE_OBJARRAY(dxJizdaViewArray)
WX_DEFINE_OBJARRAY(dxJizdaArray)

wxDEFINE_EVENT(MODEL_UPDATED, wxCommandEvent);
wxDEFINE_EVENT(MODEL_MSG, wxCommandEvent);

static bool hasString(wxArrayString &strings, wxString value)
{
    for(size_t i=0; i<strings.GetCount(); i++)
    {
        if(strings[i] == value) return true;
    }
    return false;
}

DbModel *DbModel::m_this = NULL;

DbModel::DbModel() : m_parent(NULL), m_currentRok(-1)
{
}

DbModel::DbModel(wxWindow *parent) : m_parent(parent), m_currentRok(wxDateTime::Today().GetYear())
{
    m_this = this;
}

DbModel::~DbModel()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(!m_db.IsOpen())
            return;
        try
        {
            m_db.Close();
        }
        catch(const wxSQLite3Exception& e)
        {
            wxLogDebug(wxString::Format("Close error: %s", e.GetMessage()));
        }
    }
    m_this = NULL;
}

DbModel *DbModel::instance()
{
    return m_this;
}

void DbModel::initModel()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
        wxSQLite3Database::InitializeSQLite();
}

void DbModel::shutdownModel()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
        wxSQLite3Database::ShutdownSQLite();
}

wxString DbModel::getVersion()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxString version = wxSQLite3Database::GetWrapperVersion() << "\n"
                            << "SQLite3 " << wxSQLite3Database::GetVersion();
        return version;
    }
    return wxEmptyString;
}

bool DbModel::openDatabase()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxString path = wxStandardPaths::Get().GetUserDataDir();
        if(!wxDirExists(path))
        {
            wxMkdir(path);
        }
        path << wxFILE_SEP_PATH << dxsettings.dbname();
        int err = SQLITE_OK;
        wxString errStr = wxEmptyString;
        try
        {
            m_db.Open(path);
        }
        catch(const wxSQLite3Exception &e)
        {
            err = e.GetErrorCode();
            errStr << e.GetMessage();
        }
        if(err == SQLITE_OK)
        {
            tryExecuteUpdate("PRAGMA locking_mode = EXCLUSIVE;");
            tryExecuteUpdate("BEGIN EXCLUSIVE;");
            tryExecuteUpdate("COMMIT;");
            //Try if db selectable
            try
            {
                m_db.ExecuteScalar("SELECT count(*) FROM sqlite_master WHERE type='table'");
            }
            catch(const wxSQLite3Exception &e)
            {
                errStr << e.GetMessage();
            }
            checkDb();
            //povolit cizi klice
            m_db.EnableForeignKeySupport(true);
            return true;
        }
        if(!errStr.empty())
            wxLogDebug(errStr);
    }
    return false;
}

bool DbModel::newDatabase()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxString path = wxStandardPaths::Get().GetUserDataDir();
        if(!wxDirExists(path))
        {
            wxMkdir(path);
        }
        path << wxFILE_SEP_PATH << dxsettings.dbname();
        int err = SQLITE_OK;
        wxString errStr = wxEmptyString;
        try
        {
            m_db.Open(path);
        }
        catch(const wxSQLite3Exception &e)
        {
            err = e.GetErrorCode();
            errStr << e.GetMessage();
        }
        if(err == SQLITE_OK)
        {
            tryExecuteUpdate("PRAGMA locking_mode = EXCLUSIVE;");
            tryExecuteUpdate("BEGIN EXCLUSIVE;");
            tryExecuteUpdate("COMMIT;");
            //povolit cizi klice
            m_db.EnableForeignKeySupport(true);
            //Try if db selectable
            try
            {
                m_db.ExecuteScalar("SELECT count(*) FROM sqlite_master WHERE type='table'");
            }
            catch(const wxSQLite3Exception &e)
            {
                errStr << e.GetMessage();
            }
            createDatabase();
            return true;
        }
        if(!errStr.empty())
            wxLogDebug(errStr);
    }
    return false;
}

bool DbModel::closeDatabase()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(!m_db.IsOpen())
            return true;
        try
        {
            tryExecuteUpdate("PRAGMA locking_mode = NORMAL;");
            m_db.Close();
        }
        catch(const wxSQLite3Exception& e)
        {
            wxLogDebug(wxString::Format("Close error: %s", e.GetMessage()));
        }
        return true;
    }
    return false;
}

int DbModel::getLastId(const wxString &table)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists(table))
        {
            return m_db.ExecuteScalar(wxString::Format("SELECT max(id) FROM %s", table))+1;
        }
    }
    return 1;
}

//naplni data arrays hodnotami
void DbModel::fillData()
{
    fillFirma();
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        m_currentRok = m_db.ExecuteScalar("SELECT rok FROM obdobi WHERE current=1");
    }
    fillSmtp();
    fillPriznani();
    updateStavZasob();
    fireUpdated(ID_MODEL_ALL);
}

//update dbfirma aby obsahoval IC
void DbModel::updateDbFirma()
{
    dxStringMap firmy = dxsettings.dbmap();
    dxStringIt it = firmy.begin();
    for(; it != firmy.end(); ++it)
    {
        if((*it).second.find_last_of('_') != wxString::npos && ((*it).second.Last() == '_' || (*it).second.Right(1).IsNumber()))
        {
            continue;
        }
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            int err = SQLITE_OK;
            try
            {
                m_db.Open(wxStandardPaths::Get().GetUserDataDir()+wxFILE_SEP_PATH+(*it).first);
            }
            catch(const wxSQLite3Exception &e)
            {
                wxLogDebug(e.GetMessage());
                err = e.GetErrorCode();
            }
            if(err == SQLITE_OK && m_db.TableExists("firma"))
            {
                wxArrayString columns = tableColumns("firma");
                if(hasString(columns,"nazev") && hasString(columns,"ic"))
                {
                    dxsettings.editDbnamePair((*it).first, m_db.ExecuteQuery("select nazev || '_' || ic from firma").GetString(0));
                }
                m_db.Close();
            }
        }
    }
}

void DbModel::checkDb()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        bool fireError = false;
        if(!m_db.TableExists("version")) createDatabaseTable(sqlite_tb_version);
        wxString dbVersionStr = m_db.ExecuteQuery("SELECT verze FROM version").GetString(0);
        if(dxutils::pocetZnaku(dbVersionStr,'.')==1) dbVersionStr.Append(".0");
        int dbVersion = dxutils::versionToInt(dbVersionStr);
        //upgrade z predchozich verzi db
        if(dbVersion < 1200)
        {
            tryExecuteUpdate("UPDATE version SET verze='1.20'");
            if(!m_db.TableExists("smtp")) createDatabaseTable(sqlite_tb_smtp);
            if(!m_db.TableExists("odeslany")) createDatabaseTable(sqlite_tb_odeslany);
            if(!m_db.TableExists("priznani")) createDatabaseTable(sqlite_tb_priznani);
            if(!m_db.TableExists("dphdruh")) createDatabaseTable(sqlite_tb_dphdruh);
            if(!m_db.TableExists("ulozene")) createDatabaseTable(sqlite_tb_ulozene);
            if(m_db.TableExists("dphsazba"))
            {
                tryExecuteUpdate("DROP TABLE dphsazba");
                createDatabaseTable(sqlite_tb_dphsazba);
            }
            if(m_db.TableExists("faktura"))
            {
                tryExecuteUpdate("ALTER TABLE faktura ADD COLUMN dppd TEXT");
                tryExecuteUpdate("UPDATE faktura SET dppd=duzp");
                tryExecuteUpdate("ALTER TABLE faktura ADD COLUMN dphdruhid INTEGER");
                tryExecuteUpdate("ALTER TABLE faktura ADD COLUMN kurz REAL");
                tryExecuteUpdate("ALTER TABLE faktura ADD COLUMN pausal INTEGER");
                tryExecuteUpdate("UPDATE faktura SET pausal=0");
            }
            if(m_db.TableExists("zavpoh"))
            {
                tryExecuteUpdate("ALTER TABLE zavpoh ADD COLUMN dppd TEXT");
                tryExecuteUpdate("UPDATE zavpoh SET dppd=duzp");
                tryExecuteUpdate("ALTER TABLE zavpoh ADD COLUMN dphdruhid INTEGER");
                tryExecuteUpdate("ALTER TABLE zavpoh ADD COLUMN kurz REAL");
                tryExecuteUpdate("ALTER TABLE zavpoh ADD COLUMN pausal INTEGER");
                tryExecuteUpdate("UPDATE zavpoh SET pausal=0");
            }
            if(m_db.TableExists("firma"))
            {
                tryExecuteUpdate("ALTER TABLE firma ADD COLUMN pausal INTEGER");
                tryExecuteUpdate("UPDATE firma SET pausal=0");
            }
            if(m_db.TableExists("denik"))
            {
                tryExecuteUpdate("ALTER TABLE denik ADD COLUMN pausal INTEGER");
                tryExecuteUpdate("UPDATE denik SET pausal=0");
            }
        }
        if(dbVersion < 1300)
        {
            tryExecuteUpdate("UPDATE version SET verze='1.30'");
            if(m_db.TableExists("denik") && hasTableColumn("denik", "castka") && hasTableColumn("denik", "castkacizi"))
            {
                wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT castkacizi,id FROM denik");
                while(rs.NextRow())
                {
                    if(dxutils::isSameDouble(rs.GetDouble(0),0.0))
                    {
                        tryExecuteUpdate(wxString::Format("UPDATE denik SET castkacizi=castka WHERE id=%d", rs.GetInt(1)));
                    }
                }
                rs.Finalize();
            }
            if(!m_db.TableExists("sklad")) createDatabaseTable(sqlite_tb_sklad);
            if(m_db.TableExists("rada"))
            {
                if(!hasTableId("rada", "id", "ZB")) tryExecuteUpdate("INSERT INTO rada (id,popis,prefix,znaku,cislo) VALUES ('ZB','Zásoba','ZB',6,1)");
                if(!hasTableId("rada", "id", "PR")) tryExecuteUpdate("INSERT INTO rada (id,popis,prefix,znaku,cislo) VALUES ('PR','Příjemka','PR',6,1)");
                if(!hasTableId("rada", "id", "VY")) tryExecuteUpdate("INSERT INTO rada (id,popis,prefix,znaku,cislo) VALUES ('VY','Výdejka','VY',6,1)");
                if(!hasTableId("rada", "id", "IV")) tryExecuteUpdate("INSERT INTO rada (id,popis,prefix,znaku,cislo) VALUES ('IV','Inventura','IV',6,1)");
            }
            if(!m_db.TableExists("zasoba")) createDatabaseTable(sqlite_tb_zasoba);
            if(m_db.TableExists("fapolozka") && !hasTableColumn("fapolozka", "zasobaid"))
            {
                tryExecuteUpdate("ALTER TABLE fapolozka ADD COLUMN zasobaid INTEGER");
            }
            if(!m_db.TableExists("inventura")) createDatabaseTable(sqlite_tb_inventura);
            if(!m_db.TableExists("ivpolozka")) createDatabaseTable(sqlite_tb_ivpolozka);
            if(!m_db.TableExists("pohybzasob")) createDatabaseTable(sqlite_tb_pohybzasob);
            if(m_db.TableExists("kurz") && !hasTableColumn("kurz", "jednotny"))
            {
                tryExecuteUpdate("ALTER TABLE kurz ADD COLUMN jednotny INTEGER");
                tryExecuteUpdate("UPDATE kurz SET jednotny=0");
            }
            if(m_db.TableExists("mena") && (!hasTableColumn("mena", "od") || !hasTableColumn("mena", "do")))
            {
                tryExecuteUpdate("DROP TABLE mena");
                createDatabaseTable(sqlite_tb_mena);
            }
            if(m_db.TableExists("dphdruh"))
            {
                tryExecuteUpdate("DROP TABLE dphdruh");
                createDatabaseTable(sqlite_tb_dphdruh);
            }
        }
        if(dbVersion < 1400)
        {
            tryExecuteUpdate("UPDATE version SET verze='1.40.0'");
            if(m_db.TableExists("kontakt") && !hasTableColumn("kontakt", "jmeno"))
            {
                tryExecuteUpdate("ALTER TABLE kontakt ADD COLUMN jmeno TEXT");
                tryExecuteUpdate("UPDATE kontakt SET jmeno=nazev");
            }
            if(!m_db.TableExists("ridic")) createDatabaseTable(sqlite_tb_ridic);
            if(!m_db.TableExists("phm")) createDatabaseTable(sqlite_tb_phm);
            if(!m_db.TableExists("cenaphm")) createDatabaseTable(sqlite_tb_cenaphm);
            if(!m_db.TableExists("druhvozidla")) createDatabaseTable(sqlite_tb_druhvozidla);
            if(!m_db.TableExists("nahradakm")) createDatabaseTable(sqlite_tb_nahradakm);
            if(!m_db.TableExists("vozidlo")) createDatabaseTable(sqlite_tb_vozidlo);
            if(!m_db.TableExists("jizda")) createDatabaseTable(sqlite_tb_jizda);
        }
        if(dbVersion < 1401)
        {
            tryExecuteUpdate("UPDATE version SET verze='1.40.1'");
        }
        if(dbVersion < 1402)
        {
            tryExecuteUpdate("UPDATE version SET verze='1.40.2'");
            if(m_db.TableExists("dphdruh"))
            {
                tryExecuteUpdate("DROP TABLE dphdruh");
                createDatabaseTable(sqlite_tb_dphdruh);
            }
        }
        if(dbVersion < 1403)
        {
            tryExecuteUpdate("UPDATE version SET verze='1.40.3'");
            if(m_db.TableExists("fapolozka"))
            {
                tryExecuteUpdate("ALTER TABLE fapolozka ADD COLUMN zaokrouhleni INTEGER");
                tryExecuteUpdate("UPDATE fapolozka SET zaokrouhleni=0");
            }
        }
        if(dbVersion < 1404)
        {
            tryExecuteUpdate("UPDATE version SET verze='1.40.4'");
        }
        if(dbVersion < 1405)
        {
            tryExecuteUpdate("UPDATE version SET verze='1.40.5'");
        }
        if(dbVersion < 1406)
        {
            tryExecuteUpdate("UPDATE version SET verze='1.40.6'");
            if(!m_db.TableExists("bankovniustav")) createDatabaseTable(sqlite_tb_bankovniustav);
            if(m_db.TableExists("banka"))
            {
                tryExecuteUpdate("ALTER TABLE banka ADD COLUMN iban TEXT");
                if(hasTableColumn("banka","id") && hasTableColumn("banka","cislouctu") && hasTableColumn("banka","kodbanky"))
                {
                    wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT id,cislouctu,kodbanky FROM banka");
                    while(rs.NextRow())
                    {
                        wxSQLite3Statement st = m_db.PrepareStatement("UPDATE banka SET iban=? WHERE id=?");
                        st.Bind(1, dxutils::iban(rs.GetString(2), rs.GetString(1)));
                        st.Bind(2, wxLongLong(rs.GetInt64(0)));
                        st.ExecuteUpdate();
                        st.Finalize();
                    }
                    rs.Finalize();
                }
                tryExecuteUpdate("ALTER TABLE banka ADD COLUMN bic TEXT");
                if(hasTableColumn("banka","id") && hasTableColumn("banka","kodbanky"))
                {
                    wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT id,kodbanky FROM banka");
                    while(rs.NextRow())
                    {
                        wxSQLite3Statement st = m_db.PrepareStatement("UPDATE banka SET bic=? WHERE id=?");
                        st.Bind(1, bicByKod(rs.GetString(1)));
                        st.Bind(2, wxLongLong(rs.GetInt64(0)));
                        st.ExecuteUpdate();
                        st.Finalize();
                    }
                    rs.Finalize();
                }
            }
        }
        if(dbVersion < 1407)
        {
            tryExecuteUpdate("UPDATE version SET verze='1.40.7'");
            if(m_db.TableExists("cenaphm"))
            {
                tryExecuteUpdate("DROP TABLE cenaphm");
                createDatabaseTable(sqlite_tb_cenaphm);
            }
        }
        if(dbVersion < 1408)
        {
            tryExecuteUpdate("UPDATE version SET verze='1.40.8'");
        }
        if(dbVersion < 1409)
        {
            tryExecuteUpdate("UPDATE version SET verze='1.40.9'");
        }
        if(dbVersion < 1410)
        {
            tryExecuteUpdate("UPDATE version SET verze='1.40.10'");
        }
        if(dbVersion < 1411)
        {
            tryExecuteUpdate("UPDATE version SET verze='1.40.11'");
        }
        if(dbVersion < 1412)
        {
            tryExecuteUpdate("UPDATE version SET verze='1.40.12'");
        }
        //check tabulek
        wxString msg = wxString::FromUTF8("Struktura databáze byla poškozená.\nByla obnovena, ale data mohou obsahovat chyby:\n");
        if(!checkTableVersion()) fireError = true;
        if(!checkTableAdresa())
        {
            fireError = true;
            msg << "Adresy\n";
        }
        if(!checkTableKontakt())
        {
            fireError = true;
            msg << wxString::FromUTF8("Odběratelé a dodavatelé\n");
        }
        if(!checkTableFirma())
        {
            fireError = true;
            msg << "Firma\n";
        }
        if(!checkTableObdobi())
        {
            fireError = true;
            msg << wxString::FromUTF8("Období\n");
        }
        if(!checkTableDphsazba()) fireError = true;
        if(!checkTableDphdruh()) fireError = true;
        if(!checkTableMena()) fireError = true;
        if(!checkTableKurz()) fireError = true;
        if(!checkTablePokladna())
        {
            fireError = true;
            msg << "Pokladny\n";
        }
        if(!checkTableBankovniustav()) fireError = true;
        if(!checkTableBanka())
        {
            fireError = true;
            msg << wxString::FromUTF8("Bankovní účty\n");
        }
        if(!checkTableDeniksloupec()) fireError = true;
        if(!checkTablePredkontace())
        {
            fireError = true;
            msg << wxString::FromUTF8("Předkontace\n");
        }
        if(!checkTableDenik())
        {
            fireError = true;
            msg << wxString::FromUTF8("Peněžní deník\n");
        }
        if(!checkTableRada()) fireError = true;
        if(!checkTableFatyp()) fireError = true;
        if(!checkTableFauhrada())
        {
            fireError = true;
            msg << wxString::FromUTF8("Typy úhrad faktur, závazků a pohledávek\n");
        }
        if(!checkTableFaktura())
        {
            fireError = true;
            msg << "Faktury\n";
        }
        if(!checkTableZavpoh())
        {
            fireError = true;
            msg << wxString::FromUTF8("Závazky a pohledávky\n");
        }
        if(!checkTableSklad())
        {
            fireError = true;
            msg << "Sklady\n";
        }
        if(!checkTableZasoba())
        {
            fireError = true;
            msg << wxString::FromUTF8("Zásoby\n");
        }
        if(!checkTableFapolozka())
        {
            fireError = true;
            msg << wxString::FromUTF8("Položky faktur, závazků a pohledávek\n");
        }
        if(!checkTableUhrada())
        {
            fireError = true;
            msg << wxString::FromUTF8("Úhrady faktur, závazků a pohledávek\n");
        }
        if(!checkTableOdpiskupina()) fireError = true;
        if(!checkTableMajetek())
        {
            fireError = true;
            msg << "Majetek\n";
        }
        if(!checkTableOdpis())
        {
            fireError = true;
            msg << "Odpisy majetku\n";
        }
        if(!checkTableSettings())
        {
            fireError = true;
            msg << wxString::FromUTF8("Nastavení\n");
        }
        if(!checkTableSmtp())
        {
            fireError = true;
            msg << "SMTP\n";
        }
        if(!checkTableOdeslany())
        {
            fireError = true;
            msg << wxString::FromUTF8("Odeslané faktury\n");
        }
        if(!checkTablePriznani())
        {
            fireError = true;
            msg << wxString::FromUTF8("Nastavení plátce\n");
        }
        if(!checkTableUlozene())
        {
            fireError = true;
            msg << wxString::FromUTF8("Uložená přiznání\n");
        }
        if(!checkTableInventura())
        {
            fireError = true;
            msg << "Inventury\n";
        }
        if(!checkTableIvpolozka())
        {
            fireError = true;
            msg << wxString::FromUTF8("Položky inventur\n");
        }
        if(!checkTablePohybzasob())
        {
            fireError = true;
            msg << wxString::FromUTF8("Pohyb zásob\n");
        }
        if(!checkTableRidic())
        {
            fireError = true;
            msg << wxString::FromUTF8("Řidiči\n");
        }
        if(!checkTablePhm())
        {
            fireError = true;
            msg << wxString::FromUTF8("Pohonné hmoty\n");
        }
        if(!checkTableCenaphm())
        {
            fireError = true;
            msg << wxString::FromUTF8("Cena pohonných hmot\n");
        }
        if(!checkTableDruhVozidla())
        {
            fireError = true;
            msg << "Druh vozidel\n";
        }
        if(!checkTableNahradakm())
        {
            fireError = true;
            msg << wxString::FromUTF8("Náhrada za km\n");
        }
        if(!checkTableVozidlo())
        {
            fireError = true;
            msg << "Vozidla\n";
        }
        if(!checkTableJizda())
        {
            fireError = true;
            msg << wxString::FromUTF8("Jízdy\n");
        }
        if(fireError) fireMsg(msg);
    }
}

bool DbModel::checkTableVersion()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("version"))
        {
            bool create = false;
            wxArrayString arr = tableColumns("version");
            if(!hasString(arr, "verze"))
            {
                create = true;
            }
            if(!tableRows("version"))
            {
                create = true;
            }
            if(create)
            {
                tryExecuteUpdate("DROP TABLE version");
                createDatabaseTable(sqlite_tb_version);
            }
        }
        else
        {
            createDatabaseTable(sqlite_tb_version);
        }
    }
    return true;
}

bool DbModel::checkTableAdresa()
{
    bool result = true;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("adresa"))
        {
            wxArrayString arr = tableColumns("adresa");
            if(!hasString(arr, "id"))
            {
                result = false;
                tryExecuteUpdate("DROP TABLE adresa");
                createDatabaseTable(sqlite_tb_adresa);
                return result;
            }
            if(!hasString(arr, "ulice"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE adresa ADD COLUMN ulice TEXT");
            }
            if(!hasString(arr, "cp"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE adresa ADD COLUMN cp TEXT");
            }
            if(!hasString(arr, "misto"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE adresa ADD COLUMN misto TEXT");
            }
            if(!hasString(arr, "psc"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE adresa ADD COLUMN psc TEXT");
            }
            if(!hasString(arr, "zeme"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE adresa ADD COLUMN zeme TEXT");
            }
            if(!hasTableId("adresa", "id", 0))
            {
                result = false;
                createDatabaseTable(sqlite_tb_adresa, true);
            }
        }
        else
        {
            result = false;
            createDatabaseTable(sqlite_tb_adresa);
        }
    }
    return result;
}

bool DbModel::checkTableKontakt()
{
    bool result = true;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        bool copy = false;
        if(m_db.TableExists("kontakt"))
        {
            wxArrayString arr = tableColumns("kontakt");
            if(!hasString(arr, "id"))
            {
                result = false;
                tryExecuteUpdate("DROP TABLE kontakt");
                createDatabaseTable(sqlite_tb_kontakt);
                return result;
            }
            if(!hasString(arr, "nazev"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE kontakt ADD COLUMN nazev TEXT");
            }
            if(!hasString(arr, "jmeno"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE kontakt ADD COLUMN jmeno TEXT");
                tryExecuteUpdate("UPDATE kontakt SET jmeno=nazev");
            }
            if(!hasString(arr, "ic"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE kontakt ADD COLUMN ic TEXT");
            }
            if(!hasString(arr, "dic"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE kontakt ADD COLUMN dic TEXT");
            }
            if(!hasString(arr, "email"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE kontakt ADD COLUMN email TEXT");
            }
            if(!hasString(arr, "public"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE kontakt ADD COLUMN public INTEGER");
            }
            if(!hasString(arr, "adresaid"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE kontakt ADD COLUMN adresaid INTEGER NOT NULL DEFAULT 0");
            }
            arr = tableKeys("kontakt");
            if(!hasString(arr, "adresaid|adresa|id|CASCADE|NO ACTION"))
            {
                copy = true;
                result = false;
            }
            if(copy)
            {
                tryExecuteUpdate("CREATE TEMPORARY TABLE kontakt_t (id INTEGER PRIMARY KEY AUTOINCREMENT, nazev TEXT, ic TEXT, dic TEXT, email TEXT, public INTEGER, adresaid INTEGER NOT NULL)");
                if(hasSameColumns("kontakt", "kontakt_t"))
                {
                    tryExecuteUpdate("INSERT INTO kontakt_t SELECT id,nazev,ic,dic,email,public,adresaid FROM kontakt");
                    tryExecuteUpdate("DROP TABLE kontakt");
                    tryExecuteUpdate(sqlite_tb_kontakt[0]);
                    tryExecuteUpdate("INSERT INTO kontakt SELECT * FROM kontakt_t");
                }
                tryExecuteUpdate("DROP TABLE kontakt_t");
            }
            if(!hasTableId("kontakt", "id", 0))
            {
                result = false;
                createDatabaseTable(sqlite_tb_kontakt, true);
            }
            wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT adresaid FROM kontakt");
            while(rs.NextRow())
            {
                if(!hasTableId("adresa", "id", rs.GetInt(0)))
                {
                    result = false;
                    tryExecuteUpdate(wxString::Format("INSERT INTO adresa (id) VALUES (%d)", rs.GetInt(0)));
                }
            }
            rs.Finalize();
        }
        else
        {
            result = false;
            createDatabaseTable(sqlite_tb_kontakt);
        }
    }
    return result;
}

bool DbModel::checkTableFirma()
{
    bool result = true;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        bool copy = false;
        if(m_db.TableExists("firma"))
        {
            wxArrayString arr = tableColumns("firma");
            if(!hasString(arr, "nazev"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE firma ADD COLUMN nazev TEXT");
            }
            if(!hasString(arr, "ic"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE firma ADD COLUMN ic TEXT");
            }
            if(!hasString(arr, "dic"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE firma ADD COLUMN dic TEXT");
            }
            if(!hasString(arr, "registrace"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE firma ADD COLUMN registrace TEXT");
            }
            if(!hasString(arr, "platce"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE firma ADD COLUMN platce INTEGER");
                tryExecuteUpdate("UPDATE firma SET platce=0");
            }
            if(!hasString(arr, "pausal"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE firma ADD COLUMN pausal INTEGER");
                tryExecuteUpdate("UPDATE firma SET pausal=0");
            }
            if(!hasString(arr, "telefon"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE firma ADD COLUMN telefon TEXT");
            }
            if(!hasString(arr, "email"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE firma ADD COLUMN email TEXT");
            }
            if(!hasString(arr, "www"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE firma ADD COLUMN www TEXT");
            }
            if(!hasString(arr, "logo"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE firma ADD COLUMN logo TEXT");
            }
            if(!hasString(arr, "razitko"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE firma ADD COLUMN razitko TEXT");
            }
            if(!hasTableId("adresa", "id", 1))
            {
                result = false;
                tryExecuteUpdate("INSERT INTO adresa (id) VALUES (1)");
            }
            if(!hasString(arr, "adresaid"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE firma ADD COLUMN adresaid INTEGER NOT NULL DEFAULT 1");
            }
            arr = tableKeys("firma");
            if(!hasString(arr, "adresaid|adresa|id|CASCADE|NO ACTION"))
            {
                copy = true;
                result = false;
            }
            if(copy)
            {
                tryExecuteUpdate("CREATE TEMPORARY TABLE firma_t (nazev TEXT, ic TEXT, dic TEXT, registrace TEXT, platce INTEGER, pausal INTEGER, telefon TEXT,\
                                   email TEXT, www TEXT, logo TEXT, razitko TEXT, adresaid INTEGER NOT NULL)");
                if(hasSameColumns("firma", "firma_t"))
                {
                    tryExecuteUpdate("INSERT INTO firma_t SELECT nazev,id,dic,registrace,platce,pausal,telefon,email,www,logo,razitko,adresaid FROM firma");
                    tryExecuteUpdate("DROP TABLE firma");
                    tryExecuteUpdate(sqlite_tb_firma[0]);
                    tryExecuteUpdate("INSERT INTO firma SELECT * FROM firma_t");
                }
                tryExecuteUpdate("DROP TABLE firma_t");
            }
            if(!tableRows("firma"))
            {
                result = false;
                wxString firma = "ABC_0";
                int i = 1;
                while(dxsettings.firmaExists(firma))
                {
                    firma = wxString::Format("ABC_%d", i);
                    i++;
                }
                tryExecuteUpdate(wxString::Format("INSERT INTO firma (nazev,ic,adresaid,platce,pausal) VALUES('%s','%s',1,0,0)", firma.Before('_'), firma.After('_')));
                m_firma.setNazev(firma.Before('_'));
                m_firma.setIc(firma.After('_'));
            }
        }
        else
        {
            result = false;
            createDatabaseTable(sqlite_tb_firma);
        }
    }
    return result;
}

bool DbModel::checkTableObdobi()
{
    bool result = true;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("obdobi"))
        {
            wxArrayString arr = tableColumns("obdobi");
            if(!hasString(arr, "rok"))
            {
                result = false;
                tryExecuteUpdate("DROP TABLE obdobi");
                createDatabaseTable(sqlite_tb_obdobi);
                tryExecuteUpdate(wxString::Format("INSERT INTO obdobi (rok,current) VALUES (%d,1)", wxDateTime::Today().GetYear()));
                return result;
            }
            if(!hasString(arr, "current"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE obdobi ADD COLUMN current INTEGER NOT NULL DEFAULT 0");
                tryExecuteUpdate(wxString::Format("UPDATE obdobi SET current=1 WHERE rok=%d", wxDateTime::Today().GetYear()));
            }
            if(!tableRows("obdobi"))
            {
                result = false;
                tryExecuteUpdate(wxString::Format("INSERT INTO obdobi (rok,current) VALUES (%d,1)", wxDateTime::Today().GetYear()));
            }
            bool current = false;
            wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT current FROM obdobi");
            while(rs.NextRow())
            {
                if(rs.GetBool(0))
                {
                    current = true;
                    break;
                }
            }
            rs.Finalize();
            if(!current)
            {
                tryExecuteUpdate(wxString::Format("UPDATE obdobi SET current=1 WHERE rok=%d", m_db.ExecuteScalar("SELECT max(rok) FROM obdobi")));
            }
        }
        else
        {
            result = false;
            createDatabaseTable(sqlite_tb_obdobi);
        }
    }
    return result;
}

bool DbModel::checkTableDphsazba()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("dphsazba"))
        {
            bool create = false;
            wxArrayString arr = tableColumns("dphsazba");
            if(!hasString(arr, "id"))
            {
                create = true;
            }
            if(!hasString(arr, "sazba"))
            {
                create = true;
            }
            if(!hasString(arr, "rok"))
            {
                create = true;
            }
            if(!hasString(arr, "zakladni"))
            {
                create = true;
            }
            if(tableRows("dphsazba") != rowsForInsertToTable(sqlite_tb_dphsazba))
            {
                create = true;
            }
            if(create)
            {
                tryExecuteUpdate("DROP TABLE dphsazba");
                createDatabaseTable(sqlite_tb_dphsazba);
            }
        }
        else
        {
            createDatabaseTable(sqlite_tb_dphsazba);
        }
    }
    return true;
}

bool DbModel::checkTableDphdruh()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("dphdruh"))
        {
            bool create = false;
            wxArrayString arr = tableColumns("dphdruh");
            if(!hasString(arr, "id"))
            {
                create = true;
            }
            if(!hasString(arr, "zkratka"))
            {
                create = true;
            }
            if(!hasString(arr, "popis"))
            {
                create = true;
            }
            if(!hasString(arr, "radky"))
            {
                create = true;
            }
            if(!hasString(arr, "plneni"))
            {
                create = true;
            }
            if(!hasString(arr, "vydaj"))
            {
                create = true;
            }
            if(!hasString(arr, "nabizet"))
            {
                create = true;
            }
            if(!hasString(arr, "kpp"))
            {
                create = true;
            }
            if(!hasString(arr, "krp"))
            {
                create = true;
            }
            if(!hasString(arr, "od"))
            {
                create = true;
            }
            if(!hasString(arr, "do"))
            {
                create = true;
            }
            if(!hasString(arr, "kraceny"))
            {
                create = true;
            }
            if(!hasString(arr, "kh"))
            {
                create = true;
            }
            if(tableRows("dphdruh") != rowsForInsertToTable(sqlite_tb_dphdruh))
            {
                create = true;
            }
            if(create)
            {
                tryExecuteUpdate("DROP TABLE dphdruh");
                createDatabaseTable(sqlite_tb_dphdruh);
            }
        }
        else
        {
            createDatabaseTable(sqlite_tb_dphdruh);
        }
    }
    return true;
}

bool DbModel::checkTableMena()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("mena"))
        {
            bool create = false;
            wxArrayString arr = tableColumns("mena");
            if(!hasString(arr, "id"))
            {
                create = true;
            }
            if(!hasString(arr, "kod"))
            {
                create = true;
            }
            if(!hasString(arr, "nazev"))
            {
                create = true;
            }
            if(!hasString(arr, "od"))
            {
                create = true;
            }
            if(!hasString(arr, "do"))
            {
                create = true;
            }
            if(tableRows("mena") != rowsForInsertToTable(sqlite_tb_mena))
            {
                create = true;
            }
            if(create)
            {
                tryExecuteUpdate("DROP TABLE mena");
                createDatabaseTable(sqlite_tb_mena);
            }
        }
        else
        {
            createDatabaseTable(sqlite_tb_mena);
        }
    }
    return true;
}

bool DbModel::checkTableKurz()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("kurz"))
        {
            bool create = false;
            wxArrayString arr = tableColumns("kurz");
            if(!hasString(arr, "id"))
            {
                create = true;
            }
            if(!hasString(arr, "datum"))
            {
                create = true;
            }
            if(!hasString(arr, "hodnota"))
            {
                create = true;
            }
            if(!hasString(arr, "kod"))
            {
                create = true;
            }
            if(!hasString(arr, "jednotny"))
            {
                create = true;
            }
            if(tableRows("kurz") < rowsForInsertToTable(sqlite_tb_kurz))
            {
                create = true;
            }
            if(create)
            {
                tryExecuteUpdate("DROP TABLE kurz");
                createDatabaseTable(sqlite_tb_kurz);
            }
        }
        else
        {
            createDatabaseTable(sqlite_tb_kurz);
        }
    }
    return true;
}

bool DbModel::checkTablePokladna()
{
    bool result = true;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("pokladna"))
        {
            bool create = false;
            wxArrayString arr = tableColumns("pokladna");
            if(!hasString(arr, "id"))
            {
                result = false;
                create = true;
            }
            if(!hasString(arr, "zkratka"))
            {
                result = false;
                create = true;
            }
            if(!hasString(arr, "popis"))
            {
                result = false;
                create = true;
            }
            if(!hasString(arr, "menaid"))
            {
                result = false;
                create = true;
            }
            arr = tableKeys("pokladna");
            if(!hasString(arr, "menaid|mena|id|CASCADE|NO ACTION"))
            {
                result = false;
                create = true;
            }
            if(!tableRows("pokladna"))
            {
                result = false;
                create = true;
            }
            if(create)
            {
                tryExecuteUpdate("DROP TABLE pokladna");
                createDatabaseTable(sqlite_tb_pokladna);
            }
        }
        else
        {
            result = false;
            createDatabaseTable(sqlite_tb_pokladna);
        }
    }
    return result;
}

bool DbModel::checkTableBanka()
{
    bool result = true;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("banka"))
        {
            bool create = false;
            bool iban = false;
            wxArrayString arr = tableColumns("banka");
            if(arr.Index("id")==wxNOT_FOUND)
            {
                result = false;
                create = true;
            }
            if(arr.Index("zkratka")==wxNOT_FOUND)
            {
                result = false;
                create = true;
            }
            if(arr.Index("popis")==wxNOT_FOUND)
            {
                result = false;
                create = true;
            }
            if(arr.Index("cislouctu")==wxNOT_FOUND)
            {
                result = false;
                create = true;
            }
            if(arr.Index("kodbanky")==wxNOT_FOUND)
            {
                result = false;
                create = true;
            }
            if(arr.Index("menaid")==wxNOT_FOUND)
            {
                result = false;
                create = true;
            }
            if(arr.Index("iban")==wxNOT_FOUND)
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE banka ADD COLUMN iban TEXT");
                iban = true;
            }
            if(arr.Index("bic")==wxNOT_FOUND)
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE banka ADD COLUMN bic TEXT");
            }
            arr = tableKeys("banka");
            if(arr.Index("menaid|mena|id|CASCADE|NO ACTION")==wxNOT_FOUND)
            {
                result = false;
                create = true;
            }
            if(!tableRows("banka"))
            {
                result = false;
                create = true;
            }
            if(create)
            {
                tryExecuteUpdate("DROP TABLE banka");
                createDatabaseTable(sqlite_tb_banka);
            }
            if(iban)
            {
                wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT id,cislouctu,kodbanky FROM banka");
                while(rs.NextRow())
                {
                    wxSQLite3Statement st = m_db.PrepareStatement("UPDATE banka SET iban=? WHERE id=?");
                    st.Bind(1, dxutils::iban(rs.GetString(2), rs.GetString(1)));
                    st.Bind(2, rs.GetInt(0));
                    st.ExecuteUpdate();
                    st.Finalize();
                }
                rs.Finalize();
            }
        }
        else
        {
            result = false;
            createDatabaseTable(sqlite_tb_banka);
        }
    }
    return result;
}

bool DbModel::checkTableBankovniustav()
{
    if(m_db.TableExists("bankovniustav"))
    {
        bool create = false;
        wxArrayString arr = tableColumns("bankovniustav");
        if(arr.Index("id")==wxNOT_FOUND)
        {
            create = true;
        }
        if(arr.Index("kod")==wxNOT_FOUND)
        {
            create = true;
        }
        if(arr.Index("nazev")==wxNOT_FOUND)
        {
            create = true;
        }
        if(arr.Index("bic")==wxNOT_FOUND)
        {
            create = true;
        }
        if(arr.Index("aktivni")==wxNOT_FOUND)
        {
            create = true;
        }
        if(tableRows("bankovniustav") != rowsForInsertToTable(sqlite_tb_bankovniustav))
        {
            create = true;
        }
        if(create)
        {
            tryExecuteUpdate("DROP TABLE bankovniustav");
            createDatabaseTable(sqlite_tb_bankovniustav);
        }
    }
    else
    {
        createDatabaseTable(sqlite_tb_bankovniustav);
    }
    return true;
}

bool DbModel::checkTableDeniksloupec()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("deniksloupec"))
        {
            bool create = false;
            wxArrayString arr = tableColumns("deniksloupec");
            if(!hasString(arr, "sloupec"))
            {
                create = true;
            }
            if(!hasString(arr, "popis"))
            {
                create = true;
            }
            if(!hasString(arr, "prijem"))
            {
                create = true;
            }
            if(!hasString(arr, "danovy"))
            {
                create = true;
            }
            if(tableRows("deniksloupec") != rowsForInsertToTable(sqlite_tb_deniksloupec))
            {
                create = true;
            }
            if(create)
            {
                tryExecuteUpdate("DROP TABLE deniksloupec");
                createDatabaseTable(sqlite_tb_deniksloupec);
            }
        }
        else
        {
            createDatabaseTable(sqlite_tb_deniksloupec);
        }
    }
    return true;
}

bool DbModel::checkTablePredkontace()
{
    bool result = true;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("predkontace"))
        {
            bool create = false;
            wxArrayString arr = tableColumns("predkontace");
            if(!hasString(arr, "id"))
            {
                create = true;
                result = false;
            }
            if(!hasString(arr, "zkratka"))
            {
                create = true;
                result = false;
            }
            if(!hasString(arr, "popis"))
            {
                create = true;
                result = false;
            }
            if(!hasString(arr, "prijem"))
            {
                create = true;
                result = false;
            }
            if(!hasString(arr, "deniksloupecid"))
            {
                create = true;
                result = false;
            }
            arr = tableKeys("predkontace");
            if(!hasString(arr, "deniksloupecid|deniksloupec|sloupec|CASCADE|NO ACTION"))
            {
                create = true;
                result = false;
            }
            if(create)
            {
                tryExecuteUpdate("DROP TABLE predkontace");
                createDatabaseTable(sqlite_tb_predkontace);
            }
            if(!tableRows("predkontace"))
            {
                result = false;
                clearTable("predkontace");
                createDatabaseTable(sqlite_tb_predkontace, true);
            }
        }
        else
        {
            result = false;
            createDatabaseTable(sqlite_tb_predkontace);
        }
    }
    return result;
}

bool DbModel::checkTableDenik()
{
    bool result = true;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("denik"))
        {
            bool copy = false;
            wxArrayString arr = tableColumns("denik");
            if(!hasString(arr, "id"))
            {
                result = false;
                tryExecuteUpdate("DROP TABLE denik");
                createDatabaseTable(sqlite_tb_denik);
                return result;
            }
            if(!hasString(arr, "datum"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE denik ADD COLUMN datum TEXT");
            }
            if(!hasString(arr, "cislo"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE denik ADD COLUMN cislo TEXT");
            }
            if(!hasString(arr, "castka"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE denik ADD COLUMN castka REAL");
                tryExecuteUpdate("UPDATE denik SET castka=0");
            }
            if(!hasString(arr, "castkacizi"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE denik ADD COLUMN castkacizi REAL");
                tryExecuteUpdate("UPDATE denik SET castkacizi=0");
            }
            if(!hasString(arr, "kurz"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE denik ADD COLUMN kurz REAL");
                tryExecuteUpdate("UPDATE kurz SET castka=1");
            }
            if(!hasString(arr, "reference"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE denik ADD COLUMN reference TEXT");
            }
            if(!hasString(arr, "pausal"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE denik ADD COLUMN pausal INTEGER");
                tryExecuteUpdate("UPDATE denik SET pausal=0");
            }
            if(!hasString(arr, "menaid"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE denik ADD COLUMN menaid INTEGER");
                tryExecuteUpdate("UPDATE denik SET menaid=1");
            }
            if(!hasString(arr, "predkontaceid"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE denik ADD COLUMN predkontaceid INTEGER");
                int predkontaceid = m_db.ExecuteQuery("SELECT id FROM predkontace").GetInt(0);
                tryExecuteUpdate(wxString::Format("UPDATE denik SET predkontaceid=%d", predkontaceid));
            }
            if(!hasString(arr, "pokladnaid"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE denik ADD COLUMN pokladnaid INTEGER");
                int pokladnaid = m_db.ExecuteQuery("SELECT id FROM pokladna").GetInt(0);
                tryExecuteUpdate(wxString::Format("UPDATE denik SET pokladnaid=%d", pokladnaid));
            }
            if(!hasString(arr, "bankaid"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE denik ADD COLUMN bankaid INTEGER");
            }
            if(!hasString(arr, "vypis"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE denik ADD COLUMN vypis INTEGER");
                tryExecuteUpdate("UPDATE denik SET vypis=-1");
            }
            if(!hasString(arr, "protistranaid"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE denik ADD COLUMN protistranaid INTEGER");
                tryExecuteUpdate("UPDATE denik SET protistranaid=0");
            }
            if(!hasString(arr, "obdobirok"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE denik ADD COLUMN obdobirok INTEGER");
                tryExecuteUpdate(wxString::Format("UPDATE denik SET obdobirok=%d", wxDateTime::Today().GetYear()));
            }
            arr = tableKeys("denik");
            if(!hasString(arr, "menaid|mena|id|CASCADE|NO ACTION") || !hasString(arr, "predkontaceid|predkontace|id|CASCADE|NO ACTION") ||
                    !hasString(arr, "pokladnaid|pokladna|id|CASCADE|NO ACTION") || !hasString(arr, "bankaid|banka|id|CASCADE|NO ACTION") ||
                    !hasString(arr, "protistranaid|kontakt|id|CASCADE|SET DEFAULT") || !hasString(arr, "obdobirok|obdobi|rok|CASCADE|CASCADE"))
            {
                copy = true;
                result = false;
            }
            if(copy)
            {
                tryExecuteUpdate("CREATE TEMPORARY TABLE denik_t (id INTEGER PRIMARY KEY AUTOINCREMENT, datum TEXT, cislo TEXT, popis TEXT, castka REAL,\
                                    castkacizi REAL, kurz REAL, reference TEXT, pausal INTEGER NOT NULL, menaid INTEGER NOT NULL, predkontaceid INTEGER NOT NULL,\
                                    pokladnaid INTEGER, bankaid INTEGER, vypis INTEGER, protistranaid INTEGER NOT NULL DEFAULT 0, obdobirok INTEGER NOT NULL)");
                if(hasSameColumns("denik", "denik_t"))
                {
                    tryExecuteUpdate("INSERT INTO denik_t SELECT id,datum,cislo,popis,castka,castkacizi,kurz,reference,pausal,menaid,predkontaceid,\
                                        pokladnaid,bankaid,vypis,protistranaid,obdobirok FROM denik");
                    tryExecuteUpdate("DROP TABLE denik");
                    tryExecuteUpdate(sqlite_tb_denik[0]);
                    tryExecuteUpdate("INSERT INTO denik SELECT * FROM denik_t");
                }
                tryExecuteUpdate("DROP TABLE denik_t");
            }
            //check referencovane id
            wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT menaid,id FROM denik");
            while(rs.NextRow())
            {
                if(!hasTableId("mena", "id", rs.GetInt(0)))
                {
                    result = false;
                    tryExecuteUpdate(wxString::Format("UPDATE denik SET menaid=1 WHERE id=%d", rs.GetInt(1)));
                }
            }
            rs.Finalize();
            int predkontaceid = m_db.ExecuteQuery("SELECT id FROM predkontace").GetInt(0);
            rs = m_db.ExecuteQuery("SELECT predkontaceid,id FROM denik");
            while(rs.NextRow())
            {
                if(!hasTableId("predkontace", "id", rs.GetInt(0)))
                {
                    result = false;
                    tryExecuteUpdate(wxString::Format("UPDATE denik SET predkontaceid=%d WHERE id=%d", predkontaceid, rs.GetInt(1)));
                }
            }
            rs.Finalize();
            int pokladnaid = m_db.ExecuteQuery("SELECT id FROM pokladna").GetInt(0);
            rs = m_db.ExecuteQuery("SELECT pokladnaid,id FROM denik");
            while(rs.NextRow())
            {
                if(rs.GetInt(0))
                {
                    if(!hasTableId("pokladna", "id", rs.GetInt(0)))
                    {
                        result = false;
                        tryExecuteUpdate(wxString::Format("UPDATE denik SET pokladnaid=%d WHERE id=%d", pokladnaid, rs.GetInt(1)));
                    }
                }
            }
            rs.Finalize();
            int bankaid = m_db.ExecuteQuery("SELECT id FROM banka").GetInt(0);
            rs = m_db.ExecuteQuery("SELECT bankaid,id FROM denik");
            while(rs.NextRow())
            {
                if(rs.GetInt(0))
                {
                    if(!hasTableId("banka", "id", rs.GetInt(0)))
                    {
                        result = false;
                        tryExecuteUpdate(wxString::Format("UPDATE denik SET bankaid=%d WHERE id=%d", bankaid, rs.GetInt(1)));
                    }
                }
            }
            rs.Finalize();
            rs = m_db.ExecuteQuery("SELECT protistranaid,id FROM denik");
            while(rs.NextRow())
            {
                if(!hasTableId("kontakt", "id", rs.GetInt(0)))
                {
                    result = false;
                    tryExecuteUpdate(wxString::Format("UPDATE denik SET protistranaid=0 WHERE id=%d", rs.GetInt(1)));
                }
            }
            rs.Finalize();
            int rok = wxDateTime::Today().GetYear();
            if(!hasTableId("obdobi", "rok", rok))
            {
                tryExecuteUpdate(wxString::Format("INSERT INTO obdobi (rok,current) VALUES (%d,0)", rok));
            }
            rs = m_db.ExecuteQuery("SELECT obdobirok,id FROM denik");
            while(rs.NextRow())
            {
                if(!hasTableId("obdobi", "rok", rs.GetInt(0)))
                {
                    result = false;
                    tryExecuteUpdate(wxString::Format("UPDATE denik SET obdobirok=%d WHERE id=%d", rok, rs.GetInt(1)));
                }
            }
            rs.Finalize();
        }
        else
        {
            result = false;
            createDatabaseTable(sqlite_tb_denik);
        }
    }
    return result;
}

bool DbModel::checkTableRada()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("rada"))
        {
            bool create = false;
            wxArrayString arr = tableColumns("rada");
            if(!hasString(arr, "id"))
            {
                create = true;
            }
            if(!hasString(arr, "popis"))
            {
                create = true;
            }
            if(!hasString(arr, "prefix"))
            {
                create = true;
            }
            if(!hasString(arr, "znaku"))
            {
                create = true;
            }
            if(!hasString(arr, "cislo"))
            {
                create = true;
            }
            if(tableRows("rada") != rowsForInsertToTable(sqlite_tb_rada))
            {
                create = true;
            }
            if(create)
            {
                tryExecuteUpdate("DROP TABLE rada");
                createDatabaseTable(sqlite_tb_rada);
            }
        }
        else
        {
            createDatabaseTable(sqlite_tb_rada);
        }
    }
    return true;
}

bool DbModel::checkTableFatyp()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("fatyp"))
        {
            bool create = false;
            wxArrayString arr = tableColumns("fatyp");
            if(!hasString(arr, "id"))
            {
                create = true;
            }
            if(!hasString(arr, "typ"))
            {
                create = true;
            }
            if(tableRows("fatyp") != rowsForInsertToTable(sqlite_tb_fatyp))
            {
                create = true;
            }
            if(create)
            {
                tryExecuteUpdate("DROP TABLE fatyp");
                createDatabaseTable(sqlite_tb_fatyp);
            }
        }
        else
        {
            createDatabaseTable(sqlite_tb_fatyp);
        }
    }
    return true;
}

bool DbModel::checkTableFauhrada()
{
    bool result = true;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("fauhrada"))
        {
            bool create = false;
            wxArrayString arr = tableColumns("fauhrada");
            if(!hasString(arr, "id"))
            {
                create = true;
                result = false;
            }
            if(!hasString(arr, "uhrada"))
            {
                create = true;
                result = false;
            }
            if(!hasString(arr, "pokladnou"))
            {
                create = true;
                result = false;
            }
            if(!tableRows("fauhrada"))
            {
                create = true;
                result = false;
            }
            if(create)
            {
                tryExecuteUpdate("DROP TABLE fauhrada");
                createDatabaseTable(sqlite_tb_fauhrada);
            }
        }
        else
        {
            result = false;
            createDatabaseTable(sqlite_tb_fauhrada);
        }
    }
    return result;
}

bool DbModel::checkTableFaktura()
{
    bool result = true;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("faktura"))
        {
            bool copy = false;
            wxArrayString arr = tableColumns("faktura");
            if(!hasString(arr, "id"))
            {
                result = false;
                tryExecuteUpdate("DROP TABLE faktura");
                createDatabaseTable(sqlite_tb_faktura);
                return result;
            }
            if(!hasString(arr, "protistranaid"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE faktura ADD COLUMN protistranaid INTEGER");
                tryExecuteUpdate("UPDATE faktura SET protistranaid=0");
            }
            if(!hasString(arr, "popis"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE faktura ADD COLUMN popis TEXT");
            }
            if(!hasString(arr, "celkem"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE faktura ADD COLUMN celkem REAL");
                tryExecuteUpdate("UPDATE faktura SET celkem=0");
            }
            if(!hasString(arr, "cislo"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE faktura ADD COLUMN cislo TEXT");
            }
            if(!hasString(arr, "vs"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE faktura ADD COLUMN vs TEXT");
            }
            if(!hasString(arr, "ss"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE faktura ADD COLUMN ss TEXT");
            }
            if(!hasString(arr, "ks"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE faktura ADD COLUMN ks TEXT");
            }
            if(!hasString(arr, "typid"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE faktura ADD COLUMN typid INTEGER");
                tryExecuteUpdate("UPDATE faktura SET typid=1");
            }
            if(!hasString(arr, "uhradaid"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE faktura ADD COLUMN uhradaid INTEGER");
                int uhradaid = m_db.ExecuteQuery("SELECT id FROM fauhrada").GetInt(0);
                tryExecuteUpdate(wxString::Format("UPDATE faktura SET uhradaid=%d", uhradaid));
            }
            if(!hasString(arr, "pokladnaid"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE faktura ADD COLUMN pokladnaid INTEGER");
                int pokladnaid = m_db.ExecuteQuery("SELECT id FROM pokladna").GetInt(0);
                tryExecuteUpdate(wxString::Format("UPDATE faktura SET pokladnaid=%d", pokladnaid));
            }
            if(!hasString(arr, "bankaid"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE faktura ADD COLUMN bankaid INTEGER");
            }
            if(!hasString(arr, "menaid"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE faktura ADD COLUMN menaid INTEGER");
                tryExecuteUpdate("UPDATE faktura SET menaid=1");
            }
            if(!hasString(arr, "predkontaceid"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE faktura ADD COLUMN predkontaceid INTEGER");
                int predkontaceid = m_db.ExecuteQuery("SELECT id FROM predkontace").GetInt(0);
                tryExecuteUpdate(wxString::Format("UPDATE faktura SET predkontaceid=%d", predkontaceid));
            }
            if(!hasString(arr, "obdobirok"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE faktura ADD COLUMN obdobirok INTEGER");
                tryExecuteUpdate(wxString::Format("UPDATE faktura SET obdobirok=%d", wxDateTime::Today().GetYear()));
            }
            if(!hasString(arr, "vydana"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE faktura ADD COLUMN vydana INTEGER");
                tryExecuteUpdate("UPDATE faktura SET vydana=1");
            }
            if(!hasString(arr, "platce"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE faktura ADD COLUMN platce INTEGER");
                tryExecuteUpdate("UPDATE faktura SET platce=0");
            }
            if(!hasString(arr, "vystaveni"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE faktura ADD COLUMN vystaveni TEXT");
                tryExecuteUpdate(wxString::Format("UPDATE faktura SET vystaveni='%s'", wxDateTime::Today().Format("%Y-%m-%d")));
            }
            if(!hasString(arr, "duzp"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE faktura ADD COLUMN duzp TEXT");
                tryExecuteUpdate(wxString::Format("UPDATE faktura SET duzp='%s'", wxDateTime::Today().Format("%Y-%m-%d")));
            }
            if(!hasString(arr, "dppd"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE faktura ADD COLUMN dppd TEXT");
                tryExecuteUpdate(wxString::Format("UPDATE faktura SET dppd='%s'", wxDateTime::Today().Format("%Y-%m-%d")));
            }
            if(!hasString(arr, "splatnost"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE faktura ADD COLUMN splatnost TEXT");
                tryExecuteUpdate(wxString::Format("UPDATE faktura SET splatnost='%s'", wxDateTime::Today().Format("%Y-%m-%d")));
            }
            if(!hasString(arr, "dphdruhid"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE faktura ADD COLUMN dphdruhid INTEGER");
                tryExecuteUpdate("UPDATE faktura SET dphdruhid=1");
            }
            if(!hasString(arr, "kurz"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE faktura ADD COLUMN kurz REAL");
                tryExecuteUpdate("UPDATE faktura SET kurz=1");
            }
            if(!hasString(arr, "pausal"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE faktura ADD COLUMN pausal REAL");
                tryExecuteUpdate("UPDATE faktura SET pausal=0");
            }
            arr = tableKeys("faktura");
            if(!hasString(arr, "protistranaid|kontakt|id|CASCADE|SET DEFAULT") || !hasString(arr, "typid|fatyp|id|CASCADE|NO ACTION") ||
                    !hasString(arr, "uhradaid|fauhrada|id|CASCADE|NO ACTION") || !hasString(arr, "pokladnaid|pokladna|id|CASCADE|NO ACTION") ||
                    !hasString(arr, "bankaid|banka|id|CASCADE|NO ACTION") || !hasString(arr, "menaid|mena|id|CASCADE|NO ACTION") ||
                    !hasString(arr, "predkontaceid|predkontace|id|CASCADE|NO ACTION") || !hasString(arr, "obdobirok|obdobi|rok|CASCADE|CASCADE") ||
                    !hasString(arr, "dphdruhid|dphdruh|id|CASCADE|NO ACTION"))
            {
                copy = true;
                result = false;
            }
            if(copy)
            {
                tryExecuteUpdate("CREATE TEMPORARY TABLE faktura_t (id INTEGER PRIMARY KEY AUTOINCREMENT,protistranaid INTEGER NOT NULL DEFAULT 0,\
                                   popis TEXT,celkem REAL,cislo TEXT,vs TEXT,ss TEXT,ks TEXT,typid INTEGER NOT NULL,\
                                   uhradaid INTEGER NOT NULL,pokladnaid INTEGER,bankaid INTEGER,menaid INTEGER NOT NULL,\
                                   predkontaceid INTEGER NOT NULL,obdobirok INTEGER NOT NULL,vydana INTEGER,platce INTEGER,vystaveni TEXT,\
                                   duzp TEXT,dppd TEXT,splatnost TEXT,dphdruhid INTEGER,kurz REAL,pausal INTEGER)");
                if(hasSameColumns("faktura", "faktura_t"))
                {
                    tryExecuteUpdate("INSERT INTO faktura_t SELECT id,protistranaid,\
                                       popis,celkem,cislo,vs,ss,ks,typid,\
                                       uhradaid,pokladnaid,bankaid,menaid,\
                                       predkontaceid,obdobirok,vydana,platce,vystaveni,\
                                       duzp,dppd,splatnost,dphdruhid,kurz,pausal FROM faktura");
                    tryExecuteUpdate("DROP TABLE faktura");
                    tryExecuteUpdate(sqlite_tb_faktura[0]);
                    tryExecuteUpdate("INSERT INTO faktura SELECT * FROM faktura_t");
                }
                tryExecuteUpdate("DROP TABLE faktura_t");
            }
            //check kurz
            wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT kurz,id FROM faktura");
            while(rs.NextRow())
            {
                if(wxIsSameDouble(0.0, rs.GetDouble(0)) || rs.GetDouble(0) < 0.0)
                {
                    result = false;
                    tryExecuteUpdate(wxString::Format("UPDATE faktura SET kurz=1 WHERE id=%d", rs.GetInt(1)));
                }
            }
            rs.Finalize();
            //check referencovane id
            rs = m_db.ExecuteQuery("SELECT protistranaid,id FROM faktura");
            while(rs.NextRow())
            {
                if(!hasTableId("kontakt", "id", rs.GetInt(0)))
                {
                    result = false;
                    tryExecuteUpdate(wxString::Format("UPDATE faktura SET protistranaid=0 WHERE id=%d", rs.GetInt(1)));
                }
            }
            rs.Finalize();
            rs = m_db.ExecuteQuery("SELECT typid,id FROM faktura");
            while(rs.NextRow())
            {
                if(!hasTableId("fatyp", "id", rs.GetInt(0)))
                {
                    result = false;
                    tryExecuteUpdate(wxString::Format("UPDATE faktura SET typid=1 WHERE id=%d", rs.GetInt(1)));
                }
            }
            rs.Finalize();
            int uhradaid = m_db.ExecuteQuery("SELECT id FROM fauhrada").GetInt(0);
            rs = m_db.ExecuteQuery("SELECT uhradaid,id FROM faktura");
            while(rs.NextRow())
            {
                if(!hasTableId("fauhrada", "id", rs.GetInt(0)))
                {
                    result = false;
                    tryExecuteUpdate(wxString::Format("UPDATE faktura SET uhradaid=%d WHERE id=%d", uhradaid, rs.GetInt(1)));
                }
            }
            rs.Finalize();
            int pokladnaid = m_db.ExecuteQuery("SELECT id FROM pokladna").GetInt(0);
            rs = m_db.ExecuteQuery("SELECT pokladnaid,id FROM faktura");
            while(rs.NextRow())
            {
                if(rs.GetInt(0))
                {
                    if(!hasTableId("pokladna", "id", rs.GetInt(0)))
                    {
                        result = false;
                        tryExecuteUpdate(wxString::Format("UPDATE faktura SET pokladnaid=%d WHERE id=%d", pokladnaid, rs.GetInt(1)));
                    }
                }
            }
            rs.Finalize();
            int bankaid = m_db.ExecuteQuery("SELECT id FROM banka").GetInt(0);
            rs = m_db.ExecuteQuery("SELECT bankaid,id FROM faktura");
            while(rs.NextRow())
            {
                if(rs.GetInt(0))
                {
                    if(!hasTableId("banka", "id", rs.GetInt(0)))
                    {
                        result = false;
                        tryExecuteUpdate(wxString::Format("UPDATE faktura SET bankaid=%d WHERE id=%d", bankaid, rs.GetInt(1)));
                    }
                }
            }
            rs.Finalize();
            rs = m_db.ExecuteQuery("SELECT menaid,id FROM faktura");
            while(rs.NextRow())
            {
                if(!hasTableId("mena", "id", rs.GetInt(0)))
                {
                    result = false;
                    tryExecuteUpdate(wxString::Format("UPDATE faktura SET menaid=1 WHERE id=%d", rs.GetInt(1)));
                }
            }
            rs.Finalize();
            int predkontaceid = m_db.ExecuteQuery("SELECT id FROM predkontace").GetInt(0);
            rs = m_db.ExecuteQuery("SELECT predkontaceid,id FROM faktura");
            while(rs.NextRow())
            {
                if(!hasTableId("predkontace", "id", rs.GetInt(0)))
                {
                    result = false;
                    tryExecuteUpdate(wxString::Format("UPDATE faktura SET predkontaceid=%d WHERE id=%d", predkontaceid, rs.GetInt(1)));
                }
            }
            rs.Finalize();
            int rok = wxDateTime::Today().GetYear();
            if(!hasTableId("obdobi", "rok", rok))
            {
                tryExecuteUpdate(wxString::Format("INSERT INTO obdobi (rok,current) VALUES (%d,0)", rok));
            }
            rs = m_db.ExecuteQuery("SELECT obdobirok,id FROM faktura");
            while(rs.NextRow())
            {
                if(!hasTableId("obdobi", "rok", rs.GetInt(0)))
                {
                    result = false;
                    tryExecuteUpdate(wxString::Format("UPDATE faktura SET obdobirok=%d WHERE id=%d", rok, rs.GetInt(1)));
                }
            }
            rs.Finalize();
            rs = m_db.ExecuteQuery("SELECT dphdruhid,id,platce,vydana FROM faktura");
            while(rs.NextRow())
            {
                if(rs.GetBool(2))
                {
                    if(!hasTableId("dphdruh", "id", rs.GetInt(0)))
                    {
                        result = false;
                        tryExecuteUpdate(wxString::Format("UPDATE faktura SET dphdruhid=1 WHERE id=%d", rs.GetInt(1)));
                    }
                }
                else
                {
                    if(rs.GetInt(3))
                    {
                        tryExecuteUpdate(wxString::Format("UPDATE faktura SET dphdruhid=80 WHERE id=%d", rs.GetInt(1)));
                    }
                    else
                    {
                        tryExecuteUpdate(wxString::Format("UPDATE faktura SET dphdruhid=79 WHERE id=%d", rs.GetInt(1)));
                    }
                }
            }
            rs.Finalize();
        }
        else
        {
            result = false;
            createDatabaseTable(sqlite_tb_faktura);
        }
    }
    return result;
}

bool DbModel::checkTableZavpoh()
{
    bool result = true;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("zavpoh"))
        {
            bool copy = false;
            wxArrayString arr = tableColumns("zavpoh");
            if(!hasString(arr, "id"))
            {
                result = false;
                tryExecuteUpdate("DROP TABLE zavpoh");
                createDatabaseTable(sqlite_tb_zavpoh);
                return result;
            }
            if(!hasString(arr, "protistranaid"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE zavpoh ADD COLUMN protistranaid INTEGER");
                tryExecuteUpdate("UPDATE zavpoh SET protistranaid=0");
            }
            if(!hasString(arr, "popis"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE zavpoh ADD COLUMN popis TEXT");
            }
            if(!hasString(arr, "celkem"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE zavpoh ADD COLUMN celkem REAL");
                tryExecuteUpdate("UPDATE zavpoh SET celkem=0");
            }
            if(!hasString(arr, "cislo"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE zavpoh ADD COLUMN cislo TEXT");
            }
            if(!hasString(arr, "vs"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE zavpoh ADD COLUMN vs TEXT");
            }
            if(!hasString(arr, "uhradaid"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE zavpoh ADD COLUMN uhradaid INTEGER");
                int uhradaid = m_db.ExecuteQuery("SELECT id FROM fauhrada").GetInt(0);
                tryExecuteUpdate(wxString::Format("UPDATE zavpoh SET uhradaid=%d", uhradaid));
            }
            if(!hasString(arr, "pokladnaid"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE zavpoh ADD COLUMN pokladnaid INTEGER");
                int pokladnaid = m_db.ExecuteQuery("SELECT id FROM pokladna").GetInt(0);
                tryExecuteUpdate(wxString::Format("UPDATE zavpoh SET pokladnaid=%d", pokladnaid));
            }
            if(!hasString(arr, "bankaid"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE zavpoh ADD COLUMN bankaid INTEGER");
            }
            if(!hasString(arr, "menaid"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE zavpoh ADD COLUMN menaid INTEGER");
                tryExecuteUpdate("UPDATE zavpoh SET menaid=1");
            }
            if(!hasString(arr, "predkontaceid"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE zavpoh ADD COLUMN predkontaceid INTEGER");
                int predkontaceid = m_db.ExecuteQuery("SELECT id FROM predkontace").GetInt(0);
                tryExecuteUpdate(wxString::Format("UPDATE zavpoh SET predkontaceid=%d", predkontaceid));
            }
            if(!hasString(arr, "obdobirok"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE zavpoh ADD COLUMN obdobirok INTEGER");
                tryExecuteUpdate(wxString::Format("UPDATE zavpoh SET obdobirok=%d", wxDateTime::Today().GetYear()));
            }
            if(!hasString(arr, "zavazek"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE zavpoh ADD COLUMN zavazek INTEGER");
                tryExecuteUpdate("UPDATE zavpoh SET zavazek=1");
            }
            if(!hasString(arr, "platce"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE zavpoh ADD COLUMN platce INTEGER");
                tryExecuteUpdate("UPDATE zavpoh SET platce=0");
            }
            if(!hasString(arr, "vystaveni"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE zavpoh ADD COLUMN vystaveni TEXT");
                tryExecuteUpdate(wxString::Format("UPDATE zavpoh SET vystaveni='%s'", wxDateTime::Today().Format("%Y-%m-%d")));
            }
            if(!hasString(arr, "duzp"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE zavpoh ADD COLUMN duzp TEXT");
                tryExecuteUpdate(wxString::Format("UPDATE zavpoh SET duzp='%s'", wxDateTime::Today().Format("%Y-%m-%d")));
            }
            if(!hasString(arr, "dppd"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE zavpoh ADD COLUMN dppd TEXT");
                tryExecuteUpdate(wxString::Format("UPDATE zavpoh SET dppd='%s'", wxDateTime::Today().Format("%Y-%m-%d")));
            }
            if(!hasString(arr, "splatnost"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE zavpoh ADD COLUMN splatnost TEXT");
                tryExecuteUpdate(wxString::Format("UPDATE zavpoh SET splatnost='%s'", wxDateTime::Today().Format("%Y-%m-%d")));
            }
            if(!hasString(arr, "dphdruhid"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE zavpoh ADD COLUMN dphdruhid INTEGER");
                tryExecuteUpdate("UPDATE zavpoh SET dphdruhid=1");
            }
            if(!hasString(arr, "kurz"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE zavpoh ADD COLUMN kurz REAL");
                tryExecuteUpdate("UPDATE zavpoh SET kurz=1");
            }
            if(!hasString(arr, "pausal"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE zavpoh ADD COLUMN pausal REAL");
                tryExecuteUpdate("UPDATE zavpoh SET pausal=0");
            }
            arr = tableKeys("zavpoh");
            if(!hasString(arr, "protistranaid|kontakt|id|CASCADE|SET DEFAULT") || !hasString(arr, "uhradaid|fauhrada|id|CASCADE|NO ACTION") || !hasString(arr, "pokladnaid|pokladna|id|CASCADE|NO ACTION") ||
                    !hasString(arr, "bankaid|banka|id|CASCADE|NO ACTION") || !hasString(arr, "menaid|mena|id|CASCADE|NO ACTION") ||
                    !hasString(arr, "predkontaceid|predkontace|id|CASCADE|NO ACTION") || !hasString(arr, "obdobirok|obdobi|rok|CASCADE|CASCADE") ||
                    !hasString(arr, "dphdruhid|dphdruh|id|CASCADE|NO ACTION"))
            {
                copy = true;
                result = false;
            }
            if(copy)
            {
                tryExecuteUpdate("CREATE TEMPORARY TABLE zavpoh_t (id INTEGER PRIMARY KEY AUTOINCREMENT,protistranaid INTEGER DEFAULT 0,\
                                   popis TEXT,celkem REAL,cislo TEXT,vs TEXT,uhradaid INTEGER NOT NULL,pokladnaid INTEGER,bankaid INTEGER,\
                                   menaid INTEGER NOT NULL,predkontaceid INTEGER NOT NULL,obdobirok INTEGER NOT NULL,\
                                   zavazek INTEGER,platce INTEGER,vystaveni TEXT,duzp TEXT,dppd TEXT,splatnost TEXT,dphdruhid INTEGER,kurz REAL,pausal INTEGER)");
                if(hasSameColumns("zavpoh", "zavpoh_t"))
                {
                    tryExecuteUpdate("INSERT INTO zavpoh_t SELECT id,protistranaid,\
                                       popis,celkem,cislo,vs,uhradaid,pokladnaid,bankaid,\
                                       menaid,predkontaceid,obdobirok,\
                                       zavazek,platce,vystaveni,duzp,dppd,splatnost,dphdruhid,kurz,pausal FROM zavpoh");
                    tryExecuteUpdate("DROP TABLE zavpoh");
                    tryExecuteUpdate(sqlite_tb_zavpoh[0]);
                    tryExecuteUpdate("INSERT INTO zavpoh SELECT * FROM zavpoh_t");
                }
                tryExecuteUpdate("DROP TABLE zavpoh_t");
            }
            //check kurz
            wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT kurz,id FROM zavpoh");
            while(rs.NextRow())
            {
                if(wxIsSameDouble(0.0, rs.GetDouble(0)) || rs.GetDouble(0) < 0.0)
                {
                    result = false;
                    tryExecuteUpdate(wxString::Format("UPDATE zavpoh SET kurz=1 WHERE id=%d", rs.GetInt(1)));
                }
            }
            rs.Finalize();
            //check referencovane id
            rs = m_db.ExecuteQuery("SELECT protistranaid,id FROM zavpoh");
            while(rs.NextRow())
            {
                if(!hasTableId("kontakt", "id", rs.GetInt(0)))
                {
                    result = false;
                    tryExecuteUpdate(wxString::Format("UPDATE zavpoh SET protistranaid=0 WHERE id=%d", rs.GetInt(1)));
                }
            }
            rs.Finalize();
            int uhradaid = m_db.ExecuteQuery("SELECT id FROM fauhrada").GetInt(0);
            rs = m_db.ExecuteQuery("SELECT uhradaid,id FROM zavpoh");
            while(rs.NextRow())
            {
                if(!hasTableId("fauhrada", "id", rs.GetInt(0)))
                {
                    result = false;
                    tryExecuteUpdate(wxString::Format("UPDATE zavpoh SET uhradaid=%d WHERE id=%d", uhradaid, rs.GetInt(1)));
                }
            }
            rs.Finalize();
            int pokladnaid = m_db.ExecuteQuery("SELECT id FROM pokladna").GetInt(0);
            rs = m_db.ExecuteQuery("SELECT pokladnaid,id FROM zavpoh");
            while(rs.NextRow())
            {
                if(rs.GetInt(0))
                {
                    if(!hasTableId("pokladna", "id", rs.GetInt(0)))
                    {
                        result = false;
                        tryExecuteUpdate(wxString::Format("UPDATE zavpoh SET pokladnaid=%d WHERE id=%d", pokladnaid, rs.GetInt(1)));
                    }
                }
            }
            rs.Finalize();
            int bankaid = m_db.ExecuteQuery("SELECT id FROM banka").GetInt(0);
            rs = m_db.ExecuteQuery("SELECT bankaid,id FROM zavpoh");
            while(rs.NextRow())
            {
                if(rs.GetInt(0))
                {
                    if(!hasTableId("banka", "id", rs.GetInt(0)))
                    {
                        result = false;
                        tryExecuteUpdate(wxString::Format("UPDATE zavpoh SET bankaid=%d WHERE id=%d", bankaid, rs.GetInt(1)));
                    }
                }
            }
            rs.Finalize();
            rs = m_db.ExecuteQuery("SELECT menaid,id FROM zavpoh");
            while(rs.NextRow())
            {
                if(!hasTableId("mena", "id", rs.GetInt(0)))
                {
                    result = false;
                    tryExecuteUpdate(wxString::Format("UPDATE zavpoh SET menaid=1 WHERE id=%d", rs.GetInt(1)));
                }
            }
            rs.Finalize();
            int predkontaceid = m_db.ExecuteQuery("SELECT id FROM predkontace").GetInt(0);
            rs = m_db.ExecuteQuery("SELECT predkontaceid,id FROM zavpoh");
            while(rs.NextRow())
            {
                if(!hasTableId("predkontace", "id", rs.GetInt(0)))
                {
                    result = false;
                    tryExecuteUpdate(wxString::Format("UPDATE zavpoh SET predkontaceid=%d WHERE id=%d", predkontaceid, rs.GetInt(1)));
                }
            }
            rs.Finalize();
            int rok = wxDateTime::Today().GetYear();
            if(!hasTableId("obdobi", "rok", rok))
            {
                tryExecuteUpdate(wxString::Format("INSERT INTO obdobi (rok,current) VALUES (%d,0)", rok));
            }
            rs = m_db.ExecuteQuery("SELECT obdobirok,id FROM zavpoh");
            while(rs.NextRow())
            {
                if(!hasTableId("obdobi", "rok", rs.GetInt(0)))
                {
                    result = false;
                    tryExecuteUpdate(wxString::Format("UPDATE zavpoh SET obdobirok=%d WHERE id=%d", rok, rs.GetInt(1)));
                }
            }
            rs.Finalize();
            rs = m_db.ExecuteQuery("SELECT dphdruhid,id,platce,zavazek FROM zavpoh");
            while(rs.NextRow())
            {
                if(rs.GetBool(2))
                {
                    if(!hasTableId("dphdruh", "id", rs.GetInt(0)))
                    {
                        result = false;
                        tryExecuteUpdate(wxString::Format("UPDATE zavpoh SET dphdruhid=1 WHERE id=%d", rs.GetInt(1)));
                    }
                }
                else
                {
                    if(rs.GetInt(3))
                    {
                        tryExecuteUpdate(wxString::Format("UPDATE zavpoh SET dphdruhid=79 WHERE id=%d", rs.GetInt(1)));
                    }
                    else
                    {
                        tryExecuteUpdate(wxString::Format("UPDATE zavpoh SET dphdruhid=80 WHERE id=%d", rs.GetInt(1)));
                    }
                }
            }
            rs.Finalize();
        }
        else
        {
            result = false;
            createDatabaseTable(sqlite_tb_zavpoh);
        }
    }
    return result;
}

bool DbModel::checkTableFapolozka()
{
    bool result = true;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("fapolozka"))
        {
            bool copy = false;
            wxArrayString arr = tableColumns("fapolozka");
            if(!hasString(arr, "id"))
            {
                result = false;
                tryExecuteUpdate("DROP TABLE fapolozka");
                createDatabaseTable(sqlite_tb_fapolozka);
                return result;
            }
            if(!hasString(arr, "popis"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE fapolozka ADD COLUMN popis TEXT");
            }
            if(!hasString(arr, "mj"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE fapolozka ADD COLUMN mj TEXT");
            }
            if(!hasString(arr, "pocet"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE fapolozka ADD COLUMN pocet REAL");
                tryExecuteUpdate("UPDATE fapolozka SET pocet=0");
            }
            if(!hasString(arr, "cenamj"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE fapolozka ADD COLUMN cenamj REAL");
                tryExecuteUpdate("UPDATE fapolozka SET cenamj=0");
            }
            if(!hasString(arr, "dph"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE fapolozka ADD COLUMN dph REAL");
                tryExecuteUpdate("UPDATE fapolozka SET dph=0");
            }
            if(!hasString(arr, "celkem"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE fapolozka ADD COLUMN celkem REAL");
                tryExecuteUpdate("UPDATE fapolozka SET celkem=0");
                tryExecuteUpdate("UPDATE fapolozka SET pocet=0");
                tryExecuteUpdate("UPDATE fapolozka SET cenamj=0");
                tryExecuteUpdate("UPDATE fapolozka SET dph=0");
            }
            if(!hasString(arr, "dph"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE fapolozka ADD COLUMN bezdph INTEGER");
                tryExecuteUpdate("UPDATE fapolozka SET bezdph=0");
            }
            if(!hasString(arr, "sazbaid"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE fapolozka ADD COLUMN sazbaid INTEGER");
                tryExecuteUpdate("UPDATE fapolozka SET sazbaid=0");
            }
            if(!hasString(arr, "fakturaid"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE fapolozka ADD COLUMN fakturaid INTEGER");
            }
            if(!hasString(arr, "zavpohid"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE fapolozka ADD COLUMN zavpohid INTEGER");
            }
            if(!hasString(arr, "zasobaid"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE fapolozka ADD COLUMN zasobaid INTEGER");
            }
            if(!hasString(arr, "zaokrouhleni"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE fapolozka ADD COLUMN zaokrouhleni INTEGER");
                tryExecuteUpdate("UPDATE fapolozka SET zaokrouhleni=0");
            }
            arr = tableKeys("fapolozka");
            if(!hasString(arr, "fakturaid|faktura|id|CASCADE|CASCADE") || !hasString(arr, "zavpohid|zavpoh|id|CASCADE|CASCADE") ||
                    !hasString(arr, "sazbaid|dphsazba|id|CASCADE|CASCADE") || !hasString(arr, "zasobaid|zasoba|id|CASCADE|SET NULL"))
            {
                copy = true;
                result = false;
            }
            if(copy)
            {
                tryExecuteUpdate("CREATE TEMPORARY TABLE fapolozka_t (id INTEGER PRIMARY KEY AUTOINCREMENT, popis TEXT, mj TEXT, pocet REAL, cenamj REAL,\
                                   sazbaid INTEGER, dph REAL, celkem REAL, bezdph INTEGER, fakturaid INTEGER, zavpohid INTEGER, zasobaid INTEGER)");
                if(hasSameColumns("fapolozka", "fapolozka_t"))
                {
                    tryExecuteUpdate("INSERT INTO fapolozka_t SELECT id, popis, mj, pocet, cenamj,\
                                       sazbaid, dph, celkem, bezdph, fakturaid, zavpohid, zasobaid FROM fapolozka");
                    tryExecuteUpdate("DROP TABLE fapolozka");
                    tryExecuteUpdate(sqlite_tb_fapolozka[0]);
                    tryExecuteUpdate("INSERT INTO fapolozka SELECT * FROM fapolozka_t");
                }
                tryExecuteUpdate("DROP TABLE fapolozka_t");
            }
            //check referencovane id
            wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT sazbaid,id FROM fapolozka");
            while(rs.NextRow())
            {
                if(!hasTableId("dphsazba", "id", rs.GetInt(0)))
                {
                    result = false;
                    tryExecuteUpdate(wxString::Format("UPDATE fapolozka SET sazbaid=0 WHERE id=%d", rs.GetInt(1)));
                }
            }
            rs.Finalize();
            rs = m_db.ExecuteQuery("SELECT fakturaid,id FROM fapolozka");
            while(rs.NextRow())
            {
                if(rs.GetInt(0))
                {
                    if(!hasTableId("faktura", "id", rs.GetInt(0)))
                    {
                        result = false;
                        tryExecuteUpdate(wxString::Format("DELETE FROM fapolozka WHERE id=%d", rs.GetInt(1)));
                    }
                }
            }
            rs.Finalize();
            rs = m_db.ExecuteQuery("SELECT zavpohid,id FROM fapolozka");
            while(rs.NextRow())
            {
                if(rs.GetInt(0))
                {
                    if(!hasTableId("zavpoh", "id", rs.GetInt(0)))
                    {
                        result = false;
                        tryExecuteUpdate(wxString::Format("DELETE FROM fapolozka WHERE id=%d", rs.GetInt(1)));
                    }
                }
            }
            rs.Finalize();
            rs = m_db.ExecuteQuery("SELECT zasobaid,id FROM fapolozka");
            while(rs.NextRow())
            {
                if(rs.GetInt(0))
                {
                    if(!hasTableId("zasoba", "id", rs.GetInt(0)))
                    {
                        tryExecuteUpdate(wxString::Format("UPDATE fapolozka SET zasobaid=null WHERE id=%d", rs.GetInt(1)));
                    }
                }
            }
            rs.Finalize();
            //prepocitani hodnot
            if(!result)
            {
                rs = m_db.ExecuteQuery("SELECT sazbaid,id,pocet,cenamj,bezdph,fakturaid,zavpohid,celkem FROM fapolozka");
                while(rs.NextRow())
                {
                    bool platce = false;
                    wxDateTime duzp;
                    if(hasFaktura(rs.GetInt(5)))
                    {
                        platce = m_db.ExecuteScalar(wxString::Format("SELECT platce FROM faktura WHERE id=%d", rs.GetInt(5)));
                        duzp = getFakturaDuzp(rs.GetInt(5));
                    }
                    if(hasZavPoh(rs.GetInt(6)))
                    {
                        platce = m_db.ExecuteScalar(wxString::Format("SELECT platce FROM zavpoh WHERE id=%d", rs.GetInt(6)));
                        duzp = getZavPohDuzp(rs.GetInt(6));
                    }
                    int sazba = m_db.ExecuteScalar(wxString::Format("SELECT sazba FROM dphsazba WHERE id=%d", rs.GetInt(0)));
                    double pocet = rs.GetDouble(2);
                    double cenamj = rs.GetDouble(3);
                    bool bezdph = rs.GetBool(4);
                    double dph = 0.0;
                    double celkem = 0.0;
                    if(platce)
                    {
                        if(bezdph)
                            dph = pocet*cenamj*(sazba/100.0);
                        else
                        {
                            if(duzp.IsLaterThan(wxDateTime(31,wxDateTime::Mar,2019))) dph = pocet*cenamj*(sazba/(100.0+sazba));
                            else dph = pocet*cenamj*round4(sazba/(100.0+sazba));
                        }
                        celkem = (bezdph? dph : 0.0) + cenamj*pocet;
                    }
                    else
                    {
                        celkem = cenamj*pocet;
                    }
                    if(!dxutils::isSameDouble(rs.GetDouble(7), celkem))
                    {
                        try
                        {
                            wxSQLite3Statement st = m_db.PrepareStatement("UPDATE fapolozka SET pocet=?,cenamj=?,dph=?,celkem=? WHERE id=?");
                            st.Bind(1, pocet);
                            st.Bind(2, cenamj);
                            st.Bind(3, dph);
                            st.Bind(4, celkem);
                            st.Bind(5, rs.GetInt(1));
                            st.ExecuteUpdate();
                            st.Finalize();
                        }
                        catch(const wxSQLite3Exception& e)
                        {
                            wxLogDebug(wxString::Format("Chyba v checkTableFapolozka: %s", e.GetMessage()));
                        }
                    }
                }
                rs.Finalize();
            }
            //celkem ve fakturach, zavazcich a pohledavkach
            rs = m_db.ExecuteQuery("SELECT faktura.id,faktura.celkem,SUM(fapolozka.celkem) FROM faktura JOIN fapolozka ON fapolozka.fakturaid = faktura.id GROUP BY faktura.id");
            while(rs.NextRow())
            {
                if(!dxutils::isSameDouble(rs.GetDouble(1), rs.GetDouble(2)))
                {
                    try
                    {
                        wxSQLite3Statement st = m_db.PrepareStatement("UPDATE faktura SET celkem=? WHERE id=?");
                        st.Bind(1, rs.GetDouble(2));
                        st.Bind(2, rs.GetInt(0));
                        st.ExecuteUpdate();
                        st.Finalize();
                    }
                    catch(const wxSQLite3Exception& e)
                    {
                        wxLogDebug(wxString::Format("Chyba v checkTableFapolozka: %s", e.GetMessage()));
                    }
                }
            }
            rs.Finalize();
            rs = m_db.ExecuteQuery("SELECT zavpoh.id,zavpoh.celkem,SUM(fapolozka.celkem) FROM zavpoh JOIN fapolozka ON fapolozka.zavpohid = zavpoh.id GROUP BY zavpoh.id");
            while(rs.NextRow())
            {
                if(!dxutils::isSameDouble(rs.GetDouble(1), rs.GetDouble(2)))
                {
                    try
                    {
                        wxSQLite3Statement st = m_db.PrepareStatement("UPDATE zavpoh SET celkem=? WHERE id=?");
                        st.Bind(1, rs.GetDouble(2));
                        st.Bind(2, rs.GetInt(0));
                        st.ExecuteUpdate();
                        st.Finalize();
                    }
                    catch(const wxSQLite3Exception& e)
                    {
                        wxLogDebug(wxString::Format("Chyba v checkTableFapolozka: %s", e.GetMessage()));
                    }
                }
            }
            rs.Finalize();
        }
        else
        {
            result = false;
            createDatabaseTable(sqlite_tb_fapolozka);
            tryExecuteUpdate("UPDATE faktura SET celkem=0");
            tryExecuteUpdate("UPDATE zavpoh SET celkem=0");
        }
    }
    return result;
}

bool DbModel::checkTableUhrada()
{
    bool result = true;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("uhrada"))
        {
            wxArrayString arr = tableColumns("uhrada");
            if(!hasString(arr, "castka"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE uhrada ADD COLUMN castka REAL");
                tryExecuteUpdate("UPDATE uhrada SET castka=0");
            }
            if(!hasString(arr, "id") || !hasString(arr, "denikid") || !hasString(arr, "fakturaid")
                    || !hasString(arr, "zavpohid") || !hasString(arr, "obdobirok"))
            {
                result = false;
                tryExecuteUpdate("DROP TABLE uhrada");
                createDatabaseTable(sqlite_tb_uhrada);
            }
            arr = tableKeys("uhrada");
            if(!hasString(arr, "denikid|denik|id|CASCADE|CASCADE") || !hasString(arr, "fakturaid|faktura|id|CASCADE|CASCADE") ||
                    !hasString(arr, "zavpohid|zavpoh|id|CASCADE|CASCADE") || !hasString(arr, "obdobirok|obdobi|rok|CASCADE|CASCADE"))
            {
                result = false;
                tryExecuteUpdate("CREATE TEMPORARY TABLE uhrada_t (id INTEGER PRIMARY KEY AUTOINCREMENT, castka REAL, denikid INTEGER NOT NULL, fakturaid INTEGER,\
                                 zavpohid INTEGER, obdobirok INTEGER NOT NULL)");
                if(hasSameColumns("uhrada", "uhrada_t"))
                {
                    tryExecuteUpdate("INSERT INTO uhrada_t SELECT id, castka, denikid, fakturaid\
                                       zavpohid, obdobirok FROM uhrada");
                    tryExecuteUpdate("DROP TABLE uhrada");
                    tryExecuteUpdate(sqlite_tb_uhrada[0]);
                    tryExecuteUpdate("INSERT INTO uhrada SELECT * FROM uhrada_t");
                }
                tryExecuteUpdate("DROP TABLE uhrada_t");
            }
            //check referencovane id
            wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT denikid,id FROM uhrada");
            while(rs.NextRow())
            {
                if(!hasTableId("denik", "id", rs.GetInt(0)))
                {
                    result = false;
                    tryExecuteUpdate(wxString::Format("DELETE FROM uhrada WHERE id=%d", rs.GetInt(1)));
                }
            }
            rs.Finalize();
            rs = m_db.ExecuteQuery("SELECT fakturaid,id FROM uhrada");
            while(rs.NextRow())
            {
                if(rs.GetInt(0))
                {
                    if(!hasTableId("faktura", "id", rs.GetInt(0)))
                    {
                        result = false;
                        tryExecuteUpdate(wxString::Format("DELETE FROM uhrada WHERE id=%d", rs.GetInt(1)));
                    }
                }
            }
            rs.Finalize();
            rs = m_db.ExecuteQuery("SELECT zavpohid,id FROM uhrada");
            while(rs.NextRow())
            {
                if(rs.GetInt(0))
                {
                    if(!hasTableId("zavpoh", "id", rs.GetInt(0)))
                    {
                        result = false;
                        tryExecuteUpdate(wxString::Format("DELETE FROM uhrada WHERE id=%d", rs.GetInt(1)));
                    }
                }
            }
            rs.Finalize();
            rs = m_db.ExecuteQuery("SELECT obdobirok,id FROM uhrada");
            while(rs.NextRow())
            {
                if(!hasTableId("obdobi", "rok", rs.GetInt(0)))
                {
                    result = false;
                    tryExecuteUpdate(wxString::Format("DELETE FROM uhrada WHERE id=%d", rs.GetInt(1)));
                }
            }
            rs.Finalize();
            //check castek
            rs = m_db.ExecuteQuery("SELECT denikid,id,castka FROM uhrada");
            while(rs.NextRow())
            {
                if(!dxutils::isSameDouble(m_db.ExecuteQuery(wxString::Format("SELECT castkacizi FROM denik WHERE id=%d", rs.GetInt(0))).GetDouble(0), rs.GetDouble(2)))
                {
                    result = false;
                    tryExecuteUpdate(wxString::Format("DELETE FROM uhrada WHERE id=%d", rs.GetInt(1)));
                    tryExecuteUpdate(wxString::Format("DELETE FROM denik WHERE id=%d", rs.GetInt(0)));
                }
            }
            rs.Finalize();
            rs = m_db.ExecuteQuery("SELECT fakturaid FROM uhrada");
            while(rs.NextRow())
            {
                if(rs.GetInt(0) > 0)
                {
                    double celkem = m_db.ExecuteQuery(wxString::Format("SELECT celkem FROM faktura WHERE id=%d", rs.GetInt(0))).GetDouble(0);
                    double castka = m_db.ExecuteQuery(wxString::Format("SELECT sum(castka) FROM uhrada WHERE fakturaid=%d", rs.GetInt(0))).GetDouble(0);
                    if(celkem < castka && !dxutils::isSameDouble(celkem, castka))
                    {
                        result = false;
                        tryExecuteUpdate(wxString::Format("DELETE FROM denik WHERE id IN (SELECT denikid FROM uhrada WHERE fakturaid=%d)", rs.GetInt(0)));
                        tryExecuteUpdate(wxString::Format("DELETE FROM uhrada WHERE fakturaid=%d", rs.GetInt(0)));
                    }
                }
            }
            rs.Finalize();
            rs = m_db.ExecuteQuery("SELECT zavpohid FROM uhrada");
            while(rs.NextRow())
            {
                if(rs.GetInt(0) > 0)
                {
                    double celkem = m_db.ExecuteQuery(wxString::Format("SELECT celkem FROM zavpoh WHERE id=%d", rs.GetInt(0))).GetDouble(0);
                    double castka = m_db.ExecuteQuery(wxString::Format("SELECT sum(castka) FROM uhrada WHERE zavpohid=%d", rs.GetInt(0))).GetDouble(0);
                    if(celkem < castka && !dxutils::isSameDouble(celkem, castka))
                    {
                        result = false;
                        tryExecuteUpdate(wxString::Format("DELETE FROM denik WHERE id IN (SELECT denikid FROM uhrada WHERE zavpohid=%d)", rs.GetInt(0)));
                        tryExecuteUpdate(wxString::Format("DELETE FROM uhrada WHERE zavpohid=%d", rs.GetInt(0)));
                    }
                }
            }
            rs.Finalize();
        }
        else
        {
            result = false;
            createDatabaseTable(sqlite_tb_uhrada);
        }
    }
    return result;
}

bool DbModel::checkTableOdpiskupina()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("odpiskupina"))
        {
            bool create = false;
            wxArrayString arr = tableColumns("odpiskupina");
            if(!hasString(arr, "id"))
            {
                create = true;
            }
            if(!hasString(arr, "popis"))
            {
                create = true;
            }
            if(!hasString(arr, "odpisovat"))
            {
                create = true;
            }
            if(!hasString(arr, "rov1"))
            {
                create = true;
            }
            if(!hasString(arr, "rov2"))
            {
                create = true;
            }
            if(!hasString(arr, "rovzvy"))
            {
                create = true;
            }
            if(!hasString(arr, "zry1"))
            {
                create = true;
            }
            if(!hasString(arr, "zry2"))
            {
                create = true;
            }
            if(!hasString(arr, "zryzvy"))
            {
                create = true;
            }
            if(tableRows("odpiskupina") != rowsForInsertToTable(sqlite_tb_odpiskupina))
            {
                create = true;
            }
            if(create)
            {
                tryExecuteUpdate("DROP TABLE odpiskupina");
                createDatabaseTable(sqlite_tb_odpiskupina);
            }
        }
        else
        {
            createDatabaseTable(sqlite_tb_odpiskupina);
        }
    }
    return true;
}

bool DbModel::checkTableMajetek()
{
    bool result = true;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("majetek"))
        {
            bool copy = false;
            wxArrayString arr = tableColumns("majetek");
            if(!hasString(arr, "id"))
            {
                result = false;
                tryExecuteUpdate("DROP TABLE majetek");
                createDatabaseTable(sqlite_tb_majetek);
                return result;
            }
            if(!hasString(arr, "cislo"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE majetek ADD COLUMN cislo TEXT");
            }
            if(!hasString(arr, "nazev"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE majetek ADD COLUMN nazev TEXT");
            }
            if(!hasString(arr, "popis"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE majetek ADD COLUMN popis TEXT");
            }
            if(!hasString(arr, "porizen"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE majetek ADD COLUMN porizen TEXT");
                tryExecuteUpdate("UPDATE majetek SET porizen='2016-01-01'");
            }
            if(!hasString(arr, "vyrazen"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE majetek ADD COLUMN vyrazen INTEGER");
                tryExecuteUpdate("UPDATE majetek SET vyrazen=0");
            }
            if(!hasString(arr, "datumvyrazeni"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE majetek ADD COLUMN datumvyrazeni TEXT");
                tryExecuteUpdate("UPDATE majetek SET datumvyrazeni='2016-01-01'");
            }
            if(!hasString(arr, "cena"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE majetek ADD COLUMN cena REAL");
                tryExecuteUpdate("UPDATE majetek SET cena=0");
            }
            if(!hasString(arr, "typ"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE majetek ADD COLUMN typ INTEGER");
                tryExecuteUpdate("UPDATE majetek SET odpis=0");
            }
            if(!hasString(arr, "odpis"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE majetek ADD COLUMN odpis INTEGER");
                tryExecuteUpdate("UPDATE majetek SET odpis=0");
            }
            if(!hasString(arr, "skupinaid"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE majetek ADD COLUMN skupinaid INTEGER");
                tryExecuteUpdate("UPDATE majetek SET skupinaid=1");
            }
            arr = tableKeys("majetek");
            if(!hasString(arr, "skupinaid|odpiskupina|id|CASCADE|NO ACTION"))
            {
                copy = true;
                result = false;
            }
            if(copy)
            {
                tryExecuteUpdate("CREATE TEMPORARY TABLE majetek_t (id INTEGER PRIMARY KEY AUTOINCREMENT, cislo TEXT, nazev TEXT, popis TEXT, porizen TEXT, vyrazen INTEGER,\
                                   datumvyrazeni TEXT, cena REAL, typ INTEGER NOT NULL, odpis INTEGER NOT NULL, skupinaid INTEGER)");
                if(hasSameColumns("majetek", "majetek_t"))
                {
                    tryExecuteUpdate("INSERT INTO majetek_t SELECT id, cislo, nazev, popis, porizen, vyrazen,\
                                       datumvyrazeni, cena, typ, odpis, skupinaid FROM majetek");
                    tryExecuteUpdate("DROP TABLE majetek");
                    tryExecuteUpdate(sqlite_tb_majetek[0]);
                    tryExecuteUpdate("INSERT INTO majetek SELECT * FROM majetek_t");
                }
                tryExecuteUpdate("DROP TABLE majetek_t");
            }
        }
        else
        {
            result = false;
            createDatabaseTable(sqlite_tb_majetek);
        }
    }
    return result;
}

bool DbModel::checkTableOdpis()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("odpis"))
        {
            wxArrayString arr = tableColumns("odpis");
            if(!hasString(arr, "id") || !hasString(arr, "rok") || !hasString(arr, "odpis") || !hasString(arr, "majetekid"))
            {
                tryExecuteUpdate("DROP TABLE odpis");
                createDatabaseTable(sqlite_tb_odpis);
                return false;
            }
            arr = tableKeys("odpis");
            if(!hasString(arr, "majetekid|majetek|id|CASCADE|CASCADE"))
            {
                tryExecuteUpdate("DROP TABLE odpis");
                createDatabaseTable(sqlite_tb_odpis);
                return false;
            }
        }
        else
        {
            createDatabaseTable(sqlite_tb_odpis);
            return false;
        }
    }
    return true;
}

bool DbModel::checkTableSettings()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("settings"))
        {
            wxArrayString arr = tableColumns("settings");
            if(!hasString(arr, "rok") || !hasString(arr, "splatnost"))
            {
                tryExecuteUpdate("DROP TABLE settings");
                createDatabaseTable(sqlite_tb_settings);
                return false;
            }
        }
        else
        {
            createDatabaseTable(sqlite_tb_settings);
            return false;
        }
    }
    return true;
}

bool DbModel::checkTableSmtp()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("smtp"))
        {
            wxArrayString arr = tableColumns("smtp");
            if(!hasString(arr, "server") || !hasString(arr, "port") || !hasString(arr, "ssl") || !hasString(arr, "autentifikace")
                    || !hasString(arr, "username") || !hasString(arr, "password") || !hasString(arr, "sender") || !hasString(arr, "subject")
                    || !hasString(arr, "text"))
            {
                tryExecuteUpdate("DROP TABLE smtp");
                createDatabaseTable(sqlite_tb_smtp);
                return false;
            }
        }
        else
        {
            createDatabaseTable(sqlite_tb_smtp);
            return false;
        }
    }
    return true;
}

bool DbModel::checkTableOdeslany()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("odeslany"))
        {
            wxArrayString arr = tableColumns("odeslany");
            if(!hasString(arr, "id") || !hasString(arr, "cas") || !hasString(arr, "faktura") || !hasString(arr, "prijemce"))
            {
                tryExecuteUpdate("DROP TABLE odeslany");
                createDatabaseTable(sqlite_tb_odeslany);
                return false;
            }
        }
        else
        {
            createDatabaseTable(sqlite_tb_odeslany);
            return false;
        }
    }
    return true;
}

bool DbModel::checkTablePriznani()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("priznani"))
        {
            wxArrayString arr = tableColumns("priznani");
            if(!hasString(arr, "jmeno") || !hasString(arr, "prijmeni") || !hasString(arr, "titul") || !hasString(arr, "ulice")
                    || !hasString(arr, "c_pop") || !hasString(arr, "c_orient") || !hasString(arr, "naz_obce") || !hasString(arr, "psc")
                    || !hasString(arr, "stat") || !hasString(arr, "dic") || !hasString(arr, "email") || !hasString(arr, "id_dats")
                    || !hasString(arr, "c_telef") || !hasString(arr, "c_ufo") || !hasString(arr, "c_pracufo") || !hasString(arr, "c_okec")
                    || !hasString(arr, "opr_jmeno") || !hasString(arr, "opr_prijmeni") || !hasString(arr, "opr_postaveni") || !hasString(arr, "zast_dat_nar")
                    || !hasString(arr, "zast_ev_cislo") || !hasString(arr, "zast_ic") || !hasString(arr, "zast_jmeno") || !hasString(arr, "zast_kod")
                    || !hasString(arr, "zast_nazev") || !hasString(arr, "zast_prijmeni") || !hasString(arr, "zast_typ"))
            {
                tryExecuteUpdate("DROP TABLE priznani");
                createDatabaseTable(sqlite_tb_priznani);
                return false;
            }
        }
        else
        {
            createDatabaseTable(sqlite_tb_priznani);
            return false;
        }
    }
    return true;
}

bool DbModel::checkTableUlozene()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("ulozene"))
        {
            wxArrayString arr = tableColumns("ulozene");
            if(!hasString(arr, "id") || !hasString(arr, "typ") || !hasString(arr, "popis") || !hasString(arr, "forma")
                    || !hasString(arr, "obdobi") || !hasString(arr, "mesic") || !hasString(arr, "ctvrtleti") || !hasString(arr, "rok")
                    || !hasString(arr, "od") || !hasString(arr, "do") || !hasString(arr, "xml"))
            {
                tryExecuteUpdate("DROP TABLE ulozene");
                createDatabaseTable(sqlite_tb_ulozene);
                return false;
            }
        }
        else
        {
            createDatabaseTable(sqlite_tb_ulozene);
            return false;
        }
    }
    return true;
}

bool DbModel::checkTableSklad()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("sklad"))
        {
            wxArrayString arr = tableColumns("sklad");
            if(!hasString(arr, "id") || !hasString(arr, "nazev") || !hasString(arr, "popis") || !hasString(arr, "ocenovani"))
            {
                tryExecuteUpdate("DROP TABLE sklad");
                createDatabaseTable(sqlite_tb_sklad);
                return false;
            }
        }
        else
        {
            createDatabaseTable(sqlite_tb_sklad);
            return false;
        }
    }
    return true;
}

bool DbModel::checkTableZasoba()
{
    bool result = true;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("zasoba"))
        {
            bool copy = false;
            wxArrayString arr = tableColumns("zasoba");
            if(!hasString(arr, "id"))
            {
                result = false;
                tryExecuteUpdate("DROP TABLE zasoba");
                createDatabaseTable(sqlite_tb_zasoba);
                return result;
            }
            if(!hasString(arr, "cislo"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE zasoba ADD COLUMN cislo TEXT");
            }
            if(!hasString(arr, "nazev"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE zasoba ADD COLUMN nazev TEXT");
            }
            if(!hasString(arr, "popis"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE zasoba ADD COLUMN popis TEXT");
            }
            if(!hasString(arr, "ocenovani"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE zasoba ADD COLUMN ocenovani INTEGER");
                tryExecuteUpdate("UPDATE zasoba SET ocenovani=0");
            }
            if(!hasString(arr, "mj"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE zasoba ADD COLUMN mj TEXT");
            }
            if(!hasString(arr, "pocet"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE zasoba ADD COLUMN pocet REAL");
                tryExecuteUpdate("UPDATE zasoba SET pocet=0");
            }
            if(!hasString(arr, "celkem"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE zasoba ADD COLUMN celkem REAL");
                tryExecuteUpdate("UPDATE zasoba SET celkem=0");
            }
            if(!hasString(arr, "cenamj"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE zasoba ADD COLUMN cenamj REAL");
                tryExecuteUpdate("UPDATE zasoba SET cenamj=0");
            }
            if(!hasString(arr, "bezdph"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE zasoba ADD COLUMN bezdph INTEGER");
                tryExecuteUpdate("UPDATE zasoba SET bezdph=0");
            }
            if(!hasString(arr, "dph"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE zasoba ADD COLUMN dph INTEGER");
                tryExecuteUpdate("UPDATE zasoba SET dph=0");
            }
            if(!hasString(arr, "skladid"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE zasoba ADD COLUMN skladid INTEGER");
                tryExecuteUpdate("UPDATE zasoba SET skladid=1");
            }
            arr = tableKeys("zasoba");
            if(!hasString(arr, "skladid|sklad|id|CASCADE|NO ACTION"))
            {
                copy = true;
                result = false;
            }
            if(copy)
            {
                tryExecuteUpdate("CREATE TEMPORARY TABLE zasoba_t (id INTEGER PRIMARY KEY AUTOINCREMENT, cislo TEXT, nazev TEXT, popis TEXT, ocenovani INTEGER NOT NULL, mj TEXT,\
                                   pocet REAL, celkem REAL, skladid INTEGER NOT NULL, cenamj REAL, bezdph INTEGER, dph INTEGER,\
                                   FOREIGN KEY(skladid) REFERENCES sklad(id) ON UPDATE CASCADE)");
                if(hasSameColumns("zasoba", "zasoba_t"))
                {
                    tryExecuteUpdate("INSERT INTO zasoba_t SELECT id, cislo, nazev, popis, ocenovani,\
                                       mj, pocet, celkem, skladid , cenamj, bezdph, dph FROM zasoba");
                    tryExecuteUpdate("DROP TABLE zasoba");
                    tryExecuteUpdate(sqlite_tb_zasoba[0]);
                    tryExecuteUpdate("INSERT INTO zasoba SELECT * FROM zasoba_t");
                }
                tryExecuteUpdate("DROP TABLE zasoba_t");
            }
            //check referencovane id
            if(tableRows("zasoba"))
            {
                int skladid = -1;
                if(tableRows("sklad"))
                {
                    wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT id FROM sklad");
                    while(rs.NextRow())
                    {
                        skladid = rs.GetInt(0);
                        rs.Finalize();
                        break;
                    }
                }
                else
                {
                    tryExecuteUpdate("INSERT INTO sklad (id, nazev, ocenovani) VALUES (1, 'Sklad', 0)");
                    skladid = 1;
                }
                wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT id,skladid FROM zasoba");
                while(rs.NextRow())
                {
                    if(!hasTableId("sklad", "id", rs.GetInt(1)))
                    {
                        tryExecuteUpdate(wxString::Format("UPDATE zasoba SET skladid=%d WHERE id=%d", skladid, rs.GetInt(0)));
                        result = false;
                    }
                }
                rs.Finalize();
            }
        }
        else
        {
            result = false;
            createDatabaseTable(sqlite_tb_zasoba);
        }
    }
    return result;
}

bool DbModel::checkTableInventura()
{
    bool result = true;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("inventura"))
        {
            bool copy = false;
            wxArrayString arr = tableColumns("inventura");
            if(!hasString(arr, "id"))
            {
                result = false;
                tryExecuteUpdate("DROP TABLE inventura");
                createDatabaseTable(sqlite_tb_inventura);
                return result;
            }
            if(!hasString(arr, "datum"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE inventura ADD COLUMN datum TEXT");
            }
            if(!hasString(arr, "cislo"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE inventura ADD COLUMN cislo TEXT");
            }
            if(!hasString(arr, "popis"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE inventura ADD COLUMN popis TEXT");
            }
            if(!hasString(arr, "provedl"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE inventura ADD COLUMN provedl TEXT");
            }
            if(!hasString(arr, "skladid"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE inventura ADD COLUMN skladid INTEGER");
                tryExecuteUpdate("UPDATE inventura SET skladid=1");
            }
            if(!hasString(arr, "obdobirok"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE inventura ADD COLUMN obdobirok INTEGER");
                tryExecuteUpdate(wxString::Format("UPDATE inventura SET obdobirok=%d", m_currentRok));
            }
            arr = tableKeys("inventura");
            if(!hasString(arr, "skladid|sklad|id|CASCADE|NO ACTION") || !hasString(arr, "obdobirok|obdobi|rok|CASCADE|CASCADE"))
            {
                copy = true;
                result = false;
            }
            if(copy)
            {
                tryExecuteUpdate("CREATE TEMPORARY TABLE inventura_t (id INTEGER PRIMARY KEY AUTOINCREMENT, datum TEXT, cislo TEXT, popis TEXT, provedl TEXT, skladid INTEGER NOT NULL, obdobirok INTEGER NOT NULL)");
                if(hasSameColumns("inventura", "inventura_t"))
                {
                    tryExecuteUpdate("INSERT INTO inventura_t SELECT id, datum, cislo, popis, provedl, skladid, obdobirok FROM inventura");
                    tryExecuteUpdate("DROP TABLE inventura");
                    tryExecuteUpdate(sqlite_tb_inventura[0]);
                    tryExecuteUpdate("INSERT INTO inventura SELECT * FROM inventura_t");
                }
                tryExecuteUpdate("DROP TABLE inventura_t");
            }
            //check referencovane id
            wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT id,skladid FROM inventura");
            while(rs.NextRow())
            {
                if(!hasTableId("sklad", "id", rs.GetInt(1)))
                {
                    tryExecuteUpdate(wxString::Format("DELETE FROM inventura WHERE id=%d", rs.GetInt(0)));
                    result = false;
                }
            }
            rs.Finalize();
            rs = m_db.ExecuteQuery("SELECT id,obdobirok FROM inventura");
            while(rs.NextRow())
            {
                if(!hasTableId("obdobi", "rok", rs.GetInt(1)))
                {
                    tryExecuteUpdate(wxString::Format("DELETE FROM inventura WHERE id=%d", rs.GetInt(0)));
                    result = false;
                }
            }
            rs.Finalize();
        }
        else
        {
            result = false;
            createDatabaseTable(sqlite_tb_inventura);
        }
    }
    return result;
}

bool DbModel::checkTableIvpolozka()
{
    bool result = true;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("ivpolozka"))
        {
            wxArrayString arr = tableColumns("ivpolozka");
            if(!hasString(arr, "id"))
            {
                result = false;
                tryExecuteUpdate("DROP TABLE ivpolozka");
                createDatabaseTable(sqlite_tb_ivpolozka);
                return result;
            }
            if(!hasString(arr, "evidovano"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE ivpolozka ADD COLUMN evidovano REAL");
                tryExecuteUpdate("UPDATE ivpolozka SET evidovano=0");
            }
            if(!hasString(arr, "zjisteno"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE ivpolozka ADD COLUMN zjisteno REAL");
                tryExecuteUpdate("UPDATE ivpolozka SET zjisteno=0");
            }
            if(!hasString(arr, "zasobaid"))
            {
                result = false;
                tryExecuteUpdate("DROP TABLE ivpolozka");
                createDatabaseTable(sqlite_tb_ivpolozka);
            }
            if(!hasString(arr, "inventuraid"))
            {
                result = false;
                tryExecuteUpdate("DROP TABLE ivpolozka");
                createDatabaseTable(sqlite_tb_ivpolozka);
            }
            arr = tableKeys("ivpolozka");
            if(!hasString(arr, "zasobaid|zasoba|id|CASCADE|CASCADE") || !hasString(arr, "inventuraid|inventura|id|CASCADE|CASCADE"))
            {
                result = false;
                tryExecuteUpdate("CREATE TEMPORARY TABLE ivpolozka_t (id INTEGER PRIMARY KEY AUTOINCREMENT, evidovano REAL, zjisteno REAL, zasobaid INTEGER NOT NULL, inventuraid INTEGER NOT NULL)");
                if(hasSameColumns("ivpolozka", "ivpolozka_t"))
                {
                    tryExecuteUpdate("INSERT INTO ivpolozka_t SELECT id, evidovano, zjisteno, zasobaid, inventuraid FROM inventura");
                    tryExecuteUpdate("DROP TABLE ivpolozka");
                    tryExecuteUpdate(sqlite_tb_ivpolozka[0]);
                    tryExecuteUpdate("INSERT INTO ivpolozka SELECT * FROM ivpolozka_t");
                }
                tryExecuteUpdate("DROP TABLE ivpolozka_t");
            }
            //check referencovane id
            wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT id,zasobaid FROM ivpolozka");
            while(rs.NextRow())
            {
                if(!hasTableId("zasoba", "id", rs.GetInt(1)))
                {
                    tryExecuteUpdate(wxString::Format("DELETE FROM ivpolozka WHERE id=%d", rs.GetInt(0)));
                    result = false;
                }
            }
            rs.Finalize();
            rs = m_db.ExecuteQuery("SELECT id,inventuraid FROM ivpolozka");
            while(rs.NextRow())
            {
                if(!hasTableId("inventura", "id", rs.GetInt(1)))
                {
                    tryExecuteUpdate(wxString::Format("DELETE FROM ivpolozka WHERE id=%d", rs.GetInt(0)));
                    result = false;
                }
            }
            rs.Finalize();
        }
        else
        {
            result = false;
            createDatabaseTable(sqlite_tb_ivpolozka);
        }
    }
    return result;
}

bool DbModel::checkTablePohybzasob()
{
    bool result = true;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("pohybzasob"))
        {
            bool copy = false;
            wxArrayString arr = tableColumns("pohybzasob");
            if(!hasString(arr, "id"))
            {
                result = false;
                tryExecuteUpdate("DROP TABLE pohybzasob");
                createDatabaseTable(sqlite_tb_pohybzasob);
                return result;
            }
            if(!hasString(arr, "datum"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE pohybzasob ADD COLUMN datum TEXT");
                tryExecuteUpdate(wxString::Format("UPDATE pohybzasob SET datum='%s'", wxDateTime::Now().Format("%Y-%m-%d %H:%M:%S")));
            }
            if(!hasString(arr, "cislo"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE pohybzasob ADD COLUMN cislo TEXT");
            }
            if(!hasString(arr, "pocet"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE pohybzasob ADD COLUMN pocet REAL");
                tryExecuteUpdate("UPDATE pohybzasob SET pocet=0");
            }
            if(!hasString(arr, "cenamj"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE pohybzasob ADD COLUMN cenamj REAL");
                tryExecuteUpdate("UPDATE pohybzasob SET cenamj=0");
            }
            if(!hasString(arr, "prijemka"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE pohybzasob ADD COLUMN prijemka INTEGER");
                tryExecuteUpdate("UPDATE pohybzasob SET prijemka=1");
            }
            if(!hasString(arr, "pocatek"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE pohybzasob ADD COLUMN pocatek INTEGER");
                tryExecuteUpdate("UPDATE pohybzasob SET pocatek=0");
            }
            if(!hasString(arr, "inventura"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE pohybzasob ADD COLUMN inventura INTEGER");
                tryExecuteUpdate("UPDATE pohybzasob SET inventura=0");
            }
            if(!hasString(arr, "inventuraid"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE pohybzasob ADD COLUMN inventuraid INTEGER");
                tryExecuteUpdate("UPDATE pohybzasob SET inventuraid=null");
            }
            if(!hasString(arr, "fakturaid"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE pohybzasob ADD COLUMN fakturaid INTEGER");
                tryExecuteUpdate("UPDATE pohybzasob SET fakturaid=null");
            }
            if(!hasString(arr, "zasobaid"))
            {
                result = false;
                tryExecuteUpdate("DROP TABLE pohybzasob");
                createDatabaseTable(sqlite_tb_pohybzasob);
                return result;
            }
            if(!hasString(arr, "obdobirok"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE pohybzasob ADD COLUMN obdobirok INTEGER");
                tryExecuteUpdate(wxString::Format("UPDATE pohybzasob SET obdobirok=%d", m_currentRok));
            }
            arr = tableKeys("pohybzasob");
            if(!hasString(arr, "inventuraid|inventura|id|CASCADE|CASCADE") || !hasString(arr, "fakturaid|faktura|id|CASCADE|CASCADE") ||
                    !hasString(arr, "zasobaid|zasoba|id|CASCADE|CASCADE") || !hasString(arr, "obdobirok|obdobi|rok|CASCADE|CASCADE"))
            {
                copy = true;
                result = false;
            }
            if(copy)
            {
                tryExecuteUpdate("CREATE TEMPORARY TABLE pohybzasob_t (id INTEGER PRIMARY KEY AUTOINCREMENT, cislo TEXT, pocet REAL, cenamj REAL,\
                                   prijemka INTEGER, pocatek INTEGER, inventura INTEGER, inventuraid INTEGER, fakturaid INTEGER, zasobaid INTEGER NOT NULL, obdobirok INTEGER NOT NULL)");
                if(hasSameColumns("pohybzasob", "pohybzasob_t"))
                {
                    tryExecuteUpdate("INSERT INTO pohybzasob_t SELECT id, cislo,\
                                       pocet, cenamj, prijemka, pocatek, inventura, inventuraid, fakturaid, zasobaid, obdobirok FROM pohybzasob");
                    tryExecuteUpdate("DROP TABLE pohybzasob");
                    tryExecuteUpdate(sqlite_tb_pohybzasob[0]);
                    tryExecuteUpdate("INSERT INTO pohybzasob SELECT * FROM pohybzasob_t");
                }
                tryExecuteUpdate("DROP TABLE pohybzasob_t");
            }
            //check referencovane id
            wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT id,inventuraid FROM pohybzasob");
            while(rs.NextRow())
            {
                if(rs.GetInt(1))
                {
                    if(!hasTableId("inventura", "id", rs.GetInt(1)))
                    {
                        tryExecuteUpdate(wxString::Format("UPDATE pohybzasob SET inventuraid=null WHERE id=%d", rs.GetInt(0)));
                        tryExecuteUpdate(wxString::Format("UPDATE pohybzasob SET inventura=0 WHERE id=%d", rs.GetInt(0)));
                        result = false;
                    }
                }
            }
            rs.Finalize();
            rs = m_db.ExecuteQuery("SELECT id,fakturaid FROM pohybzasob");
            while(rs.NextRow())
            {
                if(rs.GetInt(1))
                {
                    if(!hasTableId("faktura", "id", rs.GetInt(1)))
                    {
                        tryExecuteUpdate(wxString::Format("UPDATE pohybzasob SET fakturaid=null WHERE id=%d", rs.GetInt(0)));
                        result = false;
                    }
                }
            }
            rs.Finalize();
            rs = m_db.ExecuteQuery("SELECT id,zasobaid FROM pohybzasob");
            while(rs.NextRow())
            {
                if(!hasTableId("zasoba", "id", rs.GetInt(1)))
                {
                    tryExecuteUpdate(wxString::Format("DELETE FROM pohybzasob WHERE id=%d", rs.GetInt(0)));
                    result = false;
                }
            }
            rs.Finalize();
            rs = m_db.ExecuteQuery("SELECT id,obdobirok FROM pohybzasob");
            while(rs.NextRow())
            {
                if(!hasTableId("obdobi", "rok", rs.GetInt(1)))
                {
                    tryExecuteUpdate(wxString::Format("UPDATE pohybzasob SET obdobirok=%d WHERE id=%d", m_currentRok, rs.GetInt(0)));
                    result = false;
                }
            }
            rs.Finalize();
            //check datumu u pohybu ktere jsou pocatecnimi stavy
            rs = m_db.ExecuteQuery("SELECT id,pocatek,obdobirok,datum,zasobaid FROM pohybzasob ORDER BY zasobaid ASC, datum ASC");
            int rok = 0, zasobaid = 0;
            while(rs.NextRow())
            {
                if(rs.GetBool(1)) // pocatecni stav
                {
                    if(rok != rs.GetInt(2))
                    {
                        rok = rs.GetInt(2);
                        zasobaid = rs.GetInt(4);
                        wxDateTime date = rs.GetDateTime(3);
                        if(rok != date.GetYear() || wxDateTime::Jan != date.GetMonth() || 1 != date.GetDay() || 0 != date.GetHour()
                                || 0 != date.GetMinute() || 0 != date.GetSecond())
                        {
                            tryExecuteUpdate(wxString::Format("UPDATE pohybzasob SET datum='%d-01-01 00:00:00', prijemka=1 WHERE id=%d", rs.GetInt(2), rs.GetInt(0)));
                            result = false;
                        }
                    }
                    else
                    {
                        if(zasobaid != rs.GetInt(4)) //jina zasoba stejny rok
                        {
                            zasobaid = rs.GetInt(4);
                            wxDateTime date = rs.GetDateTime(3);
                            if(rok != date.GetYear() || wxDateTime::Jan != date.GetMonth() || 1 != date.GetDay() || 0 != date.GetHour()
                                    || 0 != date.GetMinute() || 0 != date.GetSecond())
                            {
                                tryExecuteUpdate(wxString::Format("UPDATE pohybzasob SET datum='%d-01-01 00:00:00', prijemka=1 WHERE id=%d", rs.GetInt(2), rs.GetInt(0)));
                                result = false;
                            }
                        }
                        else //chyba pocatecni stav muze pro rok a zasobu jen jeden
                        {
                            tryExecuteUpdate(wxString::Format("UPDATE pohybzasob SET pocatek=0 WHERE id=%d", rs.GetInt(0)));
                        }
                    }
                }
            }
            rs.Finalize();
        }
        else
        {
            result = false;
            createDatabaseTable(sqlite_tb_pohybzasob);
        }
    }
    return result;
}

bool DbModel::checkTableRidic()
{
    bool result = true;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("ridic"))
        {
            wxArrayString arr = tableColumns("ridic");
            if(!hasString(arr, "id"))
            {
                result = false;
                tryExecuteUpdate("DROP TABLE ridic");
                createDatabaseTable(sqlite_tb_ridic);
                return result;
            }
            if(!hasString(arr, "jmeno"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE ridic ADD COLUMN jmeno TEXT");
            }
        }
        else
        {
            result = false;
            createDatabaseTable(sqlite_tb_ridic);
        }
    }
    return result;
}

bool DbModel::checkTablePhm()
{
    bool result = true;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("phm"))
        {
            wxArrayString arr = tableColumns("phm");
            if(!hasString(arr, "id"))
            {
                result = false;
                tryExecuteUpdate("DROP TABLE phm");
                createDatabaseTable(sqlite_tb_phm);
                return result;
            }
            if(!hasString(arr, "jmeno"))
            {
                result = false;
                tryExecuteUpdate("DROP TABLE phm");
                createDatabaseTable(sqlite_tb_phm);
                return result;
            }
            if(tableRows("phm") != rowsForInsertToTable(sqlite_tb_phm))
            {
                tryExecuteUpdate("DROP TABLE phm");
                createDatabaseTable(sqlite_tb_phm);
            }
        }
        else
        {
            result = false;
            createDatabaseTable(sqlite_tb_phm);
        }
    }
    return result;
}

bool DbModel::checkTableCenaphm()
{
    bool result = true;
    if(m_db.TableExists("cenaphm"))
    {
        wxArrayString arr = tableColumns("cenaphm");
        if(arr.Index("id")==wxNOT_FOUND || arr.Index("phmid")==wxNOT_FOUND || arr.Index("cena")==wxNOT_FOUND || arr.Index("validfrom")==wxNOT_FOUND || arr.Index("validto")==wxNOT_FOUND)
        {
            result = false;
            tryExecuteUpdate("DROP TABLE cenaphm");
            createDatabaseTable(sqlite_tb_cenaphm);
        }
        if(tableRows("cenaphm") != rowsForInsertToTable(sqlite_tb_cenaphm))
        {
            tryExecuteUpdate("DROP TABLE cenaphm");
            createDatabaseTable(sqlite_tb_cenaphm);
        }
        arr = tableKeys("cenaphm");
        if(arr.Index("phmid|phm|id|CASCADE|CASCADE")==wxNOT_FOUND)
        {
            result = false;
            tryExecuteUpdate("DROP TABLE cenaphm");
            createDatabaseTable(sqlite_tb_cenaphm);
        }
    }
    else
    {
        result = false;
        createDatabaseTable(sqlite_tb_cenaphm);
    }
    return result;
}

bool DbModel::checkTableDruhVozidla()
{
    bool result = true;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("druhvozidla"))
        {
            wxArrayString arr = tableColumns("druhvozidla");
            if(!hasString(arr, "id") || !hasString(arr, "jmeno"))
            {
                result = false;
                tryExecuteUpdate("DROP TABLE druhvozidla");
                createDatabaseTable(sqlite_tb_cenaphm);
            }
            if(tableRows("druhvozidla") != rowsForInsertToTable(sqlite_tb_druhvozidla))
            {
                result = false;
                tryExecuteUpdate("DROP TABLE druhvozidla");
                createDatabaseTable(sqlite_tb_druhvozidla);
            }
        }
        else
        {
            result = false;
            createDatabaseTable(sqlite_tb_druhvozidla);
        }
    }
    return result;
}

bool DbModel::checkTableNahradakm()
{
    bool result = true;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("nahradakm"))
        {
            wxArrayString arr = tableColumns("nahradakm");
            if(!hasString(arr, "id") || !hasString(arr, "druhvozidlaid") || !hasString(arr, "nahrada") || !hasString(arr, "rok"))
            {
                result = false;
                tryExecuteUpdate("DROP TABLE nahradakm");
                createDatabaseTable(sqlite_tb_nahradakm);
            }
            arr = tableKeys("nahradakm");
            if(!hasString(arr, "druhvozidlaid|druhvozidla|id|CASCADE|CASCADE"))
            {
                result = false;
                tryExecuteUpdate("DROP TABLE nahradakm");
                createDatabaseTable(sqlite_tb_nahradakm);
            }
            if(tableRows("nahradakm") != rowsForInsertToTable(sqlite_tb_nahradakm))
            {
                tryExecuteUpdate("DROP TABLE nahradakm");
                createDatabaseTable(sqlite_tb_nahradakm);
            }
        }
        else
        {
            result = false;
            createDatabaseTable(sqlite_tb_nahradakm);
        }
    }
    return result;
}

bool DbModel::checkTableVozidlo()
{
    bool result = true;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("vozidlo"))
        {
            bool copy = false;
            wxArrayString arr = tableColumns("vozidlo");
            if(!hasString(arr, "id"))
            {
                result = false;
                tryExecuteUpdate("DROP TABLE vozidlo");
                createDatabaseTable(sqlite_tb_vozidlo);
                return result;
            }
            if(!hasString(arr, "druhvozidlaid"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE vozidlo ADD COLUMN druhvozidlaid INTEGER");
                tryExecuteUpdate("UPDATE vozidlo SET druhvozidlaid=1");
            }
            if(!hasString(arr, "phmid"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE vozidlo ADD COLUMN phmid INTEGER");
                tryExecuteUpdate("UPDATE vozidlo SET phmid=1");
            }
            if(!hasString(arr, "spotreba"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE vozidlo ADD COLUMN spotreba REAL");
                tryExecuteUpdate("UPDATE vozidlo SET spotreba=0.0");
            }
            if(!hasString(arr, "rz"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE vozidlo ADD COLUMN rz TEXT");
            }
            if(!hasString(arr, "jmeno"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE vozidlo ADD COLUMN jmeno TEXT");
            }
            if(!hasString(arr, "soukrome"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE vozidlo ADD COLUMN soukrome INTEGER");
                tryExecuteUpdate("UPDATE vozidlo SET soukrome=0");
            }
            arr = tableKeys("vozidlo");
            if(!hasString(arr, "druhvozidlaid|druhvozidla|id|CASCADE|CASCADE"))
            {
                copy = true;
                result = false;
            }
            if(!hasString(arr, "phmid|phm|id|CASCADE|CASCADE"))
            {
                copy = true;
                result = false;
            }
            if(copy)
            {
                tryExecuteUpdate("CREATE TEMPORARY TABLE vozidlo_t (id INTEGER PRIMARY KEY AUTOINCREMENT, druhvozidlaid INTEGER, spotreba REAL, rz TEXT, jmeno TEXT, soukrome INTEGER,"
                                 "phmid INTEGER)");
                if(hasSameColumns("vozidlo", "vozidlo_t"))
                {
                    tryExecuteUpdate("INSERT INTO vozidlo_t SELECT id, druhvozidlaid, spotreba, rz, jmeno, soukrome, phmid FROM vozidlo");
                    tryExecuteUpdate("DROP TABLE vozidlo");
                    tryExecuteUpdate(sqlite_tb_vozidlo[0]);
                    tryExecuteUpdate("INSERT INTO vozidlo SELECT * FROM vozidlo_t");
                }
                tryExecuteUpdate("DROP TABLE vozidlo_t");
            }
        }
        else
        {
            result = false;
            createDatabaseTable(sqlite_tb_vozidlo);
        }
    }
    return result;
}

bool DbModel::checkTableJizda()
{
    bool result = true;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists("jizda"))
        {
            bool copy = false;
            wxArrayString arr = tableColumns("jizda");
            if(!hasString(arr, "id"))
            {
                result = false;
                tryExecuteUpdate("DROP TABLE jizda");
                createDatabaseTable(sqlite_tb_jizda);
                return result;
            }
            if(!hasString(arr, "vozidloid"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE jizda ADD COLUMN vozidloid INTEGER");
                tryExecuteUpdate("UPDATE jizda SET vozidloid=1");
            }
            if(!hasString(arr, "ridicid"))
            {
                copy = true;
                result = false;
                tryExecuteUpdate("ALTER TABLE jizda ADD COLUMN ridicid INTEGER");
                tryExecuteUpdate("UPDATE jizda SET ridicid=1");
            }
            if(!hasString(arr, "startcas"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE jizda ADD COLUMN startcas TEXT");
            }
            if(!hasString(arr, "endcas"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE jizda ADD COLUMN endcas TEXT");
            }
            if(!hasString(arr, "startkm"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE jizda ADD COLUMN startkm INTEGER");
            }
            if(!hasString(arr, "endkm"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE jizda ADD COLUMN endkm INTEGER");
            }
            if(!hasString(arr, "cesta"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE jizda ADD COLUMN cesta TEXT");
            }
            if(!hasString(arr, "ucel"))
            {
                result = false;
                tryExecuteUpdate("ALTER TABLE jizda ADD COLUMN ucel TEXT");
            }
            arr = tableKeys("jizda");
            if(!hasString(arr, "vozidloid|vozidlo|id|CASCADE|CASCADE"))
            {
                copy = true;
                result = false;
            }
            if(!hasString(arr, "ridicid|ridic|id|CASCADE|CASCADE"))
            {
                copy = true;
                result = false;
            }
            if(copy)
            {
                tryExecuteUpdate("CREATE TEMPORARY TABLE jizda_t (id INTEGER PRIMARY KEY AUTOINCREMENT, vozidloid INTEGER, ridicid INTEGER, startcas TEXT, endcas TEXT, startkm INTEGER, endkm INTEGER, cesta TEXT, ucel TEXT)");
                if(hasSameColumns("jizda", "jizda_t"))
                {
                    tryExecuteUpdate("INSERT INTO jizda_t SELECT id, vozidloid, ridicid, startcas, endcas, startkm, endkm, cesta, ucel FROM jizda");
                    tryExecuteUpdate("DROP TABLE jizda");
                    tryExecuteUpdate(sqlite_tb_jizda[0]);
                    tryExecuteUpdate("INSERT INTO jizda SELECT * FROM jizda_t");
                }
                tryExecuteUpdate("DROP TABLE jizda_t");
            }
            //check referencovane id
            wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT id,ridicid FROM jizda");
            while(rs.NextRow())
            {
                if(!hasTableId("ridic", "id", rs.GetInt(1)))
                {
                    tryExecuteUpdate(wxString::Format("DELETE FROM jizda WHERE id=%d", rs.GetInt(0)));
                    result = false;
                }
            }
            rs.Finalize();
            rs = m_db.ExecuteQuery("SELECT id,vozidloid FROM jizda");
            while(rs.NextRow())
            {
                if(!hasTableId("vozidlo", "id", rs.GetInt(1)))
                {
                    tryExecuteUpdate(wxString::Format("DELETE FROM jizda WHERE id=%d", rs.GetInt(0)));
                    result = false;
                }
            }
            rs.Finalize();
        }
        else
        {
            result = false;
            createDatabaseTable(sqlite_tb_jizda);
        }
    }
    return result;
}

void DbModel::createDatabase()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        createDatabaseTable(sqlite_tb_version);
        createDatabaseTable(sqlite_tb_adresa);
        createDatabaseTable(sqlite_tb_kontakt);
        createDatabaseTable(sqlite_tb_firma);
        createDatabaseTable(sqlite_tb_obdobi);
        createDatabaseTable(sqlite_tb_dphsazba);
        createDatabaseTable(sqlite_tb_dphdruh);
        createDatabaseTable(sqlite_tb_mena);
        createDatabaseTable(sqlite_tb_kurz);
        createDatabaseTable(sqlite_tb_pokladna);
        createDatabaseTable(sqlite_tb_banka);
        createDatabaseTable(sqlite_tb_bankovniustav);
        createDatabaseTable(sqlite_tb_deniksloupec);
        createDatabaseTable(sqlite_tb_predkontace);
        createDatabaseTable(sqlite_tb_denik);
        createDatabaseTable(sqlite_tb_rada);
        createDatabaseTable(sqlite_tb_fatyp);
        createDatabaseTable(sqlite_tb_fauhrada);
        createDatabaseTable(sqlite_tb_faktura);
        createDatabaseTable(sqlite_tb_zavpoh);
        createDatabaseTable(sqlite_tb_sklad);
        createDatabaseTable(sqlite_tb_zasoba);
        createDatabaseTable(sqlite_tb_fapolozka);
        createDatabaseTable(sqlite_tb_uhrada);
        createDatabaseTable(sqlite_tb_odpiskupina);
        createDatabaseTable(sqlite_tb_majetek);
        createDatabaseTable(sqlite_tb_odpis);
        createDatabaseTable(sqlite_tb_settings);
        createDatabaseTable(sqlite_tb_smtp);
        createDatabaseTable(sqlite_tb_odeslany);
        createDatabaseTable(sqlite_tb_priznani);
        createDatabaseTable(sqlite_tb_ulozene);
        createDatabaseTable(sqlite_tb_inventura);
        createDatabaseTable(sqlite_tb_ivpolozka);
        createDatabaseTable(sqlite_tb_pohybzasob);
        createDatabaseTable(sqlite_tb_ridic);
        createDatabaseTable(sqlite_tb_phm);
        createDatabaseTable(sqlite_tb_cenaphm);
        createDatabaseTable(sqlite_tb_druhvozidla);
        createDatabaseTable(sqlite_tb_nahradakm);
        createDatabaseTable(sqlite_tb_vozidlo);
        createDatabaseTable(sqlite_tb_jizda);
    }
}

void DbModel::createDatabaseTable(const char *table[], bool onlyInsert)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        int i = onlyInsert?1:0;
        while(table[i] != NULL)
        {
            tryExecuteUpdate(wxString::FromUTF8(table[i]));
            ++i;
        }
    }
}

void DbModel::fillFirma()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable("SELECT * FROM firma");
        if(!tb.GetRowCount()) return;
        tb.SetRow(0);
        m_firma.setNazev(tb.GetString("nazev"));
        m_firma.setIc(tb.GetString("ic"));
        m_firma.setDic(tb.GetString("dic"));
        m_firma.setRegistrace(tb.GetString("registrace"));
        m_firma.setPlatce(tb.GetBool("platce"));
        m_firma.setTelefon(tb.GetString("telefon"));
        m_firma.setEmail(tb.GetString("email"));
        m_firma.setWww(tb.GetString("www"));
        m_firma.setLogo(tb.GetString("logo"));
        m_firma.setRazitko(tb.GetString("razitko"));
        m_firma.setAdresaid(tb.GetInt("adresaid"));
        m_firma.setPausal(tb.GetBool("pausal"));
    }
}

void DbModel::fillSmtp()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable("SELECT * FROM smtp");
        if(!tb.GetRowCount())
        {
            tryExecuteUpdate(sqlite_tb_smtp[1]);
            tb = m_db.GetTable("SELECT * FROM smtp");
        }
        tb.SetRow(0);
        m_smtp.setAutentifikace(tb.GetBool("autentifikace"));
        m_smtp.setPassword(tb.GetString("password"));
        m_smtp.setPort(tb.GetInt("port"));
        m_smtp.setSender(tb.GetString("sender"));
        m_smtp.setServer(tb.GetString("server"));
        m_smtp.setSsl(tb.GetBool("ssl"));
        m_smtp.setSubject(tb.GetString("subject"));
        m_smtp.setText(tb.GetString("text"));
        m_smtp.setUsername(tb.GetString("username"));
        if(m_smtp.sender().IsEmpty()) m_smtp.setSender(m_firma.email());
    }
}

void DbModel::fillPriznani()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable("SELECT * FROM priznani");
        if(!tb.GetRowCount()) return;
        tb.SetRow(0);
        m_priznani.setC_okec(tb.GetInt("c_okec"));
        m_priznani.setC_orient(tb.GetString("c_orient"));
        m_priznani.setC_pop(tb.GetInt("c_pop"));
        m_priznani.setC_pracufo(tb.GetInt("c_pracufo"));
        m_priznani.setC_telef(tb.GetString("c_telef"));
        m_priznani.setC_ufo(tb.GetInt("c_ufo"));
        m_priznani.setDic(tb.GetString("dic"));
        m_priznani.setEmail(tb.GetString("email"));
        m_priznani.setId_dats(tb.GetString("id_dats"));
        m_priznani.setJmeno(tb.GetString("jmeno"));
        m_priznani.setNaz_obce(tb.GetString("naz_obce"));
        m_priznani.setOpr_jmeno(tb.GetString("opr_jmeno"));
        m_priznani.setOpr_postaveni(tb.GetString("opr_postaveni"));
        m_priznani.setOpr_prijmeni(tb.GetString("opr_prijmeni"));
        m_priznani.setPrijmeni(tb.GetString("prijmeni"));
        m_priznani.setPsc(tb.GetString("psc"));
        m_priznani.setStat(tb.GetString("stat"));
        m_priznani.setTitul(tb.GetString("titul"));
        m_priznani.setUlice(tb.GetString("ulice"));
        m_priznani.setZast_dat_nar(tb.GetDate("zast_dat_nar"));
        m_priznani.setZast_ev_cislo(tb.GetString("zast_ev_cislo"));
        m_priznani.setZast_ic(tb.GetString("zast_ic"));
        m_priznani.setZast_jmeno(tb.GetString("zast_jmeno"));
        m_priznani.setZast_kod(tb.GetString("zast_kod"));
        m_priznani.setZast_nazev(tb.GetString("zast_nazev"));
        m_priznani.setZast_prijmeni(tb.GetString("zast_prijmeni"));
        m_priznani.setZast_typ(tb.GetString("zast_typ"));
    }
}

void DbModel::fireUpdated(int id)
{
    wxCommandEvent event(MODEL_UPDATED);
    event.SetId(id);
    wxPostEvent(m_parent, event);
}

void DbModel::fireMsg(const wxString &msg)
{
    wxCommandEvent event(MODEL_MSG);
    event.SetString(msg);
    wxPostEvent(m_parent, event);
}

int DbModel::getCisloZDokladu(const wxString &doklad, int prefix, int last)
{
    long cislo;
    doklad.Mid(prefix).ToLong(&cislo);
    cislo++;
    return cislo>last?cislo:last;
}

wxArrayString DbModel::tableColumns(const wxString &name)
{
    wxArrayString arr;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("PRAGMA table_info(%s)", name));
        while(rs.NextRow())
        {
            arr.Add(rs.GetString(1));
        }
        rs.Finalize();
    }
    return arr;
}

wxArrayString DbModel::tableKeys(const wxString &name)
{
    wxArrayString arr;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("PRAGMA foreign_key_list(%s)", name));
        while(rs.NextRow())
        {
            // from|table|to|on_update|on_delete
            // NO ACTION pro on_update or on_delete nedefinovane
            arr.Add(rs.GetString(3)+"|"+rs.GetString(2)+"|"+rs.GetString(4)+"|"+rs.GetString(5)+"|"+rs.GetString(6));
        }
        rs.Finalize();
    }
    return arr;
}

bool DbModel::hasTableColumn(const wxString &name, const wxString &column)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("PRAGMA table_info(%s)", name));
        while(rs.NextRow())
        {
            if(rs.GetString(1) == column)
            {
                rs.Finalize();
                return true;
            }
        }
        rs.Finalize();
    }
    return false;
}

void DbModel::clearTable(const wxString &name)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        tryExecuteUpdate("DELETE FROM "+name);
    }
}

int DbModel::tableRows(const wxString &name)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.TableExists(name))
        {
            return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM %s", name));
        }
    }
    return 0;
}

int DbModel::rowsForInsertToTable(const char *table[])
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        int i = 1;
        while(table[i] != NULL)
        {
            ++i;
        }
        return --i;
    }
    return 0;
}

bool DbModel::hasTableId(const wxString &table, const wxString &idcolumn, int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(!m_db.TableExists(table)) return false;
        else
        {
            wxArrayString arr = tableColumns(table);
            if(!hasString(arr, idcolumn)) return false;
            else
            {
                wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT %s FROM %s", idcolumn, table));
                while(rs.NextRow())
                {
                    if(id == rs.GetInt(0))
                    {
                        rs.Finalize();
                        return true;
                    }
                }
                rs.Finalize();
            }
        }
    }
    return false;
}

bool DbModel::hasTableId(const wxString &table, const wxString &idcolumn, const wxString &id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(!m_db.TableExists(table)) return false;
        else
        {
            wxArrayString arr = tableColumns(table);
            if(!hasString(arr, idcolumn)) return false;
            else
            {
                wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT %s FROM %s", idcolumn, table));
                while(rs.NextRow())
                {
                    if(id == rs.GetString(0))
                    {
                        rs.Finalize();
                        return true;
                    }
                }
                rs.Finalize();
            }
        }
    }
    return false;
}

bool DbModel::hasSameColumns(const wxString &table1, const wxString &table2)
{
    wxArrayString arr1 = tableColumns(table1);
    wxArrayString arr2 = tableColumns(table2);
    arr1.Sort();
    arr2.Sort();
    if(arr1.GetCount() != arr2.GetCount()) return false;
    for(size_t i=0; i < arr1.GetCount(); i++)
    {
        if(arr1[i] != arr2[i]) return false;
    }
    return true;
}

void DbModel::tryExecuteUpdate(const wxString &sql)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.IsOpen())
        {
            try
            {
                m_db.ExecuteUpdate(sql);
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v tryExecuteUpdate \"%s\": %s", sql, e.GetMessage()));
            }
        }
        else
        {
            wxLogDebug("Database neni otevrena");
        }
    }
}

dxRidicArray DbModel::ridicy()
{
    dxRidicArray ridicy;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable("SELECT * FROM ridic");
        if(!tb.GetRowCount()) return ridicy;
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            ridicy.Add(Ridic(tb.GetInt("id"), tb.GetString("jmeno")));
        }
    }
    return ridicy;
}

void DbModel::addRidic(const Ridic &ridic)
{
    if(!hasRidic(ridic.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("INSERT INTO ridic(id,jmeno) "
                                                              "VALUES(?,?)");
                st.Bind(1, ridic.id());
                st.Bind(2, ridic.jmeno());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v addRidic: %s", e.GetMessage()));
            }
        }
    }
}

Ridic DbModel::getRidic(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM ridic WHERE id=%d", id));
        while(rs.NextRow())
        {
            return Ridic(rs.GetInt("id"), rs.GetString("jmeno"));
        }
        rs.Finalize();
    }
    return Ridic();
}

Ridic DbModel::getRidic(const wxString &jmeno)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM ridic WHERE jmeno='%s'", jmeno));
        while(rs.NextRow())
        {
            return Ridic(rs.GetInt("id"), rs.GetString("jmeno"));
        }
        rs.Finalize();
    }
    return Ridic();
}

bool DbModel::hasRidic(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM ridic WHERE id='%d'", id));
    }
    return false;
}

bool DbModel::hasRidic(const wxString &jmeno)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM ridic WHERE jmeno='%s'", jmeno));
    }
    return false;
}

wxString DbModel::jmenoRidic(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(hasRidic(id))
            return m_db.ExecuteQuery(wxString::Format("SELECT jmeno FROM ridic WHERE id=%d", id)).GetString(0);
    }
    return wxEmptyString;
}

dxPhmArray DbModel::phm()
{
    dxPhmArray phm;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable("SELECT * FROM phm");
        if(!tb.GetRowCount()) return phm;
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            phm.Add(Phm(tb.GetInt("id"), tb.GetString("jmeno")));
        }
    }
    return phm;
}

Phm DbModel::getPhm(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM phm WHERE id=%d", id));
        while(rs.NextRow())
        {
            return Phm(rs.GetInt("id"), rs.GetString("jmeno"));
        }
        rs.Finalize();
    }
    return Phm();
}

Phm DbModel::getPhm(const wxString &jmeno)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM phm WHERE jmeno=%s", jmeno));
        while(rs.NextRow())
        {
            return Phm(rs.GetInt("id"), rs.GetString("jmeno"));
        }
        rs.Finalize();
    }
    return Phm();
}

bool DbModel::hasPhm(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM phm WHERE id='%d'", id));
    }
    return false;
}

bool DbModel::hasPhm(const wxString &jmeno)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM phm WHERE jmeno='%s'", jmeno));
    }
    return false;
}

wxString DbModel::jmenoPhm(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(hasPhm(id))
            return m_db.ExecuteQuery(wxString::Format("SELECT jmeno FROM phm WHERE id=%d", id)).GetString(0);
    }
    return wxEmptyString;
}

double DbModel::cenaPhm(int phmid, const wxDateTime &datum)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM cenaphm WHERE phmid=%d AND validfrom <= date('%s') AND validto >= date('%s')",
                                               phmid, datum.FormatISODate(), datum.FormatISODate())))
        {
            return m_db.ExecuteQuery(wxString::Format("SELECT cena FROM cenaphm WHERE phmid=%d AND validfrom <= date('%s') AND validto >= date('%s')",
                                                      phmid, datum.FormatISODate(), datum.FormatISODate())).GetDouble(0);
        }
    }
    return 0.0;
}

dxDruhVozidlaArray DbModel::druhvozidel()
{
    dxDruhVozidlaArray druhvozidel;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable("SELECT * FROM druhvozidla");
        if(!tb.GetRowCount()) return druhvozidel;
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            if(tb.GetInt("id") != 0) druhvozidel.Add(DruhVozidla(tb.GetInt("id"), tb.GetString("jmeno")));
        }
    }
    return druhvozidel;
}

DruhVozidla DbModel::getDruhVozidla(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM druhvozidla WHERE id=%d", id));
        while(rs.NextRow())
        {
            return DruhVozidla(rs.GetInt("id"), rs.GetString("jmeno"));
        }
        rs.Finalize();
    }
    return DruhVozidla();
}

DruhVozidla DbModel::getDruhVozidla(const wxString &jmeno)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM druhvozidla WHERE jmeno='%s'", jmeno));
        while(rs.NextRow())
        {
            return DruhVozidla(rs.GetInt("id"), rs.GetString("jmeno"));
        }
        rs.Finalize();
    }
    return DruhVozidla();
}

bool DbModel::hasDruhVozidla(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM druhvozidla WHERE id=%d", id));
    }
    return false;
}

bool DbModel::hasDruhVozidla(const wxString &jmeno)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM druhvozidla WHERE jmeno='%s'", jmeno));
    }
    return false;
}

wxString DbModel::jmenoDruhVozidla(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(hasDruhVozidla(id))
            return m_db.ExecuteQuery(wxString::Format("SELECT jmeno FROM druhvozidla WHERE id=%d", id)).GetString(0);
    }
    return wxEmptyString;
}

double DbModel::nahradaKm(int druhvozidlaid, int rok)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM nahradakm WHERE druhvozidlaid=%d AND rok=%d", druhvozidlaid, m_currentRok)))
            return m_db.ExecuteQuery(wxString::Format("SELECT nahrada FROM nahradakm WHERE druhvozidlaid=%d AND rok=%d", druhvozidlaid, m_currentRok)).GetDouble(0);
    }
    return 0.0;
}

dxVozidloArray DbModel::vozidla()
{
    dxVozidloArray vozidla;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT * FROM vozidlo");
        while(rs.NextRow())
        {
            vozidla.Add(Vozidlo(rs.GetInt("id"), rs.GetString("jmeno"), rs.GetString("rz"), rs.GetInt("druhvozidlaid"), rs.GetDouble("spotreba"), rs.GetBool("soukrome"),
                                rs.GetInt("phmid")));
        }
        rs.Finalize();
    }
    return vozidla;
}

void DbModel::addVozidlo(const Vozidlo &vozidlo)
{
    if(!hasVozidlo(vozidlo.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("INSERT INTO vozidlo(id,jmeno,rz,druhvozidlaid,spotreba,soukrome,phmid) "
                                                              "VALUES(?,?,?,?,?,?,?)");
                st.Bind(1, vozidlo.id());
                st.Bind(2, vozidlo.jmeno());
                st.Bind(3, vozidlo.rz());
                st.Bind(4, vozidlo.druhvozidla());
                st.Bind(5, vozidlo.spotreba());
                st.BindBool(6, vozidlo.soukrome());
                st.Bind(7, vozidlo.phm());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v addVozidlo: %s", e.GetMessage()));
            }
        }
    }
}

void DbModel::editVozidlo(const Vozidlo &vozidlo)
{
    if(hasVozidlo(vozidlo.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("UPDATE vozidlo SET jmeno=?,rz=?,druhvozidlaid=?,spotreba=?,soukrome=?,phmid=? WHERE id=?");
                st.Bind(1, vozidlo.jmeno());
                st.Bind(2, vozidlo.rz());
                st.Bind(3, vozidlo.druhvozidla());
                st.Bind(4, vozidlo.spotreba());
                st.BindBool(5, vozidlo.soukrome());
                st.Bind(6, vozidlo.phm());
                st.Bind(7, vozidlo.id());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v editVozidlo: %s", e.GetMessage()));
            }
        }
        fireUpdated(ID_MODEL_JIZDA);
    }
}

Vozidlo DbModel::getVozidlo(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM vozidlo WHERE id=%d", id));
        while(rs.NextRow())
        {
            return Vozidlo(rs.GetInt("id"), rs.GetString("jmeno"), rs.GetString("rz"), rs.GetInt("druhvozidlaid"), rs.GetDouble("spotreba"),
                           rs.GetBool("soukrome"), rs.GetInt("phmid"));
        }
        rs.Finalize();
    }
    return Vozidlo();
}

size_t DbModel::getVozidlaCount()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar("SELECT count(*) FROM vozidlo");
    }
    return 0;
}

Vozidlo DbModel::firstVozidlo()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT * FROM vozidlo");
        while(rs.NextRow())
        {
            return Vozidlo(rs.GetInt("id"), rs.GetString("jmeno"), rs.GetString("rz"), rs.GetInt("druhvozidlaid"), rs.GetDouble("spotreba"),
                           rs.GetBool("soukrome"), rs.GetInt("phmid"));
        }
        rs.Finalize();
    }
    return Vozidlo();
}

bool DbModel::deleteVozidlo(int id)
{
    tryExecuteUpdate(wxString::Format("DELETE FROM vozidlo WHERE id=%d", id));
    fireUpdated(ID_MODEL_JIZDA);
    return true;
}

bool DbModel::hasVozidlo(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM vozidlo WHERE id=%d", id));
    }
    return false;
}

bool DbModel::hasVozidlo(const wxString &rz)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM vozidlo WHERE rz='%s'", rz));
    }
    return false;
}

int DbModel::copyVozidlo(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        Vozidlo vo = getVozidlo(id);
        int newid = getLastId("vozidlo");
        addVozidlo(Vozidlo(newid, vo.jmeno(), vo.rz(), vo.druhvozidla(), vo.spotreba(), vo.soukrome(), vo.phm()));
        return newid;
    }
    return -1;
}

int DbModel::getVozidloIdByRz(const wxString &rz)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT id FROM vozidlo WHERE rz='%s'", rz));
    }
    return -1;
}

double DbModel::nahradaKmForVozidlo(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return nahradaKm(m_db.ExecuteScalar(wxString::Format("SELECT druhvozidlaid FROM vozidlo WHERE id=%d", id)), dxsettings.datum().GetYear());
    }
    return 0.0;
}

dxJizdaViewArray DbModel::jizdyview()
{
    dxJizdaViewArray jizdy;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT jizda.id,jizda.startcas,jizda.endcas,jizda.endkm-jizda.startkm,jizda.cesta,jizda.ucel,vozidlo.jmeno||' '||vozidlo.rz,ridic.jmeno,vozidlo.soukrome "
                                                  "FROM jizda INNER JOIN vozidlo ON vozidlo.id=jizda.vozidloid INNER JOIN ridic ON ridic.id=jizda.ridicid WHERE CAST(strftime('%%Y',jizda.endcas) AS INTEGER)=%d", m_currentRok));
        while(rs.NextRow())
        {
            jizdy.Add(JizdaView(rs.GetInt(0), rs.GetDateTime(1), rs.GetDateTime(2), rs.GetInt(3), rs.GetString(4), rs.GetString(5), rs.GetString(6), rs.GetString(7), rs.GetBool(8)));
        }
        rs.Finalize();
    }
    return jizdy;
}

void DbModel::addJizda(const Jizda &jizda)
{
    if(!hasJizda(jizda.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("INSERT INTO jizda(id,vozidloid,startcas,endcas,startkm,endkm,cesta,ucel,ridicid) "
                                                              "VALUES(?,?,?,?,?,?,?,?,?)");
                st.Bind(1, jizda.id());
                st.Bind(2, jizda.vozidloid());
                st.BindDateTime(3, jizda.startcas());
                st.BindDateTime(4, jizda.endcas());
                st.Bind(5, jizda.startkm());
                st.Bind(6, jizda.endkm());
                st.Bind(7, jizda.cesta());
                st.Bind(8, jizda.ucel());
                st.Bind(9, jizda.ridicid());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v addJizda: %s", e.GetMessage()));
            }
        }
        fireUpdated(ID_MODEL_JIZDA);
    }
}

void DbModel::editJizda(const Jizda &jizda)
{
    if(hasJizda(jizda.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("UPDATE jizda SET vozidloid=?,startcas=?,endcas=?,startkm=?,endkm=?,cesta=?,ucel=?,ridicid=? WHERE id=?");
                st.Bind(1, jizda.vozidloid());
                st.BindDateTime(2, jizda.startcas());
                st.BindDateTime(3, jizda.endcas());
                st.Bind(4, jizda.startkm());
                st.Bind(5, jizda.endkm());
                st.Bind(6, jizda.cesta());
                st.Bind(7, jizda.ucel());
                st.Bind(8, jizda.ridicid());
                st.Bind(9, jizda.id());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v editJizda: %s", e.GetMessage()));
            }
        }
        fireUpdated(ID_MODEL_JIZDA);
    }
}

int DbModel::copyJizda(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        Jizda ja = getJizda(id);
        int newid = getLastId("jizda");
        addJizda(Jizda(newid, ja.startcas(), ja.endcas(), ja.startkm(), ja.endkm(), ja.cesta(), ja.ucel(), ja.vozidloid(), ja.ridicid()));
        return newid;
    }
    return -1;
}

Jizda DbModel::getJizda(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM jizda WHERE id=%d", id));
        while(rs.NextRow())
        {
            return Jizda(rs.GetInt("id"), rs.GetDateTime("startcas"), rs.GetDateTime("endcas"), rs.GetInt("startkm"), rs.GetInt("endkm"), rs.GetString("cesta"),
                         rs.GetString("ucel"), rs.GetInt("vozidloid"), rs.GetInt("ridicid"));
        }
    }
    return Jizda();
}

bool DbModel::deleteJizda(int id)
{
    tryExecuteUpdate(wxString::Format("DELETE FROM jizda WHERE id=%d", id));
    fireUpdated(ID_MODEL_JIZDA);
    return true;
}

bool DbModel::hasJizda(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM jizda WHERE id=%d", id));
    }
    return false;
}

wxArrayString DbModel::cesty()
{
    wxArrayString arr;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT DISTINCT cesta FROM jizda");
        while(rs.NextRow())
        {
            arr.Add(rs.GetString(0));
        }
        rs.Finalize();
    }
    return arr;
}

size_t DbModel::cestyCount()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar("SELECT count(DISTINCT cesta) FROM jizda");
    }
    return 0;
}

wxArrayString DbModel::ucely()
{
    wxArrayString arr;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT DISTINCT ucel FROM jizda");
        while(rs.NextRow())
        {
            arr.Add(rs.GetString(0));
        }
        rs.Finalize();
    }
    return arr;
}

size_t DbModel::ucelyCount()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar("SELECT count(DISTINCT ucel) FROM jizda");
    }
    return 0;
}

int DbModel::tachometrEnd(int vozidloid)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM jizda WHERE vozidloid=%d AND CAST(strftime('%%Y',endcas) AS INTEGER)<=%d", vozidloid, m_currentRok)))
        {
            return m_db.ExecuteScalar(wxString::Format("SELECT max(endkm) FROM jizda WHERE vozidloid=%d AND CAST(strftime('%%Y',endcas) AS INTEGER)<=%d", vozidloid, m_currentRok));
        }
    }
    return 0;
}

dxJizdaArray DbModel::jizdyByVozidlo(int vozidloid)
{
    dxJizdaArray jizdy;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT id,startcas,endcas,startkm,endkm,cesta,ucel,vozidloid,ridicid "
                                                                   "FROM jizda WHERE CAST(strftime('%%Y',jizda.endcas) AS INTEGER)=%d "
                                                                   "AND jizda.vozidloid=%d ORDER BY jizda.endcas",
                                                                   m_currentRok, vozidloid));
        while(rs.NextRow())
        {
            jizdy.Add(Jizda(rs.GetInt(0), rs.GetDateTime(1), rs.GetDateTime(2), rs.GetInt(3), rs.GetInt(4), rs.GetString(5), rs.GetString(6), rs.GetInt(7), rs.GetInt(8)));
        }
        rs.Finalize();
    }
    return jizdy;
}

wxString DbModel::homepagePrijmy()
{
    double celkem = 0, danove = 0;
    wxString prijmy = wxString::FromUTF8("<b>Příjmy:</b><br>");
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT sum(denik.castka) FROM denik INNER JOIN predkontace ON predkontace.id=denik.predkontaceid "
                                                                   "INNER JOIN deniksloupec ON deniksloupec.sloupec=predkontace.deniksloupecid WHERE deniksloupec.prijem=1 AND denik.obdobirok=%d", m_currentRok));
        if(rs.NextRow())
        {
            celkem = rs.GetDouble(0);
        }
        rs.Finalize();
        rs = m_db.ExecuteQuery(wxString::Format("SELECT sum(denik.castka) FROM denik INNER JOIN predkontace ON predkontace.id=denik.predkontaceid "
                                                "INNER JOIN deniksloupec ON deniksloupec.sloupec=predkontace.deniksloupecid WHERE deniksloupec.prijem=1 AND deniksloupec.danovy=1 AND denik.obdobirok=%d", m_currentRok));
        if(rs.NextRow())
        {
            danove = rs.GetDouble(0);
        }
        rs.Finalize();
    }
    prijmy.Append(wxString::Format(wxString::FromUTF8("Daňové: %.2f Kč<br>"), danove));
    prijmy.Append(wxString::Format(wxString::FromUTF8("Celkem: %.2f Kč<br>"), celkem));
    prijmy.Append("<br>");
    return prijmy;
}

wxString DbModel::homepageVydaje()
{
    double celkem = 0, danove = 0;
    wxString vydaje = wxString::FromUTF8("<b>Výdaje:</b><br>");
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT sum(denik.castka) FROM denik INNER JOIN predkontace ON predkontace.id=denik.predkontaceid "
                                                                   "INNER JOIN deniksloupec ON deniksloupec.sloupec=predkontace.deniksloupecid WHERE deniksloupec.prijem=0 AND denik.obdobirok=%d", m_currentRok));
        if(rs.NextRow())
        {
            celkem = rs.GetDouble(0);
        }
        rs.Finalize();
        rs = m_db.ExecuteQuery(wxString::Format("SELECT sum(denik.castka) FROM denik INNER JOIN predkontace ON predkontace.id=denik.predkontaceid "
                                                "INNER JOIN deniksloupec ON deniksloupec.sloupec=predkontace.deniksloupecid WHERE deniksloupec.prijem=0 AND deniksloupec.danovy=1 AND denik.obdobirok=%d", m_currentRok));
        if(rs.NextRow())
        {
            danove = rs.GetDouble(0);
        }
        rs.Finalize();
    }
    vydaje.Append(wxString::Format(wxString::FromUTF8("Daňové: %.2f Kč<br>"), danove));
    vydaje.Append(wxString::Format(wxString::FromUTF8("Celkem: %.2f Kč<br>"), celkem));
    vydaje.Append("<br>");
    return vydaje;
}

wxString DbModel::homepageVydane()
{
    wxString faktury = wxString::FromUTF8("<b>Vydané faktury:</b><br>");
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT sum(faktura.celkem),count(*),faktura.menaid,mena.kod FROM faktura INNER JOIN mena ON mena.id=faktura.menaid "
                                                           "WHERE faktura.vydana=1 AND faktura.obdobirok=%d GROUP BY menaid", m_currentRok));
        if(tb.GetRowCount())
        {
            for(int i=0; i < tb.GetRowCount(); i++)
            {
                tb.SetRow(i);
                double uhrazeno = m_db.ExecuteQuery(wxString::Format("SELECT sum(uhrada.castka) FROM uhrada WHERE uhrada.fakturaid IN "
                                                                     "(SELECT id FROM faktura WHERE vydana=1 AND menaid=%d AND obdobirok=%d)", tb.GetInt(2), m_currentRok)).GetDouble(0);
                faktury.Append(wxString::Format(wxString::FromUTF8("Vydáno faktur v %s: %d<br>"), tb.GetInt(2)==1?wxString::FromUTF8("Kč"):tb.GetString(3), tb.GetInt(1)));
                faktury.Append(wxString::Format("Za: %.2f %s<br>", tb.GetDouble(0), tb.GetInt(2)==1?wxString::FromUTF8("Kč"):tb.GetString(3)));
                faktury.Append(wxString::Format("Z toho uhrazeno: %.2f %s<br>", uhrazeno, tb.GetInt(2)==1?wxString::FromUTF8("Kč"):tb.GetString(3)));
                if(!dxutils::isSameDouble(tb.GetDouble(0), uhrazeno)) //Mozne nejake posplatnosti
                {
                    int posplatnosti = 0;
                    wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT id,celkem FROM faktura WHERE vydana=1 AND splatnost<'%s' AND menaid=%d",
                                                                               dxsettings.datum().FormatISODate(),tb.GetInt(2)));
                    while(rs.NextRow())
                    {
                        if(rs.GetDouble(1) > m_db.ExecuteQuery(wxString::Format("SELECT total(castka) FROM uhrada WHERE fakturaid=%d", rs.GetInt(0))).GetDouble(0)) posplatnosti++;
                    }
                    rs.Finalize();
                    if(posplatnosti) faktury.Append(wxString::Format("Z toho po splatnosti: <font color=\"red\">%d</font><br>", posplatnosti));
                }
                faktury.Append("<br>");
            }
        }
        else
        {
            faktury.Append(wxString::FromUTF8("Zatím nevydána žádná faktura<br><br>"));
        }
    }
    return faktury;
}

wxString DbModel::homepagePrijate()
{
    wxString faktury = wxString::FromUTF8("<b>Přijaté faktury:</b><br>");
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT sum(faktura.celkem),count(*),faktura.menaid,mena.kod FROM faktura INNER JOIN mena ON mena.id=faktura.menaid "
                                                           "WHERE faktura.vydana<>1 AND faktura.obdobirok=%d GROUP BY menaid", m_currentRok));
        if(tb.GetRowCount())
        {
            for(int i=0; i < tb.GetRowCount(); i++)
            {
                tb.SetRow(i);
                double uhrazeno = m_db.ExecuteQuery(wxString::Format("SELECT sum(uhrada.castka) FROM uhrada WHERE uhrada.fakturaid IN "
                                                                     "(SELECT id FROM faktura WHERE vydana<>1 AND menaid=%d AND obdobirok=%d)", tb.GetInt(2), m_currentRok)).GetDouble(0);
                faktury.Append(wxString::Format(wxString::FromUTF8("Přijato faktur v %s: %d<br>"), tb.GetInt(2)==1?wxString::FromUTF8("Kč"):tb.GetString(3), tb.GetInt(1)));
                faktury.Append(wxString::Format("Za: %.2f %s<br>", tb.GetDouble(0), tb.GetInt(2)==1?wxString::FromUTF8("Kč"):tb.GetString(3)));
                faktury.Append(wxString::Format("Z toho uhrazeno: %.2f %s<br>", uhrazeno, tb.GetInt(2)==1?wxString::FromUTF8("Kč"):tb.GetString(3)));
                if(!dxutils::isSameDouble(tb.GetDouble(0), uhrazeno)) //Mozne nejake posplatnosti
                {
                    int posplatnosti = 0;
                    wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT id,celkem FROM faktura WHERE vydana<>1 AND splatnost<'%s' AND menaid=%d",
                                                                               dxsettings.datum().FormatISODate(),tb.GetInt(2)));
                    while(rs.NextRow())
                    {
                        if(rs.GetDouble(1) > m_db.ExecuteQuery(wxString::Format("SELECT total(castka) FROM uhrada WHERE fakturaid=%d", rs.GetInt(0))).GetDouble(0)) posplatnosti++;
                    }
                    rs.Finalize();
                    if(posplatnosti) faktury.Append(wxString::Format("Z toho po splatnosti: <font color=\"red\">%d</font><br>", posplatnosti));
                }
                faktury.Append("<br>");
            }
        }
        else
        {
            faktury.Append(wxString::FromUTF8("Zatím nepřijata žádná faktura<br><br>"));
        }
    }
    return faktury;
}

wxString DbModel::homepagePohledavky()
{
    wxString zavpohy = wxString::FromUTF8("<b>Pohledávky:</b><br>");
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT sum(zavpoh.celkem),count(*),zavpoh.menaid,mena.kod FROM zavpoh INNER JOIN mena ON mena.id=zavpoh.menaid "
                                                           "WHERE zavpoh.zavazek<>1 AND zavpoh.obdobirok=%d GROUP BY menaid", m_currentRok));
        if(tb.GetRowCount())
        {
            for(int i=0; i < tb.GetRowCount(); i++)
            {
                tb.SetRow(i);
                double uhrazeno = m_db.ExecuteQuery(wxString::Format("SELECT sum(uhrada.castka) FROM uhrada WHERE uhrada.zavpohid IN "
                                                                     "(SELECT id FROM zavpoh WHERE zavazek<>1 AND menaid=%d AND obdobirok=%d)", tb.GetInt(2), m_currentRok)).GetDouble(0);
                zavpohy.Append(wxString::Format(wxString::FromUTF8("Pohledávky v %s: %d<br>"), tb.GetInt(2)==1?wxString::FromUTF8("Kč"):tb.GetString(3), tb.GetInt(1)));
                zavpohy.Append(wxString::Format("Za: %.2f %s<br>", tb.GetDouble(0), tb.GetInt(2)==1?wxString::FromUTF8("Kč"):tb.GetString(3)));
                zavpohy.Append(wxString::Format("Z toho uhrazeno: %.2f %s<br>", uhrazeno, tb.GetInt(2)==1?wxString::FromUTF8("Kč"):tb.GetString(3)));
                if(!dxutils::isSameDouble(tb.GetDouble(0), uhrazeno)) //Mozne nejake posplatnosti
                {
                    int posplatnosti = 0;
                    wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT id,celkem FROM zavpoh WHERE zavazek<>1 AND splatnost<'%s' AND menaid=%d",
                                                                               dxsettings.datum().FormatISODate(),tb.GetInt(2)));
                    while(rs.NextRow())
                    {
                        if(rs.GetDouble(1) > m_db.ExecuteQuery(wxString::Format("SELECT total(castka) FROM uhrada WHERE zavpohid=%d", rs.GetInt(0))).GetDouble(0)) posplatnosti++;
                    }
                    rs.Finalize();
                    if(posplatnosti) zavpohy.Append(wxString::Format("Z toho po splatnosti: <font color=\"red\">%d</font><br>", posplatnosti));
                }
                zavpohy.Append("<br>");
            }
        }
        else
        {
            zavpohy.Append(wxString::FromUTF8("Zatím žádné pohledávky<br><br>"));
        }
    }
    return zavpohy;
}

wxString DbModel::homepageZavazky()
{
    wxString zavpohy = wxString::FromUTF8("<b>Závazky:</b><br>");
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT sum(zavpoh.celkem),count(*),zavpoh.menaid,mena.kod FROM zavpoh INNER JOIN mena ON mena.id=zavpoh.menaid "
                                                           "WHERE zavpoh.zavazek=1 AND zavpoh.obdobirok=%d GROUP BY menaid", m_currentRok));
        if(tb.GetRowCount())
        {
            for(int i=0; i < tb.GetRowCount(); i++)
            {
                tb.SetRow(i);
                double uhrazeno = m_db.ExecuteQuery(wxString::Format("SELECT sum(uhrada.castka) FROM uhrada WHERE uhrada.zavpohid IN "
                                                                     "(SELECT id FROM zavpoh WHERE zavazek=1 AND menaid=%d AND obdobirok=%d)", tb.GetInt(2), m_currentRok)).GetDouble(0);
                zavpohy.Append(wxString::Format(wxString::FromUTF8("Závazky v %s: %d<br>"), tb.GetInt(2)==1?wxString::FromUTF8("Kč"):tb.GetString(3), tb.GetInt(1)));
                zavpohy.Append(wxString::Format("Za: %.2f %s<br>", tb.GetDouble(0), tb.GetInt(2)==1?wxString::FromUTF8("Kč"):tb.GetString(3)));
                zavpohy.Append(wxString::Format("Z toho uhrazeno: %.2f %s<br>", uhrazeno, tb.GetInt(2)==1?wxString::FromUTF8("Kč"):tb.GetString(3)));
                if(!dxutils::isSameDouble(tb.GetDouble(0), uhrazeno)) //Mozne nejake posplatnosti
                {
                    int posplatnosti = 0;
                    wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT id,celkem FROM zavpoh WHERE zavazek=1 AND splatnost<'%s' AND menaid=%d",
                                                                               dxsettings.datum().FormatISODate(),tb.GetInt(2)));
                    while(rs.NextRow())
                    {
                        if(rs.GetDouble(1) > m_db.ExecuteQuery(wxString::Format("SELECT total(castka) FROM uhrada WHERE zavpohid=%d", rs.GetInt(0))).GetDouble(0)) posplatnosti++;
                    }
                    rs.Finalize();
                    if(posplatnosti) zavpohy.Append(wxString::Format("Z toho po splatnosti: <font color=\"red\">%d</font><br>", posplatnosti));
                }
                zavpohy.Append("<br>");
            }
        }
        else
        {
            zavpohy.Append(wxString::FromUTF8("Zatím žádné závazky<br><br>"));
        }
    }
    return zavpohy;
}

Firma DbModel::firma() const
{
    return m_firma;
}

bool DbModel::isFirmaPlatce()
{
    return m_firma.platce();
}

bool DbModel::isFirmaPausal()
{
    return m_firma.pausal();
}

void DbModel::setFirma(const Firma &firma)
{
    m_firma = firma;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.IsOpen())
        {
            if(m_db.ExecuteScalar("SELECT count(*) FROM firma"))
            {
                try
                {
                    wxSQLite3Statement st = m_db.PrepareStatement("UPDATE firma SET nazev=?,ic=?,dic=?,registrace=?,"
                                                                  "platce=?,telefon=?,email=?,www=?,logo=?,razitko=?,pausal=?");
                    st.Bind(1, m_firma.nazev());
                    st.Bind(2, m_firma.ic());
                    st.Bind(3, m_firma.dic());
                    st.Bind(4, m_firma.registrace());
                    st.BindBool(5, m_firma.platce());
                    st.Bind(6, m_firma.telefon());
                    st.Bind(7, m_firma.email());
                    st.Bind(8, m_firma.www());
                    st.Bind(9, m_firma.logo());
                    st.Bind(10, m_firma.razitko());
                    st.BindBool(11, m_firma.pausal());
                    st.ExecuteUpdate();
                    st.Finalize();
                }
                catch(const wxSQLite3Exception& e)
                {
                    wxLogDebug(wxString::Format("Chyba v setFirma: %s", e.GetMessage()));
                }
                fireUpdated(ID_MODEL_ALL);
            }
            else
            {
                try
                {
                    wxSQLite3Statement st = m_db.PrepareStatement("INSERT INTO firma(nazev,ic,dic,registrace,platce,"
                                                                  "telefon,email,www,logo,razitko,adresaid,pausal) "
                                                                  "VALUES(?,?,?,?,?,?,?,?,?,?,?,?)");
                    st.Bind(1, m_firma.nazev());
                    st.Bind(2, m_firma.ic());
                    st.Bind(3, m_firma.dic());
                    st.Bind(4, m_firma.registrace());
                    st.BindBool(5, m_firma.platce());
                    st.Bind(6, m_firma.telefon());
                    st.Bind(7, m_firma.email());
                    st.Bind(8, m_firma.www());
                    st.Bind(9, m_firma.logo());
                    st.Bind(10, m_firma.razitko());
                    st.Bind(11, m_firma.adresaid());
                    st.BindBool(12, m_firma.pausal());
                    st.ExecuteUpdate();
                    st.Finalize();
                }
                catch(const wxSQLite3Exception& e)
                {
                    wxLogDebug(wxString::Format("Chyba v setFirma: %s", e.GetMessage()));
                }
            }
        }
    }
}

wxString DbModel::firmaLogo() const
{
    return m_firma.logo();
}

wxString DbModel::firmaRazitko() const
{
    return m_firma.razitko();
}

dxKontaktArray DbModel::kontakty()
{
    dxKontaktArray kontakty;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable("SELECT * FROM kontakt");
        if(!tb.GetRowCount()) return kontakty;
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            kontakty.Add(Kontakt(tb.GetInt("id"), tb.GetString("nazev"), tb.GetString("jmeno"), tb.GetString("ic"), tb.GetString("dic"), tb.GetString("email"),
                                   tb.GetBool("public"), tb.GetInt("adresaid")));
        }
    }
    return kontakty;
}

void DbModel::addKontakt(const Kontakt &kontakt)
{
    if(!hasKontakt(kontakt.id()) && hasAdresa(kontakt.adresaid()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("INSERT INTO kontakt(id,nazev,jmeno,ic,dic,email,public,adresaid) "
                                                              "VALUES(?,?,?,?,?,?,?,?)");
                st.Bind(1, kontakt.id());
                st.Bind(2, kontakt.nazev());
                st.Bind(3, kontakt.jmeno());
                st.Bind(4, kontakt.ic());
                st.Bind(5, kontakt.dic());
                st.Bind(6, kontakt.email());
                st.Bind(7, kontakt.publicItem());
                st.Bind(8, kontakt.adresaid());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v addKontakt: %s", e.GetMessage()));
            }
        }
    }
}

void DbModel::editKontakt(const Kontakt &kontakt)
{
    if(hasKontakt(kontakt.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("UPDATE kontakt SET nazev=?,jmeno=?,ic=?,dic=?,email=?,public=?,"
                                                              "adresaid=? WHERE id=?");
                st.Bind(1, kontakt.nazev());
                st.Bind(2, kontakt.jmeno());
                st.Bind(3, kontakt.ic());
                st.Bind(4, kontakt.dic());
                st.Bind(5, kontakt.email());
                st.Bind(6, kontakt.publicItem());
                st.Bind(7, kontakt.adresaid());
                st.Bind(8, kontakt.id());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v editKontakt: %s", e.GetMessage()));
            }
        }
    }
}

Kontakt DbModel::getKontakt(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM kontakt WHERE id=%d", id));
        if(rs.NextRow())
        {
            return Kontakt(rs.GetInt("id"), rs.GetString("nazev"), rs.GetString("jmeno"), rs.GetString("ic"), rs.GetString("dic"), rs.GetString("email"),
                           rs.GetBool("public"), rs.GetInt("adresaid"));
        }
        rs.Finalize();
    }
    return Kontakt();
}

bool DbModel::deleteKontakt(int id)
{
    tryExecuteUpdate(wxString::Format("DELETE FROM kontakt WHERE id=%d", id));
    return true;
}

bool DbModel::hasKontakt(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM kontakt WHERE id=%d", id));
    }
    return false;
}

bool DbModel::hasKontakt(const wxString &nazev)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM kontakt WHERE nazev='%s'", nazev));
    }
    return false;
}

int DbModel::getKontaktIdByNazev(const wxString &nazev)
{
    if(hasKontakt(nazev))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            return m_db.ExecuteScalar(wxString::Format("SELECT id FROM kontakt WHERE nazev='%s'", nazev));
        }
    }
    return -1;
}

wxString DbModel::getKontaktEmail(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteQuery(wxString::Format("SELECT email FROM kontakt WHERE id=%d", id)).GetString(0);
    }
    return wxEmptyString;
}

int DbModel::copyKontakt(int id)
{
    if(!hasKontakt(id)) return -1;
    Kontakt ko = getKontakt(id);
    int newaid = getLastId("adresa");
    Adresa ad = getAdresa(ko.adresaid());
    addAdresa(Adresa(newaid, ad.ulice(), ad.cp(), ad.misto(), ad.psc(), ad.zeme()));
    int newid = getLastId("kontakt");
    int i = 1;
    wxString nazev = wxString::Format("%s (%d)", ko.nazev(), i);
    while(hasKontakt(nazev))
    {
        nazev = wxString::Format("%s (%d)", ko.nazev(), i);
        i++;
    }
    addKontakt(Kontakt(newid, nazev, ko.jmeno(), ko.ic(), ko.dic(), ko.email(), ko.publicItem(), newaid));
    return newid;
}

bool DbModel::hasKontaktDic(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT dic FROM kontakt WHERE id=%d", id));
        if(rs.NextRow())
        {
            return !rs.GetString(0).IsEmpty();
        }
        rs.Finalize();
    }
    return false;
}

wxString DbModel::kontaktDic(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT dic FROM kontakt WHERE id=%d", id));
        if(rs.NextRow())
        {
            return rs.GetString(0);
        }
    }
    return wxEmptyString;
}

dxKontaktViewArray DbModel::kontaktyview()
{
    dxKontaktViewArray kontakty;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT kontakt.id,kontakt.nazev,kontakt.jmeno,kontakt.ic,adresa.ulice||' '||adresa.cp||', '||adresa.psc||' '||adresa.misto "
                                                  "FROM kontakt INNER JOIN adresa ON adresa.id=kontakt.adresaid WHERE kontakt.public=1");
        while(rs.NextRow())
        {
            kontakty.Add(KontaktView(rs.GetInt(0), rs.GetString(1), rs.GetString(2), rs.GetString(3), rs.GetString(4)));
        }
        rs.Finalize();
    }
    return kontakty;
}

void DbModel::addObdobi(const Obdobi &obdobi)
{
    if(!hasObdobi(obdobi.rok()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("INSERT INTO obdobi(rok,current) "
                                                              "VALUES(?,?)");
                st.Bind(1, obdobi.rok());
                st.Bind(2, obdobi.current());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v addObdobi: %s", e.GetMessage()));
            }
        }
    }
}

size_t DbModel::getObdobiCount()
{
    return tableRows("obdobi");
}

bool DbModel::deleteObdobi(int rok)
{
    tryExecuteUpdate(wxString::Format("DELETE FROM obdobi WHERE rok=%d", rok));
    updateStavZasob();
    return true;
}

bool DbModel::hasObdobi(int rok)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM obdobi WHERE rok=%d", rok));
    }
    return false;
}

bool DbModel::hasObdobiGreaterThan(int rok)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM obdobi WHERE rok>%d", rok));
    }
    return false;
}

wxArrayString DbModel::getObdobiAsStrings(int greater)
{
    wxArrayString strings;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT rok FROM obdobi WHERE rok>%d ORDER BY rok ASC", greater));
        while(rs.NextRow())
        {
            strings.Add(rs.GetAsString(0));
        }
    }
    return strings;
}

int DbModel::currentRok() const
{
    return m_currentRok;
}

void DbModel::setCurrentRok(int rok)
{
    if(m_currentRok == rok)
        return;
    m_currentRok = rok;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        try
        {
            tryExecuteUpdate("UPDATE obdobi SET current=0 WHERE current=1");
            if(hasObdobi(rok))
            {
                wxSQLite3Statement st = m_db.PrepareStatement("UPDATE obdobi SET current=1"
                                                              " WHERE rok=?");
                st.Bind(1, m_currentRok);
                st.ExecuteUpdate();
                st.Finalize();
            }
            else
            {
                wxSQLite3Statement st = m_db.PrepareStatement("INSERT INTO obdobi(rok,current) "
                                                              "VALUES(?,?)");
                st.Bind(1, rok);
                st.Bind(2, true);
                st.ExecuteUpdate();
                st.Finalize();
            }
        }
        catch(const wxSQLite3Exception& e)
        {
            wxLogDebug(wxString::Format("Chyba v setCurrentRok: %s", e.GetMessage()));
        }
        updateStavZasob();
    }
    fireUpdated(ID_MODEL_ALL);
}

int DbModel::maxRok()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar("SELECT max(rok) FROM obdobi");
    }
    return -1;
}

void DbModel::updateStavZasob(int zasobaid)
{
    if(hasZasoba(zasobaid)) //update stavu jen pro current rok;
    {
        dxPohybzasobArray pohyby = pohybyzasobByZasobaId(zasobaid);
        double pocet, celkem;
        getZasobaStav(!getZasoba(zasobaid).ocenovani(), pohyby, &pocet, &celkem);
        setZasobaStav(zasobaid, pocet, celkem);
    }
    else
    {
        dxZasobaArray zsby = zasoby();
        for(size_t i=0; i < zsby.GetCount(); i++)
        {
            double pocet = 0.0, celkem = 0.0;
            if(dxsettings.dbmodel() == DB_SQLITE)
            {
                if(m_db.TableExists("pohybzasob"))
                {
                    wxArrayInt roky;
                    wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT rok FROM obdobi ORDER BY rok ASC");
                    while(rs.NextRow())
                    {
                        if(rs.GetInt(0) <= m_currentRok) roky.Add(rs.GetInt(0));
                    }
                    rs.Finalize();
                    for(size_t j=0; j < roky.GetCount(); j++)
                    {
                        dxPohybzasobArray pohyby;
                        wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT * FROM pohybzasob WHERE obdobirok=%d AND zasobaid=%d ORDER BY datum ASC", roky[j], zsby[i].id()));
                        for(int k=0; k < tb.GetRowCount(); k++)
                        {
                            tb.SetRow(k);
                            if(j && !k) //create or check pocatek
                            {
                                if(tb.GetBool("pocatek"))
                                {
                                    wxSQLite3Statement st = m_db.PrepareStatement("UPDATE pohybzasob SET pocet=?,cenamj=? WHERE id=?");
                                    st.Bind(1, pocet);
                                    st.Bind(2, !wxIsNullDouble(pocet) ? celkem/pocet : 0.0);
                                    st.Bind(3, tb.GetInt("id"));
                                    st.ExecuteUpdate();
                                    st.Finalize();
                                    if(hasPohybzasob(tb.GetInt("id")))
                                    {
                                        getPohybzasob(tb.GetInt("id")).setCenamj(!wxIsNullDouble(pocet) ? celkem/pocet : 0.0);
                                        pohyby.Add(getPohybzasob(tb.GetInt("id")));
                                    }
                                }
                                else
                                {
                                    Pohybzasob pz = Pohybzasob(getLastId("pohybzasob"), wxDateTime(1,wxDateTime::Jan,roky[j]), wxEmptyString, pocet, !wxIsNullDouble(pocet) ? celkem/pocet : 0.0,
                                                                    true, true, -1, zsby[i].id(), roky[j], false, -1);
                                    wxSQLite3Statement st = m_db.PrepareStatement("INSERT INTO pohybzasob(id,datum,cislo,pocet,cenamj,"
                                                                                  "prijemka,pocatek,fakturaid,zasobaid,obdobirok,inventura,inventuraid) "
                                                                                  "VALUES(?,?,?,?,?,?,?,?,?,?,?,?)");
                                    st.Bind(1, pz.id());
                                    st.BindDateTime(2, pz.datum());
                                    st.Bind(3, pz.cislo());
                                    st.Bind(4, pz.pocet());
                                    st.Bind(5, pz.cenamj());
                                    st.BindBool(6, pz.prijemka());
                                    st.BindBool(7, pz.pocatek());
                                    st.BindNull(8);
                                    st.Bind(9, pz.zasobaid());
                                    st.Bind(10, pz.obdobirok());
                                    st.BindBool(11, pz.inventura());
                                    st.BindNull(12);
                                    st.ExecuteUpdate();
                                    st.Finalize();
                                    pohyby.Add(pz);
                                }
                            }
                            else
                            {
                                pohyby.Add(Pohybzasob(tb.GetInt("id"), tb.GetDateTime("datum"), tb.GetString("cislo"), tb.GetDouble("pocet"), tb.GetDouble("cenamj"), tb.GetBool("prijemka"),
                                                          tb.GetBool("pocatek"), tb.GetInt("fakturaid"), tb.GetInt("zasobaid"), tb.GetInt("obdobirok"), tb.GetBool("inventura"), tb.GetInt("inventuraid")));
                            }
                        }
                        if(!tb.GetRowCount() && j)
                        {
                            Pohybzasob pz = Pohybzasob(getLastId("pohybzasob"), wxDateTime(1,wxDateTime::Jan,roky[j]), wxEmptyString, pocet, !wxIsNullDouble(pocet) ? celkem/pocet : 0.0,
                                                            true, true, -1, zsby[i].id(), roky[j], false, -1);
                            wxSQLite3Statement st = m_db.PrepareStatement("INSERT INTO pohybzasob(id,datum,cislo,pocet,cenamj,"
                                                                          "prijemka,pocatek,fakturaid,zasobaid,obdobirok,inventura,inventuraid) "
                                                                          "VALUES(?,?,?,?,?,?,?,?,?,?,?,?)");
                            st.Bind(1, pz.id());
                            st.BindDateTime(2, pz.datum());
                            st.Bind(3, pz.cislo());
                            st.Bind(4, pz.pocet());
                            st.Bind(5, pz.cenamj());
                            st.BindBool(6, pz.prijemka());
                            st.BindBool(7, pz.pocatek());
                            st.BindNull(8);
                            st.Bind(9, pz.zasobaid());
                            st.Bind(10, pz.obdobirok());
                            st.BindBool(11, pz.inventura());
                            st.BindNull(12);
                            st.ExecuteUpdate();
                            st.Finalize();
                            pohyby.Add(pz);
                        }
                        getZasobaStav(!zsby[i].ocenovani(), pohyby, &pocet, &celkem);
                    }
                }
            }
            setZasobaStav(zsby[i].id(), pocet, celkem);
        }
    }
}

dxDphSazbaArray DbModel::dphsazby()
{
    dxDphSazbaArray arr;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT * FROM dphsazba WHERE rok=%d ORDER BY sazba DESC", m_currentRok));
        if(!tb.GetRowCount()) return arr;
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            arr.Add(new DphSazba(tb.GetInt("id"), tb.GetInt("sazba"), tb.GetInt("rok"), tb.GetBool("zakladni")));
        }
        arr.Add(new DphSazba(0,0,-1,0)); //nulova sazba dph
    }
    return arr;
}

DphSazba DbModel::getDphSazba(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM dphsazba WHERE id=%d", id));
        while(rs.NextRow())
        {
            return DphSazba(rs.GetInt("id"), rs.GetInt("sazba"), rs.GetInt("rok"), rs.GetBool("zakladni"));
        }
        rs.Finalize();
    }
    return DphSazba();
}

bool DbModel::hasDphSazba(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM dphsazba WHERE id=%d", id));
    }
    return false;
}

int DbModel::getDphSazbaValue(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT sazba FROM dphsazba WHERE id=%d", id));
    }
    return 0;
}

int DbModel::getDphSazbaId(int value)
{
    if(value == 0) return 0;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT id FROM dphsazba WHERE sazba=%d AND rok=%d", value, m_currentRok));
    }
    return 0;
}

int DbModel::zakladniSazba(int rok)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT sazba FROM dphsazba WHERE rok=%d AND zakladni=1", rok==-1?m_currentRok:rok));
    }
    return 0;
}

int DbModel::prvniSnizenaSazba(int rok)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        int sazba = 0;
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT sazba FROM dphsazba WHERE rok=%d AND zakladni=0", rok==-1?m_currentRok:rok));
        while(rs.NextRow())
        {
            if(rs.GetInt(0) > sazba) sazba = rs.GetInt(0);
        }
        rs.Finalize();
        return sazba;
    }
    return 0;
}

int DbModel::druhaSnizenaSazba(int rok)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        int sazba = 100;
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT sazba FROM dphsazba WHERE rok=%d AND zakladni=0", rok==-1?m_currentRok:rok));
        while(rs.NextRow())
        {
            if(rs.GetInt(0) < sazba) sazba = rs.GetInt(0);
        }
        rs.Finalize();
        return sazba==100?0:sazba;
    }
    return 0;
}

dxDphDruhArray DbModel::dphdruhy()
{
    dxDphDruhArray dphdruhy;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable("SELECT * FROM dphdruh");
        if(!tb.GetRowCount()) return dphdruhy;
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            dphdruhy.Add(DphDruh(tb.GetInt("id"), tb.GetString("zkratka"), tb.GetString("popis"), tb.GetString("radky"), tb.GetBool("plneni"),
                                       tb.GetBool("vydaj"), tb.GetBool("nabizet"), tb.GetString("kpp"), tb.GetInt("krp"), tb.GetDate("od"), tb.GetDate("do"), tb.GetBool("kraceny"), tb.GetString("kh")));
        }
    }
    return dphdruhy;
}

DphDruh DbModel::getDphDruh(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM dphdruh WHERE id=%d", id));
        while(rs.NextRow())
        {
            return DphDruh(rs.GetInt("id"), rs.GetString("zkratka"), rs.GetString("popis"), rs.GetString("radky"), rs.GetBool("plneni"),
                           rs.GetBool("vydaj"), rs.GetBool("nabizet"), rs.GetString("kpp"), rs.GetInt("krp"), rs.GetDate("od"), rs.GetDate("do"), rs.GetBool("kraceny"), rs.GetString("kh"));
        }
        rs.Finalize();
    }
    return DphDruh();
}

dxDphDruhArray DbModel::dphdruhyNabizenePrijem()
{
    dxDphDruhArray nabizene;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable("SELECT * FROM dphdruh WHERE nabizet=1 AND vydaj=0");
        if(!tb.GetRowCount()) return nabizene;
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            nabizene.Add(DphDruh(tb.GetInt("id"), tb.GetString("zkratka"), tb.GetString("popis"), tb.GetString("radky"), tb.GetBool("plneni"),
                                       tb.GetBool("vydaj"), tb.GetBool("nabizet"), tb.GetString("kpp"), tb.GetInt("krp"), tb.GetDate("od"), tb.GetDate("do"), tb.GetBool("kraceny"), tb.GetString("kh")));
        }
    }
    return nabizene;
}

dxDphDruhArray DbModel::dphdruhyNabizeneVydaj()
{
    dxDphDruhArray nabizene;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable("SELECT * FROM dphdruh WHERE nabizet=1 AND vydaj=1");
        if(!tb.GetRowCount()) return nabizene;
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            nabizene.Add(DphDruh(tb.GetInt("id"), tb.GetString("zkratka"), tb.GetString("popis"), tb.GetString("radky"), tb.GetBool("plneni"),
                                       tb.GetBool("vydaj"), tb.GetBool("nabizet"), tb.GetString("kpp"), tb.GetInt("krp"), tb.GetDate("od"), tb.GetDate("do"), tb.GetBool("kraceny"), tb.GetString("kh")));
        }
    }
    return nabizene;
}

bool DbModel::hasDphDruh(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM dphdruh WHERE id=%d", id));
    }
    return false;
}

bool DbModel::hasDphDruh(const wxString &zkratka)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM dphdruh WHERE zkratka='%s'", zkratka));
    }
    return false;
}

int DbModel::getDphDruhIdByZkratka(const wxString &zkratka)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT id FROM dphdruh WHERE zkratka='%s'", zkratka));
    }
    return -1;
}

wxArrayString DbModel::getDphDruhKpp()
{
    wxArrayString kpp;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT DISTINCT IFNULL(kpp,0) FROM dphdruh ORDER BY kpp ASC");
        while(rs.NextRow())
        {
            if(rs.GetInt(0)) kpp.Add(rs.GetAsString(0));
        }
        rs.Finalize();
    }
    return kpp;
}

wxArrayString DbModel::getDphDruhRadky()
{
    wxArrayString radky;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT DISTINCT radky FROM dphdruh ORDER BY radky ASC");
        while(rs.NextRow())
        {
            radky.Add(rs.GetString(0));
        }
        rs.Finalize();
    }
    return radky;
}

wxArrayString DbModel::getDphDruhKh()
{
    wxArrayString kh;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::FromUTF8("SELECT DISTINCT IFNULL(kh,'Nepoužito') FROM dphdruh ORDER BY kh ASC"));
        while(rs.NextRow())
        {
            kh.Add(rs.GetString(0));
        }
        rs.Finalize();
    }
    return kh;
}

wxArrayInt DbModel::getDphDruhIdByRadek(const wxString &radky)
{
    wxArrayInt ids;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT id FROM dphdruh WHERE radky='%s'", radky));
        while(rs.NextRow())
        {
            ids.Add(rs.GetInt(0));
        }
    }
    return ids;
}

wxArrayInt DbModel::getDphDruhIdByRadek(const wxArrayString &radky)
{
    wxArrayInt ids;
    for(size_t i=0; i<radky.GetCount(); i++)
    {
        wxArrayInt ia = idbmodel->getDphDruhIdByRadek(radky[i]);
        for(size_t j=0; j<ia.GetCount(); j++)
        {
            ids.Add(ia[j]);
        }
    }
    return ids;
}

wxArrayInt DbModel::getDphDruhIdByKh(const wxString &kh)
{
    wxArrayInt ids;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT id FROM dphdruh WHERE kh='%s'", kh));
        while(rs.NextRow())
        {
            ids.Add(rs.GetInt(0));
        }
    }
    return ids;
}

void DbModel::nabizetDphDruh(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        try
        {
            wxSQLite3Statement st = m_db.PrepareStatement("UPDATE dphdruh SET nabizet=CASE WHEN nabizet=1 THEN 0 ELSE 1 END WHERE id=?");
            st.Bind(1, id);
            st.ExecuteUpdate();
            st.Finalize();
        }
        catch(const wxSQLite3Exception& e)
        {
            wxLogDebug(wxString::Format("nabizetDphDruh: %s", e.GetMessage()));
        }
    }
}

bool DbModel::someDokladForDph(const wxDateTime &start, const wxDateTime &end)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM faktura WHERE platce = 1 AND ((dppd BETWEEN '%s' AND '%s' AND vydana = 0) "
                                               "OR (duzp BETWEEN '%s' AND '%s' AND vydana = 1))", start.FormatISODate(), end.FormatISODate(),
                                               start.FormatISODate(), end.FormatISODate())))
            return true;
        if(m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM zavpoh WHERE platce = 1 AND ((dppd BETWEEN '%s' AND '%s' AND zavazek = 1) "
                                               "OR (duzp BETWEEN '%s' AND '%s' AND zavazek = 0))", start.FormatISODate(), end.FormatISODate(),
                                               start.FormatISODate(), end.FormatISODate())))
            return true;
    }
    return false;
}

bool DbModel::someDokladForDph(int id, const wxDateTime &start, const wxDateTime &end, bool mustDic)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT protistranaid FROM faktura WHERE platce = 1 AND ((dppd BETWEEN '%s' AND '%s' AND vydana = 0 AND dphdruhid = %d) "
                                                           "OR (duzp BETWEEN '%s' AND '%s' AND vydana = 1 AND dphdruhid = %d))", start.FormatISODate(), end.FormatISODate(), id,
                                                           start.FormatISODate(), end.FormatISODate(), id));
        if(mustDic)
        {
            for(int i=0; i < tb.GetRowCount(); i++)
            {
                tb.SetRow(i);
                if(hasKontaktDic(tb.GetInt(0))) return true;
            }
        }
        else
        {
            if(tb.GetRowCount()) return true;
        }
        tb = m_db.GetTable(wxString::Format("SELECT protistranaid FROM zavpoh WHERE platce = 1 AND ((dppd BETWEEN '%s' AND '%s' AND zavazek = 1 AND dphdruhid = %d) "
                                            "OR (duzp BETWEEN '%s' AND '%s' AND zavazek = 0 AND dphdruhid = %d))", start.FormatISODate(), end.FormatISODate(), id,
                                            start.FormatISODate(), end.FormatISODate(), id));
        if(mustDic)
        {
            for(int i=0; i < tb.GetRowCount(); i++)
            {
                tb.SetRow(i);
                if(hasKontaktDic(tb.GetInt(0))) return true;
            }
        }
        else
        {
            if(tb.GetRowCount()) return true;
        }
    }
    return false;
}

bool DbModel::someDokladForDph(wxArrayInt ids, const wxDateTime &start, const wxDateTime &end, bool mustDic)
{
    for(size_t i=0; i < ids.GetCount(); i++)
    {
        if(someDokladForDph(ids[i], start, end, mustDic)) return true;
    }
    return false;
}

bool DbModel::dphPlatiZakaznik(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM dphdruh WHERE id=%d AND radky LIKE '%%25%%'", id));
    }
    return false;
}

bool DbModel::kracenyDphDruh(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT kraceny FROM dphdruh WHERE id=%d", id));
    }
    return false;
}

bool DbModel::someDokladForDphSh(const wxDateTime &start, const wxDateTime &end)
{
    wxArrayInt ids = getDphDruhIdByRadek("20");
    if(someDokladForDph(ids, start, end, true)) return true;
    ids = getDphDruhIdByRadek("21");
    if(someDokladForDph(ids, start, end, true)) return true;
    ids = getDphDruhIdByRadek("31");
    if(someDokladForDph(ids, start, end, true)) return true;
    return false;
}

bool DbModel::someDokladForDphKh(const wxDateTime &start, const wxDateTime &end)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT kh FROM dphdruh WHERE id IN (SELECT dphdruhid FROM faktura WHERE (dppd BETWEEN '%s' AND '%s' AND vydana = 0) "
                                                           "OR (duzp BETWEEN '%s' AND '%s' AND vydana = 1))", start.FormatISODate(), end.FormatISODate(),
                                                           start.FormatISODate(), end.FormatISODate()));
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            if(!tb.GetString(0).IsEmpty()) return true;
        }
        tb = m_db.GetTable(wxString::Format("SELECT kh FROM dphdruh WHERE id IN (SELECT dphdruhid FROM zavpoh WHERE (dppd BETWEEN '%s' AND '%s' AND zavazek = 1) "
                                            "OR (duzp BETWEEN '%s' AND '%s' AND zavazek = 0))", start.FormatISODate(), end.FormatISODate(),
                                            start.FormatISODate(), end.FormatISODate()));
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            if(!tb.GetString(0).IsEmpty()) return true;
        }
    }
    return false;
}

bool DbModel::someDokladForDphKhA1(const wxDateTime &start, const wxDateTime &end)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT kh FROM dphdruh WHERE id IN (SELECT dphdruhid FROM faktura WHERE (dppd BETWEEN '%s' AND '%s' AND vydana = 0) "
                                                           "OR (duzp BETWEEN '%s' AND '%s' AND vydana = 1))", start.FormatISODate(), end.FormatISODate(),
                                                           start.FormatISODate(), end.FormatISODate()));
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            if(tb.GetString(0) == "A1")
            {
                return true;
            }
        }
        tb = m_db.GetTable(wxString::Format("SELECT kh FROM dphdruh WHERE id IN (SELECT dphdruhid FROM zavpoh WHERE (dppd BETWEEN '%s' AND '%s' AND zavazek = 1) "
                                            "OR (duzp BETWEEN '%s' AND '%s' AND zavazek = 0))", start.FormatISODate(), end.FormatISODate(),
                                            start.FormatISODate(), end.FormatISODate()));
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            if(tb.GetString(0) == "A1")
            {
                return true;
            }
        }
    }
    return false;
}

dxDphDokladArray DbModel::dokladyForDph(int dphdruhid, const wxDateTime &start, const wxDateTime &end, bool mustDic)
{
    dxDphDokladArray doklady;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT * FROM faktura WHERE dphdruhid=%d AND ((dppd BETWEEN '%s' AND '%s' AND vydana = 0) "
                                                           "OR (duzp BETWEEN '%s' AND '%s' AND vydana = 1))", dphdruhid, start.FormatISODate(), end.FormatISODate(),
                                                           start.FormatISODate(), end.FormatISODate()));
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            if(mustDic && !hasKontaktDic(tb.GetInt("protistranaid"))) continue;
            double kurz = tb.GetDouble("kurz");
            dxDphSazbaArray dphsazby;
            wxSQLite3Table tbp = m_db.GetTable(wxString::Format("SELECT * FROM dphsazba WHERE rok=%d ORDER BY sazba DESC", tb.GetInt("obdobirok")));
            for(int j=0; j < tbp.GetRowCount(); j++)
            {
                tbp.SetRow(j);
                dphsazby.Add(DphSazba(tbp.GetInt("id"), tbp.GetInt("sazba"), tbp.GetInt("rok"), tbp.GetBool("zakladni")));
            }
            dphsazby.Add(DphSazba(0,0,-1,0)); //nulova sazba dph
            dxFaPolozkaArray polozky;
            tbp = m_db.GetTable(wxString::Format("SELECT * FROM fapolozka WHERE fakturaid=%d", tb.GetInt("id")));
            for(int j=0; j < tbp.GetRowCount(); j++)
            {
                tbp.SetRow(j);
                polozky.Add(FaPolozka(tbp.GetInt("id"), tbp.GetString("popis"), tbp.GetString("mj"), tbp.GetDouble("pocet"),tbp.GetDouble("cenamj"), tbp.GetInt("sazbaid"),
                                      tbp.GetDouble("dph"), tbp.GetDouble("celkem"), tbp.GetBool("bezdph"), tbp.GetInt("fakturaid"), tbp.GetInt("zavpohid"), tbp.GetInt("zasobaid"),
                                      tbp.GetBool("zaokrouhleni")));
            }
            dxDphForDokladArray dphfor;
            for(size_t j=0; j < dphsazby.GetCount(); j++)
            {
                double zaklad = 0.0, dph = 0.0;
                for(size_t k=0; k < polozky.GetCount(); k++)
                {
                    if(polozky[k].sazbaid() == dphsazby[j].id())
                    {
                        zaklad += (polozky[k].celkem()-polozky[k].dph())*kurz;
                        dph += polozky[k].dph()*kurz;
                    }
                }
                dphfor.Add(DphForDoklad(dphsazby[j].sazba(), dphsazby[j].zakladni(), zaklad, dph));
            }
            doklady.Add(DphDoklad(tb.GetString("cislo"), kontaktDic(tb.GetInt("protistranaid")), tb.GetDate("duzp"), tb.GetDate("dppd"),
                                  tb.GetString("popis"), tb.GetBool("vydana")?"FV":"FP", dphfor, tb.GetDouble("celkem"),
                                  getDphDruh(dphdruhid).kpp(), wxString::Format("%d", getDphDruh(dphdruhid).krp()), tb.GetInt("protistranaid")));
        }
        tb = m_db.GetTable(wxString::Format("SELECT * FROM zavpoh WHERE dphdruhid=%d AND ((dppd BETWEEN '%s' AND '%s' AND zavazek = 1) "
                                                            "OR (duzp BETWEEN '%s' AND '%s' AND zavazek = 0))", dphdruhid, start.FormatISODate(), end.FormatISODate(),
                                                            start.FormatISODate(), end.FormatISODate()));
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            if(mustDic && !hasKontaktDic(tb.GetInt("protistranaid"))) continue;
            double kurz = tb.GetDouble("kurz");
            dxDphSazbaArray dphsazby;
            wxSQLite3Table tbp = m_db.GetTable(wxString::Format("SELECT * FROM dphsazba WHERE rok=%d ORDER BY sazba DESC", tb.GetInt("obdobirok")));
            for(int j=0; j < tbp.GetRowCount(); j++)
            {
                tbp.SetRow(j);
                dphsazby.Add(DphSazba(tbp.GetInt("id"), tbp.GetInt("sazba"), tbp.GetInt("rok"), tbp.GetBool("zakladni")));
            }
            dphsazby.Add(DphSazba(0,0,-1,0)); //nulova sazba dph
            dxFaPolozkaArray polozky;
            tbp = m_db.GetTable(wxString::Format("SELECT * FROM fapolozka WHERE zavpohid=%d", tb.GetInt("id")));
            for(int j=0; j < tbp.GetRowCount(); j++)
            {
                tbp.SetRow(j);
                polozky.Add(FaPolozka(tbp.GetInt("id"), tbp.GetString("popis"), tbp.GetString("mj"), tbp.GetDouble("pocet"),tbp.GetDouble("cenamj"), tbp.GetInt("sazbaid"),
                                          tbp.GetDouble("dph"), tbp.GetDouble("celkem"), tbp.GetBool("bezdph"), tbp.GetInt("fakturaid"), tbp.GetInt("zavpohid"),
                                          tbp.GetInt("zasobaid"), tbp.GetBool("zaokrouhleni")));
            }
            dxDphForDokladArray dphfor;
            for(size_t j=0; j < dphsazby.GetCount(); j++)
            {
                double zaklad = 0.0, dph = 0.0;
                for(size_t k=0; k < polozky.GetCount(); k++)
                {
                    if(polozky[k].sazbaid() == dphsazby[j].id())
                    {
                        zaklad += (polozky[k].celkem()-polozky[k].dph())*kurz;
                        dph += polozky[k].dph()*kurz;
                    }
                }
                dphfor.Add(DphForDoklad(dphsazby[j].sazba(), dphsazby[j].zakladni(), zaklad, dph));
            }
            doklady.Add(DphDoklad(tb.GetString("cislo"), kontaktDic(tb.GetInt("protistranaid")), tb.GetDate("duzp"), tb.GetDate("dppd"),
                                  tb.GetString("popis"), tb.GetBool("zavazek")?"ZA":"PO", dphfor, tb.GetDouble("celkem"),
                                  getDphDruh(dphdruhid).kpp(), wxString::Format("%d", getDphDruh(dphdruhid).krp()), tb.GetInt("protistranaid")));
        }
    }
    return doklady;
}

dxDphDokladArray DbModel::dokladyForDph(wxArrayInt dphdruhids, const wxDateTime &start, const wxDateTime &end, bool mustDic)
{
    dxDphDokladArray doklady;
    for(size_t i=0; i < dphdruhids.GetCount(); i++)
    {
        dxDphDokladArray ds = dokladyForDph(dphdruhids[i], start, end, mustDic);
        for(size_t j=0; j < ds.GetCount(); j++)
        {
            doklady.Add(ds[j]);
        }
    }
    return doklady;
}

bool DbModel::isDphSazbySameForDoklad(dxDphForDokladArray arr1, dxDphForDokladArray arr2)
{
    if(arr1.GetCount() != arr2.GetCount()) return false;
    for(size_t i=0; i < arr1.GetCount(); i++)
    {
        if(arr1[i].sazba() != arr2[i].sazba()) return false;
    }
    return true;
}

bool DbModel::isDphDruhValidForDatum(const wxString &zkratka, const wxDateTime &datum)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM dphdruh WHERE zkratka='%s' AND od <= date('%s') AND do >= date('%s')", zkratka, datum.FormatISODate(), datum.FormatISODate()));
    }
    return false;
}

dxMenaArray DbModel::meny()
{
    dxMenaArray meny;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT * FROM mena WHERE CAST(strftime('%%Y',od) AS INTEGER)<=%d AND CAST(strftime('%%Y',do) AS INTEGER)>=%d", m_currentRok, m_currentRok));
        if(!tb.GetRowCount()) return meny;
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            meny.Add(Mena(tb.GetInt("id"), tb.GetString("kod"), tb.GetString("nazev")));
        }
    }
    return meny;
}

Mena DbModel::getMena(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM mena WHERE id=%d", id));
        while(rs.NextRow())
        {
            return Mena(rs.GetInt("id"), rs.GetString("kod"), rs.GetString("nazev"));
        }
        rs.Finalize();
    }
    return Mena();
}

Mena DbModel::getMena(const wxString &kod)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM mena WHERE id='%s'", kod));
        while(rs.NextRow())
        {
            return Mena(rs.GetInt("id"), rs.GetString("kod"), rs.GetString("nazev"));
        }
        rs.Finalize();
    }
    return Mena();
}

Mena DbModel::itemMena(size_t index)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT * FROM mena WHERE CAST(strftime('%%Y',od) AS INTEGER)<=%d AND CAST(strftime('%%Y',do) AS INTEGER)>=%d", m_currentRok, m_currentRok));
        if(!tb.GetRowCount()) return Mena();
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            if(i == (int)index)
            {
                return Mena(tb.GetInt("id"), tb.GetString("kod"), tb.GetString("nazev"));
            }
        }
    }
    return Mena();
}

bool DbModel::hasMena(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM mena WHERE id=%d", id));
    }
    return false;
}

bool DbModel::hasMena(const wxString &kod)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM mena WHERE kod='%s'", kod));
    }
    return false;
}

wxString DbModel::zkratkaMena(int id)
{
    if(id == 1) return wxString::FromUTF8("Kč");
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteQuery(wxString::Format("SELECT kod FROM mena WHERE id=%d", id)).GetString(0);
    }
    return "";
}

wxString DbModel::nazevMena(int id)
{
    if(id == 1) return wxString::FromUTF8("Kč - Česká koruna");
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteQuery(wxString::Format("SELECT kod||' - '||nazev FROM mena WHERE id=%d", id)).GetString(0);
    }
    return "";
}

wxString DbModel::kodMena(int id)
{
    if(hasMena(id))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            return m_db.ExecuteQuery(wxString::Format("SELECT kod FROM mena WHERE id=%d", id)).GetString(0);
        }
    }
    return "CZK";
}

DenikSloupec DbModel::getDenikSloupec(int sloupec)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM deniksloupec WHERE sloupec=%d", sloupec));
        while(rs.NextRow())
        {
            return DenikSloupec(rs.GetInt("sloupec"), rs.GetString("popis"), rs.GetBool("prijem"), rs.GetBool("danovy"));
        }
        rs.Finalize();
    }
    return DenikSloupec();
}

bool DbModel::hasDenikSloupec(int sloupec)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM deniksloupec WHERE sloupec=%d", sloupec));
    }
    return false;
}

dxDenikSloupecArray DbModel::deniksloupcePrijem()
{
    dxDenikSloupecArray sloupce;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT * FROM deniksloupec WHERE prijem=1");
        while(rs.NextRow())
        {
            sloupce.Add(DenikSloupec(rs.GetInt("sloupec"), rs.GetString("popis"), rs.GetBool("prijem"), rs.GetBool("danovy")));
        }
        rs.Finalize();
    }
    return sloupce;
}

dxDenikSloupecArray DbModel::deniksloupceVydaj()
{
    dxDenikSloupecArray sloupce;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT * FROM deniksloupec WHERE prijem=0");
        while(rs.NextRow())
        {
            sloupce.Add(DenikSloupec(rs.GetInt("sloupec"), rs.GetString("popis"), rs.GetBool("prijem"), rs.GetBool("danovy")));
        }
        rs.Finalize();
    }
    return sloupce;
}

dxPredkontaceArray DbModel::predkontace()
{
    dxPredkontaceArray predkontace;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable("SELECT * FROM predkontace");
        if(!tb.GetRowCount()) return predkontace;
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            predkontace.Add(Predkontace(tb.GetInt("id"), tb.GetString("zkratka"), tb.GetString("popis"), tb.GetBool("prijem"), tb.GetInt("deniksloupecid")));
        }
    }
    return predkontace;
}

void DbModel::addPredkontace(const Predkontace &predkontace)
{
    if(hasPredkontace(predkontace.id())
            &&hasPredkontace(predkontace.zkratka()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("INSERT INTO predkontace(id,zkratka,popis,prijem,deniksloupecid) "
                                                              "VALUES(?,?,?,?,?)");
                st.Bind(1, predkontace.id());
                st.Bind(2, predkontace.zkratka());
                st.Bind(3, predkontace.popis());
                st.Bind(4, predkontace.prijem());
                st.Bind(5, predkontace.deniksloupecid());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v addPredkontace: %s", e.GetMessage()));
            }
        }
    }
}

void DbModel::editPredkontace(const Predkontace &predkontace)
{
    if(hasPredkontace(predkontace.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("UPDATE predkontace SET zkratka=?,popis=?,prijem=?,deniksloupecid=?"
                                                              " WHERE id=?");
                st.Bind(1, predkontace.zkratka());
                st.Bind(2, predkontace.popis());
                st.Bind(3, predkontace.prijem());
                st.Bind(4, predkontace.deniksloupecid());
                st.Bind(5, predkontace.id());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v editPredkontace: %s", e.GetMessage()));
            }
        }
    }
}

Predkontace DbModel::getPredkontace(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM predkontace WHERE id=%d", id));
        while(rs.NextRow())
        {
            return Predkontace(rs.GetInt("id"), rs.GetString("zkratka"), rs.GetString("popis"), rs.GetBool("prijem"), rs.GetInt("deniksloupecid"));
        }
    }
    return Predkontace();
}

Predkontace DbModel::getPredkontace(const wxString &zkratka)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM predkontace WHERE zkratka='%s'", zkratka));
        while(rs.NextRow())
        {
            return Predkontace(rs.GetInt("id"), rs.GetString("zkratka"), rs.GetString("popis"), rs.GetBool("prijem"), rs.GetInt("deniksloupecid"));
        }
    }
    return Predkontace();
}

bool DbModel::deletePredkontace(int id)
{
    if(isPredkontaceUsed(id))
    {
        Predkontace pd = getPredkontace(id);
        fireMsg(wxString::Format(wxString::FromUTF8("Nelze smazat předkontaci\n%s\n%s\nprotože je použita v dokladech"), pd.zkratka(), pd.popis()));
        return false;
    }
    Predkontace predkontace = getPredkontace(id);
    if(predkontace.isValid())
    {
        if(predkontace.zkratka() == "P102" || predkontace.zkratka() == "V102") // predkontace pro DPH prijem a vydaj
        {
            return false;
        }
        tryExecuteUpdate(wxString::Format("DELETE FROM predkontace WHERE id=%d", id));
        return true;
    }
    return false;
}

bool DbModel::hasPredkontace(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM predkontace WHERE id=%d", id));
    }
    return false;
}

bool DbModel::hasPredkontace(const wxString &zkratka)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM predkontace WHERE zkratka='%s'", zkratka));
    }
    return false;
}

bool DbModel::isPredkontacePrijem(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT prijem FROM predkontace WHERE id=%d", id));
    }
    return false;
}

bool DbModel::isPredkontaceDanova(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT danovy FROM deniksloupec WHERE sloupec=(SELECT deniksloupecid FROM predkontace WHERE id=%d)", id));
    }
    return false;
}

dxPredkontaceArray DbModel::predkontacePrijem()
{
    dxPredkontaceArray predkontace;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT * FROM predkontace WHERE prijem=1");
        while(rs.NextRow())
        {
            predkontace.Add(Predkontace(rs.GetInt("id"), rs.GetString("zkratka"), rs.GetString("popis"), rs.GetBool("prijem"), rs.GetInt("deniksloupecid")));
        }
        rs.Finalize();
    }
    return predkontace;
}

dxPredkontaceArray DbModel::predkontaceVydaj()
{
    dxPredkontaceArray predkontace;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT * FROM predkontace WHERE prijem=0");
        while(rs.NextRow())
        {
            predkontace.Add(Predkontace(rs.GetInt("id"), rs.GetString("zkratka"), rs.GetString("popis"), rs.GetBool("prijem"), rs.GetInt("deniksloupecid")));
        }
        rs.Finalize();
    }
    return predkontace;
}

int DbModel::getPredkontaceIdByZkratka(const wxString &zkratka)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT id FROM predkontace WHERE zkratka='%s'", zkratka));
    }
    return -1;
}

int DbModel::copyPredkontace(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        Predkontace pk = getPredkontace(id);
        int newid = getLastId("predkontace");
        int i = 1;
        wxString zkratka = wxString::Format("%s%d", pk.zkratka(), i);
        while(hasPredkontace(zkratka))
        {
            zkratka = wxString::Format("%s%d", pk.zkratka(), i);
            i++;
        }
        addPredkontace(Predkontace(newid, zkratka, pk.popis(), pk.prijem(), pk.deniksloupecid()));
        return newid;
    }
    return -1;
}

int DbModel::firstPredkontace()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar("SELECT id FROM predkontace LIMIT 1");
    }
    return -1;
}

dxDenikArray DbModel::denik()
{
    dxDenikArray denik;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT * FROM denik WHERE obdobirok=%d ORDER BY datum", m_currentRok));
        if(!tb.GetRowCount()) return denik;
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            denik.Add(Denik(tb.GetInt("id"), tb.GetDate("datum"), tb.GetString("cislo"), tb.GetString("popis"), tb.GetDouble("castka"),
                            tb.GetDouble("castkacizi"), tb.GetDouble("kurz"), tb.GetString("reference"), tb.GetInt("menaid"),
                            tb.GetInt("predkontaceid"), tb.GetInt("pokladnaid"),
                            tb.GetInt("bankaid"), tb.GetInt("vypis"), tb.GetInt("protistranaid"), tb.GetInt("obdobirok"), tb.GetInt("pausal")));
        }
    }
    return denik;
}

void DbModel::addDenik(const Denik &denik)
{
    if(!hasDenik(denik.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("INSERT INTO denik(id,datum,cislo,popis,castka,"
                                                              "castkacizi,kurz,reference,menaid,predkontaceid,pokladnaid,bankaid,"
                                                              "vypis,protistranaid,obdobirok,pausal) "
                                                              "VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)");
                st.Bind(1, denik.id());
                st.BindDate(2, denik.datum());
                st.Bind(3, denik.cislo());
                st.Bind(4, denik.popis());
                st.Bind(5, denik.castka());
                st.Bind(6, denik.castkacizi());
                st.Bind(7, denik.kurz());
                st.Bind(8, denik.reference());
                st.Bind(9, denik.menaid());
                st.Bind(10, denik.predkontaceid());
                hasPokladna(denik.pokladnaid()) ? st.Bind(11, denik.pokladnaid()) : st.BindNull(11);
                hasBanka(denik.bankaid()) ? st.Bind(12, denik.bankaid()) : st.BindNull(12);
                st.Bind(13, denik.vypis());
                st.Bind(14, denik.protistranaid());
                st.Bind(15, denik.obdobirok());
                st.Bind(16, denik.pausal());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v addDenik: %s", e.GetMessage()));
            }
        }
        fireUpdated(hasPokladna(denik.pokladnaid()) ? ID_MODEL_POKLADNA : ID_MODEL_BANKA);
    }
}

void DbModel::editDenik(const Denik &denik)
{
    if(hasDenik(denik.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("UPDATE denik SET datum=?,cislo=?,popis=?,"
                                                              "castka=?,castkacizi=?,kurz=?,reference=?,menaid=?,"
                                                              "predkontaceid=?,pokladnaid=?,bankaid=?,vypis=?,protistranaid=?,obdobirok=?,pausal=?"
                                                              " WHERE id=?");
                st.BindDate(1, denik.datum());
                st.Bind(2, denik.cislo());
                st.Bind(3, denik.popis());
                st.Bind(4, denik.castka());
                st.Bind(5, denik.castkacizi());
                st.Bind(6, denik.kurz());
                st.Bind(7, denik.reference());
                st.Bind(8, denik.menaid());
                st.Bind(9, denik.predkontaceid());
                hasPokladna(denik.pokladnaid()) ? st.Bind(10, denik.pokladnaid()) : st.BindNull(10);
                hasBanka(denik.bankaid()) ? st.Bind(11, denik.bankaid()) : st.BindNull(11);
                st.Bind(12, denik.vypis());
                st.Bind(13, denik.protistranaid());
                st.Bind(14, denik.obdobirok());
                st.Bind(15, denik.pausal());
                st.Bind(16, denik.id());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v editDenik: %s", e.GetMessage()));
            }
        }
        if(isDenikUhrada(denik.id()))
        {
            double castka = denik.menaid()==1?denik.castka():denik.castkacizi();
            if(dxsettings.dbmodel() == DB_SQLITE)
            {
                try
                {
                    wxSQLite3Statement st = m_db.PrepareStatement("UPDATE uhrada SET castka=? WHERE denikid=?");
                    st.Bind(1, castka);
                    st.Bind(2, denik.id());
                    st.ExecuteUpdate();
                    st.Finalize();
                }
                catch(const wxSQLite3Exception& e)
                {
                    wxLogDebug(wxString::Format("Chyba v editDenik: %s", e.GetMessage()));
                }
            }
        }
        fireUpdated(hasPokladna(denik.pokladnaid()) ? ID_MODEL_POKLADNA : ID_MODEL_BANKA);
    }
}

Denik DbModel::getDenik(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM denik WHERE id=%d", id));
        while(rs.NextRow())
        {
            return Denik(rs.GetInt("id"), rs.GetDate("datum"), rs.GetString("cislo"), rs.GetString("popis"), rs.GetDouble("castka"),
                                  rs.GetDouble("castkacizi"), rs.GetDouble("kurz"), rs.GetString("reference"), rs.GetInt("menaid"),
                                  rs.GetInt("predkontaceid"), rs.GetInt("pokladnaid"),
                                  rs.GetInt("bankaid"), rs.GetInt("vypis"), rs.GetInt("protistranaid"), rs.GetInt("obdobirok"), rs.GetInt("pausal"));
        }
    }
    return Denik();
}

bool DbModel::deleteDenik(int id)
{
    bool pokladna = true;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        pokladna = m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM pokladna WHERE id = (SELECT denik.pokladnaid FROM denik WHERE id=%d)", id));
    }
    tryExecuteUpdate(wxString::Format("DELETE FROM denik WHERE id=%d", id));
    fireUpdated(pokladna ? ID_MODEL_POKLADNA : ID_MODEL_BANKA);
    return true;
}

bool DbModel::hasDenik(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM denik WHERE id=%d", id));
    }
    return false;
}

bool DbModel::isDenikPrijem(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return isPredkontacePrijem(m_db.ExecuteScalar(wxString::Format("SELECT predkontaceid FROM denik WHERE id=%d", id)));
    }
    return false;
}

bool DbModel::isDenikDanovy(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return isPredkontaceDanova(m_db.ExecuteScalar(wxString::Format("SELECT predkontaceid FROM denik WHERE id=%d", id)));
    }
    return false;
}

dxDenikArray DbModel::denikPokladna()
{
    dxDenikArray denik;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT * FROM denik WHERE obdobirok=%d ORDER BY datum", m_currentRok));
        if(!tb.GetRowCount()) return denik;
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            if(hasPokladna(tb.GetInt("pokladnaid")))
            {
                denik.Add(Denik(tb.GetInt("id"), tb.GetDate("datum"), tb.GetString("cislo"), tb.GetString("popis"), tb.GetDouble("castka"),
                                tb.GetDouble("castkacizi"), tb.GetDouble("kurz"), tb.GetString("reference"), tb.GetInt("menaid"),
                                tb.GetInt("predkontaceid"), tb.GetInt("pokladnaid"),
                                tb.GetInt("bankaid"), tb.GetInt("vypis"), tb.GetInt("protistranaid"), tb.GetInt("obdobirok"), tb.GetInt("pausal")));
            }
        }
    }
    return denik;
}

dxDenikArray DbModel::denikPokladnaById(int pokladnaid)
{
    dxDenikArray denik;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT * FROM denik WHERE obdobirok=%d AND pokladnaid=%d ORDER BY datum", m_currentRok, pokladnaid));
        if(!tb.GetRowCount()) return denik;
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            denik.Add(Denik(tb.GetInt("id"), tb.GetDate("datum"), tb.GetString("cislo"), tb.GetString("popis"), tb.GetDouble("castka"),
                            tb.GetDouble("castkacizi"), tb.GetDouble("kurz"), tb.GetString("reference"), tb.GetInt("menaid"),
                            tb.GetInt("predkontaceid"), tb.GetInt("pokladnaid"),
                            tb.GetInt("bankaid"), tb.GetInt("vypis"), tb.GetInt("protistranaid"), tb.GetInt("obdobirok"), tb.GetInt("pausal")));
        }
    }
    return denik;
}

dxDenikArray DbModel::denikBanka()
{
    dxDenikArray denik;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT * FROM denik WHERE obdobirok=%d ORDER BY datum", m_currentRok));
        if(!tb.GetRowCount()) return denik;
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            if(hasBanka(tb.GetInt("bankaid")))
            {
                denik.Add(Denik(tb.GetInt("id"), tb.GetDate("datum"), tb.GetString("cislo"), tb.GetString("popis"), tb.GetDouble("castka"),
                                tb.GetDouble("castkacizi"), tb.GetDouble("kurz"), tb.GetString("reference"), tb.GetInt("menaid"),
                                tb.GetInt("predkontaceid"), tb.GetInt("pokladnaid"),
                                tb.GetInt("bankaid"), tb.GetInt("vypis"), tb.GetInt("protistranaid"), tb.GetInt("obdobirok"), tb.GetInt("pausal")));
            }
        }
    }
    return denik;
}

dxDenikArray DbModel::denikBankaById(int bankaid)
{
    dxDenikArray denik;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT * FROM denik WHERE obdobirok=%d AND bankaid=%d ORDER BY datum", m_currentRok, bankaid));
        if(!tb.GetRowCount()) return denik;
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            denik.Add(Denik(tb.GetInt("id"), tb.GetDate("datum"), tb.GetString("cislo"), tb.GetString("popis"), tb.GetDouble("castka"),
                            tb.GetDouble("castkacizi"), tb.GetDouble("kurz"), tb.GetString("reference"), tb.GetInt("menaid"),
                            tb.GetInt("predkontaceid"), tb.GetInt("pokladnaid"),
                            tb.GetInt("bankaid"), tb.GetInt("vypis"), tb.GetInt("protistranaid"), tb.GetInt("obdobirok"), tb.GetInt("pausal")));
        }
    }
    return denik;
}

int DbModel::getLastVypis(int bankaid)
{
    if(hasBanka(bankaid))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            int vypis = m_db.ExecuteScalar(wxString::Format("SELECT max(vypis) FROM denik WHERE bankaid=%d AND obdobirok=%d", bankaid, m_currentRok));
            if(vypis < 1) vypis++;
            return vypis;
        }
    }
    return 1;
}

bool DbModel::isDenikUhrada(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM uhrada WHERE denikid=%d", id));
    }
    return false;
}

bool DbModel::isPredkontaceUsed(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT (SELECT count(*) FROM denik WHERE predkontaceid=%d)+"
                                                   "(SELECT count(*) FROM zavpoh WHERE predkontaceid=%d)+"
                                                   "(SELECT count(*) FROM faktura WHERE predkontaceid=%d)", id, id, id));
    }
    return false;
}

int DbModel::copyDenik(int id)
{
    if(hasDenik(id))
    {
        Denik de = getDenik(id);
        int newid = getLastId("denik");
        wxString cislo;
        if(hasPokladna(de.pokladnaid()))
        {
            if(isPredkontacePrijem(de.predkontaceid())) cislo = getCisloDokladu("PP");
            else cislo = getCisloDokladu("PV");
        }
        else
        {
            if(isPredkontacePrijem(de.predkontaceid())) cislo = getCisloDokladu("BP");
            else cislo = getCisloDokladu("BV");
        }
        addDenik(Denik(newid, dxsettings.datum(), cislo, de.popis(), de.castka(), de.castkacizi(), de.kurz(), de.reference(),
                           de.menaid(), de.predkontaceid(), de.pokladnaid(), de.bankaid(), de.vypis(), de.protistranaid(), de.obdobirok(), de.pausal()));
        return newid;
    }
    return -1;
}

dxDenikArray DbModel::danovePrijmyVydajeForRok(int rok)
{
    dxDenikArray array;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT * FROM denik WHERE obdobirok=%d", rok));
        if(!tb.GetRowCount()) return array;
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            if(isPredkontaceDanova(tb.GetInt("predkontaceid")))
            {
                array.Add(Denik(tb.GetInt("id"), tb.GetDate("datum"), tb.GetString("cislo"), tb.GetString("popis"), tb.GetDouble("castka"),
                                tb.GetDouble("castkacizi"), tb.GetDouble("kurz"), tb.GetString("reference"), tb.GetInt("menaid"),
                                tb.GetInt("predkontaceid"), tb.GetInt("pokladnaid"),
                                tb.GetInt("bankaid"), tb.GetInt("vypis"), tb.GetInt("protistranaid"), tb.GetInt("obdobirok"), tb.GetInt("pausal")));
            }
        }
    }
    return array;
}

int DbModel::denikSloupecByDenikId(int id)
{
    if(hasDenik(id))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            return m_db.ExecuteScalar(wxString::Format("SELECT deniksloupecid FROM predkontace WHERE id=(SELECT predkontaceid FROM denik WHERE id=%d)", id));
        }
    }
    return -1;
}

dxDenikViewArray DbModel::denikview()
{
    dxDenikViewArray denik;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT denik.id AS id, denik.datum AS datum, denik.cislo AS cislo, denik.popis AS popis, denik.castka AS castka, predkontace.prijem AS prijem, banka.zkratka AS zkratka, denik.vypis AS vypis FROM denik "
                                                           "INNER JOIN predkontace ON predkontace.id = denik.predkontaceid "
                                                           "INNER JOIN banka ON banka.id = denik.bankaid WHERE bankaid IN (SELECT id FROM banka) AND obdobirok=%d", m_currentRok));
        if(tb.GetRowCount())
        {
            for(int i=0; i < tb.GetRowCount(); i++)
            {
                tb.SetRow(i);
                denik.Add(DenikView(tb.GetInt("id"), tb.GetDate("datum"), tb.GetString("cislo"), tb.GetString("popis"), tb.GetDouble("castka"), tb.GetInt("prijem"),
                                    tb.GetString("zkratka"), tb.GetInt("vypis")));
            }
        }
        tb.Finalize();
        tb = m_db.GetTable(wxString::Format("SELECT denik.id AS id, denik.datum AS datum, denik.cislo AS cislo, denik.popis AS popis, denik.castka AS castka, predkontace.prijem AS prijem, pokladna.zkratka AS zkratka, denik.vypis AS vypis FROM denik "
                                            "INNER JOIN predkontace ON predkontace.id = denik.predkontaceid "
                                            "INNER JOIN pokladna ON pokladna.id = denik.pokladnaid WHERE pokladnaid IN (SELECT id FROM pokladna) AND obdobirok=%d", m_currentRok));
        if(tb.GetRowCount())
        {
            for(int i=0; i < tb.GetRowCount(); i++)
            {
                tb.SetRow(i);
                denik.Add(DenikView(tb.GetInt("id"), tb.GetDate("datum"), tb.GetString("cislo"), tb.GetString("popis"), tb.GetDouble("castka"), tb.GetInt("prijem"),
                                    tb.GetString("zkratka"), tb.GetInt("vypis")));
            }
        }
        tb.Finalize();
    }
    return denik;
}

dxDenikViewArray DbModel::denikviewPokladna()
{
    dxDenikViewArray denik;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT denik.id AS id, denik.datum AS datum, denik.cislo AS cislo, denik.popis AS popis, denik.castka AS castka, predkontace.prijem AS prijem, pokladna.zkratka AS zkratka, denik.vypis AS vypis FROM denik "
                                                           "INNER JOIN predkontace ON predkontace.id = denik.predkontaceid "
                                                           "INNER JOIN pokladna ON pokladna.id = denik.pokladnaid WHERE pokladnaid IN (SELECT id FROM pokladna) AND obdobirok=%d", m_currentRok));
        if(tb.GetRowCount())
        {
            for(int i=0; i < tb.GetRowCount(); i++)
            {
                tb.SetRow(i);
                denik.Add(DenikView(tb.GetInt("id"), tb.GetDate("datum"), tb.GetString("cislo"), tb.GetString("popis"), tb.GetDouble("castka"), tb.GetInt("prijem"),
                                    tb.GetString("zkratka"), tb.GetInt("vypis")));
            }
        }
    }
    return denik;
}

dxDenikViewArray DbModel::denikviewBanka()
{
    dxDenikViewArray denik;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT denik.id AS id, denik.datum AS datum, denik.cislo AS cislo, denik.popis AS popis, denik.castka AS castka, predkontace.prijem AS prijem, banka.zkratka AS zkratka, denik.vypis AS vypis FROM denik "
                                                           "INNER JOIN predkontace ON predkontace.id = denik.predkontaceid "
                                                           "INNER JOIN banka ON banka.id = denik.bankaid WHERE bankaid IN (SELECT id FROM banka) AND obdobirok=%d", m_currentRok));
        if(tb.GetRowCount())
        {
            for(int i=0; i < tb.GetRowCount(); i++)
            {
                tb.SetRow(i);
                denik.Add(DenikView(tb.GetInt("id"), tb.GetDate("datum"), tb.GetString("cislo"), tb.GetString("popis"), tb.GetDouble("castka"), tb.GetInt("prijem"),
                                    tb.GetString("zkratka"), tb.GetInt("vypis")));
            }
        }
    }
    return denik;
}

dxPokladnaArray DbModel::pokladny()
{
    dxPokladnaArray pokladny;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable("SELECT * FROM pokladna");
        if(!tb.GetRowCount()) return pokladny;
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            pokladny.Add(Pokladna(tb.GetInt("id"), tb.GetString("zkratka"), tb.GetString("popis"), tb.GetInt("menaid")));
        }
    }
    return pokladny;
}

void DbModel::addPokladna(const Pokladna &pokladna)
{
    if(!hasPokladna(pokladna.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("INSERT INTO pokladna(id,zkratka,popis,menaid) "
                                                              "VALUES(?,?,?,?)");
                st.Bind(1, pokladna.id());
                st.Bind(2, pokladna.zkratka());
                st.Bind(3, pokladna.popis());
                st.Bind(4, pokladna.menaid());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v addPokladna: %s", e.GetMessage()));
            }
        }
    }
}

void DbModel::editPokladna(const Pokladna &pokladna)
{
    if(hasPokladna(pokladna.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("UPDATE pokladna SET zkratka=?,popis=?,menaid=?"
                                                              " WHERE id=?");
                st.Bind(1, pokladna.zkratka());
                st.Bind(2, pokladna.popis());
                st.Bind(3, pokladna.menaid());
                st.Bind(4, pokladna.id());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v editPokladna: %s", e.GetMessage()));
            }
        }
        fireUpdated(ID_MODEL_POKLADNA);
    }
}

Pokladna DbModel::getPokladna(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM pokladna WHERE id=%d", id));
        while(rs.NextRow())
        {
            return Pokladna(rs.GetInt("id"), rs.GetString("zkratka"), rs.GetString("popis"), rs.GetInt("menaid"));
        }
        rs.Finalize();
    }
    return Pokladna();
}

Pokladna DbModel::getPokladna(const wxString &zkratka)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM pokladna WHERE zkratka ='%s'", zkratka));
        while(rs.NextRow())
        {
            return Pokladna(rs.GetInt("id"), rs.GetString("zkratka"), rs.GetString("popis"), rs.GetInt("menaid"));
        }
        rs.Finalize();
    }
    return Pokladna();
}

bool DbModel::deletePokladna(int id)
{
    if(isPokladnaUsed(id))
    {
        Pokladna po = getPokladna(id);
        fireMsg(wxString::Format(wxString::FromUTF8("Nelze smazat pokladnu\n%s\n%s\nprotože je použita v dokladech"), po.zkratka(), po.popis()));
        return false;
    }
    if(dxsettings.dbmodel() == DB_SQLITE && m_db.ExecuteScalar("SELECT count(*) FROM pokladna") == 1)
    {
        fireMsg(wxString::FromUTF8("Nelze smazat poslední pokladnu,\nv programu musí alespoň jedna existovat"));
        return false;
    }
    tryExecuteUpdate(wxString::Format("DELETE FROM pokladna WHERE id=%d", id));
    return true;
}

bool DbModel::hasPokladna(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM pokladna WHERE id=%d", id));
    }
    return false;
}

bool DbModel::hasPokladna(const wxString &zkratka)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM pokladna WHERE zkratka='%s'", zkratka));
    }
    return false;
}

int DbModel::getPokladnaIdByZkratka(const wxString &zkratka)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT id FROM pokladna WHERE zkratka='%s'", zkratka));
    }
    return -1;
}

double DbModel::getPokladnaStav(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteQuery(wxString::Format("SELECT SUM(CASE WHEN predkontace.prijem=1 THEN (CASE WHEN pokladna.menaid=1 THEN castka ELSE castkacizi END) "
                                                  "ELSE -(CASE WHEN pokladna.menaid=1 THEN castka ELSE castkacizi END) END) FROM denik INNER JOIN pokladna ON pokladna.id = denik.pokladnaid "
                                                  "INNER JOIN predkontace ON predkontace.id = denik.predkontaceid WHERE pokladnaid=%d", id)).GetDouble(0);
    }
    return 0.0;
}

double DbModel::getPokladnaZustatek(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteQuery(wxString::Format("SELECT SUM(CASE WHEN predkontace.prijem=1 THEN (CASE WHEN pokladna.menaid=1 THEN castka ELSE castkacizi END) "
                                                  "ELSE -(CASE WHEN pokladna.menaid=1 THEN castka ELSE castkacizi END) END) FROM denik INNER JOIN pokladna ON pokladna.id = denik.pokladnaid "
                                                  "INNER JOIN predkontace ON predkontace.id = denik.predkontaceid WHERE pokladnaid=%d AND obdobirok<%d", id, m_currentRok)).GetDouble(0);
    }
    return 0.0;
}

double DbModel::getPokladnyStav(const wxDateTime &toDate)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
            return m_db.ExecuteQuery(wxString::Format("SELECT SUM(CASE WHEN predkontace.prijem=1 THEN castka ELSE -castka END) FROM denik INNER JOIN predkontace ON predkontace.id = denik.predkontaceid "
                                              "WHERE (SELECT count(*) FROM pokladna WHERE pokladna.id=denik.pokladnaid) AND datum < date('%s')",
                                                      wxDateTime(toDate+wxDateSpan(0,0,0,1)).FormatISODate())).GetDouble(0);
    }
    return 0.0;
}

bool DbModel::isPokladnaUsed(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM denik WHERE pokladnaid=%d", id)))
            return true;
        if(m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM faktura WHERE pokladnaid=%d", id)))
            return true;
        if(m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM zavpoh WHERE pokladnaid=%d", id)))
            return true;
    }
    return false;
}

wxString DbModel::getPokladnaZkratka(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteQuery(wxString::Format("SELECT zkratka FROM pokladna WHERE id=%d", id)).GetString(0);
    }
    return wxEmptyString;
}

dxBankaArray DbModel::banky()
{
    dxBankaArray banky;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable("SELECT * FROM banka");
        if(!tb.GetRowCount()) return banky;
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            banky.Add(Banka(tb.GetInt("id"), tb.GetString("zkratka"), tb.GetString("popis"), tb.GetString("cislouctu"),
                                  tb.GetString("kodbanky"), tb.GetString("iban"), tb.GetString("bic"), tb.GetInt("menaid")));
        }
    }
    return banky;
}

void DbModel::addBanka(const Banka &banka)
{
    if(!hasBanka(banka.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("INSERT INTO banka(id,zkratka,popis,cislouctu,"
                                                              "kodbanky,menaid,iban,bic) "
                                                              "VALUES(?,?,?,?,?,?,?,?)");
                st.Bind(1, banka.id());
                st.Bind(2, banka.zkratka());
                st.Bind(3, banka.popis());
                st.Bind(4, banka.cislouctu());
                st.Bind(5, banka.kodbanky());
                st.Bind(6, banka.menaid());
                st.Bind(7, banka.iban());
                st.Bind(8, banka.bic());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v addBanka: %s", e.GetMessage()));
            }
        }
    }
}

void DbModel::editBanka(const Banka &banka)
{
    if(hasBanka(banka.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("UPDATE banka SET zkratka=?,popis=?,menaid=?,"
                                                                              "cislouctu=?,kodbanky=?,iban=?,bic=? WHERE id=?");
                st.Bind(1, banka.zkratka());
                st.Bind(2, banka.popis());
                st.Bind(3, banka.menaid());
                st.Bind(4, banka.cislouctu());
                st.Bind(5, banka.kodbanky());
                st.Bind(6, banka.iban());
                st.Bind(7, banka.bic());
                st.Bind(8, banka.id());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v editBanka: %s", e.GetMessage()));
            }
        }
        fireUpdated(ID_MODEL_BANKA);
    }
}

Banka DbModel::getBanka(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM banka WHERE id=%d", id));
        while(rs.NextRow())
        {
            return Banka(rs.GetInt("id"), rs.GetString("zkratka"), rs.GetString("popis"), rs.GetString("cislouctu"),
                         rs.GetString("kodbanky"), rs.GetString("iban"), rs.GetString("bic"), rs.GetInt("menaid"));
        }
        rs.Finalize();
    }
    return Banka();
}

Banka DbModel::getBanka(const wxString &zkratka)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM banka WHERE zkratka='%s'", zkratka));
        while(rs.NextRow())
        {
            return Banka(rs.GetInt("id"), rs.GetString("zkratka"), rs.GetString("popis"), rs.GetString("cislouctu"),
                         rs.GetString("kodbanky"), rs.GetString("iban"), rs.GetString("bic"), rs.GetInt("menaid"));
        }
        rs.Finalize();
    }
    return Banka();
}

bool DbModel::deleteBanka(int id)
{
    if(isBankaUsed(id))
    {
        Banka ba = getBanka(id);
        fireMsg(wxString::Format(wxString::FromUTF8("Nelze smazat bankovní účet\n%s\n%s\nprotože je použit v dokladech"), ba.zkratka(), ba.popis()));
        return false;
    }
    if(dxsettings.dbmodel() == DB_SQLITE && m_db.ExecuteScalar("SELECT count(*) FROM banka") == 1)
    {
        fireMsg(wxString::FromUTF8("Nelze smazat poslední bankovní účet,\nv programu musí alespoň jeden existovat"));
        return false;
    }
    tryExecuteUpdate(wxString::Format("DELETE FROM banka WHERE id=%d", id));
    return true;
}

bool DbModel::hasBanka(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM banka WHERE id=%d", id));
    }
    return false;
}

bool DbModel::hasBanka(const wxString &zkratka)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM banka WHERE zkratka='%s'", zkratka));
    }
    return false;
}

int DbModel::getBankaIdByZkratka(const wxString &zkratka)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteQuery(wxString::Format("SELECT id FROM banka WHERE zkratka='%s'", zkratka)).GetInt(0);
    }
    return -1;
}

double DbModel::getBankaStav(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteQuery(wxString::Format("SELECT SUM(CASE WHEN predkontace.prijem=1 THEN (CASE WHEN banka.menaid=1 THEN castka ELSE castkacizi END) "
                                                  "ELSE -(CASE WHEN banka.menaid=1 THEN castka ELSE castkacizi END) END) FROM denik INNER JOIN banka ON banka.id = denik.bankaid "
                                                  "INNER JOIN predkontace ON predkontace.id = denik.predkontaceid WHERE bankaid=%d", id)).GetDouble(0);
    }
    return 0.0;
}

double DbModel::getBankaZustatek(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteQuery(wxString::Format("SELECT SUM(CASE WHEN predkontace.prijem=1 THEN (CASE WHEN banka.menaid=1 THEN castka ELSE castkacizi END) "
                                                  "ELSE -(CASE WHEN banka.menaid=1 THEN castka ELSE castkacizi END) END) FROM denik INNER JOIN banka ON banka.id = denik.bankaid "
                                                  "INNER JOIN predkontace ON predkontace.id = denik.predkontaceid WHERE bankaid=%d AND obdobirok<%d", id, m_currentRok)).GetDouble(0);
    }
    return 0.0;
}

double DbModel::getBankyStav(const wxDateTime &toDate)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
            return m_db.ExecuteQuery(wxString::Format("SELECT SUM(CASE WHEN predkontace.prijem=1 THEN castka ELSE -castka END) FROM denik INNER JOIN predkontace ON predkontace.id = denik.predkontaceid "
                                              "WHERE (SELECT count(*) FROM banka WHERE banka.id=denik.bankaid) AND datum < date('%s')",wxDateTime(toDate+wxDateSpan(0,0,0,1)).FormatISODate())).GetDouble(0);
    }
    return 0.0;
}

bool DbModel::isBankaUsed(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM denik WHERE bankaid=%d", id)))
            return true;
        if(m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM faktura WHERE bankaid=%d", id)))
            return true;
        if(m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM zavpoh WHERE bankaid=%d", id)))
            return true;
    }
    return false;
}

wxString DbModel::getBankaZkratka(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteQuery(wxString::Format("SELECT zkratka FROM banka WHERE id=%d", id)).GetString(0);
    }
    return wxEmptyString;
}

wxArrayString DbModel::kodyBank(bool aktivni)
{
    wxArrayString kody;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = aktivni?m_db.ExecuteQuery("SELECT kod FROM bankovniustav WHERE aktivni=1"):m_db.ExecuteQuery("SELECT kod FROM bankovniustav");
        while(rs.NextRow())
        {
            kody.Add(rs.GetString(0));
        }
        rs.Finalize();
    }
    return kody;
}

wxArrayString DbModel::kodyBankSNazvem(bool aktivni)
{
    wxArrayString kody;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = aktivni?m_db.ExecuteQuery("SELECT kod || ' ' || nazev FROM bankovniustav WHERE aktivni=1"):m_db.ExecuteQuery("SELECT kod || ' ' || nazev FROM bankovniustav");
        while(rs.NextRow())
        {
            kody.Add(rs.GetString(0));
        }
        rs.Finalize();
    }
    return kody;
}

bool DbModel::hasKod(const wxString &kod)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM bankovniustav WHERE kod='%s'", kod));
    }
    return false;
}

wxString DbModel::bicByKod(const wxString &kod)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT bic FROM bankovniustav WHERE kod='%s'", kod));
        while(rs.NextRow())
        {
            return rs.GetString(0);
        }
        rs.Finalize();
    }
    return wxEmptyString;
}

dxRadaArray DbModel::rady()
{
    dxRadaArray rady;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable("SELECT * FROM rada");
        if(!tb.GetRowCount()) return rady;
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            rady.Add(Rada(tb.GetString("id"), tb.GetString("popis"), tb.GetString("prefix"), tb.GetInt("znaku"),
                            tb.GetInt("cislo")));
        }
    }
    return rady;
}

void DbModel::editRada(const Rada &rada)
{
    if(hasRada(rada.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("UPDATE rada SET popis=?,prefix=?,znaku=?,"
                                                              "cislo=? WHERE id=?");
                st.Bind(1, rada.popis());
                st.Bind(2, rada.prefix());
                st.Bind(3, rada.znaku());
                st.Bind(4, rada.cislo());
                st.Bind(5, rada.id());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v editRada: %s", e.GetMessage()));
            }
        }
    }
}

Rada DbModel::getRada(const wxString &id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM rada WHERE id='%s'", id));
        while(rs.NextRow())
        {
            return Rada(rs.GetString("id"), rs.GetString("popis"), rs.GetString("prefix"), rs.GetInt("znaku"), rs.GetInt("cislo"));
        }
        rs.Finalize();
    }
    return Rada();
}

bool DbModel::hasRada(const wxString &id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM rada WHERE id='%s'", id));
    }
    return false;
}

void DbModel::increaseRadaCislo(const wxString &id)
{
    if(hasRada(id))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("UPDATE rada SET cislo=cislo+1 WHERE id=?");
                st.Bind(1, id);
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v increaseRadaCislo: %s", e.GetMessage()));
            }
        }
    }
}

wxString DbModel::getCisloDokladu(const wxString &id)
{
    if(hasRada(id))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT prefix,cislo,znaku FROM rada WHERE id='%s'", id));
            wxString doklad = rs.GetString(0);
            wxString cislo = rs.GetAsString(1);
            int zeroes = rs.GetInt(2)-(int)doklad.Length()-(int)cislo.Length();
            for(int i=0; i<zeroes; i++)
                doklad.Append('0');
            doklad.Append(cislo);
            return doklad;
            }
    }
    return wxEmptyString;
}

int DbModel::getRadaCislo(const wxString &id)
{
    if(hasRada(id))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            return m_db.ExecuteQuery(wxString::Format("SELECT cislo FROM rada WHERE id='%s'", id)).GetInt(0);
        }
    }
    return 0;
}

void DbModel::setDefaultRadaCislo()
{
    int lastPP = 1;
    int lastPV = 1;
    int lastBP = 1;
    int lastBV = 1;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT cislo FROM denik WHERE obdobirok=%d", m_currentRok));
        while(rs.NextRow())
        {
            if(rs.GetString(0).StartsWith("PP"))
            {
                lastPP = getCisloZDokladu(rs.GetString(0), 2, lastPP);
            }
            if(rs.GetString(0).StartsWith("PV"))
            {
                lastPV = getCisloZDokladu(rs.GetString(0), 2, lastPV);
            }
            if(rs.GetString(0).StartsWith("BP"))
            {
                lastBP = getCisloZDokladu(rs.GetString(0), 2, lastBP);
            }
            if(rs.GetString(0).StartsWith("BV"))
            {
                lastBV = getCisloZDokladu(rs.GetString(0), 2, lastBV);
            }
        }
        rs.Finalize();
    }
    int lastFV = 1;
    int lastFP = 1;
    int lastZFV = 1;
    int lastZFP = 1;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT cislo FROM faktura WHERE obdobirok=%d", m_currentRok));
        while(rs.NextRow())
        {
            if(rs.GetString(0).StartsWith("FV"))
            {
                lastFV = getCisloZDokladu(rs.GetString(0), 2, lastFV);
            }
            if(rs.GetString(0).StartsWith("FP"))
            {
                lastFP = getCisloZDokladu(rs.GetString(0), 2, lastFP);
            }
            if(rs.GetString(0).StartsWith("ZFV"))
            {
                lastZFV = getCisloZDokladu(rs.GetString(0), 3, lastZFV);
            }
            if(rs.GetString(0).StartsWith("ZFP"))
            {
                lastZFP = getCisloZDokladu(rs.GetString(0), 2, lastZFP);
            }
        }
        rs.Finalize();
    }
    int lastPO = 1;
    int lastZA = 1;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT cislo FROM zavpoh WHERE obdobirok=%d", m_currentRok));
        while(rs.NextRow())
        {
            if(rs.GetString(0).StartsWith("PO"))
            {
                lastPO = getCisloZDokladu(rs.GetString(0), 2, lastPO);
            }
            if(rs.GetString(0).StartsWith("ZA"))
            {
                lastZA = getCisloZDokladu(rs.GetString(0), 2, lastZA);
            }
        }
        rs.Finalize();
    }
    int lastPR = 1;
    int lastVY = 1;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT cislo FROM pohybzasob WHERE obdobirok=%d", m_currentRok));
        while(rs.NextRow())
        {
            if(rs.GetString(0).StartsWith("PR"))
            {
                lastPR = getCisloZDokladu(rs.GetString(0), 2, lastPR);
            }
            if(rs.GetString(0).StartsWith("VY"))
            {
                lastVY = getCisloZDokladu(rs.GetString(0), 2, lastVY);
            }
        }
        rs.Finalize();
    }
    int lastIV = 1;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT cislo FROM inventura WHERE obdobirok=%d", m_currentRok));
        while(rs.NextRow())
        {
            if(rs.GetString(0).StartsWith("IV"))
            {
                lastIV = getCisloZDokladu(rs.GetString(0), 2, lastIV);
            }
        }
        rs.Finalize();
    }
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        try
        {
            tryExecuteUpdate(wxString::Format("UPDATE rada SET cislo=%d WHERE id='PP'", lastPP));
            tryExecuteUpdate(wxString::Format("UPDATE rada SET cislo=%d WHERE id='PV'", lastPV));
            tryExecuteUpdate(wxString::Format("UPDATE rada SET cislo=%d WHERE id='BP'", lastBP));
            tryExecuteUpdate(wxString::Format("UPDATE rada SET cislo=%d WHERE id='BV'", lastBV));
            tryExecuteUpdate(wxString::Format("UPDATE rada SET cislo=%d WHERE id='FV'", lastFV));
            tryExecuteUpdate(wxString::Format("UPDATE rada SET cislo=%d WHERE id='FP'", lastFP));
            tryExecuteUpdate(wxString::Format("UPDATE rada SET cislo=%d WHERE id='ZFV'", lastZFV));
            tryExecuteUpdate(wxString::Format("UPDATE rada SET cislo=%d WHERE id='ZFP'", lastZFP));
            tryExecuteUpdate(wxString::Format("UPDATE rada SET cislo=%d WHERE id='PO'", lastPO));
            tryExecuteUpdate(wxString::Format("UPDATE rada SET cislo=%d WHERE id='ZA'", lastZA));
            tryExecuteUpdate(wxString::Format("UPDATE rada SET cislo=%d WHERE id='PR'", lastPR));
            tryExecuteUpdate(wxString::Format("UPDATE rada SET cislo=%d WHERE id='VY'", lastVY));
            tryExecuteUpdate(wxString::Format("UPDATE rada SET cislo=%d WHERE id='IV'", lastIV));
        }
        catch(const wxSQLite3Exception& e)
        {
            wxLogDebug(wxString::Format("Chyba v setDefaulRadaCislo: %s", e.GetMessage()));
        }
    }
}

void DbModel::addKurz(const Kurz &kurz)
{
    if(!hasKurz(kurz.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("INSERT INTO kurz(id,datum,hodnota,kod,jednotny) "
                                                              "VALUES(?,?,?,?,?)");
                st.Bind(1, kurz.id());
                st.BindDate(2, kurz.datum());
                st.Bind(3, kurz.hodnota());
                st.Bind(4, kurz.kod());
                st.BindBool(5, kurz.jednotny());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v addKurz: %s", e.GetMessage()));
            }
        }
    }
}

Kurz DbModel::getKurz(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM kurz WHERE id=%d", id));
        while(rs.NextRow())
        {
            return Kurz(rs.GetInt("id"), rs.GetDate("datum"), rs.GetDouble("hodnota"), rs.GetString("kod"), rs.GetBool("jednotny"));
        }
        rs.Finalize();
    }
    return Kurz();
}

//Kdyz neexistuje datum, return nejblizsi k datum
Kurz DbModel::getKurz(const wxString &kod, const wxDateTime &datum)
{
    if(kod == "CZK")
    {
        return Kurz(0, wxDateTime(1,wxDateTime::Jan,OBDOBI_MIN), 1.0, kod, false);
    }
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM kurz WHERE kod='%s' AND jednotny=0 AND datum <= date('%s') ORDER BY datum DESC", kod, datum.FormatISODate()));
        while(rs.NextRow())
        {
            return Kurz(rs.GetInt("id"), rs.GetDate("datum"), rs.GetDouble("hodnota"), rs.GetString("kod"), rs.GetBool("jednotny"));
        }
        rs.Finalize();
    }
    return Kurz();
}

bool DbModel::hasKurz(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM kurz WHERE id=%d AND jednotny=0", id));
    }
    return false;
}

bool DbModel::hasKurz(const wxString &kod)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM kurz WHERE kod='%s' AND jednotny=0", kod));
    }
    return false;
}

bool DbModel::hasKurz(const wxString &kod, const wxDateTime &datum)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM kurz WHERE kod='%s' AND datum = date('%s') AND jednotny=0", kod, datum.FormatISODate()));
    }
    return false;
}

bool DbModel::hasJednotnyKurz(int rok, const wxString &kod)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM kurz WHERE kod='%s' AND jednotny=1 AND datum BETWEEN '%d-01-01' AND '%d-12-31'", kod, rok, rok));
    }
    return false;
}

double DbModel::getJednotnyKurz(const wxString &kod, int rok)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT hodnota FROM kurz WHERE kod='%s' AND jednotny=1 AND datum BETWEEN '%d-01-01' AND '%d-12-31'", kod, rok, rok));
        while(rs.NextRow())
        {
            return rs.GetDouble(0);
        }
        rs.Finalize();
    }
    return 1.0;
}

FaTyp DbModel::itemFaTyp(size_t index)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        size_t i =0;
        wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT * FROM fatyp");
        while(rs.NextRow())
        {
            if(i == index)
            {
                return FaTyp(rs.GetInt("id"), rs.GetString("typ"));
            }
            i++;
        }
        rs.Finalize();
    }
    return FaTyp();
}

wxArrayString DbModel::getFaTypyAsStrings()
{
    wxArrayString fatypy;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT typ FROM fatyp");
        while(rs.NextRow())
        {
            fatypy.Add(rs.GetString(0));
        }
        rs.Finalize();
    }
    return fatypy;
}

dxFaUhradaArray DbModel::fauhrady()
{
    dxFaUhradaArray fauhrady;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT * FROM fauhrada");
        while(rs.NextRow())
        {
            fauhrady.Add(FaUhrada(rs.GetInt("id"), rs.GetString("uhrada"), rs.GetBool("pokladnou")));
        }
        rs.Finalize();
    }
    return fauhrady;
}

void DbModel::addFaUhrada(const FaUhrada &fauhrada)
{
    if(!hasFaUhrada(fauhrada.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("INSERT INTO fauhrada(id,uhrada,pokladnou) "
                                                              "VALUES(?,?,?)");
                st.Bind(1, fauhrada.id());
                st.Bind(2, fauhrada.uhrada());
                st.Bind(3, fauhrada.pokladnou());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v addFaUhrada: %s", e.GetMessage()));
            }
        }
    }
}

FaUhrada DbModel::getFaUhrada(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM fauhrada WHERE id=%d", id));
        while(rs.NextRow())
        {
            return FaUhrada(rs.GetInt("id"), rs.GetString("uhrada"), rs.GetBool("pokladnou"));
        }
        rs.Finalize();
    }
    return FaUhrada();
}

FaUhrada DbModel::getLastFaUhrada()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT * FROM fauhrada WHERE id=(SELECT max(id) FROM fauhrada)");
        while(rs.NextRow())
        {
            return FaUhrada(rs.GetInt("id"), rs.GetString("uhrada"), rs.GetBool("pokladnou"));
        }
        rs.Finalize();
    }
    return FaUhrada();
}

bool DbModel::hasFaUhrada(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM fauhrada WHERE id=%d", id));
    }
    return false;
}

bool DbModel::hasFaUhrada(const wxString &uhrada)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM fauhrada WHERE uhrada='%s'", uhrada));
    }
    return false;
}

int DbModel::getFaUhradaByPopis(const wxString &uhrada)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT id FROM fauhrada WHERE uhrada='%s'", uhrada));
    }
    return -1;
}

bool DbModel::deleteFauhrada(int id)
{
    if(isFauhradaUsed(id))
    {
        FaUhrada fu = getFaUhrada(id);
        fireMsg(wxString::Format(wxString::FromUTF8("Nelze smazat úhradu\n%s\nprotože je použita v dokladech"), fu.uhrada()));
        return false;
    }
    if(dxsettings.dbmodel() == DB_SQLITE && m_db.ExecuteScalar("SELECT count(*) FROM fauhrada") == 1)
    {
        fireMsg(wxString::FromUTF8("Nelze smazat poslední úhradu,\nv programu musí alespoň jedna existovat"));
        return false;
    }
    tryExecuteUpdate(wxString::Format("DELETE FROM fauhrada WHERE id=%d", id));
    return true;
}

bool DbModel::faUhradaPokladnou(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT pokladnou FROM fauhrada WHERE id=%d", id));
    }
    return false;
}

bool DbModel::isFauhradaUsed(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM faktura WHERE uhradaid=%d", id)))
            return true;
        if(m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM zavpoh WHERE uhradaid=%d", id)))
            return true;
    }
    return false;
}

int DbModel::firstFaUhrada()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar("SELECT id FROM fauhrada LIMIT 1");
    }
    return -1;
}

void DbModel::addFaPolozka(const FaPolozka &fapolozka)
{
    if(!hasFaPolozka(fapolozka.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("INSERT INTO fapolozka(id,popis,mj,pocet,cenamj,sazbaid,dph,"
                                                              "celkem,bezdph,fakturaid,zavpohid,zasobaid,zaokrouhleni) "
                                                              "VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?)");
                st.Bind(1, fapolozka.id());
                st.Bind(2, fapolozka.popis());
                st.Bind(3, fapolozka.mj());
                st.Bind(4, fapolozka.pocet());
                st.Bind(5, fapolozka.cenamj());
                st.Bind(6, fapolozka.sazbaid());
                st.Bind(7, fapolozka.dph());
                st.Bind(8, fapolozka.celkem());
                st.Bind(9, fapolozka.bezdph());
                hasFaktura(fapolozka.fakturaid()) ? st.Bind(10, fapolozka.fakturaid()) : st.BindNull(10);
                hasZavPoh(fapolozka.zavpohid()) ? st.Bind(11, fapolozka.zavpohid()) : st.BindNull(11);
                hasZasoba(fapolozka.zasobaid()) ? st.Bind(12, fapolozka.zasobaid()) : st.BindNull(12);
                st.Bind(13, fapolozka.zaokrouhleni());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v addFaPolozka: %s", e.GetMessage()));
            }
        }
    }
}

void DbModel::editFaPolozka(const FaPolozka &fapolozka)
{
    if(hasFaPolozka(fapolozka.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("UPDATE fapolozka SET popis=?,mj=?,pocet=?,cenamj=?,sazbaid=?,dph=?,"
                                                              "celkem=?,bezdph=?,fakturaid=?,zavpohid=?,zasobaid=?,zaokrouhleni=? "
                                                              "WHERE id=?");
                st.Bind(1, fapolozka.popis());
                st.Bind(2, fapolozka.mj());
                st.Bind(3, fapolozka.pocet());
                st.Bind(4, fapolozka.cenamj());
                st.Bind(5, fapolozka.sazbaid());
                st.Bind(6, fapolozka.dph());
                st.Bind(7, fapolozka.celkem());
                st.Bind(8, fapolozka.bezdph());
                hasFaktura(fapolozka.fakturaid()) ? st.Bind(9, fapolozka.fakturaid()) : st.BindNull(9);
                hasZavPoh(fapolozka.zavpohid()) ? st.Bind(10, fapolozka.zavpohid()) : st.BindNull(10);
                hasZasoba(fapolozka.zasobaid()) ? st.Bind(11, fapolozka.zasobaid()) : st.BindNull(11);
                st.Bind(12, fapolozka.zaokrouhleni());
                st.Bind(13, fapolozka.id());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v editFaPolozka: %s", e.GetMessage()));
            }
        }
    }
}

FaPolozka DbModel::getFaPolozka(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM fapolozka WHERE id=%d", id));
        while(rs.NextRow())
        {
            return FaPolozka(rs.GetInt("id"), rs.GetString("popis"), rs.GetString("mj"), rs.GetDouble("pocet"),
                             rs.GetDouble("cenamj"), rs.GetInt("sazbaid"), rs.GetDouble("dph"), rs.GetDouble("celkem"),
                             rs.GetBool("bezdph"), rs.GetInt("fakturaid"), rs.GetInt("zavpohid"), rs.GetInt("zasobaid"),
                             rs.GetBool("zaokrouhleni"));
        }
        rs.Finalize();
    }
    return FaPolozka();
}

bool DbModel::deleteFaPolozka(int id)
{
    tryExecuteUpdate(wxString::Format("DELETE FROM fapolozka WHERE id=%d", id));
    return true;
}

bool DbModel::hasFaPolozka(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM fapolozka WHERE id=%d", id));
    }
    return false;
}

dxFaPolozkaArray DbModel::fapolozkyByFakturaId(int fakturaid)
{
    dxFaPolozkaArray polozky;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM fapolozka WHERE fakturaid=%d", fakturaid));
        while(rs.NextRow())
        {
            polozky.Add(FaPolozka(rs.GetInt("id"), rs.GetString("popis"), rs.GetString("mj"), rs.GetDouble("pocet"),
                                  rs.GetDouble("cenamj"), rs.GetInt("sazbaid"), rs.GetDouble("dph"), rs.GetDouble("celkem"),
                                  rs.GetBool("bezdph"), rs.GetInt("fakturaid"), rs.GetInt("zavpohid"), rs.GetInt("zasobaid"),
                                  rs.GetBool("zaokrouhleni")));
        }
        rs.Finalize();
    }
    return polozky;
}

dxFaPolozkaArray DbModel::fapolozkyByZavpohId(int zavpohid)
{
    dxFaPolozkaArray polozky;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM fapolozka WHERE zavpohid=%d", zavpohid));
        while(rs.NextRow())
        {
            polozky.Add(FaPolozka(rs.GetInt("id"), rs.GetString("popis"), rs.GetString("mj"), rs.GetDouble("pocet"),
                                  rs.GetDouble("cenamj"), rs.GetInt("sazbaid"), rs.GetDouble("dph"), rs.GetDouble("celkem"),
                                  rs.GetBool("bezdph"), rs.GetInt("fakturaid"), rs.GetInt("zavpohid"), rs.GetInt("zasobaid"),
                                  rs.GetBool("zaokrouhleni")));
        }
        rs.Finalize();
    }
    return polozky;
}

double DbModel::dphFaktury(int fakturaid)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteQuery(wxString::Format("SELECT SUM(dph) FROM fapolozka WHERE fakturaid=%d", fakturaid)).GetDouble(0);
    }
    return 0.0;
}

double DbModel::dphZavPoh(int zavpohid)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteQuery(wxString::Format("SELECT SUM(dph) FROM fapolozka WHERE zavpohid=%d", zavpohid)).GetDouble(0);
    }
    return 0.0;
}

bool DbModel::deleteFaPolozkyByFakturaId(int fakturaid)
{
    tryExecuteUpdate(wxString::Format("DELETE FROM fapolozka WHERE fakturaid=%d", fakturaid));
    return true;
}

bool DbModel::deleteFaPolozkyByZavpohId(int zavpohid)
{
    tryExecuteUpdate(wxString::Format("DELETE FROM fapolozka WHERE zavpohid=%d", zavpohid));
    return true;
}

void DbModel::addFaktura(const Faktura &faktura, bool fireUpdate)
{
    if(!hasFaktura(faktura.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("INSERT INTO faktura(id,protistranaid,popis,celkem,cislo,vs,"
                                                              "ss,ks,typid,uhradaid,pokladnaid,bankaid,menaid,predkontaceid,"
                                                              "obdobirok,vydana,platce,vystaveni,duzp,dppd,splatnost,dphdruhid,kurz,pausal) "
                                                              "VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)");
                st.Bind(1, faktura.id());
                st.Bind(2, faktura.protistranaid());
                st.Bind(3, faktura.popis());
                st.Bind(4, faktura.celkem());
                st.Bind(5, faktura.cislo());
                st.Bind(6, faktura.vs());
                st.Bind(7, faktura.ss());
                st.Bind(8, faktura.ks());
                st.Bind(9, faktura.typid());
                st.Bind(10, faktura.uhradaid());
                hasPokladna(faktura.pokladnaid()) ? st.Bind(11, faktura.pokladnaid()) : st.BindNull(11);
                hasBanka(faktura.bankaid()) ? st.Bind(12, faktura.bankaid()) : st.BindNull(12);
                st.Bind(13, faktura.menaid());
                st.Bind(14, faktura.predkontaceid());
                st.Bind(15, faktura.obdobirok());
                st.Bind(16, faktura.vydana());
                st.Bind(17, faktura.platce());
                st.BindDate(18, faktura.vystaveni());
                st.BindDate(19, faktura.duzp());
                st.BindDate(20, faktura.dppd());
                st.BindDate(21, faktura.splatnost());
                hasDphDruh(faktura.dphdruhid()) ? st.Bind(22, faktura.dphdruhid()) : st.BindNull(22);
                st.Bind(23, faktura.kurz());
                st.Bind(24, faktura.pausal());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v addFaktura: %s", e.GetMessage()));
            }
        }
        if(fireUpdate) fireUpdated(faktura.vydana() ? ID_MODEL_VYDANA : ID_MODEL_PRIJATA);
    }
}

void DbModel::editFaktura(const Faktura &faktura)
{
    if(hasFaktura(faktura.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("UPDATE faktura SET protistranaid=?,popis=?,celkem=?,cislo=?,vs=?,"
                                                              "ss=?,ks=?,typid=?,uhradaid=?,pokladnaid=?,bankaid=?,menaid=?,predkontaceid=?,"
                                                              "obdobirok=?,vydana=?,platce=?,vystaveni=?,duzp=?,dppd=?,splatnost=?,dphdruhid=?,kurz=?,pausal=? "
                                                              "WHERE id=?");
                st.Bind(1, faktura.protistranaid());
                st.Bind(2, faktura.popis());
                st.Bind(3, faktura.celkem());
                st.Bind(4, faktura.cislo());
                st.Bind(5, faktura.vs());
                st.Bind(6, faktura.ss());
                st.Bind(7, faktura.ks());
                st.Bind(8, faktura.typid());
                st.Bind(9, faktura.uhradaid());
                hasPokladna(faktura.pokladnaid()) ? st.Bind(10, faktura.pokladnaid()) : st.BindNull(10);
                hasBanka(faktura.bankaid()) ? st.Bind(11, faktura.bankaid()) : st.BindNull(11);
                st.Bind(12, faktura.menaid());
                st.Bind(13, faktura.predkontaceid());
                st.Bind(14, faktura.obdobirok());
                st.Bind(15, faktura.vydana());
                st.Bind(16, faktura.platce());
                st.BindDate(17, faktura.vystaveni());
                st.BindDate(18, faktura.duzp());
                st.BindDate(19, faktura.dppd());
                st.BindDate(20, faktura.splatnost());
                hasDphDruh(faktura.dphdruhid()) ? st.Bind(21, faktura.dphdruhid()) : st.BindNull(21);
                st.Bind(22, faktura.kurz());
                st.Bind(23, faktura.pausal());
                st.Bind(24, faktura.id());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v editFaktura: %s", e.GetMessage()));
            }
        }
        fireUpdated(faktura.vydana() ? ID_MODEL_VYDANA : ID_MODEL_PRIJATA);
    }
}

Faktura DbModel::getFaktura(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM faktura WHERE id=%d", id));
        while(rs.NextRow())
        {
            return Faktura(rs.GetInt("id"), rs.GetInt("protistranaid"), rs.GetString("popis"), rs.GetDouble("celkem"),
                           rs.GetString("cislo"), rs.GetString("vs"), rs.GetString("ss"), rs.GetString("ks"), rs.GetInt("typid"),
                           rs.GetInt("uhradaid"), rs.GetInt("pokladnaid"), rs.GetInt("bankaid"), rs.GetInt("menaid"),
                           rs.GetInt("predkontaceid"), rs.GetInt("obdobirok"), rs.GetBool("vydana"), rs.GetBool("platce"),
                           rs.GetDate("vystaveni"), rs.GetDate("duzp"), rs.GetDate("dppd"), rs.GetDate("splatnost"), rs.GetInt("dphdruhid"),
                           rs.GetDouble("kurz"), rs.GetInt("pausal"));
        }
        rs.Finalize();
    }
    return Faktura();
}

bool DbModel::deleteFaktura(int id, bool fireUpdate)
{
    bool vydana = true;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        vydana = m_db.ExecuteScalar(wxString::Format("SELECT vydana FROM faktura WHERE id=%d", id));
    }
    deleteUhradaByFakturaId(id);
    deletePohybyzasobByFakturaId(id);
    tryExecuteUpdate(wxString::Format("DELETE FROM faktura WHERE id=%d", id));
    if(fireUpdate) fireUpdated(vydana ? ID_MODEL_VYDANA : ID_MODEL_PRIJATA);
    return true;
}

bool DbModel::hasFaktura(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM faktura WHERE id=%d", id));
    }
    return false;
}

bool DbModel::isFakturaVydana(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT vydana FROM faktura WHERE id=%d", id));
    }
    return false;
}

dxFakturaArray DbModel::vystavene()
{
    dxFakturaArray faktury;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM faktura WHERE vydana=1 AND obdobirok=%d ORDER BY cislo", m_currentRok));
        while(rs.NextRow())
        {
            faktury.Add(Faktura(rs.GetInt("id"), rs.GetInt("protistranaid"), rs.GetString("popis"), rs.GetDouble("celkem"),
                           rs.GetString("cislo"), rs.GetString("vs"), rs.GetString("ss"), rs.GetString("ks"), rs.GetInt("typid"),
                           rs.GetInt("uhradaid"), rs.GetInt("pokladnaid"), rs.GetInt("bankaid"), rs.GetInt("menaid"),
                           rs.GetInt("predkontaceid"), rs.GetInt("obdobirok"), rs.GetBool("vydana"), rs.GetBool("platce"),
                           rs.GetDate("vystaveni"), rs.GetDate("duzp"), rs.GetDate("dppd"), rs.GetDate("splatnost"), rs.GetInt("dphdruhid"),
                           rs.GetDouble("kurz"), rs.GetInt("pausal")));
        }
        rs.Finalize();
    }
    return faktury;
}

dxFakturaArray DbModel::prijate()
{
    dxFakturaArray faktury;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM faktura WHERE vydana<>1 AND obdobirok=%d ORDER BY cislo", m_currentRok));
        while(rs.NextRow())
        {
            faktury.Add(Faktura(rs.GetInt("id"), rs.GetInt("protistranaid"), rs.GetString("popis"), rs.GetDouble("celkem"),
                           rs.GetString("cislo"), rs.GetString("vs"), rs.GetString("ss"), rs.GetString("ks"), rs.GetInt("typid"),
                           rs.GetInt("uhradaid"), rs.GetInt("pokladnaid"), rs.GetInt("bankaid"), rs.GetInt("menaid"),
                           rs.GetInt("predkontaceid"), rs.GetInt("obdobirok"), rs.GetBool("vydana"), rs.GetBool("platce"),
                           rs.GetDate("vystaveni"), rs.GetDate("duzp"), rs.GetDate("dppd"), rs.GetDate("splatnost"), rs.GetInt("dphdruhid"),
                           rs.GetDouble("kurz"), rs.GetInt("pausal")));
        }
        rs.Finalize();
    }
    return faktury;
}

int DbModel::copyFaktura(int id)
{
    if(hasFaktura(id))
    {
        Faktura fa = getFaktura(id);
        int newid = getLastId("faktura");
        wxString cislo;
        if(fa.vydana())
        {
            if(fa.typid() == 1) cislo = getCisloDokladu("FV");
            else cislo = getCisloDokladu("ZFV");
        }
        else
        {
            if(fa.typid() == 1) cislo = getCisloDokladu("FP");
            else cislo = getCisloDokladu("ZFP");
        }
        wxString vs = wxString::Format("%d", idbmodel->currentRok());
        wxString number;
        if(fa.vydana())
        {
            if(fa.typid() == 1) number = wxString::Format("%d", getRadaCislo("FV"));
            else number = wxString::Format("%d", getRadaCislo("ZFV"));
        }
        else
        {
            if(fa.typid() == 1) number = wxString::Format("%d", getRadaCislo("FP"));
            else number = wxString::Format("%d", getRadaCislo("ZFP"));
        }
        int zeroes = 6-(int)number.Length();
        for(int i=0; i<zeroes; i++)
            vs.Append('0');
        vs.Append(number);
        addFaktura(Faktura(newid, fa.protistranaid(), fa.popis(), fa.celkem(), cislo, vs, fa.ss(), fa.ks(), fa.typid(), fa.uhradaid(),
                               fa.pokladnaid(), fa.bankaid(), fa.menaid(), fa.predkontaceid(), fa.obdobirok(), fa.vydana(), fa.platce(),
                               dxsettings.datum(), dxsettings.datum(), dxsettings.datum(), dxsettings.datum()+wxDateSpan(0,0,0,dxsettings.splatnost()), fa.dphdruhid(),
                               fa.kurz(), fa.pausal()));
        dxFaPolozkaArray polozky = fapolozkyByFakturaId(id);
        for(size_t i=0; i<polozky.GetCount(); i++)
        {
            addFaPolozka(FaPolozka(getLastId("fapolozka"), polozky[i].popis(), polozky[i].mj(), polozky[i].pocet(), polozky[i].cenamj(), polozky[i].sazbaid(), polozky[i].dph(),
                                   polozky[i].celkem(), polozky[i].bezdph(), newid, polozky[i].zavpohid(), polozky[i].zasobaid(),
                                   polozky[i].zaokrouhleni()));
        }
        return newid;
    }
    return -1;
}

dxFaDphSazbaArray DbModel::dphFakturaSouhrn(int id, bool inCzk)
{
    dxFaDphSazbaArray souhrn;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(hasFaktura(id))
        {
            wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT kurz FROM faktura WHERE id = %d", id));
            double kurz = inCzk?rs.GetDouble(0):1.0;
            rs.Finalize();
            rs = m_db.ExecuteQuery(wxString::Format("SELECT dphsazba.sazba,sum(fapolozka.celkem)-sum(fapolozka.dph),sum(fapolozka.dph) FROM fapolozka "
                                                    "INNER JOIN dphsazba ON dphsazba.id = fapolozka.sazbaid WHERE fapolozka.fakturaid=%d GROUP BY fapolozka.sazbaid ORDER BY dphsazba.sazba DESC", id));
            while(rs.NextRow())
            {
                souhrn.Add(FaDphSazba(rs.GetInt(0), rs.GetDouble(1)*kurz, rs.GetDouble(2)*kurz));
            }
            rs.Finalize();
        }
    }
    return souhrn;
}

wxString DbModel::getFakturaCisloById(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT cislo FROM faktura WHERE id=%d", id));
        while(rs.NextRow())
        {
            return rs.GetString(0);
        }
        rs.Finalize();
    }
    return wxEmptyString;
}

dxFakturaViewArray DbModel::vystaveneview()
{
    dxFakturaViewArray faktury;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT faktura.id,faktura.cislo,faktura.popis,kontakt.jmeno,faktura.celkem,faktura.vystaveni,faktura.splatnost,faktura.platce,faktura.typid "
                                                                   "FROM faktura INNER JOIN kontakt ON kontakt.id=faktura.protistranaid WHERE faktura.vydana=1 AND faktura.obdobirok=%d", m_currentRok));
        while(rs.NextRow())
        {
            faktury.Add(FakturaView(rs.GetInt(0), rs.GetString(1), rs.GetString(2), rs.GetString(3), rs.GetDouble(4), rs.GetDate(5), rs.GetDate(6), uhrazenoByFakturaId(rs.GetInt(0)),
                                    rs.GetBool(7), rs.GetInt(8)));
        }
        rs.Finalize();
    }
    return faktury;
}

dxFakturaViewArray DbModel::prijateview()
{
    dxFakturaViewArray faktury;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT faktura.id,faktura.cislo,faktura.popis,kontakt.jmeno,faktura.celkem,faktura.vystaveni,faktura.splatnost,faktura.platce,faktura.typid "
                                                                   "FROM faktura INNER JOIN kontakt ON kontakt.id=faktura.protistranaid WHERE faktura.vydana<>1 AND faktura.obdobirok=%d", m_currentRok));
        while(rs.NextRow())
        {
            faktury.Add(FakturaView(rs.GetInt(0), rs.GetString(1), rs.GetString(2), rs.GetString(3), rs.GetDouble(4), rs.GetDate(5), rs.GetDate(6), uhrazenoByFakturaId(rs.GetInt(0)),
                                    rs.GetBool(7), rs.GetInt(8)));
        }
        rs.Finalize();
    }
    return faktury;
}

wxDateTime DbModel::getFakturaVystaveni(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT vystaveni FROM faktura WHERE id=%d", id));
        while(rs.NextRow())
        {
            return rs.GetDate(0);
        }
        rs.Finalize();
    }
    return wxDateTime::Today();
}

wxDateTime DbModel::getFakturaDuzp(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT platce,vydana,vystaveni,duzp,dppd FROM faktura WHERE id=%d", id));
        while(rs.NextRow())
        {
            return rs.GetBool(0)?(rs.GetBool(1)?rs.GetDate(3):rs.GetDate(4)):rs.GetDate(2);
        }
        rs.Finalize();
    }
    return wxDateTime::Today();
}

void DbModel::addZavPoh(const ZavPoh &zavpoh, bool fireUpdate)
{
    if(!hasZavPoh(zavpoh.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("INSERT INTO zavpoh(id,protistranaid,popis,celkem,cislo,vs,"
                                                              "uhradaid,pokladnaid,bankaid,menaid,predkontaceid,"
                                                              "obdobirok,zavazek,platce,vystaveni,duzp,dppd,splatnost,dphdruhid,kurz,pausal) "
                                                              "VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)");
                st.Bind(1, zavpoh.id());
                st.Bind(2, zavpoh.protistranaid());
                st.Bind(3, zavpoh.popis());
                st.Bind(4, zavpoh.celkem());
                st.Bind(5, zavpoh.cislo());
                st.Bind(6, zavpoh.vs());
                st.Bind(7, zavpoh.uhradaid());
                hasPokladna(zavpoh.pokladnaid()) ? st.Bind(8, zavpoh.pokladnaid()) : st.BindNull(8);
                hasBanka(zavpoh.bankaid()) ? st.Bind(9, zavpoh.bankaid()) : st.BindNull(9);
                st.Bind(10, zavpoh.menaid());
                st.Bind(11, zavpoh.predkontaceid());
                st.Bind(12, zavpoh.obdobirok());
                st.Bind(13, zavpoh.zavazek());
                st.Bind(14, zavpoh.platce());
                st.BindDate(15, zavpoh.vystaveni());
                st.BindDate(16, zavpoh.duzp());
                st.BindDate(17, zavpoh.dppd());
                st.BindDate(18, zavpoh.splatnost());
                hasDphDruh(zavpoh.dphdruhid()) ? st.Bind(19, zavpoh.dphdruhid()) : st.BindNull(19);
                st.Bind(20, zavpoh.kurz());
                st.Bind(21, zavpoh.pausal());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v addZavPoh: %s", e.GetMessage()));
            }
        }
        if(fireUpdate) fireUpdated(zavpoh.zavazek() ? ID_MODEL_ZAVAZEK : ID_MODEL_POHLEDAVKA);
    }
}

void DbModel::editZavPoh(const ZavPoh &zavpoh)
{
    if(hasZavPoh(zavpoh.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("UPDATE zavpoh SET protistranaid=?,popis=?,celkem=?,cislo=?,vs=?,"
                                                              "uhradaid=?,pokladnaid=?,bankaid=?,menaid=?,predkontaceid=?,"
                                                              "obdobirok=?,zavazek=?,platce=?,vystaveni=?,duzp=?,dppd=?,splatnost=?,dphdruhid=?,kurz=?,pausal=? "
                                                              "WHERE id=?");
                st.Bind(1, zavpoh.protistranaid());
                st.Bind(2, zavpoh.popis());
                st.Bind(3, zavpoh.celkem());
                st.Bind(4, zavpoh.cislo());
                st.Bind(5, zavpoh.vs());
                st.Bind(6, zavpoh.uhradaid());
                hasPokladna(zavpoh.pokladnaid()) ? st.Bind(7, zavpoh.pokladnaid()) : st.BindNull(7);
                hasBanka(zavpoh.bankaid()) ? st.Bind(8, zavpoh.bankaid()) : st.BindNull(8);
                st.Bind(9, zavpoh.menaid());
                st.Bind(10, zavpoh.predkontaceid());
                st.Bind(11, zavpoh.obdobirok());
                st.Bind(12, zavpoh.zavazek());
                st.Bind(13, zavpoh.platce());
                st.BindDate(14, zavpoh.vystaveni());
                st.BindDate(15, zavpoh.duzp());
                st.BindDate(16, zavpoh.dppd());
                st.BindDate(17, zavpoh.splatnost());
                hasDphDruh(zavpoh.dphdruhid()) ? st.Bind(18, zavpoh.dphdruhid()) : st.BindNull(18);
                st.Bind(19, zavpoh.kurz());
                st.Bind(20, zavpoh.pausal());
                st.Bind(21, zavpoh.id());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v editZavPoh: %s", e.GetMessage()));
            }
        }
        fireUpdated(zavpoh.zavazek() ? ID_MODEL_ZAVAZEK : ID_MODEL_POHLEDAVKA);
    }
}

ZavPoh DbModel::getZavPoh(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM zavpoh WHERE id=%d", id));
        while(rs.NextRow())
        {
            return ZavPoh(rs.GetInt("id"), rs.GetInt("protistranaid"), rs.GetString("popis"), rs.GetDouble("celkem"),
                          rs.GetString("cislo"), rs.GetString("vs"), rs.GetInt("uhradaid"), rs.GetInt("pokladnaid"),
                          rs.GetInt("bankaid"), rs.GetInt("menaid"), rs.GetInt("predkontaceid"), rs.GetInt("obdobirok"),
                          rs.GetBool("zavazek"), rs.GetBool("platce"), rs.GetDate("vystaveni"), rs.GetDate("duzp"), rs.GetDate("dppd"),
                          rs.GetDate("splatnost"), rs.GetInt("dphdruhid"), rs.GetDouble("kurz"), rs.GetInt("pausal"));
        }
        rs.Finalize();
    }
    return ZavPoh();
}

bool DbModel::deleteZavPoh(int id, bool fireUpdate)
{
    bool zavazek = true;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        zavazek = m_db.ExecuteScalar(wxString::Format("SELECT zavazek FROM zavpoh WHERE id=%d", id));
    }
    deleteUhradaByZavpohId(id);
    tryExecuteUpdate(wxString::Format("DELETE FROM zavpoh WHERE id=%d", id));
    if(fireUpdate) fireUpdated(zavazek ? ID_MODEL_ZAVAZEK : ID_MODEL_POHLEDAVKA);
    return true;
}

bool DbModel::hasZavPoh(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM zavpoh WHERE id=%d", id));
    }
    return false;
}

dxZavPohArray DbModel::zavazky()
{
    dxZavPohArray zavpoh;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM zavpoh WHERE zavazek=1 AND obdobirok=%d ORDER BY cislo", m_currentRok));
        while(rs.NextRow())
        {
            zavpoh.Add(ZavPoh(rs.GetInt("id"), rs.GetInt("protistranaid"), rs.GetString("popis"), rs.GetDouble("celkem"),
                          rs.GetString("cislo"), rs.GetString("vs"), rs.GetInt("uhradaid"), rs.GetInt("pokladnaid"),
                          rs.GetInt("bankaid"), rs.GetInt("menaid"), rs.GetInt("predkontaceid"), rs.GetInt("obdobirok"),
                          rs.GetBool("zavazek"), rs.GetBool("platce"), rs.GetDate("vystaveni"), rs.GetDate("duzp"), rs.GetDate("dppd"),
                          rs.GetDate("splatnost"), rs.GetInt("dphdruhid"), rs.GetDouble("kurz"), rs.GetInt("pausal")));
        }
        rs.Finalize();
    }
    return zavpoh;
}

dxZavPohArray DbModel::pohledavky()
{
    dxZavPohArray zavpoh;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM zavpoh WHERE zavazek<>1 AND obdobirok=%d ORDER BY cislo", m_currentRok));
        while(rs.NextRow())
        {
            zavpoh.Add(ZavPoh(rs.GetInt("id"), rs.GetInt("protistranaid"), rs.GetString("popis"), rs.GetDouble("celkem"),
                          rs.GetString("cislo"), rs.GetString("vs"), rs.GetInt("uhradaid"), rs.GetInt("pokladnaid"),
                          rs.GetInt("bankaid"), rs.GetInt("menaid"), rs.GetInt("predkontaceid"), rs.GetInt("obdobirok"),
                          rs.GetBool("zavazek"), rs.GetBool("platce"), rs.GetDate("vystaveni"), rs.GetDate("duzp"), rs.GetDate("dppd"),
                          rs.GetDate("splatnost"), rs.GetInt("dphdruhid"), rs.GetDouble("kurz"), rs.GetInt("pausal")));
        }
        rs.Finalize();
    }
    return zavpoh;
}

int DbModel::copyZavPoh(int id)
{
    if(hasZavPoh(id))
    {
        ZavPoh za = getZavPoh(id);
        int newid = getLastId("zavpoh");
        wxString cislo;
        if(za.zavazek()) cislo = getCisloDokladu("ZA");
        else cislo = getCisloDokladu("PO");
        wxString vs = wxString::Format("%d", idbmodel->currentRok());
        wxString number;
        if(za.zavazek()) number = wxString::Format("%d", getRadaCislo("ZA"));
        else number = wxString::Format("%d", getRadaCislo("PO"));
        int zeroes = 6-(int)number.Length();
        for(int i=0; i<zeroes; i++)
            vs.Append('0');
        vs.Append(number);
        addZavPoh(ZavPoh(newid, za.protistranaid(), za.popis(), za.celkem(), cislo, za.vs(), za.uhradaid(), za.pokladnaid(), za.bankaid(),
                             za.menaid(), za.predkontaceid(), za.obdobirok(), za.zavazek(), za.platce(), dxsettings.datum(), dxsettings.datum(), dxsettings.datum(),
                             dxsettings.datum()+wxDateSpan(0,0,0,dxsettings.splatnost()), za.dphdruhid(), za.kurz(), za.pausal()));
        dxFaPolozkaArray polozky = fapolozkyByZavpohId(id);
        for(size_t j=0; j<polozky.GetCount(); j++)
        {
            addFaPolozka(FaPolozka(getLastId("fapolozka"), polozky[j].popis(), polozky[j].mj(), polozky[j].pocet(), polozky[j].cenamj(), polozky[j].sazbaid(), polozky[j].dph(),
                                   polozky[j].celkem(), polozky[j].bezdph(), polozky[j].fakturaid(), newid, polozky[j].zasobaid(),
                                   polozky[j].zaokrouhleni()));
        }
        return newid;
    }
    return -1;
}

dxFaDphSazbaArray DbModel::dphZavPohSouhrn(int id, bool inCzk)
{
    dxFaDphSazbaArray souhrn;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(hasZavPoh(id))
        {
            wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT kurz FROM faktura WHERE id = %d", id));
            double kurz = inCzk?rs.GetDouble(0):1.0;
            rs.Finalize();
            rs = m_db.ExecuteQuery(wxString::Format("SELECT dphsazba.sazba,sum(fapolozka.celkem)-sum(fapolozka.dph),sum(fapolozka.dph) FROM fapolozka "
                                                    "INNER JOIN dphsazba ON dphsazba.id = fapolozka.sazbaid WHERE fapolozka.zavpohid=%d GROUP BY fapolozka.sazbaid ORDER BY dphsazba.sazba DESC", id));
            while(rs.NextRow())
            {
                souhrn.Add(new FaDphSazba(rs.GetInt(0), rs.GetDouble(1)*kurz, rs.GetDouble(2)*kurz));
            }
            rs.Finalize();
        }
    }
    return souhrn;
}

dxZavPohViewArray DbModel::zavazkyview()
{
    dxZavPohViewArray zavpoh;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT zavpoh.id,zavpoh.cislo,zavpoh.popis,kontakt.nazev,zavpoh.celkem,zavpoh.vystaveni,zavpoh.splatnost "
                                                                   "FROM zavpoh INNER JOIN kontakt ON kontakt.id=zavpoh.protistranaid WHERE zavpoh.zavazek=1 AND zavpoh.obdobirok=%d", m_currentRok));
        while(rs.NextRow())
        {
            zavpoh.Add(ZavPohView(rs.GetInt(0), rs.GetString(1), rs.GetString(2), rs.GetString(3), rs.GetDouble(4), rs.GetDate(5), rs.GetDate(6), uhrazenoByZavpohId(rs.GetInt(0))));
        }
        rs.Finalize();
    }
    return zavpoh;
}

dxZavPohViewArray DbModel::pohledavkyview()
{
    dxZavPohViewArray zavpoh;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT zavpoh.id,zavpoh.cislo,zavpoh.popis,kontakt.nazev,zavpoh.celkem,zavpoh.vystaveni,zavpoh.splatnost "
                                                                   "FROM zavpoh INNER JOIN kontakt ON kontakt.id=zavpoh.protistranaid WHERE zavpoh.zavazek<>1 AND zavpoh.obdobirok=%d", m_currentRok));
        while(rs.NextRow())
        {
            zavpoh.Add(ZavPohView(rs.GetInt(0), rs.GetString(1), rs.GetString(2), rs.GetString(3), rs.GetDouble(4), rs.GetDate(5), rs.GetDate(6), uhrazenoByZavpohId(rs.GetInt(0))));
        }
        rs.Finalize();
    }
    return zavpoh;
}

wxDateTime DbModel::getZavPohDuzp(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT platce,zavazek,vystaveni,duzp,dppd FROM zavpoh WHERE id=%d", id));
        while(rs.NextRow())
        {
            return rs.GetBool(0)?(rs.GetBool(1)?rs.GetDate(4):rs.GetDate(3)):rs.GetDate(2);
        }
        rs.Finalize();
    }
    return wxDateTime::Today();
}

bool DbModel::hasPohledavkyDluhyCizi(int rok)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT id,celkem FROM faktura WHERE obdobirok<=%d AND menaid<>1", rok));
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            if(tb.GetDouble(1) > m_db.ExecuteQuery(wxString::Format("SELECT total(castka) FROM uhrada WHERE fakturaid=%d", tb.GetInt(0))).GetDouble(0)) return true;
        }
    }
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT id,celkem FROM zavpoh WHERE obdobirok<=%d AND menaid<>1", rok));
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            if(tb.GetDouble(1) > m_db.ExecuteQuery(wxString::Format("SELECT total(castka) FROM uhrada WHERE zavpohid=%d", tb.GetInt(0))).GetDouble(0)) return true;
        }
    }
    return false;
}

bool DbModel::hasJednotneKurzyForPohledavkyDluhy(int rok)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT id,menaid,celkem FROM faktura WHERE obdobirok<=%d AND menaid<>1", rok));
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            //pro rok == aktualni kalendarni se pouziji kurzy z predchoziho
            if(!dxutils::isSameDouble(tb.GetDouble(2), m_db.ExecuteQuery(wxString::Format("SELECT total(castka) FROM uhrada WHERE fakturaid=%d", tb.GetInt(0))).GetDouble(0))
                    && !hasJednotnyKurz(rok==wxDateTime::Today().GetYear()?rok-1:rok, zkratkaMena(tb.GetInt(1)))) return false;
        }
    }
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT id,menaid,celkem FROM zavpoh WHERE obdobirok<=%d AND menaid<>1", rok));
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            //pro rok == aktualni kalendarni se pouziji kurzy z predchoziho
            if(!dxutils::isSameDouble(tb.GetDouble(2), m_db.ExecuteQuery(wxString::Format("SELECT total(castka) FROM uhrada WHERE zavpohid=%d", tb.GetInt(0))).GetDouble(0))
                    && !hasJednotnyKurz(rok==wxDateTime::Today().GetYear()?rok-1:rok, zkratkaMena(tb.GetInt(1)))) return false;
        }
    }
    return true;
}

double DbModel::getPohledavkyStav(int rok)
{
    double result = 0.0;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT id,menaid,celkem FROM faktura WHERE obdobirok<=%d AND vydana=1", rok));
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            if(!isFakturaUhrazena(tb.GetInt("id")))
            {
                if(tb.GetInt("menaid") == 1) result += tb.GetDouble("celkem") - uhrazenoByFakturaId(tb.GetInt("id"));
                //pro rok == aktualni kalendarni se pouziji kurzy z predchoziho
                else result += (tb.GetDouble("celkem") - uhrazenoByFakturaId(tb.GetInt("id"))) * getJednotnyKurz(zkratkaMena(tb.GetInt("menaid")), rok==wxDateTime::Today().GetYear()?rok-1:rok);
            }
        }
    }
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT id,menaid,celkem FROM zavpoh WHERE obdobirok<=%d AND zavazek<>1", rok));
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            if(!isZavpohUhrazena(tb.GetInt("id")))
            {
                if(tb.GetInt("menaid") == 1) result += tb.GetDouble("celkem") - uhrazenoByZavpohId(tb.GetInt("id"));
                //pro rok == aktualni kalendarni se pouziji kurzy z predchoziho
                else result += (tb.GetDouble("celkem") - uhrazenoByZavpohId(tb.GetInt("id"))) * getJednotnyKurz(zkratkaMena(tb.GetInt("menaid")), rok==wxDateTime::Today().GetYear()?rok-1:rok);
            }
        }
    }
    return result;
}

double DbModel::getZavazkyStav(int rok)
{
    double result = 0.0;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT id,menaid,celkem FROM faktura WHERE obdobirok<=%d AND vydana<>1", rok));
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            if(!isFakturaUhrazena(tb.GetInt("id")))
            {
                if(tb.GetInt("menaid") == 1) result += tb.GetDouble("celkem") - uhrazenoByFakturaId(tb.GetInt("id"));
                else result += (tb.GetDouble("celkem") - uhrazenoByFakturaId(tb.GetInt("id"))) * getJednotnyKurz(zkratkaMena(tb.GetInt("menaid")), rok==wxDateTime::Today().GetYear()?rok-1:rok);
            }
        }
    }
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT id,menaid,celkem,zavazek FROM zavpoh WHERE obdobirok<=%d AND zavazek=1", rok));
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            if(!isZavpohUhrazena(tb.GetInt("id")))
            {
                if(tb.GetInt("menaid") == 1) result += tb.GetDouble("celkem") - uhrazenoByZavpohId(tb.GetInt("id"));
                else result += (tb.GetDouble("celkem") - uhrazenoByZavpohId(tb.GetInt("id"))) * getJednotnyKurz(zkratkaMena(tb.GetInt("menaid")), rok==wxDateTime::Today().GetYear()?rok-1:rok);
            }
        }
    }
    return result;
}

void DbModel::addUhrada(const Uhrada &uhrada)
{
    if(!hasUhrada(uhrada.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("INSERT INTO uhrada(id,castka,denikid,fakturaid,zavpohid,obdobirok) "
                                                              "VALUES(?,?,?,?,?,?)");
                st.Bind(1, uhrada.id());
                st.Bind(2, uhrada.castka());
                st.Bind(3, uhrada.denikid());
                hasFaktura(uhrada.fakturaid()) ? st.Bind(4, uhrada.fakturaid()) : st.BindNull(4);
                hasZavPoh(uhrada.zavpohid()) ? st.Bind(5, uhrada.zavpohid()) : st.BindNull(5);
                st.Bind(6, uhrada.obdobirok());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v addUhrada: %s", e.GetMessage()));
            }
        }
        if(hasFaktura(uhrada.fakturaid()))
        {
            bool vydana = true;
            if(dxsettings.dbmodel() == DB_SQLITE)
            {
                vydana = m_db.ExecuteScalar(wxString::Format("SELECT vydana FROM faktura WHERE id=%d", uhrada.fakturaid()));
            }
            fireUpdated(vydana ? ID_MODEL_VYDANA : ID_MODEL_PRIJATA);
        }
        else
        {
            bool zavazek = true;
            if(dxsettings.dbmodel() == DB_SQLITE)
            {
                zavazek = m_db.ExecuteScalar(wxString::Format("SELECT zavazek FROM zavpoh WHERE id=%d", uhrada.zavpohid()));
            }
            fireUpdated(zavazek ? ID_MODEL_ZAVAZEK : ID_MODEL_POHLEDAVKA);
        }
    }
}

bool DbModel::hasUhrada(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM uhrada WHERE id=%d", id));
    }
    return false;
}

double DbModel::uhrazenoByFakturaId(int fakturaid)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteQuery(wxString::Format("SELECT total(castka) FROM uhrada WHERE fakturaid=%d", fakturaid)).GetDouble(0);
    }
    return 0.0;
}

double DbModel::uhrazenoByZavpohId(int zavpohid)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteQuery(wxString::Format("SELECT total(castka) FROM uhrada WHERE zavpohid=%d", zavpohid)).GetDouble(0);
    }
    return 0.0;
}

bool DbModel::isFakturaUhrazena(int fakturaid)
{
    if(hasFaktura(fakturaid))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT celkem,(SELECT total(uhrada.castka) FROM uhrada WHERE uhrada.fakturaid=%d) FROM faktura WHERE faktura.id=%d", fakturaid, fakturaid));
            if(rs.NextRow())
            {
                return rs.GetDouble(0) <= rs.GetDouble(1);
            }
            rs.Finalize();
        }
    }
    return false;
}

bool DbModel::isFakturaPosplatnosti(int fakturaid)
{
    if(isFakturaUhrazena(fakturaid))
    {
        return false;
    }
    if(hasFaktura(fakturaid) && getFaktura(fakturaid).splatnost().IsEarlierThan(dxsettings.datum()))
    {
        return true;
    }
    return false;
}

bool DbModel::isZavpohUhrazena(int zavpohid)
{
    if(hasZavPoh(zavpohid))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT celkem,(SELECT total(uhrada.castka) FROM uhrada WHERE uhrada.zavpohid=%d) FROM zavpoh WHERE zavpoh.id=%d", zavpohid, zavpohid));
            if(rs.NextRow())
            {
                return rs.GetDouble(0) <= rs.GetDouble(1);
            }
            rs.Finalize();
        }
    }
    return false;
}

bool DbModel::isZavpohPosplatnosti(int zavpohid)
{
    if(isZavpohUhrazena(zavpohid))
    {
        return false;
    }
    if(hasZavPoh(zavpohid) && getZavPoh(zavpohid).splatnost().IsEarlierThan(dxsettings.datum()))
    {
        return true;
    }
    return false;
}

bool DbModel::deleteUhradaByFakturaId(int fakturaid)
{
    bool result = someUhradaForFaktura(fakturaid);
    tryExecuteUpdate(wxString::Format("DELETE FROM denik WHERE id IN (SELECT denikid FROM uhrada WHERE fakturaid=%d)", fakturaid));
    fireUpdated(ID_MODEL_BANKA);
    fireUpdated(ID_MODEL_POKLADNA);
    return result;
}

bool DbModel::deleteUhradaByZavpohId(int zavpohid)
{
    bool result = someUhradaForZavpoh(zavpohid);
    tryExecuteUpdate(wxString::Format("DELETE FROM denik WHERE id IN (SELECT denikid FROM uhrada WHERE zavpohid=%d)", zavpohid));
    fireUpdated(ID_MODEL_BANKA);
    fireUpdated(ID_MODEL_POKLADNA);
    return result;
}

bool DbModel::someUhradaForFaktura(int fakturaid)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM uhrada WHERE fakturaid=%d", fakturaid));
    }
    return false;
}

bool DbModel::someUhradaForZavpoh(int zavpohid)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM uhrada WHERE zavpohid=%d", zavpohid));
    }
    return false;
}

dxDenikArray DbModel::denikUhradForFaktura(int fakturaid)
{
    dxDenikArray denik;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT * FROM denik WHERE id IN (SELECT denikid FROM uhrada WHERE fakturaid=%d)", fakturaid));
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            denik.Add(Denik(tb.GetInt("id"), tb.GetDate("datum"), tb.GetString("cislo"), tb.GetString("popis"), tb.GetDouble("castka"),
                                  tb.GetDouble("castkacizi"), tb.GetDouble("kurz"), tb.GetString("reference"), tb.GetInt("menaid"),
                                  tb.GetInt("predkontaceid"), tb.GetInt("pokladnaid"),
                                  tb.GetInt("bankaid"), tb.GetInt("vypis"), tb.GetInt("protistranaid"), tb.GetInt("obdobirok"), tb.GetInt("pausal")));
        }
    }
    return denik;
}

dxDenikArray DbModel::denikUhradForZavpoh(int zavpohid)
{
    dxDenikArray denik;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT * FROM denik WHERE id IN (SELECT denikid FROM uhrada WHERE zavpohid=%d)", zavpohid));
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            denik.Add(Denik(tb.GetInt("id"), tb.GetDate("datum"), tb.GetString("cislo"), tb.GetString("popis"), tb.GetDouble("castka"),
                                  tb.GetDouble("castkacizi"), tb.GetDouble("kurz"), tb.GetString("reference"), tb.GetInt("menaid"),
                                  tb.GetInt("predkontaceid"), tb.GetInt("pokladnaid"),
                                  tb.GetInt("bankaid"), tb.GetInt("vypis"), tb.GetInt("protistranaid"), tb.GetInt("obdobirok"), tb.GetInt("pausal")));
        }
    }
    return denik;
}

bool DbModel::someNeuhrazenaFakturaOrZavpoh()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT id FROM faktura");
        while(rs.NextRow())
        {
            if(!isFakturaUhrazena(rs.GetInt(0)))
            {
                rs.Finalize();
                return true;
            }
        }
        rs.Finalize();
        rs = m_db.ExecuteQuery("SELECT id FROM zavpoh");
        while(rs.NextRow())
        {
            if(!isZavpohUhrazena(rs.GetInt(0)))
            {
                rs.Finalize();
                return true;
            }
        }
        rs.Finalize();
    }
    return false;
}

dxFakturaViewArray DbModel::neuhrazeneFakturyView()
{
    dxFakturaViewArray faktury;
    wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT faktura.id,faktura.cislo,faktura.popis,kontakt.jmeno,faktura.celkem,faktura.vystaveni,faktura.splatnost,faktura.platce,faktura.typid "
                                                               "FROM faktura INNER JOIN kontakt ON kontakt.id=faktura.protistranaid WHERE faktura.obdobirok<=%d", m_currentRok));
    while(rs.NextRow())
    {
        if(!isFakturaUhrazena(rs.GetInt(0)))
        {
            faktury.Add(FakturaView(rs.GetInt(0), rs.GetString(1), rs.GetString(2), rs.GetString(3), rs.GetDouble(4), rs.GetDate(5), rs.GetDate(6), uhrazenoByFakturaId(rs.GetInt(0)),
                                    rs.GetBool(7), rs.GetInt(8)));
        }
    }
    rs.Finalize();
    return faktury;
}

dxZavPohViewArray DbModel::neuhrazeneZavpohyView()
{
    dxZavPohViewArray zavpohy;
    wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT zavpoh.id,zavpoh.cislo,zavpoh.popis,kontakt.nazev,zavpoh.celkem,zavpoh.vystaveni,zavpoh.splatnost "
                                                               "FROM zavpoh INNER JOIN kontakt ON kontakt.id=zavpoh.protistranaid WHERE zavpoh.obdobirok<=%d", m_currentRok));
    while(rs.NextRow())
    {
        if(!isZavpohUhrazena(rs.GetInt(0)))
        {
            zavpohy.Add(ZavPohView(rs.GetInt(0), rs.GetString(1), rs.GetString(2), rs.GetString(3), rs.GetDouble(4), rs.GetDate(5), rs.GetDate(6), uhrazenoByZavpohId(rs.GetInt(0))));
        }
    }
    rs.Finalize();
    return zavpohy;
}

dxOdpiskupinaArray DbModel::odpiskupiny()
{
    dxOdpiskupinaArray skupiny;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable("SELECT * FROM odpiskupina");
        if(!tb.GetRowCount()) return skupiny;
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            skupiny.Add(Odpiskupina(tb.GetInt("id"), tb.GetString("popis"), tb.GetInt("odpisovat"),
                                              tb.GetDouble("rov1"), tb.GetDouble("rov2"), tb.GetDouble("rovzvy"),
                                              tb.GetDouble("zry1"), tb.GetDouble("zry2"), tb.GetDouble("zryzvy")));
        }
    }
    return skupiny;
}

bool DbModel::hasOdpiskupina(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM odpiskupina WHERE id=%d", id));
    }
    return false;
}

Odpiskupina DbModel::getOdpiskupina(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM odpiskupina WHERE id=%d", id));
        while(rs.NextRow())
        {
            return Odpiskupina(rs.GetInt("id"), rs.GetString("popis"), rs.GetInt("odpisovat"),
                               rs.GetDouble("rov1"), rs.GetDouble("rov2"), rs.GetDouble("rovzvy"),
                               rs.GetDouble("zry1"), rs.GetDouble("zry2"), rs.GetDouble("zryzvy"));
        }
        rs.Finalize();
    }
    return Odpiskupina();
}

dxMajetekArray DbModel::majetek()
{
    dxMajetekArray items;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT * FROM majetek WHERE CAST(STRFTIME('%%Y',porizen) AS INTEGER) <= %d ORDER BY cislo", m_currentRok));
        if(!tb.GetRowCount()) return items;
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            items.Add(Majetek(tb.GetInt("id"), tb.GetString("cislo"), tb.GetString("nazev"),
                              tb.GetString("popis"), tb.GetDate("porizen"), tb.GetBool("vyrazen"),
                              tb.GetDate("datumvyrazeni"), tb.GetDouble("cena"),tb.GetInt("typ"),
                              tb.GetInt("odpis"), tb.GetInt("skupinaid")));
        }
    }
    return items;
}

void DbModel::addMajetek(const Majetek &majetek)
{
    if(!hasMajetek(majetek.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("INSERT INTO majetek(id,cislo,nazev,popis,porizen,vyrazen,"
                                                              "datumvyrazeni,cena,typ,odpis,skupinaid) "
                                                              "VALUES(?,?,?,?,?,?,?,?,?,?,?)");
                st.Bind(1, majetek.id());
                st.Bind(2, majetek.cislo());
                st.Bind(3, majetek.nazev());
                st.Bind(4, majetek.popis());
                st.BindDate(5, majetek.porizen());
                st.BindBool(6, majetek.vyrazen());
                st.BindDate(7, majetek.datumvyrazeni());
                st.Bind(8, majetek.cena());
                st.Bind(9, majetek.typ());
                st.Bind(10, majetek.odpis());
                st.Bind(11, majetek.skupinaid());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v addMajetek: %s", e.GetMessage()));
            }
        }
        fireUpdated(ID_MODEL_MAJETEK);
    }
}

void DbModel::editMajetek(const Majetek &majetek)
{
    if(hasMajetek(majetek.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("UPDATE majetek SET cislo=?,nazev=?,popis=?,porizen=?,vyrazen=?,"
                                                              "datumvyrazeni=?,cena=?,typ=?,odpis=?,skupinaid=?"
                                                              " WHERE id=?");
                st.Bind(1, majetek.cislo());
                st.Bind(2, majetek.nazev());
                st.Bind(3, majetek.popis());
                st.BindDate(4, majetek.porizen());
                st.BindBool(5, majetek.vyrazen());
                st.BindDate(6, majetek.datumvyrazeni());
                st.Bind(7, majetek.cena());
                st.Bind(8, majetek.typ());
                st.Bind(9, majetek.odpis());
                st.Bind(10, majetek.skupinaid());
                st.Bind(11, majetek.id());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v editMajetek: %s", e.GetMessage()));
            }
        }
        fireUpdated(ID_MODEL_MAJETEK);
    }
}

Majetek DbModel::getMajetek(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM majetek WHERE id=%d", id));
        while(rs.NextRow())
        {
            return Majetek(rs.GetInt("id"), rs.GetString("cislo"), rs.GetString("nazev"),
                           rs.GetString("popis"), rs.GetDate("porizen"), rs.GetBool("vyrazen"),
                           rs.GetDate("datumvyrazeni"), rs.GetDouble("cena"), rs.GetInt("typ"),
                           rs.GetInt("odpis"), rs.GetInt("skupinaid"));
        }
        rs.Finalize();
    }
    return Majetek();
}

bool DbModel::deleteMajetek(int id)
{
    tryExecuteUpdate(wxString::Format("DELETE FROM majetek WHERE id=%d", id));
    fireUpdated(ID_MODEL_MAJETEK);
    return true;
}

bool DbModel::hasMajetek(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM majetek WHERE id=%d", id));
    }
    return false;
}

double DbModel::getZustatkovaCenaMajetku(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteQuery(wxString::Format("SELECT CASE WHEN majetek.odpis=0 OR majetek.odpis=3 THEN 0 "
                                                  "ELSE majetek.cena-(SELECT TOTAL(odpis.odpis) FROM odpis WHERE odpis.majetekid=%d AND odpis.rok<=%d) END"
                                                  " FROM majetek WHERE majetek.id=%d", id, m_currentRok, id)).GetDouble(0);
    }
    return 0.0;
}

int DbModel::copyMajetek(int id)
{
    if(hasMajetek(id))
    {
        Majetek ma = getMajetek(id);
        int newid = getLastId("majetek");
        addMajetek(Majetek(newid, getCisloDokladu("MA"), ma.nazev(), ma.popis(), ma.porizen(), ma.vyrazen(), ma.datumvyrazeni(),
                               ma.cena(), ma.typ(), ma.odpis(), ma.skupinaid()));
        return newid;
    }
    return -1;
}

double DbModel::getMajetekStav(const wxDateTime &toDate, bool hmotny)
{
    calculateOdpisy();
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(hmotny)
        {
            return m_db.ExecuteQuery(wxString::Format("SELECT SUM(majetek.cena)-(SELECT TOTAL(odpis.odpis) FROM odpis WHERE odpis.rok<%d AND "
                                                      "odpis.majetekid IN (SELECT majetek.id FROM majetek WHERE majetek.porizen < date('%s') "
                                                      "AND majetek.vyrazen=0 AND majetek.typ=1)) FROM majetek WHERE majetek.porizen < date('%s') "
                                                      "AND majetek.vyrazen=0 AND majetek.typ=1 AND (majetek.odpis=1 OR majetek.odpis=2)",
                                                      wxDateTime(toDate+wxDateSpan(0,0,0,1)).GetYear(), wxDateTime(toDate+wxDateSpan(0,0,0,1)).FormatISODate(),
                                                      wxDateTime(toDate+wxDateSpan(0,0,0,1)).FormatISODate())).GetDouble(0);
        }
        else
        {
            return m_db.ExecuteQuery(wxString::Format("SELECT SUM(majetek.cena)-(SELECT TOTAL(odpis.odpis) FROM odpis WHERE odpis.rok<%d AND "
                                                      "odpis.majetekid IN (SELECT majetek.id FROM majetek WHERE majetek.porizen < date('%s') "
                                                      "AND majetek.vyrazen=0 AND majetek.typ<>1)) FROM majetek WHERE majetek.porizen < date('%s') "
                                                      "AND majetek.vyrazen=0 AND majetek.typ<>1 AND (majetek.odpis=1 OR majetek.odpis=2)",
                                                      wxDateTime(toDate+wxDateSpan(0,0,0,1)).GetYear(), wxDateTime(toDate+wxDateSpan(0,0,0,1)).FormatISODate(),
                                                      wxDateTime(toDate+wxDateSpan(0,0,0,1)).FormatISODate())).GetDouble(0);
        }
    }
    return 0.0;
}

wxString DbModel::majetekTypAsString(int typ)
{
    if(typ == 1) return wxString::FromUTF8("Dlouhodobý hmotný");
    else if(typ == 2) return wxString::FromUTF8("Dlouhodobý nehmotný");
    else return wxString::FromUTF8("Drobný");
}

wxString DbModel::majetekOdpisAsString(int odpis)
{
    if(odpis == 0) return wxString::FromUTF8("Bez odpisů");
    else if(odpis == 1) return wxString::FromUTF8("Rovnoměrný");
    else if(odpis == 2) return wxString::FromUTF8("Zrychlený");
    else return wxString::FromUTF8("Vyloučený");
}

dxMajetekViewArray DbModel::majetekview()
{
    dxMajetekViewArray items;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT * FROM majetek WHERE CAST(STRFTIME('%%Y',porizen) AS INTEGER) <= %d", m_currentRok));
        if(!tb.GetRowCount()) return items;
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            items.Add(MajetekView(tb.GetInt("id"), tb.GetString("cislo"), tb.GetString("nazev"), tb.GetDate("porizen"), tb.GetBool("vyrazen"),
                                  tb.GetDouble("cena"), getZustatkovaCenaMajetku(tb.GetInt("id")), majetekTypAsString(tb.GetInt("typ")), majetekOdpisAsString(tb.GetInt("odpis"))));
        }
    }
    return items;
}

void DbModel::addOdpis(const Odpis &odpis)
{
    if(!hasOdpis(odpis.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("INSERT INTO odpis(id,rok,odpis,majetekid) "
                                                              "VALUES(?,?,?,?)");
                st.Bind(1, odpis.id());
                st.Bind(2, odpis.rok());
                st.Bind(3, odpis.odpis());
                st.Bind(4, odpis.majetekid());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v addOdpis: %s", e.GetMessage()));
            }
        }
        fireUpdated(ID_MODEL_MAJETEK);
    }
}

bool DbModel::hasOdpis(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM odpis WHERE id=%d", id));
    }
    return false;
}

bool DbModel::hasOdpis(int majetekid, int rok)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM odpis WHERE majetekid=%d AND rok=%d", majetekid, rok));
    }
    return false;
}

double DbModel::getOdpisy(int majetekid, int kroku)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteQuery(wxString::Format("SELECT TOTAL(odpis) FROM odpis WHERE majetekid=%d AND rok<=%d", majetekid, kroku)).GetDouble(0);
    }
    return 0.0;
}

double DbModel::calculateRovnomernyOdpis(double cena, double koeficient)
{
    return floor((cena*koeficient)/100);
}

double DbModel::calculateZrychlenyOdpis(double cena, double koeficient, int rok, int majetekid, int porizen)
{
    double zustatkovacena = cena - getOdpisy(majetekid, rok);
    int let = rok - porizen;
    return floor((2*zustatkovacena)/(koeficient-let));
}

void DbModel::calculateOdpisy()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        int rok = wxDateTime::Today().GetYear();
        int porizen, odpisovat;
        wxSQLite3Table tb = m_db.GetTable("SELECT majetek.id,CAST(STRFTIME('%Y',majetek.porizen) AS INTEGER) AS rok,majetek.cena,majetek.odpis,odpiskupina.odpisovat,"
                                          "odpiskupina.rov1,odpiskupina.rov2,odpiskupina.zry1,odpiskupina.zry2 FROM majetek INNER JOIN odpiskupina ON odpiskupina.id=majetek.skupinaid "
                                          "WHERE majetek.odpis=1 OR majetek.odpis=2");
        for(int i=0; i<tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            porizen = tb.GetInt(1);
            odpisovat = tb.GetInt(4);
            if(tb.GetInt(3)==1) //rovnomerny odpis
            {
                for(int j=0; j<odpisovat && porizen+j<=rok; j++)
                {
                    if(!m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM odpis WHERE majetekid=%d AND rok=%d", tb.GetInt(0), porizen+j))) //nema odpis pro dany rok
                    {
                        if(!j) //prvni rok
                        {
                            addOdpis(Odpis(getLastId("odpis"), porizen+j, calculateRovnomernyOdpis(tb.GetDouble(2), tb.GetDouble(5)), tb.GetInt(0)));
                        }
                        else if(j+1 == odpisovat) //posledni rok
                        {
                            addOdpis(Odpis(getLastId("odpis"), porizen+j, tb.GetDouble(2)-getOdpisy(tb.GetInt(0), porizen+j-1), tb.GetInt(0)));
                        }
                        else
                        {
                            addOdpis(Odpis(getLastId("odpis"), porizen+j, calculateRovnomernyOdpis(tb.GetDouble(2), tb.GetDouble(6)), tb.GetInt(0)));
                        }
                    }
                }
            }
            else //zrychleny
            {
                for(int j=0; j<odpisovat && porizen+j<=rok; j++)
                {
                    if(!m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM odpis WHERE majetekid=%d AND rok=%d", tb.GetInt(0), porizen+j))) //nema odpis pro dany rok
                    {
                        if(!j) //prvni rok
                        {
                            addOdpis(Odpis(getLastId("odpis"), porizen+j, floor(tb.GetDouble(2)/tb.GetDouble(7)), tb.GetInt(0)));
                        }
                        else if(j+1 == odpisovat) //posledni rok
                        {
                            addOdpis(Odpis(getLastId("odpis"), porizen+j, tb.GetDouble(2)-getOdpisy(tb.GetInt(0), porizen+j-1), tb.GetDouble(0)));
                        }
                        else
                        {
                            addOdpis(Odpis(getLastId("odpis"), porizen+j, calculateZrychlenyOdpis(tb.GetDouble(2), tb.GetDouble(8), porizen+j, tb.GetDouble(0), porizen), tb.GetDouble(0)));
                        }
                    }
                }
            }
        }
    }
    fireUpdated(ID_MODEL_MAJETEK);
}

dxOdpisArray DbModel::odpisyMajetku(int majetekid, int kroku)
{
    if(kroku == -1) kroku = m_currentRok;
    dxOdpisArray odpisy;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM odpis WHERE majetekid=%d AND rok<=%d ORDER BY rok ASC", majetekid, kroku));
        while(rs.NextRow())
        {
            odpisy.Add(Odpis(rs.GetInt("id"), rs.GetInt("rok"), rs.GetDouble("odpis"), rs.GetInt("majetekid")));
        }
        rs.Finalize();
    }
    return odpisy;
}

void DbModel::addSettings(const Settings &settings)
{
    if(!hasSettings(settings.rok()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            if(m_db.IsOpen())
            {
                try
                {
                    wxSQLite3Statement st = m_db.PrepareStatement("INSERT INTO settings(rok,splatnost) "
                                                                  "VALUES(?,?)");
                    st.Bind(1, settings.rok());
                    st.Bind(2, settings.splatnost());
                    st.ExecuteUpdate();
                    st.Finalize();
                }
                catch(const wxSQLite3Exception& e)
                {
                    wxLogDebug(wxString::Format("Chyba v addSettings: %s", e.GetMessage()));
                }
            }
        }
    }
}

void DbModel::editSettings(const Settings &settings)
{
    if(hasSettings(settings.rok()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("UPDATE settings SET splatnost=?"
                                                              " WHERE rok=?");
                st.Bind(1, settings.splatnost());
                st.Bind(2, settings.rok());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v editSettings: %s", e.GetMessage()));
            }
        }
    }
    //Pokud neni settings pro dany rok
    else
    {
        addSettings(Settings(settings.rok(), settings.splatnost()));
    }
}

bool DbModel::hasSettings(int rok)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(!m_db.IsOpen())
        {
            return false;
        }
        try
        {
            return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM settings WHERE rok=%d", rok));
        }
        catch(const wxSQLite3Exception& e)
        {
            wxLogDebug(wxString::Format("Chyba v hasSettings: %s", e.GetMessage()));
        }
    }
    return false;
}

int DbModel::getSplatnost(int rok)
{
    if(hasSettings(rok))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            return m_db.ExecuteScalar(wxString::Format("SELECT splatnost FROM settings WHERE rok=%d", rok));
        }
    }
    addSettings(Settings(rok, 14));
    return 14;
}

Smtp DbModel::smtp() const
{
    return m_smtp;
}

void DbModel::setSmtp(const Smtp &smtp)
{
    m_smtp = smtp;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.IsOpen())
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("UPDATE smtp SET server=?,port=?,ssl=?,autentifikace=?,"
                                                              "username=?,password=?,sender=?,subject=?,text=?");
                st.Bind(1, m_smtp.server());
                st.Bind(2, m_smtp.port());
                st.BindBool(3, m_smtp.ssl());
                st.Bind(4, m_smtp.autentifikace());
                st.Bind(5, m_smtp.username());
                st.Bind(6, m_smtp.password());
                st.Bind(7, m_smtp.sender());
                st.Bind(8, m_smtp.subject());
                st.Bind(9, m_smtp.text());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v setSmtp: %s", e.GetMessage()));
            }
        }
    }
}

void DbModel::setSmtpSender(const wxString &sender)
{
    m_smtp.setSender(sender);
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.IsOpen())
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("UPDATE smtp SET sender=?");
                st.Bind(1, sender);
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v setSmtpSender: %s", e.GetMessage()));
            }
        }
    }
}

dxOdeslanyArray DbModel::odeslane()
{
    dxOdeslanyArray odeslane;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT * FROM odeslany WHERE CAST(strftime('%%Y',cas) AS INTEGER)=%d", wxDateTime::Today().GetYear()));
        if(!tb.GetRowCount()) return odeslane;
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            odeslane.Add(Odeslany(tb.GetInt("id"), tb.GetDateTime("cas"), tb.GetString("faktura"),
                                      tb.GetString("prijemce")));
        }
    }
    return odeslane;
}

void DbModel::addOdeslany(const Odeslany &odeslany)
{
    if(!hasOdeslany(odeslany.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("INSERT INTO odeslany(id,cas,faktura,prijemce) "
                                                              "VALUES(?,?,?,?)");
                st.Bind(1, odeslany.id());
                st.BindDateTime(2, odeslany.cas());
                st.Bind(3, odeslany.faktura());
                st.Bind(4, odeslany.prijemce());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v addOdeslany: %s", e.GetMessage()));
            }
        }
    }
}

bool DbModel::hasOdeslany(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM odeslany WHERE id=%d", id));
    }
    return false;
}

dxUlozeneArray DbModel::ulozene()
{
    dxUlozeneArray ulozene;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable("SELECT * FROM ulozene");
        if(!tb.GetRowCount()) return ulozene;
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            ulozene.Add(Ulozene(tb.GetInt("id"), tb.GetInt("typ"), tb.GetString("popis"), tb.GetString("forma"), tb.GetInt("obdobi"),
                                tb.GetInt("mesic"), tb.GetInt("ctvrtleti"), tb.GetInt("rok"), tb.GetDate("od"), tb.GetDate("do"), tb.GetString("xml")));
        }
    }
    return ulozene;
}

void DbModel::addUlozene(const Ulozene &ulozene)
{
    if(!hasUlozene(ulozene.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("INSERT INTO ulozene(id,typ,popis,forma,obdobi,mesic,ctvrtleti,rok,od,do,xml) "
                                                              "VALUES(?,?,?,?,?,?,?,?,?,?,?)");
                st.Bind(1, ulozene.id());
                st.Bind(2, ulozene.typ());
                st.Bind(3, ulozene.popis());
                st.Bind(4, ulozene.forma());
                st.Bind(5, ulozene.obdobi());
                st.Bind(6, ulozene.mesic());
                st.Bind(7, ulozene.ctvrtleti());
                st.Bind(8, ulozene.rok());
                st.BindDate(9, ulozene.from());
                st.BindDate(10, ulozene.to());
                st.Bind(11, ulozene.xml());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v addUlozene: %s", e.GetMessage()));
            }
        }
    }
}

void DbModel::editUlozene(const Ulozene &ulozene)
{
    if(hasUlozene(ulozene.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("UPDATE ulozene SET typ=?,popis=?,forma=?,obdobi=?,mesic=?,ctvrtleti=?,"
                                                              "rok=?,od=?,do=?,xml=? "
                                                              "WHERE id=?");
                st.Bind(1, ulozene.typ());
                st.Bind(2, ulozene.popis());
                st.Bind(3, ulozene.forma());
                st.Bind(4, ulozene.obdobi());
                st.Bind(5, ulozene.mesic());
                st.Bind(6, ulozene.ctvrtleti());
                st.Bind(7, ulozene.rok());
                st.BindDate(8, ulozene.from());
                st.BindDate(9, ulozene.to());
                st.Bind(10, ulozene.xml());
                st.Bind(11, ulozene.id());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v editUlozene: %s", e.GetMessage()));
            }
        }
    }
}

Ulozene DbModel::getUlozene(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM ulozene WHERE id=%d", id));
        while(rs.NextRow())
        {
            return Ulozene(rs.GetInt("id"), rs.GetInt("typ"), rs.GetString("popis"), rs.GetString("forma"), rs.GetInt("obdobi"),
                           rs.GetInt("mesic"), rs.GetInt("ctvrtleti"), rs.GetInt("rok"), rs.GetDate("od"), rs.GetDate("do"), rs.GetString("xml"));
        }
        rs.Finalize();
    }
    return Ulozene();
}

int DbModel::getUlozeneId(int typ, const wxString &forma, int obdobi, const wxDateTime &from, const wxDateTime &to)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT id FROM ulozene WHERE typ=%d AND forma='%s' AND obdobi=%d AND od=date('%s') AND do=date('%s')",
                                                   typ, forma, obdobi, from.FormatISODate(), to.FormatISODate()));
    }
    return -1;
}

size_t DbModel::getUlozeneCount()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar("SELECT count(*) FROM ulozene");
    }
    return 0;
}

bool DbModel::deleteUlozene(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        Ulozene ul = getUlozene(id);
        if(ul.isValid())
        {
            wxRemoveFile(ul.xml());
            tryExecuteUpdate(wxString::Format("DELETE FROM ulozene WHERE id=%d", id));
            return true;
        }
    }
    return false;
}

bool DbModel::hasUlozene(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM ulozene WHERE id=%d", id));
    }
    return false;
}

bool DbModel::hasUlozene(int typ, const wxString &forma, int obdobi, const wxDateTime &from, const wxDateTime &to)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM ulozene WHERE typ=%d AND forma='%s' AND obdobi=%d AND od=date('%s') AND do=date('%s')",
                                                   typ, forma, obdobi, from.FormatISODate(), to.FormatISODate()));
    }
    return false;
}

bool DbModel::someUlozeneDphShv()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar("SELECT count(*) FROM ulozene WHERE typ=1");
    }
    return false;
}

dxUlozeneArray DbModel::ulozeneDphShv()
{
    dxUlozeneArray shv;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable("SELECT * FROM ulozene WHERE typ=1");
        if(!tb.GetRowCount()) return shv;
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            shv.Add(Ulozene(tb.GetInt("id"), tb.GetInt("typ"), tb.GetString("popis"), tb.GetString("forma"), tb.GetInt("obdobi"),
                                tb.GetInt("mesic"), tb.GetInt("ctvrtleti"), tb.GetInt("rok"), tb.GetDate("od"), tb.GetDate("do"), tb.GetString("xml")));
        }
    }
    return shv;
}

bool DbModel::someUlozeneDphDp3(int rok, bool radne)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(!radne)
            return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM ulozene WHERE rok=%d AND typ=0", rok));
        else
            return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM ulozene WHERE rok=%d AND typ=0 AND (forma='B' OR forma='O')", rok));
    }
    return false;
}

dxUlozeneArray DbModel::ulozeneDphDp3(int rok, bool radne)
{
    dxUlozeneArray dp3;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(!radne)
        {
            wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT * FROM ulozene WHERE rok=%d AND typ=0", rok));
            if(!tb.GetRowCount()) return dp3;
            for(int i=0; i < tb.GetRowCount(); i++)
            {
                tb.SetRow(i);
                dp3.Add(Ulozene(tb.GetInt("id"), tb.GetInt("typ"), tb.GetString("popis"), tb.GetString("forma"), tb.GetInt("obdobi"),
                                    tb.GetInt("mesic"), tb.GetInt("ctvrtleti"), tb.GetInt("rok"), tb.GetDate("od"), tb.GetDate("do"), tb.GetString("xml")));
            }
        }
        else
        {
            wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT * FROM ulozene WHERE rok=%d AND typ=0 AND (forma='B' OR forma='O')", rok));
            if(!tb.GetRowCount()) return dp3;
            for(int i=0; i < tb.GetRowCount(); i++)
            {
                tb.SetRow(i);
                dp3.Add(Ulozene(tb.GetInt("id"), tb.GetInt("typ"), tb.GetString("popis"), tb.GetString("forma"), tb.GetInt("obdobi"),
                                    tb.GetInt("mesic"), tb.GetInt("ctvrtleti"), tb.GetInt("rok"), tb.GetDate("od"), tb.GetDate("do"), tb.GetString("xml")));
            }
        }
    }
    return dp3;
}

wxString DbModel::ulozeneXml(int id)
{
    if(hasUlozene(id))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            return m_db.ExecuteQuery(wxString::Format("SELECT xml FROM ulozene WHERE id=%d", id)).GetString(0);
        }
    }
    return wxEmptyString;
}

Priznani DbModel::priznani() const
{
    return m_priznani;
}

void DbModel::setPriznani(const Priznani &priznani)
{
    m_priznani = priznani;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.IsOpen())
        {
            try
            {
                if(!m_db.ExecuteScalar("SELECT count(*) FROM priznani"))
                {
                    tryExecuteUpdate("INSERT into priznani (jmeno) VALUES ('Gogo')");
                }
                wxSQLite3Statement st = m_db.PrepareStatement("UPDATE priznani SET c_okec=?,c_orient=?,c_pop=?,c_pracufo=?,"
                                                              "c_telef=?,c_ufo=?,dic=?,email=?,id_dats=?,jmeno=?,naz_obce=?,"
                                                              "opr_jmeno=?,opr_postaveni=?,opr_prijmeni=?,prijmeni=?,"
                                                              "psc=?,stat=?,titul=?,ulice=?,zast_dat_nar=?,zast_ev_cislo=?,"
                                                              "zast_ic=?,zast_jmeno=?,zast_kod=?,zast_nazev=?,zast_prijmeni=?,"
                                                              "zast_typ=?");
                st.Bind(1, m_priznani.c_okec());
                st.Bind(2, m_priznani.c_orient());
                st.Bind(3, m_priznani.c_pop());
                st.Bind(4, m_priznani.c_pracufo());
                st.Bind(5, m_priznani.c_telef());
                st.Bind(6, m_priznani.c_ufo());
                st.Bind(7, m_priznani.dic());
                st.Bind(8, m_priznani.email());
                st.Bind(9, m_priznani.id_dats());
                st.Bind(10, m_priznani.jmeno());
                st.Bind(11, m_priznani.naz_obce());
                st.Bind(12, m_priznani.opr_jmeno());
                st.Bind(13, m_priznani.opr_postaveni());
                st.Bind(14, m_priznani.opr_prijmeni());
                st.Bind(15, m_priznani.prijmeni());
                st.Bind(16, m_priznani.psc());
                st.Bind(17, m_priznani.stat());
                st.Bind(18, m_priznani.titul());
                st.Bind(19, m_priznani.ulice());
                st.BindDate(20, m_priznani.zast_dat_nar());
                st.Bind(21, m_priznani.zast_ev_cislo());
                st.Bind(22, m_priznani.zast_ic());
                st.Bind(23, m_priznani.zast_jmeno());
                st.Bind(24, m_priznani.zast_kod());
                st.Bind(25, m_priznani.zast_nazev());
                st.Bind(26, m_priznani.zast_prijmeni());
                st.Bind(27, m_priznani.zast_typ());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v setPriznani: %s", e.GetMessage()));
            }
        }
    }
}

dxSkladArray DbModel::sklady()
{
    dxSkladArray sklady;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable("SELECT * FROM sklad");
        if(!tb.GetRowCount()) return sklady;
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            sklady.Add(Sklad(tb.GetInt("id"), tb.GetString("nazev"), tb.GetString("popis"), tb.GetInt("ocenovani")));
        }
    }
    return sklady;
}

void DbModel::addSklad(const Sklad &sklad)
{
    if(!hasSklad(sklad.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("INSERT INTO sklad(id,nazev,popis,ocenovani) "
                                                              "VALUES(?,?,?,?)");
                st.Bind(1, sklad.id());
                st.Bind(2, sklad.nazev());
                st.Bind(3, sklad.popis());
                st.Bind(4, sklad.ocenovani());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v addSklad: %s", e.GetMessage()));
            }
        }
    }
}

void DbModel::editSklad(const Sklad &sklad)
{
    if(hasSklad(sklad.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("UPDATE sklad SET nazev=?,popis=?,"
                                                              "ocenovani=? WHERE id=?");
                st.Bind(1, sklad.nazev());
                st.Bind(2, sklad.popis());
                st.Bind(3, sklad.ocenovani());
                st.Bind(4, sklad.id());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v editSklad: %s", e.GetMessage()));
            }
        }
        fireUpdated(ID_MODEL_ZASOBA);
    }
}

Sklad DbModel::getSklad(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM sklad WHERE id=%d", id));
        while(rs.NextRow())
        {
            return Sklad(rs.GetInt("id"), rs.GetString("nazev"), rs.GetString("popis"), rs.GetInt("ocenovani"));
        }
        rs.Finalize();
    }
    return Sklad();
}

Sklad DbModel::getSklad(const wxString &nazev)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM sklad WHERE nazev='%s'", nazev));
        while(rs.NextRow())
        {
            return Sklad(rs.GetInt("id"), rs.GetString("nazev"), rs.GetString("popis"), rs.GetInt("ocenovani"));
        }
        rs.Finalize();
    }
    return Sklad();
}

size_t DbModel::getSkladyCount()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar("SELECT count(*) FROM sklad");
    }
    return 0;
}

bool DbModel::deleteSklad(int id)
{
    if(isSkladUsed(id))
    {
        fireMsg(wxString::Format(wxString::FromUTF8("Nelze smazat sklad\n%s\nprotože je použit v zásobách"), getSklad(id).nazev()));
        return false;
    }
    tryExecuteUpdate(wxString::Format("DELETE FROM sklad WHERE id=%d", id));
    return true;
}

Sklad DbModel::firstSklad()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT * FROM sklad");
        while(rs.NextRow())
        {
            return Sklad(rs.GetInt("id"), rs.GetString("nazev"), rs.GetString("popis"), rs.GetInt("ocenovani"));
        }
        rs.Finalize();
    }
    return Sklad();
}

bool DbModel::hasSklad(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM sklad WHERE id=%d", id));
    }
    return false;
}

bool DbModel::hasSklad(const wxString &nazev)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM sklad WHERE nazev='%s'", nazev));
    }
    return false;
}

wxString DbModel::getSkladNazevById(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteQuery(wxString::Format("SELECT nazev FROM sklad WHERE id=%d", id)).GetString(0);
    }
    return wxEmptyString;
}

bool DbModel::isSkladUsed(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM zasoba WHERE skladid=%d", id));
    }
    return false;
}

dxZasobaArray DbModel::zasoby()
{
    dxZasobaArray zsby;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable("SELECT * FROM zasoba ORDER BY cislo");
        if(!tb.GetRowCount()) return zsby;
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            zsby.Add(Zasoba(tb.GetInt("id"), tb.GetString("cislo"), tb.GetString("nazev"), tb.GetString("popis"), tb.GetInt("ocenovani"),
                                    tb.GetString("mj"), tb.GetDouble("pocet"), tb.GetDouble("celkem"), tb.GetInt("skladid"), tb.GetDouble("cenamj"),
                                    tb.GetBool("bezdph"), tb.GetInt("dph")));
        }
    }
    return zsby;
}

void DbModel::addZasoba(const Zasoba &zasoba)
{
    if(!hasZasoba(zasoba.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("INSERT INTO zasoba(id,cislo,nazev,popis,ocenovani,"
                                                              "mj,pocet,celkem,skladid,cenamj,bezdph,dph) "
                                                              "VALUES(?,?,?,?,?,?,?,?,?,?,?,?)");
                st.Bind(1, zasoba.id());
                st.Bind(2, zasoba.cislo());
                st.Bind(3, zasoba.nazev());
                st.Bind(4, zasoba.popis());
                st.Bind(5, zasoba.ocenovani());
                st.Bind(6, zasoba.mj());
                st.Bind(7, zasoba.pocet());
                st.Bind(8, zasoba.celkem());
                st.Bind(9, zasoba.skladid());
                st.Bind(10, zasoba.cenamj());
                st.BindBool(11, zasoba.bezdph());
                st.Bind(12, zasoba.dph());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v addZasoba: %s", e.GetMessage()));
            }
        }
        fireUpdated(ID_MODEL_ZASOBA);
    }
}

void DbModel::editZasoba(const Zasoba &zasoba)
{
    if(hasZasoba(zasoba.id()))
    {
        if(getZasoba(zasoba.id()).ocenovani() != zasoba.ocenovani()) updateStavZasob(zasoba.id());
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("UPDATE zasoba SET cislo=?,nazev=?,popis=?,ocenovani=?,"
                                                              "mj=?,skladid=?,cenamj=?,bezdph=?,dph=?"
                                                              " WHERE id=?");
                st.Bind(1, zasoba.cislo());
                st.Bind(2, zasoba.nazev());
                st.Bind(3, zasoba.popis());
                st.Bind(4, zasoba.ocenovani());
                st.Bind(5, zasoba.mj());
                st.Bind(6, zasoba.skladid());
                st.Bind(7, zasoba.cenamj());
                st.BindBool(8, zasoba.bezdph());
                st.Bind(9, zasoba.dph());
                st.Bind(10, zasoba.id());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v editZasoba: %s", e.GetMessage()));
            }
        }
        fireUpdated(ID_MODEL_ZASOBA);
    }
}

Zasoba DbModel::getZasoba(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM zasoba WHERE id=%d", id));
        while(rs.NextRow())
        {
            return Zasoba(rs.GetInt("id"), rs.GetString("cislo"), rs.GetString("nazev"), rs.GetString("popis"), rs.GetInt("ocenovani"),
                          rs.GetString("mj"), rs.GetDouble("pocet"), rs.GetDouble("celkem"), rs.GetInt("skladid"), rs.GetDouble("cenamj"),
                          rs.GetBool("bezdph"), rs.GetInt("dph"));
        }
        rs.Finalize();
    }
    return Zasoba();
}

size_t DbModel::getZasobyCount()
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar("SELECT count(*) FROM zasoba");
    }
    return 0;
}

size_t DbModel::getZasobyCountBySkladId(int skladid)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM zasoba WHERE skladid=%d", skladid));
    }
    return 0;
}

bool DbModel::deleteZasoba(int id)
{
    tryExecuteUpdate(wxString::Format("DELETE FROM zasoba WHERE id=%d", id));
    fireUpdated(ID_MODEL_ZASOBA);
    return true;
}

bool DbModel::hasZasoba(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM zasoba WHERE id=%d", id));
    }
    return false;
}

int DbModel::copyZasoba(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        Zasoba za = getZasoba(id);
        int newid = getLastId("zasoba");
        addZasoba(Zasoba(newid, getCisloDokladu("ZB"), za.nazev(), za.popis(), za.ocenovani(), za.mj(), 0.0, 0.0, za.skladid(), za.cenamj(), za.bezdph(), za.dph()));
        return newid;
    }
    return -1;
}

void DbModel::getZasobaStav(bool fifo, dxPohybzasobArray pohyby, double *pocet, double *celkem)
{
    *pocet = 0.0;
    *celkem = 0.0;
    pohyby.Sort(dxSortPohybzasobByDatumAsc);
    if(fifo) //FIFO
    {
        wxArrayDouble pocty;
        for(size_t i=0; i < pohyby.GetCount(); i++)
        {
            if(pohyby[i].prijemka())
            {
                pocty.Add(pohyby[i].pocet());
            }
            else
            {
                pocty.Add(0.0);
            }
        }
        for(size_t i=0; i < pohyby.GetCount(); i++)
        {
            if(!pohyby[i].prijemka())
            {
                double p = pohyby[i].pocet();
                for(size_t j=0; j < i; j++)
                {
                    if(wxIsNullDouble(p)) break;
                    if(p > pocty[j])
                    {
                        p -= pocty[j];
                        pocty[j] = 0.0;
                    }
                    else
                    {
                        pocty[j] = pocty[j]-p;
                        p = 0.0;
                    }
                }
            }
        }
        for(size_t i=0; i < pohyby.GetCount(); i++)
        {
            if(pohyby[i].prijemka())
            {
                *pocet += pocty[i];
                *celkem += pocty[i]*pohyby[i].cenamj();
            }
        }
    }
    else
    {
        double citatel = 0.0, jmenovatel = 0.0;
        for(size_t i=0; i < pohyby.GetCount(); i++)
        {
            if(pohyby[i].prijemka())
            {
                *pocet += pohyby[i].pocet();
                citatel += pohyby[i].pocet()*pohyby[i].cenamj();
                jmenovatel += pohyby[i].pocet();
            }
            else
            {
                *pocet -= pohyby[i].pocet();
                wxIsNullDouble(jmenovatel) ? citatel = 0.0 : citatel -= pohyby[i].pocet()*(citatel/jmenovatel);
                jmenovatel -= pohyby[i].pocet();
            }
        }
        wxIsNullDouble(jmenovatel) ? *celkem = 0.0 : *celkem = (*pocet)*(citatel/jmenovatel);
    }
}

void DbModel::getZasobaStav(int id, const wxDateTime &datum, double *pocet, double *celkem)
{
    dxPohybzasobArray pohybyall = pohybyzasobByZasobaId(id);
    dxPohybzasobArray pohyby;
    for(size_t i=0; i < pohybyall.GetCount(); i++)
    {
        if(pohybyall[i].datum().IsEarlierThan(datum))
        {
            pohyby.Add(pohybyall[i]);
        }
    }
    pohyby.Sort(dxSortPohybzasobByDatumAsc);
    getZasobaStav(hasZasoba(id)?!getZasoba(id).ocenovani():true,pohyby,pocet,celkem);
}

double DbModel::stavZasobyToDate(int id, const wxDateTime &datum)
{
    double pocet = 0.0;
    dxPohybzasobArray pohyby = pohybyzasobByZasobaId(id);
    pohyby.Sort(dxSortPohybzasobByDatumAsc);
    for(size_t i=0; i < pohyby.GetCount(); i++)
    {
        if(pohyby[i].datum().IsEarlierThan(datum))
        {
            if(pohyby[i].prijemka()) pocet += pohyby[i].pocet();
            else pocet -= pohyby[i].pocet();
        }
    }
    return pocet;
}

dxZasobaArray DbModel::zasobyBySkladId(int skladid)
{
    dxZasobaArray zasoby;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM zasoba WHERE skladid=%d", skladid));
        while(rs.NextRow())
        {
            zasoby.Add(Zasoba(rs.GetInt("id"), rs.GetString("cislo"), rs.GetString("nazev"), rs.GetString("popis"), rs.GetInt("ocenovani"),
                              rs.GetString("mj"), rs.GetDouble("pocet"), rs.GetDouble("celkem"), rs.GetInt("skladid"), rs.GetDouble("cenamj"),
                              rs.GetBool("bezdph"), rs.GetInt("dph")));
        }
        rs.Finalize();
    }
    return zasoby;
}

bool DbModel::zasobyOnSklad(int skladid)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM zasoba WHERE skladid=%d", skladid));
    }
    return false;
}

void DbModel::changeZasobaOcenovani(int id, int ocenovani)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        try
        {
            wxSQLite3Statement st = m_db.PrepareStatement("UPDATE zasoba SET ocenovani=? WHERE id=?");
            st.Bind(1, ocenovani);
            st.Bind(2, id);
            st.ExecuteUpdate();
            st.Finalize();
        }
        catch(const wxSQLite3Exception& e)
        {
            wxLogDebug(wxString::Format("Chyba v changeZasobaOcenovani: %s", e.GetMessage()));
        }
    }
    updateStavZasob(id);
    fireUpdated(ID_MODEL_ZASOBA);
}

double DbModel::zasobaNakupniCena(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteQuery(wxString::Format("SELECT cenamj FROM pohybzasob WHERE zasobaid=%d AND datum=(SELECT max(datum) FROM pohybzasob WHERE zasobaid=%d AND prijemka=1)", id, id)).GetDouble(0);
    }
    return 0.0;
}

double DbModel::getZasobyStav(const wxDateTime &toDate)
{
    double stav = 0.0;
    dxZasobaArray zsby = zasoby();
    for(size_t i=0; i < zsby.GetCount(); i++)
    {
        double pocet, celkem;
        getZasobaStav(zsby[i].id(), toDate, &pocet, &celkem);
        stav += celkem;
    }
    return stav;
}

void DbModel::setZasobaStav(int id, double pocet, double celkem)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        try
        {
            wxSQLite3Statement st = m_db.PrepareStatement("UPDATE zasoba SET pocet=?,celkem=? WHERE id=?");
            st.Bind(1, pocet);
            st.Bind(2, celkem);
            st.Bind(3, id);
            st.ExecuteUpdate();
            st.Finalize();
        }
        catch(const wxSQLite3Exception& e)
        {
            wxLogDebug(wxString::Format("Chyba v setZasobaStav: %s", e.GetMessage()));
        }
    }
}

dxZasobaViewArray DbModel::zasobyview()
{
    dxZasobaViewArray zsby;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable("SELECT zasoba.id,zasoba.cislo,zasoba.nazev,zasoba.pocet,zasoba.celkem,sklad.nazev FROM zasoba INNER JOIN sklad ON zasoba.skladid = sklad.id");
        if(!tb.GetRowCount()) return zsby;
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            zsby.Add(ZasobaView(tb.GetInt(0), tb.GetString(1), tb.GetString(2), tb.GetDouble(3), tb.GetDouble(4), tb.GetString(5)));
        }
    }
    return zsby;
}

dxInventuraArray DbModel::inventury()
{
    dxInventuraArray inventury;
    if(dxsettings.dbmodel() == DB_SQLITE)
        {
            wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT * FROM inventura WHERE obdobirok=%d", m_currentRok));
            if(!tb.GetRowCount()) return inventury;
            for(int i=0; i < tb.GetRowCount(); i++)
            {
                tb.SetRow(i);
                inventury.Add(Inventura(tb.GetInt("id"), tb.GetDateTime("datum"), tb.GetString("cislo"), tb.GetString("popis"), tb.GetString("provedl"),
                                              tb.GetInt("skladid"), tb.GetInt("obdobirok")));
            }
        }
    return inventury;
}

void DbModel::addInventura(const Inventura &inventura)
{
    if(!hasInventura(inventura.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("INSERT INTO inventura(id,datum,cislo,popis,provedl,skladid,obdobirok) "
                                                              "VALUES(?,?,?,?,?,?,?)");
                st.Bind(1, inventura.id());
                st.BindDateTime(2, inventura.datum());
                st.Bind(3, inventura.cislo());
                st.Bind(4, inventura.popis());
                st.Bind(5, inventura.provedl());
                st.Bind(6, inventura.skladid());
                st.Bind(7, inventura.obdobirok());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v addInventura: %s", e.GetMessage()));
            }
        }
        // fireUpdated() fired by addPohybzasob;
    }
}

void DbModel::editInventura(const Inventura &inventura)
{
    if(hasInventura(inventura.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("UPDATE inventura SET datum=?,cislo=?,popis=?,provedl=?,skladid=?,obdobirok=? "
                                                              "WHERE id=?");
                st.BindDateTime(1, inventura.datum());
                st.Bind(2, inventura.cislo());
                st.Bind(3, inventura.popis());
                st.Bind(4, inventura.provedl());
                st.Bind(5, inventura.skladid());
                st.Bind(6, inventura.obdobirok());
                st.Bind(7, inventura.id());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v editInventura: %s", e.GetMessage()));
            }
        }
        // fireUpdated() fired by editPohybzasob;
    }
}

Inventura DbModel::getInventura(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM inventura WHERE id=%d", id));
        while(rs.NextRow())
        {
            return Inventura(rs.GetInt("id"), rs.GetDateTime("datum"), rs.GetString("cislo"), rs.GetString("popis"), rs.GetString("provedl"),
                             rs.GetInt("skladid"), rs.GetInt("obdobirok"));
        }
        rs.Finalize();
    }
    return Inventura();
}

bool DbModel::deleteInventura(int id)
{
    deletePohybyzasobByInventuraId(id); //updatestavzasob called from it
    tryExecuteUpdate(wxString::Format("DELETE FROM inventura WHERE id=%d", id));
    fireUpdated(ID_MODEL_ZASOBA);
    return true;
}

bool DbModel::hasInventura(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM inventura WHERE id=%d", id));
    }
    return false;
}

wxString DbModel::getInventuraCisloById(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteQuery(wxString::Format("SELECT cislo FROM inventura WHERE id=%d", id)).GetString(0);
    }
    return wxEmptyString;
}

bool DbModel::inventuraIncludedZasoba(int id, int zasobaid)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM ivpolozka WHERE inventuraid=%d AND zasobaid=%d", id, zasobaid));
    }
    return false;
}

wxDateTime DbModel::getInventuraLastDateForZasoba(int zasobaid)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT max(datum) FROM inventura WHERE id IN (SELECT inventuraid FROM ivpolozka WHERE zasobaid=%d) AND obdobirok=%d", zasobaid, m_currentRok));
        while(rs.NextRow())
        {
            if(!rs.IsNull(0)) return rs.GetDate(0);
        }
        rs.Finalize();
    }
    return wxDateTime(1, wxDateTime::Jan, m_currentRok);
}

wxDateTime DbModel::getInventuraLastDateForSklad(int skladid)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT max(datum) FROM inventura WHERE skladid=%d AND obdobirok=%d", skladid, m_currentRok));
        while(rs.NextRow())
        {
            if(!rs.IsNull(0)) return rs.GetDate(0);
        }
        rs.Finalize();
    }
    return wxDateTime(1, wxDateTime::Jan, m_currentRok);
}

void DbModel::addIvPolozka(const IvPolozka &ivpolozka)
{
    if(!hasIvPolozka(ivpolozka.id()) && hasInventura(ivpolozka.inventuraid()) && hasZasoba(ivpolozka.zasobaid()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("INSERT INTO ivpolozka(id,evidovano,zjisteno,zasobaid,inventuraid) "
                                                              "VALUES(?,?,?,?,?)");
                st.Bind(1, ivpolozka.id());
                st.Bind(2, ivpolozka.evidovano());
                st.Bind(3, ivpolozka.zjisteno());
                st.Bind(4, ivpolozka.zasobaid());
                st.Bind(5, ivpolozka.inventuraid());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v addIvPolozka: %s", e.GetMessage()));
            }
        }
    }
}

void DbModel::editIvPolozka(const IvPolozka &ivpolozka)
{
    if(hasIvPolozka(ivpolozka.id()) && hasInventura(ivpolozka.inventuraid()) && hasZasoba(ivpolozka.zasobaid()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("UPDATE ivpolozka SET evidovano=?,zjisteno=?,zasobaid=?,inventuraid=? "
                                                              "WHERE id=?");
                st.Bind(1, ivpolozka.evidovano());
                st.Bind(2, ivpolozka.zjisteno());
                st.Bind(3, ivpolozka.zasobaid());
                st.Bind(4, ivpolozka.inventuraid());
                st.Bind(5, ivpolozka.id());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v editIvPolozka: %s", e.GetMessage()));
            }
        }
    }
}

bool DbModel::hasIvPolozka(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM ivpolozka WHERE id=%d", id));
    }
    return false;
}

dxIvPolozkaArray DbModel::ivpolozkyByInventuraId(int inventuraid)
{
    dxIvPolozkaArray polozky;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3Table tb = m_db.GetTable(wxString::Format("SELECT * FROM ivpolozka WHERE inventuraid=%d", inventuraid));
        if(!tb.GetRowCount()) return polozky;
        for(int i=0; i < tb.GetRowCount(); i++)
        {
            tb.SetRow(i);
            polozky.Add(IvPolozka(tb.GetInt("id"), tb.GetDouble("evidovano"), tb.GetDouble("zjisteno"), tb.GetInt("zasobaid"), tb.GetInt("inventuraid")));
        }
    }
    return polozky;
}

void DbModel::addPohybzasob(const Pohybzasob &pohybzasob)
{
    if(!hasPohybzasob(pohybzasob.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("INSERT INTO pohybzasob(id,datum,cislo,pocet,cenamj,"
                                                              "prijemka,pocatek,fakturaid,zasobaid,obdobirok,inventura,inventuraid) "
                                                              "VALUES(?,?,?,?,?,?,?,?,?,?,?,?)");
                st.Bind(1, pohybzasob.id());
                st.BindDateTime(2, pohybzasob.datum());
                st.Bind(3, pohybzasob.cislo());
                st.Bind(4, pohybzasob.pocet());
                st.Bind(5, pohybzasob.cenamj());
                st.BindBool(6, pohybzasob.prijemka());
                st.BindBool(7, pohybzasob.pocatek());
                hasFaktura(pohybzasob.fakturaid()) ? st.Bind(8, pohybzasob.fakturaid()) : st.BindNull(8);
                st.Bind(9, pohybzasob.zasobaid());
                st.Bind(10, pohybzasob.obdobirok());
                st.BindBool(11, pohybzasob.inventura());
                hasInventura(pohybzasob.inventuraid()) ? st.Bind(12, pohybzasob.inventuraid()) : st.BindNull(12);
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v addPohybzasob: %s", e.GetMessage()));
            }
        }
        updateStavZasob(pohybzasob.zasobaid());
        fireUpdated(ID_MODEL_ZASOBA);
    }
}

void DbModel::editPohybzasob(const Pohybzasob &pohybzasob)
{
    if(hasPohybzasob(pohybzasob.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("UPDATE pohybzasob SET datum=?,cislo=?,pocet=?,cenamj=?,"
                                                              "prijemka=?,pocatek=?,fakturaid=?,zasobaid=?,obdobirok=?,inventura=?,inventuraid=? "
                                                              "WHERE id=?");
                st.BindDateTime(1, pohybzasob.datum());
                st.Bind(2, pohybzasob.cislo());
                st.Bind(3, pohybzasob.pocet());
                st.Bind(4, pohybzasob.cenamj());
                st.BindBool(5, pohybzasob.prijemka());
                st.BindBool(6, pohybzasob.pocatek());
                hasFaktura(pohybzasob.fakturaid()) ? st.Bind(7, pohybzasob.fakturaid()) : st.BindNull(7);
                st.Bind(8, pohybzasob.zasobaid());
                st.Bind(9, pohybzasob.obdobirok());
                st.BindBool(10, pohybzasob.inventura());
                hasInventura(pohybzasob.inventuraid()) ? st.Bind(11, pohybzasob.inventuraid()) : st.BindNull(11);
                st.Bind(12, pohybzasob.id());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v editPohybzasob: %s", e.GetMessage()));
            }
        }
        updateStavZasob(pohybzasob.zasobaid());
        fireUpdated(ID_MODEL_ZASOBA);
    }
}

Pohybzasob DbModel::getPohybzasob(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM pohybzasob WHERE id=%d", id));
        while(rs.NextRow())
        {
            return Pohybzasob(rs.GetInt("id"), rs.GetDateTime("datum"), rs.GetString("cislo"), rs.GetDouble("pocet"), rs.GetDouble("cenamj"), rs.GetBool("prijemka"),
                              rs.GetBool("pocatek"), rs.GetInt("fakturaid"), rs.GetInt("zasobaid"), rs.GetInt("obdobirok"), rs.GetBool("inventura"), rs.GetInt("inventuraid"));
        }
        rs.Finalize();
    }
    return Pohybzasob();
}

Pohybzasob DbModel::getPohybzasobByInventura(int zasobaid, int inventuraid)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM pohybzasob WHERE zasobaid=%d AND inventuraid=%d", zasobaid, inventuraid));
        while(rs.NextRow())
        {
            return Pohybzasob(rs.GetInt("id"), rs.GetDateTime("datum"), rs.GetString("cislo"), rs.GetDouble("pocet"), rs.GetDouble("cenamj"), rs.GetBool("prijemka"),
                              rs.GetBool("pocatek"), rs.GetInt("fakturaid"), rs.GetInt("zasobaid"), rs.GetInt("obdobirok"), rs.GetBool("inventura"), rs.GetInt("inventuraid"));
        }
        rs.Finalize();
    }
    return Pohybzasob();
}

Pohybzasob DbModel::getPohybzasobByFaktura(int zasobaid, int fakturaid)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM pohybzasob WHERE zasobaid=%d AND fakturaid=%d", zasobaid, fakturaid));
        while(rs.NextRow())
        {
            return Pohybzasob(rs.GetInt("id"), rs.GetDateTime("datum"), rs.GetString("cislo"), rs.GetDouble("pocet"), rs.GetDouble("cenamj"), rs.GetBool("prijemka"),
                              rs.GetBool("pocatek"), rs.GetInt("fakturaid"), rs.GetInt("zasobaid"), rs.GetInt("obdobirok"), rs.GetBool("inventura"), rs.GetInt("inventuraid"));
        }
        rs.Finalize();
    }
    return Pohybzasob();
}

bool DbModel::deletePohybzasob(int id)
{
    int zasobaid = getPohybzasob(id).zasobaid();
    tryExecuteUpdate(wxString::Format("DELETE FROM pohybzasob WHERE id=%d", id));
    updateStavZasob(zasobaid);
    fireUpdated(ID_MODEL_ZASOBA);
    return true;
}

bool DbModel::hasPohybzasob(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM pohybzasob WHERE id=%d", id));
    }
    return false;
}

dxPohybzasobArray DbModel::pohybyzasobByZasobaId(int zasobaid)
{
    dxPohybzasobArray pohyby;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM pohybzasob WHERE zasobaid=%d", zasobaid));
        while(rs.NextRow())
        {
            pohyby.Add(Pohybzasob(rs.GetInt("id"), rs.GetDateTime("datum"), rs.GetString("cislo"), rs.GetDouble("pocet"), rs.GetDouble("cenamj"), rs.GetBool("prijemka"),
                              rs.GetBool("pocatek"), rs.GetInt("fakturaid"), rs.GetInt("zasobaid"), rs.GetInt("obdobirok"), rs.GetBool("inventura"), rs.GetInt("inventuraid")));
        }
        rs.Finalize();
    }
    pohyby.Sort(dxSortPohybzasobByDatumAsc);
    return pohyby;
}

bool DbModel::deletePohybyzasobByInventuraId(int inventuraid)
{
    //mazani v db diky ON DELETE CASCADE
    bool result = false;
    wxArrayInt zasoby;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT zasobaid FROM pohybzasob WHERE inventuraid=%d", inventuraid));
        while(rs.NextRow())
        {
            zasoby.Add(rs.GetInt(0));
            result = true;
        }
        rs.Finalize();
    }
    for(size_t i=0; i < zasoby.GetCount(); i++)
    {
        updateStavZasob(zasoby[i]);
    }
    fireUpdated(ID_MODEL_ZASOBA);
    return result;
}

bool DbModel::deletePohybyzasobByFakturaId(int fakturaid, bool deleteDB)
{
    //mazani v db diky ON DELETE CASCADE
    bool result = false;
    wxArrayInt zasoby;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT zasobaid FROM pohybzasob WHERE fakturaid=%d", fakturaid));
        while(rs.NextRow())
        {
            zasoby.Add(rs.GetInt(0));
            result = true;
        }
        rs.Finalize();
    }
    if(deleteDB)
    {
        tryExecuteUpdate(wxString::Format("DELETE FROM pohybzasob WHERE fakturaid=%d", fakturaid));
    }
    for(size_t i=0; i < zasoby.GetCount(); i++)
    {
        updateStavZasob(zasoby[i]);
    }
    fireUpdated(ID_MODEL_ZASOBA);
    return result;
}

void DbModel::editPohybzasobDatum(int zasobaid, int fakturaid, const wxDateTime &datum)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        try
        {
            wxSQLite3Statement st = m_db.PrepareStatement("UPDATE pohybzasob SET datum=? WHERE zasobaid=? AND fakturaid=?");
            st.BindDateTime(1, datum);
            st.Bind(2, zasobaid);
            st.Bind(3, fakturaid);
            st.ExecuteUpdate();
            st.Finalize();
        }
        catch(const wxSQLite3Exception& e)
        {
            wxLogDebug(wxString::Format("Chyba v editPohybzasobDatum: %s", e.GetMessage()));
        }
    }
}

void DbModel::sortDenik(dxDenikArray &arr, DbModel::DSort sort, bool asc)
{
    switch(sort)
    {
    case DS_DATUM:
    {
        if(asc) arr.Sort(dxSortDenikByDatumAsc);
        else arr.Sort(dxSortDenikByDatumDsc);
        return;
    }
    case DS_CISLO:
    {
        if(asc) arr.Sort(dxSortDenikByCisloAsc);
        else arr.Sort(dxSortDenikByCisloDsc);
        return;
    }
    case DS_POPIS:
    {
        if(asc) arr.Sort(dxSortDenikByPopisAsc);
        else arr.Sort(dxSortDenikByPopisDsc);
        return;
    }
    case DS_CASTKA:
    {
        if(asc) arr.Sort(dxSortDenikByCastkaAsc);
        else arr.Sort(dxSortDenikByCastkaDsc);
        return;
    }
    case DS_PRIJEM:
    {
        if(asc) arr.Sort(dxSortDenikByPrijemAsc);
        else arr.Sort(dxSortDenikByPrijemDsc);
        return;
    }
    case DS_POKLADNA:
    {
        if(asc) arr.Sort(dxSortDenikByPokladnaAsc);
        else arr.Sort(dxSortDenikByPokladnaDsc);
        return;
    }
    case DS_BANKA:
    {
        if(asc) arr.Sort(dxSortDenikByBankaAsc);
        else arr.Sort(dxSortDenikByBankaDsc);
        return;
    }
    case DS_VYPIS:
    {
        if(asc) arr.Sort(dxSortDenikByVypisAsc);
        else arr.Sort(dxSortDenikByVypisDsc);
        return;
    }
    case DS_SLOUPEC:
    {
        if(asc) arr.Sort(dxSortDenikBySloupecAsc);
        else arr.Sort(dxSortDenikBySloupecDsc);
        return;
    }
    case DS_PAUSAL:
    {
        if(asc) arr.Sort(dxSortDenikByPausalAsc);
        else arr.Sort(dxSortDenikByPausalDsc);
        return;
    }
    default: return;
    }
}

void DbModel::sortDenikView(dxDenikViewArray &arr, DbModel::DSort sort, bool asc)
{
    switch(sort)
    {
    case DS_DATUM:
    {
        if(asc) arr.Sort(dxSortDenikViewByDatumAsc);
        else arr.Sort(dxSortDenikViewByDatumDsc);
        return;
    }
    case DS_CISLO:
    {
        if(asc) arr.Sort(dxSortDenikViewByCisloAsc);
        else arr.Sort(dxSortDenikViewByCisloDsc);
        return;
    }
    case DS_POPIS:
    {
        if(asc) arr.Sort(dxSortDenikViewByPopisAsc);
        else arr.Sort(dxSortDenikViewByPopisDsc);
        return;
    }
    case DS_CASTKA:
    {
        if(asc) arr.Sort(dxSortDenikViewByCastkaAsc);
        else arr.Sort(dxSortDenikViewByCastkaDsc);
        return;
    }
    case DS_PRIJEM:
    {
        if(asc) arr.Sort(dxSortDenikViewByPrijemAsc);
        else arr.Sort(dxSortDenikViewByPrijemDsc);
        return;
    }
    case DS_POKLADNA:
    case DS_BANKA:
    {
        if(asc) arr.Sort(dxSortDenikViewByPokladnaBankaAsc);
        else arr.Sort(dxSortDenikViewByPokladnaBankaDsc);
        return;
    }
    case DS_VYPIS:
    {
        if(asc) arr.Sort(dxSortDenikViewByVypisAsc);
        else arr.Sort(dxSortDenikViewByVypisDsc);
        return;
    }
    default: return;
    }
}

void DbModel::sortFakturyView(dxFakturaViewArray &arr, DbModel::FZPSort sort, bool asc)
{
    switch(sort) {
    case FZP_DOKLAD:
    {
        if(asc) arr.Sort(dxSortFakturyViewByDokladAsc);
        else arr.Sort(dxSortFakturyViewByDokladDsc);
        return;
    }
    case FZP_POPIS:
    {
        if(asc) arr.Sort(dxSortFakturyViewByPopisAsc);
        else arr.Sort(dxSortFakturyViewByPopisDsc);
        return;
    }
    case FZP_PROTISTRANA:
    {
        if(asc) arr.Sort(dxSortFakturyViewByProtistanaAsc);
        else arr.Sort(dxSortFakturyViewByProtistranaDsc);
        return;
    }
    case FZP_VYSTAVENA:
    {
        if(asc) arr.Sort(dxSortFakturyViewByVystavenaAsc);
        else arr.Sort(dxSortFakturyViewByVystavenaDsc);
        return;
    }
    case FZP_SPLATNA:
    {
        if(asc) arr.Sort(dxSortFakturyViewBySplatnaAsc);
        else arr.Sort(dxSortFakturyViewBySplatnaDsc);
        return;
    }
    case FZP_CELKEM:
    {
        if(asc) arr.Sort(dxSortFakturyViewByCelkemAsc);
        else arr.Sort(dxSortFakturyViewByCelkemDsc);
        return;
    }
    case FZP_UHRADIT:
    {
        if(asc) arr.Sort(dxSortFakturyViewByUhraditAsc);
        else arr.Sort(dxSortFakturyViewByUhraditDsc);
        return;
    }
    default: return;
    }
}

void DbModel::sortZavPohView(dxZavPohViewArray &arr, DbModel::FZPSort sort, bool asc)
{
    switch(sort) {
    case FZP_DOKLAD:
    {
        if(asc) arr.Sort(dxSortZavPohViewByDokladAsc);
        else arr.Sort(dxSortZavPohViewByDokladDsc);
        return;
    }
    case FZP_POPIS:
    {
        if(asc) arr.Sort(dxSortZavPohViewByPopisAsc);
        else arr.Sort(dxSortZavPohViewByPopisDsc);
        return;
    }
    case FZP_PROTISTRANA:
    {
        if(asc) arr.Sort(dxSortZavPohViewByProtistanaAsc);
        else arr.Sort(dxSortZavPohViewByProtistranaDsc);
        return;
    }
    case FZP_VYSTAVENA:
    {
        if(asc) arr.Sort(dxSortZavPohViewByVystavenaAsc);
        else arr.Sort(dxSortZavPohViewByVystavenaDsc);
        return;
    }
    case FZP_SPLATNA:
    {
        if(asc) arr.Sort(dxSortZavPohViewBySplatnaAsc);
        else arr.Sort(dxSortZavPohViewBySplatnaDsc);
        return;
    }
    case FZP_CELKEM:
    {
        if(asc) arr.Sort(dxSortZavPohViewByCelkemAsc);
        else arr.Sort(dxSortZavPohViewByCelkemDsc);
        return;
    }
    case FZP_UHRADIT:
    {
        if(asc) arr.Sort(dxSortZavPohViewByUhraditAsc);
        else arr.Sort(dxSortZavPohViewByUhraditDsc);
        return;
    }
    default: return;
    }
}

void DbModel::sortMajetekView(dxMajetekViewArray &arr, DbModel::MSort sort, bool asc)
{
    switch(sort) {
    case MS_CISLO:
    {
        if(asc) arr.Sort(dxSortMajetekViewByCisloAsc);
        else arr.Sort(dxSortMajetekViewByCisloDsc);
        return;
    }
    case MS_NAZEV:
    {
        if(asc) arr.Sort(dxSortMajetekViewByNazevAsc);
        else arr.Sort(dxSortMajetekViewByNazevDsc);
        return;
    }
    case MS_TYP:
    {
        if(asc) arr.Sort(dxSortMajetekViewByTypAsc);
        else arr.Sort(dxSortMajetekViewByTypDsc);
        return;
    }
    case MS_ODPIS:
    {
        if(asc) arr.Sort(dxSortMajetekViewByOdpisAsc);
        else arr.Sort(dxSortMajetekViewByOdpisDsc);
        return;
    }
    case MS_PORIZEN:
    {
        if(asc) arr.Sort(dxSortMajetekViewByPorizenAsc);
        else arr.Sort(dxSortMajetekViewByPorizenDsc);
        return;
    }
    case MS_PORCENA:
    {
        if(asc) arr.Sort(dxSortMajetekViewByPorcenaAsc);
        else arr.Sort(dxSortMajetekViewByPorcenaDsc);
        return;
    }
    case MS_ZUSCENA:
    {
        if(asc) arr.Sort(dxSortMajetekViewByZuscenaAsc);
        else arr.Sort(dxSortMajetekViewByZuscenaDsc);
        return;
    }
    default: return;
    }
}

void DbModel::sortFaPolozkyByDphSazba(dxFaPolozkaArray &arr)
{
    arr.Sort(dxSortFaPolozkaByDphSazba);
}

void DbModel::sortFaPolozky(dxFaPolozkaArray &arr)
{
    arr.Sort(dxSortFaPolozky);
}

void DbModel::sortOdeslany(dxOdeslanyArray &arr, DbModel::OSort sort, bool asc)
{
    switch(sort) {
    case OS_CAS:
    {
        if(asc) arr.Sort(dxSortOdeslanyByCasAsc);
        else arr.Sort(dxSortOdeslanyByCasDsc);
        return;
    }
    case OS_FAKTURA:
    {
        if(asc) arr.Sort(dxSortOdeslanyByFakturaAsc);
        else arr.Sort(dxSortOdeslanyByFakturaDsc);
        return;
    }
    case OS_PRIJEMCE:
    {
        if(asc) arr.Sort(dxSortOdeslanyByPrijemceAsc);
        else arr.Sort(dxSortOdeslanyByPrijemceDsc);
    }
    default: return;
    }
}

void DbModel::sortUlozene(dxUlozeneArray &arr)
{
    arr.Sort(dxSortUlozeneDph);
}

void DbModel::sortDphDoklad(dxDphDokladArray &arr)
{
    arr.Sort(dxSortDphDoklad);
}

void DbModel::sortZasobyView(dxZasobaViewArray &arr, DbModel::ZSort sort, bool asc)
{
    switch(sort) {
    case ZB_CISLO:
    {
        if(asc) arr.Sort(dxSortZasobyViewByCisloAsc);
        else arr.Sort(dxSortZasobyViewByCisloDsc);
        return;
    }
    case ZB_NAZEV:
    {
        if(asc) arr.Sort(dxSortZasobyViewByNazevAsc);
        else arr.Sort(dxSortZasobyViewByNazevDsc);
        return;
    }
    case ZB_STAV:
    {
        if(asc) arr.Sort(dxSortZasobyViewByPocetAsc);
        else arr.Sort(dxSortZasobyViewByPocetDsc);
        return;
    }
    case ZB_HODNOTA:
    {
        if(asc) arr.Sort(dxSortZasobyViewByCelkemAsc);
        else arr.Sort(dxSortZasobyViewByCelkemDsc);
        return;
    }
    case ZB_SKLAD:
    {
        if(asc) arr.Sort(dxSortZasobyViewBySkladAsc);
        else arr.Sort(dxSortZasobyViewBySkladDsc);
        return;
    }
    default: return;
    }
}

void DbModel::sortJizdaView(dxJizdaViewArray &arr, DbModel::JSort sort, bool asc)
{
    switch(sort) {
    case JA_DATUM:
    {
        if(asc) arr.Sort(dxSortJizdaViewByDatumAsc);
        else arr.Sort(dxSortJizdaViewByDatumDsc);
        return;
    }
    case JA_CESTA:
    {
        if(asc) arr.Sort(dxSortJizdaViewByCestaAsc);
        else arr.Sort(dxSortJizdaViewByCestaDsc);
        return;
    }
    case JA_UCEL:
    {
        if(asc) arr.Sort(dxSortJizdaViewByUcelAsc);
        else arr.Sort(dxSortJizdaViewByUcelDsc);
        return;
    }
    case JA_KM:
    {
        if(asc) arr.Sort(dxSortJizdaViewByKmAsc);
        else arr.Sort(dxSortJizdaViewByKmDsc);
        return;
    }
    case JA_VOZIDLO:
    {
        if(asc) arr.Sort(dxSortJizdaViewByVozidloAsc);
        else arr.Sort(dxSortJizdaViewByVozidloDsc);
        return;
    }
    case JA_RIDIC:
    {
        if(asc) arr.Sort(dxSortJizdaViewByRidicAsc);
        else arr.Sort(dxSortJizdaViewByRidicDsc);
        return;
    }
    default: return;
    }
}

void DbModel::sortDphDruh(dxDphDruhArray &arr, int sort, bool asc)
{
    switch(sort) {
    case 0:
    {
        if(asc) arr.Sort(dxSortDphDruhViewByZkratkaAsc);
        else arr.Sort(dxSortDphDruhViewByZkratkaDsc);
        return;
    }
    case 1:
    {
        if(asc) arr.Sort(dxSortDphDruhViewByPopisAsc);
        else arr.Sort(dxSortDphDruhViewByPopisDsc);
        return;
    }
    case 2:
    {
        if(asc) arr.Sort(dxSortDphDruhViewByPlneniAsc);
        else arr.Sort(dxSortDphDruhViewByPlneniDsc);
        return;
    }
    case 3:
    {
        if(asc) arr.Sort(dxSortDphDruhViewByRadkyAsc);
        else arr.Sort(dxSortDphDruhViewByRadkyDsc);
        return;
    }
    case 4:
    {
        if(asc) arr.Sort(dxSortDphDruhViewByKppAsc);
        else arr.Sort(dxSortDphDruhViewByKppDsc);
        return;
    }
    case 5:
    {
        if(asc) arr.Sort(dxSortDphDruhViewByKhAsc);
        else arr.Sort(dxSortDphDruhViewByKhDsc);
        return;
    }
    case 6:
    {
        if(asc) arr.Sort(dxSortDphDruhViewByNabizetAsc);
        else arr.Sort(dxSortDphDruhViewByNabizetDsc);
        return;
    }
    default:
    {
        if(asc) arr.Sort(dxSortDphDruhViewByIdAsc);
        else arr.Sort(dxSortDphDruhViewByIdDsc);
        return;
    }
    }
}

void DbModel::addAdresa(const Adresa &adresa)
{
    if(!hasAdresa(adresa.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("INSERT INTO adresa(id,ulice,cp,misto,psc,zeme) "
                                                              "VALUES(?,?,?,?,?,?)");
                st.Bind(1, adresa.id());
                st.Bind(2, adresa.ulice());
                st.Bind(3, adresa.cp());
                st.Bind(4, adresa.misto());
                st.Bind(5, adresa.psc());
                st.Bind(6, adresa.zeme());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v addAdresa: %s", e.GetMessage()));
            }
        }
    }
}

void DbModel::editAdresa(const Adresa &adresa)
{
    if(hasAdresa(adresa.id()))
    {
        if(dxsettings.dbmodel() == DB_SQLITE)
        {
            try
            {
                wxSQLite3Statement st = m_db.PrepareStatement("UPDATE adresa SET ulice=?,cp=?,psc=?,misto=?,"
                                                              "zeme=? WHERE id=?");
                st.Bind(1, adresa.ulice());
                st.Bind(2, adresa.cp());
                st.Bind(3, adresa.psc());
                st.Bind(4, adresa.misto());
                st.Bind(5, adresa.zeme());
                st.Bind(6, adresa.id());
                st.ExecuteUpdate();
                st.Finalize();
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogDebug(wxString::Format("Chyba v editAdresa: %s", e.GetMessage()));
            }
        }
    }
}

Adresa DbModel::getAdresa(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM adresa WHERE id=%d", id));
        if(rs.NextRow())
        {
            return Adresa(rs.GetInt("id"), rs.GetString("ulice"), rs.GetString("cp"), rs.GetString("misto"),
                          rs.GetString("psc"), rs.GetString("zeme"));
        }
    }
    return Adresa();
}

bool DbModel::deleteAdresa(int id)
{
    if(id < 2) //defaultni a firmy
        return true;
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        if(m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM kontakt WHERE adresaid=%d", id)))
        {
            fireMsg(wxString::FromUTF8("Nelze smazat adresu\nprotože je použita v kontaktech"));
            return true; //nemazat adresu kterou ma nejaky kontakt
        }
    }
    tryExecuteUpdate(wxString::Format("DELETE FROM adresa WHERE id=%d", id));
    return true;
}

bool DbModel::hasAdresa(int id)
{
    if(dxsettings.dbmodel() == DB_SQLITE)
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM adresa WHERE id=%d", id));
    }
    return false;
}
